<?php 

#[AllowDynamicProperties]

class CRDB {
	function __construct() {		
		$this->email        	= 'tibapay@emedical.co.tz';
		$this->password  		= 'password@123';
	}

	function getToken() {
        global $app, $client;

		$url 					= $client->crdb_server_address.'/auth/token';
		$body 				    = json_encode (
			array (
				'email'			=> $this->email,
				'password'		=> $this->password
			)
		);

        $curl_options           = array (
            'method'            => 'POST',
            'url'               => $url,
			'fields'            => $body,
			'headers'   		=> array (
                'Content-Type'  => 'application/json'
            )
        );

        $token_json             = $app->curl($curl_options);

        if($token = json_decode($token_json->message)) {
            return $token->returnData;
        } else {
            die($app->renderAlertError('Could not get access token'));
        }
	}

	/**
	 * Get control number and updates invoice
	 *
	 * @param integer $invoice_id   Invoice id
	 * @return null
	*/ 
	function requestControlNumber(int $invoice_id) {
		global $app, $client;
		$invoice 				= new Invoice($invoice_id);

		$data 					= array (
			'uid'				=> strval($invoice->id),
			'date'				=> date('Y-m-d'),
			'amount'			=> $invoice->payable,
			'company_uid'		=> strval($client->crdb_company_uid),
			'customer_uid'		=> strval($invoice->personnel_id),
			'account_uid'		=> strval($client->crdb_account_uid),
			'amount_type'		=> $client->crdb_amount_type
		);

		$curl_options           = array (
            'method'            => 'POST',
            'url'               => $client->crdb_server_address.'/api/payment/generateControlNumber',
			'fields'            => json_encode($data),
			'headers'   		=> array (
                'Content-Type'  => 'application/json',
				'Authorization' => 'Bearer '.$this->getToken()->accessToken
            )
        );

		$results_json           = $app->curl($curl_options);
		if($results = json_decode($results_json->message)) {
			if(isset($results->returnData->control_number)) {
				$bill_data 		= array (
					'reference'			=> 'Control no',
					'reference_no'		=> $results->returnData->control_number,
					'billed_reference'	=> $results->returnData->control_number
				);
				$invoice->bill($bill_data);
				echo $app->renderAlertSuccess($results->message, $results->returnData->control_number, 'modal_lg', 1);
			}
        } else {
            die($app->renderAlertError('Could not get control number'));
        }
	}

	
	function receivePayments() {
		global $app, $client;

		$data 					= array (
			'q'					=> 'query',
			'company_uid'		=> strval($client->crdb_company_uid),
			'account_uid'		=> strval($client->crdb_account_uid)
		);

		$curl_options           = array (
            'method'            => 'POST',
            'url'               => $client->crdb_server_address.'/api/payment/listPendingPaymentPaidControlNumber',
			'fields'            => json_encode($data),
			'headers'   		=> array (
                'Content-Type'  => 'application/json',
				'Authorization' => 'Bearer '.$this->getToken()->accessToken
            )
        );

		$results_json           = $app->curl($curl_options);

		if($results = json_decode($results_json->message)) {
			
			foreach($results->returnData as $row) {
				echo 'Invoice no = '.$row->id.', Control no = '.$row->control_number.'<br />';
				$invoice_id 			= $app->getColumnValue('billing_invoices', 'id', 'reference_no', $row->control_number);
				$invoice				= new Invoice($invoice_id);
				
				$payment_data 			= array (
					'payment'			=> 1,
					'payment_method'	=> 'CRDB',
					'payment_channel'	=> 'CRDB',
					'payment_receipt'	=> $row->transaction_no,
				);
				$paid 					= $invoice->pay($payment_data);
				if($paid) {
					$confirmed			= $this->confirmPayment($row->id);
					var_dump($confirmed);
				}
			}
        } else {
            die($app->renderAlertError('Could not get payments'));
        }
	}

	function confirmPayment($payment_id) {
		global $app, $client;

		$data 					= array (
			'id'				=> $payment_id
		);

		$curl_options           = array (
            'method'            => 'POST',
            'url'               => $client->crdb_server_address.'/api/payment/updatePaid',
			'fields'            => json_encode($data),
			'headers'   		=> array (
                'Content-Type'  => 'application/json',
				'Authorization' => 'Bearer '.$this->getToken()->accessToken
            )
        );

		$results_json           = $app->curl($curl_options);
		if($results = json_decode($results_json->message)) {
            return $results->returnData;
        } else {
            die($app->renderAlertError('Could not update payments'));
        }
	}
}

$crdb = new CRDB();

?>