<?php

require('ArrayToXml.php');

class GEPG {

	function convertArrayToXML(array $array, string $root) {
		$xml = new ArrayToXml();
		$xml = $xml->convert($array, $root, true, 'UTF-8', '1.0', array(), true, true);
		$xml = trim(str_replace(array("\r", "\n"), '', $xml));
		return $xml;
	}

	function extractStringBetweenStrings($string, $start, $end){
		$string 	= ' ' . $string;
		$ini 		= strpos($string, $start);
		if ($ini == 0) return '';
		$ini 		+= strlen($start);
		$len 		= strpos($string, $end, $ini) - $ini;
		return substr($string, $ini, $len);
	}

	function addSignature($xml_dom, $data_tag) {
		global $client, $root, $app;

		$client_private_key = $root.CLIENT_PATH.'keys/gepg_client_private.pfx';
		$client_certificate = file_get_contents($client_private_key);

		if(openssl_pkcs12_read($client_certificate, $certificate_info, $client->gepg_client_password)) {
			$data_string		= $this->extractStringBetweenStrings($xml_dom, '<'.$data_tag.'>', '</'.$data_tag.'>');
			$data_string		= '<'.$data_tag.'>'.$data_string.'</'.$data_tag.'>';
		
			if(openssl_sign($data_string, $signature, $certificate_info['pkey'], 'sha1WithRSAEncryption')) {
				$signature  	= base64_encode($signature);
				$xml_unsigned	= simplexml_load_string($xml_dom);
				$xml_unsigned->gepgSignature = $signature;
				$xml_signed 	= $xml_unsigned->asXML();
				return trim(str_replace(array("\r", "\n"), '', $xml_signed));
			} else {
				echo $app->renderAlertError(openssl_error_string(), 'GEPG:addSignature');
			}
		} else {
			echo $app->renderAlertError(openssl_error_string(), 'GEPG:addSignature');
		}
	}

	public function verifySignature($xml_dom, $data_tag) {
		global $client, $root, $app;
 
		$signature_string	= $this->extractStringBetweenStrings($xml_dom, '<gepgSignature>', '</gepgSignature>');
		$data_string		= $this->extractStringBetweenStrings($xml_dom, '<'.$data_tag.'>', '</'.$data_tag.'>');
		$data_string		= '<'.$data_tag.'>'.$data_string.'</'.$data_tag.'>';

		$gepg_public_key   	= $root.CLIENT_PATH.'keys/gepg_server_public.pfx';
		$gepg_certificate   = file_get_contents($gepg_public_key);
		$signature 			= base64_decode($signature_string);

		if(openssl_pkcs12_read($gepg_certificate, $cert_info, $client->gepg_server_password)) {
			if(isset($cert_info['extracerts']['0'])) {
				$verified = openssl_verify($data_string, $signature, $cert_info['extracerts']['0']);
				return $data_string;
			} else {
				echo $app->renderAlertError(openssl_error_string(), 'GEPG:verifySignature');
				return false;
			}
		} else {
			echo $app->renderAlertError(openssl_error_string(), 'GEPG:verifySignature');
			return false;
		}
	}

	function getResponseStatus($xml_dom, $status_tag, $status_subtag='', $status_subsubtag='TrxStsCode') {
		global $root;

		if(isset($xml_dom->$status_tag->$status_subtag)) {
			$status_codes 	= $xml_dom->$status_tag->$status_subtag->$status_subsubtag;
		} else {
			$status_codes 	= $xml_dom->$status_tag->$status_subsubtag;
		}

		$status_codes 		= str_replace(' ', '', $status_codes);
		$status_codes 		= explode(';', $status_codes);

		$description_codes  = array();
		$codes_description 	= file_get_contents($root.'api/gepg/gepg_status_codes.txt');
		$codes_description	= explode("\n", $codes_description);

		foreach($codes_description as $code_description) {
			$description	= explode('=', $code_description);
			$code			= $description[0];
			$desc			= $description[1];

			if(in_array($code, $status_codes)) {
				$description_codes[$code]	= $desc;
			}
		}
		return $description_codes;
	}

	function responseSuccess($description_codes) {
		return array_key_exists('7101', $description_codes);
	}

	function requestControlNumber($invoice_id) {
		global $root, $app, $client, $user;

		$invoice 					= new Invoice($invoice_id);
		$phone 						= strlen($invoice->phone) ? '0'.substr($invoice->phone, -9) : '0'.substr($client->client_phone, -9);
		$email 						= strlen($invoice->email) ? $invoice->email : $client->client_email;

		$invoice_items 				= array();
		foreach($invoice->items() as $invoice_item) {
			if($invoice_item->price > 0) {
				$item_obj 				= new Item($invoice_item->item_id);
				$invoice_item_array['BillItem'] = array (
					'BillItemRef'		=> $item_obj->id,
					'UseItemRefOnPay'	=> 'N',
					'BillItemAmt'		=> $invoice_item->price,
					'BillItemEqvAmt'	=> $invoice_item->price,
					'BillItemMiscAmt'	=> 0,
					'GfsCode'			=> $item_obj->getCode('gfs_code')
				);
				$invoice_items[]		= $invoice_item_array;
			}
		}

		$invoice_data_array			= array (
			'gepgBillSubReq'		=> array (
				'BillHdr'			=> array (
					'SpCode'		=> $client->gepg_sp_code,
					'RtrRespFlg'	=> 'true'
				),
		
				'BillTrxInf'		=> array (
					'BillId'		=> $invoice->id,
					'SubSpCode'		=> $client->gepg_subsp_code,
					'SpSysId'		=> $client->gepg_system_id,
					'BillAmt'		=> $invoice->payable,
					'MiscAmt'		=> 0,
					'BillExprDt'	=> date('Y-m-d', strtotime('+ '.$client->gepg_bill_validity)).'T23:59:59',
					'PyrId'			=> $invoice->personnel_id,
					'PyrName'		=> $invoice->name,
					'BillDesc'		=> $invoice->subtype,
					'BillGenDt'		=> date('Y-m-d').'T'.date('H:i:s'),
					'BillGenBy'		=> $user->person()->name,
					'BillApprBy'	=> $user->person()->name,
					'PyrCellNum'	=> $phone,
					'PyrEmail'		=> $email,
					'Ccy'			=> $client->currency_code,
					'BillEqvAmt'	=> $invoice->payable,
					'RemFlag'		=> 'true',
					'BillPayOpt'	=> 3,
					'BillItems'		=> $invoice_items
				)
			)
		);

		$invoice_data_xml 			= $this->convertArrayToXML($invoice_data_array, 'Gepg');
		$invoice_data_xml 			= str_replace('</BillItems><BillItems>', '', $invoice_data_xml);
		$invoice_data_signed 		= $this->addSignature($invoice_data_xml, 'gepgBillSubReq');

		// Debug request
		file_put_contents($root.'api/gepg/debug/request_control_number_request.xml', $invoice_data_signed);

		$curl_options           	= array (
            'url'               	=> $client->gepg_server_address.'/api/bill/sigqrequest',
            'method'            	=> 'POST',
			'fields'    			=> $invoice_data_signed,
            'headers'   			=> array (
                'Content-Type'  	=> 'application/xml;charset=UTF-8',
                'Gepg-Com' 			=> 'default.sp.in',
				'Gepg-Code'			=> $client->gepg_sp_code,
				'Content-Length'	=> strlen($invoice_data_signed)
            )
        );

		$response 					= $app->curl($curl_options);
		if($response->status_code == 200) {
			
			// Debug response
			file_put_contents($root.'api/gepg/debug/request_control_number_response.xml', $response->message);
			$response_xml			= $this->verifySignature($response->message, 'gepgBillSubReqAck');
			$response_obj 			= simplexml_load_string($response->message);
			$status_codes 			= $this->getResponseStatus($response_obj, 'gepgBillSubReqAck', '', 'TrxStsCode');
			if($this->responseSuccess($status_codes)) {
				$invoice->changeProgress('Billed');
				$invoice->set('payment_channel', 'GEPG');
				
				// Check for ctrl no in 5 seconds
				sleep(5);
				$invoice->read();

				if(strlen($invoice->reference_no)) {
					echo $app->renderAlertSuccess('Control number received successfully', $invoice->reference_no, 'modal_lg', 1);
				} else {
					echo $app->renderAlertSuccess('Control number requested successfully', 'Waiting', 'modal_lg', 1);
				}
			} else {
				foreach(array_keys($status_codes) as $status_code) {
					echo $app->renderAlertError($status_code.' &middot; '.$status_codes[$status_code], 'GEPG Error');
				}
			}
		} else {
			$app->renderAlertError($response->message);
		}
	}

	function receiveControlNumber() {
		global $app, $root;

		// Get input data
		$raw_input	= fopen('php://input','r');
		$raw_xml 	= stream_get_contents($raw_input);

		// Debug raw data
		file_put_contents($root.'api/gepg/debug/receive_control_number_rawdata.xml', $raw_xml);
		$verified_xml			= $this->verifySignature($raw_xml, 'gepgBillSubResp');
		$xml_obj 				= simplexml_load_string($raw_xml);
		$xml_obj				= json_decode(json_encode($xml_obj));
		$status_codes 			= $this->getResponseStatus($xml_obj, 'gepgBillSubResp', 'BillTrxInf');

		if($this->responseSuccess($status_codes)) {
			$control_no 		= $xml_obj->gepgBillSubResp->BillTrxInf->PayCntrNum;
			$invoice_no 		= $xml_obj->gepgBillSubResp->BillTrxInf->BillId;
			$invoice			= new Invoice($invoice_no);

			if(is_numeric($control_no) && $control_no > 99) {
				$invoice->update (
					array (
						'reference'			=> 'Control no',
						'reference_no'  	=> $control_no,
						'billed_reference'	=> $control_no
					)
				);
			}

			$aknowledge_data_array	= array (
					'gepgBillSubRespAck'	=> array (
					'TrxStsCode' 			=> 7101
				)
			);
		} else {
			$aknowledge_data_array	= array (
					'gepgBillSubRespAck'	=> array (
					'TrxStsCode' 			=> 7201
				)
			);
		}

		// Acknowledge
		$aknowledge_data_xml 			= $this->convertArrayToXML($aknowledge_data_array, 'Gepg');
		$aknowledge_data_signed 		= $this->addSignature($aknowledge_data_xml, 'gepgBillSubRespAck');

		// Debug aknowledgment
		file_put_contents($root.'api/gepg/debug/receive_control_number_acknowledgement.xml', $aknowledge_data_signed);
		ob_end_clean();
		header("Content-type: text/xml");
		die($aknowledge_data_signed);
	}

	function cancelControlNumber($invoice) {
		global $root, $app, $client, $user;

		// Prepare invoice data
		$invoice_data_array		= array (
			'gepgBillCanclReq'	=> array (
				'SpCode'		=> $client->gepg_sp_code,
				'SpSysId'		=> $client->gepg_system_id,
				'CanclReasn'	=> 'Operation aborted',
				'BillId'		=> $invoice->id,
			)
		);

		$invoice_data_xml 			= $this->convertArrayToXML($invoice_data_array, 'Gepg');
		$invoice_data_signed 		= $this->addSignature($invoice_data_xml, 'gepgBillCanclReq');

		// Debug cancellation request
		file_put_contents($root.'api/gepg/debug/cancel_control_number_request.xml', $invoice_data_signed);

		$curl_options           	= array (
            'url'               	=> $client->gepg_server_address.'/api/bill/sigcancel_request',
            'method'            	=> 'POST',
			'fields'    			=> $invoice_data_signed,
            'headers'   			=> array (
                'Content-Type'  	=> 'application/xml;charset=UTF-8',
                'Gepg-Com' 			=> 'default.sp.in',
				'Gepg-Code'			=> $client->gepg_sp_code,
				'Content-Length'	=> strlen($invoice_data_signed)
            )
        );

		$response 					= $app->curl($curl_options);
		if($response->status_code == 200) {
			
			// Debug cancellation response
			file_put_contents($root.'api/gepg/debug/cancel_control_number_response.xml', $response->message);

			$response_xml			= $this->verifySignature($response->message, 'gepgBillCanclResp');
			$response_obj 			= simplexml_load_string($response->message);
			$status_codes 			= $this->getResponseStatus($response_obj, 'gepgBillCanclResp', 'BillCanclTrxDt', 'TrxStsCode');

			if(array_key_exists(7283, $status_codes)) {
				echo $app->renderAlertSuccess('Control number cancelled successfully');
				$invoice->changeProgress('Returned');
			} else {
				foreach(array_keys($status_codes) as $status_code) {
					echo $app->renderAlertError($status_code.' &middot; '.$status_codes[$status_code], 'GEPG Error');
				}
			}
		} else {
			$app->renderAlertError($response->message);
		}
	}

	function receivePayment() {
		global $app, $root;

		// Get input data
		$raw_input	= fopen('php://input','r');
		$raw_xml 	= stream_get_contents($raw_input);

		// Debug raw data
		file_put_contents($root.'api/gepg/debug/receive_payment_rawdata.xml', $raw_xml);
		$verified_xml			= $this->verifySignature($raw_xml, 'gepgPmtSpInfo');
		$xml_obj 				= simplexml_load_string($raw_xml);
		$xml_obj				= json_decode(json_encode($xml_obj));

		if(true) {
			$invoice_no 		= $xml_obj->gepgPmtSpInfo->PymtTrxInf->BillId;
			$reference_no 		= $xml_obj->gepgPmtSpInfo->PymtTrxInf->PayRefId;
			$control_no 		= $xml_obj->gepgPmtSpInfo->PymtTrxInf->PayCtrNum;
			$billed_amount 		= $xml_obj->gepgPmtSpInfo->PymtTrxInf->BillAmt;
			$paid_amount 		= $xml_obj->gepgPmtSpInfo->PymtTrxInf->PaidAmt;
			$payment_channel 	= $xml_obj->gepgPmtSpInfo->PymtTrxInf->UsdPayChnl;
			$payment_receipt 	= $xml_obj->gepgPmtSpInfo->PymtTrxInf->PspReceiptNumber;
			$payment_bank 		= $xml_obj->gepgPmtSpInfo->PymtTrxInf->PspName;
			$payment_account 	= $xml_obj->gepgPmtSpInfo->PymtTrxInf->CtrAccNum;

			if(is_numeric($invoice_no) && is_numeric($control_no)) {
				$invoice		= new Invoice($invoice_no);
				$payment_data 	= array (
					'payment'			=> 1,
					'payment_method'	=> 'GEPG',
					'payment_channel'	=> $payment_channel,
					'payment_reference'	=> $reference_no,
					'payment_receipt'	=> $payment_receipt,
					'payment_name'		=> $payment_bank,
					'payment_notes'		=> $payment_account
				);
				if(!$invoice->payment) {
					$paid 			= $invoice->pay($payment_data);
				}
			}

			$aknowledge_data_array	= array (
					'gepgPmtSpInfoAck'	=> array (
					'TrxStsCode' 		=> 7101
				)
			);
		} else {
			$aknowledge_data_array	= array (
					'gepgPmtSpInfoAck'	=> array (
					'TrxStsCode' 		=> 7201
				)
			);
		}

		// Acknowledge
		$aknowledge_data_xml 			= $this->convertArrayToXML($aknowledge_data_array, 'Gepg');
		$aknowledge_data_signed 		= $this->addSignature($aknowledge_data_xml, 'gepgPmtSpInfoAck');

		// Debug aknowledgment
		file_put_contents($root.'api/gepg/debug/receive_payment_acknowledgement.xml', $aknowledge_data_signed);
		ob_end_clean();
		header("Content-type: text/xml");
		die($aknowledge_data_signed);
	}

	function requestReconciliation($reconciliation_id, $reconciliation_date, $reconciliation_option) {
		global $root, $app, $client, $user;
		$date = date('Y-m-d', strtotime($reconciliation_date)).' 23:59:59';

		// Prepare recon data
		$reconciliation_data_array	= array (
			'gepgSpReconcReq'		=> array (
				'SpReconcReqId' 	=> $reconciliation_id,
				'SpCode'			=> $client->gepg_sp_code,
				'SpSysId'			=> $client->gepg_system_id,
				'TnxDt'         	=> $reconciliation_date,
				'ReconcOpt'     	=> $reconciliation_option
			)
		);

		$reconciliation_data_xml 	= $this->convertArrayToXML($reconciliation_data_array, 'Gepg');
		$reconciliation_data_signed = $this->addSignature($reconciliation_data_xml, 'gepgSpReconcReq');
		
		// Debug recon request
		file_put_contents($root.'api/gepg/debug/request_reconciliation_request.xml', $reconciliation_data_signed);

		$curl_options           	= array (
            'url'               	=> $client->gepg_server_address.'/api/reconciliations/sig_sp_qrequest',
            'method'            	=> 'POST',
			'fields'    			=> $reconciliation_data_signed,
            'headers'   			=> array (
                'Content-Type'  	=> 'application/xml;charset=UTF-8',
                'Gepg-Com' 			=> 'default.sp.in',
				'Gepg-Code'			=> $client->gepg_sp_code,
				'Content-Length'	=> strlen($reconciliation_data_signed)
            )
        );

		$response 					= $app->curl($curl_options);
		if($response->status_code == 200) {
			
			// Debug recon response
			file_put_contents($root.'api/gepg/debug/request_reconciliation_response.xml', $response->message);

			$response_xml			= $this->verifySignature($response->message, 'gepgSpReconcReqAck');
			$response_obj 			= simplexml_load_string($response->message);
			$status_codes 			= $this->getResponseStatus($response_obj, 'gepgSpReconcReqAck', '', 'ReconcStsCode');

			if($this->responseSuccess($status_codes)) {
				echo $app->renderAlertSuccess('Reconciliation for the day ended '.$date.' requested successfully');
			} else {
				foreach(array_keys($status_codes) as $status_code) {
					echo $app->renderAlertError($status_code.' &middot; '.$status_codes[$status_code], 'GEPG Error');
				}
			}
		} else {
			$app->renderAlertError($response->message);
		}
	}

	function receiveReconciliation() {
		global $app, $root, $now, $notification;

		// Get input data
		$raw_input	= fopen('php://input','r');
		$raw_xml 	= stream_get_contents($raw_input);

		// Debug raw data
		file_put_contents($root.'api/gepg/debug/receive_reconciliation_rawdata.xml', $raw_xml);
		//$verified_xml			= $this->verifySignature($raw_xml, 'gepgSpReconcResp');

		$xml_obj 				= simplexml_load_string($raw_xml);
		$xml_obj				= json_decode(json_encode($xml_obj));

		if(isset($xml_obj->gepgSpReconcResp)) {
			$status_codes 		= $this->getResponseStatus($xml_obj, 'gepgSpReconcResp', 'ReconcBatchInfo', 'ReconcStsCode');
		} else {
			$status_codes 		= $this->getResponseStatus($xml_obj, 'ReconcBatchInfo', '', 'ReconcStsCode');
		}

		if($this->responseSuccess($status_codes)) {

			$obj 				= new Table('accounting_gepg_reconciliation');
			$rows 				= $xml_obj->gepgSpReconcResp->ReconcTrans;
			$receive_date 		= date('Y-m-d');

			foreach($rows as $row) {

				$payer_name		= strlen($row->DptName) > 3 ? $row->DptName : $row->ReconcRsv1;
				$create_data	= array (
					'id'				=> 0,
					'request_id'		=> $xml_obj->gepgSpReconcResp->ReconcBatchInfo->SpReconcReqId,
					'bill_id'			=> $row->SpBillId,
					'control_no'		=> $row->BillCtrNum,
					'parent_id'	=> $row->pspTrxId,
					'amount_paid'		=> $row->PaidAmt,
					'currency'			=> $row->CCy,
					'payer_reference'	=> $row->PayRefId,
					'transaction_date'	=> $row->TrxDtTm,
					'account_no'		=> $row->CtrAccNum,
					'payer_channel'		=> is_object($row->UsdPayChnl) ? '' : $row->UsdPayChnl,
					'bank_name'			=> $payer_name,
					'bank_code'			=> $row->PspCode,
					'payer_phone'		=> is_object($row->DptCellNum) ? '' : $row->DptCellNum,
					'payer_name'		=> is_object($row->DptName) ? '' : $app->cleanText($row->DptName),
					'payer_email'		=> is_object($row->DptEmailAddr) ? '' : $row->DptEmailAddr,
					'remarks'			=> is_object($row->Remarks) ? '' : $app->cleanText($row->Remarks),
					'reserved1'			=> is_object($row->ReconcRsv1) ? '' : $app->cleanText($row->ReconcRsv1),
					'reserved2'			=> is_object($row->ReconcRsv2) ? '' : $app->cleanText($row->ReconcRsv2),
					'reserved3'			=> is_object($row->ReconcRsv3) ? '' : $app->cleanText($row->ReconcRsv3),
					'f1'				=> 0,
					'f2'				=> 0,
					'f3'				=> 0,
					'f4'				=> 0,
					'f5'				=> 0,
					'f6'				=> 0,
					'f7'				=> 0,
					'f8'				=> 0,
					'f9'				=> 0,
					'created_user_id'	=> 0,
					'created_datetime'	=> $now,
					'modified_user_id'	=> 0,
					'modified_datetime'	=> 0,
					'status'			=> 1
				);

				$obj->create($create_data);
			}
			
			$aknowledge_data_array		= array (
				'gepgSpReconcRespAck'	=> array(
					'ReconcStsCode' 	=> 7101
				)
			);
		} else {

			// Push failed notification

			$alert_content 			= '<h6>Reconciliation request failed with the following error(s):</h6>';
			foreach(array_keys($status_codes) as $status_code) {
				$alert_content 		.= $status_code.' &middot; '.$status_codes[$status_code];
			}

			$notify_data            = array (
				'priority'          => 1,
				'section'           => 'GEPG',
				'section_id'        => $xml_obj->gepgSpReconcResp->ReconcBatchInfo->SpReconcReqId,
				'subsection'        => '',
				'subsection_id'     => 0,
				'title'             => 'GEPG Reconciliation',
				'subtitle'          => 'Reconciliation request failed',
				'action1'           => '<div class="col-12 col-lg-6 col-xl-4 col-xxl-3 py-2"> <a href="__ROOT__pages/accounting/gepg_reconciliation" class="btn w-100 btn-outline-primary fw-normal"> <span class="fa-light fa-info-circle fa-fw"></span> Try again </a> </div>',
				'alert_content'     => $alert_content,
				'personnel_id'      => 1
			);
		
			$notification->push($notify_data);

			$aknowledge_data_array		= array (
				'gepgSpReconcRespAck'	=> array(
					'ReconcStsCode' 	=> 7201
				)
			);
		}

		// Acknowledge
		$aknowledge_data_xml 			= $this->convertArrayToXML($aknowledge_data_array, 'Gepg');
		$aknowledge_data_signed 		= $this->addSignature($aknowledge_data_xml, 'gepgSpReconcRespAck');

		// Debug aknowledgment
		file_put_contents($root.'api/gepg/debug/receive_reconciliation_acknowledgement.xml', $aknowledge_data_signed);
		ob_end_clean();
		header("Content-type: text/xml");
		die($aknowledge_data_signed);
	}

}

$gepg   = new GEPG();

?>