<?php

$root 	= '../../'; 
$submod = 'sales';
require_once($root.'config.php'); 
require_once($root.'res/classes/Sale.php'); 
require_once('GEPG.php');

$gepg 		= new GEPG();
$sale 		= new Sale($id);

// If repay, pull GFS code from original sale
if($sale->name 	== 'Repay') {
	$parent_sale = new Sale($sale->name_id);
	$sale_name	= $parent_sale->name;
} else {
	$sale_name	= $sale->name;
}

// Convert Wallet sale to Direct sale
if($sale_name == 'Wallet') $sale_name = 'Sale';

// Get GFS code from Sale name
$category 	= $main->getCategoryByCode('Income', $sale_name);
$gfs_code	= $category->f1;

// Prepare bill header with no items
$data		= array (
	'gepgBillSubReq'		=> array(

		'BillHdr'			=> array(
			'SpCode'		=> $com->gepg_sp_code,
			'RtrRespFlg'	=> 'true'
		),

		'BillTrxInf'		=> array(
			'BillId'		=> $id,
			'SubSpCode'		=> $com->gepg_sub_sp_code,
			'SpSysId'		=> $com->gepg_sp_system_id,
			'BillAmt'		=> $sale->payable,
			'MiscAmt'		=> 0,
			'BillExprDt'	=> date('Y-m-d', strtotime('+ '.$com->gepg_bill_validity)).'T23:59:59',
			'PyrId'			=> $sale->person_id,
			'PyrName'		=> $sale->person_name,
			'BillDesc'		=> $main->translate($sale->name),
			'BillGenDt'		=> date('Y-m-d').'T'.date('H:i:s'),
			'BillGenBy'		=> $sale->create_staff->name,
			'BillApprBy'	=> $sale->create_staff->name,
			'PyrCellNum'	=> $sale->person->phone,
			'PyrEmail'		=> $com->email,
			'Ccy'			=> $com->currency,
			'BillEqvAmt'	=> $sale->payable,
			'RemFlag'		=> 'true',
			'BillPayOpt'	=> 3,
			'BillItems'		=> array()
		)
	)
);

// Generate xml
$unsigned_xml 			= new SimpleXMLElement("<Gepg></Gepg>", 1);
$main->array2xml($data, $unsigned_xml);

// Clean xml
$raw_xml				= trim(str_replace(array("\r", "\n"), '', $unsigned_xml->asXML()));

// Add multiple items
// $bill_items 	= '<BillItem><BillItemRef>'.$id.'</BillItemRef><UseItemRefOnPay>N</UseItemRefOnPay><BillItemAmt>'.$sale->payable.'</BillItemAmt><BillItemEqvAmt>'.$sale->payable.'</BillItemEqvAmt><BillItemMiscAmt>0</BillItemMiscAmt><GfsCode>'.$gfs_code.'</GfsCode></BillItem>';

$bill_items 	= '';
foreach($sale->items() as $item) {
	$item_obj 	= new Item($item->item_id);
	$gfs		= is_numeric($item_obj->tags) ? $item_obj->tags : $gfs_code;

	$bill_item 	= '<BillItem><BillItemRef>'.$item->item_id.'</BillItemRef><UseItemRefOnPay>N</UseItemRefOnPay><BillItemAmt>'.$item->price.'</BillItemAmt><BillItemEqvAmt>'.$item->price.'</BillItemEqvAmt><BillItemMiscAmt>0</BillItemMiscAmt><GfsCode>'.$gfs.'</GfsCode></BillItem>';
	$bill_items .= $bill_item;
}
$raw_xml				= str_replace('<BillItems/>', '<BillItems>'.$bill_items.'</BillItems>', $raw_xml);

// Sign xml
$data_string			= $gepg->getDataString($raw_xml, 'gepgBillSubReq');
$signature				= $gepg->sign($data_string);

// Add signature to data array
$signed_xml				= '<?xml version="1.0" encoding="UTF-8" standalone="yes"?><Gepg>'.$data_string.'<gepgSignature>'.$signature.'</gepgSignature></Gepg>';

// Save signed xml in debug folder
file_put_contents('debug/request_control_number_request.xml', $signed_xml);

// Send request
$ch = curl_init($com->gepg_address.'/api/bill/sigqrequest');

curl_setopt($ch, CURLOPT_POST, true);
curl_setopt($ch, CURLOPT_POSTFIELDS, $signed_xml);
curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
curl_setopt($ch, CURLOPT_HTTPHEADER, array(
'Content-Type: application/xml',
'Gepg-Com:default.sp.in',
'Gepg-Code:'.$com->gepg_sp_code,
'Content-Length:'.strlen($signed_xml)));

$response = curl_exec($ch);
curl_close($ch);

// Save received response in debug folder
file_put_contents('debug/request_control_number_response.xml', $response);

// Did we get any response?

if($response) {
	// Get data and signature from response
	$data_string	= $gepg->getDataString($response, 'gepgBillSubReqAck');
	$signature		= $gepg->getSignatureString($response, 'gepgSignature');
	$verified 		= $gepg->verify($data_string, $signature);

	if($verified) {
		$response_xml 	= simplexml_load_string($response);
		$status_codes 	= $response_xml->gepgBillSubReqAck->TrxStsCode;
		$status_codes 	= (array) $status_codes;
		$status_codes 	= $status_codes[0];
		$status_codes_array	= explode(';', $status_codes);

		if(in_array('7101', $status_codes_array)) {
			$update_data = array(
				'bill_staff_id'		=> $staff_id,
				'bill_date'			=> $now,
				'status'			=> 'Requesting'
			); 

			$sale->update($update_data);
			
			sleep(5);

			$sale->fetch();
			echo $sale->status();
		} else {

			$message		= '<h4 class="font-16 margin-b-15">While requesting control number, Emedical received the following errors from GEPG</h4><ol>';
			foreach($gepg->translateCodes($status_codes_array) as $error) {
				$message	.= '<li>'.$error['code'].' - '.$error['desc'].'</li>';
			}
			$message		.= '</ol>';
			echo $main->renderPanel('warning', '<span class="fa fa-exclamation-triangle fa-fw"></span> MESSAGE FROM GEPG', $message);
		}
	} else {
		echo $main->renderPanel('danger', '<span class="fa fa-exclamation-triangle fa-fw"></span> VERIFICATION ERROR', 'Emedical cannot verify the received response from GEPG.');
	}
} else {
	echo $main->renderPanel('danger', '<span class="fa fa-exclamation-triangle fa-fw"></span> CONNECTION ERROR', 'Emedical cannot communicate with GEPG at this time.');
}

?>