<?php

$root 	= '../../'; 
$submod = 'sales';
require_once($root.'config.php'); 
require_once('GEPG.php');
ob_end_clean();

$gepg 	        = new GEPG();
$request_date   = isset($date) ? $date : date('Y-m-d', strtotime('Yesterday'));
$request_id		= str_replace('-', '', $request_date);
$request_id		= $request_id.date('Hi');

$data	= array (
	'gepgSpReconcReq'	=> array (
        'SpReconcReqId' => $request_id,
        'SpCode'		=> $com->gepg_sp_code,
        'SpSysId'		=> $com->gepg_sp_system_id,
        'TnxDt'         => $request_date,
        'ReconcOpt'     => 1
	)
);

// Generate xml
$unsigned_xml 			= new SimpleXMLElement("<Gepg></Gepg>", 1);
$main->array2xml($data, $unsigned_xml);

// Get and sign data string
$raw_xml				= trim(str_replace(array("\r", "\n"), '', $unsigned_xml->asXML()));			
$data_string			= $gepg->getDataString($raw_xml, 'gepgSpReconcReq');
$signature				= $gepg->sign($data_string);

// Add signature to data array
$signed_xml				= '<?xml version="1.0" encoding="UTF-8" standalone="yes"?><Gepg>'.$data_string.'<gepgSignature>'.$signature.'</gepgSignature></Gepg>';

// Save signed xml in debug folder
file_put_contents('debug/request_reconciliation_request.xml', $signed_xml);

// Send request
$ch = curl_init($com->gepg_address.'/api/reconciliations/sig_sp_qrequest');
curl_setopt($ch, CURLOPT_POST, true);
curl_setopt($ch, CURLOPT_POSTFIELDS, $signed_xml);
curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
curl_setopt($ch, CURLOPT_HTTPHEADER, array(
'Content-Type: application/xml',
'Gepg-Com:default.sp.in',
'Gepg-Code:'.$com->gepg_sp_code,
'Content-Length:'.strlen($signed_xml)));

$response = curl_exec($ch);
curl_close($ch);

// Save signed xml in debug folder
file_put_contents('debug/request_reconciliation_response.xml', $response);

if($response) {
	// Get data and signature from response
	$data_string	= $gepg->getDataString($response, 'gepgSpReconcReqAck');
	$signature		= $gepg->getSignatureString($response, 'gepgSignature');
	$verified 		= $gepg->verify($data_string, $signature);

	if($verified) {
		$response_xml 	= simplexml_load_string($response);
		$status_codes 	= $response_xml->gepgSpReconcReqAck->ReconcStsCode;
		$status_codes_array	= explode(';', $status_codes);

		if(in_array('7101', $status_codes_array)) {
			// Send notification
			$str = "SELECT staff_id FROM ppl_staff WHERE tags LIKE '%acc%' ";
			foreach($main->select($str) as $row) {
				$alert_data = array (
					'name'				=> 'Reconciliation',
					'name_id'			=> $request_id,
					'title'				=> 'Reconciliation requested',
					'description'		=> 'Reconciliation for the day ended '.$main->evening($request_date).' success: <b>'.$gepg->translateCodes($status_codes_array, 1).'</b>',
					'link'				=> 'Check reconciliation|acc/reconciliation/?sd='.$request_date,
					'staff_id'			=> $row['staff_id'],
					'end_date'			=> date('Y-m-d H:i:s', strtotime(' + 1 day ')),
					'status'			=> 2,
					'priority'			=> 4
				);
				$alert->push($alert_data);
			}
		 
			echo $main->renderPanel('success', '<span class="fa fa-check fa-fw"></span> RECONCILIATION REQUESTED', 'GEPG reconciliation for the day ended '.$request_date.' has been requested successfully.');
		} else {
			// Send notification
			$str = "SELECT staff_id FROM ppl_staff WHERE tags LIKE '%acc%' ";
			foreach($main->select($str) as $row) {
				$alert_data = array (
					'name'				=> 'Reconciliation',
					'name_id'			=> $request_id,
					'title'				=> 'Reconciliation request failed',
					'description'		=> 'Reconciliation for the day ended '.$main->evening($request_date).' failed <b>'.$gepg->translateCodes($status_codes_array, 1).'</b>',
					'link'				=> 'Check reconciliation|acc/reconciliation/?sd='.$request_date,
					'staff_id'			=> $row['staff_id'],
					'end_date'			=> date('Y-m-d H:i:s', strtotime(' + 1 day ')),
					'status'			=> 2,
					'priority'			=> 3
				);
				$alert->push($alert_data);
			}
			$message		= 'While requesting Reconciliation, Emedical received the following errors:<ol>';
			foreach($gepg->translateCodes($status_codes_array) as $error) {
				$message	.= '<li>'.$error['code'].' - '.$error['desc'].'</li>';
			}
			$message		.= '</ol>';
			echo $main->renderPanel('warning', '<span class="fa fa-exclamation-triangle fa-fw"></span> MESSAGE FROM GEPG', $message);
		}
	} else {
		echo $main->renderPanel('danger', '<span class="fa fa-exclamation-triangle fa-fw"></span> VERIFICATION ERROR', 'Emedical cannot verify the received response from GEPG.');
	}
} else {
	echo $main->renderPanel('danger', '<span class="fa fa-exclamation-triangle fa-fw"></span> CONNECTION ERROR', 'Emedical cannot communicate with GEPG at this time.');
}

?>