<?php 

class NHIF extends Insurance {
	function __construct($id=0) {		
		$this->table        = 'billing_insurances';
		$this->primary_key  = 'id';
		parent::__construct($id);
	}

	public function getToken($token_url) {
        global $app, $client;

		$username 			    = $this->username == 'NHIF_USERNAME' ? 'integrationuser' 	: $client->nhif_api_username;
		$password 			    = $this->password == 'NHIF_PASSWORD' ? 'nhif@2018'			: $client->nhif_api_password;

        $curl_options           = array(
            'method'            => 'POST',
            'url'               => $token_url,
            'fields'            => array (
                'grant_type'    => 'password',
                'username'      => $username,
                'password'      => $password
            )
        );

        $token_json             = $app->curl($curl_options);

        if($token = json_decode($token_json->message)) {
            return $token;
        } else {
            die($app->renderAlertError('Could not get access token'));
        }
	}

	public function getCardDetails($card_no) { 
		global $app, $client;

        $card_details_url      	= $client->auth_service_url.'verification/GetCardDetails?CardNo='.$card_no;
        $token	                = $this->getToken($client->auth_token_url);

        $curl_options           = array (
            'url'               => $card_details_url,
            'method'            => 'GET',
            'headers'   		=> array (
                'Content-Type'  => 'application/x-www-form-urlencoded;charset=UTF-8',
                'Authorization' => $token->token_type.' '.$token->access_token
            )
        );

		return $app->curl($curl_options);
	}
	
	public function authorizeCard($card_no, $visit_type_id, $referral_no=0, $remarks='') {
		global $app, $client;

        $authorization_url      = $client->auth_service_url.'verification/AuthorizeCard?CardNo='.$card_no.'&VisitTypeID='.$visit_type_id.'&ReferralNo='.$referral_no.'&Remarks='.urlencode($remarks);
        
		$token	                = $this->getToken($client->auth_token_url);

        $curl_options           = array (
            'url'               => $authorization_url,
            'method'            => 'GET',
            'headers'   		=> array (
                'Content-Type'  => 'application/x-www-form-urlencoded;charset=UTF-8',
                'Authorization' => $token->token_type.' '.$token->access_token
            )
        );

		return $app->curl($curl_options);
	}

	public function getPricePackageWithExcludedServices() {
		global $app, $client;

        $price_package_url      = $client->claim_service_url.'Packages/GetPricePackageWithExcludedServices?FacilityCode='.$client->nhif_facility_code;
        $token	                = $this->getToken($client->claim_token_url);

        $curl_options           = array (
            'url'               => $price_package_url,
            'method'            => 'GET',
			'timeout'			=> 900,
            'headers'   		=> array (
                'Content-Type'  => 'application/x-www-form-urlencoded;charset=UTF-8',
                'Authorization' => $token->token_type.' '.$token->access_token
            )
        );

		return $app->curl($curl_options);
	}

	function getCreateScheme($code, $name) {
		global $app;

		$sql = "SELECT id FROM billing_insurance_schemes WHERE code = '$code' OR name = '$name' ";
		foreach($app->read($sql) as $row) {
			return new Scheme($row->id);
		}

		$sql = "SELECT id FROM billing_pricelists WHERE scheme = '$code' ";
		foreach($app->read($sql) as $row) {
			$pricelist_found = 1;
			$pricelist  = new Pricelist($row->id);
		}

		if(!$pricelist_found) {
			die($app->renderAlertError('No pricelist found with scheme: '.$code));
		}

		$scheme 					= new Scheme();
		$scheme_data    			= $scheme->addCommonFields (
			array (
				'code'              => $code,
				'name'              => $name,
				'details'           => '',
				'insurance'         => $this->code,
				'insurance_id'      => $this->id,
				'pricelist'         => $pricelist->code,
				'pricelist_id'      => $pricelist->id,
				'icon'              => 'fa-light fa-credit-card',
				'color'             => 'black',
				'tags'              => ''
			)
        );
        return new Scheme($scheme->create($scheme_data));
	}

	public function submitFolio($folio_id) {
		global $app, $client;

		$folio 					= new Folio($folio_id);
		$folio_array 			= $folio->getFolioArray();
		$data_string			= json_encode($folio_array);
		$claim_submission_url   = $client->claim_service_url.'claims/SubmitFolios';
        $token	                = $this->getToken($client->claim_token_url);

        $curl_options           = array (
            'url'               => $claim_submission_url,
            'method'            => 'POST',
			'fields'            => $data_string,
			'timeout'			=> 900,
            'headers'   		=> array (
                'Content-Type'  => 'Content-Type: application/json',
                'Authorization' => $token->token_type.' '.$token->access_token
            )
        );

		return $app->curl($curl_options);
	}

	public function getSubmittedFolios($claim_year, $claim_month) {
		global $app, $client;

		$token	                = $this->getToken($client->claim_token_url);
		$claim_submission_url   = $client->claim_service_url.'claims/getSubmittedClaims?FacilityCode='.$client->nhif_facility_code.'&ClaimYear='.$claim_year.'&ClaimMonth='.$claim_month;
		
		$curl_options           = array (
            'url'               => $claim_submission_url,
            'method'            => 'GET',
			'timeout'			=> 900,
            'headers'   		=> array (
                'Content-Type'  => 'Content-Type: application/x-www-form-urlencoded;charset=UTF-8',
                'Authorization' => $token->token_type.' '.$token->access_token
            )
        );

		return $app->curl($curl_options);
	}

	function syncSubmittedFolios($claim_year=0, $claim_month=0) {
		global $app, $user_id, $now, $root;
		
		$claim_year         = $claim_year ? $claim_year : date('Y');
		$claim_month        = $claim_month ? $claim_month : date('m');
		$claim_date 		= $claim_year.'-'.$claim_month.'-01';
		$submitted_folios   = $this->getSubmittedFolios($claim_year, $claim_month);

		$html 				= '';
		if($submitted_folios->status_code == '200') {
			$folios 		= json_decode($submitted_folios->message);
			$payable 		= 0;

			foreach($folios as $folio) {
				if(0) {
					$folio_id 	= $folio->BillNo;
				} else {
					$folio_id 	= $app->getColumnValue('billing_folios', 'id', 'authorization_no', $folio->AuthorizationNo);
				}

				$payable	+= $folio->AmountClaimed;
				$folio_obj 	= new Folio($folio_id);
				$notes 		= json_encode($folio);

				if($folio_obj->id > 0 && !$folio_obj->received) {
					$folio_obj->changeProgress('Received', 'received_quantity', $notes, '10');

					$folio_obj->update (
						array (
							'year'				=> $folio->ClaimYear,
							'month'				=> $folio->ClaimMonth,
							'received_datetime'	=> substr(str_replace('T', ' ', $folio->DateSubmitted), 0, 19),
							'reference'			=> 'SubmissionNo',
							'reference_no'		=> $folio->SubmissionNo,
							'received_notes'	=> $notes,
							'received_user_id'	=> $user_id,
							'payable'			=> $folio->AmountClaimed,
							'progress_datetime'	=> $now
						)
					);
				} else {
					echo 'Invalid folio id '.$folio_id.'<br />';
				}
			}

			file_put_contents($root.CLIENT_PATH.'cache/nhif_received_folios_'.$claim_year.'_'.$claim_month.'.json', $submitted_folios->message);

			$html			.= '<div class="border p-3 mb-3 rounded bg-success" style="--bs-bg-opacity: .1;">
			<h5 class="fw-bold"> Success '.$submitted_folios->status_code.' </h5>
			<p>'.number_format(count($folios)).' folios amount TZS '.$app->formatMoney($payable).' delivered to NHIF in '.date('F Y', strtotime($claim_date)).'</p>
			</div>';
		} else {    
			$html			.= '<div class="border p-3 mb-3 rounded bg-danger" style="--bs-bg-opacity: .1;">
			<h5 class="fw-bold"> Error '.$submitted_folios->status_code.' </h5>
			<p>'.$app->cleanText($submitted_folios->message).'</p>
			</div>';
		}

		return $html;
	}
}

?>