<?php 

#[AllowDynamicProperties]

class NMB {

	function getToken() {
        global $app, $client;

		$url 					= $client->nmb_server_address.'/v2/auth';
		$body 				    = json_encode (
			array (
				'username'		=> $app->uncleanText($client->nmb_api_username),
				'password'		=> $app->uncleanText($client->nmb_api_password)
			)
		);

        $curl_options           = array (
            'method'            => 'POST',
            'url'               => $url,
			'fields'            => $body,
			'headers'   		=> array (
                'Content-Type'  => 'application/json'
            )
        );

        $token_json             = $app->curl($curl_options);

        if($token = json_decode($token_json->message)) {
            return $token->token;
        } else {
            die($app->renderAlertError('Could not get access token'));
        }
	}

	/**
	 * Get control number and updates invoice
	 *
	 * @param integer $invoice_id   Invoice id
	 * @return null
	*/ 
	function requestControlNumber(int $invoice_id) {
		global $app, $client;
		$invoice 				= new Invoice($invoice_id);

		$data 					= array (
			'reference'			=> $client->nmb_invoice_prefix.$invoice->id,
			'student_name'		=> $invoice->name,
			'student_id'		=> $invoice->id,
			'amount'			=> $invoice->payable,
			'type'				=> $invoice->subtype,
			'code'				=> 10,
			'allow_partial'		=> FALSE,
			'callback_url'		=> $client->nmb_invoice_callback,
			'token'				=> $this->getToken()
		);

		$curl_options           = array (
            'method'            => 'POST',
            'url'               => $client->nmb_server_address.'/v2/invoice_submission',
			'fields'            => json_encode($data),
			'headers'   		=> array (
                'Content-Type'  => 'application/json'
            )
        );

		$control           		= $app->curl($curl_options);
		$message 				= json_decode($control->message);

		if($control->status_code == 200) { 
			if($message->description == 'Success') {
				$control_number         = $client->nmb_invoice_prefix.$invoice->id;
				$bill_data 		        = array (
					'reference'			=> 'Control no',
					'reference_no'		=> $control_number,
					'billed_reference'	=> $control_number
				);
				$invoice->bill($bill_data);
				echo $app->renderAlertSuccess('Control number received successfully', $control_number, 'modal_lg', 1);
			} else {
				echo $app->renderAlertWarning($message->description);
			}
		} else {
			echo $app->renderAlertError($message->description);
		}
	}
	
	function receivePayment() {
		global $app, $client;

		// Get input data
		$raw_input	        = fopen('php://input','r');
		$raw_json 	        = stream_get_contents($raw_input);
		$filename           = 'debug/'.date('YmdHis').'.json';
		file_put_contents($filename, $raw_json);

		$data 				= json_decode($raw_json);
		$invoice_id 		= $app->getColumnValue('billing_invoices', 'id', 'reference', $data->reference);

		if($invoice_id) {
			$invoice 				= new Invoice($invoice_id);
			$payment_data 			= array (
				'payment'			=> 1,
				'payment_method'	=> 'NMB',
				'payment_channel'	=> 'NMB',
				'payment_receipt'	=> $data->receipt,
				'payment_reference'	=> $data->receipt,
				'payment_account'	=> $data->account_number
			);
			$paid 					= $invoice->pay($payment_data);
			if($paid) {
				$invoice->set('payment_reference', $data->receipt);
				$confirmed			= $this->confirmPayment($invoice_id);
			}
		} else {
			$result             = json_encode (
				array (
					'status'    => 0,
					'message'   => 'Invoice does not exist'
				)
			);
	
			header('Content-Type: application/json');
			echo $result;
		}
	}

	function confirmPayment($invoice_id) {
		global $app, $client;

		$invoice 				= new Invoice($invoice_id);

		$data 					= array (
			'reference'			=> $client->nmb_invoice_prefix.$invoice->id,
			'student_name'		=> $invoice->name,
			'student_id'		=> $invoice->id,
			'amount'			=> 0,
			'type'				=> $invoice->subtype,
			'code'				=> 10,
			'allow_partial'		=> FALSE,
			'callback_url'		=> $client->nmb_invoice_callback,
			'token'				=> $this->getToken()
		);

		$curl_options           = array (
            'method'            => 'POST',
            'url'               => $client->nmb_server_address.'/v2/invoice_update',
			'fields'            => json_encode($data),
			'headers'   		=> array (
                'Content-Type'  => 'application/json'
            )
        );
		$control           		= $app->curl($curl_options);
		$message 				= json_decode($control->message);

		$result             	= json_encode (
			array (
				'status'    	=> 1,
				'message'   	=> 'Success'
			)
		);
		
		header('Content-Type: application/json');
		echo $result;
	}
}

$nmb = new NMB();

?>