<?php

class WebSocket {

	function listen(string $port) {
		$chat_socket = socket_create(AF_INET, SOCK_STREAM, SOL_TCP);
		socket_set_option($chat_socket, SOL_SOCKET, SO_REUSEADDR, 1);
		socket_bind($chat_socket, '0.0.0.0', $port);
		socket_listen($chat_socket);
		return $chat_socket;
	}

	function send($message) {
		global $client_socket_array;
		$message_length = strlen($message);
		foreach($client_socket_array as $client_socket) {
			@socket_write($client_socket,$message,$message_length);
		}
		return true;
	}

	function unseal($socket_data) {
		$length 	= ord($socket_data[1]) & 127;
		if($length == 126) {
			$masks 	= substr($socket_data, 4, 4);
			$data 	= substr($socket_data, 8);
		} elseif($length == 127) {
			$masks 	= substr($socket_data, 10, 4);
			$data 	= substr($socket_data, 14);
		} else {
			$masks 	= substr($socket_data, 2, 4);
			$data 	= substr($socket_data, 6);
		}
		$socket_data = '';
		for ($i = 0; $i < strlen($data); ++$i) {
			$socket_data .= $data[$i] ^ $masks[$i%4];
		}
		return $socket_data;
	}

	function seal($socket_data) {
		$b1 = 0x80 | (0x1 & 0x0f);
		$length = strlen($socket_data);
		
		if($length <= 125)
			$header = pack('CC', $b1, $length);
		elseif($length > 125 && $length < 65536)
			$header = pack('CCn', $b1, 126, $length);
		elseif($length >= 65536)
			$header = pack('CCNN', $b1, 127, $length);
		return $header.$socket_data;
	}

	function doHandshake($received_header,$client_socket_resource, $host_name, $port) {
		$headers	= array();
		$lines 		= preg_split("/\r\n/", $received_header);
		foreach($lines as $line) {
			$line 	= chop($line);
			if(preg_match('/\A(\S+): (.*)\z/', $line, $matches)) {
				$headers[$matches[1]] = $matches[2];
			}
		}

		$sec_key 	= $headers['Sec-WebSocket-Key'];
		$sec_accept = base64_encode(pack('H*', sha1($sec_key . '258EAFA5-E914-47DA-95CA-C5AB0DC85B11')));
		$buffer  	= "HTTP/1.1 101 Web Socket Protocol Handshake\r\n" .
		"Upgrade: websocket\r\n" .
		"Connection: Upgrade\r\n" .
		"WebSocket-Origin: $host_name\r\n" .
		"WebSocket-Location: ws://$host_name:$port/demo/shout.php\r\n".
		"Sec-WebSocket-Accept:$sec_accept\r\n\r\n";
		socket_write($client_socket_resource,$buffer,strlen($buffer));
	}
	
	function newConnectionACK($client_ip_address) {
		$message 		= 'New client ' . $client_ip_address.' joined';
		$message_array 	= array('message'=>$message,'message_type'=>'chat-connection-ack');
		$ACK 			= $this->seal(json_encode($message_array));
		return $ACK;
	}
	
	function connectionDisconnectACK($client_ip_address) {
		$message 		= 'Client ' . $client_ip_address.' disconnected';
		$message_array 	= array('message'=>$message,'message_type'=>'chat-connection-ack');
		$ACK 			= $this->seal(json_encode($message_array));
		return $ACK;
	}
	
	function createChatBoxMessage($chat_user,$chat_box_message) {
		$message		= $chat_user . ": <div class='chat-box-message'>" . $chat_box_message . "</div>";
		$message_array 	= array('message'=>$message,'message_type'=>'chat-box-html');
		$chat_message  	= $this->seal(json_encode($message_array));
		return $chat_message;
	}
}

$chat_socket = socket_create(AF_INET, SOCK_STREAM, SOL_TCP);
socket_set_option($chat_socket, SOL_SOCKET, SO_REUSEADDR, 1);
socket_bind($chat_socket, '0.0.0.0', PORT);
socket_listen($chat_socket);

?>