<?php 

require($root.'libraries/jwt/JWT.php');

use Firebase\JWT\JWT;
use Firebase\JWT\Key;

class WCF extends Insurance {
	function __construct($id=0) {		
		$this->table        = 'billing_insurances';
		$this->primary_key  = 'id';
		parent::__construct($id);
	}

    function jwt_encode($data) {
        global $client;

        $encoded_data = JWT::encode($data, $client->wcf_api_key, 'HS512');
        return $encoded_data;
    }

    function authorizeCard($card_no, $visit_type_id, $referral_no=0, $remarks='') {
		global $app, $client;

        $authorization_url      = $client->wcf_server_address;
        $authorization_data     = array (
            'registrationNo'    => $client->wcf_hcp_code,
            'insuaranceID'      => 0,
            'facilityType'      => 2,
            'notificationNo'    => $card_no,
            'visitType'         => 1,
            'category'          => 1,
            'qaName'            => 'Kumlaza Swai',
            'qaPhone'           => '0798262384',
            'exp'               => time() + 300
        );

        $encrypted_data         = $this->jwt_encode($authorization_data);

        $payload                = array (
            'action'            => 107,
            'hcp_code'          => $client->wcf_hcp_code,
            'data'              => $encrypted_data
        );

        $body 				    = json_encode($payload);

        //file_put_contents('payload.json', $body);

        $curl_options           = array (
            'url'               => $authorization_url,
            'method'            => 'POST',
            'headers'   		=> array (
                'Content-Type'  => 'application/json'
            ),
            'fields'            => $body
        );

		return $app->curl($curl_options);
	}

    function downloadPricelist() {
		global $app, $client;

        $pricelist_url      = $client->wcf_server_address;
        $pricelist_data     = array (
            'registrationNo'    => $client->wcf_hcp_code,
            'facilityType'      => 2,
            'exp'               => time() + 300
        );

        $encrypted_data         = $this->jwt_encode($pricelist_data);

        $payload                = array (
            'action'            => 147,
            'hcp_code'          => $client->wcf_hcp_code,
            'data'              => $encrypted_data
        );

        $body 				    = json_encode($payload);

        $curl_options           = array (
            'url'               => $pricelist_url,
            'method'            => 'POST',
            'headers'   		=> array (
                'Content-Type'  => 'application/json'
            ),
            'fields'            => $body
        );

		return $app->curl($curl_options);
	}


    function approveService($item_code, $authorization_no, $practioner_no, $remarks) {
		global $app, $client;

        $services               = array (
            'serviceID'         => $item_code,
            'reason'            => $remarks,
            'practitionerID'    => 9
        );

        $approval_url           = $client->wcf_server_address;
        $approval_data          = array (
            'registrationNo'    => $client->wcf_hcp_code,
            'facilityType'      => 2,
            'isIntergrated'     => 1,
            'authorizationNo'   => $authorization_no,
            'serviceList'       => array($services),
            'exp'               => time() + 300
        );

        file_put_contents('approval_request.json', json_encode($approval_data));

        $encrypted_data         = $this->jwt_encode($approval_data);

        $payload                = array (
            'action'            => 134,
            'hcp_code'          => $client->wcf_hcp_code,
            'data'              => $encrypted_data
        );

        $body 				    = json_encode($payload);

        $curl_options           = array (
            'url'               => $approval_url,
            'method'            => 'POST',
            'headers'   		=> array (
                'Content-Type'  => 'application/json'
            ),
            'fields'            => $body
        );

		return $app->curl($curl_options);
	}



    function requestReferral($authorization_no, $referral_to_facility, $referral_category, $remarks, $requested_by, $requester_phone) {
		global $app, $client;

        $referral               = array (
            'referralToFacility'=>  $referral_to_facility,
            'referralCategory'  =>  $referral_category,
            'remark'            =>  $remarks,
            'requestedBy'       =>  $requested_by,
            'requesterPhone'    =>  $requester_phone
        );

        $referral_url           = $client->wcf_server_address;
        $referral_data          = array (
            'authorizationNo'   => $authorization_no,
            'referralData'      => $referral,
            'exp'               => time() + 300
        );

        file_put_contents('referral_request.json', json_encode($referral_data));

        $encrypted_data         = $this->jwt_encode($referral_data);

        $payload                = array (
            'action'            => 138,
            'hcp_code'          => $client->wcf_hcp_code,
            'data'              => $encrypted_data
        );

        $body 				    = json_encode($payload);

        $curl_options           = array (
            'url'               => $referral_url,
            'method'            => 'POST',
            'headers'   		=> array (
                'Content-Type'  => 'application/json'
            ),
            'fields'            => $body
        );

		return $app->curl($curl_options);
	}
}

?>