public function numberAuthorization(Request $request): array|string
    {
        $notificationNo = $request->notificationNo;
        $visitType = $request->visitType;
        $category = $request->authCategory;
        $qaName = $request->qaName;
        $qaPhone = $request->qaPhone;

        $data = [
            "registrationNo" => config('app.wcf_registration_no'),
            "insuaranceID" => 0,
            "facilityType" => 2,
            "notificationNo" => $notificationNo,
            "visitType" => $visitType,
            "category" => $category,
            "qaName" => $qaName,
            "qaPhone" => $qaPhone,
            "exp" => time() + 60
        ];

        $encrypted_data = JWT::encode($data, config('app.wcf_key'), 'HS512');

        $payload = [
            "action" => "107",
            "hcp_code" => config('app.wcf_registration_no'),
            "data" => $encrypted_data
        ];
        return $this->wcfRequest($payload);
    }

private function wcfRequest($payload): array|string
    {
        
        try {
            $ch = curl_init(config('app.wcf_api_endpoint'));
            curl_setopt($ch, CURLOPT_POST, true);
            curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($payload));
            curl_setopt($ch, CURLOPT_FRESH_CONNECT, true);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            $response = curl_exec($ch);
            if (!$response) {
                $error = ('Error: "' . curl_error($ch) . '" - Code: ' .
                    curl_errno($ch));
                return $this->response(false, $error);
            }
            $returned_data = json_decode($response, true);
            curl_close($ch);
            $response = $returned_data;
            //print_r($response);exit();
            if ($response && isset($response['code'])) {
                return $this->response(true, $response);
            } else {
                return $this->response(false, "API ERROR");
            }
        } catch (\Exception $error) {
            return $this->response(false, $error->getMessage());
        }
    }