window.addEventListener('load', function() { 
    app.list_current_page       = 1;
    app.list_records_perpage    = 30;
    app.root = root;
    app.list_loading            = 0;

    // Theme
    var device_theme            = window.matchMedia('(prefers-color-scheme: dark)').matches ? 'dark' : 'light';
    var cookie_theme            = $.cookie('user_theme');
    if(cookie_theme != device_theme) {
        $.cookie('user_theme', device_theme);
        document.documentElement.setAttribute('data-bs-theme', device_theme);
    }

    app.loadList();
    $('.reload-onclick').click(function() {app.loadList()});
    $('.filter-toggle').click(function() {$('.filter-hidden').slideToggle(200)});
    $('.remote-modal').on('click', function() {
        var modal_url   = $(this).attr('href');
        var modal_id    = $(this).data('bs-target');
        $(modal_id+'_content').html('<div class="py-4">'+loading_bar+'</div>').load(modal_url, function() {

        });
    });

    $('.onkeyup-reload-list').keyup(function(e) {
        if(typeof loaderTimer != 'undefined') clearTimeout(loaderTimer);
        if (e.keyCode == 13) {
            app.loadList();
        } else {
            loaderTimer = setTimeout(function() {
                app.loadList();
            }, 1000);
        }
    });

    $('.create-button').click(function() {
        var button      = $(this);
        var url         = button.data('url');
        if(url.indexOf('?') > -1) {
            url         = url+'&action=create';
        } else {
            url         = url+'?action=create';
        }

        var required    = button.data('require');
        var success     = true;

        if(required.length) {
            var required_fields = required.split(',');

            // Check required fields
            required_fields.forEach(function(required_field) {
                var field_name  = required_field;
                var field_value = $('#'+field_name).val();
                var pretty_name = field_name.split('_').join(' ');
                pretty_name     = pretty_name[0].toUpperCase() + pretty_name.substring(1);

                if(field_value == '' || field_value == '0') {
                    app.showAlert(pretty_name, 'Please choose '+pretty_name+' before creating new entry.', 'danger');
                    success = false;
                } else {
                    url     += '&'+field_name+'='+field_value;
                }
            });
        }

        // Add filter controls
        $('.filter-control').each(function() {
            url     += '&'+this.name+'='+this.value;
        });

        // Open create modal
        if(success) {
            $('#modal_lg_content').html('<div class="py-4">'+loading_bar+'</div>').load(encodeURI(url));
            $('#modal_lg').modal({backdrop:'static', keyboard:false});
            $('#modal_lg').modal('show');
        }
    });

    $('#notification_container').load(r+'pages/info/notifications/notifications.php?uid='+uid+'&submodule_code='+submodule_code+'&r='+app.rootToNumber(r));
    this.setInterval(function() {
        $('#notification_container').load(r+'pages/info/notifications/notifications.php?uid='+uid+'&submodule_code='+submodule_code+'&r='+app.rootToNumber(r));
    }, 5000);

    // Modals
    modal_top   = new bootstrap.Modal('#modal_top', {
        backdrop:'static', 
        keyboard:false
    });

    modal_lg    = new bootstrap.Modal('#modal_lg', {
        backdrop:'static', 
        keyboard:false
    });

    $(modal_top).on('hidden.bs.modal', function() {
        idle_warning_shown = false;
    });
});

class App {
    constructor() {

    }

    prepareAjaxForm(form_id) {
        var progress_container  = $('#'+form_id+'_progress_container');
        var progress_bar 	    = $('#'+form_id+'_progress_bar');
        var submit_button 	    = $('#'+form_id+'_submit_button');
        var submit_button_html  = submit_button.html();
        var response_container 	= $('#'+form_id+'_response');
        var options             = {
            beforeSend: function() {
                submit_button.attr('disabled', 'disabled').html('<span class="fa fa-spin fa-spinner"></span>');
                progress_container.removeClass('d-none');
                response_container.html('');
            },
            
            uploadProgress: function(event, position, total, percentComplete) {
                progress_bar.html(percentComplete + '%').css('width', percentComplete + '%');
              },
            
            complete: function(response) {
                response_container.html(response.responseText);
                submit_button.html(submit_button_html);
                setTimeout(function() {submit_button.removeAttr('disabled')}, 1200);
                progress_bar.css('width', '0%').html('0%');
                progress_container.addClass('d-none');
            }
        }  
        submit_button.click(function() {$('#'+form_id).submit()});       
        $('#'+form_id).ajaxForm(options);
    }

    loadList(options=[]) {

        if(app.list_loading) return; // Prevent reloading while we are currently loading

        app.list_loading = 1;
        setTimeout(function() {app.list_loading = 0;}, 10);

        var parameters              = '';
        var list_current_page       = 'list_current_page' in options ? options.list_current_page : this.list_current_page;
        var list_records_perpage    = 'list_records_perpage' in options ? options.list_records_perpage : this.list_records_perpage;
        
        $('.filter-control').each(function() {
            if(this.name) parameters += '&'+this.name+'='+this.value;
        });

        var url = encodeURI('list.php?list_current_page='+list_current_page+'&list_records_perpage='+list_records_perpage+'&'+parameters);
        url     = url.replace('+','%2B');

        $('#list_container').html(loading_ring).load(url, function() {
            app.initializeModals();
            app.initializeControls();
        });
    }

    loadListPage(list_current_page) {
        app.list_current_page = list_current_page;
        var options = {"list_current_page":list_current_page};
        this.loadList(options);
    }

    // Dynamic modals

    initializeModals() {
        setTimeout(function() {
            $('.remote-modal').on('click', function() {
                var parameters  = '';
                var modal_id    = $(this).data('bs-target');
                var modal_url   = $(this).attr('href');

                $('.filter-control').each(function() {
                    if(this.name) modal_url += '&'+this.name+'='+this.value;
                });
        
                modal_url = encodeURI(modal_url);
        
                $(modal_id+'_content').html('<div class="py-4">'+loading_bar+'</div>').load(modal_url, function() {
                    //app.initializeControls();
                });
            });

            $('.top-modal').on('click', function() {
                var modal_url = encodeURI($(this).data('href'));
                $('#modal_top_content').html('<div class="py-4">'+loading_bar+'</div>').load(root+modal_url, function() {
                    //app.initializeControls();
                });
                $('#modal_top').modal('show');
            });
            
        }, 100);
    }

    closeModal(modal_id, load_list) {
        if(load_list) this.loadList();
        setTimeout (
            function () {
                $('#'+modal_id).modal('hide');
            }, 1000
        ); 
    }

    initializeControls() {
        app.check_dependant_inputs();
        app.check_attachment_target();
        $('.onchange-reload-list').on('change', function() {
            var options = {"id": $(this).prop('id')};
            app.loadList(options);
        });
        $('.select-search').selectpicker({liveSearch: true});
        
        $('.bs-toggle').bootstrapToggle();
        $('.auto-flip').flip();
        $('.manual-flip').flip({trigger: 'manual'});
        $('[data-flip]').on('click', function () {
            let flip_id = $(this).data('flip');
            $('#'+flip_id).flip('toggle');
        });

        $('.ajax-loader').on('click', function () {
            var url         = $(this).data('url');
            var container   = $(this).data('container');
            app.ajaxLoad(container, url);
        });

        $('.ajax-link').on('click', function () {
            var link        = $(this).data('link');
            var target      = $(this).data('target');
            app.ajaxLoad(target, link, true);
        });

        $('.toggle-child').on('change', function() {
            var check_state = $(this).is(':checked');
            var child_class = $(this).data('child-class');
            var toggle_state= check_state ? 'on' : 'off';

            if($('.'+child_class).hasClass('bs-toggle')) {
                $('.'+child_class).bootstrapToggle(toggle_state);
            } else {
                $('.'+child_class).prop('checked', check_state);
            }
        });

        setTimeout(function() {$('.autofocus').focus(); $('.bs-tooltip').tooltip();}, 100);

        $('.year-picker').each(function() {
            let datepicker_id = $(this).prop('id');
            const picker = new tempusDominus.TempusDominus(
                document.getElementById(datepicker_id), {
                    restrictions: {
                        minDate: undefined,
                        maxDate: undefined,
                        disabledDates: [],
                        enabledDates: [],
                        daysOfWeekDisabled: [],
                        disabledTimeIntervals: [],
                        disabledHours: [],
                        enabledHours: []
                    },
                    display: {
                        icons: {
                            type: 'icons',
                            time: 'fa-regular fa-clock',
                            date: 'fa-regular fa-calendar',
                            up: 'fa-regular fa-angle-up',
                            down: 'fa-regular fa-angle-down',
                            previous: 'fa-regular fa-chevron-left',
                            next: 'fa-regular fa-chevron-right',
                            today: 'fa-regular fa-calendar-check',
                            clear: 'fa-regular fa-trash',
                            close: 'fa-regular fa-xmark'
                        },
                        sideBySide: false,
                        calendarWeeks: false,
                        viewMode: 'years',
                        toolbarPlacement: 'bottom',
                        keepOpen: false,
                        buttons: {
                        today: false,
                        clear: false,
                        close: false
                    },
                    components: {
                        calendar: true,
                        date: true,
                        month: true,
                        year: true,
                        decades: true,
                        clock: true,
                        hours: true,
                        minutes: true,
                        seconds: false
                    },
                    inline: false,
                    theme: 'light'
                },
                stepping: 1,
                useCurrent: true,
                defaultDate: undefined,
                localization: {
                    dayViewHeaderFormat: { month: 'long', year: 'numeric' },
                    locale: 'default',
                    format: 'yyyy-MM-dd',
                    hourCycle: 'h23'
                },
                keepInvalid: false,
                debug: false,
                allowInputToggle: false,
                multipleDates: false,
                multipleDatesSeparator: '; ',
                promptTimeOnDateChange: false,
                promptTimeOnDateChangeTransitionDelay: 200,
                meta: {},
                container: undefined
            })
        });

        $('.date-picker').each(function() {
            let datepicker_id = $(this).prop('id');
            const picker = new tempusDominus.TempusDominus(
                document.getElementById(datepicker_id), {
                    restrictions: {
                        minDate: undefined,
                        maxDate: undefined,
                        disabledDates: [],
                        enabledDates: [],
                        daysOfWeekDisabled: [],
                        disabledTimeIntervals: [],
                        disabledHours: [],
                        enabledHours: []
                    },
                    display: {
                        icons: {
                            type: 'icons',
                            time: 'fa-regular fa-clock',
                            date: 'fa-regular fa-calendar',
                            up: 'fa-regular fa-angle-up',
                            down: 'fa-regular fa-angle-down',
                            previous: 'fa-regular fa-chevron-left',
                            next: 'fa-regular fa-chevron-right',
                            today: 'fa-regular fa-calendar-check',
                            clear: 'fa-regular fa-trash',
                            close: 'fa-regular fa-xmark'
                        },
                        sideBySide: false,
                        calendarWeeks: false,
                        viewMode: 'calendar',
                        toolbarPlacement: 'bottom',
                        keepOpen: false,
                        buttons: {
                        today: false,
                        clear: false,
                        close: false
                    },
                    components: {
                        calendar: true,
                        date: true,
                        month: true,
                        year: true,
                        decades: true,
                        clock: true,
                        hours: true,
                        minutes: true,
                        seconds: false,
                        useTwentyfourHour: undefined
                    },
                    inline: false,
                    theme: 'light'
                },
                stepping: 1,
                useCurrent: true,
                defaultDate: undefined,
                localization: {
                    dayViewHeaderFormat: { month: 'long', year: 'numeric' },
                    locale: 'default',
                    format: 'yyyy-MM-dd'
                },
                keepInvalid: false,
                debug: false,
                allowInputToggle: false,
                multipleDates: false,
                multipleDatesSeparator: '; ',
                promptTimeOnDateChange: false,
                promptTimeOnDateChangeTransitionDelay: 200,
                meta: {},
                container: undefined
            })
        });
        

        $('.time-picker').each(function() {
            let datepicker_id = $(this).prop('id');
            const picker = new tempusDominus.TempusDominus(
                document.getElementById(datepicker_id), {
                    restrictions: {
                        minDate: undefined,
                        maxDate: undefined,
                        disabledDates: [],
                        enabledDates: [],
                        daysOfWeekDisabled: [],
                        disabledTimeIntervals: [],
                        disabledHours: [],
                        enabledHours: []
                    },
                    display: {
                        icons: {
                            type: 'icons',
                            time: 'fa-regular fa-clock',
                            date: 'fa-regular fa-calendar',
                            up: 'fa-regular fa-angle-up',
                            down: 'fa-regular fa-angle-down',
                            previous: 'fa-regular fa-chevron-left',
                            next: 'fa-regular fa-chevron-right',
                            today: 'fa-regular fa-calendar-check',
                            clear: 'fa-regular fa-trash',
                            close: 'fa-regular fa-xmark'
                        },
                        sideBySide: false,
                        calendarWeeks: false,
                        viewMode: 'clock',
                        toolbarPlacement: 'bottom',
                        keepOpen: false,
                        buttons: {
                        today: false,
                        clear: false,
                        close: false
                    },
                    components: {
                        calendar: true,
                        date: true,
                        month: true,
                        year: true,
                        decades: true,
                        clock: true,
                        hours: true,
                        minutes: true,
                        seconds: false
                    },
                    inline: false,
                    theme: 'light'
                },
                stepping: 1,
                useCurrent: true,
                defaultDate: undefined,
                localization: {
                    dayViewHeaderFormat: { month: 'long', year: 'numeric' },
                    locale: 'default',
                    format: 'H:mm',
                    hourCycle: 'h23'
                },
                keepInvalid: false,
                debug: false,
                allowInputToggle: false,
                multipleDates: false,
                multipleDatesSeparator: '; ',
                promptTimeOnDateChange: false,
                promptTimeOnDateChangeTransitionDelay: 200,
                meta: {},
                container: undefined
            })
        });


        
        

        $('.bs-table').bootstrapTable({
            classes: 'table table-hover',
            buttonsClass: ' my-0 btn-outline-primary text-decoration-none border',
            buttonsAlign: 'left',
            pagination: false,
            //search: true,
            formatSearch: function() {return 'Search table'},
            sortable: true,
            pageSize: 100,
            
            showPrint: true,
            showExport: true,
            exportTypes: ['excel', 'doc', 'json', 'csv', 'xml', 'txt', 'sql'],

            iconsPrefix: '',
            icons: {
                print: 'fa-regular fa-print\"></i> Print table <i class="',
                export: 'fa-regular fa-file-export\"></i> Export <i class="'
            },
            buttonsOrder: ['print', 'export']
        });
        
        
        $('select.onchange-reload-children').change(function() {
            var parent_id       = $(this).attr('id');
            var parent_value    = $(this).val();

            var child_element   = $('select[data-parent="'+parent_id+'"]');
            var child_id        = $(child_element).attr('id');
            var child_css       = $(child_element).attr('class');
            var child_style     = $(child_element).data('style');
            var child_parent    = $(child_element).data('parent');
            var child_source    = $(child_element).data('source');
            var child_container = $('#'+child_id+'_container').parent();
            var child_source    = child_source+'?id='+child_id+'&'+parent_id+'='+parent_value+'&css='+child_css+'&style='+child_style+'&parent='+child_parent;
            $(child_container).html(loading_bar).load(encodeURI(app.root+child_source), function() {
                
            });
        });

        // Money input control

        $(document).on('keypress , paste', '.money', function (e) {
            if (/^-?\d*[,.]?(\d{0,3},)*(\d{3},)?\d{0,3}$/.test(e.key)) {
                $('.money').on('input', function () {
                    e.target.value = app.numberSeparator(e.target.value);
                });
            } else {
                e.preventDefault();
                return false;
            }
        });

        // Wizards 

        $.each($('[data-check-input]'), function () {
            var wizard_link         = $(this);
            var wizard_checks       = wizard_link.data('check-input').replace(/ /g, '').split(',');

            $.each(wizard_checks, function (input_index, input_id) {       
                var wizard_check    = $('#'+input_id);
        
                var wizard_active       = true;
                $.each(wizard_checks, function (input_index_checked, input_id_checked) {
                    var wizard_checked  = $('#'+input_id_checked).val();
                    if(!wizard_checked) {
                        wizard_active   = false;
                    }
                });
        
                if(wizard_active) {
                    wizard_link.removeClass('disabled');
                } else {
                    wizard_link.addClass('disabled');
                }
        
                wizard_check.on('keyup keypress blur change', function() {
                    wizard_active           = true;
                    $.each(wizard_checks, function (input_index_checked, input_id_checked) {
                        var wizard_checked  = $('#'+input_id_checked).val();
                        if(!wizard_checked) {
                            wizard_active   = false;
                        }
                    });
        
                    if(wizard_active) {
                        wizard_link.removeClass('disabled');
                    } else {
                        wizard_link.addClass('disabled');
                    }
                });
            });
        });

        // Add paramers to edit buttons
    }

    check_dependant_inputs() {
        $.each($('[data-dependable-inputs]'), function () {
            var this_input          = $(this);
            var this_input_id       = $(this).prop('id');
            var dependable_inputs   = this_input.data('dependable-inputs').replace(/ /g, '').split(',');
            var show_this_input     = 1;
    
            $.each(dependable_inputs, function (input_index, input_parameters) {
                let input_params    = input_parameters.replace(/ /g, '').split(';');
                let input_id        = input_params[0];
                let input_operator  = input_params[1];
                let input_result    = input_params[2];
                let input_val       = $('#'+input_id).val();
    
                if(input_operator == 'equal') {
                    if(input_val != input_result) {
                        show_this_input = 0;
                    }
                }
    
                $('#'+input_id).change(function() { app.check_dependant_inputs() });
            });
    
            if(show_this_input) {
                $('#'+this_input_id+'_parent_container').fadeIn(250);
            } else {
                $('#'+this_input_id+'_parent_container').fadeOut(250);
            }
        });
    }

    check_attachment_target() {
        $.each($('[data-target-preview]'), function () {
            var this_input          = $(this);
            var this_input_id       = $(this).prop('id');
            var target_preview_id   = $(this_input).data('target-preview');
            var target_preview      = $('#'+target_preview_id);
            var target_preview_size = $('#'+target_preview_id+'_size');

            $(this_input).on('change', function(event) {
                var attachment      = event.target.files[0];
                if(attachment.type == 'image/jpeg' || attachment.type == 'image/png' || attachment.type == 'image/gif') {
                    $(target_preview).attr('src', URL.createObjectURL(attachment));
                } else {
                    $(target_preview).html('<span class="fa-light fa-paperclip fa-3x"></span>');
                }
                $(target_preview_size).html(app.filesize(attachment.size));
            });
        });
    }
    
    showAlert(title, message, color) {
        var icon    = 'fa-regular fa-info-circle';
        title       = title.charAt(0).toUpperCase() + title.slice(1);
        // var color   = 'primary';

        var html = '<div class="row g-0">'+
        '<div class="bg-'+color+' col-lg-3 text-white text-center p-3 d-none d-lg-block rounded-start" style=" --bs-text-opacity: .9; margin:-1px">'+
        '<h1> <span class=""></span> </h1>'+
        '<h4>  </h1>'+
        '<small class="text-white mb-4" style="--bs-text-opacity: .6;"></small>'+
        '</div>'+
        '<div class="col-lg-9">'+
        '<div class="modal-header">'+
        '<h3 class="modal-title fw-bold"><span class="fa-regular fa-info-circle fa-fw"></span> '+title+' </h3>'+
        '<button type="button" class="btn-close" data-bs-dismiss="modal" aria-label="Close"></button>'+
        '</div>'+
        '<div class="modal-body mb-4">'+message+'</div>'+
        '<div class="modal-footers p-3 text-start">'+
        '<button type="button" class="btn btn-outline-'+color+' btn-lg" data-bs-dismiss="modal"> <span class="fa fa-arrow-left fa-fw"></span> Back &nbsp; </button>'+
        '</div>'+
        '</div>'+
        '</div>';

        $('#modal_md_content').html(html);
        $('#modal_md').modal('show');
    }

    numberSeparator(Number) {
        Number += '';
        var commaCounter = 10;
        for (var i = 0; i < commaCounter; i++) {
            Number = Number.replace(',', '');
        }
        var x   = Number.split('.');
        var y   = x[0];
        var z   = x.length > 1 ? '.' + x[1] : '';
        var rgx = /(\d+)(\d{3})/;

        while (rgx.test(y)) {
            y   = y.replace(rgx, '$1' + ',' + '$2');
        }
        commaCounter++;
        return y + z;
    }

    formatMoney(money, prefix) {
        if(prefix === undefined) prefix = '';
        return accounting.formatMoney(money, prefix);
    }

    unformatMoney(money) {
        return accounting.unformat(money);
    }

    getCategoryByCode(category_type, category_code) {
        // Makes an ajax call and returns an instance of the category with the passed category_type and category_code
        var url = root+'html/get_category_by_code.php?category_type='+category_type+'&category_code='+category_code;
        $.getJSON(url, function(data) {
            return data;
        });
    }

    filesize(filesize) {
        if(filesize < 1024) return filesize+' B';
        if(filesize < 1024000) return Math.round(filesize / 1024)+' KB';
        if(filesize < 1024000000) return Math.round(filesize / 1024000)+' MB';
        return Math.round(filesize / 1024000000)+' GB';
    }

    ajaxLoad(container, url, initialize_controls=0) {
        $('#'+container).html('<br /><br />'+loading_circle+'<br /><br />')
        .load(url, function() { 
            if(initialize_controls) app.initializeControls(); 
        });
    }

    loadReport(start_date, end_date, section) {
        $('#start_date').val(start_date);
        if (end_date.length && end_date != undefined) {
            $('#end_date').val(end_date);
        }
        this.loadList();
    }

    rootToNumber(root) {
        return root.split('../').length - 1;
    }

    numberToRoot(root_number) {
        root = '';
        for(var count = 0; count < root_number; count++) {
            root = root+'../';
        }
        return root;
    }

    showModal(modal_id, url) {
        var modal_obj   = new bootstrap.Modal('#'+modal_id, {
            backdrop:'static', 
            keyboard:false
        });
        $('#'+modal_id+'_content').html('<div class="py-4">'+loading_bar+'</div>').load(encodeURI(url));
        modal_obj.show();
    }
}

// Global variables

loading_bar     = '<div class="text-center"><div class="lds-ellipsis"><div></div><div></div><div></div><div></div></div></div>';
loading_ring    = '<div class="text-center m-5"><div class="spinner-border" style="width: 4rem; height: 4rem;" role="status"></div></div>';
loading_circle  = '<div class="text-center"><div class="spinner-border" style="width: 4rem; height: 4rem;" role="status"></div></div>';
dom_ready       = true;
app             = new App();