class Transaction {
    constructor() {
        this.initializeTransaction();
    }

    loadStoreItems() {
        var store_id    = $('#trans_store_id').val();
        var scheme_id   = $('#trans_scheme_id').val();
        var url         = root+'html/transactions/store_items_select.php?store_id='+store_id+'&transaction=Adjustment&scheme_id='+scheme_id;
        $('#trans_item_container').html(loading_bar).load(encodeURI(url));
    }

    initializeTransaction() {
        this.trans_items    = [];
        this.store_items    = [];

        $('#trans_store_id, #trans_scheme_id').on('change', function() {
            transaction.resetTransaction();
            transaction.loadStoreItems();
        });
    }

    addItemToCart() {
        var trans_item_id   = $('#trans_item_id').val();
        transaction.addItem(trans_item_id);
    }

    initializeItem() {
        transaction.setItemQuantity();
        transaction.setItemPacks();
        transaction.setItemPrice();
        transaction.checkAddItemButton();
    }

    setItemQuantity() {
        $('#trans_item_quantity').val(1);
        $('#trans_item_dose').val(1);
        $('#trans_item_times').selectpicker('val', '1');
        $('#trans_item_days').selectpicker('val', '1');
    }

    calculatePrescription() {
        var trans_item_id       = $('#trans_item_id').val();
        var trans_item_units    = $('#trans_item_units').val();
        var trans_item          = this.store_items[trans_item_id];

        if(trans_item_units == 'Each' || trans_item_units == 'Tab' || trans_item_units == 'Pcs') {
            var item_dose       = parseInt($('#trans_item_dose').val());
            var item_times      = parseInt($('#trans_item_times').val());
            var item_days       = parseInt($('#trans_item_days').val());
            var item_quantity   = item_dose * item_times * item_days;
    
            $('#trans_item_quantity').val(item_quantity);
        }
    }

    setItemPacks() {
        var trans_item_id   = $('#trans_item_id').val();
        var trans_item      = this.store_items[trans_item_id];
        var trans_pack_html = `<select class="form-control line-controlled select-search border border-1 border-secondary" id="trans_item_uom" onchange="transaction.setItemPrice()">`;
        trans_pack_html 	+= `<option value="${trans_item.uom}"> ${trans_item.uom} </option>`;

        for (let trans_pack_code in trans_item.packs) {
            var trans_pack  = trans_item.packs[trans_pack_code];
            trans_pack_html +=` <option value="${trans_pack.code}"> ${trans_pack.name} of ${trans_pack.size} </option>`;
        };
      
        trans_pack_html     += `</select>`;

        $('#trans_item_uom_container').html(trans_pack_html);
        $('#trans_item_uom').selectpicker({liveSearch: true});
    }

    setItemPrice() {
        if($('#trans_scheme_id').val()) {
            var trans_scheme_id     = $('#trans_scheme_id').val();
            var trans_pricelist     = this.insurance_schemes[trans_scheme_id].pricelist;
        } else if($('#trans_pricelist_id').val()) {
            var trans_pricelist_id  = $('#trans_pricelist_id').val();
            var trans_pricelist     = this.pricelists[trans_pricelist_id].code;
        } else {
            alert('Error: Pricelist is not defined');
            return;
        }

        var trans_item_id           = $('#trans_item_id').val();
        var trans_item              = this.store_items[trans_item_id];
        var trans_item_uom          = $('#trans_item_uom').val(); 
        var trans_item_pack_size    = trans_item_uom == trans_item.uom ? 1 : trans_item.packs[trans_item_uom].size;  
        if(trans_pricelist == 'COST') {
            $('#trans_item_unit_price').removeClass('readonly');
            var cost                = parseFloat(trans_item.pricelists['COST'].price);
            var item_unit_price     = parseFloat(cost);
        } else if(trans_item.pricelists[trans_pricelist].pricemode == 'Margin') {
            var cost                = parseFloat(trans_item.pricelists['COST'].price);
            var margin              = parseFloat(trans_item.pricelists['COST'].price * trans_item.pricelists[trans_pricelist].margin / 100 * trans_item_pack_size);
            var item_unit_price     = parseFloat(cost + margin);
            $('#trans_item_unit_price').addClass('readonly');
        } else if(trans_item.pricelists[trans_pricelist].pricemode == 'Manual') {
            var item_unit_price     = trans_item.pricelists[trans_pricelist].price * trans_item_pack_size;
            $('#trans_item_unit_price').removeClass('readonly');
        } else if(trans_item.pricelists[trans_pricelist].pricemode == 'Disabled') {
            var item_unit_price     = -1;
            $('#trans_item_unit_price').addClass('readonly');
        } else {
            var item_unit_price     = trans_item.pricelists[trans_pricelist].price * trans_item_pack_size;
            $('#trans_item_unit_price').addClass('readonly');
        }

        $('#trans_item_unit_price').val(app.formatMoney(item_unit_price));
    }

    checkAddItemButton() {
        if($('#trans_item_id').val()) {
            $('#trans_item_add_button').removeClass('disabled readonly');
        } else {
            $('#trans_item_add_button').addClass('disabled readonly');
        }
    }

    addItem(item_id) {
        if(transaction.trans_items[item_id]) {
            alert(transaction.trans_items[item_id].name+' is already in your cart');
            return false;
        }

        var trans_item              = transaction.store_items[item_id];
        var trans_item_pack         = $('#trans_item_uom').val(); 
        var trans_item_pack_size    = 1;

        if(trans_item.packs[trans_item_pack] != undefined) {
            trans_item_pack_size    = trans_item.packs[trans_item_pack].size;
        }
        var trans_item_details      = $('#trans_item_details').val();
        //var trans_item_pack_size    = trans_item_pack == trans_item.uom ? 1 : trans_item.packs[trans_item_pack].size;
        var trans_item_quantity     = parseFloat(app.unformatMoney($('#trans_item_quantity').val()));  
        var trans_item_unit_cost    = parseFloat(trans_item.pricelists['COST'].price);
        var trans_item_cost         = parseFloat(app.unformatMoney(trans_item_quantity) * app.unformatMoney(trans_item_unit_cost));  
        var trans_item_unit_price   = parseFloat(app.unformatMoney($('#trans_item_unit_price').val()));  
        var trans_item_price        = parseFloat(app.unformatMoney(trans_item_quantity) * app.unformatMoney(trans_item_unit_price));  
        
        if(trans_item.tags.indexOf('wallet-withdraw') > -1) {
            trans_item_unit_price   = Math.abs(trans_item_unit_price) * -1;
            trans_item_price        = Math.abs(trans_item_price) * -1;
        }

        var trans_item_route        = $('#trans_item_route').val();
        var trans_item_dose         = parseInt($('#trans_item_dose').val());
        var trans_item_times        = parseInt($('#trans_item_times').val());
        var trans_item_days         = parseInt($('#trans_item_days').val());
        var trans_item_units        = $('#trans_item_units').val();

        var item_line       = {
            'item_id'       : trans_item.id,
            'name'          : trans_item.name,
            'details'       : trans_item_details,
            'uom'           : trans_item.uom,
            'pack'          : trans_item_pack,
            'size'          : trans_item_pack_size,
            'qty'           : trans_item_quantity * trans_item_pack_size,
            'quantity'      : trans_item_quantity,
            'unit_cost'     : trans_item_unit_cost,
            'cost'          : trans_item_cost,
            'unit_price'    : trans_item_unit_price,
            'price'         : trans_item_price,
            'route'         : trans_item_route,
            'dose'          : trans_item_dose,
            'times'         : trans_item_times,
            'days'          : trans_item_days,
            'dose_units'    : trans_item_units
        };
        transaction.addItemLine(item_line);
    }

    addItemLine(item_line, editable=true) {

        var trans_treatment = $('#trans_treatment').val();
        var item_id         = item_line.item_id;
        var item_details    = item_line.details.length > 1 ? '<div class="text-muted"><small>'+item_line.details+'</small></div>' : '';

        if(item_line.route != undefined) {
            item_details    = item_details+'<div class="text-muted fw-bold"><small>Route: '+item_line.route+', Dose: '+item_line.dose+' '+item_line.dose_units+', Frequency: '+item_line.times+', Days: '+item_line.days+'</small></div>'
        }
        
        this.trans_items[item_id] = item_line;
        var item_row        = `<tr class="trans-item display-none" id="trans_item_${item_id}">
        <td class="ps-0">
        ${item_line.name} <b class="text-muted">(${item_line.pack})</b>${item_details}
        <input id="trans_item_${item_id}_unit_cost" type="hidden" value="${app.formatMoney(item_line.unit_cost)}" /> </td>
        <input id="trans_item_${item_id}_cost" type="hidden" value="${app.formatMoney(item_line.cost)}" readonly disabled /> </td>
        
        </td>
        <td class="w-10" style="max-width:50px">  <input id="trans_item_${item_id}_quantity" type="number" class="form-control line-control text-center money" value="${item_line.quantity}" onchange="transaction.updateItemLine(${item_id}, $('#trans_item_${item_id}_quantity').val(), $('#trans_item_${item_id}_unit_cost').val(), $('#trans_item_${item_id}_unit_price').val())" /> </td>
        <td class="w-12" style="max-width:100px"> <input id="trans_item_${item_id}_unit_price" type="text" class="form-control line-control text-end money" value="${app.formatMoney(item_line.unit_price)}" onchange="transaction.updateItemLine(${item_id}, $('#trans_item_${item_id}_quantity').val(), $('#trans_item_${item_id}_unit_cost').val(), $('#trans_item_${item_id}_unit_price').val())" /> </td>
        <td class="w-12" style="max-width:100px"> <input id="trans_item_${item_id}_price" type="text" class="form-control line-control text-end money" value="${app.formatMoney(item_line.price)}" readonly disabled /> </td>
        <td class="pe-0">
        <a href="#" class="text-pointer text-danger" onclick="transaction.removeItem(${item_line.item_id})">
        <span class="fa-light fa-trash-alt"></span>
        </a>
        </td>
        </tr>`;

        if(trans_treatment == 'TreatmentChart') {
            item_row        = `<tr class="trans-item display-none" id="trans_item_${item_id}">
            <td class="ps-0">
            ${item_line.name} <b class="text-muted">(${item_line.pack})</b>
            <input id="trans_item_${item_id}_unit_cost" type="hidden" value="${app.formatMoney(item_line.unit_cost)}" /> </td>
            <input id="trans_item_${item_id}_cost" type="hidden" value="${app.formatMoney(item_line.cost)}" readonly disabled /> </td>
            
            <div class="d-none">
            <input id="trans_item_${item_id}_quantity" type="number" class="form-control line-control text-center money" value="${item_line.quantity}" onchange="transaction.updateItemLine(${item_id}, $('#trans_item_${item_id}_quantity').val(), $('#trans_item_${item_id}_unit_cost').val(), $('#trans_item_${item_id}_unit_price').val())" />
            <input id="trans_item_${item_id}_unit_price" type="text" class="form-control line-control text-end money" value="${app.formatMoney(item_line.unit_price)}" onchange="transaction.updateItemLine(${item_id}, $('#trans_item_${item_id}_quantity').val(), $('#trans_item_${item_id}_unit_cost').val(), $('#trans_item_${item_id}_unit_price').val())" />
            <input id="trans_item_${item_id}_price" type="text" class="form-control line-control text-end money" value="${app.formatMoney(item_line.price)}" readonly disabled />
            </div>

            </td>
            <td>${item_line.dose} ${item_line.dose_units}</td>
            <td>${item_line.route}</td>

            <td class="pe-0">
            <a href="#" class="text-pointer text-danger" onclick="transaction.removeItem(${item_line.item_id})">
            <span class="fa-light fa-trash-alt"></span>
            </a>
            </td>
            </tr>`;
        }

        $('#trans_items_container').append(item_row);


        transaction.trans_items[item_id] = item_line;
        transaction.calculatePrice();
    }

    removeItem(item_id) {
        var trans_item      = transaction.trans_items[item_id];
        if(confirm('Are you sure you want to remove '+trans_item.name+' from the list?')) {
            delete transaction.trans_items[item_id];
            $('#trans_item_'+item_id).fadeOut(100).remove();
            transaction.calculatePrice();
        }
    }

    calculatePrice() {
        var trans_cost          = 0;
        var trans_cost          = parseFloat(trans_cost);
        var trans_subtotal      = 0;
        var trans_subtotal      = parseFloat(trans_subtotal);
        var trans_items         = [];
        for (let trans_item_id in this.trans_items) {
            if(this.trans_items[trans_item_id]) {
                trans_cost      += parseFloat(this.trans_items[trans_item_id].cost);
                trans_subtotal  += parseFloat(this.trans_items[trans_item_id].price);
                trans_items.push(this.trans_items[trans_item_id]);
            }
        }
        $('#trans_cost').val(app.formatMoney(trans_cost));
        $('#trans_subtotal').val(app.formatMoney(trans_subtotal));
        $('#trans_items').val(JSON.stringify(trans_items));
    }

    updateItemLine(item_id, item_quantity, item_unit_cost, item_unit_price) {
        var quantity                                        = app.unformatMoney(item_quantity);
        var unit_cost                                       = app.unformatMoney(item_unit_cost);
        var cost                                            = quantity * unit_cost;
        var unit_price                                      = app.unformatMoney(item_unit_price);
        var price                                           = quantity * unit_price;
        transaction.trans_items[item_id].quantity           = quantity;
        transaction.trans_items[item_id].qty                = quantity * transaction.trans_items[item_id].size;
        transaction.trans_items[item_id].requested_quantity = transaction.trans_items[item_id].qty;
        transaction.trans_items[item_id].unit_cost          = unit_cost;
        transaction.trans_items[item_id].cost               = cost;
        transaction.trans_items[item_id].unit_price         = unit_price;
        transaction.trans_items[item_id].price              = price;

        $('#trans_item_'+item_id+'_price').val(app.formatMoney(price));
        transaction.calculatePrice();
    }

    resetTransaction() {
        $('.trans-item').remove();
        this.trans_items    = [];
        transaction.calculatePrice();
    }

    loadStoreConsumptionIntoCart(store_id, period) {
        if(!store_id.length || !period.length) {
            alert('Please choose store and consumption period');
            return;
        }
        $('#trans_store_consumption').html(loading_bar).load(root+'pages/inventory/settings/stores/store_consumption.php?store_id='+store_id+'&period='+period);
    }

    loadTrackingDetails(transaction_type, transaction_id) {
        $('#trans_tracking_container').html('<div class="p-3 pb-4"><div class="text-center">'+loading_circle+'<br />Tracking Transaction...</div></div>').load(root+'html/transactions/tracking_details.php?transaction_type='+transaction_type+'&transaction_id='+transaction_id+'&r='+app.rootToNumber(r));
    }
}

transaction = new Transaction();