<?php

if(date('Y-m-d') > '2024-06-01') die('Licence for '.date('F Y').' is not updated');

#[AllowDynamicProperties]

class App {
    function __construct() { 
        global $db;
        $this->db = $db;

        //$this->readColors();
    }

    function readColors() {
        $colors         = array();
        foreach($this->read("SELECT code, f3, f4 FROM settings_categories WHERE type = 'ColorType'") as $row) {
            $row->hex   = $row->f3;
            $row->rgb   = $row->f4;
            $colors[$row->code] = $row;
        }
        $this->colors = (object) $colors;
    }
    
    //////////////////////////////////// HTML RENDERS //////////////////////////////////

    /* Alerts */

    function renderAlert(array $options = array()) {

        // Returns bootstrap alert 

        $message    = isset($options['message'])    ? $options['message']   : '';
        $class      = isset($options['class'])      ? $options['class']     : 'primary';
        $modal_id   = isset($options['modal_id'])   ? $options['modal_id'] : '';
        $load_list  = isset($options['load_list'])  ? $options['load_list'] : false;
        $icon       = isset($options['icon'])       ? '<span class="'.$options['icon'].' fa-fw"></span> ' : '';

        if(isset($options['title'])) {
            if(substr($options['title'], 0, 1) == '<') {
                $title = $options['title'];
            } else {
                $title = '<b>'.$icon.$options['title'].':</b> ';
            }
        } else {
            $title = '';
        }
        
        $html       = '<div class="alert alert-'.$class.' mb-2" role="alert">'.$title.$message.'</div>';

        if(strlen($modal_id)) $html .= $this->renderScript('app.closeModal(\''.$modal_id.'\', \''.$load_list.'\')');

        return $html;
    }

    function renderAlertEmpty($message='', $class='', $icon='') {

        // Returns bootstrap alert

        $message    = strlen($message)    ? $message   : 'We looked everywhere we coluldn\'t find any records';
        $class      = strlen($class)      ? $class   : 'warning';
        $icon       = strlen($icon)       ? $icon : 'fa-regular fa-circle-exclamation';
        //$title      = strlen($title)      ? $title : 'No records';
        
        $html       = '<div class="alert alert-'.$class.' mb-2 py-4 text-center" role="alert">
        <h1 class="my-2"><span class="'.$icon.'"></span></h1>
        <h4 class="my-3">'.$message.'</h4>
        </div>';

        return $html;
    }

    function renderAlertPrimary($message = '', $title='Information', $modal_id='', $load_list=false) {
        $message    = strlen($message) ? $message : 'Information';
        $html       = $this->renderAlert (
            array (
                'title'     => $title,
                'message'   => $message,
                'class'     => 'primary',
                'icon'      => 'fa-solid fa-info-circle',
                'modal_id'  => $modal_id,
                'load_list' => $load_list
            )
        );
        return $html;
    }

    function renderAlertSecondary($message = '', $title='Information', $modal_id='', $load_list=false) {
        $message    = strlen($message) ? $message : 'Information';
        $html       = $this->renderAlert (
            array (
                'title'     => $title,
                'message'   => $message,
                'class'     => 'secondary',
                'icon'      => 'fa-solid fa-info-circle',
                'modal_id'  => $modal_id,
                'load_list' => $load_list
            )
        );
        return $html;
    }

    function renderAlertSuccess($message = '', $title='Success', $modal_id='', $load_list=false) {
        $message    = strlen($message) ? $message : 'Operation success';
        $html       = $this->renderAlert (
            array (
                'title'     => $title,
                'message'   => $message,
                'class'     => 'success',
                'icon'      => 'fa-solid fa-check-circle',
                'modal_id'  => $modal_id,
                'load_list' => $load_list
            )
        );
        return $html;
    }
    
    function renderAlertInfo($message = '', $title='Information', $modal_id='', $load_list=false) {
        $message    = strlen($message) ? $message : 'Information';
        $html       = $this->renderAlert (
            array (
                'title'     => $title,
                'message'   => $message,
                'class'     => 'info',
                'icon'      => 'fa-solid fa-info-circle',
                'modal_id'  => $modal_id,
                'load_list' => $load_list
            )
        );
        return $html;
    }

    function renderAlertWarning($message = '', $title='Warning', $modal_id='', $load_list=false) {
        $message    = strlen($message) ? $message : 'Warning';
        $html       = $this->renderAlert (
            array (
                'title'     => $title,
                'message'   => $message,
                'class'     => 'warning',
                'icon'      => 'fa-solid fa-exclamation-triangle',
                'modal_id'  => $modal_id,
                'load_list' => $load_list
            )
        );
        return $html;
    }

    function renderAlertError($message = '', $title='Error', $modal_id='', $load_list=false) {
        $message    = strlen($message) ? $message : 'Failed';
        $html       = $this->renderAlert (
            array (
                'title'     => $title,
                'message'   => $message,
                'class'     => 'danger',
                'icon'      => 'fa-solid fa-exclamation-triangle',
                'modal_id'  => $modal_id,
                'load_list' => $load_list
            )
        );
        return $html;
    }

    /* Tables */

    function renderTableOpen(array $options = array()) {

        // Returns html to start boostrap table

        $columns    = $options['columns'];
        $css        = isset($options['css']) ? $options['css'] : '';
        $id         = isset($options['id']) ? $options['id'] : time();

        if(strpos($css, 'table-bordered') === false) {
            $html   = '<style> .th-inner {padding-left:0! important; } </style>';
        } else {
            $html   = '';
        }

        if(in_array('table-containerless', explode(' ', $css))) {
            $html  .= '<div><div class="table-responsive">';
        } else {
            $html  .= '<div class="bg-secondary bg-opacity-05 p-3 rounded border" style=""><div class="table-responsive">';
        }
        $html      .= '<table class="table table-hover align-middle '.$css.'" style="background-color:transparent">
		<thead><tr>';

        foreach($columns as $column) {
            $name   = $column['name'];
            $css    = isset($column['css']) ? $column['css'] : '';
            $css    = isset($column['css']) ? $column['css'] : '';
            $attr   = isset($column['attr']) ? $column['attr'] : '';
            
            if(strtolower($name) == 'options' || strtolower($name) == 'chagua') {
                $css    = 'text-center '.$css;
            }

            $html   .= '<th class="ps-1 '.$css.'" '.$attr.'><h5><b>'.$name.'</b></h5></th>';
		}

        $html       .= '</tr></thead><tbody id="'.$id.'">';
        return $html;
    }

    function renderTableClose(array $options = array()) { 

        $css        = isset($options['css']) ? $options['css'] : '';

        // Returns html to close table opened by renderTableOpen()

        $html   = '</tbody></table>';
       
        $html  .= '</div></div>';
        return $html;
    }

    /* Modals */

    function renderModalOpen(array $options = array()) {
        global $get;
        
        $title          = isset($options['title']) ? $options['title'] : '';
        $name           = isset($options['name']) ? $options['name'] : '';
        $icon           = isset($options['icon']) ? $options['icon'] : '';
        $shortname      = isset($options['shortname']) ? $options['shortname'] : '';
        $details        = isset($options['details']) ? $options['details'] : '';
        $color          = isset($options['color']) ? $options['color'] : 'primary';
        $heading        = isset($options['heading']) ? $options['heading'] : '';
        $left_width     = isset($options['left_width']) ? $options['left_width'] : 3;
        $close_button   = isset($options['close_button']) ? $options['close_button'] : true;
        $close_button_html = $close_button ? '<button type="button" class="btn-close" data-bs-dismiss="modal" aria-label="Close"></button>' : '';
        $right_width    = 12 - $left_width;
        $left_html      = isset($options['left_html']) ? $options['left_html'] : '<h1 class="text-white my-3"> <span class="'.$icon.'"></span> </h1>';
        if(strlen($name) && !isset($options['left_html'])) {
            $left_html .= '<h4 class="text-white my-4">'.$name.'</h4><hr />';
        }

        if(isset($options['title'])) {
            $title = $options['title'];
        } else {
            if($get->action == 'open') $title = $shortname.' details';
            if($get->action == 'create') $title = '<span class="fa-regular fa-file-circle-plus fa-fw"></span> Create new '.strtolower($shortname);
            if($get->action == 'edit') $title = '<span class="fa-regular fa-pen-to-square fa-fw"></span> Edit '.strtolower($shortname);
            if($get->action == 'copy') $title = '<span class="fa-regular fa-copy fa-fw"></span> Copy '.strtolower($shortname);
            if($get->action == 'delete') $title = 'Confirm action';
        }

        $margin = $color == 'danger' ? '-1px' : '0';

        $html  = '';

        $html .= '<div class="row">
        <div class="col-lg-'.$left_width.' bg-primary text-white text-center rounded-start p-4 desktop-pad-right-0" margin:'.$margin.' ">
        '.$left_html.' ';
        
        if(isset($options['tabs'])) {
            $tabs               = $options['tabs'];
            $html               .= '<nav class="nav nav-pills flex-column mt-4 mb-5">';
            $count              = count($tabs);
            $current            = 0;

            foreach($tabs as $tab) {
                $tab_name       = $tab['name'];
                $tab_title      = isset($tab['title']) ? $tab['title'] : '';
                $tab_css        = isset($tab['css']) ? $tab['css'] : '';
                $tab_icon       = isset($tab['icon']) ? $tab['icon'] : 'fa fa-ellipsis';
                $tab_icon       = str_replace('fa-light', 'fa-regular', $tab_icon);
                $tab_attr       = isset($tab['attr']) ? $tab['attr'] : '';
                $tab_id         = $tab['id'];
                $tab_icon       = str_replace('solid', 'regular', $tab_icon);
                $html           .= '<a id="'.$tab_id.'_wizard_button" class="fw-bold nav-link px-0 modal-wizard-link '.$tab_id.'-link '.$tab_id.'-wizard-button py-3 rounded-0 rounded-start '.$tab_css.'" style="margin-right:-1px; font-size:1.15em" href="#" data-icon=\''.$tab_icon.'\' data-title=\''.$tab_title.'\' data-bs-toggle="tab" data-bs-target="#'.$tab_id.'_content" '.$tab_attr.' onclick="$(\'#'.$tab_id.'_button\').trigger(\'click\')"><span class="'.$tab_icon.' fa-fw"></span>&nbsp; '.$tab_name.' </a>';

                if(++$current < $count) {
                    //$html       .= '<div class="text-center"><small><span class="fa-solid fa-arrow-down"></span></small></div>';
                }
            }
            $html               .= '</nav>';
        }

        // Skip icon if title came with icon
        if(strpos($title, 'fa-') === false) {
            $icon   = str_replace('fa-light', 'fa-solid', $icon);
            $icon   = '<span class="'.$icon.'"></span> &nbsp;';
        } else {
            $icon   = '';
        }
        
        $html .= '</div>
        <div class="col-lg-'.$right_width.' desktop-left-10 desktop-rounded-right m-0">
        <div class="modal-header p-4 pb-0 mobile-px-2">
        <h3 id="modal_title" class="modal-title fw-bold my-1"> '.$icon.''.str_replace('Fa-', 'fa-', $title).' </h3>
        '.$close_button_html.'
        </div>
        <div class="modal-body p-4 mobile-px-2">';

        $html .= '<script>
        app.initializeControls();
        $(\'.nav-link\').on(\'click\', function() { 
            var modal_title = $(this).data(\'title\');
            var modal_icon  = $(this).data(\'icon\');
            if(modal_icon != undefined) {
                var modal_icon  = modal_icon.replace(\'regular\', \'solid\');
            }
            if(modal_title != undefined && modal_title.length) {
                $(\'#modal_title\').html(\'<span class="\'+modal_icon+\' fa-fw"></span>&nbsp; \'+modal_title);
            }
        });
        </script>';
        return $html;
    }

    function renderModalClose(array $options = array()) {
        global $get;

        $form_id    = isset($options['form_id']) ? strtolower($options['form_id']) : 'ajax_form';
        $color      = isset($options['color']) ? $options['color'] : 'primary';
        $html       = '</div><div id="'.$form_id.'_footer" class="modal-footer py-0 my-0 mobile-px-2">';
        //$color      = 'secondary';

        if(isset($options['type'])) {
            $html       .= '<div class="row w-100 m-0">
            <div class="col-12 col-lg-4"></div>
            <div class="col-12 col-lg-12">
            <div class="row pb-4">
            <div class="col-6">'; 
        } else {
            $html       .= '<div class="row w-100 m-0">
            <div class="col-12 col-lg-4"></div>
            <div class="col-12 col-lg-8 mobile-px-0">
            <div class="row pb-4">
            <div class="col-6">';
        }

        $cancel_label = isset($options['cancel_label']) ? $options['cancel_label'] : '<span class="fa fa-arrow-left fa-fw"></span> Back';

        if(strlen($cancel_label)) {
            $html       .= '<button id="'.$form_id.'_cancel_button" type="button" class="btn btn-lg btn-outline-'.$color.' w-100" data-bs-dismiss="modal"> '.$cancel_label.'</button>';
        }

        $html       .= '</div><div class="col-6">';

        if($get->action != 'open') {
            $action_label = isset($options['action_label']) ? $options['action_label'] : '<span class="fa fa-check fa-fw"></span> Submit';
            if(strlen($action_label)) {
                $html   .= '<button id="'.$form_id.'_submit_button" type="button" class="btn btn-lg btn-'.$color.' w-100"> '.$action_label.' </button>';
            }
        }
        
        $html       .= '</div></div>';
        $html       .= '</div></div>';
        $html       .= '</div></div></div>';
        
        return $html;
    }

    /* Tabs */

    function renderTabsNavigation(array $tabs = array()) {
        $html               = '<ul class="nav nav-tabs nav-fill flex-column flex-md-row" id="tabs" role="tablist">';
        foreach($tabs as $tab) {
            $tab['css']     = isset($tab['css']) ? $tab['css'] : '';
            $tab['icon']    = isset($tab['icon']) ? $tab['icon'] : 'fa fa-ellipsis';
            $html           .= '<li class="nav-item" role="presentation">
            <button class="nav-link '.$tab['css'].' " id="'.$tab['id'].'_button" data-bs-toggle="tab" data-bs-target="#'.$tab['id'].'_content" type="button" role="tab"> <span class="'.$tab['icon'].' fa-fw"></span> '.$tab['name'].' </button>
            </li>';
        }
        $html               .= '</ul>';
        return $html;
    }

    function renderTabsOpen(array $options = array()) {
        $html = '<div class="tab-content pt-4" id="tabs_content"><div>';
        return $html;
    } 

    function renderTabsNext($tab_id, $css='') {
        $html = '</div><div class="tab-pane fade show '.$css.'" id="'.$tab_id.'_content" role="tabpanel" tabindex="0">';
        return $html;
    }

    function renderTabsClose() {
        $html = '</div></div>';
        return $html;
    } 

    function renderRowOpen($css='') {
        $css = strlen($css) ? $css : 'row';
        return '<div class="'.$css.'">';
    }

    function renderRowClose() {
        return '</div>';
    }

    function renderColumnOpen($css='') {
        $css = strlen($css) ? $css : 'col-12 col-sm-6 col-lg-4 col-xxl-3 pb-4';
        return '<div class="'.$css.'">';
    }

    function renderColumnClose() {
        return '</div>';
    }

    function renderCard($options = array()) {
        global  $set;
        $links  = isset($options['links']) ? $options['links'] : '';
        $icon   = isset($options['icon']) ? $options['icon'] : 'fa-light fa-pills';
        $name   = isset($options['name']) ? $options['name'] : 'Item name here';
        $body   = isset($options['body']) ? $options['body'] : '';

        $html   = $this->renderColumnOpen();
        $html   .= '<div class="card border-0 shadow-sm">';

        if(isset($options['cover'])) {
            $cover  = $options['cover'];
            $links  = str_replace('primary', 'outline-light border-0', $links);
            $html   .= '<div class="card-img-top p-3 text-center" style="min-height:160px; background-image: linear-gradient(to right, rgba(0, 0, 0,0.50), rgba(0, 0, 0,0.50)),
            url('.$cover->renderOpenPath().'); background-position:center; background-size:cover;">';
        } else {
            $html   .= '<div class="card-img-top p-3 text-center" style="min-height:160px">';
        }

        $html   .= '<p class="mt-5">'.$links.'</p>';
        
        /*
        $html   .= '<div class="row">
        <div class="col">
        <div class="card-body rounded bg-white border border-1 shadow-sm" style="position:relative; bottom:-30px;">
        <span class="'.$icon.' fa-2x"></span>
        </div></div>
        <div class="col"></div>
        <div class="col"></div>
        </div>';
        */
       
        $html   .= '</div>';

        $html   .= '<div class="card-body">';
        $html   .= '<h5 class="my-1 fw-light" style="min-height:50px">'.$name.'</h5>';
        $html   .= $body;

        $html   .= '</div>';
        $html   .= '</div>';
        $html   .= $this->renderColumnClose();
        return $html;
    }

    /* Wizards */

    function renderWizardNavigation(array $tabs = array()) {
        $html               = '<ul class="nav nav-tabs nav-fill flex-column flex-md-row d-none" id="tabs" role="tablist">';
        foreach($tabs as $tab) {
            $id      = $tab['id'];
            $name    = $tab['name'];
            $css     = isset($tab['css']) ? $tab['css'] : '';
            $icon    = isset($tab['icon']) ? $tab['icon'] : 'fa fa-ellipsis';
            $html    .= '<li class="nav-item" role="presentation">
            <button class="nav-link '.$css.' '.$id.'-button " id="'.$id.'_button" data-bs-toggle="tab" data-bs-target="#'.$id.'_content" type="button" role="tab"> <span class="'.$icon.' fa-fw"></span> '.$name.' </button>
            </li>';
        }
        $html               .= '</ul>';
        return $html;
    }

    function renderWizardOpen(array $options = array()) {
        $html = '<div class="tab-content id="tabs_content"><div>';
        return $html;
    } 

    function renderWizardNext($tab_id, $title='', $css='', $icon='') {
        $icon   = strlen($icon) ? '<span class="'.$icon.' fa-fw"></span>&nbsp; ' : '';
        $title  = strlen($title) ? '<h5 class="fw-light mt-1 mb-4">'.$icon.$title.'</h5>' : '';
        $html   = '</div><div class="tab-pane fade px-0 pt-3 '.$css.'" id="'.$tab_id.'_content" role="tabpanel" tabindex="0" style="margin-top:-26px">'.$title.'';
        return $html;
    }

    function renderWizardClose() {
        $html = '</div></div>';
        return $html;
    } 

    function renderWizardButtons(array $buttons) {
        $html   = '<div class="row pb-3"><div class="col-12 col-lg-4"></div><div class="col-12 col-lg-8"><div class="row">';
        $html   .= '<div class="col-6 mt-3">';
        if(isset($buttons['prev'])) {
            $button = $buttons['prev'];
            $css    = isset($button['css']) ? $button['css'] : '';
            $id     = isset($button['id']) ? $button['id'] : '';
            $icon   = isset($button['icon']) ? $button['icon'] : 'fa-regular fa-arrow-left';
            $label  = isset($button['label']) ? $button['label'] : 'Rudi';
            $label  = '<span class="'.$icon.' fa-fw"></span> '.$label;
            $attr   = isset($button['attr']) ? $button['attr'] : '';

            $html   .= '<a class="btn btn-outline-primary w-100 '.$id.'-link '.$css.'" onclick="$(\'.modal-wizard-link\').removeClass(\'active\'); $(\'.'.$id.'-wizard-button\').addClass(\'active\'); $(\'#'.$id.'_button\').trigger(\'click\')" '.$attr.'> '.$label.' </a>';
        }

        $html   .= '</div>
     
        <div class="col-6 mt-3">';

        if(isset($buttons['next'])) {
            $button = $buttons['next'];
            $css    = isset($button['css']) ? $button['css'] : '';
            $id     = isset($button['id']) ? $button['id'] : '';
            $icon   = isset($button['icon']) ? $button['icon'] : 'fa-regular fa-arrow-right';
            $label  = isset($button['label']) ? $button['label'] : 'Endelea';
            $label  = $label.' <span class="'.$icon.' fa-fw"></span>';
            $attr   = isset($button['attr']) ? $button['attr'] : '';

            $html   .= '<a class="btn btn-primary w-100 '.$id.'-link '.$css.'" onclick="$(\'.modal-wizard-link\').removeClass(\'active\'); $(\'.'.$id.'-wizard-button\').addClass(\'active\'); $(\'#'.$id.'_button\').trigger(\'click\')" '.$attr.'> '.$label.' </a>';
        }
        $html   .= '</div></div></div></div>';
        return $html;
    }

    function renderWizardBigButtons(array $buttons) {
        $html   = '<div class="row" style="position:relative; bottom:0px">';
        $html   .= '<div class="col-12 col-sm-6 mt-3">';
        if(isset($buttons['prev'])) {
            $button = $buttons['prev'];
            $css    = isset($button['css']) ? $button['css'] : '';
            $id     = isset($button['id']) ? $button['id'] : '';
            $icon   = isset($button['icon']) ? $button['icon'] : 'fa-regular fa-arrow-left';
            $label  = isset($button['label']) ? $button['label'] : 'Rudi';
            $label  = '<span class="'.$icon.' fa-fw"></span> '.$label;
            $attr   = isset($button['attr']) ? $button['attr'] : '';

            $html   .= '<a class="btn btn-outline-primary w-100 btn-lg '.$id.'-link '.$css.'" onclick="$(\'.modal-wizard-link\').removeClass(\'active\'); $(\'.'.$id.'-wizard-button\').addClass(\'active\'); $(\'#'.$id.'_button\').trigger(\'click\')" '.$attr.'> '.$label.' </a>';
        }

        $html   .= '</div>
     
        <div class="col-12 col-sm-6 mt-3">';

        if(isset($buttons['next'])) {
            $button = $buttons['next'];
            $css    = isset($button['css']) ? $button['css'] : '';
            $id     = isset($button['id']) ? $button['id'] : '';
            $icon   = isset($button['icon']) ? $button['icon'] : 'fa-regular fa-arrow-right';
            $label  = isset($button['label']) ? $button['label'] : 'Endelea';
            $label  = $label.' <span class="'.$icon.' fa-fw"></span>';
            $attr   = isset($button['attr']) ? $button['attr'] : '';

            $html   .= '<a class="btn btn-primary w-100 btn-lg '.$id.'-link '.$css.'" onclick="$(\'.modal-wizard-link\').removeClass(\'active\'); $(\'.'.$id.'-wizard-button\').addClass(\'active\'); $(\'#'.$id.'_button\').trigger(\'click\')" '.$attr.'> '.$label.' </a>';
        }
        $html   .= '</div></div>';
        return $html;
    }


    /* Forms */ 

    function renderFormOpen($options=array()) {

        // Returns html for start <form> tag

        $form_id        = isset($options['form_id']) ? $options['form_id'] : 'ajax_form';
        $form_action    = isset($options['form_action']) ? $options['form_action'] : 'process.php';
        $css            = isset($options['css']) ? $options['css'] : '';
        
        $html = '<form autocomplete="off" class="'.$css.'" id="'.strtolower($form_id).'" method="post" action="'.$form_action.'" enctype="multipart/form-data">
        <input autocomplete="false" name="hidden" type="text" style="display:none;">';
        return $html;
    } 

    function renderFormClose($options=array()) {
        $form_id    = isset($options['form_id']) ? $options['form_id'] : 'ajax_form';
        $html       = '<div class="progress d-none" id="'.$form_id.'_progress_container">
        <div class="progress-bar" role="progressbar" style="width:0%" id="'.$form_id.'_progress_bar">
        0%
        </div>
        </div>
        <div id="'.$form_id.'_response">
        </div>
        </form>';
        $html   .= $this->renderScript('app.prepareAjaxForm(\''.$form_id.'\')');

        return $html;
    }

    function renderFormControl(array $options = array()) {

        global $app, $get;

        if(isset($options['value'])) {
            if(is_array($options['value'])) {
                $value  = $options['value'];
            } else {
                $value  = html_entity_decode($options['value']);
            }
        } else {
            $value      = '';
        }

        $name           = $options['name'];
        $id             = preg_replace('/[^a-zA-Z0-9_\s]/', '', $name);
        $label          = isset($options['label']) ? (strlen($options['label']) && $options['label'] != 'none' ? $options['label'] : ucfirst(strtolower($name))) : ucfirst(strtolower($name));
        $label          = str_replace('_', ' ', $label);
        $type           = isset($options['type']) ? $options['type'] : 'text';
        //$value          = isset($options['value']) ? (is_array($options['value'] || is_object($options['value'])) ? $options['value'] : html_entity_decode($options['value'])) : '';
        $values         = isset($options['values']) ? $options['values'] : array();
        $css            = isset($options['css']) ? $options['css'] : '';
        $placeholder    = isset($options['placeholder']) ? $options['placeholder'] : '';
        $attributes     = isset($options['attributes']) ? $options['attributes'] : '';
        $styles         = explode(' ', $css);
        $form_text      = $placeholder;

        //$placeholder    = '';

        // If there is form-control-plaintext, remove line-control css
        if(in_array('form-control-plaintext', $styles)) {
            if($type == 'select') {
                $css = str_replace('form-control-plaintext', 'line-control-plaintext w-100', $css);
            } else {
                $css = str_replace('line-control', 'line-control-plaintext', $css);
            }
        }

        $html           = '<div id="'.$id.'_container">';

        // Floating labels?
        if(in_array('placeholder-floating', $styles)) {
           $html    .= '<div class="form-floating">';
        } else {
            $html   .= '<div>';
        }

        if($type == 'hidden') {
            $html   = '<input type="'.$type.'" name="'.$name.'" value="'.$value.'" placeholder="'.$placeholder.'" id="'.$id.'" class="'.$css.'" '.$attributes.' />';
            return $html;
        }

        switch($type) {
            case 'select': {
                if(in_array('data-list', $styles)) {
                    $html .= '<input name="'.$name.'" id="'.$id.'" value="'.$value.'" placeholder="'.$placeholder.'" class="'.$css.'" list="data_list_'.$id.'" '.$attributes.' title="'.$placeholder.'">
                    <datalist id="data_list_'.$id.'">';
                    
                    foreach($values as $val) {
                        if(is_array($val)) {
                            $val_name   = $val['name'];
                            $val_value  = $val['value'];
                            $selected   = $value == $val_value ? 'selected' : ''; 
                            $attr       = isset($val['attr']) ? $val['attr'] : ''; 
                            $html       .= '<option value="'.$val_value.'" '.$selected.' '.$attr.'>'.$val_name.'</option>';
                        }  elseif(is_object($val)) {
                            $val_name   = $val->name;
                            $val_value  = $val->value;
                            $selected   = $value == $val_value ? 'selected' : ''; 
                            $attr       = isset($val->attr) ? $val->attr : ''; 

                            if(isset($val->html)) {
                                $html   .= '<option value="'.$val_value.'" '.$selected.' data-content="'.$val->html.' '.$attr.'">'.$val_name.'</option>';
                            } elseif(isset($val->icon, $val->color)) {
                                $html   .= '<option value="'.$val_value.'" '.$selected.' data-content="<span class=\'text-'.$val->color.'\'><span class=\''.$val->icon.' fa-fw\'></span> '.$val_name.' </span>" '.$attr.'>'.$val_name.'</option>';
                            } else {
                                $html   .= '<option value="'.$val_value.'" '.$selected.' '.$attr.'>'.$val_name.'</option>';
                            }
                            
                        } else {
                            $selected   = $value == $val ? 'selected' : '';
                            $attr       = isset($val['attr']) ? $val['attr'] : ''; 
                            $html       .= '<option '.$selected.' '.$attr.'>'.$val.'</option>';
                        }
                    }

                    $html .= '</datalist>';
                } else {
                    
                    $html   .= '<select name="'.$name.'" id="'.$id.'" class="select-search px-0 '.$css.'" '.$attributes.' title="'.$placeholder.'">';

                    // Default all?
                    if(in_array('default-none', $styles)) {
                        $html .= '<option value=""> (None) </option>';
                    } elseif(in_array('default-all', $styles)) {
                        $html .= '<option value=""> (All) </option>';
                    } elseif(in_array('default-select', $styles)) {
                        $html .= '<option value="">- Select -</option>';
                    } elseif(in_array('default-chagua', $styles)) {
                        $html .= '<option value="">- Chagua -</option>';
                    }
                
                    foreach($values as $val) {
                        if(is_array($val)) {
                            $val_name   = $val['name'];
                            $val_value  = $val['value'];
                            $val_icon   = isset($val['icon']) ? 'data-icon="'.$val['icon'].' fa-fw"' : '';
                            $selected   = $value == $val_value ? 'selected' : ''; 
                            $attr       = isset($val['attr']) ? $val['attr'] : ''; 

                            if(strpos($attributes, 'multiple') !== false) {
                                $vals   = explode(',', $value);
                                if(in_array($val_value, $vals)) $selected = 'selected';
                            }

                            if(isset($val['html'])) {
                                $html   .= '<option value="'.$val_value.'" '.$selected.' data-content="'.$val['html'].'" '.$attr.'>'.$val_name.'</option>';
                            } elseif(isset($val['icon'], $val['color'])) {
                                $html   .= '<option value="'.$val_value.'" '.$selected.' data-content="<span class=\'text-'.$val['color'].'\'><span class=\''.$val['icon'].' fa-fw\'></span> '.$val_name.' </span>" '.$attr.'>'.$val_name.'</option>';
                            } else {
                                $html       .= '<option value="'.$val_value.'" '.$val_icon.' '.$selected.' '.$attr.'>'.$val_name.'</option>';
                            }
                            
                        }  elseif(is_object($val)) {
                            $val_name   = $val->name;
                            $val_value  = $val->value;
                            $val_icon   = isset($val->icon) ? 'data-icon="'.$val->icon.' fa-fw"' : '';
                            $selected   = $value == $val_value ? 'selected' : ''; 
                            $attr       = isset($val->attr) ? $val->attr : ''; 

                            if(isset($val->html)) {
                                $html   .= '<option value="'.$val_value.'" '.$selected.' data-content="'.$val->html.'" '.$attr.'>'.$val_name.'</option>';
                            } elseif(isset($val->icon, $val->color)) {
                                $html   .= '<option value="'.$val_value.'" '.$selected.' data-content="<span class=\'text-'.$val->color.'\'><span class=\''.$val->icon.' fa-fw\'></span> '.$val_name.' </span>" '.$attr.'>'.$val_name.'</option>';
                            } else {
                                $html   .= '<option value="'.$val_value.'" '.$selected.' '.$attr.'>'.$val_name.'</option>';
                            }
                        } else {
                            $attr       = isset($val['attr']) ? $val['attr'] : ''; 
                            if(strpos($attributes, 'multiple') !== false) {
                                if(is_array($value)) {
                                    $vals   = $value;
                                } else {
                                    $vals   = explode(',', $value);  
                                }
                                //if(in_array($val_value, $vals)) $selected = 'selected';
                            } else {
                                $selected   = $value == $val ? 'selected' : '';
                                $html       .= '<option '.$selected.' '.$attr.'>'.$val.'</option>';
                            }
                        }
                    }
                    $html   .= '</select>';
                }
                break;
            } case 'textarea': {

                $rows = '4';
                if(in_array('rows-1', $styles)) $rows = '1';
                if(in_array('rows-2', $styles)) $rows = '2';
                if(in_array('rows-3', $styles)) $rows = '3';
                if(in_array('rows-4', $styles)) $rows = '4';
                if(in_array('rows-5', $styles)) $rows = '5';
                if(in_array('rows-6', $styles)) $rows = '6';
                if(in_array('rows-7', $styles)) $rows = '7';
                if(in_array('rows-8', $styles)) $rows = '8';
                if(in_array('rows-9', $styles)) $rows = '9';

                $html   .= '<textarea name="'.$name.'" rows="'.$rows.'" placeholder="'.$placeholder.'" id="'.$id.'" class="'.$css.'" '.$attributes.'>'.$value.'</textarea>';
                break;
            } case 'text': {
                $html   .= '<input type="text" name="'.$name.'" value="'.$value.'" placeholder="'.$placeholder.'" id="'.$id.'" class="'.$css.'" '.$attributes.' autocomplete="off" />';
                break;
            } case 'button': {
                $html   .= '<a id="'.$id.'" href="'.$value.'" class="'.$css.'">'.$label.'</a>';
                break;
            } case 'file': {
                $html .= '<label class="w-100" for="'.$id.'">
                <a class="btn btn-outline-secondary w-100 '.$css.'" style="width:100%"> '.$placeholder.' </a>
                <input type="file" name="'.$name.'" id="'.$id.'" style="display:none" value="'.$value.'" '.$attributes.' />
                </label>';
                break;
            } default: {
                $html   .= '<input type="'.$type.'" name="'.$name.'" value="'.$value.'" placeholder="'.$placeholder.'" id="'.$id.'" class="'.$css.'" '.$attributes.' />';
            }
        }

        // Deal with different input styles

        if(in_array('grid-control', $styles)) {
            $html = '<div class="row gx-3 mb-4" id="'.$name.'_parent_container">
            <label id="'.$name.'_label" for="'.$name.'" class="col-sm-4 col-form-label fw-bold">'.$label.' : </label>
            <div class="col-sm-8"> '.$html;

            if(in_array('placeholder-underline', $styles) && strlen($form_text) && ($get->action == 'create' || $get->action == 'copy' || $get->action == 'edit')) {
                $html .= '<small class="form-text" style="color:rgba(100,100,100,0.5)"><i>'.$form_text.'</i></small>';
            }
            
            $html .= '</div></div>';
        }

        // Floating labels, again

        if(in_array('placeholder-floating', $styles)) {
            $html .= '<label for="'.$id.'">'.$placeholder.'</label>';
        }

        $html .= '</div></div>';

        return $html;
    }

    function renderInputText($name, $value='', $placeholder='', $css='', $attributes='', $label='') {
        return $this->renderFormControl(
            array ('type'=>'text', 'name'=>$name, 'value'=>$value, 'placeholder'=>$placeholder, 'css'=>$this->form_control.' '.$css, 'attributes'=>$this->control_attributes.' '.$attributes, 'label'=>$label)
        );
    }

    function renderInputPassword($name, $value='', $placeholder='', $css='', $attributes='', $label='') {
        return $this->renderFormControl(
            array ('type'=>'password', 'name'=>$name, 'value'=>$value, 'placeholder'=>$placeholder, 'css'=>$this->form_control.' '.$css, 'attributes'=>$this->control_attributes.' '.$attributes, 'label'=>$label)
        );
    }

    function renderInputYear($name, $value='', $placeholder='', $css='', $attributes='', $label='') {
        return $this->renderFormControl(
            array ('type'=>'text', 'name'=>$name, 'value'=>$value, 'placeholder'=>$placeholder, 'css'=>$this->form_control.' '.$css.' year-picker', 'attributes'=>$this->control_attributes.' '.$attributes, 'label'=>$label)
        );
    }

    function renderInputDate($name, $value='', $placeholder='', $css='', $attributes='', $label='') {
        return $this->renderFormControl(
            array ('type'=>'text', 'name'=>$name, 'value'=>$value, 'placeholder'=>$placeholder, 'css'=>$this->form_control.' '.$css.' date-picker', 'attributes'=>$this->control_attributes.' '.$attributes, 'label'=>$label)
        );
    }

    function renderInputTime($name, $value='', $placeholder='', $css='', $attributes='', $label='') {
        return $this->renderFormControl(
            array ('type'=>'text', 'name'=>$name, 'value'=>$value, 'placeholder'=>$placeholder, 'css'=>$this->form_control.' '.$css.' time-picker', 'attributes'=>$this->control_attributes.' '.$attributes, 'label'=>$label)
        );
    }

    function renderInputTextarea($name, $value='', $placeholder='', $css='', $attributes='', $label='') {
        return $this->renderFormControl(
            array ('type'=>'textarea', 'name'=>$name, 'value'=>$value, 'placeholder'=>$placeholder, 'css'=>$this->form_control.' '.$css, 'attributes'=>$this->control_attributes.' '.$attributes, 'label'=>$label)
        );
    }

    function renderInputHidden($name, $value='') {
        return $this->renderFormControl(
            array ('type'=>'hidden', 'name'=>$name, 'value'=>$value)
        );
    }

    function renderInputIcon($icon, $placeholder='', $css='', $attributes='', $label='') {
        $icons = $this->query("SELECT code value, name, icon, color FROM content_icons ORDER BY code");
        if(!strlen($icon)) $icon = 'fa-regular fa-square-dashed'; 
        return $this->renderFormControl (
            array ('type'=>'select', 'name'=>'icon', 'value'=>$icon, 'values'=>$icons, 'placeholder'=>$placeholder, 'css'=>$this->form_control.' default-select '.$css, 'attributes'=>$this->control_attributes.' '.$attributes, 'label'=>$label)
        );
    }
    
    function renderInputColor($color, $placeholder='', $css='', $attributes='', $label='') {
        $colors = $this->getCategories('ColorType');
        return $this->renderFormControl(
            array ('type'=>'select', 'name'=>'color', 'value'=>$color, 'values'=>$colors, 'placeholder'=>$placeholder, 'css'=>$this->form_control.' '.$css, 'attributes'=>$this->control_attributes.' '.$attributes, 'label'=>$label)
        );
    }

    function renderInputTags($value='', $placeholder='', $css='', $attributes='', $label='') {
        return $this->renderFormControl(
            array ('type'=>'text', 'name'=>'tags', 'value'=>$value, 'placeholder'=>$placeholder, 'css'=>$this->form_control.' '.$css, 'attributes'=>$this->control_attributes.' '.$attributes, 'label'=>$label)
        );
    }

    function renderInputAccess($value='', $placeholder='', $css='', $attributes='', $label='') {
        return $this->renderFormControl(
            array ('type'=>'textarea', 'name'=>'access', 'value'=>$value, 'placeholder'=>$placeholder, 'css'=>$this->form_control.' '.$css, 'attributes'=>$this->control_attributes.' '.$attributes, 'label'=>$label)
        );
    }

    function renderInputSelect($name, $value='', $placeholder='', $css='', $attributes='', $table_name='', $column_id='code', $column_name='name', $condition='', $label='') {
        
        $options            = array();
        if(is_array($table_name)) {
            $options        = $table_name;
        } else {
            $rows           = $this->getTableOptions($table_name, $column_id, $column_name, $condition);
            $pretty_id      = explode('.', $column_id);
            $pretty_id      = end($pretty_id);
            $pretty_name    = explode('.', $column_name);
            $pretty_name    = end($pretty_name);

            foreach($rows as $row) {
                $options[] = array(
                    'name'  => $row->$pretty_name,
                    'value' => $row->$pretty_id
                );
            }
        }

        return $this->renderFormControl(
            array ('type'=>'select', 'name'=>$name, 'value'=>$value, 'values'=>$options, 'placeholder'=>$placeholder, 'css'=>$this->form_control.' '.$css, 'attributes'=>$this->control_attributes.' '.$attributes, 'label'=>$label)
        );
    }

    function renderInputSelectNew($name, $value='', $placeholder='', $css='', $attributes='', $data_source='', $label='') {

        $data_source        = $this->stringToObject($data_source);
        $table_name         = $data_source->table;
        $column_id          = $data_source->id;
        $column_name        = $data_source->name;
        $condition          = $data_source->condition;

        $options            = array();
        if(is_array($table_name)) {
            $options        = $table_name;
        } else {
            $rows           = $this->getTableOptions($table_name, $column_id, $column_name, $condition);
            $pretty_id      = explode('.', $column_id);
            $pretty_id      = end($pretty_id);
            $pretty_name    = explode('.', $column_name);
            $pretty_name    = end($pretty_name);

            foreach($rows as $row) {
                $options[] = array(
                    'name'  => $row->$pretty_name,
                    'value' => $row->$pretty_id
                );
            }
        }

        return $this->renderFormControl(
            array ('type'=>'select', 'name'=>$name, 'value'=>$value, 'values'=>$options, 'placeholder'=>$placeholder, 'css'=>$this->form_control.' '.$css, 'attributes'=>$this->control_attributes.' '.$attributes, 'label'=>$label)
        );
    }


    function renderInputPeople($name, $value='', $placeholder='', $css='', $attributes='', $label='', $joined_table='') {
        if(is_array($joined_table)) {
            $people = $joined_table;
        } else {
            $people = array();
        }
        
        return $this->renderInputSelect($name, $value, $placeholder, $css, $attributes, $people, '', '', '', $label);
    }

    function renderInputMoney($name, $value='', $placeholder='', $css='', $attributes='', $label='') {
        return $this->renderFormControl(
            array ('type'=>'text', 'name'=>$name, 'value'=>$value, 'placeholder'=>$placeholder, 'css'=>$this->form_control.' money '.$css, 'attributes'=>$this->control_attributes.' '.$attributes, 'label'=>$label)
        );
    }
 
    function renderInputNumber($name, $value='', $placeholder='', $css='', $attributes='', $label='') {
        return $this->renderFormControl(
            array ('type'=>'number', 'name'=>$name, 'value'=>$value, 'placeholder'=>$placeholder, 'css'=>$this->form_control.' '.$css, 'attributes'=>$this->control_attributes.' '.$attributes, 'label'=>$label)
        );
    }

    function renderInputButton($name, $value='', $placeholder='', $css='', $attributes='', $label='') {
        return $this->renderFormControl(
            array ('type'=>'button', 'name'=>$name, 'value'=>$value, 'placeholder'=>$placeholder, 'css'=>$this->form_control.' '.$css, 'attributes'=>$this->control_attributes.' '.$attributes, 'label'=>$label)
        );
    }

    function renderInputFields($obj, $fields_count, $placeholder='', $css='', $attributes='', $label='') {

        // Returns html for extra input fields, f1 - f $fields_count
        global $app;
        $html                   = '';
        if($obj->table == 'settings_categories' || $obj->table == 'settings_category_types') {
            if(!isset($obj->type)) $obj->type = '';
            $type_id    = $app->getColumnValue('settings_category_types', 'id', 'code', $obj->type);
            $type       = new CategoryType($type_id);
            
            for($counter = 1; $counter <= $fields_count; $counter++) {
                $field_value = 'f'.$counter;
               
                if(strlen($type->$field_value)) {
                    // $labels = explode(',', $type->$field_value);
                    $labels = explode(';', $type->$field_value);
                    if(count($labels) > 3) {
                        $data_source    = $this->stringToObject($type->$field_value);
                        $label          = isset($data_source->label) ? $data_source->label : 'f'.$counter;
                        /*
                        $label  = $labels[0];
                        $table  = $labels[1];
                        $code   = isset($labels[2]) ? $labels[2] : 'code';
                        $name   = isset($labels[3]) ? $labels[3] : 'name';
                        $cond   = isset($labels[4]) ? $this->uncleanText($labels[4]) : '';
                        $html   .= $this->renderInputSelect('f'.$counter, $obj->$field_value, '', 'line-control grid-control default-none select-search', '', $table, $code, $name, $cond, ucfirst($label));
                        */
                        $html   .= $this->renderInputSelectNew('f'.$counter, $obj->$field_value, '', 'line-control grid-control default-none select-search', '', $type->$field_value, ucfirst($label));
                    } else {
                        $label  = $type->$field_value;
                        if($counter > 5) {
                            $html       .= $this->renderInputTextarea('f'.$counter, $obj->$field_value, '', 'line-control grid-control', '', ucfirst($label));
                        } else {
                            $html       .= $this->renderInputText('f'.$counter, $obj->$field_value, '', 'line-control grid-control', '', ucfirst($label));
                        }
                    }
                } else {
                    $label  = 'f'.$counter;
                    if($counter > 5) {
                        $html       .= $this->renderInputTextarea('f'.$counter, $obj->$field_value, '', 'line-control grid-control', '', ucfirst($label));
                    } else {
                        $html       .= $this->renderInputText('f'.$counter, $obj->$field_value, '', 'line-control grid-control', '', ucfirst($label));
                    }
                }
            }
            return $html;
        } else {
            $label          = 'f';
        }

        for($counter = 1; $counter <= $fields_count; $counter++) {
            $field_value    = 'f'.$counter;
            $label          = $field_value;
            $html           .= $this->renderInputText('f'.$counter, $obj->$field_value, '', 'line-control grid-control', '', ucfirst($label));
        }
        return $html;
    }

    function renderInputCounter($name, $value, $counter, $placeholder='', $css='', $attributes='', $label='') {
        $numbers         = array();
        for($count = 0; $count < $counter; $count++) $numbers[] = $count;

        return $this->renderFormControl(
            array ('type'=>'select', 'name'=>$name, 'value'=>$value, 'values'=>$numbers, 'placeholder'=>$placeholder, 'css'=>$this->form_control.' '.$css, 'attributes'=>$this->control_attributes.' '.$attributes, 'label'=>$label)
        );
    }

    function renderInputFile($name, $value='', $placeholder='', $css='', $attributes='', $label='') {
        return $this->renderFormControl(
            array ('type'=>'file', 'name'=>$name, 'value'=>$value, 'placeholder'=>$placeholder, 'css'=>$this->form_control.' '.$css, 'attributes'=>$this->control_attributes.' '.$attributes, 'label'=>$label)
        );
    }

    function renderInputFiles($name, $value='', $placeholder='', $css='', $attributes='', $label='') {
        return $this->renderFormControl(
            array ('type'=>'file', 'name'=>$name, 'value'=>$value, 'placeholder'=>$placeholder, 'css'=>$this->form_control.' '.$css, 'attributes'=>$this->control_attributes.' multiple '.$attributes, 'label'=>$label)
        );
    }

    // Links

    function renderLinkOpen(array $options=array()) {

        // Returns html for link to open item in list

        $id     = isset($options['id']) ? $options['id'] : 0;
        $name   = isset($options['name']) ? $options['name'] : 'Item name';
        $modal  = isset($options['modal']) ? $options['modal'] : 'modal_lg';
        $html   = '<a class="remote-modal text-decoration-none fw-semibold" data-bs-toggle="modal" data-bs-target="#'.$modal.'" href="form.php?action=open&id='.$id.'">'.$name.'</a>';
        return $html;
    }

    function renderButtonOpen(array $options=array()) {

        // Returns html for link to open item in list

        $id     = isset($options['id']) ? $options['id'] : 0;
        $name   = isset($options['name']) ? $options['name'] : 'Item name';
        $css    = isset($options['css']) ? $options['css'] : 'btn btn-default';

        $html   = '<a class="remote-modal bs-tooltip '.$css.'" title="Open" data-bs-toggle="modal" data-bs-target="#modal_lg" data-bs-backdrop="static" data-bs-keyboard="false" tabindex="-1" href="form.php?action=open&id='.$id.'"> <span class="fa-regular fa-up-right-from-square"></span></a>';
        return $html;
    }

    function renderButtonCopy(array $options=array()) {

        // Returns html for link to cop item in list

        $id     = isset($options['id']) ? $options['id'] : 0;
        $name   = isset($options['name']) ? $options['name'] : 'Item name';
        $css    = isset($options['css']) ? $options['css'] : 'btn btn-default';

        $html   = '<a class="remote-modal bs-tooltip '.$css.'" title="Copy" data-bs-toggle="modal" data-bs-target="#modal_lg" data-bs-backdrop="static" data-bs-keyboard="false" tabindex="-1" href="form.php?action=copy&id='.$id.'"> <span class="fa-regular fa-copy"></span> </a>';
        return $html;
    }

    function renderButtonEdit(array $options=array()) {

        // Returns html for link to cop item in list

        $id     = isset($options['id']) ? $options['id'] : 0;
        $name   = isset($options['name']) ? $options['name'] : 'Item name';
        $title   = isset($options['title']) ? $options['title'] : 'Edit';
        $css    = isset($options['css']) ? $options['css'] : 'btn btn-default';

        $html   = '<a class="remote-modal bs-tooltip '.$css.'" title="'.$title.'" data-bs-toggle="modal" data-bs-target="#modal_lg" data-bs-backdrop="static" data-bs-keyboard="false" tabindex="-1" href="form.php?action=edit&id='.$id.'"> <span class="fa-regular fa-pen-to-square"></span> </a>';
        return $html;
    }

    function renderButtonDelete(array $options=array()) {

        // Returns html for link to cop item in list

        $id     = isset($options['id']) ? $options['id'] : 0;
        $name   = isset($options['name']) ? $options['name'] : 'Item name';
        $css    = isset($options['css']) ? $options['css'] : 'btn btn-default';

        $html   = '<a class="remote-modal bs-tooltip '.$css.'" title="Delete" data-bs-toggle="modal" data-bs-target="#modal_md" href="form.php?action=delete&id='.$id.'"> <span class="fa-regular fa-trash"></span> </a>';
        return $html;
    }

    function renderButtonRun(array $options=array()) {

        // Returns html for link to cop item in list

        $id     = isset($options['id']) ? $options['id'] : 0;
        $name   = isset($options['name']) ? $options['name'] : 'Item name';
        $css    = isset($options['css']) ? $options['css'] : 'btn btn-default';

        $html   = '<a class="remote-modal bs-tooltip '.$css.'" title="Run" data-bs-toggle="modal" data-bs-target="#modal_lg" data-bs-backdrop="static" data-bs-keyboard="false" tabindex="-1" href="form.php?action=run&id='.$id.'"> <span class="fa-regular fa-play"></span> </a>';
        return $html;
    }

    function renderVerticalPad($pad='0px') {
        return '<div class="d-none d-lg-block" style="padding-bottom:'.$pad.'"> &nbsp; </div>';
    }

    // Javascript

    function renderScript($script) {
        $html = '<script>'.$script.'</script>';
        return $html;
    }

    // Data sanitization

    function cleanText($text) {   
        // Returns sanitized text ready for db storage
        return htmlspecialchars($text, ENT_QUOTES, 'UTF-8');
    }

    function uncleanText($text) {
        // Returns unsanitized text from db
        return htmlspecialchars_decode(html_entity_decode($text, ENT_COMPAT, 'UTF-8'), ENT_QUOTES);
    }

    function readText($text, $class='', $target='_blank') {
        $str    = $this->uncleanText($text);
        $str    = preg_replace('!((http\:\/\/|ftp\:\/\/|https\:\/\/)|www\.)([-a-zA-Zа-яА-Я0-9\~\!\@\#\$\%\^\&\*\(\)_\-\=\+\\\/\?\.\:\;\'\,]*)?!ism', '<a class="'.$class.'" href="//$3" target="'.$target.'">$1$3</a>', $str);
        $str    = nl2br($str);
        return  $str;
    }

    public function readReportParameters($raw_parameters, $class='', $target='_blank') {
		$clean_parameters 		= $this->nl2br($raw_parameters, '\n');
		$clean_parameters 		= preg_replace('/[\x00-\x1F\x7F]/u', '', $clean_parameters);
		$parameters 			= json_decode($clean_parameters);
	
		$html					= '<div class="row">';

		foreach($parameters as $parameter) {
			$title				= $parameter->title;
			$size				= $parameter->size;
			$value				= $parameter->value;
			$value				= nl2br(preg_replace('!((http\:\/\/|ftp\:\/\/|https\:\/\/)|www\.)([-a-zA-Zа-яА-Я0-9\~\!\@\#\$\%\^\&\*\(\)_\-\=\+\\\/\?\.\:\;\'\,]*)?!ism', '<a class="'.$class.'" href="//$3" target="'.$target.'">$1$3</a>', $value));
            $html               .= '<div class="col-sm-'.$size.' my-2"><h6 class="fw-bold text-secondary">'.$title.'</h6>'.$value.'</div>';
        }

        $html					.= '</div>';
        return $html;
	}


    public function readParameters($raw_parameters, $class='', $target='_blank') {
		global $root;
	
		$clean_parameters 		= $this->nl2br($raw_parameters, '\n');
		$clean_parameters 		= preg_replace('/[\x00-\x1F\x7F]/u', '', $clean_parameters);
		$parameters 			= json_decode($clean_parameters);
		$even_color				= 'rgb(240,240,240)';
		$odd_color				= 'rgb(250,250,250)';
		$bg_color 				= $even_color;

		$str					= '<table style="width:100%; padding:6px">
<tr style="width:100%;">
';
		$total_width			= 0;
		$factor					= 8.3;
		$count					= 0;

		foreach($parameters as $parameter) {
			$type				= $parameter->type;
			$title				= $parameter->title;
			$size				= $parameter->size;
			$value				= $parameter->value;
			$value				= nl2br(preg_replace('!((http\:\/\/|ftp\:\/\/|https\:\/\/)|www\.)([-a-zA-Zа-яА-Я0-9\~\!\@\#\$\%\^\&\*\(\)_\-\=\+\\\/\?\.\:\;\'\,]*)?!ism', '<a class="'.$class.'" href="//$3" target="'.$target.'">$1$3</a>', $value));
			$width 				= $size * $factor;

			if($total_width >= 92) { 
				// Go to new line
				$total_width 	= 0;
				$str			.= '
</tr>

<tr style="width:96%;">';
				if($bg_color == $even_color) {
					$bg_color = $odd_color;
				} else {
					$bg_color = $even_color;
				}
				
			} 
			
			$break 	= '';

			if($type == 'title' && !strlen(trim($title))) {

			} else {
				$str 			.= '
    <td style="width:'.$width.'%; padding:4px"><b>'.$title.$break.' </b>'.$value.'</td>';
			}

			$total_width		+= $width;
		}

		$str					.= '
</tr>

</table>';

$str = str_replace('<tr style="width:96%;">
</tr>', '', $str);

		return $str;
	}

    public function cleanJson($text) {
        // JSON_INVALID_UTF8_IGNORE;
        $text = str_replace(PHP_EOL, '\\n', $text);
		$text = preg_replace('/[\x00-\x1F\x7F]/u', '', $text);
        return $text;
	}

    public function nl2br($str) {
		return str_replace(array("\r\n", "\r", "\n"), "<br />", $str);
	}

    function formatName($name) {
        $name   = ucwords(strtolower($this->cleanJson($name)));
        $name   = str_replace('( ', '(', ucwords(str_replace('(', '( ', $name)));
        return $name;
    }

    function explodeByNewLine($text) {
        return preg_split('/[\r\n]+/', $text);
    }

    ///////////////////////// DATABASE OPERATIONS /////////////////////////////////

    function query($sql) {

        // Queries the database and returns results set

        global $db;
        return   $db->read($sql);
    }

    function read($sql) {
        return $this->query($sql);
    }

    function select($sql) {
        return $this->query($sql);
    }

    function getTableOptions($table_name, $column_id='id', $column_name='name', $condition="") {

        // Returns array of items in a table

        if(strlen($condition)) $condition = " AND $condition ";
        $pretty_id      = explode('.', $column_id);
        $pretty_id      = end($pretty_id);
        $pretty_name    = explode('.', $column_name);
        $pretty_name    = end($pretty_name);

        $tables         = explode(',', $table_name);
        $column_order   = $table_name == 'settings_categories' ? 'priority' : 'name';

        if(count($tables) == 3) {
            $column_order       = $column_name;
            // Joined tables
            $table_one          = $tables[0];
            $table_two          = $tables[1];
            $joined_condition   = $tables[2];
            $sql                = "SELECT $column_id $pretty_id, $column_name $pretty_name FROM $table_one JOIN $table_two ON $joined_condition WHERE $table_one.status = 1 $condition ORDER BY $column_order ";
        } else {
            // Simple table
            $sql                = "SELECT $column_id $pretty_id, $column_name $pretty_name FROM $table_name WHERE status = 1 $condition ORDER BY $column_order ";
        }
        
        return $this->query($sql);
    }

    function array2Json($rows, $key='') {

        // Takes array or object and returns Json optionally with keys

        $data   = array();
        if(strlen($key)) {
            foreach($rows as $row) {
                $data[$row->$key] = $row;
            }
        } else {
            $data   = $rows;
        }
        return json_encode($data);
    }

    public function encodeImageBase64($image_path) {
        $image_type 	        = 'image/jpeg';
        if(strpos($image_path, 'pages')) {
            $address            = $_SERVER['REQUEST_SCHEME'].'://'.$_SERVER['HTTP_HOST'].$_SERVER['REQUEST_URI'];
            $first_addresses    = explode('pages', $address);
            $first_address      = $first_addresses[0];
            $second_addresses   = explode('pages', $image_path);
            $second_address     = $second_addresses[1];
            $full_address       = $first_address.'pages'.$second_address;
            $image_data 	    = file_get_contents($full_address);
        } else {
            $image_data 	    = file_get_contents($image_path);
        }
       
		$image_base64           = 'data:image/'.$image_type.';base64,'.base64_encode($image_data);
		return $image_base64;
	}

    function getCategories($type, $column_id='code',  $column_name='name, icon, color', $condition='') {

        // Returns array of categories for use in input fields
        $condition  = strlen($condition) > 3 ? " AND $condition " : "$condition";
        $condition .= "AND type = '$type' ";

        if(strlen($column_name) && strlen($column_id)) {
            $sql    = "SELECT $column_id value, $column_name FROM settings_categories WHERE settings_categories.status > 0 $condition ORDER BY settings_categories.priority, settings_categories.name";
        } else {
            $sql    = "SELECT * FROM settings_categories WHERE settings_categories.status > 0 $condition ORDER BY settings_categories.priority, settings_categories.name";
        }
        return $this->query($sql);
    }

    function getAllCategories($type, $condition='') {

        // Returns array of category objects
        $condition  = strlen($condition) > 3 ? " AND $condition " : "$condition";
        $condition .= "AND type = '$type' AND status = 1 ";
        $sql        = "SELECT id FROM settings_categories WHERE settings_categories.status > 0 $condition ORDER BY settings_categories.priority, settings_categories.name";
        $categories = array();
        foreach($this->query($sql) as $row) {
            $categories[] = new Category($row->id);
        }
        return $categories;
    }

    function user($user_id) {
        return new User($user_id);
    }

    function getColumnValue($table_name, $column_return, $column_check, $column_value, $condition=1) {
        $sql    = "SELECT $column_return FROM $table_name WHERE $column_check = '$column_value' AND $condition ";
        $rows   = $this->query($sql);
        foreach($rows as $row) return $row->$column_return;
        return 0;
    }

    // Misclaneous functions
    function tags2json(string $tags) {
        // Converts comma separted strings to tags
        if(strlen($tags)) return json_encode(explode(',', $tags));
        return json_encode(array());
    }

    function formatMoney($money) {
        global $client;
        $money = is_numeric($money) ? $money : 0;
        return number_format($money, 2);
    }

    function formatCurrency($money) {
        global $client;
        $money = is_numeric($money) ? $money : 0;
        return $client->currency_code.' '.number_format($money, 2);
    }

    function unformatMoney($money) {
        $symbol             = strpos($money, '-') === false ? '' : '-';
        $clean_string       = preg_replace('/([^0-9\.,])/i', '', $money);
        $numbers_string     = preg_replace('/([^0-9\.])/i', '', $money);
        $numbers_string     = strlen($numbers_string) ? $numbers_string : 0;
        $erased_separators  = strlen($clean_string) - strlen($numbers_string - 1);
        $commas_or_dots     = preg_replace('/([,])/', '', $clean_string, $erased_separators );
        $removed_separators = preg_replace('/(,)(?=[0-9]{3,}$)/', '',  $commas_or_dots);
        $amount             = (float) str_replace(',', '.', $removed_separators);
        return $symbol.$amount;
    }

    public function readNumber($number) {
		$hyphen      = '-';
		$conjunction = ' and ';
		$separator   = ', ';
		$negative    = 'negative ';
		$decimal     = ' point ';
		$dictionary  = array(
			0                   => 'zero',
			1                   => 'one',
			2                   => 'two',
			3                   => 'three',
			4                   => 'four',
			5                   => 'five',
			6                   => 'six',
			7                   => 'seven',
			8                   => 'eight',
			9                   => 'nine',
			10                  => 'ten',
			11                  => 'eleven',
			12                  => 'twelve',
			13                  => 'thirteen',
			14                  => 'fourteen',
			15                  => 'fifteen',
			16                  => 'sixteen',
			17                  => 'seventeen',
			18                  => 'eighteen',
			19                  => 'nineteen',
			20                  => 'twenty',
			30                  => 'thirty',
			40                  => 'fourty',
			50                  => 'fifty',
			60                  => 'sixty',
			70                  => 'seventy',
			80                  => 'eighty',
			90                  => 'ninety',
			100                 => 'hundred',
			1000                => 'thousand',
			1000000             => 'million',
			1000000000          => 'billion',
			1000000000000       => 'trillion',
			1000000000000000    => 'quadrillion',
			1000000000000000000 => 'quintillion'
		);

		if (!is_numeric($number)) {
			return false;
		}
		
		if (($number >= 0 && (int) $number < 0) || (int) $number < 0 - PHP_INT_MAX) {
			// overflow
			trigger_error(
				'read_number only accepts numbers between -' . PHP_INT_MAX . ' and ' . PHP_INT_MAX,
				E_USER_WARNING
			);
			return false;
		}

		if ($number < 0) {
			return $negative . $this->readNumber(abs($number));
		}

		$string = $fraction = null;

		if (strpos($number, '.') !== false) {
			list($number, $fraction) = explode('.', $number);
		}

        error_reporting(E_ERROR | E_WARNING | E_PARSE);
        
		switch (true) {
			case $number < 21:
				$string = $dictionary[$number];
				break;
			case $number < 100:
				$tens   = ((int) ($number / 10)) * 10;
				$units  = $number % 10;
				$string = $dictionary[$tens];
				if ($units) {
					$string .= $hyphen . $dictionary[$units];
				}
				break;
			case $number < 1000:
				$hundreds   = $number / 100;
				$remainder  = $number % 100;
				$string     = $dictionary[$hundreds] . ' ' . $dictionary[100];
				if ($remainder) {
					$string .= $conjunction . $this->readNumber($remainder);
				}
				break;
			default:
				$baseUnit 		= pow(1000, floor(log($number, 1000)));
				$numBaseUnits 	= (int) ($number / $baseUnit);
				$remainder 		= $number % $baseUnit;
				$string 		= $this->readNumber($numBaseUnits) . ' ' . $dictionary[$baseUnit];
				if ($remainder) {
					$string 	.= $remainder < 100 ? $conjunction : $separator;
					$string 	.= $this->readNumber($remainder);
				}
				break;
		}

		if (null !== $fraction && is_numeric($fraction)) {
			$string .= $decimal;
			$words = array();
			foreach (str_split((string) $fraction) as $number) {
				$words[] = $dictionary[$number];
			}
			$string .= implode(' ', $words);
		}

		return $string;
	}

    // Pagenation
    function pagenate(array $data) {

        global $get;
        $current_page       = isset($get->list_current_page) ? $get->list_current_page : 1;
        $records_perpage    = isset($get->list_records_perpage) ? $get->list_records_perpage : 30;
        //$records_perpage    = 30;

        $zero_page          = $current_page - 1;
        $records            = $data;

        $records_all        = count($records);
        $records_current    = 1;

        if($records_all < ($records_perpage + 1)) {
			$records_pages = 1;
		} else {
			$records_pages_raw = $records_all/$records_perpage;
			$records_pages_raw = explode('.', $records_pages_raw);	

			if($records_all % $records_perpage) {
				if($records_pages_raw[1]) {
					$records_pages  = $records_pages_raw[0] + 1;
				} else {
					$records_pages  = $records_pages_raw[0];
				}
			} else {
				$records_pages      = $records_pages_raw[0];
			}
		}

        $current_page = $current_page <= $records_pages ? $current_page : $records_pages;

        // Pages
        $first_page         = 1;
        $previous_page      = ($current_page - 1) > 0 ? ($current_page - 1) : $current_page;
        $next_page          = ($current_page + 1) <= $records_pages ? ($current_page + 1) : $current_page;
        $last_page          = $records_pages;
        $current_item       = $records_perpage * ($current_page - 1) + 1;
        $current_item_limit = $current_page * $records_perpage;
        $current_item_limit = $current_item_limit <= $records_all ? $current_item_limit : $records_all;

        // Disable buttons
        $disabled_first     = $current_page == 1 ? 'disabled' : '';
        $disabled_previous  = $current_page > 1 ? '' : 'disabled';
        $disabled_next      = $current_page < $records_pages ? '' : 'disabled';
        $disabled_last      = $current_page == $records_pages ? 'disabled' : '';

        // Jump to page intervals
        $page_interval  = 1;
        if($records_pages > 999) $page_interval = 1;
        if($records_pages > 9999) $page_interval = 10;
        if($records_pages > 99999) $page_interval = 100;
        if($records_pages > 999999) $page_interval = 1000;
        if($records_pages > 9999999) $page_interval = 10000;
        
        $pagenation = '<div class="row text-light mt-2 mb-4">';

        $pagenation .= '<div class="col-lg-6 col-xxl-4 mt-4">
        <div class="btn-group select-page d-flex border rounded bg-secondary bg-opacity-05 p-1" role="group">
        <a type="button" class="btn btn-link text-decoration-none no-pointer"> Items '.$current_item.' - '.$current_item_limit.' out of '.number_format($records_all).' </a>
        </div></div>';

        $pagenation .= '<div class="col-lg-6 col-xxl-4 mt-4">
        <div class="btn-group select-page d-flex border bg-secondary bg-opacity-05 p-1" role="group">
        <a type="button" title="First page" class="bs-tooltip btn btn-link text-decoration-none '.$disabled_first.'" onclick="app.loadListPage('.$first_page.')"> <span class="fa-regular fa-backward-fast"></span></a>
        <a type="button" title="Prev page" class="bs-tooltip btn btn-link text-decoration-none '.$disabled_previous.'" onclick="app.loadListPage('.$previous_page.')"> <span class="fa-regular fa-backward"></span></a>
        <select class="select-search bs-tooltip" data-style="btn btn-link text-decoration-none" onchange="app.loadListPage(this.value)">';

        if($current_page <= $records_pages) {
            $pagenation .= '<option value="'.$current_page.'">Page '.$current_page.' / '.number_format($records_pages).'</option>'; 
        }

        if($records_all > $records_current) {
            for($number = 0; $number < $records_pages; ($number = $number + $page_interval)) {
                if($number) {
                    if($number != $current_page) {
                        $pagenation .= '<option value="'.$number.'">'.$number.' / '.number_format($records_pages).'</option>'; 
                    }
                }
            }
        }
       
       $pagenation .='</select>
        <a type="button" title="Next page" class="bs-tooltip btn btn-link text-decoration-none '.$disabled_next.'" onclick="app.loadListPage('.$next_page.')"> <span class="fa-regular fa-forward fa-fw"></span> </a>
        <a type="button" title="Last page" class="bs-tooltip btn btn-link text-decoration-none '.$disabled_last.'" onclick="app.loadListPage('.$last_page.')"> <span class="fa-regular fa-forward-fast fa-fw"></span> </a>
        </div></div>';

        $pagenation .= '<div class="col-lg-6 col-xxl-4 mt-4">
        <div class="btn-group d-flex border bg-secondary bg-opacity-05 p-1" role="group">
        <button title="First page" class="bs-tooltip btn btn-link text-decoration-none no-pointer"> Show </button>
        <select class="select-search bs-tooltip" data-style="btn btn-link text-decoration-none" onchange="app.list_records_perpage = this.value; app.loadList()">';
       
        if($records_all > $records_current) {
            for($number = 10; $number < 10000001; ($number < 100 ? $number = $number + 10 : $number = $number * 10)) {
                if($records_all > ($number + 1)) {
                    $selected   = $number == $records_perpage ? 'selected' : '';
                    $pagenation .= '<option value="'.$number.'" '.$selected.'>'.number_format($number).' items </option>'; 
                }
            }
        }
        $pagenation .= '<option value="10000000">All items</option>';
        
        $pagenation .= '</select>
        <button title="Last page" class="bs-tooltip btn btn-link text-decoration-none no-pointer"> per<span style="color:transparent">_</span>page </button>
        </div></div>';

        $pagenation .= '</div>';

        $data       = array_slice($data, ($zero_page * $records_perpage), $records_perpage);

        $results    = array (
            'rows'          => $data,
            'pagination'    => $pagenation
        );

        return json_decode(json_encode($results));
    }

    function validate($control) {
        // Validates user input, returns a valid input or error
        
        $control        = json_decode(json_encode($control));
        $control->id    = isset($control->id) ? $control->id : $control->name;
        $highlight      = $this->renderScript('$(\'#'.$control->id.'\').focus()');
        $message        = isset($control->message) ? $control->message : str_replace('_', ' ', ucfirst($control->name)).' is required. ';

        // Min
        if(isset($control->min)) {
            if(strlen($control->value) < $control->min) {
                echo $highlight;
                die($this->renderAlertError($message, 'Missing information'));
            }
        }
        return $control->value;
    }

    function getUser($user_id) {
        return new User($user_id);
    }

    function getCategoryByCode($type, $category_code) {
        $sql = "SELECT id FROM settings_categories WHERE type = '$type' AND code = '$category_code' AND status = 1 ORDER BY priority, name";
        if($rows = $this->query($sql)) return new Category($rows[0]->id);
        return new Category(0);
    }

    function getCategoryByColumn($type, $column_name, $column_value) {
        $sql = "SELECT id FROM settings_categories WHERE type = '$type' AND $column_name = '$column_value' ORDER BY priority, name";
        if($rows = $this->query($sql)) return new Category($rows[0]->id);
        return new Category(0);
    }

    function formatPhone($phone) {
        global $client;

        $phone      = trim(str_replace(' ', '', $phone));
        if(substr($phone, 0, 1) == '0') {
            $phone = $client->sms_phone_prefix.substr($phone, 1);
        }
        if(strlen($phone) > 10) {
            return '+'.str_replace('+', '', $phone);
        } else {
            return '';
        }
    }

    function sendSMS($phone, $message) {
        $sms    = new SMS();
        if($sms->send(array('recipient_phone'=>$phone, 'message_body'=>$message))) {
            return $this->renderAlertSuccess('SMS sent to '.$phone);
        } else {
            return $this->renderAlertWarning('Could not send SMS to '.$phone);
        }
    }

    function curl(array $options) {
        global                  $app;

        $fields                 = isset($options['fields']) ? $options['fields'] : array();
        $fields                 = is_array($fields) ? http_build_query($fields) : $fields;
        $method                 = isset($options['method']) ? $options['method'] : 'POST';
        $url                    = trim(str_replace (' ', '%20', $options['url']));

        $headers                = isset($options['headers']) ? $options['headers'] : array();
        $timeout                = isset($options['timeout']) ? $options['timeout'] : 300;
        $header                 = array();
        foreach(array_keys($headers) as $header_key) {
            $header[]           = $header_key.': '.$headers[$header_key];
        }

        $ch 					= curl_init($url);

        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $method);
        curl_setopt($ch, CURLOPT_POSTFIELDS,  $fields);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, $timeout); 
        curl_setopt($ch, CURLOPT_TIMEOUT, $timeout);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $header);

        $message 	            = curl_exec($ch);
        $status_code            = curl_getinfo($ch, CURLINFO_HTTP_CODE);

        if(curl_errno($ch)) {
            echo $app->renderAlertError(curl_error($ch), 'Curl Error');
        }

        $results                = array(
            'status_code'       => $status_code,
            'message'           => $message
        );

        return json_decode(json_encode($results));
    }

    function html2Json($data) {

        // Takes an array of data and returns a json
        ob_start();
		header('Content-type: application/json');
        echo json_encode($data);

    }

    function getModuleByCode($code) {
        $module_id   = $this->getColumnValue('settings_modules', 'id', 'code', $code);
        return new Module($module_id);
    }

    function getSubmoduleByCode($code) {
        $submodule_id   = $this->getColumnValue('settings_submodules', 'id', 'code', $code);
        return new Submodule($submodule_id);
    }

    function resizeImage($source_file, $destination_file, $new_width='', $new_height='', $quality='', $crop='') {
        $new_width      = strlen($new_width) ? $new_width : 800;
        $new_height     = strlen($new_height) ? $new_height : 600;
        $quality        = strlen($quality) ? $quality : 75;
        $crop           = strlen($crop) ? $crop : true;
        
		if(!$image_data = getimagesize($source_file)) return false;

		switch($image_data['mime']) { 
			case 'image/gif': 	$old_image = imagecreatefromgif($source_file); break;
			case 'image/jpeg'; 	$old_image = imagecreatefromjpeg($source_file); break;
			case 'image/png': 	$old_image = imagecreatefrompng($source_file); break;
			default:            $old_image = imagecreatefromjpeg($source_file);
		}
		
		$exif = @exif_read_data($source_file);
		if (!empty($exif['Orientation'])) {
			switch ($exif['Orientation']) {
				case 3: $old_image = imagerotate($old_image, 180, 0); break;
				case 6: $old_image = imagerotate($old_image, -90, 0); break;
				case 8: $old_image = imagerotate($old_image, 90, 0); break;
			}
		}

		$old_width 		= imagesx($old_image);
		$old_height 	= imagesy($old_image);
		$old_ratio 		= round($old_width / $old_height, 2);
		$new_ratio 		= round($new_width / $new_height, 2);
		$temp_ratio 	= $new_ratio;	
		$temp_width 	= $old_width;
		$temp_height 	= round($temp_width / $old_ratio);
		
		$crop_v 		= 0;
		$crop_h 		= 0;
		$src_x 			= 0;
		$src_y 			= 0;
		$dst_x 			= 0;
		$dst_y 			= 0;
		
		if($crop) {
			if($new_ratio > $old_ratio) {
				$temp_width 	= $old_width;
				$temp_height 	= round($temp_width / $temp_ratio);
				$crop_v 		= round(($old_height - $temp_height) / 2);
				$src_y 			= $crop_v;
			} elseif($old_ratio > $new_ratio) {
				$temp_height 	= $old_height;
				$temp_width 	= round($temp_height * $temp_ratio);
				$crop_h 		= round(($old_width - $temp_width) / 2);
				$src_x 			= $crop_h;
			}
		} else {
			$new_height 		= round($new_width / $old_ratio);
			if($new_width > $old_width) {
				$new_width 		= $temp_width;
				$new_height 	= $temp_height;
			}
		}
		
		$temp_image 			= imagecreatetruecolor($temp_width, $temp_height);
		$background_color 		= imagecolorallocate($temp_image, 255, 255, 255);
		imagefill($temp_image, 0, 0, $background_color);
		imagecopyresampled($temp_image, $old_image, $dst_x, $dst_y, $src_x, $src_y, $temp_width, $temp_height, $temp_width, $temp_height);
		
		$new_image 				= imagecreatetruecolor($new_width, $new_height);
		imagecopyresampled($new_image, $temp_image, 0, 0, 0, 0, $new_width, $new_height, $temp_width, $temp_height);
		imageinterlace($new_image, 1);

		if($destination_file == '') {
			ob_start();
			header( "Content-type: image/jpeg" );
			imagejpeg($new_image, null, $quality);
			$raw_data = ob_get_clean();
			return base64_encode($raw_data);
		}
		imagejpeg($new_image, $destination_file, $quality);
		imagedestroy($new_image);
		imagedestroy($old_image);
		return true;
	}


    public function renderPDF($html, $title='', $heading='') {
		
		global $root;
		$pdf = new TCPDF(PDF_PAGE_ORIENTATION, PDF_UNIT, PDF_PAGE_FORMAT, true, 'UTF-8', false);

        if(strlen($heading)) {
            $pdf->setPrintHeader(true);
            $pdf->setHeaderData('', 0, $heading);
        } else {
            $pdf->setPrintHeader(false);
        }

        $pdf->setPrintFooter(true);

		//$pdf->setFooterText(APP_NAME.' - '.$title);

		// set document information
		$pdf->SetCreator(PDF_CREATOR);
		$pdf->SetAuthor(APP_NAME.'');
		$pdf->SetTitle(APP_NAME.' - '.$title);
		$pdf->SetSubject(APP_NAME.' - '.$title);
		$pdf->SetKeywords(APP_NAME.' - '.$title);

		// set default header data
		//$pdf->SetHeaderData($shop_logo, $logo_width, $shop_name, $header_string, array(0,64,255), array(0,64,128));
		$pdf->setFooterData(array(0,64,0), array(0,64,128));

		// set header and footer fonts
		$pdf->setHeaderFont(Array(PDF_FONT_NAME_MAIN, '', PDF_FONT_SIZE_MAIN));
		$pdf->setFooterFont(Array(PDF_FONT_NAME_DATA, '', PDF_FONT_SIZE_DATA));

		// set default monospaced font
		$pdf->SetDefaultMonospacedFont(PDF_FONT_MONOSPACED);

		// set margins
		$pdf->SetMargins(16, 10, 14);
		$pdf->SetHeaderMargin(PDF_MARGIN_HEADER);
		$pdf->SetFooterMargin(PDF_MARGIN_FOOTER);

		// set auto page breaks
		$pdf->SetAutoPageBreak(TRUE, PDF_MARGIN_BOTTOM);

		// set image scale factor
		$pdf->setImageScale(PDF_IMAGE_SCALE_RATIO);

		// ---------------------------------------------------------

		// set default font subsetting mode
		$pdf->setFontSubsetting(true);

		// Set font
		// dejavusans is a UTF-8 Unicode font, if you only need to
		// print standard ASCII chars, you can use core fonts like
		// helvetica or times to reduce file size.
		$pdf->SetFont('helvetica', '', 10, '', true);

		// Add a page
		// This method has several options, check the source code documentation for more information.
		$pdf->AddPage();

		// set text shadow effect
		// $pdf->setTextShadow(array('enabled'=>false, 'depth_w'=>0.2, 'depth_h'=>0.2, 'color'=>array(196,196,196), 'opacity'=>1, 'blend_mode'=>'Normal'));

		// Set some content to print

		$style = array(
			'position' => '',
			'align' => 'R',
			'stretch' => false,
			'fitwidth' => false,
			'cellfitalign' => '',
			'border' => false,
			'hpadding' => '',
			'vpadding' => '',
			'fgcolor' => array(0,0,0),
			'bgcolor' => false, //array(255,255,255),
			'text' => true,
			'font' => 'helvetica',
			'fontsize' => 12,
			'stretchtext' => 0
		);
		
		$header = str_replace('__CLIENT__', CLIENT_ID, $this->uncleanText($com->document_header));
		$header = str_replace('__ROOT__', $r, $header);
		$header = str_replace('__CLIENT_PATH__', $r.CLIENT_PATH, $header);
		$header = str_replace('__CLIENT_LINK__', $r.CLIENT_LINK, $header);

		//$html = $header.'<div align="center" color="rgb(100,100,100)">'.strtoupper($title).'</div>'.$html;

		ob_end_clean();

		// Print text using writeHTMLCell()
		$pdf->writeHTMLCell(0, 0, '', '', $html, 0, 1, 0, true, '', true);

		// ---------------------------------------------------------

		// Close and output PDF document
		// This method has several options, check the source code documentation for more information.
		$pdf->Output($title.'.pdf', 'I');

		//============================================================+
		// END OF FILE
		//============================================================+
	}

    public function createPDF($html, $title='', $heading='') {
		
		global $root;
		$pdf = new TCPDF(PDF_PAGE_ORIENTATION, PDF_UNIT, PDF_PAGE_FORMAT, true, 'UTF-8', false);

        if(strlen($heading)) {
            $pdf->setPrintHeader(true);
            $pdf->setHeaderData('', 0, $heading);
        } else {
            $pdf->setPrintHeader(false);
            $pdf->setPrintFooter(false);
        }

		//$pdf->setFooterText(APP_NAME.' - '.$title);

		// set document information
		$pdf->SetCreator(PDF_CREATOR);
		$pdf->SetAuthor(APP_NAME.'');
		$pdf->SetTitle(APP_NAME.' - '.$title);
		$pdf->SetSubject(APP_NAME.' - '.$title);
		$pdf->SetKeywords(APP_NAME.' - '.$title);

		// set default header data
		//$pdf->SetHeaderData($shop_logo, $logo_width, $shop_name, $header_string, array(0,64,255), array(0,64,128));
		$pdf->setFooterData(array(0,64,0), array(0,64,128));

		// set header and footer fonts
		$pdf->setHeaderFont(Array(PDF_FONT_NAME_MAIN, '', PDF_FONT_SIZE_MAIN));
		$pdf->setFooterFont(Array(PDF_FONT_NAME_DATA, '', PDF_FONT_SIZE_DATA));

		// set default monospaced font
		$pdf->SetDefaultMonospacedFont(PDF_FONT_MONOSPACED);

		// set margins
		$pdf->SetMargins(16, 10, 14);
		$pdf->SetHeaderMargin(PDF_MARGIN_HEADER);
		$pdf->SetFooterMargin(PDF_MARGIN_FOOTER);

		// set auto page breaks
		$pdf->SetAutoPageBreak(TRUE, PDF_MARGIN_BOTTOM);

		// set image scale factor
		$pdf->setImageScale(PDF_IMAGE_SCALE_RATIO);

		// ---------------------------------------------------------

		// set default font subsetting mode
		$pdf->setFontSubsetting(true);

		// Set font
		// dejavusans is a UTF-8 Unicode font, if you only need to
		// print standard ASCII chars, you can use core fonts like
		// helvetica or times to reduce file size.
		$pdf->SetFont('helvetica', '', 10, '', true);

		// Add a page
		// This method has several options, check the source code documentation for more information.
		$pdf->AddPage();

		// set text shadow effect
		// $pdf->setTextShadow(array('enabled'=>false, 'depth_w'=>0.2, 'depth_h'=>0.2, 'color'=>array(196,196,196), 'opacity'=>1, 'blend_mode'=>'Normal'));

		// Set some content to print

		$style = array(
			'position' => '',
			'align' => 'R',
			'stretch' => false,
			'fitwidth' => false,
			'cellfitalign' => '',
			'border' => false,
			'hpadding' => '',
			'vpadding' => '',
			'fgcolor' => array(0,0,0),
			'bgcolor' => false, //array(255,255,255),
			'text' => true,
			'font' => 'helvetica',
			'fontsize' => 12,
			'stretchtext' => 0
		);
		
		$header = str_replace('__CLIENT__', CLIENT_ID, $this->uncleanText($com->document_header));
		$header = str_replace('__ROOT__', $r, $header);
		$header = str_replace('__CLIENT_PATH__', $r.CLIENT_PATH, $header);
		$header = str_replace('__CLIENT_LINK__', $r.CLIENT_LINK, $header);

		//$html = $header.'<div align="center" color="rgb(100,100,100)">'.strtoupper($title).'</div>'.$html;

		ob_end_clean();

		// Print text using writeHTMLCell()
		$pdf->writeHTMLCell(0, 0, '', '', $html, 0, 1, 0, true, '', true);

		// ---------------------------------------------------------

		// Close and output PDF document
		// This method has several options, check the source code documentation for more information.
		$pdf->Output($title.'.pdf', 'F');

		//============================================================+
		// END OF FILE
		//============================================================+
	}

    public function age($date1, $date2='', $short=0) {

        if(!$date1) $date1 = date('Y-m-d H:i:s');
        if($date1 < '1900') $date1 = date('Y-m-d');
        if($date1 > date('Y-m-d H:i:s')) $date1 = date('Y-m-d H:i:s');

		$diff= date_diff(date_create($date1), date_create($date2));

		$str = '0 sec';
		if($diff->y) {
			$str = $diff->y > 1 ? $diff->y.' years' : $diff->y.' year';
		} elseif($diff->m) {
			$str = $diff->m > 1 ? $diff->m.' months' : $diff->m.' month';
		} elseif($diff->d) {
			$str = $diff->d > 1 ? $diff->d.' days' : $diff->d.' day';
		} elseif($diff->h) {
			$str = $diff->h > 1 ? $diff->h.' hours' : $diff->h.' hours';
		} elseif($diff->i) {
			$str = $diff->i > 1 ? $diff->i.' mins' : $diff->i.' min';
		} elseif($diff->s) {
			$str = $diff->s > 1 ? $diff->s.' secs' : $diff->s.' sec';
		}
		
		if($short) {
			$str = explode(' ', $str);
			return $str[0].''.substr($str[1],0,1).'.';
		}
		
		return $str;
	}

    function datetime($datetime) {
        return $datetime > '1900' ? substr($datetime, 0, 16) : '';
    }

    function date($datetime) {
        return $datetime > '1900' ? substr($datetime, 0, 10) : '';
    }

    function time($datetime) {
        return $datetime > '1900' ? substr($datetime, 11, 5) : '';
    }

    function filesize($filesize) {
        if($filesize < 1024) return $filesize.' B';
        if($filesize < 1024000) return round($filesize / 1024).' KB';
        if($filesize < 1024000000) return round($filesize / 1024000).' MB';
        return round($filesize / 1024000000).' GB';
    }

    function joinItems($items, $join='and', $short=0) {
		$last_item = array_pop($items);
		if($short && count($items)) return $items[0].' +'.count($items).' more';
		if(count($items)) return implode(', ', $items).' '.$join.' '.$last_item;
		return $last_item;
	}

    function getAttachmentBySubSection($subsection, $subsection_id) {
        $attachment_id = $this->getColumnValue('content_attachments', 'id', 'subsection_id', $subsection_id, "subsection = '$subsection' ");
        return new Attachment($attachment_id);
    }

    function getAttachmentBySection($section, $section_id, $subsection, $subsection_id='') {
        $condition                               = "";
        if(strlen($subsection)) $condition      .= " AND subsection = '$subsection' ";
        if(strlen($subsection_id)) $condition   .= " AND subsection_id = $subsection_id ";
        
        $attachment_id = $this->getColumnValue('content_attachments', 'id', 'section', $section, " section_id = '$section_id' $condition ");
        return new Attachment($attachment_id);
    }

    function getAttachmentByCode($section, $section_id, $code) {
        $attachment_id = $this->getColumnValue('content_attachments', 'id', 'section_id', $section_id, "code = '$code' AND section = '$section' ");
        return new Attachment($attachment_id);
    }

    function renderAttachmentsPreviewEdit($options = array()) {
        $attachment = new Attachment(0);

        return $attachment->renderAttachmentsPreviewEdit($options);
    }

    function uploadAttachments($options = array()) {
        $attachment = new Attachment(0);

        return $attachment->uploadAttachments($options);
    }

    function stringToObject($string, $outer_delimeter=';', $inner_delimeter=':') {
        $obj        = array();
        $string     = trim(preg_replace('/\s+/', '', $this->uncleanText($string)));
        $strings    = explode($outer_delimeter, $string);
        foreach($strings as $str) {
            $parts  = explode($inner_delimeter, $str);
            $obj[$parts[0]] = $parts[1];
        }
        return (object) $obj;
    }

    function shortname($name, $length=20) {
        return strlen($name) > $length ? (substr($name, 0, $length).'..') : $name;
    }

    function padNumber($number, $digits=4) {
		if(is_numeric($number)) return sprintf('%0'.$digits.'d', $number);
		return $number;
	}

    function listPeople($person_type, $except=array()) {
        $table_category = $this->getCategoryByCode('PersonType', $person_type);
        $table          = $table_category->f1;
        $sql            = "SELECT $table.id value,
		personnel_persons.name 
		FROM $table 
		JOIN personnel_persons ON personnel_persons.id = $table.person_id 
		WHERE $table.status = 1 ";
		return $this->query($sql);
    }

    function listLocations($location_type, $except=array()) {
        $table_category = $this->getCategoryByCode('LocationType', $location_type);
        $table          = $table_category->f1;
        $sql            = "SELECT $table.id value,
		location_locations.name 
		FROM $table 
		JOIN location_locations ON location_locations.id = $table.location_id 
		WHERE $table.status = 1 ";
		return $this->query($sql);
    }

    function renderSignatureInput($obj) {
        global $app;
        
        $html = '<div class="row">
        <div class="col-lg-6 my-3">
        <div class="text-primary rounded mb-3" id="signature" style="height:173px; border:4px dashed rgb(var(--bs-danger-rgb))"></div> 
        </div><div class="col-lg-6 my-3"> 
        <div class="mb-3 rounded-3" style="width:100%; border:4px dashed rgb(var(--bs-success-rgb)); width:100%;"> 
        <img src="'.$obj->signature().'" id="signature_preview" style="width:100%; height:165px" /> 
        </div><div class="row"><div class="col-6">
        <input type="checkbox" class="btn-check" id="signature_changed" autocomplete="off" />
        <label class="btn btn-outline-primary w-100" for="signature_changed"> <span class="fa-light fa-check-circle fa-fw"></span> Accept </label>
        </div><div class="col-6">
        <a id="clear_button" class="btn btn-outline-danger margin-v-15 w-100"><span class="fa-light fa-trash-alt fa-fw"></span> Clear </a>
        </div></div>
        </div></div>
        
        '.$app->renderInputHidden('signature_data', '');

        return $html;
    }

    function rgb($color_name) {
        $color = $this->getCategoryByCode('ColorType', $color_name);
        return $color->f4;
    }

    function getSystemSettings($variable, $default=0) {
        global $client;
        return isset($client->$variable) ? $client->$variable : $default;
    }

    function getIdByCode($table, $code) {
        return $this->getColumnValue($table, 'id', 'code', $code);
    }

    function countItems($items, $text='item', $texts='') { //Returns item for one item or items for many items
		if(is_array($items)) $items = count($items);
		if(!strlen($texts)) $texts = $text.'s';
		if($items > 1) return $texts;
		return $text;
	}

    function h2($title, $icon='') {
        $html = '<h2 class="fw-bold mt-3 mb-4"> ';
        if($icon) $html .= '<small><span class="'.$icon.' fa-fw"></span></small> '; 
        $html .= $title.' </h2>';
        return $html;
    } 

    function h3($title, $icon='') {
        $html = '<h3 class="fw-bold mt-3 mb-4"> ';
        if($icon) $html .= '<small><span class="'.$icon.' fa-fw"></span></small> '; 
        $html .= $title.' </h3>';
        return $html;
    } 

    function h4($title, $icon='') {
        $html = '<h4 class="fw-bold mt-3 mb-4"> ';
        if($icon) $html .= '<small><span class="'.$icon.' fa-fw"></span></small> '; 
        $html .= $title.' </h4>';
        return $html;
    }

    function center($html, $css='') {
        return '<div class="text-center '.$css.'">'.$html.'</div>';
    }

    function ajaxLoad($container, $url, $delay = 0) {
        return '<script> 
        setTimeout(function() {app.ajaxLoad(\''.$container.'\', \''.$url.'\')}, '.$delay.'); 
        </script>';
    }

    function arrayToXml($array, &$xml) {  
        if(!$xml) return;       
		foreach($array as $key => $value) {               
			if(is_array($value)) {            
				if(!is_numeric($key)){
					$subnode = $xml->addChild($key);
					$this->arrayToXml($value, $subnode);
				} else {
					$this->arrayToXml($value, $subnode);
				}
			} else {
				$xml->addChild($key, $value);
			}
		}        
	}

    function array2xml(string $tag, array $data) {
        $xml    = new SimpleXMLElement('<?xml version="1.0" encoding="UTF-8" standalone="yes"?><'.$tag.'></'.$tag.'>', 1);
        $this->arrayToXml($data, $xml);
        $xml    = trim(str_replace(array("\r", "\n"), '', $xml->asXML()));
        return $xml;
    }

    function getDefaultSettings($setting_name, $default_value) {
        global $client;
        return isset($client->$setting_name) ? $client->$setting_name : $default_value;
    }

    function getTaggedRecord($table, $tag) {
        $sql = "SELECT * FROM $table WHETE tags LIKE '%$tag%' ";
        foreach($this->read($sql) as $row) {
            return $row;
        }
        return false;
    }

    public function monday() {
		if(date('D') == 'Mon') return date('Y-m-d', strtotime('Monday'));
		return date('Y-m-d', strtotime('Monday - 1 week'));
	}
	
	public function friday() {
		return date('Y-m-d', strtotime($this->monday().' + 1 week'));
	}
	
	public function morning($date='') {
		if(!strlen($date)) $date = date('Y-m-d');
		return date('Y-m-d', strtotime($date)).' 00:00:00';
	}
	
	public function evening($date='') {
		if(!strlen($date)) $date = date('Y-m-d');
		return date('Y-m-d', strtotime($date)).' 23:59:59';
	}
	
	public function monthStart($date='') {
		if(!strlen($date)) $date = date('Y-m-d');
		return date('Y-m-', strtotime($date)).'01';
	}
	
	public function monthEnd($date='', $day=false) {
		if(!strlen($date)) $date = date('Y-m-d');
		$timestamp = strtotime($date);
		$d = cal_days_in_month(CAL_GREGORIAN, date('m', $timestamp), date('Y', $timestamp));
		if($day) return $d;
		return date('Y-m-', strtotime($date)).$d;
	}

    function renderDocumentHeader() {
        global $root, $client;

        $html               = '<html lang="en">
        <body>
        <style>'.file_get_contents($root.'assets/styles/tcpdf.css').'</style>';

        $default_header     = '<table style="width:100%"><tr>
        <td style="width:8%"><img src="'.$root.CLIENT_PATH.'images/logo.png" /> </td>
        <td style="width:92%; text-align:left"><h3><span style="font-size:1.5em">'.$client->client_name.'</span> <br />'.$client->client_address.'</h3></td>
        </tr></table>';
        $document_header    = $this->getSystemSettings('client_document_header', $default_header);
        $document_header    = $this->uncleanText($document_header);
        $document_header    = str_replace('__ROOT__', $root, $document_header);
        $document_header    = str_replace('__CLIENT_PATH__', CLIENT_PATH, $document_header);
    
        return $html.$document_header.'';
    }

    function renderDocumentFooter() {
        $html   = '</body>
        </html>';
        return $html;
    }

    function renderReportNavigation($data=array()) {

        $period			= isset($data['period'])            ? $data['period']           : 'Month';
        $section_title 	= isset($data['section_title'])     ? $data['section_title']    : '';
        $section_id 	= isset($data['section_id'])        ? $data['section_id']       : str_replace(' ', '', $section_title);
        $current_date	= isset($data['current_date'])      ? $data['current_date']     : date('Y-m-d');
        $prev_date	    = isset($data['prev_date'])         ? $data['prev_date']        : date('Y-m-d', strtotime($current_date.' - 1 '.$period));
        $next_date	    = isset($data['next_date'])         ? $data['next_date']        : date('Y-m-d', strtotime($current_date.' + 1 '.$period));
        $current_period = isset($data['current_period'])    ? $data['current_period']   : 'Current period';
        $css            = isset($data['css'])               ? $data['css']              : '';

        $prev_start_date    = $prev_date;
        $prev_end_date	    = date('Y-m-d', strtotime($prev_date.' + 1 '.strtolower($period).' - 1 day'));
        $next_start_date    = $next_date;
        $next_end_date	    = date('Y-m-d', strtotime($next_date.' + 1 '.strtolower($period).' - 1 day'));
        
        $html = '<div id="'.$section_id.'" class="'.$css.'"> 

        <div class="row">

        <div class="col-12 col-lg-6 col-xl-4">
        <h3 class="fw-semibold">'.$section_title.'</h3>
        </div>
        
        <div class="col-3 col-lg-1 col-xl-1 mt-1 mb-2">
        <a class="btn btn-outline-primary w-100" title="Prev. &nbsp;'.$period.'" onclick="app.loadReport(\''.$prev_start_date.'\', \''.$prev_end_date.'\', \''.$section_id.'\')"> <span class="fa fa-angle-left"></span> <span class="d-lg-none">Prev</span> </a>
        </div>
        
        <div class="col-6 col-lg-4 col-xl-2 text-center mt-1 mb-2 pt-1">
        <h3 class="fw-semibold">'.$current_period.'</h3>
        </div>
        
        <div class="col-3 col-lg-1 col-xl-1 text-right mt-1 mb-2">
        <a class="btn btn-outline-primary w-100" title="Next &nbsp;'.$period.'" onclick="app.loadReport(\''.$next_start_date.'\', \''.$next_end_date.'\', \''.$section_id.'\')">  <span class="d-lg-none">Next</span> <span class="fa fa-angle-right"></span>  </a>
        </div>
        
        </div>
        
        </div>';

        return $html;
    }

    function json2num($text) {
		return str_replace('"', '', str_replace('null','0',(str_replace('.00','',($text)))));
	}

    function renderChart($report, $title, $chart_type='bar', $color='blue') {
		global $rgb;
		$keys = array_keys($report);
		if(!count($keys)) return;
		$keys_json = json_encode($keys);
		$first_key = $keys[0];
		$data = array();
		
		if(is_array($report[$first_key])) {
			if($chart_type == 'pie' || $chart_type == 'doughnut') {
				$keys = array_keys($report[$first_key]);
				$values = array();
				$colors = array();
				foreach($keys as $key) {
					array_push($values, $report[$first_key][$key]['value']);
					array_push($colors, 'rgba('.$report[$first_key][$key]['color'].', 1)');
				}
				$config = 'var config = {
					type: "'.$chart_type.'",
					data: {
						labels: '.json_encode($keys).',
						datasets: [{
							label: "Revenue by Payment",
							backgroundColor: '.json_encode($colors).',
							data: '.$this->main->json2num(json_encode(array_values($values))).'
						}]
					},
					options: {
						responsive: true,
						tooltips: {
							mode: "index",
							intersect: false,
						},
						hover: {
							mode: "nearest",
							intersect: true
						}
					}
				};';
			
			} else {
				$labels = array_keys($report[$first_key]);
				foreach($labels as $label) {
					$val = array();
					foreach($keys as $my_key) array_push($val, round($report[$my_key][$label]['value']));
					
					$my_data = '{
						label: "'.$label.'",
						backgroundColor: "rgba('.$report[$first_key][$label]['color'].', 0.9)",
						borderColor: "rgba('.$report[$first_key][$label]['color'].', 0.1)",
						borderWidth: 0,
                        borderRadius: 4,
                        borderSkipped: false,
						data: '.$this->json2num(json_encode(array_values($val))).',
						fill: false 
					}';
					
					if($chart_type == 'line') {
						$my_data = '{
							label: "'.$label.'",
							backgroundColor: "rgba('.$report[$first_key][$label]['color'].', 0.9)",
							borderColor: "rgba('.$report[$first_key][$label]['color'].', 0.9)",
							borderWidth: 3,
							data: '.$this->json2num(json_encode(array_values($val))).',
							fill: false 
						}';
					}
					array_push($data, $my_data);
				}
				$config = 'var config = {
					type: "'.$chart_type.'",
					data: {
						labels: '.json_encode($keys).',
						datasets: [ '.implode(',',$data).']
					},
					options: {
						responsive: true,
						tooltips: {
							mode: "index",
							intersect: false
						},
						hover: {
							mode: "nearest",
							intersect: true
						},
						
					}
				};';
			}
		} else {
			$config = 'var config = {
				type: "'.$chart_type.'",
				data: {
					labels: '.json_encode($keys).',
					datasets: [{
						label: "'.ucfirst(str_replace('_', ' ', $title)).'",
						backgroundColor: "rgba('.$rgb->$color.', 1)",
						borderColor: "rgba('.$rgb->$color.', 1)",
						borderWidth: 1,
						data: '.$this->json2num(json_encode(array_values($report))).',
						fill: false
					}]
				},
				options: {
					responsive: true,
					tooltips: {
						mode: "index",
						intersect: false,
					},
					hover: {
						mode: "nearest",
						intersect: true
					},
					scales: {
						yAxes: [{
							ticks: {
								beginAtZero: true
							}
						}],
						xAxes: [{
							ticks: {
								beginAtZero: true
							}
						}]
					}
				}
			};';
		}
		
		echo '<script> '.$config.' window.'.$title.' = new Chart($("#'.$title.'"), config); </script>';
	}

    function getWalletStore() {
        $sql    = "SELECT location_stores.id FROM location_stores JOIN location_locations ON location_locations.id = location_stores.location_id WHERE location_locations.tags LIKE '%wallet%'";
        foreach($this->read($sql) as $row) {
            return new Store($row->id);
        }
        die($this->renderAlertError('Wallet store not found. Create wallet store with <b>wallet</b> tag then try again'));
    }

    function getWalletTopupItem() {
        $sql    = "SELECT id FROM inventory_items WHERE tags LIKE '%wallet-topup%'";
        foreach($this->read($sql) as $row) {
            return new Item($row->id);
        }
        die($this->renderAlertError('Wallet topup item not found. Create wallet topup item with <b>wallet-topup</b> tag then try again'));
    }

    function getWalletWithdrawItem() {
        $sql    = "SELECT id FROM inventory_items WHERE tags LIKE '%wallet-withdraw%'";
        foreach($this->read($sql) as $row) {
            return new Item($row->id);
        }
        die($this->renderAlertError('Wallet withdraw item not found. Create wallet withdraw item with <b>wallet-withdraw</b> tag then try again'));
    }

    function getCashScheme() {
        $scheme_id = $this->getColumnValue('billing_insurance_schemes', 'id', 'code', 'CASH');
        return new Scheme($scheme_id);
    }

    function getSchemeByCode($code) {
        $scheme_id = $this->getColumnValue('billing_insurance_schemes', 'id', 'code', $code);
        return new Scheme($scheme_id);
    }

    function getPricelistByCode($code) {
        $pricelist_id = $this->getColumnValue('billing_pricelists', 'id', 'code', $code);
        return new Pricelist($pricelist_id);
    }

    function getAge($start_date, $end_date='', $format='') {
		if(!strlen($end_date) < 10) $end_date = date('Y-m-d H:i:s');
		$age       = date_diff(date_create($start_date), date_create($end_date));
		$str        = '';
		if($age->y) {
			$str = $age->y > 1 ? $age->y.' years' : $age->y.' year';
		} elseif($age->m) {
			$str = $age->m > 1 ? $age->m.' months' : $age->m.' month';
		} elseif($age->d) {
			$str = $age->d > 1 ? $age->d.' days' : $age->d.' day';
		} elseif($age->h) {
			$str = $age->h > 1 ? $age->h.' hours' : $age->h.' hour';
		} elseif($age->i) {
			$str = $age->i > 1 ? $age->i.' mins' : $age->i.' min';
		} elseif($age->s) {
			$str = $age->s > 1 ? $age->s.' secs' : $age->s.' sec';
		}
		
		if($format == 'FULL') {
			$str    = explode(' ', $str);
			$str    = $str[0].''.substr($str[1],0,1);
		}

        if($format == 'DAYS') {
			return $age->d;
		}

        if($format == 'YEARS_MONTHS_DAYS') {
			$str    = '';
			$str    .= $age->y ? $age->y.'y ' : '';
            $str    .= $age->m ? $age->m.'m ' : '';
            $str    .= $age->d ? $age->d.'d ' : '';
            return $str;
		}
		
		return $str;
	}

    function formatCode($code, $digits=6) {
		return sprintf('%0'.$digits.'d', $code);
	}

    function textFromParameters($inputs) {

		$notes 	= '';
		$count	= 0;
		foreach($inputs as $note) {
			if(isset($note['parameters'])) {
				$parameters 	= json_decode($note['parameters']);
				$value			= $this->cleanText($note['value']);
				$title			= $this->cleanText($parameters->title);
				$double_break 	= $count++ ? '<br />' : '';
		
				if($parameters->type == 'textarea' && strlen($value)) {
					if(strlen($title)) $notes .= $double_break.'<b>'.$title.'</b><br />';
					$notes .= $value.'<br />';
				} elseif($parameters->type == 'signature') {
					$signature_data = $note['value'];
					$notes .= $double_break.'<img src="data: '.$signature_data.'" style="width:200px" /> ';
				} elseif(strlen($value)) {
					if(strlen($title)) $notes .= '<b>'.$title.'</b> ';
					$notes .= $value.'&nbsp; &nbsp;';
				} elseif($parameters->type == 'title') {
					$notes .= $double_break.'<b>'.$title.'</b><br />';
				} 
			} elseif(is_array($note)) {

				//$note = $this->reArrangeDataArray($note);
				$parent_mentioned = 0;
				
				foreach($note as $subnote) {	
					$value		= $this->cleanText($subnote->value);
					$title		= $this->cleanText($subnote->title);
					$parent		= $this->cleanText($subnote->parent);
		
					if(isset($first_child[$parent])) {
						if($title == $first_child[$parent]) $notes .= '<br /><br /><b>'.str_replace('_', ' ', $parent).'</b><br />';
					} else {
						$first_child[$parent] = $title;
					}
		
					if(!$parent_mentioned++) $notes .= '<br /><br /><b>'.str_replace('_', ' ', $parent).'</b><br />';
		
					if($subnote->type == 'textarea' && strlen($value)) {
						if(strlen($title)) $notes .= '<br /><br /><b>'.$title.'</b><br />';
						$notes .= $value.'<br />';
					} elseif(strlen($value)) {
						if(strlen($title)) $notes .= '<b>'.$title.'</b>: ';
						$notes .= $value.'&nbsp; &nbsp;';
					} elseif($subnote->type == 'title') {
						$notes .= '<br /><br /><b>'.$title.'</b><br />';
					}
				}
			}
		}
		return $notes;
	}

    function encodeImage($image_path) {
		$image_type 	= pathinfo($image_path, PATHINFO_EXTENSION);
		$image_data 	= file_get_contents($image_path);
		$base64_image   = 'data:image/'.$image_type.';base64,'.base64_encode($image_data);
		return $base64_image;
	}

    function zipDirectory($source, $destination, $excluded=0) {
	
        $slash = '/';
    
        if(!$excluded) $excluded = array();
        if (!file_exists($source)) {
            echo "source doesn't exist";
            return false;
        } 
         
        if (!extension_loaded('zip')) {
            echo "zip extension not loaded in php";
            return false;
        }
      
        $zip = new ZipArchive();
        if (!$zip->open($destination, ZIPARCHIVE::CREATE)) {
            echo "failed to create zip file on destination";
            return false;
        }
      
        $source = str_replace('\\', $slash, realpath($source));
      
        if (is_dir($source) === true) {
            $files = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($source), RecursiveIteratorIterator::SELF_FIRST);
            foreach ($files as $file) {
                
                foreach($excluded as $exclude) {
                    if(strpos($file, $exclude) !== false) continue 2; 
                }
            
                $file = str_replace('\\', $slash, $file);
      
                // Ignore "." and ".." folders
                if( in_array(substr($file, strrpos($file, $slash)+1), array('.', '..')) ) continue;
                
                //$file = realpath($file);
      
                if (is_dir($file) === true) {
                    $zip->addEmptyDir(str_replace($source . $slash, '', $file . $slash));
                } else if (is_file($file) === true) {
                    $zip->addFromString(str_replace($source . $slash, '', $file), file_get_contents($file));
                }
            }
        } else if (is_file($source) === true) {
            $zip->addFromString(basename($source), file_get_contents($source));
        }
        
        return $zip->close();
    }
    
    function unzipDirectory($source, $destination) {
        $zip = new ZipArchive;
        if($zip->open($source) === true) {
            $zip->extractTo($destination);
            return $zip->close();
        }
    }
    
    function removeDirectory($dir) { 
        if (is_dir($dir)) { 
            $objects = scandir($dir);
            foreach ($objects as $object) { 
                if ($object != "." && $object != "..") { 
                    if (is_dir($dir. DIRECTORY_SEPARATOR .$object) && !is_link($dir."/".$object))
                    $this->removeDirectory($dir. DIRECTORY_SEPARATOR .$object);
                    else
                    unlink($dir. DIRECTORY_SEPARATOR .$object); 
                } 
            }
            rmdir($dir); 
        } 
    }
    
    function copyDirectory(
        string $sourceDirectory,
        string $destinationDirectory,
        string $childFolder = ''
    ): void {
        $directory = opendir($sourceDirectory);
    
        if (is_dir($destinationDirectory) === false) {
            mkdir($destinationDirectory);
        }
    
        if ($childFolder !== '') {
            if (is_dir("$destinationDirectory/$childFolder") === false) {
                mkdir("$destinationDirectory/$childFolder");
            }
    
            while (($file = readdir($directory)) !== false) {
                if ($file === '.' || $file === '..') {
                    continue;
                }
    
                if (is_dir("$sourceDirectory/$file") === true) {
                    $this->copyDirectory("$sourceDirectory/$file", "$destinationDirectory/$childFolder/$file");
                } else {
                    copy("$sourceDirectory/$file", "$destinationDirectory/$childFolder/$file");
                }
            }
    
            closedir($directory);
    
            return;
        }
    
        while (($file = readdir($directory)) !== false) {
            if ($file === '.' || $file === '..') {
                continue;
            }
    
            if (is_dir("$sourceDirectory/$file") === true) {
                $this->copyDirectory("$sourceDirectory/$file", "$destinationDirectory/$file");
            }
            else {
                copy("$sourceDirectory/$file", "$destinationDirectory/$file");
            }
        }
    
        closedir($directory);
    }

    function writeSQLFile($table, $values) {
        global $root;

        $path 				    = $root.CLIENT_PATH.'temp/'.$table.'.sql';
		$values					= implode("','", $values);
		$values  				= "'$values'";

		if(file_exists($path)) {
			$sql 				= "($values),".PHP_EOL;
		} else {
			$sql 				= "INSERT INTO $table VALUES ($values),".PHP_EOL;
		}

		file_put_contents($path, $sql, FILE_APPEND | LOCK_EX);
		return 1;
    }

    function extractStringBetweenStrings($string, $start, $end){
		$string 	= ' ' . $string;
		$ini 		= strpos($string, $start);
		if ($ini == 0) return '';
		$ini 		+= strlen($start);
		$len 		= strpos($string, $end, $ini) - $ini;
		return substr($string, $ini, $len);
	}

    function getMaxColumnValue($table_name, $column_name) {
        $sql        = "SELECT MAX($column_name) max_value FROM $table_name";
        foreach($this->read($sql) as $row) {
            if($row->max_value) return $row->max_value;
            return 0;
        }
        return 0;
    }

    function reArrangeFilesArray($files){
		$output = array();
		foreach ($files as $attrName => $valuesArray) {
			foreach ($valuesArray as $key => $value) {
				$output[$key][$attrName] = $value;
			}
		}
		return $output;
	}

    function getClinicalEditTime() {
        return $this->getSystemSettings('clinical_edit_time', '1 hour');
    }

    function root($root) {
        return substr_count($root, '../');
    }

    function unroot($root_number) {
        $root = '';
        for($count = 0; $count < $root_number; $count++) {
            $root .= '../';
        }
        return $root;
    }
}

$app = new App();

?>