<?php

#[AllowDynamicProperties]

class Crud {
    function __construct($id=0) {
		global $db, $app;
		
		// Read table fields into this object attributes

		if(isset($this->table)) {
			$sql				= "DESCRIBE $this->table";
			$results			= $db->read($sql);

			foreach($results as $result) {
				$field 			= $result->Field;
				if(strpos($result->Type, 'int') !== false) {
					$this->$field 	= 0;
				} elseif(strpos($result->Type, 'decimal') !== false) {
					$this->$field 	= 0.00;
				} else {
					$this->$field 	= '';
				}
			}
		}
		$this->ignore_errors	= '';
		$this->id 				= $id;
		$this->object_name 		= 'Record';
		$this->status 			= 1;
		if($id) $this->read();
	}

	function create($values, $table='') {

		// Inserts a record into this.table and returns last inserted id

		global $db;
		$table		= strlen($table) ? $table : $this->table;
		$vals		= array();
		foreach($values as $value) {
			//$value = $db->real_escape_string($value);
			array_push($vals, "'$value'");
		}
		$values		= implode(',', $vals);
		$sql 		= "INSERT $this->ignore_errors INTO $table VALUES ($values) "; 
		$created	= $db->query($sql);
		return $created->lastInsertId();
	}

	function created($data) {
		global $app;
		if($this->create($data)) {
			return $app->renderAlertSuccess($this->object_name.' created successfully', 'Created', 'modal_lg', true);
		}
	}

	function read($sql='') {	

		// Reads all fields in this.table where primary_key = this.id

		global $db;

		if(!strlen($sql)) {
			$sql 				= "SELECT * FROM $this->table WHERE $this->primary_key = '$this->id' "; 
			if(isset($this->parent_table)) {
				$table 			= $this->table;
				$primary_key 	= $this->primary_key;
				$parent_table 	= $this->parent_table;
				$parent_key 	= $this->parent_key;
				$secondary_key	= $this->secondary_key;

				$sql 			= "SELECT * FROM $table 
				JOIN $parent_table ON $parent_table.$parent_key = $table.$parent_key
				WHERE $primary_key = $this->id "; 
			}
		}
 
		$rows	= $db->read($sql);
		foreach ($rows as $row) {
			foreach ($row as $index => $value) {
				$this->$index = $value;
			}
		}
	}

	function update($data, $table='', $primary_key='', $key_value='') {
		
		// Updates $table with information received in $data where $this->primary_key = $this->id

		global $db, $app;
		$table			= strlen($table) ? $table : $this->table;
		$primary_key	= strlen($primary_key) ? $primary_key : $this->primary_key;
		$key_value		= strlen($key_value) ? $key_value : $this->$primary_key;

		$parameters 	= array();
		$columns 		= array_keys($data);

		// Get columns definition from table to be updated

		$fields 		= array();
		$columns_sql	= "DESCRIBE $table";
		$rows			= $db->read($columns_sql);
		
		foreach($rows as $row) {
			$fields[$row->Field] = $row->Type;
		}

		foreach($columns as $column) {

			// Decide of column value needs quotes or not
			
			if(isset($fields[$column])) {
				$value			= $data[$column];
				$field_types 	= explode('(', $fields[$column]);
				$field_type 	= $field_types[0];
				$non_texts 		= array('int', 'decimal');

				if($field_type == 'json' && 0) {
					$value		= "JSON_SET(`$column`, '$.*', $value)";
				} elseif(!in_array($field_type, $non_texts)) {
					$value		= "'".$value."'";
				}
			} else {
				$error =  array (
					'title'     => 'DATABASE ERROR',
					'message'   => 'Field '.$column.' does not exist in table '.$table,
					'class'     => 'danger',
					'icon'      => 'far fa-exclamation-triangle'
				);
				die($app->renderAlert($error));
			}
			array_push($parameters, "$column = $value");
		}

		$parameters 	= implode(', ', $parameters);
		$sql 			= "UPDATE $this->ignore_errors $table SET $parameters WHERE $primary_key = '$key_value' ";
		$query 			= $db->update($sql);

		return $query->rowCount();
	}

	function updated(array $data) {
		global $app;
		if($this->update($data)) {
			return $app->renderAlertSuccess($this->object_name.' updated successfully', 'Updated', 'modal_lg', true);
		}
	}

	function delete(array $data=array()) {
		if($this->tagged('nodelete')) {
			global $app;
			die($app->renderAlertError('Please remove <b>nodelete</b> tag first'));
		}
		$data = count($data) ? $data : array (
			'modified_user_id'  => 0,
			'modified_datetime' => 0,
			'status'            => 0
		);
		// return $this->destroy($data);
		return $this->update($data);
	}

	function deleted() {
		global $app;
		if($this->delete()) {
			return $app->renderAlertSuccess($this->object_name.' deleted successfully', 'Deleted', 'modal_md', true);
		}
	}

	function renderDeleteConfirmation($submodule) {
		global $user;
		$user->checkSubmoduleAccess($submodule, 'delete');
    	return $this->deleted();
	}

	function destroy() {
		global $db;
		$primary_key	= $this->primary_key;
		$key_value		= $this->$primary_key;

		$sql 			= "DELETE FROM $this->table WHERE $primary_key = $key_value";
		$query 			= $db->update($sql);
		return $query->rowCount();
	}

	function renderModalDelete($id, $options=array()) {
		global $app;
		$modal_options  = array (
			'name'     		=> '',
			'shortname'		=> $this->shortname,
			'title'    		=> '<span class="fa solid fa-trash-alt fa-fw"></span> Confirm action',
			'icon'     		=> '',
			'color'    		=> 'danger',
			'details'  		=> '',
			'form_id'  		=> 'ajax_form_delete_'.$this->link,
			'form_action'   => isset($options['form_action']) ? $options['form_action'] : 'process.php',
			'type'			=> 'confirm_action',
			'action_label'	=> '<span class="fa solid fa-trash-alt fa-fw"></span> Delete'
		);
	
		$html = $app->renderModalOpen($modal_options);
		$html .= $app->renderFormOpen($modal_options);
		$html .= $app->renderInputHidden('id', $id);
		$html .= $app->renderInputHidden('action', 'delete');
		$html .= '<h4>Do you want to delete this '.strtolower($this->shortname).'?</h4>';

		$html .= $app->renderFormClose($modal_options);
		$html .= $app->renderModalClose($modal_options);

		return $html;
	}

	function renderModalFormOpen(array $options=array()) {
		global $app;
		$modal_options					= $options;
		$modal_options['name']  		= isset($modal_options['name']) ? $modal_options['name'] : $this->name;
		$modal_options['shortname']  	= isset($modal_options['shortname']) ? $modal_options['shortname'] : $this->shortname;
		$modal_options['icon']  		= isset($modal_options['icon']) ? $modal_options['icon'] : $this->icon;
		$modal_options['color']  		= isset($modal_options['color']) ? $modal_options['color'] : 'primary';
		$modal_options['details']  		= isset($modal_options['details']) ? $modal_options['details'] : $this->details;
		$modal_options['form_id']  		= isset($modal_options['form_id']) ? $modal_options['form_id'] : 'ajax_form_'.$this->link;
		$modal_options['form_action']  	= isset($modal_options['form_action']) ? $modal_options['form_action'] : 'process.php';

		$html = $app->renderModalOpen($modal_options);
		$html .= $app->renderFormOpen($modal_options);
		return $html;
	}

	function renderModalOpen(array $options=array()) {
		global $app;
		$modal_options					= $options;
		$modal_options['name']  		= isset($modal_options['name']) ? $modal_options['name'] : $this->name;
		$modal_options['shortname']  	= isset($modal_options['shortname']) ? $modal_options['shortname'] : $this->shortname;
		$modal_options['icon']  		= isset($modal_options['icon']) ? $modal_options['icon'] : $this->icon;
		$modal_options['color']  		= isset($modal_options['color']) ? $modal_options['color'] : 'primary';
		$modal_options['details']  		= isset($modal_options['details']) ? $modal_options['details'] : $this->details;
		$modal_options['form_id']  		= isset($modal_options['form_id']) ? $modal_options['form_id'] : 'ajax_form_'.$this->link;
		$modal_options['form_action']  	= isset($modal_options['form_action']) ? $modal_options['form_action'] : 'process.php';

		$html = $app->renderModalOpen($modal_options);
		return $html;
	}

	function renderModalFormClose($options=array(), $footer = true) {
		global $app;

		$modal_options					= $options;

		$modal_options['name']     		= isset($options['name']) ? $options['name'] : $this->name;
		$modal_options['shortname']		= isset($options['shortname']) ? $options['shortname'] : $this->shortname;
		$modal_options['title']    		= isset($options['title']) ? $options['title'] : $this->title;
		$modal_options['icon']     		= isset($options['icon']) ? $options['icon'] : $this->icon;
		$modal_options['color']    		= isset($options['color']) ? $options['color'] : 'primary';
		$modal_options['details']  		= isset($options['details']) ? $options['details'] : $this->details;
		$modal_options['form_id'] 		= isset($options['form_id']) ? $options['form_id'] : 'ajax_form_'.$this->link;
		$modal_options['form_action']   = isset($options['form_action']) ? $options['form_action'] : 'process.php';
		$modal_options['action_label']  = isset($options['action_label']) ? $options['action_label'] : NULL;

		$form_id 			= $modal_options['form_id'];
		$html 				= $app->renderFormClose($modal_options);
    	$html 				.= $app->renderModalClose($modal_options);
		//if(!$footer) $html 	.= $app->renderScript('$(\'#'.$form_id.'_footer\').hide();');
		return $html;
	}

	function renderModalClose($options=array(), $footer = true) {
		global $app;
		$modal_options  = array (
			'name'     		=> isset($options['name']) ? $options['name'] : $this->name,
			'shortname'		=> isset($options['shortname']) ? $options['shortname'] : $this->shortname,
			'title'    		=> isset($options['title']) ? $options['title'] : $this->title,
			'icon'     		=> isset($options['icon']) ? $options['icon'] : $this->icon,
			'color'    		=> isset($options['color']) ? $options['color'] : 'primary',
			'details'  		=> isset($options['details']) ? $options['details'] : $this->details,
			'form_id'  		=> isset($options['form_id']) ? $options['form_id'] : 'ajax_form_'.$this->link,
			'form_action'   => isset($options['form_action']) ? $options['form_action'] : 'process.php'
		);

		$form_id 			= $modal_options['form_id'];
    	$html 				= $app->renderModalClose($modal_options);
		//if(!$footer) $html 	.= $app->renderScript('$(\'#'.$form_id.'_footer\').hide();');
		return $html;
	}

	function renderTableBody($options) {

		global $app;
		$rows 		= $options['rows'];
		$columns 	= $options['columns'];
		$html		= '';

		foreach($rows as $row) {
			$link_options    = array('id'=>$row->id, 'name'=>$row->name);
			$html 	.= '<tr>';

			foreach($columns as $column) {
				if(isset($column['value'])) {
					$column_value 	= $column['value'];
					$column_css 	= isset($column['css']) ? $column['css'] : '';
					$column_css 	.= isset($column['cssx']) ? ' '.$column['cssx'] : '';
					$css_array 		= explode(' ', $column_css);

					// Column value transformations
					if(in_array('money', $css_array)) $row->$column_value = $app->formatMoney($row->$column_value);
					
					if($column_value == 'name') {
						$html .= '<td>'.$app->renderLinkOpen($link_options).'</td>';
					} elseif($column_value == 'icon/color') {
						$icon 	= $row->icon;
						$color	= $row->color;
						$html .= '<td class="text-center"><span class="text-'.$color.' '.$icon.'"></span></td>';
					} elseif($column_value == 'options') {
						$html .= '<td class="text-center text-nowrap">';
						foreach($options['options'] as $option) {
							if($option['type'] == 'copy') $html .= $app->renderButtonCopy($link_options);
							if($option['type'] == 'edit') $html .= $app->renderButtonEdit($link_options);
							if($option['type'] == 'delete') $html .= $app->renderButtonDelete($link_options);
							if($option['type'] == 'run') $html .= $app->renderButtonRun($link_options);
							if($option['type'] == 'custom') { 							
								$id     = isset($option['id']) ? $option['id'] : $link_options['id'];
								$name   = isset($option['name']) ? $option['name'] : 'Item name';
								$css    = isset($option['css']) ? $option['css'] : 'btn btn-default';
								$form   = isset($option['form']) ? $option['form'] : 'form.php';
								$icon   = isset($option['icon']) ? $option['icon'] : 'fa-regular fa-copy';
								$title  = isset($option['title']) ? $option['title'] : 'Custom';
								$action = isset($option['action']) ? $option['action'] : 'open';
								
								$html .= '<a class="remote-modal bs-tooltip '.$css.'" title="'.$title.'" data-bs-toggle="modal" data-bs-target="#modal_lg" data-bs-backdrop="static" data-bs-keyboard="false" tabindex="-1" href="'.$form.'?action='.$action.'&id='.$id.'"> <span class="'.$icon.'"></span> </a>';
							}
						}
						$html .= '</td>';
					} else {
						$html .= '<td class="'.$column_css.'">'.$row->$column_value.'</td>';
					}
				}
			}

			$html 	.= '</tr>';
		}
		
		return $html;
	}

	
	function renderListTable(array $options=array()) {
		// Returns html for list of items in a table
		global $app;

		// Default list options, if options not provided
		if(!count($options)) {
			$options            = array (
				'columns'       => array (
					array('column'=>'id'),
					array('title'=>'code', 'css'=>'w-15', 'column'=>'code'),
					array('title'=>'name', 'css'=>'', 'column'=>'name'),
					array('title'=>'icon/color', 'css'=>'w-10 text-center'),
					array('title'=>'options', 'css'=>'w-20 text-end'),
					array('column'=>'icon'),
					array('column'=>'color')
				),
				'options'       => array (
					array('type'=>'copy'), array('type'=>'edit'), array('type'=>'delete')
				),
				'extra'			=> array('condition'=>"status = 1", 'order'=>"name")
			);
		}

		// Extract fields with column attribute in order to select in the table 
		$selected_columns 	= array();
		$data_columns		= array();
		foreach($options['columns'] as $column) {
			if(isset($column['column'])) {
				$selected_columns[] = $column['column'];
			}

			if(isset($column['title'])) {
				$data_columns[] = $column['data'];
			}
		}

		// Render table header th with all fields with title attribute
		$html = $app->renderTableOpen($options);

		// Select from the database all fields with column attribute
		$columns	= implode(', ', $selected_columns);

		// Append joined tables if any
		$sql = "SELECT $columns FROM $this->table ";
		if(isset($options['joins'])) {
			foreach($options['joins'] as $joined_table) {
				$sql 		.= $joined_table;	
			}	
		}

		// Conditions
		if(isset($options['conditions'])) {
			$conditions		= implode("AND ", $options['conditions']);
			$sql 		   .= " WHERE $conditions ";
		}

		// Orders
		if(isset($options['orders'])) {
			$orders		= implode(", ", $options['orders']);
			$sql 		.= "ORDER BY $orders ";
		}

		// Limits
		if(isset($options['limits'])) {
			$limits 	= $options['limits'];
			$sql 		.= " LIMIT $limits ";
		}

		// Fetch data from the database
		$rows 		= $app->query($sql);

		// Render all columns with title attribute
		foreach($rows as $row) {
			$row_id 		= $this->primary_key;
			$id         	= $row->$row_id;
			$name       	= $row->name;
			$fun_options    = array('id'=>$id, 'name'=>$name); 
		
			$html .= '<tr>';

			foreach($data_columns as $data_column) {
				if($data_column == 'name') {
					$html .= '<td>'.$app->renderLinkOpen($fun_options).'</td>';
				} elseif($data_column == 'icon/color') {
					$icon 	= $row->icon;
					$color	= $row->color;
					$html .= '<td class="text-center"><span class="text-'.$color.' '.$icon.'"></span></td>';
				} elseif($data_column == 'options') {
					$html .= '<td class="text-end text-nowrap">';
					foreach($options['options'] as $option) {
						if($option['type'] == 'copy') $html .= $app->renderButtonCopy($fun_options);
						if($option['type'] == 'edit') $html .= $app->renderButtonEdit($fun_options);
						if($option['type'] == 'delete') $html .= $app->renderButtonDelete($fun_options);
					}
					$html .= '</td>';
				} else {
					$html .= '<td>'.$row->$data_column.'</td>';
				}
			}
			
			$html .= '</tr>';
		}
		
		$html .= $app->renderTableClose();
		return $html;
	}

	function renderIconColor() {
		return '<span class="text-'.$this->color.' '.$this->icon.'"></span>';
	}

	function renderCard($options = array()) {
		global $app;
		
		$link_options		= array('name'=>$this->name, 'id'=>$this->id, 'css'=>'btn btn-primary');
		$links				= $app->renderButtonOpen($link_options).$app->renderButtonEdit($link_options).$app->renderButtonCopy($link_options).$app->renderButtonDelete($link_options);
		$options['links']	= isset($options['links']) ? $options['links'] : $links;
		$options['name']	= isset($options['name']) ? $options['name'] : $this->name;
		$options['body']	= isset($options['body']) ? $options['body'] : '';
		
		return $app->renderCard($options);
	}

	function getDefaultFilters() {
		// Returns array of filters used in list display
		global $app;
		$filters            = array (
			array (
				'type'      => 'search',
				'css'       => 'col-xl-8 col-lg-6 pt-2 pb-3',
				'control'   => $app->renderInputText('keyword', '', 'Find '.$this->name.' by name', 'form-control form-control-lg filter-control onkeyup-reload-list', 'autofocus')
			), 
			array (
				'type'      => 'buttons',
				'control'   => array (
					array('type'=>'advanced'),
					array('type'=>'refresh'),
					array('type'=>'create')
				)
			)
		);

		return $filters; 
	}

	function insertSystemSettings($template) {
		global $root, $client;

		foreach($client as $name => $value) {
			$variable = '__SETTINGS:'.strtoupper($name).'__';
			$template = str_replace($variable, $value, $template);
		}

		$template	= str_replace('__ROOT__', $root, $template);
		$template	= str_replace('__CLIENT_PATH__', CLIENT_PATH, $template);
		$template	= str_replace('__CLIENT_LINK__', CLIENT_LINK, $template);
		
		return $template;
	}

	function insertDataIntoTemplate($template) {

		$properties = get_object_vars($this);
		foreach($properties as $name => $value) {
			$variable = '__'.strtoupper($name).'__';
			$template = str_replace($variable, $value, $template);
		}

		return $this->insertSystemSettings($template);
	}

	function renderAuditFields() {
		global $app, $user;
		$html  = '';
		$html .= $app->renderInputPeople('created_by', $this->created_user_id, '', 'line-control-plaintext grid-control default-none select-search', '', '', $user->getUsers());
		$html .= $app->renderInputText('created_on', $app->datetime($this->created_datetime), '', 'line-control-plaintext grid-control placeholder-underline');
		$html .= $app->renderInputPeople('modified_by', $this->modified_user_id, '', 'line-control-plaintext grid-control select-search default-none', '', '', $user->getUsers());
		$html .= $app->renderInputText('modified_on', $app->datetime($this->modified_datetime), '', 'line-control-plaintext grid-control placeholder-underline');
		$html .= $app->renderInputSelect('status', $this->status, '', 'line-control grid-control select-search', '', 'settings_categories', 'code', 'name', "type='ObjectStatus'");
		return $html;
	}

	function tagged($tag) {
		if(isset($this->tags)) {
			return (strpos($this->tags, $tag) > -1);
		}
		return false;
	}

	function set($column, $value) {
		return $this->update(array($column=>$value));
	}

	function addCommonFields($data) {
		global $now, $user_id;

		$data = array_merge (
			array('id'=>0), 
			$data, 
			array (
				'f1'                    => '',
				'f2'                    => '',
				'f3'                    => '',
				'f4'                    => '',
				'f5'                    => '',
				'f6'                    => '',
				'f7'                    => '',
				'f8'                    => '',
				'f9'                    => '',
				'created_user_id'       => $user_id,
				'created_datetime'      => $now,
				'modified_user_id'      => 0,
				'modified_datetime'     => 0,
				'status'                => 1
			)
		);
		return $data;
	}

	function nextCode($column='code') {
		global $app;
		$sql = "SELECT (MAX($column) + 1) AS code FROM $this->table ";
		foreach($app->read($sql) as $row) {
			if(is_numeric($row->code)) return $row->code;
		}
		return 1;
	}

	function getNextCode($column='code', $condition=1) {
		global $app;
		$sql = "SELECT (MAX($column) + 1) AS code FROM $this->table WHERE 1 AND $condition ";
		foreach($app->read($sql) as $row) {
			if(is_numeric($row->code)) return $row->code;
		}
		return 1;
	}

	function createdUser() {
		return new User($this->created_user_id);
	}

	function modifiedUser() {
		return new User($this->modified_user_id);
	}

	function renderLinkSelect() {
		$html = '<span class="form-switch">
		<input role="button" class="form-check-input" style="margin-top:12px" type="checkbox" id="check_'.$this->id.'" />
		</span>';
		return $html;
	}

}

?>