<?php 

class Transaction extends Crud {
	function __construct($id=0) {	
		parent::__construct($id);
        if(!$id) $this->urgency = 'Routine';
	}

	function createWithMinimumData($data) {
		global $app, $user_id, $now;

        $items                          = isset($data['items']) ? htmlspecialchars($data['items'], ENT_QUOTES) : '[]';

		$create_data                    = $this->addCommonFields ( 
            array (
                'type'                  => isset($data['type']) ? $data['type'] : $this->type,
                'type_id'               => isset($data['type_id']) ? $data['type_id'] : $this->type_id,
                'subtype'               => isset($data['subtype']) ? $data['subtype'] : $this->subtype,
                'subtype_id'            => isset($data['subtype_id']) ? $data['subtype_id'] : $this->subtype_id,
                'parent'                => isset($data['parent']) ? $data['parent'] : '',
                'parent_id'             => isset($data['parent_id']) ? $data['parent_id'] : 0,
                'reference'             => isset($data['reference']) ? $data['reference'] : '',
                'reference_no'          => isset($data['reference_no']) ? $data['reference_no'] : '',
                'personnel'             => isset($data['personnel']) ? $data['personnel'] : '',
                'personnel_id'          => isset($data['personnel_id']) ? $data['personnel_id'] : 0,
                'store_id'              => isset($data['store_id']) ? $data['store_id'] : 0,
                'code'                  => isset($data['code']) ? $data['code'] : 0,
                'name'                  => isset($data['name']) ? $app->formatName($data['name']) : '',
                'birthdate'             => isset($data['birthdate']) ? ($data['birthdate'] > '1900-01-01' ? $data['birthdate'] : date('Y-m-d')) : date('Y-m-d'),
                'sex'                   => isset($data['sex']) ? $data['sex'] : '',
                'phone'                 => isset($data['phone']) ? $data['phone'] : '',
                'email'                 => isset($data['email']) ? $data['email'] : '',
                'pricelist'             => isset($data['pricelist']) ? $data['pricelist'] : '',
                'pricelist_id'          => isset($data['pricelist_id']) ? $data['pricelist_id'] : 0,
                'insurance'             => isset($data['insurance']) ? $data['insurance'] : '',
                'insurance_id'          => isset($data['insurance_id']) ? $data['insurance_id'] : 0,
                'scheme'                => isset($data['scheme']) ? $data['scheme'] : '',
                'scheme_id'             => isset($data['scheme_id']) ? $data['scheme_id'] : 0,
                'product'               => isset($data['product']) ? $data['product'] : '',
                'product_id'            => isset($data['product_id']) ? $data['product_id'] : 0,
                'card_no'               => isset($data['card_no']) ? $data['card_no'] : '',
                'authorization_no'      => isset($data['authorization_no']) ? $data['authorization_no'] : '',
                'supplementary'         => isset($data['supplementary']) ? $data['supplementary'] : '',
                'folio_id'              => isset($data['folio_id']) ? $data['folio_id'] : 0,
                'folio_no'              => isset($data['folio_no']) ? $data['folio_no'] : 0,
                'year'                  => isset($data['year']) ? $data['year'] : date('Y'),
                'month'                 => isset($data['month']) ? $data['month'] : date('m'),
                'date'                  => isset($data['date']) ? $data['date'] : date('Y-m-d'),
                'visit_id'              => isset($data['visit_id']) ? $data['visit_id'] : 0,
                'visit_location_id'     => isset($data['visit_location_id']) ? $data['visit_location_id'] : 0,
                'start_datetime'        => isset($data['start_datetime']) ? $data['start_datetime'] : 0,
                'end_datetime'          => isset($data['end_datetime']) ? $data['end_datetime'] : 0,
                'urgency'               => isset($data['urgency']) ? $data['urgency'] : 'Routine',
                'urgency_code'          => isset($data['urgency_code']) ? $data['urgency_code'] : 3,
                'progress'              => isset($data['progress']) ? (strlen($data['progress'] > 1) ? $data['progress'] : 'Requested') : 'Requested',
                'progress_code'         => isset($data['progress_code']) ? ($data['progress_code'] > 0 ? $data['progress_code'] : 1) : 1,
                'progress_notes'        => isset($data['progress_notes']) ? $data['progress_notes'] : '',
                'progress_user_id'      => isset($data['progress_user_id']) ? $data['progress_user_id'] : $user_id,
                'progress_datetime'     => isset($data['progress_datetime']) ? $data['progress_datetime'] : $now,
                'attachments'           => isset($data['attachments']) ? $data['attachments'] : '[]',
                'items'                 => $items,
                'cost'                  => isset($data['cost']) ? $app->unformatMoney($data['cost']) : 0,
                'subtotal'              => isset($data['subtotal']) ? $app->unformatMoney($data['subtotal']) : 0,
                'tax'                   => isset($data['tax']) ? $app->unformatMoney($data['tax']) : 0,
                'shipping'              => isset($data['shippimg']) ? $app->unformatMoney($data['shippimg']) : 0,
                'discount'              => isset($data['discount']) ? $app->unformatMoney($data['discount']) : 0,
                'exemption'             => isset($data['exemption']) ? $app->unformatMoney($data['exemption']) : 0,
                'promise'               => isset($data['promise']) ? $app->unformatMoney($data['promise']) : 0,
                'payable'               => isset($data['payable']) ? $app->unformatMoney($data['payable']) : (isset($data['subtotal']) ? $app->unformatMoney($data['subtotal']) : 0),
                'paid'                  => isset($data['paid']) ? $app->unformatMoney($data['paid']) : 0,
                'due'                   => isset($data['due']) ? $app->unformatMoney($data['due']) : 0,
                'requested'             => 1,
                'requested_notes'       => isset($data['requested_notes']) ? $data['requested_notes'] : '',
                'requested_user_id'     => $user_id,
                'requested_datetime'    => $now,
                'authorized'            => 0,
                'authorized_notes'      => '',
                'authorized_user_id'    => 0,
                'authorized_datetime'   => 0,
                'approved'              => 0,
                'approved_notes'        => '',
                'approved_user_id'      => 0,
                'approved_datetime'     => 0,
                'transferred'           => 0,
                'transferred_notes'     => '',
                'transferred_user_id'   => 0,
                'transferred_datetime'  => 0,
                'received'              => 0,
                'received_notes'        => '',
                'received_user_id'      => 0,
                'received_datetime'     => 0,
                'confirmed'             => 0,
                'confirmed_notes'       => '',
                'confirmed_user_id'     => 0,
                'confirmed_datetime'    => 0,
                'billed'                => 0,
                'billed_reference'      => isset($data['billed_reference']) ? $data['billed_reference'] : '',
                'billed_notes'          => '',
                'billed_user_id'        => 0,
                'billed_datetime'       => 0,
                'payment'               => 0,
                'payment_method'        => '',
                'payment_channel'       => 0,
                'payment_account'       => 0,
                'payment_account_id'    => 0,
                'payment_transaction'   => '',
                'payment_reference'     => 0,
                'payment_receipt'       => 0,
                'payment_name'          => 0,
                'payment_phone'         => '',
                'payment_notes'         => '',
                'payment_user_id'       => 0,
                'payment_datetime'      => 0,
                'forwarded'             => 0,
                'forwarded_to'          => 0,
                'forwarded_id'          => 0,
                'forwarded_notes'       => '',
                'forwarded_user_id'     => 0,
                'forwarded_datetime'    => 0,
                'dispensed'             => 0,
                'dispensed_notes'       => '',
                'dispensed_user_id'     => 0,
                'dispensed_datetime'    => 0,
                'returned'              => 0,
                'returned_notes'        => '',
                'returned_user_id'      => 0,
                'returned_datetime'     => 0,
                'printed'               => 0,
                'printed_notes'         => '',
                'printed_user_id'       => 0,
                'printed_datetime'      => 0
            )
        );

        $transaction_id                     = $this->create($create_data);

        if(isset($data['items'])) {
            foreach(json_decode($data['items']) as $item) {
                $item                       = (array) $item; 
                $item['type']               = isset($data['type']) ? $data['type'] : $this->type;
                $item['type_id']            = isset($data['type_id']) ? $data['type_id'] : $this->type_id;
                $item['subtype']            = isset($data['subtype']) ? $data['subtype'] : $this->subtype;
                $item['subtype_id']         = isset($data['subtype_id']) ? $data['subtype_id'] : $this->subtype_id;
                $item['parent']             = get_class($this);
                $item['parent_id']          = $transaction_id;
                $item['insurance_id']       = isset($data['insurance_id']) ? $data['insurance_id'] : 0;
                $item['scheme_id']          = isset($data['scheme_id']) ? $data['scheme_id'] : 0;
                $item['store_id']           = $data['store_id'];
                $item['requested']          = 1;
                $item['requested_quantity'] = isset($item['requested_quantity']) ? $item['requested_quantity'] : $item['quantity'];
                $item['requested_notes']    = isset($data['requested_notes']) ? $data['requested_notes'] : '';

                $transaction_item           = new TransactionItem($item['parent'], $transaction_id);
                $transaction_item->createWithMinimumData($item);
            }
        }

        $this->id = $transaction_id;
        $this->updateInlineItems();

		return $transaction_id;
	}

	function personnel($user_id) {
		$user	= new User($this->$user_id);
		return $user->personnel();
	}

	function store($store_id='store_id') {
		return new Store($this->$store_id);
	}

    function items() {
        global  $app;
        $sql    = "SELECT * FROM $this->items_table WHERE parent_id = $this->id ";
        $rows   = array();
        foreach($app->read($sql) as $row) {
            $rows[$row->item_id] = $row;
        }
        return $rows;
    }

    function checkItemsCount(array $items) {
        global $app;
        if(!count($items)) {
            die($app->renderAlertError('Please add atleast one item to your cart.'));
        }
    }

    function getCustomerByCode($code) {
        global $app;
        $customer_id = $app->getColumnValue('billing_customers', 'id', 'code', $code);
        return new Customer($customer_id);
    }

    function getInsuranceByCode($code) {
        global $app;
        $insurance_id = $app->getColumnValue('billing_insurances', 'id', 'code', $code);
        return new Insurance($insurance_id);
    }

    function getSchemeByCode($code) {
        global $app;
        $scheme_id = $app->getColumnValue('billing_insurance_schemes', 'id', 'code', $code, "status = 1");
        return new Scheme($scheme_id);
    }

    function createSupplierIfNotExists($options) {
        if($options['id']) {
            return new Supplier($options['id']);
        } else {
            $supplier       = new Supplier();
            $supplier_id    = $supplier->createWithMinimumData($options);
            return new Supplier($supplier_id);
        }
    }

    function renderProgressBlock($progress_code) {
        global $app;
        $code               = $progress_code == 'Paid' ? 'Payment' : $progress_code;
        $progress           = $app->getCategoryByCode('TransactionProgress', $progress_code);
        $notes              = strtolower($code.'_notes');
        
        if($progress_code == 'Created') $progress   = $app->getCategoryByCode('TransactionProgress', 'Requested');
        if($notes == 'created_notes') $notes   = 'requested_notes';
        $user               = strtolower($code.'_user_id');
        $datetime           = strtolower($code.'_datetime');
        $progress_user      = new User($this->$user);
        $progress_employee  = $progress_user->personnel();
        $progress_datetime  = $this->$datetime;

        $html               = '<img src="'.$progress_employee->picture().'" class="rounded-circle mx-auto d-block bg-light mt-2 mb-1" style="width:40%; max-width:100px; border:5px solid white;" />';
        $html               .= '<div style="display:inline-block; position:relative; top:-35px; right:-32px;"> <span class="bg-primary text-white" style="padding:3px 4px; border:4px solid white; border-radius:50%;"> <small><span class="'.$progress->icon.' fa-fw"></span></small> </span></div>';
        $html               .= '<h4 class="fw-bold" style="margin-top:-15px">'.$code.'</h4>';
        $html               .= '<div>'.$progress_user->person()->name.'</div>';
        $html               .= '<div>'.$app->datetime($progress_datetime).'</div>';

        if(strlen($this->$notes)) {
            $html           .= '<div class="text-muted my-3"><i>'.$this->$notes.'</i></div>';
        }
        
        return              $html;
    }

    function renderVerticalCreator() {
        global $user_id, $now;

        $this->created_user_id  = $user_id;
        $this->created_datetime = $now;
        return $this->renderProgressBlock('Created');
    }

    function renderVerticalProgress() {
        global $app;
        $progress   = array();
        $glue       = '<div class="my-3"><span class="fa-solid fa-down-long"></span></div>';
        $sql        = "SELECT code FROM settings_categories WHERE type = 'TransactionProgress' ORDER BY priority";
        foreach($app->read($sql) as $row) {
            $code   = $row->code == 'Paid' ? 'Payment' : $row->code;

            $progress_user = strtolower($code).'_user_id';
            if($this->$progress_user) {
                $progress[] = $this->renderProgressBlock($row->code);
            }
        }
        return implode($glue, $progress);
    }

    function renderButtonProgress($options = array()) {
        global $app;

        $progress       = $this->progress;
        $progress_code  = $this->progress_code;
        $items          = $this->items();
        $count          = 0;
        $total_count    = 0;
        $modal          = isset($options['modal']) ? $options['modal'] : 'modal_lg';

        if(is_array($items)) {
            foreach($items as $item) {
                if($item->status > 0) {
                    $total_count++;
                    if($item->progress_code > $progress_code) {
                        $count          = 1;
                        $progress       = $item->progress;
                        $progress_code  = $item->progress_code;
                    } elseif($item->progress_code == $progress_code) {
                        $count++;
                    }
                }
            }
        }
        
        $progress_obj   = $app->getCategoryByCode('TransactionProgress', $progress);
        $html           = '<a class="btn btn-outline-'.$progress_obj->color.' w-100 rounded-pill remote-modal text-decoration-none text-nowrap fw-normal" data-bs-toggle="modal" data-bs-target="#'.$modal.'" href="form.php?action=open&id='.$this->id.'">'.$progress_obj->code.' '.$count.'/'.$total_count.'</a>';
        return          $html;
    }

    function hasItemProgress($progress) {
        $items          = json_decode($this->items) ;
        if(!$items) $items = $this->getItems();

        foreach($items as $item) {
            if($item->progress == $progress) return true;
        }
        return false;
    }

    function maxItemProgressCode() {
        $progress_code = 0;
        $items          = json_decode($this->items) ;
        if(!$items) $items = $this->getItems();

        foreach($items as $item) {
            if($item->progress_code > $progress_code) $progress_code = $item->progress_code;
        }
        return $progress_code;
    }

    function renderUrgencyButton() {
        $urgency    = $this->urgency();
        $html       = '<a class="text-'.$urgency->color.' text-truncate"><span class="'.$urgency->icon.' fa-fw"></span> '.$urgency->code.'</a>';
        return      $html;
    }

    function  renderItemsJson() {
        global $app;
        $sql			= "SELECT * FROM $this->items_table WHERE parent_id = $this->id ";
        $items 	        = array();
        foreach($app->read($sql) as $row) {
            $items[$row->id]	= $row;
        }
        return          $app->array2json($items);
    }

    function  renderCreateItemsJson() {
        global $app;
        $sql			= "SELECT id, item_id, name, details, uom, pack, size, qty, quantity, unit_cost, cost, unit_price, price 
        FROM $this->items_table WHERE parent_id = $this->id ";
        $items 	        = array();
        foreach($app->read($sql) as $row) {
            $items[$row->id]	= $row;
        }
        return          $app->array2json($items);
    }

    function getItems() {
        global  $app;
        $sql    = "SELECT * FROM $this->items_table WHERE parent_id = $this->id ";
        return $app->read($sql);
    }

    function mergeItems($first_items, $second_items) {
        global $app;

        $return_items   = array();

        foreach(array_keys($second_items) as $second_item_id) {
            if(isset($first_items[$second_item_id])) {
                // Check UOM compatibility
                if($first_items[$second_item_id]->pack == $second_items[$second_item_id]->pack) {
                    $first_items[$second_item_id]->id                   = $first_items[$second_item_id]->item_id;
                    $first_items[$second_item_id]->qty                  += $second_items[$second_item_id]->qty;
                    $first_items[$second_item_id]->quantity             += $second_items[$second_item_id]->quantity;
                    $first_items[$second_item_id]->requested_quantity   += $second_items[$second_item_id]->requested_quantity;
                    $first_items[$second_item_id]->cost                 = $first_items[$second_item_id]->unit_cost * $first_items[$second_item_id]->quantity;
                    $first_items[$second_item_id]->price                = $first_items[$second_item_id]->unit_price * $first_items[$second_item_id]->quantity;
                } else {
                    die($app->renderAlertError('Cannot add '.$second_items[$second_item_id]->quantity.' '.$second_items[$second_item_id]->pack.' to '.$first_items[$second_item_id]->quantity.' '.$first_items[$second_item_id]->pack, $second_items[$second_item_id]->name));
                }
            } else {
                $second_items[$second_item_id]->id              = $second_items[$second_item_id]->item_id;
                $second_items[$second_item_id]->parent     = $this->type;
                $second_items[$second_item_id]->parent_id  = $this->id;
                $first_items[$second_item_id]                   = $second_items[$second_item_id];
            }
        }
        foreach($first_items as $first_item) {
            $return_items[] = $first_item;
        }
        return $return_items;
    }

    function resetItems($items) {
        $this->clearItems();
        $transaction_item   = new TransactionItem($this->type, $this->id);
        foreach($items as $item) {  
            $transaction_item->createWithMinimumData((array) $item);
        }
        $this->set('items', json_encode($items));
    }

    function clearItems() {
        global $db;
        $sql = "DELETE FROM $this->items_table WHERE parent_id = $this->id ";
        return $db->update($sql);
    }

    function updateInlineItems() {
        $items = array();

        foreach($this->items() as $item) {  
            $items[] = $item;
        }
        $this->set('items', json_encode($items));
    }

    function urgency() {
        global $app;
        return $app->getCategoryByCode('UrgencyType', $this->urgency);
    }

    function progress() {
        global $app;
        return $app->getCategoryByCode('TransactionProgress', $this->progress);
    }

    function changeProgress($progress, $quantity_column='requested_quantity', $notes='', $progress_code=0) {
        global $app, $now, $user_id;

        $column                 = strtolower($progress);
        $progress_obj           = $app->getCategoryByCode('TransactionProgress', $progress);
        $progress_code          = $progress_code ? $progress_code : $progress_obj->priority;
        
        if($column == 'paid')   {
            $column             = 'payment';
            $progress           = 'payment';
        }

        $trans_data             = array (
            'progress'          => $progress_obj->code,
            'progress_code'     => $progress_code,
            $column             => 1,
            $column.'_notes'    => $notes,
            $column.'_user_id'  => $user_id,
            $column.'_datetime' => $now
        );

        $updated = $this->update($trans_data);
        if($updated) {
            foreach($this->items() as $item) {
                $updated_column = strtolower($progress.'_quantity');
                $trans_item     = new TransactionItem($this->type, $this->id, $item->id);
                if($quantity_column) {
                    if($item->$quantity_column) $trans_data[$updated_column] = $item->$quantity_column;   
                }       
                $trans_item->update($trans_data);
            }
        }

        $this->updateInlineItems();
        
        return $updated;
    }

    function renderConsumptionControls() {
        global $app;

        $periods    = array (
            array ('name'=>'Average daily consumption', 'value'=>'Day'),
            array ('name'=>'Average weekly consumption', 'value'=>'Week'),
            array ('name'=>'Average monthly consumption', 'value'=>'Month'),
            array ('name'=>'Average quartely consumption', 'value'=>'Quarter'),
            array ('name'=>'Average semi annually consumption', 'value'=>'SemiAnnual'),
            array ('name'=>'Average annual consumption', 'value'=>'Annual')  
        );
        $html = '<div id="trans_store_consumption" class="my-3"><div class="trans-container"> <h5 class="trans-subtitle"><span class="fa-regular fa-store fa-fw"></span>&nbsp; Average store consumption:</h5></div></div>';
        $html   .= '<div class="row">';
        $html   .= '<div class="col-12 col-sm-6 my-2">'.$app->renderInputSelect('trans_consumption_period', '', 'Choose consumption period', 'select-search', 'data-style="btn-outline-secondary"', $periods).'</div>';
        $html   .= '<div class="col-12 col-sm-6 my-2"><a class="btn w-100 btn-primary" onclick="transaction.loadStoreConsumptionIntoCart($(\'#trans_store_id\').val(), $(\'#trans_consumption_period\').val())"> <span class="fa-regular fa-upload fa-fw"></span> Load consumption into cart </a> </div>';
        $html   .= '</div>';

        return $html;
    }

    function renderAuthorizeConfirmButton() {
        global $app;

        $options    = array (
            array('name'=>'Authorize this request', 'value'=>'authorized', 'icon'=>'fa-regular fa-lock-open'),
            array('name'=>'Return to request stage', 'value'=>'returned', 'icon'=>'fa-regular fa-pencil')
        );

        $html = '<div class="row">
        <div class="col-sm-12 my-4">
        <h5 class="my-3"> <span class="fa-solid fa-check fa-fw"></span> &nbsp;Choose action: </h5>
        '.$app->renderInputSelect('trans_authorized', '', 'Choose action', 'select-search', 'data-style="btn-outline-secondary"', $options).'
        </div> <div class="col-sm-6 my-3">
        </div> <div class="col-sm-12 mb-3">
        <h5 class="my-3"> <span class="fa-regular fa-note-sticky fa-fw"></span> &nbsp;Notes or comments: </h5>
        '.$app->renderInputTextarea('trans_authorized_notes', $this->authorized_notes, 'Enter optional notes').'
        </div></div>';

        return $html;
    }

    function renderApproveConfirmButton() {
        global $app;

        $options    = array (
            array('name'=>'Approve this request', 'value'=>'approved', 'icon'=>'fa-regular fa-check-double'),
            array('name'=>'Return to request stage', 'value'=>'returned', 'icon'=>'fa-regular fa-pencil')
        );

        $html = '<div class="row">
        <div class="col-sm-6 my-4">
        '.$app->renderInputSelect('trans_approved', '', 'Choose your option', 'select-search', 'data-style="btn-outline-secondary"', $options).'
        </div> <div class="col-sm-6 my-3">
        </div> <div class="col-sm-12 mb-3">
        '.$app->renderInputTextarea('trans_approved_notes', $this->approved_notes, 'Enter optional notes', 'line-controls').'
        </div></div>';

        return $html;
    }

    function renderTransferConfirmButton() {
        global $app;

        $html = '<div class="row">
        <div class="col-sm-6 my-3">
        <input name="trans_approved" class="bs-toggle w-100" type="checkbox" id="trans_transferred" data-width="100%" 
        data-style="w-100" data-onlabel="<span class=\'fa-light fa-lock-open fa-fw\'></span> Transfer confirmed" data-offlabel="<span class=\'fa-light fa-lock fa-fw\'></span> Transfer not confirmed" />
        </div><div class="col-sm-12 mb-3">'.$app->renderInputTextarea('trans_transferred_notes', $this->transferred_notes, 'Enter optional transfer notes here', 'line-controls').'</div></div>';
        
        return $html;
    }

    function renderReceiveConfirmButton() {
        global $app;

        $options    = array (
            array('name'=>'Receive selected items', 'value'=>'Received', 'icon'=>'fa-regular fa-inbox-in'),
            array('name'=>'Return request to approval', 'value'=>'Approved', 'icon'=>'fa-regular fa-check-double')
        );

        $html = '<div class="row">
        <div class="col-sm-6 my-4">
        '.$app->renderInputSelect('trans_received', '', 'Choose action', 'select-search', 'data-style="btn-outline-secondary"', $options).'
        </div> <div class="col-sm-6 my-3">
        </div> <div class="col-sm-12 mb-3">
        '.$app->renderInputTextarea('trans_received_notes', $this->received_notes, 'Enter optional notes', 'line-controls').'
        </div></div>';

        return $html;
    }

    function renderForwardConfirmButton($title, $table_name) {
        global $app, $client;

        $html       = '<div class="row">
        <div class="col-sm-6 my-4">
        <input name="trans_forwarded" class="bs-toggle" type="checkbox" id="trans_forwarded" data-width="100%" 
        data-style="w-100" data-onlabel="<span class=\'fa-light fa-lock-open fa-fw\'></span> Forward confirmed" data-offlabel="<span class=\'fa-light fa-lock fa-fw\'></span> Forward not confirmed" />
        </div>';
        
        $requests       = array (
            array (
                'name'  => 'Create new '.$title, 
                'value' => 'New'
            )
        );
        $requests_sql   = "SELECT id, subtotal, created_datetime FROM $table_name WHERE progress = 'Requested' ORDER BY id DESC ";
        foreach($app->read($requests_sql) as $row) {
            $requests[] = array (
                'name'  => 'Add to #'.$row->id.' '.$client->currency_code.' '.number_format($row->subtotal).' created '.$app->date($row->created_datetime),
                'value' => $row->id
            );
        }  
        $html .= '<div class="col-12 col-sm-6 my-4">';
        $html .= $app->renderInputSelect('trans_forward_option', '', 'Choose where to forward request', 'form-control select-search', ' data-style="btn-outline-secondary" ', $requests, 'code', 'name', "type=''", 'Category');
        $html .= '</div> <div class="col-sm-12 mb-3">';
        $html .= $app->renderInputTextarea('trans_forwarded_notes', $this->forwarded_notes, 'Enter optional forwarding notes here', 'line-controls');
        $html .= '</div></div>';

        return $html;
    }

    function renderForwardButton() {
        global $app;
        $html = '<div class="row">
        <div class="col-sm-6 my-3">
        <input name="trans_forwarded" class="toggle-child bs-toggle w-100" type="checkbox" id="trans_forwarded" data-width="100%" 
        data-style="" data-onlabel="<span class=\'fa-light fa-lock-open fa-fw\'></span> Forward request" data-offlabel="<span class=\'fa-light fa-lock fa-fw\'></span> Not Forwarded" />
        </div><div class="col-sm-12 mb-3">'.$app->renderInputTextarea('trans_forwarded_notes', $this->approved_notes, 'Enter optional forward notes here', 'line-controls').'</div></div>';
        return $html;
    }

    function renderTransferButton() {
        global $app;

        $html = '<div class="row">
        <div class="col-sm-6 my-3">
        <input name="trans_transferred" class="toggle-child bs-toggle w-100" type="checkbox" id="trans_transferred" data-width="100%" 
        data-style="" data-onlabel="<span class=\'fa-light fa-lock-open fa-fw\'></span> Transfer selected items" data-offlabel="<span class=\'fa-light fa-lock fa-fw\'></span> Do not transfer" />
        </div><div class="col-sm-12 mb-3">'.$app->renderInputTextarea('trans_transferred_notes', $this->transferred_notes, 'Enter optional transfare notes here', 'line-controls').'</div></div>';

        return $html;
    }

    function renderInsuranceControl($insurance_id=0) {
        global $app;
        if(!$insurance_id) $insurance_id = $this->getInsuranceByCode('CASH')->id;

        $html   = '<div class="trans-container"> <h5 class="trans-subtitle"><span class="fa-regular fa-tags fa-fw"></span> Sponsor: </h5>';
		$html   .= $app->renderInputSelect('trans_insurance_id', $insurance_id, '', 'line-control select-search onchange-reload-children '.$this->readonly().'', 'data-order=priority', 'billing_insurances', 'id', 'name');
		$html   .= '</div>';
        return $html;
    }

    function renderInsuranceSchemeControl($scheme_id='') {
        global $app;
        if(!$scheme_id) $scheme_id = $this->getSchemeByCode('CASH')->id;

        $scheme         = new Scheme($scheme_id);
        $insurance_id   = $scheme->insurance_id; 
        $html           = '<div class="trans-container"> <h5 class="trans-subtitle"><span class="fa-regular fa-tags fa-fw"></span> Scheme: </h5>';
        $html           .= $app->renderInputSelect('trans_scheme_id', $scheme_id, '', 'line-control select-search '.$this->readonly().'', 'data-parent="trans_insurance_id" data-source="html/transactions/schemes_select.php"', 'billing_insurance_schemes', 'id', 'name', "status = 1 AND insurance_id IN(SELECT id FROM billing_insurances WHERE status = 1)");
		$html           .= '</div>';
        return $html;
    }

    function renderPricelistControl($pricelist_id=0) {
        global $app;
        if(!$pricelist_id) $pricelist_id = $this->pricelist_id;

        $html   = '<div class="trans-container"> <h5 class=""> <span class="fa-regular fa-clock fa-fw"></span> Pricelist: </h5>';
		$html   .= $app->renderInputSelect('trans_pricelist_id', $pricelist_id, 'Choose pricelist', 'line-control select-search', '', 'billing_pricelists', 'id', 'name');
		$html   .= '</div>';
        $html   .= $this->renderPricelistsJson();
        return $html;
    }

    function renderServiceControl() {
        global $app;
        if(!$this->subtype) $this->subtype = 'DirectSale';
        
        $html   = '<div class="trans-container"> <h5 class="trans-subtitle"> <span class="fa-regular fa-user-nurse fa-fw"></span> Bill type: </h5>';
		$html   .= $app->renderInputSelect('trans_subtype', $this->subtype, 'Choose service type', 'line-control select-search onchange-reload-children '.$this->readonly().'', '', 'settings_categories', 'code', 'name', "type='SaleType'");
		$html   .= '</div>';
        return $html;
    }

    function renderUrgencyControl() {
        global $app;
        $html   = '<div class="trans-container mb-4"> <h5 class="trans-subtitle"> <span class="fa-regular fa-clock fa-fw"></span> &nbsp;Urgency: </h5>';
		$html   .= $app->renderInputSelect('trans_urgency', $this->urgency, 'Choose urgency', 'line-control select-search '.$this->readonly().'', '', 'settings_categories', 'code', 'name', "type='UrgencyType'");
		$html   .= '</div>';
        return $html;
    }

    function renderReferenceControl($title='Reference') {
        global $app;
        $html   = '<div class="trans-container mb-4"> <h5 class="trans-subtitle"> <span class="fa-regular fa-file-invoice fa-fw"></span> &nbsp;'.$title.': </h5>';
		$html   .= $app->renderInputText('trans_reference_no', $this->reference_no, 'Enter '.strtolower($title), 'line-control');
		$html   .= '</div>';
        return $html;
    }

    function renderDateControl($title='Delivery date') {
        global $app;
        $html   = '<div class="trans-container mb-4"> <h5 class="trans-subtitle"> <span class="fa-regular fa-calendar fa-fw"></span> '.$title.': </h5>';
		$html   .= $app->renderInputText('trans_date', $this->date, 'Pick '.strtolower($title), 'line-control date-picker');
		$html   .= '</div>';
        return $html;
    }

    function renderStoreControl($label='Store', $access='') {
        global $app, $user;

        if($this->id) $access = '';
        $stores     = $user->listStoresWithAccess($access);
        $store_id   = $this->store_id ? $this->store_id : $user->personnel()->store_id;

        $html       = '<div class="trans-container mb-4"> <h5 class="trans-subtitle"> <span class="fa-regular fa-store fa-fw"></span> '.$label.': </h5>';
		$html       .= $app->renderInputSelect('trans_store_id', $store_id, 'Choose '.strtolower($label), 'line-control select-search '.$this->readonly().'', 'onchange="transaction.initializeTransaction()" data-parent="trans_subtype" data-source="html/transactions/stores_select.php"', $stores);
		$html       .= '</div>';

        return $html;
    }

    function renderActionControl($actions = array()) {
        global $app;
        
        if(!$actions) $actions = array (
            array('name'=>'Accept request', 'value'=>'Accepted'),
            array('name'=>'Reject request','value'=>'Rejected')
        );

        $html = '<div class="trans-container mb-5"> <h5 class="trans-subtitle"><span class="fa-regular fa-bolt fa-fw"></span>&nbsp; Choose action: </h5>';
		$html .= $app->renderInputSelect('trans_action', '', 'Choose action', 'line-control select-search', '', $actions);
		$html .= '</div>';
        return $html;
    }

    function renderNotesControl($action='Requested', $control='textarea') {
        global $app;
        $notes_type = strtolower($action.'_notes');

        $html   = '<div class="trans-container my-4"> <h5 class="my-3"> <span class="fa-regular fa-note-sticky fa-fw"></span> &nbsp;Notes or comments: </h5>';
        if($control == 'text') {
            $html .= $app->renderInputText('trans_'.$notes_type, $this->$notes_type, '', 'line-control');
        } else {
            $html .= $app->renderInputTextarea('trans_'.$notes_type, $this->$notes_type, '', 'line-control');
        }
        $html .= '</div>';
        return $html;
    }

    function  renderInsuranceSchemesJson() {
        global $app;
        $sql			= "SELECT * FROM billing_insurance_schemes WHERE status = 1 AND insurance_id IN(SELECT id FROM billing_insurances WHERE status = 1)";
        $schemes 		= array();
        foreach($app->read($sql) as $row) {
            $schemes[$row->id]	= $row;
        }
        $html           = '<script> transaction.insurance_schemes = '.$app->array2json($schemes).' </script>';
        return $html;
    }

    function  renderPricelistsJson() {
        global $app;
        $sql			= "SELECT * FROM billing_pricelists ";
        $pricelists 	= array();
        foreach($app->read($sql) as $row) {
            $pricelists[$row->id]	= $row;
        }
        $html           = '<script> transaction.pricelists = '.$app->array2json($pricelists).' </script>';
        return $html;
    }

    function destroy() {
        foreach($this->items() as $item) {
            $trans_item     = new TransactionItem($this->type, $this->id, $item->id);
            $trans_item->destroy();
        }
        return parent::destroy();
    }

    function initializeTransaction() {
        return '<script> transaction.initializeTransaction();</script>';
    }

    function renderTransactionItemsEditable($title='Items in your cart') {
		global $app;
        
		$html = '<div class="bg-primary rounded p-4 mb-4" style="--bs-bg-opacity:0.1"> 
		<h5 class="title my-3 d-none"><span class="fa-regular fa-cart-circle-check fa-fw"></span> &nbsp;'.$title.':</h5>
        <div class="table-responsive">
		<table class="table table-hover align-middle bg-transparent">
		<thead class="h5">
		<tr>
		<th class="ps-0">Item name</th>
		<th class="w-12 text-center">Qty</th>
		<th class="w-18 text-end">Price</th>
		<th class="w-18 text-end">Amount</th>
		<th class="" style="width:10px"> <span class="fa-solid fa-ellipsis-h"></span></th>
		</tr>
		</thead>
		<tbody id="trans_items_container">
        
        <script>
        var items   = '.$this->renderCreateItemsJson().';
        transaction.initializeTransaction();
        transaction.loadStoreItems();
        for(var item_id in items) {
            transaction.addItemLine(items[item_id], true);
        }
        </script>

        </tbody>
		<tfoot>
		<tr class="">
		<th class="ps-0 fw-semibold">Subtotal</th>
		<th></th>
		<th></th>
		<th class="text-end">'.$app->renderInputText('trans_subtotal', $app->formatMoney($this->subtotal), 'Subtotal', 'line-control-plaintext text-end fw-semibold', 'readonly').'</th>
		<th></th>
        </tr>
		</tfoot>
		</table>
		</div>
        </div>';

        $html .= $app->renderInputHidden('trans_cost', '');

		return $html;
	}

	function renderTransactionItemsReadonly($title = 'Items in your cart') {
		global $app;

		$html = '<div class="trans-container my-4"> <h5 class="trans-subtitle"> <span class="fa-regular fa-cart-circle-check fa-fw"></span> &nbsp;'.$title.':</h5>
		<div class="card p-3 my-3 bg-primary" style="--bs-bg-opacity: .05;">
        <div class="table-responsive">
		<table class="table table-hover align-middle">
		<thead class="h5">
		<tr>
		<th class="ps-0">Item name</th>
		<th class="text-center w-12">Qty</th>
		<th class="w-20 text-end">Price</th>
		<th class="w-20 text-end">Amount</th>
		</tr>
		</thead>
		<tbody id="trans_items_container">';

		foreach($this->items() as $item) {
            $details = strlen($item->details) ? '<div class="text-muted"><small>'.$item->details.'</small></div>' : '';
			$html   .= '<tr>
			<td class="ps-0 py-3">'.$item->name.' <b class="text-muted">('.$item->pack.')</b> '.$details.'</td>
			<td class="text-center">'.$item->requested_quantity.'</td>
			<td class="text-end">'.$app->formatMoney($item->unit_price).'</td>
			<td class="text-end">'.$app->formatMoney($item->price).'</td>
			</tr>';
		}

		$html .= '</tbody>
		<tfoot>
		<tr>
		<th class="ps-0 fw-semibold">Subtotal</th>
		<th></th>
		<th></th>
		<th class="text-end">'.$app->renderInputText('trans_subtotal', $app->formatMoney($this->subtotal), 'Subtotal', 'line-control-plaintext text-end fw-semibold', 'readonly').'</th>
        </tr>
		</tfoot>
		</table>
		</div>
		</div>
        </div>';
		return $html;
	}

	function renderTransactionItemsWithBatchControl($title = 'Transfer items') {
		global $app;

		$html = '
		<div class="trans-container mb-4"> <h5 class="trans-subtitle"> <span class="fa-regular fa-cart-circle-check fa-fw"></span> &nbsp;'.$title.':</h5>
		<div class="card p-3 my-3 bg-primary" style="--bs-bg-opacity: .05;">
        <div class="table-responsive">
		<table class="table table-hover align-middle">
		<thead class="h5">
		<tr>
		<th class="w-5 p-1">
		<div class="form-check form-switch">
		<input role="button" class="form-check-input toggle-child" data-child-class="trans-item" type="checkbox" id="trans_check_all" checked checked onclick="return false" />
		</div>
		</th>
		<th class="">Item name</th>
		<th class="w-5 text-center">Qty</th>
		<th class="w-20 text">Batch</th>
		<th class="w-20 text">Expiry</th>
		</tr>
		</thead>

		<tbody id="trans_items_container">';

		foreach($this->items() as $item) {
			$item_obj = new Item($item->item_id);
			$html .= '<tr>
			<td class="p-0 ps-1 pt-1">
			<div class="form-check form-switch">
			<input name="trans_itemss[checked][]" value="'.$item->item_id.'" role="button" class="form-check-input trans-item" type="checkbox" id="trans_check_'.$item->item_id.'" checked onclick="return false" />
			</div>
			</td>
			<td class="">
            '.$item->name.' ('.$item->pack.')
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][transaction_item_id]', $item->id).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][item_id]', $item->item_id).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][name]', $item->name).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][uom]', $item->uom).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][pack]', $item->pack).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][size]', $item->size).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][unit_cost]', $item->unit_cost).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][unit_price]', $item->unit_price).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][quantity]', $item->quantity).'
            </td>
			<td class="text-center">'.$item->approved_quantity.'</td>
			<td>'.$app->renderInputSelect('trans_items['.$item->item_id.'][batch]', '', '', 'data-list', '', $item_obj->listBatch($this->store_id)).'</td>
			<td>'.$app->renderInputText('trans_items['.$item->item_id.'][expiry]', '', '', 'date-picker').'</td>
			</tr>';
		} 

		$html .= '</tbody>
		</table>
		</div>
		</div>
        </div>';

		return $html;
	}

    function renderTransactionItemsReceive() {
		global $app;

		$html = '<div class="trans-container">
		<h6 class="fw-bold"><span class="fa-regular fa-cart-circle-check fa-fw"></span> &nbsp;Items to be received:</h6>
		<div class="table-responsive">
		<table class="table table-hover align-middle">
		<thead>
		<tr>
        <th class="w-5">
        <div class="form-check form-switch" style="margin-bottom:-1px">
		<input role="button" class="form-check-input toggle-child" data-child-class="trans-item" type="checkbox" id="trans_check_all" />
		</div>
        </th> 
		<th class="ps-0">ITEM NAME (UNITS)</th>
		<th class="w-15 text-center">ORDERED</th>
		<th class="w-15 text-end">RECEIVED</th>
		<th class="w-15 text-end">RECEIVING</th>       
		</tr>
		</thead>
		<tbody id="trans_items_container">';
        
        foreach($this->items() as $item) {
            $item_obj   = new Item($item->item_id);
            $html       .= '<tr>
            <td class="align-top py-3">
			<div class="form-check form-switch">
			<input name="trans_items['.$item->item_id.'][checked]" value="'.$item->item_id.'" role="button" class="form-check-input trans-item" type="checkbox" id="trans_check_'.$item->item_id.'" />
			</div>
			</td>
            <td class="ps-0 py-3">
            <div class="pb-3">'.$item->name.' ('.$item->pack.')</div>
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][transaction_item_id]', $item->id).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][item_id]', $item->item_id).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][name]', $item->name).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][uom]', $item->uom).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][pack]', $item->pack).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][size]', $item->size).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][unit_cost]', $item->unit_cost).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][unit_price]', $item->unit_price).'
            <div class="row gx-3">
            <div class="col-6">'.$app->renderInputSelect('trans_items['.$item->item_id.'][batch]', '', 'Batch', 'data-list', '', $item_obj->listBatch($this->store_id)).'</div>
			<div class="col-6">'.$app->renderInputText('trans_items['.$item->item_id.'][expiry]', '', 'Expiry', 'date-picker').'</div>
            </div>
            </td>
            <td class="py-3 align-bottom">'.$app->renderInputNumber('trans_items['.$item->item_id.'][requested]', $item->requested, '', '').'</td>
            <td class="py-3 align-bottom">'.$app->renderInputNumber('trans_items['.$item->item_id.'][received]', $item->received_quantity, '', '').'</td>
            <td class="py-3 align-bottom">'.$app->renderInputNumber('trans_items['.$item->item_id.'][quantity]', ($item->requested_quantity - $item->received_quantity), '', '').'</td>
            </tr>';
        }

        $html .='</tbody>
		</table>
		</div>
		</div>';

		return $html;
	}

    function createGRN($options) {
        $obj                = new PurchaseGRN();
        $items              = $options['items'];
        $subtotal           = 0;
        $trans_items        = array();

        // Create GRN
        foreach($items as &$item) {
            $price          = $item['unit_price'] * $item['quantity'];
            $item['price']  = $price;
            $subtotal       += $price;
            $trans_items[]  = $item;
        }

        $create_data        = array (
            'type'          => 'PurchaseGRN',
            'urgency'       => $this->urgency,
            'store_id'      => $this->store_id,
            'name'          => $this->store()->location()->name,
            'items'         => json_encode($trans_items),
            'cost'          => $subtotal,
            'subtotal'      => $subtotal,
            'progeress'     => 'Requested',
            'progress_code' => 1
        );
        $transaction_id     = $obj->createWithMinimumData($create_data);

        // Register GRN items
        $transaction_item               = new TransactionItem('PurchaseGRN', $transaction_id);
        foreach($items as &$item) {
            $item['parent']        = 'PurchaseGRN';
            $item['parent_id']     = $transaction_id;
            $item['store_id']           = $this->store_id;
            $item['requested']          = $item['quantity'];
            $item['requested_quantity'] = $item['quantity'];

            $transaction_item->createWithMinimumData($item);
        }      
    }
    
	function renderItemsControls($transaction = 'DirectSale') {
		global $app;

        $html = $app->renderInputHidden('trans_items');

        $html .= $this->renderInsuranceSchemesJson();

		$html .= '<div class="row gx-3 mb-4">
        <div class="col-12 mt-3">
        <div class="">
        <h5 class="my-3 d-none"> <span class="fa-regular fa-cart-arrow-down fa-fw"></span>&nbsp; Add to cart: </h5>
        <div class="row gx-3">
        <div class="mb-3 col-12 col-lg-4">
        '.$app->renderInputSelect('type', $this->type, 'Filter categories', 'line-controlled select-search default-all', 'data-style="btn-outline-secondary"', array()).'
        </div><div id="trans_item_container" class="mb-3 col-12 col-lg-8">
        '.$app->renderInputSelect('trans_item_id', '', 'Choose item', 'line-controlled select-search default-none', 'data-style="btn-outline-secondary"', array()).'
        </div><div class="mb-3 col-12 col-lg-4">
        '.$app->renderInputNumber('trans_item_quantity', '', 'Quantity', 'line-controlled border border-1 border-secondary').'
        </div><div class="mb-3 col-12 col-lg-4" id="trans_item_uom_container">
        '.$app->renderInputSelect('trans_item_uom', '', 'Units', 'line-controlled select-search', 'data-style="btn-outline-secondary"', array()).'
        </div><div class="mb-3 col-12 col-lg-4">
        '.$app->renderInputMoney('trans_item_unit_price', '', 'Unit price', 'line-controlled text-end border border-1 border-secondary readonly').'
        </div><div class="mb-3 col-12 col-lg-8">
        '.$app->renderInputText('trans_item_details', '', 'Item specifications', 'line-controlled border border-1 border-secondary').'
        </div><div class="mb-3 col-12 col-lg-4">
        <a id="trans_item_add_button" class="btn btn-outline-primary w-100" onclick="transaction.addItemToCart()"> <span class="fa-solid fa-check fa-fw"></span> Add to cart </a>
        </div></div></div></div></div>';

		return $html;
	}

    function renderItemsButton() {
        global $app;
        $items      = count(json_decode($this->items));
        $html       =  '<a class="btn btn-sm btn-outline-secondary w-100 remote-modal text-decoration-none fw-bold" data-bs-toggle="modal" data-bs-target="#modal_lg" href="form.php?action=open&id='.$this->id.'">
        '.$this->progress.' '.$items.'/'.$items.'</a>';

        return $html;
    }

    function renderOptionsButtons() {
        global $app;
        $html       = '';

        if($this->progress == 'Requested') {
            $html   .=  '<a title="Authorize" class="remote-modal bs-tooltip btn btn-default" data-bs-toggle="modal" data-bs-target="#modal_lg" data-bs-backdrop="static" data-bs-keyboard="false" tabindex="-1" href="form.php?action=authorize&id='.$this->id.'" aria-label="Authorize"> <span class="fa-regular fa-lock-open"></span> </a>
            <a title="Edit" class="remote-modal bs-tooltip btn btn-default" data-bs-toggle="modal" data-bs-target="#modal_lg" data-bs-backdrop="static" data-bs-keyboard="false" tabindex="-1" href="form.php?action=edit&id='.$this->id.'" aria-label="Edit"> <span class="fa-regular fa-pen-to-square"></span> </a>';
        } elseif($this->progress == 'Authorized') {
            $html   .=  '<a title="Transfer" class="remote-modal bs-tooltip btn btn-default" data-bs-toggle="modal" data-bs-target="#modal_lg" data-bs-backdrop="static" data-bs-keyboard="false" tabindex="-1" href="form.php?action=transfer&id='.$this->id.'" aria-label="Transfer"> <span class="fa-regular fa-up-from-bracket"></span> </a>';
        }
    
        if($this->progress_code < 4) {
            $html   .=  '<a title="Delete" class="remote-modal bs-tooltip btn btn-default" data-bs-toggle="modal" data-bs-target="#modal_lg" href="form.php?action=delete&id='.$this->id.'" aria-label="Delete"> <span class="fa-regular fa-trash"></span> </a>';
        }

        return $html;
    }

    function formatCode() {
		$digits = 6;
		return sprintf('%0'.$digits.'d', $this->id);
	}

    function renderStores() {
        global $app, $user;
        $html   = '<div class="col-12 col-lg-6 mb-3"><div class="trans-container mb-4"> <h5 class="trans-subtitle"> <span class="fa-regular fa-store fa-fw"></span> Store:</h6>';
		$html   .= $app->renderInputSelect('trans_store_id', $this->store_id, 'Choose store', 'line-controlled select-search default-none', '', $user->listStores('Adjust'));
		$html   .= '</div></div>';
        return  $html;
    }

    function renderUrgency() {
        global $app, $user;
        $html   = '<div class="trans-container mb-4"> <h5 class="trans-subtitle"> <span class="fa-regular fa-bolt fa-fw"></span> Urgency:</h5>';
		$html   .= $app->renderInputSelect('trans_urgency', $this->urgency, 'Choose urgency', 'line-controlled select-search default-none', '', 'settings_categories', 'code', 'name', "type='UrgencyType'");
		$html   .= '</div>';
        return  $html;
    }

    function renderNotes() {
        global $app, $user;
        $html   = '<div class="col-12 my-4"><div class="trans-container"> <h5 class="my-3"> <span class="fa-regular fa-sticky-note fa-fw"></span> Notes or Comments:</h4>';
		$html   .= $app->renderInputTextarea('trans_created_notes', $this->requested_notes, '', 'line-control');
		$html   .= '</div></div>';
        return  $html;
    }

    function renderLinkAuthorize() {
        return '<a title="Authorize" class="remote-modal bs-tooltip btn btn-default" data-bs-toggle="modal" data-bs-target="#modal_lg" data-bs-backdrop="static" data-bs-keyboard="false" tabindex="-1" href="form.php?action=authorize&id='.$this->id.'" aria-label="Authorize"> <span class="fa-regular fa-lock-open"></span> </a>';
    }
    
    function renderLinkApprove() {
        return '<a title="Approve" class="remote-modal bs-tooltip btn btn-default" data-bs-toggle="modal" data-bs-target="#modal_lg" data-bs-backdrop="static" data-bs-keyboard="false" tabindex="-1" href="form.php?action=approve&id='.$this->id.'" aria-label="Authorize"> <span class="fa-regular fa-check-double"></span> </a>';
    }

    function renderLinkTransfer() {
        return '<a title="Transfer" class="remote-modal bs-tooltip btn btn-default" data-bs-toggle="modal" data-bs-target="#modal_lg" data-bs-backdrop="static" data-bs-keyboard="false" tabindex="-1" href="form.php?action=transfer&id='.$this->id.'" aria-label="Transfer"> <span class="fa-regular fa-upload"></span> </a>';
    }

    function renderLinkConfirm() {
        return '<a title="Confirm" class="remote-modal bs-tooltip btn btn-default" data-bs-toggle="modal" data-bs-target="#modal_lg" data-bs-backdrop="static" data-bs-keyboard="false" tabindex="-1" href="form.php?action=confirm&id='.$this->id.'" aria-label="Confirm"> <span class="fa-regular fa-arrow-up-from-bracket"></span> </a>';
    }

    function renderLinkReceive() {
        return '<a title="Receive" class="remote-modal bs-tooltip btn btn-default" data-bs-toggle="modal" data-bs-target="#modal_lg" data-bs-backdrop="static" data-bs-keyboard="false" tabindex="-1" href="form.php?action=receive&id='.$this->id.'" aria-label="Receive"> <span class="fa-regular fa-inbox-in"></span> </a>';
    }

    function renderLinkEdit() {
        return '<a title="Edit" class="remote-modal bs-tooltip btn btn-default" data-bs-toggle="modal" data-bs-target="#modal_lg" data-bs-backdrop="static" data-bs-keyboard="false" tabindex="-1" href="form.php?action=edit&id='.$this->id.'" aria-label="Edit"> <span class="fa-regular fa-pen-to-square"></span> </a>';
    }

    function renderLinkCopy() {
        return '<a title="Copy" class="remote-modal bs-tooltip btn btn-default" data-bs-toggle="modal" data-bs-target="#modal_lg" data-bs-backdrop="static" data-bs-keyboard="false" tabindex="-1" href="form.php?action=copy&id='.$this->id.'" aria-label="Edit"> <span class="fa-regular fa-copy"></span> </a>';
    }

    function renderLinkDelete() {
        return '<a title="Delete" class="remote-modal bs-tooltip btn btn-default" data-bs-toggle="modal" data-bs-target="#modal_md" href="form.php?action=delete&id='.$this->id.'" aria-label="Delete"> <span class="fa-regular fa-trash"></span> </a>';
    }

    function renderLinkCancel() {
        return '<a title="Cancel" class="remote-modal bs-tooltip btn btn-default" data-bs-toggle="modal" data-bs-target="#modal_lg" href="form.php?action=cancel&id='.$this->id.'" aria-label="Cancel"> <span class="fa-regular fa-trash-can-xmark"></span> </a>';
    }

    function renderLinkForward($title='Forward') {
        return '<a title="Create '.$title.'" class="remote-modal bs-tooltip btn btn-default" data-bs-toggle="modal" data-bs-target="#modal_lg" data-bs-backdrop="static" data-bs-keyboard="false" tabindex="-1" href="form.php?action=forward&id='.$this->id.'&title='.$title.'" aria-label="'.$title.'"> <span class="fa-regular fa-forward"></span> </a>';
    }

    function renderLinkPrint() {
        return '<a title="Print" target="blank" class="bs-tooltip btn btn-default" href="print.php?action=print&id='.$this->id.'" aria-label="Print"> <span class="fa-regular fa-print"></span> </a>';
    }

    function renderLinkExtend() {
        return '<a title="Extend delivery" class="remote-modal bs-tooltip btn btn-default" data-bs-toggle="modal" data-bs-target="#modal_lg" data-bs-backdrop="static" data-bs-keyboard="false" tabindex="-1" href="form.php?action=extend&id='.$this->id.'" aria-label="Extend"> <span class="fa-regular fa-calendars"></span> </a>';
    }

    function renderButtonUrgency() {
        $urgency    = $this->urgency();
        return '<span class="text-'.$urgency->color.'"> <span class="'.$urgency->icon.' fa-fw"></span> '.$this->urgency.'</span>';
    }

    function customer() {
        return new Customer($this->personnel_id);
    }

    function getCostPricelistId() {
        global $app;
        $pricelist_id  = $app->getColumnValue('billing_pricelists', 'id', 'code', 'COST');
        return $pricelist_id;
    }

    function adjustItemStock(array $data) {
        $item_data  = $data['item'];
        $adjust     = $data['adjust'];

        $item       = new Item($item_data->item_id);
        return $item->adjustStock($data);
    }

    function renderAuthorizeConfirmation($submodule, $post) {
        global $app, $user;

        if($post->trans_action == 'Authorized') {
            if($this->changeProgress('Authorized', 'requested_quantity', $post->trans_authorized_notes)) {
                $this->updateInlineItems();
                return $app->renderAlertSuccess($submodule->shortname.' authorized successfully', 'Success', 'modal_lg', 1);
            }
        } elseif($post->trans_action == 'Rejected') {
            if($this->changeProgress('Requested', 'requested_quantity', $post->trans_authorized_notes)) {
                return $app->renderAlertSuccess($submodule->shortname.' returned to request stage', 'Success', 'modal_lg', 1);
            }
        } else {
            die($app->renderAlertError('Please choose action'));
        }
    }

    function renderApproveConfirmation($submodule, $post) {
        global $app, $user;

        if($post->trans_action == 'Approved') {
            if($this->changeProgress('Approved', 'authorized_quantity', $post->trans_approved_notes)) {
                $this->updateInlineItems();
                return $app->renderAlertSuccess($submodule->shortname.' approved successfully', 'Success', 'modal_lg', 1);
            }
        } elseif($post->trans_action == 'Rejected') {
            if($this->changeProgress('Requested', 'requested_quantity', $post->trans_approved_notes)) {
                return $app->renderAlertSuccess($submodule->shortname.' returned to request stage', 'Success', 'modal_lg', 1);
            }
        } else {
            die($app->renderAlertError('Please choose action'));
        }
    }

    function user($action) {
        $user_type = $action.'_user_id';
        return new User($this->$user_type);
    }

    function renderPDF($title='Transaction') {
        global $root, $app;

        $title_upper    = strtoupper($title);

        $html   = '<table width="100%; padding:2px">
        <tr>
        <td width="15%">
        <img src="'.$root.CLIENT_PATH.'images/urt.png" width="90px" />
        </td>
        <td width="70%" style="text-align:center">

        <span style="font-size:1.2em"><b>THE UNITED REPUBLIC OF TANZANIA
        <br /> MINISTRY OF HEALTH</b><br /></span>

        <span style="text-align:center; font-size:1.5em"><b>MUHIMBILI ORTHOPAEDIC INSTITUTE (MOI)</b></span>

        <div style="text-align:center; color:rgb(100,100,100); font-size:1.1em">Offering Services in Orthopaedics, Neurosurgey and Traumatology</div><br />

        </td>
        <td width="15%" align="right">
        <img src="'.$root.CLIENT_PATH.'images/logo.png" width="90px" />
        </td>
        </tr>
        </table>
    
    <p style="border-top:2px solid rgb(200,200,200)"></p>';

    $html .= '<table class="w-100 pb-2">
    <tr>
    <td class="w-50"><h2>Internal Stock Requisition</h2></td>
    <td class="w-50 text-end"><h2>No: '.$this->formatCode($this->id).'</h2></td>
    </tr>
    </table>';

    $html .= '<table class="border-muted p-5 text-success">
    <tr><td class="bg-light"><b>'.$title_upper.' DETAILS</b></td></tr>
    <tr><td class="w-20 bg-white">Requisition no :</td><td class="w-80 bg-white text-dark"><b>'.$this->formatCode($this->id).'</b></td></tr>
    <tr><td class="w-20 bg-white">Store name :</td><td class="w-80 bg-white text-dark"><b>'.$this->store()->location()->name.'</b></td></tr>
    </table>';

    $html .= '<table class="border-muted p-5">
    <tr><td class="bg-light"><b>'.$title_upper.' DETAILS</b></td></tr>
    <tr><td class="w-20 bg-white">Requisition no :</td><td class="w-80 bg-white text-dark"><b>'.$this->formatCode($this->id).'</b></td></tr>
    <tr><td class="w-20 bg-white">Store name :</td><td class="w-80 bg-white text-dark"><b>'.$this->store()->location()->name.'</b></td></tr>
    </table>';
     
/*
        $html   .= '<div class="body">';

        $html   .= '<div style="text-align:center"><b>'.strtoupper($this->type).'</b></div>';


        $html   .= '</div>';
        */

        //echo $html;

        $app->renderPDF($html, $this->type);
    }

    function filters() {
        global $app;

        $filters            = array (
            array (
                'css'       => 'col-xl-4 col-lg-6 mt-2 mb-3',
                'type'      => 'search'
            ), 
            array (
                'css'       => 'col-6 col-lg-3 col-xl-2 mt-2 mb-3',
                'control'   => $app->renderInputDate('start_date', '', 'Start date', 'filter-control form-control-lg')
            ),
            array (
                'css'       => 'col-6 col-lg-3 col-xl-2 mt-2 mb-3',
                'control'   => $app->renderInputDate('end_date', '', 'End date', 'filter-control form-control-lg')
            ),
            array (
                'type'      => 'buttons',
                'control'   => array (
                    array('type'=>'refresh'),
                    array('type'=>'create', 'url'=>'form.php')
                )
            ),
            array (
                'control'   => $app->renderInputSelect('department', '', 'Choose department', 'filter-control form-control-lg default-all select-search onchange-reload-children', 'data-style="btn-outline-secondary btn-lg"', 'location_departments,location_locations,location_departments.location_id=location_locations.id', 'location_departments.id', 'location_locations.name', '', 'Department')
            ),
            array (
                'control'   => $app->renderInputSelect('store', '', 'Choose store', 'filter-control form-control-lg default-all select-search onchange-reload-list', 'data-parent="department" data-source="pages/procurement/stores_select.php" data-style="btn-outline-secondary btn-lg"', 'location_stores,location_locations,location_stores.location_id=location_locations.id', 'location_stores.id', 'location_locations.name', 'location_stores.id = 0', 'Store')
            ),
            array (
                'control'   => $app->renderInputSelect('progress', '', 'Choose status', 'filter-control default-all onchange-reload-list', 'data-style="btn-outline-secondary btn-lg"', 'settings_categories', 'code', 'name', "type='TransactionProgress'")
            )
        );
        return $filters;
    }

    function renderFormProcess($submodule, $post) {
        global $app, $user;

        if($post->action == 'delete') {

            // $user->checkStoreAccess($store_id, 'delete');

            $deleted    = $this->renderDeleteConfirmation($submodule);
            $this->timeline(array('action'=>'Delete', 'details'=>'Deleted '.strtolower($this->type)));
            return $deleted;

        } elseif($post->action == 'create' || $post->action == 'copy' || $post->action == 'edit' || $post->action == 'return') {
            $this->validateInputs($post);
            if($post->action == 'create' || $post->action == 'copy') {
                $created    = $this->renderCreateConfirmation($submodule, $post);
                $this->timeline(array('action'=>'Created', 'details'=>'Created new '.strtolower($this->type)));
                return $created;

            } elseif($post->action == 'edit') {  
                $updated     = $this->renderEditConfirmation($submodule, $post);
                $this->timeline(array('action'=>'Modified', 'details'=>'Updated '.strtolower($this->type)));
                return $updated;

            } elseif($post->action == 'return') {  
                $returned   = $this->renderReturnConfirmation($submodule, $post);
                $this->timeline(array('action'=>'Return', 'details'=>'Returned '.strtolower($this->type)));
                return $returned;

            }
        } elseif($post->action == 'authorize') {
            $authorized     = $this->renderAuthorizeConfirmation($submodule, $post);
            $this->timeline(array('action'=>'Return', 'details'=>'Returned '.strtolower($this->type)));
            return $authorized;

        } elseif($post->action == 'approve') {
            $approved       = $this->renderApproveConfirmation($submodule, $post);
            $this->timeline(array('action'=>'Approve', 'details'=>'approved '.strtolower($this->type)));
            return $approved;

        } elseif($post->action == 'extend') {
            $extended       = $this->renderExtendConfirmation($submodule, $post);
            $this->timeline(array('action'=>'Updated', 'details'=>'Updated '.strtolower($this->type)));
            return $extended;

        } elseif($post->action == 'forward') {
            $forwarded      = $this->renderForwardConfirmation($submodule, $post);
            $this->timeline(array('action'=>'Forward', 'details'=>'Forwarded '.strtolower($this->type)));
            return $forwarded;

        } elseif($post->action == 'transfer') {
            $transferred    = $this->renderTransferConfirmation($submodule, $post);
            $this->timeline(array('action'=>'Transfer', 'details'=>'Transferred '.strtolower($this->type)));
            return $transferred;

        } elseif($post->action == 'receive') {
            $received       = $this->renderReceiveConfirmation($submodule, $post);
            $this->timeline(array('action'=>'Receive', 'details'=>'Received '.strtolower($this->type)));
            return $received;

        } elseif($post->action == 'bill') {
            $billed         = $this->renderBillConfirmation($submodule, $post);
            $this->timeline(array('action'=>'Bill', 'details'=>'Billed '.strtolower($this->subtype)));
            return $billed;

        } elseif($post->action == 'pay') {
            $paid           = $this->renderPaymentConfirmation($submodule, $post);
            $this->timeline(array('action'=>'Pay', 'details'=>'Received '.strtolower($this->subtype).' payment'));
            return $paid;

        } elseif($post->action == 'dispense') {
            $dispensed      = $this->renderDispenseConfirmation($submodule, $post);
            $this->timeline(array('action'=>'Dispense', 'details'=>'Dispensed '.strtolower($this->subtype)));
            return $dispensed;

        } elseif($post->action == 'cancel') {
            $cancelled      = $this->renderCancelConfirmation($submodule, $post);
            $this->timeline(array('action'=>'Cancel', 'details'=>'Cancelled '.strtolower($this->type)));
            return $cancelled;

        } else {
            return $app->renderAlertError('Unknown action '.$post->action.' when processing transaction');
        }
    }

    function validateInputs($post) {
        global $app;

        $store_id   = $app->validate(array('name'=>'trans_store_id', 'message'=>'Please choose store', 'value'=>$post->trans_store_id, 'min'=>1));
        $items      = $app->uncleanText($post->trans_items);
        if(!json_decode($items)) die($app->renderAlertError('Please add atleast one item to your cart.'));
    }

    function subtype() {
        global $app;
        return $app->getCategoryByCode('SaleType', $this->subtype);
    }

    function renderTrackingButton() {
        global $app;
        $progress       = $app->getCategoryByCode('TransactionProgress', $this->progress);
        $code           = $this->progress == 'Paid' ? 'Payment' : $this->progress;
        $user           = strtolower($code.'_user_id');
        $user           = new User($this->$user);
        $datetime       = strtolower($code.'_datetime');
        $datetime       = $this->$datetime;
        $date           = date('Y-m-d', strtotime($datetime));
        if($date == date('Y-m-d')) $date = 'Today';
        if($date == date('Y-m-d', strtotime('Yesterday'))) $date = 'Yesterday';
        
        $title_label    = '<b>'.$this->type.' '.$this->progress.'</b>';
        $title_label    .= '<div class="px-4"><small class="text-muted">By '.$app->shortname($user->person()->name, 15).' '.$date.' at '.date('H:i', strtotime($datetime)).'</small></div>';

        $button_label   = '<span class="fa-regular fa-magnifying-glass fa-fw"></span> Track '.$this->type.' <span class="fa-solid fa-caret-down"></span>';
        $summary        = '<div class="row">
        <div class="col-lg-6 py-2"><span class="fa-solid fa-check-circle fa-fw"></span> '.$title_label.'</div>
        <div class="col-lg-6 py-3"><a onclick="transaction.loadTrackingDetails(\''.$this->type.'\', '.$this->id.')" class="btn btn-outline-primary w-100 fw-semibold">'.$button_label.'</a></div>
        </div><div id="trans_tracking_container"></div>';

        $html = '<div class="rounded p-4 bg-primary bg-opacity-10 mb-3">'.$summary.'</div>';
        return $html;
    }

    function checkStockAvailability($post) {
        global $app;

        $items                  = json_decode($app->uncleanText($post->trans_items));
        $scheme                 = new Scheme($post->trans_scheme_id);
        $insurance              = new Insurance($post->trans_insurance_id);

        foreach($items as $row) {
            $item               = new Item($row->item_id);
            $requested_stock    = $row->qty;
            $available_stock    = $item->getStock($post->trans_store_id);

            if($row->price < 0 && $this->type != 'Wallet' && $post->trans_subtype != 'Wallet' && $this->type != 'SaleReturn') {
                die($app->renderAlertError('Price cannot be negative', $item->name));
            }

            if(($item->type == 'Product') && ($requested_stock > $available_stock)) {
                die($app->renderAlertError($item->name.' requested '.$requested_stock.' out of '.$available_stock, 'Out of stock'));
            }

            if($insurance->code == 'NHIF') {
                $customer       = new Customer($post->trans_customer_id);
                $product        = $customer->product;
                // $product        = 'NH003';
                $pricelist      = $item->getPriceDetails($scheme->pricelist_id);

                if(strlen($product) > 2 && strpos($pricelist->excluded, $product) !== false) {
                    die($app->renderAlertError($item->name.' is excluded for patients with product code <b>'.$product.'</b>', 'Excluded service'));
                }
            }
        }
    }


    function readonly() {
        return $this->id ? 'readonly' : '';
    }

    function timeline($data = array()) {
        global $user;

        // Log event
        $user->timeline (
            array (
                'action'			=> $data['action'],
                'subject'			=> $this->type,
                'subject_id'		=> $this->id,
                'personnel'			=> $this->personnel,
                'personnel_id'		=> $this->personnel_id,
                'name'	            => $this->name,
                'details'			=> $data['details'],
                'visit_id'          => $this->visit_id,
                'visit_location_id' => $this->visit_location_id
            )
        );
    }

    function visit() {
        return new Visit($this->visit_id);
    }
}

?>