<?php 

class Sale extends Transaction {
	function __construct($id=0) {	
		global $app;
		parent::__construct($id);
	}

	function renderLinkBill() {
        return '<a title="Bill" class="remote-modal bs-tooltip btn btn-default" data-bs-toggle="modal" data-bs-target="#modal_lg" data-bs-backdrop="static" data-bs-keyboard="false" tabindex="-1" href="form.php?action=bill&id='.$this->id.'" aria-label="Bill"> <span class="fa-regular fa-memo-circle-check"></span> </a>';
    }

	function renderLinkInvoice() {
        return '<a title="Print Invoice" class="bs-tooltip btn btn-default" target="blank" href="print_invoice.php?id='.$this->id.'" aria-label="Invoice"> <span class="fa-regular fa-file-invoice-dollar"></span> </a>';
    }

	function renderLinkReceipt() {
        return '<a title="Print Receipt" class="bs-tooltip btn btn-default" target="blank" href="print.php?id='.$this->id.'" aria-label="Receipt"> <span class="fa-regular fa-receipt"></span> </a>';
    }

	function renderLinkDispense() {
        return '<a title="Dispense" class="remote-modal bs-tooltip btn btn-default" data-bs-toggle="modal" data-bs-target="#modal_lg" data-bs-backdrop="static" data-bs-keyboard="false" tabindex="-1" href="form.php?action=dispense&id='.$this->id.'" aria-label="Dispense"> <span class="fa-regular fa-arrow-up-from-bracket"></span> </a>';
    }

	function customer() {
		return new Customer($this->personnel_id);
	}

	function renderTransactionHeader() {

		$insurance_id 	= $this->insurance_id ? $this->insurance_id : $this->customer()->insurance_id;
		$scheme_id 		= $this->scheme_id ? $this->scheme_id : $this->customer()->scheme_id;
		
		$html = $this->id ? $this->renderTrackingButton() : '';
		$html .= '<div class="bg-primary bg-opacity-10 rounded p-4 pb-0 mb-4">';
		$html .= '<div class="row">';
		$html .= '<div class="col-12 col-lg-6 mb-4">'.$this->renderInsuranceControl($insurance_id).'</div>';
		$html .= '<div class="col-12 col-lg-6 mb-4">'.$this->renderInsuranceSchemeControl($scheme_id).'</div>';
		$html .= '<div class="col-12 col-lg-6 mb-4">'.$this->renderServiceControl().'</div>';
		$html .= '<div class="col-12 col-lg-6 mb-4">'.$this->renderStoreControl('Store', 'create-sales').'</div>';
		$html .= '</div>';
		$html .= '</div>';
		return $html;
	}

	function createCustomerIfNotExists($data) {
        $customer = new Customer();
        return $customer->createIfNotExists($data);
    }

	function folio() {
		return new Folio();
	}

	function renderModalForm($submodule, $obj, $get, $form_action='process.php') {
		global $app;

		$html				= '';
		$this->shortname 	= $submodule->shortname;

		if($get->action == 'delete') {
			$submodule->shortname = $this->shortname;
			$html .= $submodule->renderModalDelete($get->id);

		} else {

			if($get->action == 'bill') {
				$title = 'Create Invoice';
			} elseif($get->action == 'create') {
				$title = 'Create '.strtolower($this->shortname);
			} else {
				$title = ucfirst($get->action).' '.$this->shortname;
			}

			$modal_options  = array(
				'name'      => $this->shortname,
				'title'     => $title,
				'form_action'	=> $form_action,
				'left_html'	=> '<div id="customer_bio_container">'.$this->renderCustomerBio($this->name).'</div>'
			);

			$html .= $submodule->renderModalFormOpen($modal_options);
			$html .= $app->renderInputHidden('action', $get->action);
			$html .= $app->renderInputHidden('id', $get->id);	

			if($get->action == 'create' || $get->action == 'copy' || $get->action == 'edit') {
				require('customers_form.php');
			} elseif($get->action == 'bill') {
				$html .= $obj->renderTransactionHeader();
				$html .= $obj->renderTransactionItemsBill();
				$html .= $obj->renderNotesControl('Billed');
			} elseif($get->action == 'pay') {
				$html .= $obj->renderTransactionHeader();
				$html .= $obj->renderTransactionItemsReadonly('Billed items');
				$wallet_balance 	= $this->customer()->wallet()->subtotal;
				$channels 	= array();

				if($this->insurance == 'CASH') {
					if($this->subtype == 'Wallet') {
						if($this->subtotal > 0) {
							$html .= '<div class="my-4">'.$app->renderAlertWarning('<span class="h5 float-end fw-bold">TZS '.number_format($wallet_balance,2).'</span>', 'Wallet cannot be used on this transaction').'</div>';
						} else {
							$channels 	= array (
								array('name'=>'Wallet', 'value'=>'WALLET')
							);
							$html .= '<div class="my-4">'.$app->renderAlertSuccess('<span class="h5 float-end fw-bold">TZS '.number_format($wallet_balance,2).'</span>', 'Wallet must be used on this transaction').'</div>';
						}
					} elseif($wallet_balance >= $this->subtotal) {
						$html .= '<div class="my-4">'.$app->renderAlertSuccess('<span class="h5 float-end fw-bold">TZS '.number_format($wallet_balance,2).'</span>', 'Wallet balance').'</div>';
					} else {
						$html .= '<div class="my-4">'.$app->renderAlertWarning('<span class="h5 float-end fw-bold">TZS '.number_format($wallet_balance,2).'</span>', 'Insufficient wallet balance').'</div>';
					}
				} else {
					$html .= '<div class="my-4">'.$app->renderAlertWarning('<span class="h5 float-end fw-bold">TZS '.number_format($wallet_balance,2).'</span>', 'Wallet cannot be used on this transaction').'</div>';
				}

				$html .= '<div class="row">';
				$html .= '<div class="col-12 col-lg-6 my-4">'.$this->renderPaymentChannelControl($channels).'</div>';
				$html .= '<div class="col-12 col-lg-6 my-4">'.$this->renderAccountControl().'</div>';
				$html .= '</div>';
				$html .= $obj->renderNotesControl('Payment'); 
			} elseif($get->action == 'cancel') {
				$html .= $obj->renderTransactionHeader();
				$html .= $obj->renderTransactionItemsReadonly();
				$html .= $obj->renderNotesControl('Returned');
			} elseif($get->action == 'dispense') {
				$html .= $obj->renderTransactionHeader();
				$html .= $obj->renderTransactionItemsDispense('Billed items');
				$html .= $obj->renderNotesControl('Transferred');
			} elseif($get->action == 'open') {
				$html .= $obj->renderTransactionHeader();
				$html .= $obj->renderTransactionItemsReadonly();
			}

			$html     .= $submodule->renderModalFormClose();
		}
		return $html;	
	}

	function renderCreateConfirmation($submodule, $post) {
        global $app, $user, $now;

		$services = array ('Laboratory', 'Radiology', 'Procedure');
		if(app_name == 'emedical' && in_array($post->trans_subtype, $services)) {
			$service_type	= $post->trans_subtype;
			$service_obj 	= new $service_type();
			$customer		= new Customer($post->trans_customer_id);
			$patient 		= $customer->patient();
			$visit_location = $patient->visitLocation();
			$visit_id 		= isset($post->visit_id) ? $post->visit_id : $visit_location->visit_id;
			$visit			= new Visit($visit_id);
			$folio_id 		= isset($post->folio_id) ? $post->folio_id : $visit->folio_id;
			$folio 			= new Folio($folio_id);

			if($visit_location->id || isset($post->visit_id, $post->folio_id)) {
				$post->patient_id 			= $patient->id;
				$post->visit_id 			= $visit_id;
				$post->visit_location_id 	= $visit_location->id;
				return $service_obj->renderCreateConfirmation($submodule, $post);
			} else {
				$service_name = $service_type == 'Pharmacy' ? 'Prescription' : $service_type;
				die($app->renderAlertError($service_name.' should be requested from the patient file. Use direct sale.'));
			}
		} else {
			$customer		= new Customer($post->trans_customer_id);
			$patient 		= $customer->patient();
			$visit_location = $patient->visitLocation();
			$visit_id 		= isset($post->visit_id) ? $post->visit_id : $visit_location->visit_id;
			$visit			= new Visit($visit_id);
			$folio_id 		= isset($post->folio_id) ? $post->folio_id : $visit->folio_id;
			$folio 			= new Folio($folio_id);
		}

        $user->checkSubmoduleAccess($submodule, 'create');

        $items          = $app->uncleanText($post->trans_items);
		$store          = new Store($post->trans_store_id);
		$customer       = new Customer($post->trans_customer_id);
		$patient        = $customer->patient();
		$customer_name  = $post->customer_keyword;
		$insurance      = new Insurance($post->trans_insurance_id);
		$scheme         = new Scheme($post->trans_scheme_id);
		$pricelist      = new Pricelist($scheme->pricelist_id);
		//$visit          = $patient->visit();
		$location       = $patient->location();
		//$folio          = $visit->folio();
		$items_obj 		= json_decode($items);
		if(!$items_obj) die($app->renderAlertError('Please add at least one item'));

		$this->checkStockAvailability($post);

		$customer               = $this->createCustomerIfNotExists(
			array (
				'id'            => $post->trans_customer_id,
				'name'          => $post->trans_customer_name,
				'phone'         => $post->trans_customer_phone,
				'insurance_id'  => $post->trans_insurance_id,
				'scheme_id'     => $post->trans_scheme_id
			)
		);

        $create_data            = array (
			'type'              => $this->type,
			'subtype'           => $post->trans_subtype,
			'store_id'          => $post->trans_store_id,

			'personnel'         => 'Customer',
			'personnel_id'      => $post->trans_customer_id,
			'code'              => $customer->code,
			'name'              => $post->trans_customer_name,
			'birthdate'         => $post->trans_customer_birthdate,
			'sex'               => $post->trans_customer_sex,
			'phone'             => $post->trans_customer_phone,
			'email'             => $customer->person()->email,

			'pricelist'         => $pricelist->code,
			'pricelist_id'      => $pricelist->id,
			'insurance'         => $insurance->code,
			'insurance_id'      => $insurance->id,
			'scheme'            => $scheme->code,
			'scheme_id'         => $scheme->id,

			'folio_id'          => $folio->id,
			'folio_no'          => $folio->folio_no,

			'visit_id'          => $visit->id,
			'visit_location_id' => $location->id,

			'requested_notes'   => $post->trans_requested_notes,
			'items'             => $items,
			'cost'              => $post->trans_cost,
			'subtotal'          => $post->trans_subtotal,
			'progeress'         => 'Requested',
			'progress_code'     => 1
		);

        $transaction_id         = $this->createWithMinimumData($create_data);
		$this->id 				= $transaction_id;
		$this->read();

        if($transaction_id) {
			
			// Auto bill?
			if(isset($post->trans_auto_bill)) {
				$invoice 					= new Invoice();
				$invoice_id     			= $invoice->createWithMinimumData($create_data);
			}

			// Auto pay?
			if(isset($post->trans_auto_pay)) {
				
				$invoice 					= new Invoice($invoice_id);
				$payment_data				= array (
					'payment_method'		=> 'CASH',
					'payment_channel'		=> 'CASH',
					'payment_account'		=> 'CASH',
					'payment_account_id'	=> 13,
					'payment_receipt'		=> $invoice->id,
					'payment_name'			=> $invoice->name,
					'payment_phone'			=> $invoice->phone,
					'payment_notes'			=> 'Direct payment',
					'items'					=> $invoice->items()
				);
				
				$payment_id 				= $invoice->pay($payment_data);
				$invoice->payTransactionItems($payment_data);
				$invoice->changeProgress('Paid');

				$bill_obj 					= new Bill($transaction_id);
				$bill_obj->payFromInvoice($this->id);
				$bill_obj->update(
					array (
						'payment'				=> 1,
						'paid'					=> $bill_obj->subtotal
					)
				);
				$this->changeProgress('Paid');
			}

			// Auto dispense?
			if(isset($post->trans_auto_dispense)) {

				$dispensed_items     		= $this->items();
				$dispensed_item_ids			= array_keys($dispensed_items);
				$dispensing_items    		= array();

				foreach($dispensed_item_ids as $dispensed_item_id) {
					$dispensed_item        			= (array) $dispensed_items[$dispensed_item_id];
					$dispensed_item['store_id']     = $this->store_id;
					$dispensed_item['parent']    	= $this->type; 
					$dispensed_item['parent_id'] 	= $this->id; 
					$dispensing_items[]         	= $dispensed_item;
				}

				// Adjust stock from source store
				foreach($dispensing_items as $adjust_options) {
					$item_obj                   	= new Item($adjust_options['item_id']);
					$transaction_item           	= new TransactionItem('Bill', $this->id, $adjust_options['id']);
					$adjust_options['adjust']   	= true;
					$adjust_options['item']     	= $transaction_item;
					$quantity 						= $adjust_options['quantity'];

					// Source store
					$adjust_options['store_id']     = $this->store_id;
					$adjust_options['quantity']     = $quantity * -1;
					$adjusted                   	= $item_obj->adjustStock($adjust_options);

					$dispense_data 					= array (
						'progress'          		=> 'Dispensed',
						'progress_code'         	=> 10,
						'dispensed'          		=> 1,
						'dispensed_quantity' 		=> $transaction_item->dispensed_quantity + $quantity,
						'dispensed_user_id'  		=> $user->id,
						'dispensed_datetime' 		=> $now
					);

					$transaction_item->update($dispense_data);

					$all_dispensed = true;
					foreach($this->items() as $item) {
						if($item->dispensed_quantity < $item->quantity) $all_dispensed = false;
					}
			
					if($all_dispensed) {
						$update_data = array (
							'progress'				=> 'Dispensed',
							'progress_code'         => 10,
							'progress_code'			=> 10,
							'dispensed_user_id'		=> $user->id,
							'dispensed_datetime'	=> $now
						);
						$this->update($update_data);

						if($this->subtype == 'Pharmacy' || $this->subtype == 'Prescription' || $this->subtype == 'Consumable') {
							$subtype 					= $this->subtype == 'Consumable' ? $this->subtype : 'Prescription';
							$request 					= new $subtype($this->subtype_id);
							$update_data = array (
								'progress'				=> 'Dispensed',
								'progress_code'         => 10,
								'progress_code'			=> 10
							);
							$request->update($update_data);
						}
					}

					$this->updateInlineItems();
					if($this->subtype == 'Pharmacy' || $this->subtype == 'Prescription' || $this->subtype == 'Consumable') {
						$subtype 					= $this->subtype == 'Consumable' ? $this->subtype : 'Prescription';
						$request 					= new $subtype($this->subtype_id);
						$request->updateInlineItems();
					}

				}

				echo $app->renderAlertSuccess('Items dispensed successfully', 'Success', 'modal_lg', 1);
			}

			$_SESSION['bill_id']		   =  $transaction_id ;
            return $app->renderAlertSuccess($submodule->shortname.' created successfully', 'Success', 'modal_lg', true);
        }
    }

	function renderEditConfirmation($submodule, $post) {
        global $app, $user;

        $user->checkSubmoduleAccess($submodule, 'edit');   

        $items                  = $app->uncleanText($post->trans_items);
        $store                  = new Store($post->trans_store_id);
		$insurance 				= new Insurance($post->trans_insurance_id);
		$scheme 				= new Insurance($post->trans_scheme_id);
		$items_obj 				= json_decode($items);
		if(!$items_obj) die($app->renderAlertError('Please add at least one item'));

		$this->checkStockAvailability($post);

        $update_data            = array (
            'store_id'          => $post->trans_store_id,
            'insurance'         => $insurance->code,
            'insurance_id'      => $insurance->id,
            'scheme'         	=> $scheme->code,
            'scheme_id'      	=> $scheme->id,
            'store_id'          => $post->trans_store_id,
            'requested_notes'   => $post->trans_requested_notes,
            'items'             => $items,
            'cost'              => $app->unformatMoney($post->trans_subtotal),
            'subtotal'          => $app->unformatMoney($post->trans_subtotal)
        );
        $updated                = $this->update($update_data);
        if($updated) {
            $this->resetItems(json_decode($items));
			$this->updateInlineItems();
            return $app->renderAlertSuccess($submodule->shortname.' updated successfully', 'Success', 'modal_lg', 1);
        }
    }

	function renderTableRow() {
		global $app;

		$link_options   = array('id'=>$this->id, 'name'=>$this->name, 'modal'=>'modal_lg');
		$scheme         = new Scheme($this->scheme_id);
		$html 			= '';

		$html .= '<tr>
		<td class="text-muted fw-semibold">'.$app->datetime($this->created_datetime).'</td>
		<td> <div class="fw-semibold text-auto-truncate">'.$this->id.' '.$this->subtype.'</div><div class="text-muted">'.$this->store()->location()->name.'</div></td>
		<td><div class="mb-1">'.$app->renderLinkOpen($link_options).'</div> <div>File '.$app->formatCode($this->code).' &rarr; '.$app->age($this->birthdate).' '.$this->sex.'</div></td>
		<td>
		<div class="fw-semibold">'.$app->shortname($scheme->insurance()->code, 8).'</div>
		<div class="">'.$app->shortname($scheme->name, 9).'</div>
		</td>
		
		<td class="text-end fw-bold text-muted">';

		$html .= $app->formatMoney($this->subtotal);

		if(strlen($this->reference_no)) $html .= '<br /><small class="text-success">'.$this->reference_no.'</small>';
		
		$html .= '</td>
		<td class="text-end fw-bold text-muted">'.$app->formatMoney($this->paid).'</td>
		
		<td class="text-center">'.$this->renderButtonProgress().'</td>
		<td class="text-center text-nowrap">';

		if($this->hasItemProgress('Paid')) {
			if(($this->subtype == 'Pharmacy' || $this->subtype == 'Prescription' || $this->subtype == 'Consumable' || $this->subtype == 'DirectSale')) {
				$html .= $this->renderLinkDispense();
			}
		}

		if($this->hasItemProgress('Paid') || $this->hasItemProgress('Dispensed')) {
			$html .= $this->renderLinkReceipt();
		}

		if($this->hasItemProgress('Requested')) {
			if(($this->subtype == 'Pharmacy' || $this->subtype == 'Prescription') && $this->subtype_id) {
				if($this->authorized_user_id || app_name == 'salebook') {
					$html .= $this->renderLinkBill();
				} else {
					$html .= '<a class="btn btn-default bs-tooltip" title="Locked"> <span class="fa-regular fa-lock"></span></a>';
				}
			} else {
				$html .= $this->renderLinkBill();
			}
		}

		$html .= $this->renderLinkCopy();

		if($this->maxItemProgressCode() < 2) {
			$html .= $this->renderLinkEdit();
			$html .= $this->renderLinkDelete();
		}

		$html .= '</td>
		</tr>';

		return $html;
	}

	function renderGrandTotal() {
		global $app, $root;

		$html   = '<div class="bg-light p-3 pb-2 rounded mb-4" style="border:1px solid rgb(225,225,230)">';
		$html   .= '<div class="row h6 fw-bold">';
		$html 	.= '<div class="col-12 col-lg-6"> <span class="fa-regular fa-tags fa-fw"></span> &nbsp;Invoice amount: </div>';
		$html 	.= '<div class="col-12 col-lg-6 text-end pe-3"> TZS <span id="trans_invoice_amount">0</span> </div>';
		$html 	.= '</div>';
		
		$html   .= '</div>';
		
        return $html;
	}

	function renderTransactionItemsReadonly($title = 'Billed items') {
		global $app;

		$html = '<div class="trans-container mt-3"> <h5 class="trans-subtitle"> <span class="fa-regular fa-cart-circle-check fa-fw"></span> &nbsp;'.$title.':</h6>
		<div class="card p-3 my-3 bg-primary" style="--bs-bg-opacity: .05;">
		<div class="table-responsive">
		<table class="table table-hover align-middle">
		<thead class="h5">
		<tr>
		<th class="w-5"></th>
		<th class="">Item name</th>
		<th class="text-center w-12">Qty</th>
		<th class="w-20 text-end">Price</th>
		<th class="w-20 text-end">Amount</th>
		</tr>
		</thead>
		<tbody id="trans_items_container">';

		foreach($this->items() as $item) {
			$icon	= '<span class="fa-solid fa-plus"></span>';
			if($item->progress == 'Billed') $icon	= '<span class="fa-regular fa-money-check-dollar-pen"></span>';
            if($item->progress == 'Paid') $icon	= '<span class="fa-solid fa-check-double"></span>';
            if($item->progress == 'Dispensed') $icon	= '<span class="fa-solid fa-arrow-up-from-bracket"></span>';
            
			$details = strlen($item->details) ? '<div class="text-muted"><small>'.$item->details.'</small></div>' : '';
			$html   .= '<tr>
			<td class="text-success text-center">'.$icon.'</td>
			<td class="">'.$item->name.' <b class="text-muted">('.$item->pack.')</b> '.$details;

			if($item->approval_no) $html   .= '<div class="my-2 text-success"><span class="fa-regular fa-check"></span> Approval no: '.$item->approval_no.'</div>';

			$html   .= '</td><td class="text-center">'.$item->requested_quantity.'</td>
			<td class="text-end">'.$app->formatMoney($item->unit_price).'</td>
			<td class="text-end">'.$app->formatMoney($item->price).'</td>
			</tr>';
		}

		$html .= '</tbody>
		<tfoot>
		<tr>
		<th></th>
		<th class="fw-semibold">Subtotal</th>
		<th></th>
		<th></th>
		<th class="text-end">'.$app->renderInputText('trans_subtotal', $app->formatMoney($this->subtotal), 'Subtotal', 'line-control-plaintext text-end fw-semibold', 'readonly').'</th>
        </tr>
		</tfoot>
		</table>
		</div>
		</div>
		</div>';
		return $html;
	}

	function renderCustomerBio($keyword) {
		return $this->customer()->renderBio($keyword);
	}

	function insurance() {
		return new Insurance($this->insurance_id);
	}

	function scheme() {
		return new Scheme($this->scheme_id);
	}

	function renderPromiseForm() {
		global $app;

		$html = $this->renderTransactionHeader();

		$html .= ''.$app->renderInputHidden('trans_id', $this->id).'
		<div class="trans-container mb-4"> <h5 class="trans-subtitle"> <span class="fa-regular fa-cart-circle-check fa-fw"></span> &nbsp;Billed items:</h5>
		<div class="card p-3 my-3 bg-primary" style="--bs-bg-opacity: .05;">
        <div class="table-responsive">
		<table class="table table-hover align-middle">
		<thead class="h5">
		<tr>
		<th class="">Item name</th>
		<th class="w-20 text-end">Billed</th>
		<th class="w-20 text-end">Promise</th>
		<th class="w-20 text-end">Invoice</th>
		</tr>
		</thead>

		<tbody id="trans_items_container">';

		$total_price 		= 0;
		$total_promise		= 0;
		$total_payable		= 0;

		foreach($this->items() as $item) {

			$price			= $item->price;
			$promise		= $item->price;
			$payable		= $price - $promise;

			$total_price 	+= $price;
			$total_promise+= $promise;
			$total_payable	+= $payable;

			$html 		.= '<tr>
			<td class="">
			'.$item->quantity.' x '.$item->name.'
			'.$app->renderInputHidden('trans_items['.$item->item_id.'][transaction_item_id]', $item->id).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][item_id]', $item->item_id).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][name]', $item->name).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][uom]', $item->uom).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][pack]', $item->pack).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][size]', $item->size).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][unit_cost]', $item->unit_cost).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][cost]', $item->cost).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][unit_price]', $item->unit_price).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][quantity]', $item->quantity).'
			'.$app->renderInputHidden('trans_items['.$item->item_id.'][qty]', $item->qty).'
			</td>
			<td class="py-3">'.$app->renderInputText('trans_items['.$item->item_id.'][price]', $app->formatMoney($price), '', 'form-control-plaintext readonly text-end money', 'data-item-id='.$item->item_id.'').'</td>
			<td class="py-3">'.$app->renderInputText('trans_items['.$item->item_id.'][promise]', $app->formatMoney($promise), '', 'line-control text-end money trans-bill-item', 'data-item-id='.$item->item_id.'').'</td>
			<td class="py-3">'.$app->renderInputText('trans_items['.$item->item_id.'][payable]', $app->formatMoney($payable), '', 'form-control-plaintext readonly text-end money fw-semibold', 'data-item-id='.$item->item_id.'').'</td>
			</tr>';
		}

		$html .= '</tbody>

		<tfoot class="fw-semibold">
		<tr>
		<th class="py-3">Total</th>
		<th class="w-20 text-end">'.$app->renderInputText('trans_subtotal', $app->formatMoney($total_price), '', 'form-control-plaintext readonly text-end money fw-semibold').'</th>
		<th class="w-20 text-end">'.$app->renderInputText('trans_promise', $app->formatMoney($total_promise), '', 'form-control-plaintext readonly text-end money fw-semibold').'</th>
		<th class="w-20 text-end">'.$app->renderInputText('trans_payable', $app->formatMoney($total_payable), '', 'form-control-plaintext readonly text-end money fw-semibold').'</th>
		</tr>
		</tfoot>

		</table>
		</div>
		</div>
        </div>';

		return $html;
	}


	function renderTopForm() {
		global $app;

		$html = $this->renderTransactionHeader();

		$html = '';

		$html .= ''.$app->renderInputHidden('trans_id', $this->id).'
		<div class="trans-container mb-4"> <h5 class="trans-subtitle"> <span class="fa-regular fa-cart-circle-check fa-fw"></span> &nbsp;Billed items:</h5>
		<div class="card p-3 my-3 bg-primary" style="--bs-bg-opacity: .05;">
        <div class="table-responsive">
		<table class="table table-hover align-middle">
		<thead class="h5">
		<tr>
		<th class="">Item name</th>
		<th class="w-20 text-end">Billed</th>
		<th class="w-20 text-end">Topup</th>
		<th class="w-20 text-end">Total bill</th>
		</tr>
		</thead>

		<tbody id="trans_items_container">';

		foreach($this->items() as $item) {
			$html 		.= '<tr>
			<td class="">
			'.$item->quantity.' x '.$item->name.'
			'.$app->renderInputHidden('trans_items['.$item->item_id.'][transaction_item_id]', $item->id).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][item_id]', $item->item_id).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][name]', $item->name).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][uom]', $item->uom).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][pack]', $item->pack).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][size]', $item->size).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][unit_cost]', $item->unit_cost).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][cost]', $item->cost).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][unit_price]', $item->unit_price).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][quantity]', $item->quantity).'
			'.$app->renderInputHidden('trans_items['.$item->item_id.'][qty]', $item->qty).'
			</td>
			<td class="py-3">'.$app->renderInputText('trans_items['.$item->item_id.'][price]', $app->formatMoney($item->price), '', 'form-control-plaintext readonly text-end money', 'data-item-id='.$item->item_id.'').'</td>
			<td class="py-3">'.$app->renderInputText('trans_items['.$item->item_id.'][promise]', $app->formatMoney(0), '', 'line-control text-end money trans-bill-item', 'data-item-id='.$item->item_id.'').'</td>
			<td class="py-3">'.$app->renderInputText('trans_items['.$item->item_id.'][payable]', $app->formatMoney($item->price), '', 'form-control-plaintext readonly text-end money fw-semibold', 'data-item-id='.$item->item_id.'').'</td>
			</tr>';
		}

		$html .= '</tbody>

		<tfoot class="fw-semibold">
		<tr>
		<th class="py-3">Total</th>
		<th class="w-20 text-end">'.$app->renderInputText('trans_subtotal', $app->formatMoney($this->subtotal), '', 'form-control-plaintext readonly text-end money fw-semibold').'</th>
		<th class="w-20 text-end">'.$app->renderInputText('trans_topup', $app->formatMoney($this->promise), '', 'form-control-plaintext readonly text-end money fw-semibold').'</th>
		<th class="w-20 text-end">'.$app->renderInputText('trans_payable', $app->formatMoney($this->payable), '', 'form-control-plaintext readonly text-end money fw-semibold').'</th>
		</tr>
		</tfoot>

		</table>
		</div>
		</div>
        </div>';

		return $html;
	}


	function renderExemptionForm() {
		global $app;

		$html = $this->renderTransactionHeader();

		$html .= ''.$app->renderInputHidden('trans_id', $this->id).'
		<div class="trans-container mb-4"> <h5 class="trans-subtitle"> <span class="fa-regular fa-cart-circle-check fa-fw"></span> &nbsp;Billed items:</h5>
		<div class="card p-3 my-3 bg-primary" style="--bs-bg-opacity: .05;">
        <div class="table-responsive">
		<table class="table table-hover align-middle">
		<thead class="h5">
		<tr>
		<th class="">Item name</th>
		<th class="w-20 text-end">Billed</th>
		<th class="w-20 text-end">Exemption</th>
		<th class="w-20 text-end">Invoice</th>
		</tr>
		</thead>

		<tbody id="trans_items_container">';

		$total_price 		= 0;
		$total_exemption	= 0;
		$total_payable		= 0;
		foreach($this->items() as $item) {
			$price			= $item->price;
			$exemption		= $item->price;
			$payable		= $price - $exemption;

			$total_price 	+= $price;
			$total_exemption+= $exemption;
			$total_payable	+= $payable;

			$html 			.= '<tr>
			<td class="">
			'.$item->quantity.' x '.$item->name.'
			'.$app->renderInputHidden('trans_items['.$item->item_id.'][transaction_item_id]', $item->id).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][item_id]', $item->item_id).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][name]', $item->name).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][uom]', $item->uom).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][pack]', $item->pack).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][size]', $item->size).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][unit_cost]', $item->unit_cost).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][cost]', $item->cost).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][unit_price]', $item->unit_price).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][quantity]', $item->quantity).'
			'.$app->renderInputHidden('trans_items['.$item->item_id.'][qty]', $item->qty).'
			</td>
			<td class="py-3">'.$app->renderInputText('trans_items['.$item->item_id.'][price]', $app->formatMoney($price), '', 'form-control-plaintext readonly text-end money', 'data-item-id='.$item->item_id.'').'</td>
			<td class="py-3">'.$app->renderInputText('trans_items['.$item->item_id.'][exemption]', $app->formatMoney($price), '', 'line-control text-end money trans-bill-item', 'data-item-id='.$item->item_id.'').'</td>
			<td class="py-3">'.$app->renderInputText('trans_items['.$item->item_id.'][payable]', $app->formatMoney($payable), '', 'form-control-plaintext readonly text-end money fw-semibold', 'data-item-id='.$item->item_id.'').'</td>
			</tr>';
		}

		$html .= '</tbody>

		<tfoot class="fw-semibold">
		<tr>
		<th class="py-3">Total</th>
		<th class="w-20 text-end">'.$app->renderInputText('trans_subtotal', $app->formatMoney($total_price), '', 'form-control-plaintext readonly text-end money fw-semibold').'</th>
		<th class="w-20 text-end">'.$app->renderInputText('trans_exemption', $app->formatMoney($total_exemption), '', 'form-control-plaintext readonly text-end money fw-semibold').'</th>
		<th class="w-20 text-end">'.$app->renderInputText('trans_payable', $app->formatMoney($total_payable), '', 'form-control-plaintext readonly text-end money fw-semibold').'</th>
		</tr>
		</tfoot>

		</table>
		</div>
		</div>
        </div>';

        $html .= '<div class="row">
        <div class="col-sm-12 my-3">
        <h5 class="my-2"> <span class="fa-solid fa-check fa-fw"></span> &nbsp;Exemption category: </h5>
        '.$app->renderInputSelect('trans_subtype', '', 'Exemption category', 'line-control-lg ', 'data-style="btn-outline-secondary btn-lg"', 'settings_categories', 'code', 'name', 'type="ExemptionType"').'
        </div><div class="col-sm-12 my-3">
        <h5 class="my-3"> <span class="fa-regular fa-note-sticky fa-fw"></span> &nbsp;Notes or comments: </h5>
        '.$app->renderInputTextarea('trans_requested_notes', $this->requested_notes, 'Enter optional notes', 'form-control-lg').'
        </div></div>';

		return $html;
	}
	
}

?>