<?php 

$root               = '../../../../';
$submodule_code     = 'BillingBulk';

require($root.'config/config.php');
$cache_path	        = $root.CLIENT_PATH.'cache/nhif_pricelist.json';

if(!file_exists($cache_path)) {
    die($app->renderAlertWarning('Please download NHIF items before listing them', 'No items found'));
}

$raw_nhif_items		= file_get_contents($cache_path);
$all_nhif_items		= json_decode($raw_nhif_items);
$raw_nhif_items		= $all_nhif_items->PricePackage;
$excluded_items 	= $all_nhif_items->ExcludedServices;

$nhif_items 		= array();

$nhif_schemes 		= array(
	'1001'			=> array('id'=>'1001', 'code'=>'Standard', 'name'=>'Standard NHIF Benefit Scheme'),
	'2001'			=> array('id'=>'2001', 'code'=>'Bunge', 'name'=>'Bunge Health Insurance Scheme'),
	'3001'			=> array('id'=>'3001', 'code'=>'BoT', 'name'=>'BoT Health Insurance Scheme'),
	'4001'			=> array('id'=>'4001', 'code'=>'Option1', 'name'=>'Supplementary Option 1'),
	'5001'			=> array('id'=>'5001', 'code'=>'Option2', 'name'=>'Supplementary Option 2'),
	'6001'			=> array('id'=>'6001', 'code'=>'Option3', 'name'=>'Supplementary Option 3'),
	'7001'			=> array('id'=>'7001', 'code'=>'CRDB', 'name'=>'CRDB Health Insurance Scheme'),
	'8001'			=> array('id'=>'8001', 'code'=>'NMB', 'name'=>'NMB Health Insurance Scheme')
);

$nhif_scheme_keys 	= array_keys($nhif_schemes);

// Foreach remote item, sync with local item
$counter = 0;
$local_items 	= array();
foreach($raw_nhif_items as $raw_nhif_item) {

	$nhif_code	= $raw_nhif_item->ItemCode;
	$alt_name	= $app->cleanText($raw_nhif_item->ItemName.' '.$raw_nhif_item->Strength);
	$shortname	= $app->shortname($alt_name, 40);
	$uom		= $raw_nhif_item->Dosage;
	$scheme		= $raw_nhif_item->SchemeID;
	$price		= $raw_nhif_item->UnitPrice;
	$status 	= $raw_nhif_item->IsActive ? 'Active' : 'Inactive';
	$restricted = $raw_nhif_item->IsRestricted ? 'Yes' : 'No';
	$max 		= $raw_nhif_item->MaximumQuantity > -1 ? $raw_nhif_item->MaximumQuantity : 'None';

	if(strlen($get->keyword)) {
		if(is_numeric(($get->keyword))) {
			if(strpos(strtolower($nhif_code), strtolower($get->keyword)) === false) continue;
		} else {
			if(strpos(strtolower($alt_name), strtolower($get->keyword)) === false) continue;
		}
	}

	// Update local item with details from remote items
	$local_items[$nhif_code]['nhif_code']	= $nhif_code;
	$local_items[$nhif_code]['alt_name']	= $app->shortname($alt_name, 50);
	$local_items[$nhif_code]['uom']			= $uom;
	$local_items[$nhif_code]['status']		= $status;
	$local_items[$nhif_code]['approval']	= $restricted;
	$local_items[$nhif_code]['max']			= $max;
	$local_items[$nhif_code]['dosage']		= $uom;

	// Update pricegroup based on scheme ID
	$local_items[$nhif_code][$scheme] 		= $price;

	// Excluded ?
	$local_items[$nhif_code]['excluded']	= '';

	foreach($excluded_items as $excluded_item) {
		$item_code		= $excluded_item->ItemCode;
		$scheme_id		= $excluded_item->SchemeID;
		$product_codes 	= $excluded_item->ExcludedForProducts;

		if($item_code == $nhif_code) {
			$local_items[$nhif_code]['excluded'] = $product_codes;
		}
	}
	$counter++;
}

$nhif_items 	= $local_items;

echo $app->renderAlertPrimary('Listing '.count($nhif_items).' items from National Health Insurance Fund');

$table_headers = array (
    array('name'=>'Code', 'css'=>'w-20'),
    array('name'=>'Item name'),
    array('name'=>'Dosage', 'css'=>'w-20')
);

foreach($nhif_schemes as $nhif_scheme) {
	$table_headers[] = array('name'=>$nhif_scheme['code'], 'css'=>'w-20 text-end');
}

$table_headers[] = array('name'=>'Excluded', 'css'=>'w-20');
$table_headers[] = array('name'=>'Approval', 'css'=>'w-5');
$table_headers[] = array('name'=>'Max', 'css'=>'w-5');
$table_headers[] = array('name'=>'Status', 'css'=>'w-5 text-center');

$table_options  = array (
    'columns'   => $table_headers,
	'css'		=> 'table-bordered my-4 bs-table'
);

echo $app->renderTableOpen($table_options);

foreach($nhif_items as $nhif_item) {
	echo '<tr>
	<td class="fw-bold">'.$nhif_item['nhif_code'].'</td>
	<td style="min-width:200px">'.$nhif_item['alt_name'].'</td>
	<td>'.$nhif_item['dosage'].'</td>';

	foreach($nhif_scheme_keys as $nhif_scheme_key) {
		$scheme_price = isset($nhif_item[$nhif_scheme_key]) ? $nhif_item[$nhif_scheme_key] : 0;
		echo '<td class="text-end text-muted fw-bold">'.number_format($scheme_price).'</td>';
	}

	echo '<td class="text-muted">'.$nhif_item['excluded'].'</td>
	<td class="text-center">'.$nhif_item['approval'].'</td>
	<td class="text-center">'.$nhif_item['max'].'</td>
	<td>'.$nhif_item['status'].'</td>
	</tr>';
}

echo $app->renderTableClose();

?>