<?php 

require($root.'libraries/tcpdf/tcpdf_custom_color.php');

class Folio extends Transaction {
	function __construct($id=0) {	
		global $app;
		$this->trans_type 	= 'Folio';
		$this->table        = 'billing_folios';
		$this->items_table	= 'billing_folio_items';
		$this->primary_key  = 'id';
		parent::__construct($id);
		$this->type 		= 'Folio';
		if(!strlen($this->authorization_no)) {
			$visit 			= $this->visit();
			$this->update (
				array (
					'card_no'			=> $visit->card_no,
					'authorization_no'	=> $visit->authorization_no
				)
			);
			$this->read();
		}
	}

	/*
	function getNextFolioNo() {
		global $app;
		$year 	= $this->year ? $this->year : date('Y');
		$month 	= $this->month ? $this->month : date('m');
		$sql	= "SELECT folio_no FROM billing_folios WHERE year = $year AND month = $month ORDER BY folio_no DESC ";
		foreach($app->read($sql) as $row) {
			return $row->folio_no + 1;
		}
		return 1;
	}
	*/

	function getNextFolioNo() {
		global $app;
		$year 				= $this->year ? $this->year : date('Y');
		$month 				= $this->month ? $this->month : date('m');
		$existing_folios 	= array();
		$sql				= "SELECT folio_no FROM billing_folios WHERE year = $year AND month = $month ";
		foreach($app->read($sql) as $row) {
			$existing_folios[] = $row->folio_no;
		}

		for($new_folio = $this->folio_no; $new_folio < 30000; $new_folio++) {
			if(!$new_folio) continue;
			if(!in_array($new_folio, $existing_folios)) return $new_folio;
		}

		return 1;
	}

	function addFolioItemsFromInvoice($invoice_id) {
		$invoice 					= new Invoice($invoice_id);
		foreach($invoice->items() as $item) {
			$data 					= (array) $item;
			$data['parent']	= 'Folio';
			$data['parent_id']	= $this->id;
			$folio_item 			= new TransactionItem('Folio', $this->id);
			$folio_item_id 			= $folio_item->createWithMinimumData($data);
			if($folio_item_id) {
				$this->set('subtotal', ($this->subtotal + $data['price']));
			}
		}
		$this->updateInlineItems();
	}

	function renderLinkOpen($options = array()) {
		$modal  = isset($options['modal']) ? $options['modal'] : 'modal_lg';
        return '<a title="Open" class="remote-modal bs-tooltip btn btn-default" data-bs-toggle="modal" data-bs-target="#'.$modal.'" data-bs-backdrop="static" data-bs-keyboard="false" tabindex="-1" href="form.php?action=open&id='.$this->id.'" aria-label="Authorize"> <span class="fa-regular fa-up-right-from-square"></span> </a>';
    }

	function renderLinkUpdate($options = array()) {
		$modal  = isset($options['modal']) ? $options['modal'] : 'modal_lg';
        return '<a title="Update" class="remote-modal bs-tooltip btn btn-default" data-bs-toggle="modal" data-bs-target="#'.$modal.'" data-bs-backdrop="static" data-bs-keyboard="false" tabindex="-1" href="edit_form.php?action=edit&id='.$this->id.'" aria-label="Edit"> <span class="fa-regular fa-pen-to-square"></span> </a>';
    }

	function renderLinkSign($options = array()) {
		$modal  = isset($options['modal']) ? $options['modal'] : 'modal_lg';
        return '<a title="Sign" class="remote-modal bs-tooltip btn btn-default" data-bs-toggle="modal" data-bs-target="#'.$modal.'" data-bs-backdrop="static" data-bs-keyboard="false" tabindex="-1" href="form.php?action=edit&id='.$this->id.'" aria-label="Edit"> <span class="fa-regular fa-pen-to-square"></span> </a>';
    }

	function renderLinkAction($options = array()) {
		$modal  = isset($options['modal']) ? $options['modal'] : 'modal_lg';
        return '<a title="Action" class="remote-modal bs-tooltip btn btn-default" data-bs-toggle="modal" data-bs-target="#'.$modal.'" data-bs-backdrop="static" data-bs-keyboard="false" tabindex="-1" href="action_form.php?action=action&id='.$this->id.'" aria-label="Action"> <span class="fa-regular fa-bolt"></span> </a>';
    }

	function getFolioItems() {

		$items = array();
		
		foreach($this->items() as $item) {

			if(!$item->status) continue;

			$item_obj 			= new Item($item->item_id);
			$item->name_alt 	= $item_obj->name_alt;
			$item->nhif_code 	= $item_obj->code6;
			$subtype 			= $item->subtype;
			$item_id 			= $item->item_id;
			if($subtype == 'Procedure') $subtype = 'Procedures';
			if($subtype == 'Prescription' || $subtype == 'Pharmacy') $subtype = 'Medications';
			$items[$subtype][$item_id]	= $item;
		}

		return $items;
	}


	function getItemsArray() {

		$folio_items 			= array();
		$clinician 				= $this->getClinician();
		foreach($this->getFolioItems() as $folio_services) {
			foreach($folio_services as $folio_service) {
				$folio_items[] 	= array (
					'ItemCode'		=> $folio_service->nhif_code,
					'OtherDetails'	=> '',
					'ItemQuantity'	=> $folio_service->quantity,
					'UnitPrice'		=> $folio_service->unit_price,
					'AmountClaimed'	=> $folio_service->price,
					'ApprovalRefNo'	=> $folio_service->approval_no,
					'CreatedBy'		=> $clinician->person()->name,
					'DateCreated'	=> $folio_service->created_datetime,
					'LastModifiedBy'=> null,
					'LastModified'	=> null
				);
			}
		}

		return $folio_items;
	}

	function visit() {
		return new Visit($this->visit_id);
	}

	function getClinician() {
		global $app, $user;
		$clinician_user_id	= 0;

		// Created notes
		$notes_sql 			= "SELECT clinical_notes.created_user_id, personnel_users.qualification_id 
		FROM clinical_notes JOIN personnel_users ON personnel_users.id = clinical_notes.created_user_id 
		WHERE clinical_notes.visit_id = $this->visit_id ORDER BY personnel_users.qualification_id";
		foreach($app->read($notes_sql) as $row) {
			$clinician_user_id	= $row->created_user_id;
		}
		if($clinician_user_id) {
			$user 				= new User($clinician_user_id);
			return $user->personnel();
		}

		// Edited notes
		$notes_sql 			= "SELECT clinical_notes.modified_user_id, personnel_users.qualification_id 
		FROM clinical_notes JOIN personnel_users ON personnel_users.id = clinical_notes.modified_user_id 
		WHERE clinical_notes.visit_id = $this->visit_id ORDER BY personnel_users.qualification_id";
		foreach($app->read($notes_sql) as $row) {
			$clinician_user_id	= $row->modified_user_id;
		}
		if($clinician_user_id) {
			$user 				= new User($clinician_user_id);
			return $user->personnel();
		}

		// Added diagnoses
		$diagnoses_sql 			= "SELECT clinical_diagnoses.created_user_id, personnel_users.qualification_id 
		FROM clinical_diagnoses JOIN personnel_users ON personnel_users.id = clinical_diagnoses.created_user_id 
		WHERE clinical_diagnoses.visit_id = $this->visit_id ORDER BY personnel_users.qualification_id";
		foreach($app->read($diagnoses_sql) as $row) {
			$clinician_user_id	= $row->created_user_id;
		}
		if($clinician_user_id) {
			$user 				= new User($clinician_user_id);
			return $user->personnel();
		}

		return $user->personnel();

	}

	function generateFormPdf($return = false) {
		global $root, $app, $client, $user;
		$insurance 				= new Insurance($this->insurance_id);
		$visit 					= new Visit($this->visit_id);
		$visit_location 		= new VisitLocation($visit->getFirstVisitLocationId());
		$person 				= $visit->patient()->person();
		$customer 				= $this->customer();
		$preliminary_diagnoses 	= implode(', ', $visit->getProvisionalDiagnosesCodes());
		$final_diagnoses 		= implode(', ', $visit->getConfirmedlDiagnosesCodes());
		$bills 					= $this->getFolioItems();
		$clinician 				= $this->getClinician();
		
		$clinician_signature 	= $app->getAttachmentByCode('Employee', $clinician->id, 'EmployeeSignature');
		$clinician_sig_path 	= $clinician_signature->path();
		
		if(!$clinician_signature->exists()) {
			die($app->renderAlertError('Clinician signature does not exists!', $clinician->person()->name));
		}

		$customer_signature 	= $app->getAttachmentBySection('Visit', $this->visit_id, 'PatientSignature');
		$customer_sig_path 		= $customer_signature->path();
		if(!$customer_signature->exists()) {
			die($app->renderAlertError('Customer signature does not exists!', $customer->person()->name));
		}

		$claimant_signature 	= $app->getAttachmentByCode('Employee', $user->personnel()->id, 'EmployeeSignature');
		$claimant_sig_path 		= $claimant_signature->path();
		if(!$claimant_signature->exists()) {
			die($app->renderAlertError('Claimant signature does not exists!', $user->person()->name));
		}

		$title 					= 'Patient NHIF form '.$this->id;

		$pdf = new MYPDF('L', PDF_UNIT, PDF_PAGE_FORMAT, true, 'UTF-8', false);
		//$pdf->setPrintHeader(false);
		$pdf->setPrintFooter(false);
		$pdf->SetTitle('Patient NHIF form '.$this->id);
		$pdf->SetDefaultMonospacedFont(PDF_FONT_MONOSPACED);
		$pdf->SetMargins(16, 10, 14);
		$pdf->SetHeaderMargin(PDF_MARGIN_HEADER);
		$pdf->SetFooterMargin(PDF_MARGIN_FOOTER);
		$pdf->SetAutoPageBreak(TRUE, PDF_MARGIN_BOTTOM);
		$pdf->setImageScale(PDF_IMAGE_SCALE_RATIO);
		$pdf->setFontSubsetting(true);
		$pdf->SetFont('helvetica', '', 12, '', true);
		$pdf->AddPage();

		$html = '<table>
		<tr>
		<td style="width:20%"><img src="'.$app->encodeImage($root.'assets/images/other/nhif.png').'" style="width:60px" /></td>
		<td style="width:60%; text-align:center; font-weight:bold; font-size:1.2em"><div style="font-size:1.5em">CONFIDENTIAL</div> THE NHIF - HEALTH PROVIDER IN / OUT PATIENT CLAIM FORM</td>
		<td style="width:20%; text-align:right; font-size:0.9em">FORM NHIF 2A <br />Regulation 18 (1) <br /> S/N: <b>'.$insurance->facility_code.'/'.$this->year.'/'.$this->month.'/'.$this->folio_no.'</b></td>
		</tr>
		</table>';
		
		$html .= '<div><br /><b style="font-size:1.1em">A: PATIENT PARTICULARS </b><br /> </div>';

		$html .= '<table> <tr><td style="width:50%">
		1. Facility Name &nbsp; &nbsp; : <b>'.$client->client_name.'</b> <br />
		2. Address&nbsp; &nbsp; &nbsp; &nbsp;&nbsp; &nbsp; &nbsp; &nbsp;: <b>'.$client->client_address.'</b>
		
		</td><td style="width:50%">

		3. Department&nbsp; &nbsp; &nbsp; &nbsp; :<b> '.$visit_location->location()->location()->name.'</b> <br />
		4. Attendance date : <b>'.$app->date($visit_location->created_datetime).'</b>';

		$html .= '</td></tr></table><br /><br />';

	
		$html .= '<table> <tr>
		<td style="width:50%">
		1. Patient\'s Name &nbsp; : <b>'.$person->name.'</b> <br />
		2. DOB &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; : <b>'.$person->birthdate.'</b> <br />
		3. Sex &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; : <b>'.$person->sex.'</b> <br />
		4. Occupation&nbsp; &nbsp; &nbsp; &nbsp; &nbsp;: <b>'.$person->occupation.'</b> <br />
		5. Patient\'s File No : <b>'.$app->formatCode($customer->code, 6).'</b>

		</td><td style="width:50%">
		6. Patient\'s Address: <b>'.$person->address.'</b> <br />
		7. Card No &nbsp;&nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; : <b>'.$customer->card_no.'</b> <br />
		8. Authorization No&nbsp;: <b>'.$this->authorization_no.'</b> <br />
		9. Vote No &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; : <b>'.$customer->employer_code.' </b> <br />
		10. Preliminary Dx &nbsp;: <b>'.$preliminary_diagnoses.'</b> <br />
		11. Final Diagnoses : <b>'.$final_diagnoses.'</b> 
		
		</td></tr></table><br />';


		$html .= '<div><b style="font-size:1.1em">B: DETAILS / COST OF SERVICES </b></div>';

		// BILLS

		$total		= 0;
		foreach(array_keys($bills) as $bill) {

			if($bill == 'Consultation' && $this->subtype == 'Inpatient') continue; 
			//$html .= '<div><table style="width:100%; padding:4px; background-color:rgb(220,220,220); border:1px solid rgb(150,150,150); margin:0"><tr><td>'.$bill.'</td></tr></table></div><br />';

			$bill_name = strtoupper($bill);

			$html .= '<table style="width:100%; padding:6px"><tr><td style="width:100%"></td></tr></table>
			
			<table style="width:100%; padding:6px; background-color:rgb(180,180,150); border:1px solid rgb(100,100,100); margin:0">
			<tr><td style="width:100%"><b>'.$bill_name.'</b></td> </tr></table> 
			
			<table style="width:100%; padding:6px; border:1px solid rgb(180,180,150); margin:0">

			<tr>
			<td style="padding:6px; width:50%; border-top:1px solid rgb(100,100,100); border-right:1px solid rgb(100,100,100);">ITEM NAME</td>
			<td style="padding:6px; width:10%; text-align:center; border-top:1px solid rgb(100,100,100); border-right:1px solid rgb(100,100,100);">CODE</td>
			<td style="padding:6px; width:10%; text-align:center; border-top:1px solid rgb(100,100,100); border-right:1px solid rgb(100,100,100);">QTY</td>
			<td style="padding:6px; width:15%; text-align:right; border-top:1px solid rgb(100,100,100); border-right:1px solid rgb(100,100,100);">UNIT PRICE</td> 
			<td style="padding:6px; width:15%; text-align:right; border-top:1px solid rgb(100,100,100); border-right:0px solid rgb(100,100,100);">COST</td>
			</tr>';

			$subtotal 	= 0;
			foreach($bills[$bill] as $row) {

				// Prescription?

				if($bill == 'Medications'  && 0) {

					$p2 	= new Prescription($row);
					$items2	= $p2->items();

					$html .= '<tr>
					<td style="width:50%; border-top:1px solid rgb(150,150,150); border-right:1px solid rgb(150,150,150);">'.$row['alt_name'];
					
					foreach($items2 as $item2) {
						if($item2->item_id == $row['item_id']) {
							$html .= '<div style="color:rgb(100,100,100);"><span style="font-size:0.2em"><br /></span>'.$item2->dose.' '.$item2->units.' '.$item2->times.' Times per Day for '.$item2->days.' '.$this->main->countItems($item2->days, 'Day').'</div>';
						}
					}

					$html .= '<tr>
					<td style="width:50%; border-top:1px solid rgb(150,150,150); border-right:1px solid rgb(150,150,150);">'.$row->name_alt.'</td>
					<td style="width:10%; text-align:center; border-top:1px solid rgb(150,150,150); border-right:1px solid rgb(150,150,150);">'.$row->nhif_code.'</td>
					<td style="width:10%; text-align:center; border-top:1px solid rgb(150,150,150); border-right:1px solid rgb(150,150,150);">'.$row->quantity.'</td>
					<td style="width:15%; text-align:right; border-top:1px solid rgb(150,150,150); border-right:1px solid rgb(150,150,150);">'.number_format($row->unit_price).'</td> 
					<td style="width:15%; text-align:right; border-top:1px solid rgb(150,150,150); border-right:0px solid rgb(150,150,150);">'.number_format($row->price).'</td>
					</tr>';
				} else {
					$html .= '<tr>
					<td style="width:50%; border-top:1px solid rgb(150,150,150); border-right:1px solid rgb(150,150,150);">'.$row->name_alt.'</td>
					<td style="width:10%; text-align:center; border-top:1px solid rgb(150,150,150); border-right:1px solid rgb(150,150,150);">'.$row->nhif_code.'</td>
					<td style="width:10%; text-align:center; border-top:1px solid rgb(150,150,150); border-right:1px solid rgb(150,150,150);">'.$row->quantity.'</td>
					<td style="width:15%; text-align:right; border-top:1px solid rgb(150,150,150); border-right:1px solid rgb(150,150,150);">'.number_format($row->unit_price).'</td> 
					<td style="width:15%; text-align:right; border-top:1px solid rgb(150,150,150); border-right:0px solid rgb(150,150,150);">'.number_format($row->price).'</td>
					</tr>';
				}
				
				$subtotal += $row->price;
			}

			$html .= '<tr style="font-weight:bold">
			<td style="width:50%; border-top:1px solid rgb(150,150,150); border-right:0px solid rgb(150,150,150);">SUBTOTAL</td> 
			<td style="border-top:1px solid rgb(150,150,150); border-right:0px solid rgb(150,150,150);"></td>
			<td style="border-top:1px solid rgb(150,150,150); border-right:0px solid rgb(150,150,150);"></td>
			<td style="border-top:1px solid rgb(150,150,150); border-right:1px solid rgb(150,150,150);"></td>
			<td style="width:15%; text-align:right; border-top:1px solid rgb(150,150,150); border-right:0px solid rgb(150,150,150);">'.number_format($subtotal).'</td>
			</tr>';
		
			$html .= '</table>';
			$total += $subtotal;
		}

		$html .= '<table style="width:100%; padding:4px; border:1px solid rgb(150,150,150); margin:0">
		<tr><td style="width:70%; border-right:0px solid rgb(150,150,150);"><b>GRAND TOTAL</b> </td> 
		<td style="width:15%; text-align:right; border-right:1px solid rgb(150,150,150);"></td> 
		<td style="width:15%; text-align:right"><b>'.number_format($total).'</b></td> 
		</tr></table>';

		// ATTENDING CLINICIAN

		$html .= '<br /><div><b style="font-size:1.1em">C: ATTENDING CLINICIAN </b><br /> <br /></div>';

		$html .= '<table> <tr><td style="width:50%">
		<img src="'.$app->encodeImage($clinician_sig_path).'" style="width:150px; position:relative; top:-1em" />
		</td><td style="width:50%">

		Name: <b>'.$clinician->person()->name.'</b> <br />
		Qualifications: <b>'.$clinician->designation.'</b> <br />
		MCT Reg No:   <b>'.$clinician->person()->getIdentityByCode('MCT')->number.'</b>';

		
		$html .= '</td></tr></table><br />';


		// UTHIBITISHO WA MGONJWA / PATIENT CERTIFICATION
		
		$html .= '<div><b style="font-size:1.1em">D: UTHIBITISHO WA MGONJWA / PATIENT CERTIFICATION </b><br /> </div>';

		$html .= 'Nathibitisha kuwa nimepokea huduma zilizoainishwa hapo juu na natambua kwamba ni kosa kisheria kukiri kupata matibabu ambayo hayakutolewa. 
		I certify that I received the above mentioned services as witnessed by my signature hereunder and I understand that it is illegal to provide false testimony.<br /><br /><br />';

		$html .= '<table> <tr><td style="width:50%">
		
		<img src="'.$app->encodeImage($customer_sig_path).'" style="width:150px; position:relative; top:-1em" />
		
		</td><td style="width:50%">
		Jina (Name) : <b>'.$this->customer()->person()->name.'</b> <br />
		Namba ya Simu (Phone)&nbsp;: <b>'.$this->customer()->person()->phone.'</b> <br />
		Tarehe (Date)&nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; : <b>'.$app->date($this->visit()->checkout_datetime).'</b>';

		$html .= '</td></tr></table><br /><br />';

		$html .= '<table> <tr><td style="width:50%; vertical-align:top">';
		$html .= '<div><b style="font-size:1.1em">F: CLAIMANT CERTIFICATION </b><br /> </div>';

		$html .= 'I certify that I have provided the above services. <br /><br />
		Name: <b>'.$user->personnel()->person()->name.'</b> <br /><br /><br />
		<img src="'.$app->encodeImage($claimant_sig_path).'" style="width:150px; position:relative; top:-1em" />';

		$html .= '</td><td style="width:50%; vertical-align:top">';

		/*
		if(file_exists(CLIENT_PATH.'images/stamp.png')) {
			$html .= '<div><b style="font-size:1.1em">OFFICIAL STAMP </b> </div>';
			$html .= '<img src="'.$this->main->encodeImage(CLIENT_PATH.'images/stamp.png').'" style="width:140px;" />';
		}
		*/

		$html .= '</td></tr></table>';

		//CLAIMANT CERTIFICATION
	
		//$pdf->Rect(0,0,420,210,'F','', $fill_color = array(255, 250, 210));
		$pdf->SetFillColor(255, 250, 225);

		$pdf->writeHTMLCell(0, 0, '', '', $html, 0, 1, 0, true, '', true);
		
		if($return) {
			return $pdf->Output($title.'.pdf', 'S');
		} else {
			$pdf->Output($title.'.pdf', 'I');
		}
	}

	public function generateNotesHTML($return=0) {
		global $app;

		$customer 			= $this->customer();
		$customer_person 	= $customer->person();
		$visit 				= $this->visit();
		$events				= $this->getEvents();

		$html 	= '';
		$html 	.= '<div align="center" color="rgb(50,50,50)" style="font-size:2.0em"><b>CONFIDENTIAL</b></div>';

		$html .= '<div style="text-align:center; font-size:1.4em"><b>'.strtoupper('Patient Personal Information').':</b></div><br />';

		$html .= '<table style="width:100%; padding:0; margin:0; color:rgb(50,50,50); ">
		<tr><td style="width:17%"><b style="color:rgb(150,150,150)"> Fullname 			</b> </td><td>  '.$customer_person->name.'		</td></tr>
		<tr><td><b style="color:rgb(150,150,150)"> Age / sex 		</b> </td><td>  '.$customer_person->age().' '.$customer_person->sex.' </td></tr>
		<tr><td><b style="color:rgb(150,150,150)"> Address 			</b> </td><td>  '.$customer_person->address.'		</td></tr>
		<tr><td><b style="color:rgb(150,150,150)"> File number 		</b> </td><td>  '.$customer->formatCode().'			</td></tr>
		<tr><td><b style="color:rgb(150,150,150)"> NHIF package 	</b> </td><td>  '.$customer->scheme()->name.'</td></tr>
		<tr><td><b style="color:rgb(150,150,150)"> Card number 		</b> </td><td>  '.$customer->card_no.'		</td></tr>
		<tr><td><b style="color:rgb(150,150,150)"> Authorization 	</b> </td><td>  '.$visit->authorization_no.'		</td></tr>
		<tr><td><b style="color:rgb(150,150,150)"> Checked in 		</b> </td><td>  '.$app->date($visit->checkin_datetime).'				</td></tr>
		<tr><td><b style="color:rgb(150,150,150)"> Checked out 		</b> </td><td>  '.$app->date($visit->checkout_datetime).'				</td></tr>';
		$html .= '</table>';

		$html .= '<br /><div style="text-align:center; font-size:1.4em"><b>'.strtoupper('Patient Clinical History (TIMELINE)').':</b></div><br />';

		$html .= '<table style="width:100%; padding:0; margin:0; color:rgb(50,50,50)">';

		foreach($events as $event) {
			$html .= '<tr>
			<td style="width:15%; color:rgb(150,150,150)">
			<b style="font-size:1.1em">'.(date('d M', strtotime($event['date'])).' '.date('H:i', strtotime($event['date']))).'</b>
			</td>
			<td style="width:0%"></td>
			<td style="color:rgb(90,90,90); width:81%; font-size:1.1em"><b>'.($event['title']).'</b> <span style="color:rgb(130,130,130)">@ '.$event['location'].'</span></td>
			</tr>

			<tr>
			<td></td>
			<td></td>
			<td></td>
			</tr>
			
			<tr>
			<td style="width:7%; border-right: 4px solid rgb(200,200,200);"></td>
			<td style="width:11%"></td>
			<td style="width:81%; text-align:justify;">'.$event['details'].'</td>
			</tr>
			
			<tr>
			<td><br /><br /></td>
			<td></td>
			<td></td>
			</tr>';
		}

		$html .= '</table>';
		$html .= '</div>';

		$html = '<center style="background-color:rgb(250,250,250)"><div style="font-family:Arial; width:800px; max-width:800px; background-color:rgb(255,255,255); border:1px solid rgb(200,200,200); padding:15px">'.$html.'</div></center>';

		if($return) {
			return $html;
		} else {
			echo $html;
		}
	}


	public function getEvents() {
		global $app;
		$events		= array();
		$visit 		= $this->visit();

		// Registration

		$employee		= new Employee($visit->created_user_id);
		$event 			= array(
			'date'		=> $visit->created_datetime,
			'name'		=> 'Card authorised',
			'title'		=> $employee->person()->name.' ('.$employee->designation.') registered patient',
			'details'	=> 'Patient registered. <br />Card authorized, authorization number: <b>'.$visit->authorization_no.'</b>',
			'location'	=> 'Registration desk'
		);
		$events[$visit->created_datetime]	= $event;

		// Notes 
		$notes			= $this->getNotes();
		foreach($notes as $note) {
			$event = array(
				'date'		=> $note->created_datetime,
				'name'		=> $note->name,
				'title'		=> $note->create_staff_name.' ('.$note->create_staff_designation.') added '.$note->name,
				'details'	=> $note->notes,
				'location'	=> $note->location
			);
			$events[$note->created_datetime]	= $event;
		}

		// Diagnoses 
		$diagnoses_str = "SELECT clinical_diagnoses.code, 
		clinical_diagnoses.name, 
		clinical_diagnoses.certainty, 
		clinical_diagnoses.created_datetime, 
		personnel_persons.name create_staff_name,
		personnel_employees.designation create_staff_designation,

		IF (clinical_visit_locations.clinic_id > 1, (SELECT name FROM location_locations JOIN clinical_clinics  WHERE clinic_id = clinical_visit_locations.clinic_id LIMIT 1), 
		(SELECT name FROM location_locations JOIN clinical_wards ON clinical_wards.location_id = location_locations.id WHERE ward_id = clinical_visit_locations.ward_id LIMIT 1)) AS location

		FROM clinical_diagnoses
		JOIN personnel_users ON personnel_users.id = clinical_diagnoses.created_user_id
		JOIN personnel_persons ON personnel_persons.id = personnel_users.person_id
		JOIN personnel_employees ON personnel_employees.person_id = personnel_persons.id
		
		JOIN clinical_visit_locations ON clinical_visit_locations.id = clinical_diagnoses.visit_location_id
		
		WHERE clinical_diagnoses.visit_id = $this->visit_id AND clinical_diagnoses.status = 1";

		$diagnoses		= $app->read($diagnoses_str);

		$dx_counter 	= 0;
		foreach($diagnoses as $diagnosis) {
			$dx_counter++;
			
			$diagnosis->created_datetime = date('Y-m-d H:i:s', strtotime($diagnosis->created_datetime.' + '.$dx_counter.' seconds'));

			$events[$diagnosis->created_datetime] 	= array(
				'date'		=> $diagnosis->created_datetime,
				'name'		=> $diagnosis->name,
				'title'		=> $diagnosis->create_staff_name.' ('.$diagnosis->create_staff_designation.') added '.$diagnosis->certainty.' diagnosis',
				'details'	=> '<i style="color:rgb(90,90,90)">'.$diagnosis->code.' '.$diagnosis->name.'</i>',
				'location'	=> $diagnosis->location
			);
		}

		// Laboratory investigations

		foreach($this->getServices('Laboratory') as $labtest) {

			$lab_details 	= '';
			$lab_count		= 0;
			foreach($labtest['details'] as $details) {
				$lab_details .= ++$lab_count.'. <b>'.$details['name'].'</b>';
				if(strlen($details['report']) > 3) {
					$lab_details .= '<br /><i style="color:rgb(90,90,90)">'.$details['report'].'</i>';
				}
				$lab_details .= '<br />';
			}
			
			$events[$labtest['create_date']] 	= array(
				'date'		=> $labtest['create_date'],
				'name'		=> 'Investigation',
				'title'		=> $labtest['create_staff_name'].' ('.$labtest['create_staff_designation'].') ordered laboratory investigation',
				'details'	=> $lab_details,
				'location'	=> $labtest['location']
			);
		}

		// Radiology investigations

		foreach($this->getServices('Radiology') as $radiotest) {

			$radio_details 	= '';
			$radio_count		= 0;
			foreach($radiotest['details'] as $details) {
				$radio_details .= ++$radio_count.'. <b>'.$details['name'].'</b>';
				if(strlen($details['report']) > 3) {
					$radio_details .= '<br /><i style="color:rgb(90,90,90)">'.$details['report'].'</i>';
				}
				$radio_details .= '<br />';
			}
			
			$events[$radiotest['create_date']] 	= array(
				'date'		=> $radiotest['create_date'],
				'name'		=> 'Investigation',
				'title'		=> $radiotest['create_staff_name'].' ('.$radiotest['create_staff_designation'].') ordered radiology investigation',
				'details'	=> $radio_details,
				'location'	=> $radiotest['location']
			);
		}

		// Procedures

		foreach($this->getServices('Procedure') as $procedure) {

			$proc_details 	= '';
			$proc_count		= 0;
			foreach($procedure['details'] as $details) {
				$proc_details .= ++$proc_count.'. <b>'.$details['name'].'</b>';
				if(strlen($details['report']) > 3) {
					$proc_details .= '<br /><i style="color:rgb(90,90,90)">'.$details['report'].'</i>';
				}
				$proc_details .= '<br />';
			}
			
			$events[$procedure['create_date']] 	= array(
				'date'		=> $procedure['create_date'],
				'name'		=> 'Investigation',
				'title'		=> $procedure['create_staff_name'].' ('.$procedure['create_staff_designation'].') requested a procedure',
				'details'	=> $proc_details,
				'location'	=> $procedure['location']
			);
		}

		// Prescriptions

		foreach($this->getServices('Prescription') as $prescription) {

			$presc_details 	= '';
			$presc_count	= 0;
			foreach($prescription['details'] as $details) {
				$presc_details .= ++$presc_count.'. <b>'.$details['name'].'</b>';
				if(strlen($details['report']) > 3) {
					$presc_details .= '<br /><i style="color:rgb(90,90,90)">'.$details['report'].'</i>';
				}
				$presc_details .= '<br />';
			}
			
			$events[$prescription['create_date']] 	= array(
				'date'		=> $prescription['create_date'],
				'name'		=> 'Investigation',
				'title'		=> $prescription['create_staff_name'].' ('.$prescription['create_staff_designation'].') prescribed medications',
				'details'	=> $presc_details,
				'location'	=> $prescription['location']
			);
		}

		ksort($events);

		return $events;

	}

	public function getServices($service) {
		global $app;

		if($service == 'Procedure') {
			$table_name 	= 'clinical_procedures';
			$details_table	= 'clinical_procedure_items';
		} elseif($service == 'Prescription') {
			$table_name 	= 'clinical_prescriptions';
			$details_table	= 'clinical_prescription_items';
		} else {
			$table_name 	= 'clinical_'.strtolower($service);
			$details_table	= 'clinical_'.strtolower($service).'_items';
		}
		
		$services		= array();

		$request_str	= "SELECT $table_name.id, 
		personnel_persons.name create_staff_name,
		personnel_employees.designation create_staff_designation,
		$table_name.created_datetime,

		clinical_visit_locations.clinic_id,
		clinical_visit_locations.ward_id,

		IF (clinical_visit_locations.clinic_id > 1, (SELECT name FROM location_locations JOIN clinical_clinics  WHERE clinic_id = clinical_visit_locations.clinic_id LIMIT 1), 
		(SELECT name FROM location_locations JOIN clinical_wards ON clinical_wards.location_id = location_locations.id WHERE ward_id = clinical_visit_locations.ward_id LIMIT 1)) AS location

		FROM $table_name 
		JOIN clinical_visit_locations ON clinical_visit_locations.id = $table_name.visit_location_id
		JOIN personnel_users ON personnel_users.id = $table_name.created_user_id
		JOIN personnel_persons ON personnel_persons.id = personnel_users.person_id
		JOIN personnel_employees ON personnel_employees.person_id = personnel_users.person_id
		
		WHERE $table_name.visit_id = $this->visit_id ";

		$requests		= $app->read($request_str);

		foreach($requests as $request) {
			$request_id	= $request->id;
			$details 	= array();

			if($service == 'Prescription' || $service == 'Pharmacy') {
				$details_str= "SELECT name, uom, dose, times, days
				FROM $details_table WHERE parent_id = $request_id ";
			} else {
				$details_str= "SELECT name, report
				FROM $details_table WHERE parent_id = $request_id ";
			}

			$details_datas = $app->read($details_str);

			foreach($details_datas as $details_data) {
				if($service == 'Prescription' || $service == 'Pharmacy') {
					$details[]	= array(
						'name'			=> $details_data->name,
						'report'		=> '&nbsp; &nbsp; '.$details_data->dose.' '.$details_data->uom.', '.$details_data->times.' times a day for '.$details_data->days.' days'
					);
				} else {
					$details[]	= array(
						'name'			=> $details_data->name,
						'report'		=> $details_data->report
					);
				}
			}

			if($request->ward_id) {
				$location 	= new Ward($request->ward_id);
			} else {
				$location	= new Clinic($request->clinic_id);
			}
			$request->location	= $location->location()->name;

			$services[] 	= array(
				'create_date'				=> $request->created_datetime,
				'create_staff_name'			=> $request->create_staff_name,
				'create_staff_designation'	=> $request->create_staff_designation,
				'details'					=> $details,
				'location'					=> $request->location
			);
		}

		return $services;
	}

	public function getNotes() {
		global $app;

		$notes = array();

		$str = "SELECT 
		clinical_notes.subtype name,
		clinical_notes.notes,
		clinical_notes.created_user_id,
		clinical_notes.created_datetime,
		clinical_notes.visit_location_id,
		personnel_persons.name create_staff_name,
		personnel_employees.designation create_staff_designation,
		clinical_visit_locations.clinic_id,
		clinical_visit_locations.ward_id,
		
		IF (clinical_visit_locations.clinic_id > 1, (SELECT name FROM location_locations JOIN clinical_clinics  WHERE clinic_id = clinical_visit_locations.clinic_id LIMIT 1), 
		(SELECT name FROM location_locations JOIN clinical_wards ON clinical_wards.location_id = location_locations.id WHERE ward_id = clinical_visit_locations.ward_id LIMIT 1)) AS location

		FROM clinical_notes
		JOIN clinical_visit_locations ON clinical_visit_locations.id = clinical_notes.visit_location_id
		JOIN personnel_users ON personnel_users.id = clinical_notes.created_user_id
		JOIN personnel_employees ON personnel_employees.person_id = personnel_users.person_id
		JOIN personnel_persons ON personnel_persons.id = personnel_users.person_id
		WHERE clinical_visit_locations.visit_id = $this->visit_id ";
		foreach($app->read($str) as $row) {
			if($row->ward_id) {
				$location 	= new Ward($row->ward_id);
			} else {
				$location	= new Clinic($row->clinic_id);
			}
			$row->location	= $location->location()->name;
			$notes[] = (object) $row;
		};
		return $notes;
	}

	public function getFolioArray()  {
		global $app, $user, $now, $client;
		$customer 					= $this->customer();
		$patient 					= $customer->patient();
		$person 					= $customer->person();
		$age 						= date('Y') - date('Y', strtotime($person->birthdate));
		$visit 						= $this->visit();

		$folio						= array (
			'FacilityCode'			=> $client->nhif_facility_code,
			'ClaimYear'				=> (int) date('Y', strtotime($this->date)),
			'ClaimMonth'			=> (int) date('m', strtotime($this->date)),
			'FolioNo'				=> (int) $this->folio_no,
			'SerialNo'				=> $client->nhif_facility_code.'/'.date('Y', strtotime($this->date)).'/'.date('m', strtotime($this->date)).'/'.$this->folio_no,
			'CardNo'				=> $customer->card_no,
			'FirstName'				=> $person->firstname(),
			'LastName'				=> $person->lastname(),
			'Gender'				=> $person->sex,
			'DateOfBirth'			=> $person->birthdate,
			'Age'					=> (int) $age,
			'TelephoneNo'			=> str_replace('+255', '0', $person->phone),
			'PatientFileNo'			=> (string) $patient->code,
			'BillNo'				=> (string) $this->folio_id,
			//'PatientFile'			=> $this->getNotesPDF(1),
			'ClinicalNotes'			=> $this->generateNotesHTML(1),
			'AuthorizationNo'		=> $this->authorization_no,
			'ClaimFile'				=> base64_encode($this->generateFormPDF(1)),
			'AttendanceDate'		=> $app->date($visit->created_datetime),
			'PatientTypeCode'		=> $this->subtype == 'Inpatient' ? 'IN' : 'OUT',
			'DateAdmitted'			=> $this->subtype == 'Inpatient' ? $app->date($visit->checkin_datetime) : null,
			'DateDischarged'		=> $this->subtype == 'Inpatient' ? $app->date($visit->checkout_datetime) : null,
			'PractitionerNo'		=> $this->getClinician()->person()->getIdentityByCode('MCT')->number,
			'DelayReason'			=> '',
			'LateAuthorizationReason' => '',
			'LateSubmissionReason'  => $this->requested_notes,
			'AmountClaimed'			=> (float) $this->getAmountClaimed(),
			'FolioDiseases'			=> $this->getDiseasesArray(),
			'FolioItems'			=> $this->getItemsArray(),
			
			'CreatedBy'				=> $user->person()->name,
			'DateCreated'			=> $app->date($now),
			'LastModifiedBy'		=> null,
			'LastModified'			=> null
		);

		$folios[]					= $folio;

		$data['entities']			= $folios;

		return $data;
	}

	function getAmountClaimed() {
		$amount 	= 0;
		foreach($this->getFolioItems() as $folio_services) {
			foreach($folio_services as $folio_service) {
				$amount	+= $folio_service->price;
			}
		}
		return $amount;
	}

	function getDiseasesArray() {
		global $app, $now;

		$diagnoses	= array();
		$done 		= array();

		foreach($app->read("SELECT * FROM clinical_diagnoses WHERE visit_id = $this->visit_id AND status = 1") as $diagnosis) {
			$certainty 				= $diagnosis->certainty == 'Confirmed' ? 'Final' : 'Provisional';
			$code 					= $diagnosis->code;
			if($diagnosis->created_user_id) {
				$created_user 		= new User($diagnosis->created_user_id);
			} else {
				$created_user 		= $this->getClinician();
			}
			
			$combo 					= $certainty.$code;
			if(in_array($combo, $done)) continue;
			$diagnoses[]	= array (
				'DiseaseCode'		=> $code,
				'Status'			=> $certainty,
				'Remarks'			=> null,
				'CreatedBy'			=> $created_user->person()->name,
				'DateCreated'		=> $app->date($now),
				'LastModifiedBy'	=> null,
				'LastModified'		=> null
			);
			$done[]					= $combo;
		}
		return $diagnoses;
	}

	function getSubtotal() {
		global $app;
		$sum = 0;
		$sql = "SELECT SUM(price) AS subtotal FROM billing_folio_items WHERE parent_id = $this->id ";
		foreach($app->read($sql) as $row) {
			$sum += $row->subtotal;
		}
		return $sum;
	}

	function getClaimedAmount($insurance_id=0) {
		global $app;
		$sum = 0;
		if(!$insurance_id) $insurance_id = $this->insurance_id;

		$sql = "SELECT billing_folio_items.price
		FROM billing_folio_items 
		WHERE billing_folio_items.status = 1 
		AND billing_folio_items.parent_id = $this->id 
		AND billing_folio_items.insurance_id = $insurance_id 
		AND billing_folio_items.subtype IN ('Consultation', 'Laboratory', 'Radiology', 'Procedure', 'Pharmacy', 'Prescription', 'Admission')";

		foreach($app->read($sql) as $row) {
			$sum += $row->price;
		}
		return $sum;
	}
}

?>