<?php 

$root           = '../../../';
$submodule_code = 'Claims';
require($root.'config/config.php');

$folio          = new Folio($get->id);
$visit          = new Visit($folio->visit_id);
$customer       = new Customer($folio->personnel_id);
$patient        = $customer->patient();
$person         = $customer->person();
$readonly       = $folio->folio_no > 30000 ? '' : ' readonly ';

$tabs           = array (
    array (
        'id'    => 'folio', 
        'name'  => 'Folio details', 
        'title' => 'Folio details', 
        'icon'  => 'fa-solid fa-address-card',
        'css'   => 'active'
    ),
    array (
        'id'    => 'items', 
        'name'  => 'Folio items', 
        'title' => 'Folio items', 
        'icon'  => 'fa-solid fa-list-check'
    ),
    array (
        'id'    => 'notes', 
        'name'  => 'Clinical notes', 
        'title' => 'Clinical notes', 
        'icon'  => 'fa-solid fa-notes-medical',
        'attr'  => 'onclick="loadContentOnce(\'PatientNotes_container\', \''.$root.'pages/clinical/patient/notes/list.php?patient_id='.$patient->id.'&visit_id='.$visit->id.'&action=create&r='.$app->root($r).'\')"'
    ),
    array (
        'id'    => 'diagnoses', 
        'name'  => 'Diagnoses', 
        'title' => 'Diagnoses', 
        'icon'  => 'fa-solid fa-stethoscope',
        'attr'  => 'onclick="loadContentOnce(\'PatientDiagnoses_container\', \''.$root.'pages/clinical/patient/diagnoses/list.php?patient_id='.$patient->id.'&visit_id='.$visit->id.'&action=create&claim=1&r='.$app->root($r).'\')"'
    )
);

$modal_options      = array (
    'tabs'          => $tabs,
    'title'         => '<span class="fa-solid fa-bolt fa-fw"></span>&nbsp; Update folio details',
    'left_html'     => $customer->renderBio(),
    'cancel_label'  => '<span class="fa fa-arrow-left fa-fw"></span> Exit '
);

$form_options       = array (
    'form_action'   => 'edit_process.php'
);

echo $app->renderModalOpen($modal_options, 0);
echo $app->renderFormOpen($form_options);
echo $app->renderWizardNavigation($tabs);
echo $app->renderWizardOpen();

echo $app->renderWizardNext('folio', '', 'active show');

echo '<div class="card p-4 mb-4 bg-primary" style="--bs-bg-opacity: .05;">';

echo $app->renderInputHidden('id', $get->id);
echo $app->renderInputHidden('folio_id', $folio->id);

echo $app->renderInputText('code', $patient->code(), '', 'form-control-plaintext grid-control', 'required readonly', 'File number');
echo $app->renderInputText('name', $person->name, '', 'form-control-plaintext grid-control', '', 'Patient name');
echo $app->renderInputText('birthdate', $app->date($person->birthdate).' ('.$person->age().' ago)', '', 'form-control-plaintext grid-control', '', 'Date of birth');
echo $app->renderInputText('address', $person->village()->name.', '.$person->district()->name.', '.$person->region()->name, '', 'form-control-plaintext grid-control', '', 'Home address');
echo $app->renderInputText('phone', $person->phone, '', 'form-control-plaintext grid-control', '', 'Phone number');
echo $app->renderInputText('scheme', $customer->scheme()->name, '', 'form-control-plaintext grid-control', '', 'NHIF Scheme');
echo '<div id="next_folio_no_container">';
echo $app->renderInputText('folio_no', $folio->folio_no, '', 'line-control grid-control'.$readonly, '', 'Folio number');
echo '</div>';
echo $app->renderInputText('card_no', $visit->card_no, '', 'line-control grid-control', '', 'Card number');
echo $app->renderInputText('authorization_no', $visit->authorization_no, '', 'line-control grid-control', '', 'Authorization');
echo $app->renderInputSelect('visit_type', $folio->subtype, '', 'line-control grid-control', '', array('Outpatient', 'Inpatient'), '1', '2', '3', 'Folio type');
echo $app->renderInputText('checkin', $app->date($visit->checkin_datetime), '', 'line-control grid-control date-picker', '', 'Check-in date');
echo $app->renderInputText('checkout', $app->date($visit->checkout_datetime), '', 'line-control grid-control date-picker', '', 'Check-out date');
echo $app->renderInputTextarea('late_submission_reason', $folio->requested_notes, '', 'line-control grid-control', '', 'Late submit reason');

echo '</div>';

echo '<a class="btn btn-primary" onclick="$(\'#next_folio_no_container\').html(loading_bar).load(\'get_next_folio_no.php?id='.$get->id.'\')"> <span class="fa-regular fa-rotate fa-fw"></span> Reset folio number </a>';

echo $app->renderWizardNext('items');

$options            = array (
    'columns'       => array (
        array('name'=>'Code', 'css'=>'w-12 text-end'),
        array('name'=>'Details', 'css'=>''),
        array('name'=>'Approval', 'css'=>'w-10'),
        array('name'=>'Qty', 'css'=>'w-10 text-center'),
        array('name'=>'Price', 'css'=>'w-12 money text-end'),
        array('name'=>'Status', 'css'=>'w-5 text-center'),
    ),
    'css'           => 'table-containerless'
);

echo '<div class="border p-4 mb-4 rounded bg-primary" style="--bs-bg-opacity: .05;">';

echo $app->renderTableOpen($options);

foreach($folio->items() as $item) {
    $item_obj           = new Item($item->item_id);
    $color              = $item->status ? '' : 'text-gray5';

    echo '<tr class="'.$color.'">
    <td>'.$app->renderInputText('items['.$item->id.'][nhif_code]', $item_obj->code6, '', 'line-control text-end fw-semibold '.$color.' ').'</td>
    <td class="">
    '.$app->renderInputHidden('items['.$item->id.'][id]', $item->id).'
    '.$app->renderInputHidden('items['.$item->id.'][item_id]', $item->item_id).'
    <span class="'.$color.' fw-bold">'.$item->name.' ('.$item->uom.')</span>
    <div class="'.(strlen($color) ? $color : 'text-muted').'">'.$item_obj->name_alt.'</div>
    </td>
    <td>'.$app->renderInputText('items['.$item->id.'][approval_no]', $item->approval_no, '', 'line-control text-end '.$color.'').'</td>
    <td>'.$app->renderInputNumber('items['.$item->id.'][quantity]', $item->quantity, '', 'line-control text-end '.$color.' ').'</td>
    <td>'.$app->renderInputText('items['.$item->id.'][unit_price]', $app->formatMoney($item->unit_price), '', 'line-control text-end money fw-bold '.$color.' ').'</td>
    <td class="text-end"><div class="form-check form-switch h5 my-1 text-end"><input name="items['.$item->id.'][active]" value="1" role="button" class="form-check-input text-end" type="checkbox" '.($item->status ? 'checked' : '').' /></div></td>
    </tr>';
}

echo $app->renderTableClose($options);

echo '</div>';

echo $app->renderWizardNext('notes', '', 'active show');

echo '<div id="PatientNotes_container"></div>';

echo $app->renderWizardNext('diagnoses');
echo '<div id="PatientDiagnoses_container"></div>';

echo $app->renderWizardClose();
echo $app->renderFormClose($form_options);
echo $app->renderModalClose($modal_options);

?>

<script>

var loaded_pages    = [];

function loadContentOnce(container, url) {
    if(jQuery.inArray(container, loaded_pages) < 0) {
        $('#'+container).html(loading_circle).load(url, function() {
            loaded_pages.push(container);
            app.initializeControls();
        });
    }
}

</script>