<?php 

$root           = '../../../';
$submodule_code = 'Claims';
require($root.'config/config.php');
require($root.'api/nhif/NHIF.php');

$visit          = new Visit($get->visit_id);
$folio          = new Folio($get->folio_id);

$options            = array (
    'columns'       => array (
        array('name'=>'Date', 'css'=>'w-15'),
        array('name'=>'Details', 'css'=>''),
        array('name'=>'Qty', 'css'=>'w-5 text-center'),
        array('name'=>'Amount', 'css'=>'w-10 money text-end'),
        array('name'=>'Status', 'css'=>'w-15')
    ),
    'css'           => 'table-containerless'
);

echo '<div class="border p-3 rounded bg-primary" style="--bs-bg-opacity: .05;">';

echo $app->renderTableOpen($options);

$clinical_notes = $visit->getClinicalNotes();
$provisional_dx = $visit->getProvisionalDiagnoses();
$confirmed_dx   = $visit->getConfirmedDiagnoses();

$clinician_signature_error    = 0;
foreach($clinical_notes as $row) {
    $user           = new User($row->created_user_id);
    $employee       = $user->personnel();
    $signature      = $app->getAttachmentByCode('Employee', $employee->id, 'EmployeeSignature');
    if(!$signature->id) {
        $person     = $employee->person();
        $clinician_signature_error = $person->firstname().' signature?';
    }
}

$visit_signature    = $app->getAttachmentByCode('Visit', $visit->id, 'PatientSignature');
$items_total        = 0;

foreach($folio->items() as $item) {

    $color          = 'text-success';
    $message        = 'OK';
    $item_obj       = new Item($item->item_id);

    if($item->insurance_id != $folio->insurance_id) {
        $item_insurance = new Insurance($item->insurance_id);
        $color      = 'text-gray5';
        $message    = $item_insurance->code;
    } elseif(!$visit->card_no) {
        $color      = 'text-danger';
        $message    = 'Card number?';
    } elseif(!$visit->authorization_no) {
        $color      = 'text-danger';
        $message    = 'Authorization?';
    } elseif(!$clinical_notes) {
        $color      = 'text-danger';
        $message    = 'Clinical notes?';
    } elseif($clinician_signature_error) {
        $color      = 'text-danger';
        $message    = $clinician_signature_error;
    } elseif(!$visit_signature->id) {
        $color      = 'text-danger';
        $message    = 'Patient signature?';
    } elseif(!$provisional_dx && !$confirmed_dx) {
        $color      = 'text-danger';
        $message    = 'Diagnoses?';
    } elseif(!$item_obj->code6) {
        $color      = 'text-danger';
        $message    = 'Item mapping?';
    } elseif($item->status == 0) {
        $color      = 'text-gray5';
        $message    = 'Not claimed!';
    } else {
        $items_total += $item->price;
    }

    echo '<tr>
    <td class="'.$color.'"><span class="'.$color.'">'.$app->date($item->created_datetime).'</span></td>
    <td class="'.$color.' py-3"><span class="fw-bold '.$color.'">'.$item->name.' ('.$item->uom.')</span>
    <div class="'.($color == 'text-gray5' ? $color : 'text-muted').'">'.$item_obj->code6.' &middot; '.$item_obj->name_alt.'</div>';

    if($item->approval_no) {
        echo '<div class="fw-semibold"><span class="fa-solid fa-check"></span> Approval: '.$item->approval_no.'</div>';
    }
    
    echo '</td>
    <td class="'.$color.' text-center"><span class="'.$color.'">'.$item->quantity.'</span></td>
    <td class="text-end fw-bold "><span class="'.$color.'">'.number_format($item->price, 2).'</span></td>
    <td class="'.$color.'"><span class="'.$color.'">'.$message.'</span></td>
    </tr>';
}

echo '<tr>
<td></td>
<td></td>
<td></td>
<td class="text-end fw-bold ">'.number_format($items_total, 2).'</td>
<td></td>
</tr>';

echo $app->renderTableClose($options);

echo '</div>';

?>