<?php 

$root               = '../../../';
$submodule_code     = 'Claims';
require($root.'config/config.php');
require($root.'api/nhif/NHIF.php');

if($get->progress == 'Summary') {
    require('summary.php');
    die();
}

$nhif               = new NHIF();
$keyword            = $get->keyword;

$start_date         = strlen($get->start_date) ? $app->morning($get->start_date) : $app->morning($app->monthStart());
$end_date           = strlen($get->end_date) ? $app->evening($get->end_date) : $app->evening($app->monthEnd());
$date_filter        = " AND billing_folios.created_datetime BETWEEN '$start_date' AND '$end_date' ";
$insurance_filter   = $get->insurance_id ? " AND billing_folios.insurance_id = $get->insurance_id " : " ";
$visits_filter      = strlen($get->visit_type) ? " AND billing_folios.subtype = '$get->visit_type' " : "";
$progress_filter    = strlen($get->progress) ? " AND billing_folios.progress = '$get->progress' " : "";

if($get->progress == 'Received') {

    $claim_year     = date('Y', strtotime($start_date));
    $claim_month    = date('n', strtotime($start_date));
    $claim_date 	= $claim_year.'-'.$claim_month.'-01';

    if($keyword == 'sync_received_folios') {
        echo $nhif->syncSubmittedFolios($claim_year, $claim_month);
        $keyword    = '';
    }
		
    $date_filter    = " AND billing_folios.year = $claim_year AND billing_folios.month = $claim_month ";
    $sql            = "SELECT progress_datetime FROM billing_folios WHERE received = 1 AND billing_folios.year = $claim_year AND billing_folios.month = $claim_month ORDER BY progress_datetime DESC LIMIT 1";
    
    $progress_datetime  = 0;
    $received_folios    = $app->read($sql);
    foreach($received_folios as $row) $progress_datetime = $row->progress_datetime;

    if($progress_datetime) {
        echo '<div class="border p-3 mb-3 rounded bg-success" style="--bs-bg-opacity: .1;">
        <h5 class="fw-bold mb-3"> Last updated '.$app->age($progress_datetime).' ago </h5>
        <p>The list of successfully received folios in '.date('F Y').' was updated on '.$app->datetime($progress_datetime).'
        <div class="mt-3"><a class="btn btn-primary" onclick="syncSubmittedFolios()"> <span class="fa-regular fa-sync-alt fa-fw"></span> Update received folios now</a></div></p>
        </div>';
    } else {
        echo '<div class="border p-3 mb-3 rounded bg-danger" style="--bs-bg-opacity: .1;">
        <h5 class="fw-bold mb-3"> List not updated </h5>
        <p>The list of successfully received folios in '.date('F Y').' was never updated.
        <div class="mt-3"><a class="btn btn-primary" onclick="syncSubmittedFolios()"> <span class="fa-regular fa-sync-alt fa-fw"></span> Update received folios now</a></div></p>
        </div>';
    }
}

$obj                = new Folio();  
 
if(strlen($keyword)) {
    if(is_numeric($keyword)) {
        $sql                = "SELECT 
        billing_folios.id
        FROM billing_folios
        WHERE billing_folios.status  = 1
        $date_filter
        $insurance_filter
        $visits_filter
        $progress_filter
        AND (
            billing_folios.id = $keyword
            OR billing_folios.code = '$keyword'
            OR billing_folios.card_no = '$keyword'
            OR billing_folios.folio_no = '$keyword'
            OR billing_folios.authorization_no = '$keyword'
        )
        ORDER BY id DESC ";
    } else {
        $sql                = "SELECT 
        billing_folios.id
        FROM billing_folios
        WHERE billing_folios.status  = 1
        $date_filter
        $insurance_filter
        $visits_filter
        $progress_filter
        AND billing_folios.name LIKE '%$keyword%'
        ORDER BY id DESC ";
    }
} else {
    $sql    = "SELECT 
    billing_folios.id
    FROM billing_folios
    WHERE billing_folios.status = 1
    $date_filter
    $insurance_filter
    $visits_filter
    $progress_filter
    ORDER BY id DESC ";
}

$data               = $app->pagenate($app->read($sql));

$options            = array (
    'columns'       => array (
        array('name'=>'Date', 'css'=>'w-10'),
        array('name'=>'Folio', 'css'=>'w-15'),
        array('name'=>'Patient', 'css'=>''),
        array('name'=>'Subtotal', 'css'=>'w-10 money text-end'),
        array('name'=>'Claimed', 'css'=>'w-10 money text-end'),
        array('name'=>'Status', 'css'=>'w-15 text-center'),
        array('name'=>'Options', 'css'=>'w-15 text-end')
    ),
    'css'           => 'bs-table'
);

$current_date			= $start_date;
$current_period			= date('M Y', strtotime($start_date));

echo $app->renderReportNavigation (
	array (
		'period'		=> 'Month',
		'section_id'	=> 'summary',
		'section_title'	=> 'Monthly overview',
		'current_period'=> $current_period,
		'current_date'	=> $current_date
	)
);

echo $app->renderTableOpen($options);

foreach($data->rows as $row) {
    $obj            = new Folio($row->id);
    $visit          = new Visit($obj->visit_id);
    $visit_location = new VisitLocation($visit->getLastVisitLocationId());

    if($visit_location->ward_id && $obj->subtype == 'Outpatient') {
        $obj->set('subtype', 'Inpatient');
        $obj->read();
    }

    if($visit_location->ward_id && !$obj->folio_no) {
        $obj->set('folio_no', $obj->getNextFolioNo());
        $obj->read();
    }

    $link_options   = array ('name'=>$obj->code.' &middot; '.$obj->name.'<div class="text-muted fw-normal mt-1">Card '.$obj->card_no.'</div>', 'id'=>$obj->id, 'modal'=>'modal_xl');
    echo '<tr>
    <td class="fw-semibold text-muted">'.$app->date($obj->date).'</td>
    <td class="fw-semibold py-3">'.($obj->subtype == 'Inpatient' ? 'IPD' : 'OPD').' &middot; '.$app->formatCode($obj->folio_no, 6).'<div class="text-muted fw-normal mt-1">Auth '.$obj->authorization_no.'</div></td>
    <td class="py-2">'.$app->renderLinkOpen($link_options).'</td>
    <td class="text-end text-muted fw-bold">'.number_format($obj->getSubtotal(), 2);
    
    echo '</td>
    <td class="text-end text-muted fw-bold">'.number_format($obj->getClaimedAmount(),2);
    if($obj->received) echo '<div class="text-success fw-normal">'.$obj->reference_no.'</div>';
    echo '</td>
    <td class="px-3">'.$obj->renderButtonProgress($link_options).'</td>
    <td>';

    if($visit_location->ward_id && $visit->admitted) {
        echo '<a class="bs-tooltip btn btn-outline-danger rounded-pill px-3" title="Admitted"> <span class="fa-regular fa-lock"></span> Admitted </a>';
    } else {
        if(strlen($obj->progress == 'Returned')) {
            echo '<a class="remote-modal bs-tooltip btn btn-default" title="'.$obj->returned_notes.'" data-bs-toggle="modal" data-bs-target="#modal_lg" href="form.php?action=open&visit_id='.$obj->visit_id.'&folio_id='.$obj->id.'"> <span class="fa-regular fa-info-circle"></span> </a>';
        } else {
            //echo $obj->renderLinkOpen($link_options);
        }
      
        if($obj->progress != 'Forwarded' && $obj->progress != 'Received')  {
            if($user->tagged('claimant')) echo '<a class="remote-modal bs-tooltip btn btn-default" title="Accomodation" data-bs-toggle="modal" data-bs-target="#modal_lg" href="add_form.php?action=create&visit_id='.$obj->visit_id.'&folio_id='.$obj->id.'"> <span class="fa-solid fa-plus"></span> </a>';
            echo $obj->renderLinkUpdate($link_options);
        }

        echo $obj->renderLinkAction($link_options);
    }

    echo '</td>
    </tr>';
}


echo $app->renderTableClose();

echo $data->pagination;

?>

<script>

function syncSubmittedFolios() {
    var temp_keyword    = $('keyword').val();
    $('#keyword').val('sync_received_folios');
    app.loadList();
    $('#keyword').val(temp_keyword);
}

</script>