<?php 

$root               = '../../../';
$submodule_code     = 'Claims';
require($root.'config/config.php');

$folio_year         = $get->year;
$folio_month        = $get->month;

$claims             = array();

$folios_sql         = "SELECT 
billing_folio_items.subtype,
billing_folios.folio_no, 
billing_folios.code, 
billing_folio_items.price 
FROM billing_folio_items 
JOIN billing_folios ON billing_folio_items.parent_id = billing_folios.id 
WHERE billing_folios.year   = $folio_year
AND billing_folios.month    = $folio_month 
AND billing_folios.progress = 'Received'
AND billing_folio_items.insurance_id = $get->insurance_id
AND billing_folio_items.subtype IN ('Consultation', 'Laboratory', 'Radiology', 'Procedure', 'Pharmacy', 'Prescription', 'Admission')
AND billing_folio_items.status = 1";

$folios             = array();
$beneficiaries      = array();
$amount             = 0;
$claims             = array (
    'Consultation'  => 0,
    'Laboratory'    => 0,
    'Radiology'     => 0,
    'Procedure'     => 0,
    'Pharmacy'      => 0,
    'Admission'     => 0
);

$subtypes           = array_keys($claims);

foreach($app->read($folios_sql) as $row) {
    if(!in_array($row->code, $beneficiaries)) $beneficiaries[]  = $row->code;
    if(!in_array($row->folio_no, $folios)) $folios[]            = $row->folio_no;

    if(!in_array($row->subtype, $subtypes)) $row->subtype = 'Pharmacy';

    $claims[$row->subtype]  += $row->price;
    $amount                 += $row->price;
}

$html = $app->renderDocumentHeader();

$html .= $app->center('<h3>Monthly Claim Summary</h3>');

$html .= '<table class="w-100 px-10 py-7 border">
<tr><td class="border-bottom" style="width:30%">Name of facility</td><td class="fw-bold border-bottom" style="width:70%">'.$client->client_name.'</td></tr>
<tr><td class="border-bottom">Address</td><td class="fw-bold border-bottom">'.$client->client_address.'</td></tr>
<tr><td class="border-bottom">Ownership</td><td class="fw-bold border-bottom">'.$app->getSYstemSettings('nhif_facility_ownership', 'Government').'</td></tr>
<tr><td class="border-bottom">Facility code</td><td class="fw-bold border-bottom">'.$app->getSystemSettings('nhif_facility_code').'</td></tr>
<tr><td class="border-bottom">Claim month</td><td class="fw-bold border-bottom">'.date('F Y', strtotime($folio_year.'-'.$folio_month.'-01')).'</td></tr>
<tr><td class="border-bottom">Number of beneficiaries</td><td class="fw-bold border-bottom">'.number_format(count($beneficiaries)).'</td></tr>
<tr><td class="border-bottom">Number of folios</td><td class="fw-bold border-bottom">'.number_format(count($folios)).'</td></tr>
<tr><td class="border-bottom">Total amount claimed</td><td class="fw-bold border-bottom">TZS '.number_format($amount).'</td></tr>
</table>';

$html .= $app->center($app->h3('Breakdown of Amount Claimed'));

$html .= '<table class="w-100 px-10 py-7 border">';
foreach($subtypes as $claim) {
    $html .= '<tr><td class="border-bottom" style="width:30%">'.$claim.'</td><td class="fw-bold border-bottom" style="width:70%">TZS '.number_format($claims[$claim]).'</td></tr>';
}
$html .= '<tr><td style="width:30%">Total</td><td class="fw-bold" style="width:70%">TZS '.number_format($amount).'</td></tr>';
$html .= '</table>';

$claimant_signature 	= $app->getAttachmentByCode('Employee', $user->personnel()->id, 'EmployeeSignature');
$claimant_sig_path 		= $claimant_signature->path();
if(!$claimant_signature->exists()) {
    die($app->renderAlertError('Claimant signature does not exists!', $user->person()->name));
}

$html .= '<br /><br /><table class="w-100 px-10 py-7 border">
<tr>
<td class="w-25 text-center border-0">Prepared by <div class="my-2 fw-bold">'.$user->person()->name.'</div></td>
<td class="w-25 text-center border-0">Designation <div class="my-2 fw-bold">'.$user->personnel()->designation.'</div></td>
<td class="w-25 text-center border-0">Contacts <div class="my-2 fw-bold">'.$user->person()->phone.'</div></td>
<td class="w-25 text-center border-0">Signature <div class="my-2 fw-bold"><img src="'.$app->encodeImage($claimant_sig_path).'" style="width:150px; height:100px" /></div></td>
</tr>
</table>';

$html .= '</div>';

echo $html;

$app->renderPDF($html, 'Claim summary');

?>