<?php 

$start_date         = strlen($get->start_date) ? $app->morning($get->start_date) : $app->morning($app->monthStart());
$end_date           = strlen($get->end_date) ? $app->evening($get->end_date) : $app->evening($app->monthEnd());

$folio_year         = date('Y', strtotime($start_date));
$folio_month        = date('m', strtotime($start_date));

$current_date		= $start_date;
$current_period		= date('M Y', strtotime($start_date));
$claims             = array();

$folios_sql         = "SELECT 
billing_folio_items.subtype,
billing_folios.folio_no, 
billing_folios.code, 
billing_folio_items.price 
FROM billing_folio_items 
JOIN billing_folios ON billing_folio_items.parent_id = billing_folios.id 
WHERE billing_folios.year   = $folio_year
AND billing_folios.month    = $folio_month 
AND billing_folios.progress = 'Received'
AND billing_folio_items.insurance_id = $get->insurance_id
AND billing_folio_items.subtype IN ('Consultation', 'Laboratory', 'Radiology', 'Procedure', 'Pharmacy', 'Prescription', 'Admission')
AND billing_folio_items.status = 1";

$folios             = array();
$beneficiaries      = array();
$amount             = 0;
$claims             = array (
    'Consultation'  => 0,
    'Laboratory'    => 0,
    'Radiology'     => 0,
    'Procedure'     => 0,
    'Pharmacy'      => 0,
    'Admission'     => 0
);

$subtypes           = array_keys($claims);

foreach($app->read($folios_sql) as $row) {
    if(!in_array($row->code, $beneficiaries)) $beneficiaries[]  = $row->code;
    if(!in_array($row->folio_no, $folios)) $folios[]            = $row->folio_no;

    if(!in_array($row->subtype, $subtypes)) $row->subtype = 'Pharmacy';

    $claims[$row->subtype]  += $row->price;
    $amount                 += $row->price;
}

echo $app->renderReportNavigation (
	array (
		'period'		=> 'Month',
		'section_id'	=> 'summary',
		'section_title'	=> 'Monthly overview',
		'current_period'=> $current_period,
		'current_date'	=> $current_date
	)
);

echo '<div class="border p-4 my-3 rounded bg-secondary" style="--bs-bg-opacity:0.05">';

echo $app->renderDocumentHeader();

echo $app->center($app->h3('Monthly Claim Summary'));

echo '<table class="table border-0 table-hover">
<tr><td class="py-3" style="width:30%">Name of facility</td><td class="py-3 fw-bold" style="width:70%">'.$client->client_name.'</td></tr>
<tr><td class="py-3">Address</td><td class="py-3 fw-bold">'.$client->client_address.'</td></tr>
<tr><td class="py-3">Ownership</td><td class="py-3 fw-bold">'.$app->getSYstemSettings('nhif_facility_ownership', 'Government').'</td></tr>
<tr><td class="py-3">Facility code</td><td class="py-3 fw-bold">'.$app->getSystemSettings('nhif_facility_code').'</td></tr>
<tr><td class="py-3">Claim month</td><td class="py-3 fw-bold">'.date('F Y', strtotime($start_date)).'</td></tr>
<tr><td class="py-3">Number of beneficiaries</td><td class="py-3 fw-bold">'.number_format(count($beneficiaries)).'</td></tr>
<tr><td class="py-3">Number of folios</td><td class="py-3 fw-bold">'.number_format(count($folios)).'</td></tr>
<tr><td class="py-3">Total amount claimed</td><td class="py-3 fw-bold">TZS '.number_format($amount).'</td></tr>
</table>';

echo $app->center($app->h3('Breakdown of Amount Claimed'));

echo '<table class="table border-0 table-hover">';
foreach($subtypes as $claim) {
    echo '<tr><td class="py-3" style="width:30%">'.$claim.'</td><td class="py-3 fw-bold" style="width:70%">TZS '.number_format($claims[$claim]).'</td></tr>';
}
echo '<tr><td class="py-3" style="width:30%">Total</td><td class="py-3 fw-bold" style="width:70%">TZS '.number_format($amount).'</td></tr>';
echo '</table>';

$claimant_signature 	= $app->getAttachmentByCode('Employee', $user->personnel()->id, 'EmployeeSignature');
$claimant_sig_path 		= $claimant_signature->path();
if(!$claimant_signature->exists()) {
    die($app->renderAlertError('Claimant signature does not exists!', $user->person()->name));
}

echo '<table class="table border-0 mt-5">
<tr>
<td class="w-25 text-center border-0">Prepared by <div class="my-2 fw-bold">'.$user->person()->name.'</div></td>
<td class="w-25 text-center border-0">Designation <div class="my-2 fw-bold">'.$user->personnel()->designation.'</div></td>
<td class="w-25 text-center border-0">Contacts <div class="my-2 fw-bold">'.$user->person()->phone.'</div></td>
<td class="w-25 text-center border-0">Signature <div class="my-2 fw-bold"><img src="'.$app->encodeImage($claimant_sig_path).'" style="width:150px; height:100px" /></div></td>
</tr>
</table>';

echo '</div>';

echo '<a target="blank" href="print_summary.php?year='.$folio_year.'&month='.$folio_month.'&insurance_id='.$get->insurance_id.'" class="btn btn-outline-primary btn-lg w-100 my-3"> <span class="fa-regular fa-file-pdf fa-fw"></span> Print Claim Summary </a></a>';

?>