<?php 

class Customer extends Crud {
	function __construct($id=0) {		
		$this->table        = 'billing_customers';
		$this->primary_key  = 'id';
		parent::__construct($id);
		if(!$id) {
			global $app;
			$this->scheme_obj 	= $app->getCashScheme();
			$this->scheme 		= $this->scheme_obj->code;
			$this->scheme_id 	= $this->scheme_obj->id;
			$this->insurance_obj 	= $this->scheme_obj->insurance();
			$this->insurance 	= $this->scheme_obj->insurance()->code;
			$this->insurance_id = $this->scheme_obj->insurance_id;
			$this->type 		= 'Individual';
		}
	}

	function person($person_id='') {
		$person_id 	= strlen($person_id) ? $person_id : $this->person_id;
		$person		= new Person($person_id);
		$person->personnel = 'Customer';
		return $person;
	}

	function patient() {
		if($this->personnel == 'Patient') {
			return new Patient($this->personnel_id);
		} else {
			return new Patient();
		}
	}

	function picture() {
		global $app, $root;
		$attachment = $app->getAttachmentByCode('Customer', $this->id, 'ProfilePicture');
		if($attachment->exists()) {
			return $attachment->renderOpenPath();
		} elseif($this->type == 'Company') {
			return $root.'assets/images/customers/company.png';
		} else {
			$person		= $this->person();
			$age_string = $person->age();
			$sex		= strtolower($person->sex);
			$picture 	= 'individual.png';
			$ages		= explode(' ', $age_string);
			
			if((count($ages) == 2) && ($sex == 'male' || $sex == 'female')) {
				$age_number		= $ages[0];
				$age_units		= $ages[1];
				$agegroup 	= 'baby';
				if($age_units 	== 'year' || $age_units 	== 'years') {
					if($age_number > 5) 	$agegroup = 'child';
					if($age_number > 18) 	$agegroup = 'adult';
					if($age_number > 60) 	$agegroup = 'old';
				}
				$picture	= $sex.'-'.$agegroup.'.png';
			}
			return $root.'assets/images/customers/'.$picture;
		}
	}

	function signature() {
		global $app, $root;
		$attachment = $app->getAttachmentByCode('Customer', $this->id, 'CustomerSignature');
		if($attachment->exists()) return $attachment->renderOpenPath();
		return $root.'assets/images/default/signature.png';
	}

	function formatCode() {
		$digits = 8;
		if(strlen($this->code) < 7) $digits = 6;
		return sprintf('%0'.$digits.'d', $this->code);
	}

	function insurance() {
		return new Insurance($this->insurance_id);
	}

	function scheme() {
		return new Scheme($this->scheme_id);
	}

	function renderProfileCard() {
		global $root, $app;
		$color 				= 'secondary';
		$person 			= $this->person();
		$button_options 	= array('id'=>$this->id, 'css'=>'btn btn-'.$color.' btn-lg');
		$buttons			= $app->renderButtonOpen($button_options).''.$app->renderButtonEdit($button_options).''.$app->renderButtonCopy($button_options).''.$app->renderButtonDelete($button_options);
		$person->sex 		= $person->sex ? $person->sex : 'ago';

		$html 				= '<div id="card_customer_'.$this->id.'" class="auto-flip bs-tooltip" title="Click card to flip">
		<div class="shadow shadow-lg card text-center front">
		<div class="card-img-top bg-'.$color.' text-white py-4" style="height:240px; min-height:240px">
		'.$buttons.'
		</div>
		<img src="'.$this->picture().'" class="rounded-circle mx-auto d-block bg-light" style="width:80%; max-width:200px; margin-top:-140px; border:8px solid white">
		<div class="card-body" style="min-height:180px;">
		
		<h5 class="fw-bold text-truncate">'.$person->name.'</h5>
		<h4 class="fw-bold"> '.$this->formatCode().' </h4>
		<h5>'.ucwords($person->age().' '.$person->sex).'</h5>
		<h5 class="text-muted text-truncate my-3"><span class="fa-regular fa-tags fa-fw"></span> '.$app->shortname($this->renderInsuranceScheme(), 30).' </h5>

		</div></div>
		
		<div class="shadow card text-center back">
		
		<div class="card-img-top bg-'.$color.' text-white py-4" style="height:240px; min-height:240px">
		<a class="btn btn-primary bs-tooltip" data-flip="card_customer_'.$this->id.'" title="Flip"> <span class="fa-solid fa-arrow-left"></span> </a>
		<div><h5> Registered </h5></div>
		<div>'.$app->datetime($this->created_datetime).'</div>
		<div>'.$this->createdUser()->person()->name.'</div>
		<div>'.$app->age($this->created_datetime).' ago </div>

		</div>
		<img src="'.$this->picture().'" class="rounded-circle mx-auto d-block bg-light" style="visibility:hidden; width:75%; max-width:190px; margin-top:-120px; border:7px solid white">
		<div class="card-body" style="min-height:180px;">
		<h5><span class="fa-light fa-phone fa-fw"></span></h5>
		<h4 class="fw-light">'.$person->phone.'</h4>
		<div class="fw-light"> '.$person->village()->name.' - '.$person->district()->name.' - '.$person->region()->name.'</div>
		</div></div></div>';
		
		return $html;
	}

	function renderBio($name='') {
		global $app;

		if($this->id) {
			$person = $this->person();
			$html	= '<img src="'.$this->picture().'" class="rounded-circle mx-auto d-block bg-light my-3" style="width:85%; max-width:185px; border:8px solid white" />
			<div class="px-3 me-2 text-white">
			<h4 class="fw-bold"> '.$this->formatCode().' </h4>
			<h5 class="fw-bold text-truncate">'.$person->name.'</h5>
			<h5>'.ucwords($person->age().' '.$person->sex).'</h5>
			<h6 class="text-truncate my-3"><span class="fa-regular fa-tags fa-fw"></span> '.$this->renderInsuranceScheme().'</h6>
			</div>';

		} else {
			$html = '<img src="'.$this->picture().'" class="rounded-circle mx-auto d-block bg-light my-3" style="width:75%; max-width:175px; border:8px solid white" />
			<div class="rounded p-3 bg-white mt-3 mb-4 me-2">'.
			$app->renderInputText('temp_customer_name', $name, 'Customer name', 'line-control', 'onchange="setCustomerBio()"').
			'<div class="row gx-2"><div class="col-6 my-3">'.
			$app->renderInputText('temp_customer_birthdate', '', 'Birthdate', 'line-control year-picker', 'onchange="setCustomerBio()"').
			'</div><div class="col-6 my-3">'.
			$app->renderInputSelect('temp_customer_sex', '', 'Sex', 'select-search line-control', 'onchange="setCustomerBio()"', array('Male', 'Female')).
			'</div><div class="col-12">'.
			$app->renderInputText('temp_customer_phone', '', 'Phone no', 'line-control', 'onchange="setCustomerBio()"').
			'</div></div>
			</div>';
		}
		
		return $html;
	}

	function renderInsuranceScheme() {
		$html = $this->scheme()->name;
		return $html;
	}

	function renderSaleProfile($customer_keyword='') {
		global $app;

		$html = '<div class="card bg-light mb-3" style="border:1px solid rgb(225,225,230)">
		<div class="row g-0">
		<div class="col-12 col-lg-4 text-center" style="margin-bottom:90px">
		<div class="rounded-top" style="height:100px;">
		<img src="'.$this->picture().'" class="rounded-circle mx-auto d-block bg-light" style="width:150px; max-width:190px; margin-top:20px; border:8px solid white">
		</div>
		</div><div class="col-12 col-lg-8">
		<div class="card-body pt-3">';

		if(strlen($customer_keyword)) {
			$person		= $this->person();
			$name		= $customer_keyword;
			$phone		= $person->phone;
			$email		= $person->email;
			$birthdate 	= date('Y-m-d');
			$sex		= $person->sex;
			$readonly 	= '';
		} else {
			$person		= $this->person();
			$name		= $person->name;
			$phone		= $person->phone;
			$email		= $person->email;
			$birthdate 	= $person->birthdate;
			$sex		= $person->sex;
			$readonly 	= 'readonly';
		}

			$html	.= '<div class="row"><div class="col-12 my-1">';
			$html 	.= $app->renderInputText('trans_customer_name', $name, 'Enter customer name', 'line-control form-control-lg '.$readonly.'');
			$html	.= '</div><div class="col-lg-6 my-1">';
			$html 	.= '<div class="row g-1"><div class="col-1 col-lg-2 pt-2"><span class="fa-light fa-venus-mars"></span></div> <div class="col-11 col-lg-10">';
			$html 	.= $app->renderInputSelect('trans_customer_sex', $sex, 'Choose sex', 'select-search line-control '.$readonly.'', '', array('Male', 'Female'));
			$html	.= '</div></div> </div><div class="col-lg-6 my-1">';
			$html 	.= '<div class="row g-1"><div class="col-1 col-lg-2 pt-1"><span class="fa-light fa-baby"></span></div> <div class="col-11 col-lg-10">';
			$html 	.= $app->renderInputText('trans_customer_birthdate', $birthdate, 'Date of birth', 'line-control date-picker '.$readonly.'');
			$html	.= '</div></div> </div><div class="col-lg-6 my-1">';
			$html 	.= '<div class="row g-1"><div class="col-1 col-lg-2 pt-1"><span class="fa-light fa-phone"></span></div> <div class="col-11 col-lg-10">';
			$html 	.= $app->renderInputText('trans_customer_phone', $phone, 'Phone number', 'line-control '.$readonly.'');
			$html	.= '</div></div> </div><div class="col-lg-6 my-1">';
			$html 	.= '<div class="row g-1"><div class="col-1 col-lg-2 pt-1"><span class="fa-light fa-envelope-open"></span></div> <div class="col-11 col-lg-10">';
			$html 	.= $app->renderInputText('trans_customer_email', $email, 'Email address', 'line-control '.$readonly.'');
			$html	.= '</div></div> </div><div class="col-lg-6 my-1 mt-2">';
			$html	.= '<a class="btn w-100 btn-outline-primary my-1" onclick="$(\'#customer_search_container\').slideDown(100);"> <span class="fa-light fa-arrow-left fa-fw"></span> Change customer</a>';
			$html	.= '</div></div>';
		
		$html .= '
		</div>
		</div>
		</div>
		</div>';

		$html .= $app->renderInputHidden('trans_customer_id', $this->id);
		$html .= $app->renderInputHidden('trans_customer_code', $this->code);

		return $html;
	} 

	function createIfNotExists($data) {
        if($data['id']) {
            return new Customer($data['id']);
        } else {
            $customer       = new Customer();
            $customer_id    = $customer->createWithMinimumData($data);
            return new Customer($customer_id);
        }
    }

	function createWithMinimumData(array $data) {
		
		// Create person
		$person 					= new Person();
        $person_id                  = $person->createWithMinimumData(
			array (
				'personnel'         => 'Customer',
				'name'              => $data['name'],
				'phone'             => $data['phone']
			)
		);
        $person                     = new Person($person_id);
		$customer					= new Customer();

        // If person created create customer
        if($person_id) {
            $customer_data              = $this->addCommonFields (
               array (
                    'person_id'         => $person_id,
                    'branch_id'         => 1,
                    'personnel'         => 'Customer',
                    'personnel_id'      => 0,
                    'code'              => $this->nextCode(),
                    'type'              => 'Individual',       
                    'subtype'           => '',
                    'category'          => '',
                    'subcategory'       => '',
                    'registration_no'   => '',
                    'insurance'         => '',
                    'insurance_id'      => $data['insurance_id'],
                    'card_no'           => '',
                    'card_expiry'       => 0,
                    'membership_no'     => '',
                    'authorization_no'  => '',
                    'supplementary'     => '',
                    'scheme'            => '',
                    'scheme_id'         => $data['scheme_id'],
                    'scheme_code'       => '',
                    'product'           => '',
                    'product_id'        => 0,
                    'product_code'      => '',
                    'employer'          => '',
                    'employer_id'       => 0,
                    'employer_code'     => '',
                    'tin'               => '',
                    'vrn'               => ''
               )
            );
            $customer_id    			= $this->create($customer_data);
			$customer					= new Supplier($customer_id);
           
            if($customer_id) {
                $person->set('personnel_id', $customer_id);
                $customer->set('personnel_id', $customer_id);
            }
		}
		return $customer_id;
	}

	function wallet() {
		global $app;
		$wallet_id 	= $app->getColumnValue('billing_wallets', 'id', 'personnel_id', $this->id);
		if($wallet_id) {
			return new Wallet($wallet_id);
		} else {
			$pricelist			= new Pricelist($app->getColumnValue('billing_pricelists', 'id', 'code', 'CASH'));
			$insurance			= new Insurance($app->getColumnValue('billing_insurances', 'id', 'code', 'CASH'));
			$scheme				= new Scheme($app->getColumnValue('billing_insurance_schemes', 'id', 'code', 'CASH'));
			$person 			= $this->person();
			$wallet 			= new Wallet();
			
			$create_data       	= array (
				'type'         	=> 'Wallet',
				'subtype'      	=> 'Customer',
				'store_id'     	=> 0,
	
				'personnel'    	=> 'Customer',
				'personnel_id' 	=> $this->id,
				'code'         	=> $this->code,
				'name'         	=> $person->name,
				'birthdate'    	=> $person->birthdate,
				'sex'          	=> $person->sex,
				'phone'        	=> $person->phone,
				'email'        	=> $person->email,
	
				'pricelist'    	=> $pricelist->code,
				'pricelist_id' 	=> $pricelist->id,
				'insurance'    	=> $insurance->code,
				'insurance_id' 	=> $insurance->id,
				'scheme'       	=> $scheme->code,
				'scheme_id'    	=> $scheme->id,
	
				'items'        	=> json_encode(array()),
				'cost'         	=> 0,
				'subtotal'     	=> 0,
				'progeress'    	=> 'Requested',
				'progress_code'	=> 1
			);
			$wallet_id      	= $wallet->createWithMinimumData($create_data);
		}
		return new Wallet($wallet_id);
	}

	function getPromises($options=array()) {
		global $app;
		
		$bills_str      = "SELECT billing_invoices.id, 
		billing_invoices.created_datetime, 
		billing_invoices.subtype, 
		billing_invoice_items.item_id, 
		billing_invoice_items.name, 
		billing_invoice_items.quantity quantity, 
		billing_invoice_items.price billed,
		billing_insurances.paymode,
		(billing_invoice_items.paid + IFNULL(billing_promise_items.paid, 0)) paid, 
		billing_invoice_items.exemption exemption, 
		(IFNULL(billing_promise_items.promise, 0) - IFNULL(billing_promise_items.paid, 0)) promise,
		(billing_invoice_items.price - billing_invoice_items.paid - IFNULL(billing_promise_items.paid, 0) - billing_invoice_items.exemption) payable
		FROM billing_invoice_items 
		JOIN billing_invoices ON billing_invoices.id = billing_invoice_items.parent_id 
		JOIN clinical_visit_locations ON clinical_visit_locations.id = billing_invoices.visit_location_id 
		JOIN billing_insurances ON billing_insurances.id = billing_invoices.insurance_id
		LEFT JOIN billing_promise_items ON billing_promise_items.type_id = billing_invoices.type_id AND billing_promise_items.item_id = billing_invoice_items.item_id
		WHERE billing_invoices.personnel_id = $this->id
		AND billing_invoice_items.promise > 0 
		AND billing_invoices.type = 'Bill'
		AND billing_invoices.payment = 1 ";

		$bills      = array();

		foreach($app->read($bills_str) as $bill) {
			$subtype            = $bill->subtype;
			if($subtype == 'Prescription') $subtype = 'Pharmacy';
			if($subtype == 'Procedure') $subtype = 'Procedures';
			if($bill->paymode != 'CASH') {
				$bill->paid 	= 0;
				$bill->payable 	= $bill->billed;
			}
			if(isset($bills[$subtype][$bill->item_id])) {
				$bills[$subtype][$bill->item_id]->quantity  += $bill->quantity;
				$bills[$subtype][$bill->item_id]->billed    += $bill->billed;
				$bills[$subtype][$bill->item_id]->paid      += $bill->paid;
				$bills[$subtype][$bill->item_id]->exemption += $bill->exemption;
				$bills[$subtype][$bill->item_id]->promise   += $bill->promise;
				$bills[$subtype][$bill->item_id]->payable   += $bill->payable;
			} else {
				$bills[$subtype][$bill->item_id]            = $bill;
			}
		}

		return $bills;
	}

	function getPaidPromises() {
		global $app;
		$today 		= $app->morning($app->monthStart());
		$sql		= "SELECT id FROM billing_invoices WHERE personnel_id = $this->id AND parent = 'Promise' AND created_datetime > '$today' AND payment = 1 ";
		return $app->query($sql);
	}
}

?>