<?php 

class Exemption extends Sale {
	function __construct($id=0) {	
		$this->table        = 'billing_exemptions';
		$this->items_table	= 'billing_exemption_items';
		$this->primary_key  = 'id';
		parent::__construct($id);
		$this->type 		= 'Exemption';	
	}

	function renderLinkRepay() {
        return '<a title="Add Payment" class="remote-modal bs-tooltip btn btn-default" data-bs-toggle="modal" data-bs-target="#modal_lg" data-bs-backdrop="static" data-bs-keyboard="false" tabindex="-1" href="form.php?action=repay&id='.$this->id.'" aria-label="Repay"> <span class="fa-regular fa-cart-plus"></span> </a>';
    }

	function renderCreateConfirmation($submodule, $post) {
		global $app, $user_id, $now;

		if(strlen($post->trans_subtype) < 3) {
			return $app->renderAlertError('Please choose exemption category');
		}

		// Prepare items
		$trans_items	= array();

		foreach($post->trans_items as $trans_item) {
			$trans_item 	= (object) $trans_item;
			$trans_items[] 	= array (
				'item_id'			=> $trans_item->item_id,
				'name'				=> $trans_item->name,
				'uom'				=> $trans_item->uom,
				'pack'				=> $trans_item->pack,
				'qty'				=> $trans_item->qty,
				'quantity'			=> $trans_item->quantity,
				'unit_cost'			=> $trans_item->unit_cost,
				'cost'				=> $trans_item->cost,
				'unit_price'		=> $trans_item->unit_price,
				'price'				=> $trans_item->price,
				'exemption'			=> $app->unformatMoney($trans_item->exemption),
				'payable'			=> $app->unformatMoney($trans_item->payable)
			);
		}

		$bill 							= new Bill($post->trans_id);
		$exemption_data 				= $bill;
		$exemption_data->type			= 'Bill';
		$exemption_data->type_id		= $bill->id;
		$exemption_data->subtype		= $bill->subtype;
		$exemption_data->subtype_id		= $bill->subtype_id;
		$exemption_data->reference		= $post->trans_subtype;
		$exemption_data->reference_no	= $post->trans_subtype;
		$exemption_data->parent			= 'Exemption';
		$exemption_data->exemption		= $app->unformatMoney($post->trans_exemption);
		$exemption_data->payable		= $exemption_data->payable - $exemption_data->exemption;
		$exemption_data->created_notes 	= $post->trans_requested_notes;
		$exemption_data->created_user_id= $user_id;
		$exemption_data->datetime		= $now;
		$exemption_data->items			= json_encode($trans_items);

		$exemption_data					= (array) $exemption_data;
		$exemption_id 					= $this->createWithMinimumData($exemption_data);

		if($exemption_id) {
			return $app->renderAlertSuccess('Exemption initialized successfully', 'Success', 'modal_lg', 1);
		}
	}


	function renderTableRow() {
		global $app;

		$link_options   = array('id'=>$this->id, 'name'=>$this->name, 'modal'=>'modal_lg');
 
		$html = '<tr>
		<td class="text-muted fw-semibold">'.$app->datetime($this->created_datetime).'</td>
		<td> <div class="fw-semibold text-auto-truncate">'.$this->type_id.' '.$this->subtype.'</div><div class="text-muted">'.$this->store()->location()->name.'</div></td>
		<td><div class="mb-1">'.$app->renderLinkOpen($link_options).'</div> <div>File '.$app->formatCode($this->code).' &rarr; '.$app->age($this->birthdate).' '.$this->sex.'</div></td>

		<td class="">'.$app->getCategoryByCode('ExemptionType', $this->reference)->name.'</td>

		<td class="text-end fw-bold text-muted">'.$app->formatMoney($this->subtotal).'</td>
		<td class="text-end fw-bold text-muted">'.$app->formatMoney($this->exemption).'</td>
		
		<td class="text-center">'.$this->renderButtonProgress().'</td>
		<td class="text-center text-nowrap">';

		if($this->progress == 'Requested') {
			$html .= $this->renderLinkAuthorize();
			$html .= $this->renderLinkDelete();
		} elseif($this->progress == 'Authorized' || $this->progress == 'Billed') {
			//$html .= $this->renderLinkRepay();
		}

		$html .= '</td>
		</tr>';

		return $html;
	}

	function renderTransactionItemsReadonly($title='Billed items') {
		global $app;

		$html = $this->renderTransactionHeader();

		$html .= ''.$app->renderInputHidden('trans_id', $this->id).'
		<div class="trans-container mb-4"> <h5 class="trans-subtitle"> <span class="fa-regular fa-cart-circle-check fa-fw"></span> &nbsp;Billed items:</h5>
		<div class="card p-3 my-3 bg-primary" style="--bs-bg-opacity: .05;">
        <div class="table-responsive">
		<table class="table table-hover align-middle">
		<thead class="h5">
		<tr>
		<th class="">Item name</th>
		<th class="w-20 text-end">Billed</th>
		<th class="w-20 text-end">Exemption</th>
		<th class="w-20 text-end">Paid</th>
		</tr>
		</thead>

		<tbody id="trans_items_container">';

		foreach($this->items() as $item) {
			$html 		.= '<tr>
			<td class="">
			'.$item->quantity.' x '.$item->name.'
			'.$app->renderInputHidden('trans_items['.$item->item_id.'][transaction_item_id]', $item->id).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][item_id]', $item->item_id).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][name]', $item->name).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][uom]', $item->uom).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][pack]', $item->pack).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][size]', $item->size).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][unit_cost]', $item->unit_cost).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][cost]', $item->cost).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][unit_price]', $item->unit_price).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][quantity]', $item->quantity).'
			'.$app->renderInputHidden('trans_items['.$item->item_id.'][qty]', $item->qty).'
			</td>
			<td class="py-3">'.$app->renderInputText('trans_items['.$item->item_id.'][price]', $app->formatMoney($item->price), '', 'form-control-plaintext readonly text-end money', 'data-item-id='.$item->item_id.'').'</td>
			<td class="py-3">'.$app->renderInputText('trans_items['.$item->item_id.'][exemption]', $app->formatMoney($item->exemption), '', 'form-control-plaintext text-end money trans-bill-item', 'data-item-id='.$item->item_id.'').'</td>
			<td class="py-3">'.$app->renderInputText('trans_items['.$item->item_id.'][paid]', $app->formatMoney($item->paid), '', 'form-control-plaintext readonly text-end money fw-semibold', 'data-item-id='.$item->item_id.'').'</td>
			</tr>';
		}

		$html .= '</tbody>

		<tfoot class="fw-semibold">
		<tr>
		<th class="py-3">Total</th>
		<th class="w-20 text-end">'.$app->renderInputText('trans_subtotal', $app->formatMoney($this->subtotal), '', 'form-control-plaintext readonly text-end money fw-semibold').'</th>
		<th class="w-20 text-end">'.$app->renderInputText('trans_exemption', $app->formatMoney($this->exemption), '', 'form-control-plaintext readonly text-end money fw-semibold').'</th>
		<th class="w-20 text-end">'.$app->renderInputText('trans_paid', $app->formatMoney($this->paid), '', 'form-control-plaintext readonly text-end money fw-semibold').'</th>
		</tr>
		</tfoot>

		</table>
		</div>
		</div>
        </div>';

		return $html;
	}

	function renderAuthorizeConfirmButton() {
        global $app;

        $options    = array (
            array('name'=>'Authorize this request', 'value'=>'Authorized', 'icon'=>'fa-regular fa-lock-open'),
            array('name'=>'Reject this request', 'value'=>'Rejected', 'icon'=>'fa-regular fa-pencil')
        );

        $html = '<div class="row">
        <div class="col-sm-12">
        <h5 class="my-3"> <span class="fa-solid fa-check fa-fw"></span> &nbsp;Choose action: </h5>
        '.$app->renderInputSelect('trans_action', '', 'Choose action', 'select-search form-control-lg', 'data-style="btn-outline-secondary"', $options).'
        </div> <div class="col-sm-6 my-3">
        </div> <div class="col-sm-12 mb-3">
        <h5 class="my-3"> <span class="fa-regular fa-note-sticky fa-fw"></span> &nbsp;Notes or comments: </h5>
        '.$app->renderInputTextarea('trans_authorized_notes', $this->authorized_notes, 'Enter optional notes', 'form-control-lg').'
        </div></div>';

        return $html;
    }

	function renderAuthorizeConfirmation($submodule, $post) {
        global $app, $user;
        if($post->trans_action == 'Authorized') {
			$bill 		= new Bill($this->type_id);
			foreach($this->items() as $item) {
				$sql 	= "UPDATE billing_bill_items SET exemption = $item->exemption, payable = $item->payable WHERE parent_id = $bill->id AND item_id = $item->item_id ";
				$app->query($sql);
			}
			$bill->update ( 
					array (
						'exemption'	=> $this->exemption,
						'payable'	=> $this->payable
				)
			);

			$bill->updateInlineItems();
			$bill->read();
			$bill->createInvoice();

            if($this->changeProgress('Authorized', 'requested_quantity', $post->trans_authorized_notes)) {
                $this->updateInlineItems();
                return $app->renderAlertSuccess($submodule->shortname.' authorized successfully', 'Success', 'modal_lg', 1);
            }
        } elseif($post->trans_action == 'Rejected') {
            if($this->changeProgress('Requested', 'requested_quantity', $post->trans_authorized_notes)) {
                $this->updateInlineItems();
				return $app->renderAlertSuccess($submodule->shortname.' returned to request stage', 'Success', 'modal_lg', 1);
            }
        } else {
            die($app->renderAlertError('Please choose action'));
        }
    }


	function renderRepayForm() {
		global $app;

		$html = $this->renderTransactionHeader();

		$html .= ''.$app->renderInputHidden('trans_id', $this->id).'
		<div class="trans-container mb-4"> <h5 class="trans-subtitle"> <span class="fa-regular fa-cart-circle-check fa-fw"></span> &nbsp;Exemptiond items:</h5>
		<div class="card p-3 my-3 bg-primary" style="--bs-bg-opacity: .05;">
        <div class="table-responsive">
		<table class="table table-hover align-middle">
		<thead class="h5">
		<tr>
		<th class="">Item name</th>
		<th class="w-20 text-end">Exemption</th>
		<th class="w-20 text-end">Paid</th>
		<th class="w-20 text-end">Invoice</th>
		</tr>
		</thead>

		<tbody id="trans_items_container">';

		foreach($this->items() as $item) {
			$html 		.= '<tr>
			<td class="">
			'.$item->quantity.' x '.$item->name.'
			'.$app->renderInputHidden('trans_items['.$item->item_id.'][transaction_item_id]', $item->id).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][item_id]', $item->item_id).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][name]', $item->name).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][uom]', $item->uom).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][pack]', $item->pack).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][size]', $item->size).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][unit_cost]', $item->unit_cost).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][cost]', $item->cost).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][unit_price]', $item->unit_price).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][quantity]', $item->quantity).'
			'.$app->renderInputHidden('trans_items['.$item->item_id.'][qty]', $item->qty).'
			</td>
			<td class="py-3">'.$app->renderInputText('trans_items['.$item->item_id.'][exemption]', $app->formatMoney($item->exemption), '', 'form-control-plaintext readonly text-end money', 'data-item-id='.$item->item_id.'').'</td>
			<td class="py-3">'.$app->renderInputText('trans_items['.$item->item_id.'][price]', $app->formatMoney($item->paid), '', 'form-control-plaintext readonly text-end money', 'data-item-id='.$item->item_id.'').'</td>
			<td class="py-3">'.$app->renderInputText('trans_items['.$item->item_id.'][payable]', $app->formatMoney($item->exemption - $item->paid), '', 'line-control text-end money fw-semibold trans-invoice-item', 'data-item-id='.$item->item_id.'').'</td>
			</tr>';
		}

		$html .= '</tbody>

		<tfoot class="fw-semibold">
		<tr>
		<th class="py-3">Total</th>
		<th class="w-20 text-end">'.$app->renderInputText('trans_subtotal', $app->formatMoney($this->exemption), '', 'form-control-plaintext readonly text-end money fw-semibold').'</th>
		<th class="w-20 text-end">'.$app->renderInputText('trans_exemption', $app->formatMoney($this->paid), '', 'form-control-plaintext readonly text-end money fw-semibold').'</th>
		<th class="w-20 text-end">'.$app->renderInputText('trans_invoice', $app->formatMoney($this->exemption - $this->paid), '', 'form-control-plaintext readonly text-end money fw-semibold').'</th>
		</tr>
		</tfoot>

		</table>
		</div>
		</div>
        </div>';

		return $html;
	}


	function payFromInvoiceId($invoice_id) {
		global $app;

		$invoice = new Invoice($invoice_id);

		// Update items

		foreach($invoice->items() as $item) {
			$sql	= "UPDATE $this->items_table SET paid = paid + $item->paid WHERE parent_id = $this->id AND item_id = $item->item_id ";
			$app->query($sql);
		}

		$this->set('paid', ($this->paid + $invoice->paid));
		$this->read();

		$this->changeProgress('Billed');
		$this->updateInlineItems();

		if($this->exemption == $this->paid) {
			$this->changeProgress('Paid');
			$this->updateInlineItems();
		}
	}

}

?>