<?php 

class Invoice extends Sale {
	function __construct($id=0) {	
		global $app;
		$this->table        = 'billing_invoices';
		$this->items_table	= 'billing_invoice_items';
		$this->primary_key  = 'id';
		parent::__construct($id);
		$this->type 		= 'Invoice';	
		//if(!$id) $this->scheme_id = 1;
	}

	function renderTableHeader() {
		global $app;

		$options            = array (
			'columns'       => array (
				array('name'=>'DATE', 'css'=>'w-10'),
				array('name'=>'INVOICE', 'css'=>'w-10'),
				array('name'=>'CUSTOMER', 'css'=>''),
				array('name'=>'SPONSOR', 'css'=>'w-10'),
				array('name'=>'BILLED', 'css'=>'w-10 money text-end'),
				array('name'=>'PAID', 'css'=>'w-10 money text-end'),
				array('name'=>'STATUS', 'css'=>'w-15 text-center'),
				array('name'=>'OPTIONS', 'css'=>'w-15 text-end')
			)
		);
		
		return $app->renderTableOpen($options);
	}

	function renderTableRow() {
		global $app;

		$link_options   = array('id'=>$this->id, 'name'=>$this->name, 'modal'=>'modal_lg');
		$scheme         = new Scheme($this->scheme_id);
		$html 			= '';

		$html .= '<tr>
		<td class="text-muted fw-semibold">'.$app->datetime($this->created_datetime).'</td>
		<td> <div class="fw-semibold text-auto-truncate">'.$this->id.' '.$this->subtype.'</div><div class="text-muted">'.$this->store()->location()->name.'</div></td>
		<td><div class="mb-1">'.$app->renderLinkOpen($link_options).'</div> <div>File '.$app->formatCode($this->code).' &rarr; '.$app->age($this->birthdate).' '.$this->sex.'</div></td>
		<td>
		<div class="fw-semibold">'.$app->shortname($scheme->insurance()->code, 8).'</div>
		<div class="">'.$app->shortname($scheme->name, 9).'</div>
		</td>
		
		<td class="text-end fw-bold text-muted">';

		$html .= $app->formatMoney($this->subtotal);

		if(strlen($this->reference_no)) $html .= '<br /><small class="text-success">'.$this->reference_no.'</small>';
		
		$html .= '</td>
		<td class="text-end fw-bold text-muted">'.$app->formatMoney($this->paid).'</td>
		
		<td class="text-center">'.$this->renderButtonProgress().'</td>
		<td class="text-center text-nowrap">';
		
		if(strlen($this->progress == 'Requested')) {
			$html .= $this->renderLinkPay();
			//$html .= $this->renderLinkCopy();
			$html .= $this->renderLinkPrint();
			$html .= $this->renderLinkCancel();
		} elseif(strlen($this->progress == 'Billed')) {
			//$html .= $this->renderLinkCopy();
			$html .= $this->renderLinkPay();
			$html .= $this->renderLinkInvoice();
			//$html .= $this->renderLinkTransferInvoice();
			$html .= $this->renderLinkCancel();
		} elseif(strlen($this->progress == 'Paid')) {
			//$html .= $this->renderLinkCopy();
			$html .= $this->renderLinkPrint();
		} 
		
		$html .= '</td>
		</tr>';

		return $html;
	}

	function renderLinkPay() {
        return '<a title="Payment" class="remote-modal bs-tooltip btn btn-default" data-bs-toggle="modal" data-bs-target="#modal_lg" data-bs-backdrop="static" data-bs-keyboard="false" tabindex="-1" href="form.php?action=pay&id='.$this->id.'" aria-label="Pay"> <span class="fa-regular fa-memo-circle-check"></span> </a>';
    }

	function renderPaymentChannelControl($channels=array()) {
        global $app;

		if(!$channels) {
			$channels	= array();
			$sql 		= "SELECT code, name FROM settings_categories WHERE type = 'PaymentChannel'";
			foreach($app->read($sql) as $row) {
				$channels[] = array(
					'name'	=> $row->name,
					'value'	=> $row->code
				);
			}
		}

        $html   = '<div class="trans-container"> <h5 class="trans-subtitle"> <span class="fa-regular fa-tags fa-fw"></span> &nbsp;Payment method: </h5>';
		$html   .= $app->renderInputSelect('trans_payment_channel', $this->payment_channel, 'Choose payment method', 'line-control select-search onchange-reload-children', '', $channels);
		$html   .= '</div>';
        return $html;
    }

    function renderAccountControl($scheme_id='') {
        global $app;
        if(!$scheme_id) $scheme_id = $this->scheme_id;

        $scheme         = new Scheme($scheme_id);
        $insurance_id   = $scheme->insurance_id; 
        $html           = '<div class="trans-container"> <h5 class="trans-subtitle"> <span class="fa-regular fa-tags fa-fw"></span> &nbsp;Payment account: </h5>';
        $html           .= $app->renderInputSelect('trans_payment_account_id', $this->payment_account_id, '', 'line-control select-search', 'data-parent="trans_payment_channel" data-source="html/transactions/accounts_deposit_select.php"', 'accounting_accounts', 'id', 'name', "0");
		$html           .= '</div>';
        return $html;
    }

	function renderPaymentConfirmation($submodule, $post) {
        global $app, $user, $now, $root;

		if(!strlen($post->trans_payment_channel)) {
			die($app->renderAlertError('Please choose payment channel', 'Payment channel required'));
		}

		if(!isset($post->trans_payment_account_id)) {
			die($app->renderAlertError('Please choose payment account', 'Payment account required'));
		}

		$channel        = $app->getCategoryByCode('PaymentChannel', $post->trans_payment_channel);
		$script 		= 'pay_process_'.strtolower($channel->code).'.php';

		if(file_exists($script)) {
			require('pay_process_'.strtolower($channel->code).'.php');
		} else {
			die($app->renderAlertError('The file '.$script.' does not exist'));
		}
    }

	function renderCancelConfirmation($submodule, $post) {
        global $app, $user, $now, $root;

		if(!strlen($post->trans_returned_notes)) {
			die($app->renderAlertError('Please enter cancellation reason in notes', 'Cancellation notes required'));
		}

		if($this->payment_channel == 'GEPG') {
			require($root.'api/gepg/GEPG.php');
			
			// Cancel control number
			$gepg->cancelControlNumber($this);

		}

		$this->set('requested_notes', $post->trans_returned_notes);
		$this->set('returned_notes', $post->trans_returned_notes);
		$this->changeProgress('Returned');
		if($this->parent_id) {
			$bill = new Bill($this->parent_id);
			$bill->changeProgress('Requested');
		}
		echo $app->renderAlertSuccess('Invoice canceled successfully','', 'modal_lg', true);
    }


	function bill(array $data=array()) {
		global $user, $now;

		$payment_data				= array (
			'reference'				=> isset($data['reference']) ? $data['reference'] : $this->reference,
			'reference_no'			=> isset($data['reference_no']) ? $data['reference_no'] : $this->reference_no,
			'billed_reference'		=> isset($data['billed_reference']) ? $data['billed_reference'] : $this->billed_reference,
			'billed_notes'			=> isset($data['billed_notes']) ? $data['billed_notes'] : $this->billed_notes,
			'billed_user_id'		=> $user->id,
			'billed_datetime'		=> $now,

			'payment_method'		=> isset($data['payment_method']) ? $data['payment_method'] : $this->payment_channel,
			'payment_channel'		=> isset($data['payment_channel']) ? $data['payment_channel'] : $this->payment_channel,
			'payment_account'		=> isset($data['payment_account']) ? $data['payment_account'] : $this->payment_account,
			'payment_account_id'	=> isset($data['payment_account_id']) ? $data['payment_account_id'] : $this->payment_account_id,
		);
		
		$this->changeProgress('Billed');
		$this->update($payment_data);
		$this->updateInlineItems();
	}
	

	function pay($data=array()) {
		global $app, $user, $now, $client;

		$customer 					= $this->customer();
		$wallet		 				= $customer->wallet();

		$progress 					= 'Paid'; 

		$payment_data				= array (
			'progress'				=> $progress,
			'progress_code'			=> 8,
			
			'paid'					=> isset($data['paid']) ? $data['paid'] : $this->payable,

			'payment'				=> 1,
			'payment_method'		=> isset($data['payment_method']) ? $data['payment_method'] : $this->payment_method,
			'payment_channel'		=> isset($data['payment_channel']) ? $data['payment_channel'] : $this->payment_channel,
			'payment_account'		=> isset($data['payment_account']) ? $data['payment_account'] : $this->payment_account,
			'payment_account_id'	=> isset($data['payment_account_id']) ? $data['payment_account_id'] : $this->payment_account_id,
			'payment_transaction'	=> isset($data['payment_transaction']) ? $data['payment_transaction'] : $this->payment_transaction,
			'payment_reference'		=> isset($data['payment_reference']) ? $data['payment_reference'] : $this->payment_reference,
			'payment_receipt'		=> isset($data['payment_receipt']) ? $data['payment_receipt'] : $this->payment_receipt,
			'payment_name'			=> isset($data['payment_name']) ? $data['payment_name'] : $this->payment_name,
			'payment_phone'			=> isset($data['payment_phone']) ? $data['payment_phone'] : $this->payment_phone,
			'payment_notes'			=> isset($data['payment_notes']) ? $data['payment_notes'] : $this->payment_notes,
			
			'payment_user_id'		=> $user->id,
			'payment_datetime'		=> $now
		);

		if($this->subtype == 'Wallet') {
			// Prevent negative balance
			if($this->payable < 0) {
				if($wallet->subtotal + $this->payable < 0) {
					die($app->renderAlertError('You are trying to withdraw TZS '.$app->formatMoney(abs($this->payable)).' from TZS '.$app->formatMoney($wallet->subtotal), 'Insufficient funds'));
				}
			}
			$wallet->topup($this->id);
		} elseif($payment_data['payment_channel'] == 'WALLET') {
			$subtotal 				= -(abs($this->payable));
			if($wallet->subtotal + $subtotal < 0) {
				die($app->renderAlertError('You are trying to pay TZS '.$app->formatMoney(abs($this->payable)).' from TZS '.$app->formatMoney($wallet->subtotal), 'Insufficient funds'));
			}
			$wallet->topup($this->id);
		}
		
		$paid = $this->update($payment_data);

        if($paid) {
			$payment_data['items']	= $this->items();
			$this->payTransactionItems($payment_data);

			if($this->parent == 'Bill') {
				$bill_obj 		= new Bill($this->parent_id);
				$bill_obj->payFromInvoice($this->id);
			}

			$this->updateInlineItems();

			// If this is service bill (Laboratory, Procedure, Radiology, Prescription) update service accordingly

			$services 								= array ('Laboratory', 'Procedure', 'Radiology', 'Prescription', 'Pharmacy', 'Consumable');
			if(in_array($this->subtype, $services) && $this->subtype_id) {
				$request_class 						= $this->subtype;
				if($request_class == 'Pharmacy')	$request_class = 'Prescription';
				$request 							= new $request_class($this->subtype_id);
				$request_item_class 				= $request->type.'Item';

				foreach($this->items() as $request_item) {
					$request_item_id                = $app->getColumnValue($request->items_table, 'id', 'item_id', $request_item->item_id, "parent_id = $this->subtype_id");
					$transaction_item               = new $request_item_class($request->type, $request->id, $request_item_id);
		
					$transaction_item->update (
						array (
							'progress'              => $progress,
							'progress_code'        	=> 8,
							'payment'             	=> 1,
							'payment_notes'    		=> '',
							'payment_user_id'     	=> $user->id,
							'payment_datetime'    	=> $now,
						)
					);
				}
				
				$request->updateInlineItems();
			}

			// Update promise to pay

			if($this->parent == 'Promise') {
				$promise = new Promise($this->parent_id);
				$promise->payFromInvoiceId($this->id);
			}

			// Journal

			$journal 				= new Journal();
			$journal_data           = array (
				'id'                => 0,
				'date'              => $now,
				'name'              => $this->name,
				'details'           => '',
				'parent'       		=> $this->type,
				'parent_id'    		=> $this->id,
				'personnel'         => $this->personnel,
				'personnel_id'      => $this->personnel_id,
				'debit'             => '',
				'debit_id'          => 0,
				'credit'            => '',
				'credit_id'         => 0,
				'amount'            => $this->paid,
				'currency'          => $client->currency_code,
				'rate'              => $client->currency_rate,
				'f1'                => '',
				'f2'                => '',
				'f3'                => '',
				'f4'                => '',
				'f5'                => '',
				'f6'                => '',
				'f7'                => '',
				'f8'                => '',
				'f9'                => '',
				'created_user_id'   => $this->created_user_id,
				'created_datetime'  => $this->created_datetime,
				'modified_user_id'  => 0,
				'modified_datetime' => 0,
				'status'            => 1
			);
			$journal->create($journal_data);

			return $paid;
            //return $app->renderAlertSuccess('Payment received successfully', 'Success', 'modal_lg', 1);
        }
	}

	function payTransactionItems($data) {
		global $app, $user, $now;

		$progress 						= $this->insurance()->paymode == 'CASH' ? 'Paid' : 'Billed'; 
		$items							= $data['items'];

		foreach($items as $item) {
			$transaction_item			= new TransactionItem($this->type, $this->id, $item->id);
			$payment_data 				= array (
				'progress'				=> $progress,
				'progress_code'			=> 8,
				'paid'					=> $transaction_item->payable ? $transaction_item->payable : $transaction_item->price,
				'payment'				=> 1,
				'payment_quantity'		=> $transaction_item->requested_quantity,
				'payment_channel'		=> isset($data['payment_channel']) ? $data['payment_channel'] : $transaction_item->payment_channel,
				'payment_method'		=> isset($data['payment_method']) ? $data['payment_method'] : $transaction_item->payment_method,
				'payment_account_id'	=> isset($data['payment_account_id']) ? $data['payment_account_id'] : $transaction_item->payment_account_id,
				'payment_transaction'	=> isset($data['payment_transaction']) ? $data['payment_transaction'] : $transaction_item->payment_transaction,
				'payment_reference'		=> isset($data['payment_reference']) ? $data['payment_reference'] : $transaction_item->payment_reference,
				'payment_receipt'		=> isset($data['payment_receipt']) ? $data['payment_receipt'] : $transaction_item->payment_receipt,
				'payment_name'			=> isset($data['payment_name']) ? $data['payment_name'] : $transaction_item->payment_name,
				'payment_phone'			=> isset($data['payment_phone']) ? $data['payment_phone'] : $transaction_item->payment_phone,
				'payment_notes'			=> isset($data['payment_notes']) ? $data['payment_notes'] : $transaction_item->payment_notes,
				'payment_user_id'		=> $user->id,
				'payment_datetime'		=> $now
			);
			$transaction_item			->update($payment_data);
		}
	}

	function renderDeleteConfirmation($submodule) {
		global $root, $gepg;

		if($this->payment_channel == 'GEPG') {
			require($root.'api/gepg/GEPG.php');

			// Cancel control number
			$gepg->cancelControlNumber($this);

		}
	}

	function renderHtmlInvoiceGEPG() {
		global $app, $client, $user, $root;
		$items	        	= $this->items();
		$title 				= 'Invoice';
		$cookie_name 		= app_name;
		
		$qr     			= array (
			'opType'        => '2',
			'shortCode'     => '001001',
			'billReference' => $this->reference,
			'amount'        => $this->subtotal,
			'billCcy'       => $client->currency_code,
			'billExprDt'    => date('y-m-d', strtotime($this->created_datetime.' + 30 days')),
			'billPayOpt'    => '1',
			'billRsv01'     => $client->client_name.' | '.$user->personnel()->person()->firstname()
		);
		
		$qr     			= base64_encode(json_encode($qr));
		
		$items  			= '<table class="table">';
		
		foreach($this->items() as $item) {
			$items .= '<tr><td style="width:70%" colspan="2">'.$item->quantity.' X '.$item->name.' <span class="text-muted">@'.$client->currency_code.' '.number_format($item->unit_price, 2).'</span></td><td style="text-align:right">'.$client->currency_code.' '.number_format($item->price, 2).'</td></tr>';
		}
		
		$items .= '<tr><td style="width:70%" colspan="2"><b>Total Billed Amount </b></td><td style="text-align:right"><b> '.$client->currency_code.' '.number_format($this->subtotal, 2).' </b></td></tr>';
		
		$items .= '</table>';
		
		include($root.'html/header.php');

		echo '<body style="background-color:rgb(240,240,240); padding:20px">
			
		<center>
		
		<div style="width:800px; background-color:rgb(255,255,255); border:0px solid rgb(220,220,220); padding:15px; text-align:left">
		
		<div class="text-center">
		<img src="'.$root.'assets/images/other/urt.png" alt="URT" style="width:100px" />
		<h4><b>The United Republic of Tanzania</b></h4>
		<h5>'.$client->client_name.'</h5>
		Government Bill
		</div>
		
		<br />
		
		<table class="table table-condensed">
		<tr> <td style="width:30%">File Number</td> <td style="width:40%">: <b>'.$app->formatCode($this->code).'</b></td> 
		<td rowspan="6" style="width:30%; padding:15px">  <img src="'.$root.'/libraries/tcpdf/qrcode.php?qr='.$qr.'" alt="'.$qr.'" style="width:200px; height:200px" />  </td>
		<tr> <td style="width:30%">Customer Name</td> <td style="width:40%">: <b>'.$this->name.'</b></td> </tr>
		<tr> <td style="width:30%">Customer phone</td> <td style="width:40%">: <b>'.$this->phone.'</b></td> </tr>
		<tr> <td style="width:30%">Insurance / Scheme</td> <td style="width:70%">: <b>'.$this->insurance()->name.' / '.$this->scheme()->name.'</b></td> </tr>
		<tr> <td style="width:30%">Control Number</td> <td style="width:40%">: <b>'.$this->reference_no.'</b></td> 
		
		</tr>
		<!-- <tr> <td style="width:30%">Payment Reference</td> <td style="width:40%">: <b>'.$this->subtype.'</b></td> </tr> -->
		<tr> <td style="width:30%">Service Provider Code</td> <td style="width:40%">: <b>'.$client->gepg_sp_code.'</b></td> </tr>
		<tr> <td style="width:30%">Bill Description</td> <td style="width:70%" colspan="2">: <b>'.$this->subtype.'</b></td> </tr>
		<tr> <td style="width:30%">Billed Items</td> <td style="width:70%" colspan="2">'.$items.'</td> </tr>
		
		<tr> <td style="width:30%">Amount in Words</td> <td style="width:70%" colspan="2"><b>'.ucwords($app->readNumber(round($this->subtotal, 2))).' Tanzanian Shillings</b> </td> </tr>
		<tr> <td style="width:30%">Expires on</td> <td style="width:70%" colspan="2"><b>'.date('d F y', strtotime($this->created_datetime.' + 30 days')).'</b> </td> </tr>
		<tr> <td style="width:30%">Prepared by</td> <td style="width:70%" colspan="2"><b>'.$user->person()->name.'</b> </td> </tr>
		
		<tr> <td style="width:30%">Collection Centre</td> <td style="width:70%" colspan="2"><b>'.$client->client_name.'</b> </td> </tr>
		<tr> <td style="width:30%">Printed by</td> <td style="width:70%" colspan="2"><b>'.$user->personnel()->person()->name.'</b> </td> </tr>
		<tr> <td style="width:30%">Printed on</td> <td style="width:70%" colspan="2"><b>'.date('d F y H:i').'</b> </td> </tr>
		<tr> <td style="width:30%">Signature</td> <td style="width:70%" colspan="2"><b><br />.................................................</b> </td> </tr>
		
		</table>
		
		<table class="table table-borderless">
		<tr>
		<td style="width:50%">
		<center><div><b> = JINSI YA KULIPA = </b></div></center>
		<div>1. Kupitia bank: Fika tawi lolote au wakala wa <b>NMB, CRDB au BOT,</b> namba ya kumbukumbu <b>'.$this->reference.'</b></div>
		<div>2. Kupitia mitandao ya simu: </div>
		<ul>
		<div>Ingia kwenye menyu ya mtandao husika</div>
		<div>Chagua 4: Lipa bill</div>
		<div>Chagua 5: Malipo ya serikali</div>
		<div>Ingiza  <b>'.$this->reference.'</b> kama namba ya kumbukumbu </div>
		</ul>
		</td>
		
		<td style="width:50%">
		<center><div><b> = HOW TO PAY = </b></div></center>
		<div>1. Via bank: Visit any branch or bank agent of <b>NMB, CRDB au BOT,</b> Reference number <b>'.$this->reference.'</b></div>
		<div>2. Via mobile network operators (MNOs): </div>
		<ul>
		<div>Enter to the respective USSD menu of MNO</div>
		<div>Select 4: Make payment</div>
		<div>Select 5: Government payments</div>
		<div>Enter  <b>'.$this->reference.'</b> as reference </div>
		</ul>
		</td>
		</tr>
		</table>
		
		</div>
		
		</center>
		
		</body>';
	}

	function renderHtmlInvoice() {
		global $app, $client, $user, $root;
		$invoice_no         = $app->formatCode($this->id, 6);
        $items  			= '<table class="table">';
		
		foreach($this->items() as $item) {
			$items .= '<tr><td style="width:70%" colspan="2">'.$item->quantity.' X '.$item->name.' <span class="">@'.$client->currency_code.' '.number_format($item->unit_price, 2).'</span></td><td style="text-align:right">'.$client->currency_code.' '.number_format($item->price, 2).'</td></tr>';
		}
		
		$items .= '<tr><td style="width:70%" colspan="2"><b>Total Billed Amount </b></td><td style="text-align:right"><b> '.$client->currency_code.' '.number_format($this->subtotal, 2).' </b></td></tr>';
		
		$items .= '</table>';

		$title 	= 'Invoice';
		
		include($root.'html/header.php');
		
		echo '<body>
		
		<center>
		
		<div class="text-center">
		<img src="'.$root.CLIENT_LINK.'/images/logo.png" alt="logo" style="width:100px" />
		<h4><b>'.$client->client_name.'</b></h4>
		Invoice No. <b>'.$invoice_no.'</b>
		</div>
		
		<br />
		
		<table class="table table-condensed">
		<tr> <td style="width:30%" class="ps-0">File no</td> <td style="width:70%" class="px-0"> <b>'.$app->formatCode($this->code, 6).'</b></td> </tr>
		<tr> <td style="width:30%" class="ps-0">Control no</td> <td style="width:70%" class="px-0"> <b>'.$this->reference_no.'</b></td> </tr>
		<tr> <td style="width:30%" class="ps-0">Customer</td> <td style="width:70%" class="px-0"> <b>'.$this->name.'</b></td> </tr>
		<tr> <td style="width:30%" class="ps-0">Sponsor</td> <td style="width:70%" class="px-0"> <b>'.$this->insurance()->name.' / '.$this->scheme()->name.'</b></td> </tr>
		<tr> <td style="width:30%" class="ps-0">Service</td> <td style="width:70%" class="px-0"> <b>'.$this->subtype.'</b></td> </tr>';
		
        if(($this->subtype == 'Prescription' || $this->subtype == 'Pharmacy') && $this->subtype_id) {
            $pharmacy   = new Pharmacy($this->subtype_id);
            foreach($pharmacy->items() as $item) {
                echo '<tr> <td colspan="2" class="border-0 px-0">'.$item->name.'</td></tr>
                <tr> <td colspan="2" class="border-0 px-0 ">'.$item->dose.' '.$item->uom.' '.$item->times.' Times a Day for '.$item->days.' Days</td></tr>
                <tr><td colspan="2" class="px-0">'.$item->quantity.' '.$item->uom.' @ '.$client->currency_code.' '.number_format($item->unit_price, 2).' <span class="float-end fw-bold">= '.$client->currency_code.' '.number_format($item->price, 2).'</span> </td></tr>';
            }
        } else {
            foreach($this->items() as $item) {
                echo '<tr> <td colspan="2" class="border-0 px-0">'.$item->name.'</td></tr>
                <tr><td colspan="2" class="px-0">'.$item->quantity.' '.$item->uom.' X '.$client->currency_code.' '.number_format($item->unit_price, 2).' <span class="float-end fw-bold">'.$client->currency_code.' '.number_format($item->price, 2).'</span> </td></tr>';
            }
        }
        
        echo '<tr> <td colspan="2" class="px-0">Total amount paid<span class="fw-bold float-end">'.$client->currency_code.' '.number_format($this->subtotal, 2).'</span></td> </tr>

		<tr> <td style="width:30%" class="ps-0">In Words</td> <td style="width:70%" class="px-0"> <b>'.ucwords($app->readNumber(round($this->subtotal))).' Tanzanian Shillings</b> </td> </tr>
        <tr> <td style="width:30%" class="ps-0">Issued by</td> <td style="width:70%" class="px-0"><b> '.$user->person()->name.'</b> </td> </tr>
		<tr> <td style="width:30%" class="ps-0">Date</td> <td style="width:70%" class="px-0"> <b>'.date('d F Y H:i', strtotime($this->billed_datetime)).'</b> </td> </tr>
		<tr> <td style="width:30%" class="ps-0">Signature</td> <td style="width:70%" class="px-0"><b><br />.................................................</b> </td> </tr>

		</table> 
		
		</div>
		
		</center>
		
		</body>';
	}

	function renderHtmlTransferInvoice() {
		global $app, $client, $user, $root;
		$items	        	= $this->items();
		$title 				= 'Invoice';
		$cookie_name 		= app_name;
		
		$qr     			= array (
			'opType'        => '2',
			'shortCode'     => '001001',
			'billReference' => $this->reference,
			'amount'        => $this->subtotal,
			'billCcy'       => $client->currency_code,
			'billExprDt'    => date('y-m-d', strtotime($this->created_datetime.' + 30 days')),
			'billPayOpt'    => '1',
			'billRsv01'     => $client->client_name.' | '.$user->personnel()->person()->firstname()
		);
		
		$qr     			= base64_encode(json_encode($qr));
		
		$items  			= '<table class="table">';
		
		foreach($this->items() as $item) {
			$items .= '<tr><td style="width:70%" colspan="2">'.$item->quantity.' X '.$item->name.' <span class="text-muted">@'.$client->currency_code.' '.number_format($item->unit_price, 2).'</span></td><td style="text-align:right">'.$client->currency_code.' '.number_format($item->price, 2).'</td></tr>';
		}
		
		$items .= '<tr><td style="width:70%" colspan="2"><b>Total Billed Amount </b></td><td style="text-align:right"><b> '.$client->currency_code.' '.number_format($this->subtotal, 2).' </b></td></tr>';
		
		$items .= '</table>';
		
		include($root.'html/header.php');

		echo '<body style="background-color:rgb(240,240,240); padding:20px">
		
		<center>
		
		<div style="width:800px; background-color:rgb(255,255,255); border:0px solid rgb(220,220,220); padding:15px; text-align:left">
		
		<div class="text-center">
		<img src="'.$root.'assets/images/other/urt.png" alt="URT" style="width:100px" />
		<h4><b>The United Republic of Tanzania</b></h4>
		<h5>'.$client->client_name.'</h5>
		Order Form for Electronic Funds Transfer to National Microfinance Bank
		</div>
		
		<br />
		
		<h5><b>(A) Remitter / Tax Payer Details</b></h5>

		<table class="table table-condensed">

		<tr> <td style="width:30%">Name of Account Holder</td> <td style="width:70%" colspan="2"><b><br />..................................................................................................</b> </td> </tr>
		<tr> <td style="width:30%">Name of Commercial Bank</td> <td style="width:70%" colspan="2"><b><br />..................................................................................................</b> </td> </tr>
		<tr> <td style="width:30%">Bank Account Number</td> <td style="width:70%" colspan="2"><b><br />..................................................................................................</b> </td> </tr>
		<tr> <td style="width:30%">Signatories</td> <td style="width:70%" colspan="2"><b><br />................................................  &nbsp; ...............................................</b>
		<br />Signature of the Transfer One &nbsp;&nbsp;&nbsp; Signature of Transfer Two </td> </tr>

		</table>

		<h5><b>(B) Beneficiary Details</b></h5>

		<table class="table table-condensed">

		<tr> <td style="width:30%">Beneficiary Details</td> <td style="width:40%">: <b>'.$client->client_name.'</b> </td> 
		<td rowspan="6" style="width:30%; padding:15px">

		<img src="'.$root.'/libraries/tcpdf/qrcode.php?qr='.$qr.'" alt="'.$qr.'" style="width:200px; height:200px" />  

		</td>

		</tr>
		<tr> <td style="width:30%">Bank Name</td> <td style="width:40%" colspan="2">: <b>'.$client->gepg_bank_name.'</b> </td> </tr>
		<tr> <td style="width:30%">Account Number</td> <td style="width:40%" colspan="2">: <b>'.$client->gepg_account_no.'</b> </td> </tr>
		<tr> <td style="width:30%">Swift Code</td> <td style="width:40%" colspan="2">: <b>'.$client->gepg_swift_code.'</b> </td> </tr>
		<tr> <td style="width:30%">Control Number</td> <td style="width:40%">: <b>'.$this->reference.'</b></td> 
		<tr> <td style="width:30%">Beneficiary Account (Field 59 of MT103)</td> <td style="width:40%" colspan="2">: <b>/'.$client->gepg_account_no.'</b> </td> </tr>
		<tr> <td style="width:30%">Payment Reference (Field 70 of MT103)</td> <td style="width:40%" colspan="2">: <b>/ROC/'.$this->reference.'</b> </td> </tr>
		<tr> <td style="width:30%">Transfer Amount</td> <td style="width:40%" colspan="2">: <b>'.$client->currency_code.' '.number_format($this->subtotal).'</b> </td> </tr>
		<tr> <td style="width:30%">Amount in Words</td> <td style="width:40%" colspan="2">: <b>'.ucwords($app->readNumber(round($this->subtotal))).' Tanzanian Shillings</b> </td> </tr>
		<tr> <td style="width:30%">Being Payment For</td> <td style="width:40%">: <b>'.$this->name.'</b></td> </tr>

		<tr> <td style="width:30%">Billed Items</td> <td style="width:70%" colspan="2">'.$items.'</td> </tr>

		<tr> <td style="width:30%">Expires on</td> <td style="width:70%" colspan="2"><b>'.date('d F y', strtotime($this->created_datetime.' + 30 days')).'</b> </td> </tr>
		<tr> <td style="width:30%">Prepared by</td> <td style="width:70%" colspan="2"><b>'.$user->person()->name.'</b> </td> </tr>

		<tr> <td style="width:30%">Collection Centre</td> <td style="width:70%" colspan="2"><b>'.$client->client_name.'</b> </td> </tr>
		<tr> <td style="width:30%">Printed by</td> <td style="width:70%" colspan="2"><b>'.$user->person()->name.'</b> </td> </tr>
		<tr> <td style="width:30%">Printed on</td> <td style="width:70%" colspan="2"><b>'.date('d F y').'</b> </td> </tr>
		<tr> <td style="width:30%">Signature</td> <td style="width:70%" colspan="2"><b><br />.................................................</b> </td> </tr>

		</table>

		<h5><b> Notes to Commercial Bank:</b></h5>

		<ol>
		<li>Please capture the above information correctly. Do not change or add any text, symbols or digits on the information</li>
		<li>Field 59 of MT103 is an <b>Account Number</b> with value <b>/'.$client->gepg_account_no.'</b>. Must be captured correctly </li>
		<li>Field 70 of MT103 is a <b>Control Number</b> with value <b>/ROC/'.$this->reference.'</b>. Must be captured correctly </li>
		</ol>

		</div>

		</center>

		</body>';
	}

	function renderHtmlReceipt() {
		global $app, $client, $user, $root;
		$items	        	= $this->items();
		$title 				= 'Receipt';
		$cookie_name 		= app_name;
		$receipt_no			= strlen($this->payment_reference) ? $this->payment_reference : $this->formatCode($this->id);
		
		$items  			= '<table class="table">';
		
		foreach($this->items() as $item) {
			$items .= '<tr><td style="width:70%" colspan="2">'.$item->quantity.' X '.$item->name.' <span class="text-muted">@'.$client->currency_code.' '.number_format($item->unit_price, 2).'</span></td><td style="text-align:right">'.$client->currency_code.' '.number_format($item->price, 2).'</td></tr>';
		}
		
		$items .= '<tr><td style="width:70%" colspan="2"><b>Total Billed Amount </b></td><td style="text-align:right"><b> '.$client->currency_code.' '.number_format($this->subtotal, 2).' </b></td></tr>';
		
		$items .= '</table>';
		
		include($root.'html/header.php');
		
		echo '<body style="background-color:rgb(240,240,240); padding:20px">
		
		<center>
		
		<div style="width:800px; background-color:rgb(255,255,255); border:0px solid rgb(220,220,220); padding:15px; text-align:left">
		
		<div class="text-center">
		<img src="'.$root.'assets/images/other/urt.png" alt="URT" style="width:100px" />
		<h4><b>The United Republic of Tanzania</b></h4>
		<h5>'.$client->client_name.'</h5>
		Government Exchequer Receipt
		</div>
		
		<br />
		
		<table class="table table-condensed">
		<tr> <td style="width:30%">Receipt Number</td> <td style="width:70%">: <b>'.$receipt_no.'</b></td> </tr>
		<tr> <td style="width:30%">Received From</td> <td style="width:70%">: <b>'.$this->name.'</b></td> </tr>
		<tr> <td style="width:30%">Insurance / Scheme</td> <td style="width:70%">: <b>'.$this->insurance()->name.' / '.$this->scheme()->name.'</b></td> </tr>
		<tr> <td style="width:30%">Amount</td> <td style="width:70%">: <b>'.$client->currency_code.' '.number_format($this->subtotal, 2).'</b></td> </tr>
		<tr> <td style="width:30%">Amount in Words</td> <td style="width:70%">: <b>'.ucwords($app->readNumber(round($this->subtotal))).' Tanzanian Shillings</b> </td> </tr>
		<tr> <td style="width:30%">Outstanding Balance</td> <td style="width:70%">: <b>'.$client->currency_code.' '.number_format(0).'</b></td> </tr>

		<tr> <td style="width:30%">In Respect of</td> <td style="width:70%">'.$items.'</td> </tr>
		<tr> <td style="width:30%">Billed by</td> <td style="width:40%">: <b>'.$this->user('billed')->person()->name.'</b></td> </tr>
		<tr> <td style="width:30%">Date billed</td> <td style="width:70%">: <b>'.date('d F Y H:i', strtotime($this->billed_datetime)).'</b> </td> </tr>
		<tr> <td style="width:30%">Bill Reference</td> <td style="width:70%">: <b>'.$receipt_no.'</b></td> </tr>
		<tr> <td style="width:30%">Control Number</td> <td style="width:70%">: <b>'.$this->reference.'</b></td> </tr>

		<tr> <td style="width:30%">Issued by</td> <td style="width:70%"><b>: '.$user->person()->name.'</b> </td> </tr>
		<tr> <td style="width:30%">Date issued</td> <td style="width:70%">: <b>'.date('d F Y H:i', strtotime($this->payment_datetime)).'</b> </td> </tr>
		<tr> <td style="width:30%">Signature</td> <td style="width:70%"><b><br />.................................................</b> </td> </tr>

		</table> 
		
		</div>
		
		</center>
		
		</body>';
	}

	function renderLinkInvoice() {
        return '<a title="Print Invoice" class="bs-tooltip btn btn-default" target="blank" href="print_invoice.php?id='.$this->id.'" aria-label="Invoice"> <span class="fa-regular fa-file-invoice-dollar"></span> </a>';
    }

	function renderLinkTransferInvoice() {
        return '<a title="Print Transfer" class="bs-tooltip btn btn-default" target="blank" href="print_transfer_invoice.php?id='.$this->id.'" aria-label="Invoice"> <span class="fa-regular fa-file-invoice-dollar"></span> </a>';
    }
}

?>