<?php 

class SaleReturn extends Sale {
	function __construct($id=0) {	
		global $app;
		$this->table        = 'billing_returns';
		$this->items_table	= 'billing_return_items';
		$this->primary_key  = 'id';
		parent::__construct($id);
		$this->type 		= 'SaleReturn';	
	}

	function renderModalForm($submodule, $obj, $get, $form_action='process.php') {
		global $app;

		$html				= '';
		$this->shortname 	= $submodule->shortname;

		if($get->action == 'delete') {
			$submodule->shortname = $this->shortname;
			$html .= $submodule->renderModalDelete($get->id);

		} else {

			if($get->action == 'create') {
				$title = 'Create a new '.$this->shortname;
			} else {
				$title = ucfirst($get->action).' '.$this->shortname;
			}

			$modal_options  = array(
				'title'     => $title,
				'left_html'	=> '<div id="customer_bio_container"></div>'
			);
			
			$html .= $submodule->renderModalFormOpen($modal_options);
			$html .= $app->renderInputHidden('action', $get->action);
			$html .= $app->renderInputHidden('id', $get->id);	

			if($get->action == 'create' || $get->action == 'copy' || $get->action == 'edit') {
				require('transactions_form.php');
			} elseif($get->action == 'open') {
				$html .= $obj->renderTransactionHeader();
				$html .= $obj->renderTransactionItemsReadonly();
				$html .= $obj->renderNotesControl('Requested');
			} elseif($get->action == 'cancel') {
				$html .= $obj->renderTransactionHeader();
				$html .= $obj->renderTransactionItemsReadonly();
				$html .= $obj->renderNotesControl('Returned');
			} elseif($get->action == 'authorize') {
                $html .= $obj->renderTransactionHeader();
                $html .= $obj->renderTransactionItemsReadonly();
                $html .= $obj->renderActionControl( array (
                    array('name'=>'Authorize sale return', 'value'=>'Authorized', 'icon'=>'fa-regular fa-check-circle fa-fw'),
                    array('name'=>'Reject sale return','value'=>'Rejected', 'icon'=>'fa-regular fa-circle-xmark fa-fw')
                ));
                $html .= $obj->renderNotesControl('Authorized');
            } elseif($get->action == 'approve') {
                $html .= $obj->renderTransactionHeader();
                $html .= $obj->renderTransactionItemsReadonly();
                $html .= $obj->renderActionControl( array (
                    array('name'=>'Approve sale return', 'value'=>'Approved', 'icon'=>'fa-regular fa-check-circle fa-fw'),
                    array('name'=>'Reject sale return','value'=>'Rejected', 'icon'=>'fa-regular fa-circle-xmark fa-fw')
                ));
                $html .= $obj->renderNotesControl('Approved');
            } elseif($get->action == 'transfer') {
                $html .= $obj->renderTransactionHeader();
                $html .= $obj->renderTransactionItemsReadonly();

				if($this->type == 'SaleReturn') {
					if($this->subtotal < 0) {
						$html .= '<div class="mb-4">'.$app->renderAlertSuccess('TZS '.$app->formatMoney(abs($this->subtotal)).' will be returned to customer wallet.', 'Wallet topup').'</div>';
					}
				}

                $html .= $obj->renderActionControl( array (
                    array('name'=>'Confirm sale return', 'value'=>'Transferred', 'icon'=>'fa-regular fa-check-circle fa-fw'),
                    array('name'=>'Reject sale return','value'=>'Rejected', 'icon'=>'fa-regular fa-circle-xmark fa-fw')
                ));
                $html .= $obj->renderNotesControl('Transferred');
            } 

			$html     .= $submodule->renderModalFormClose();
		}
		return $html;	
	}

	function renderCreateConfirmation($submodule, $post) {
        global $app, $user;

        $user->checkSubmoduleAccess($submodule, 'create');

        $items          = $app->uncleanText($post->trans_items);
		$store          = new Store($post->trans_store_id);
		$customer       = new Customer($post->trans_customer_id);
		$patient        = $customer->patient();
		$insurance      = new Insurance($post->trans_insurance_id);
		$scheme         = new Scheme($post->trans_scheme_id);
		$pricelist      = new Pricelist($scheme->pricelist_id);
		$visit          = $patient->visit();
		$location       = $patient->location();
		$folio          = $visit->folio();
		$items_obj 		= json_decode($items);
		if(!$items_obj) die($app->renderAlertError('Please add at least one item'));

		if(strlen($post->trans_reason) < 3) {
			die($app->renderAlertError('Please choose return reason'));
		}

		if(strlen($post->trans_requested_notes) < 3) {
			die($app->renderAlertError('Please return reasons or comments'));
		}

		$this->checkStockAvailability($post);

		$trans_reason_category	= $app->getCategoryByCode('SaleReturnType', $post->trans_reason);

		$customer               = $this->createCustomerIfNotExists(
			array (
				'id'            => $post->trans_customer_id,
				'name'          => $post->trans_customer_name,
				'phone'         => $post->trans_customer_phone,
				'insurance_id'  => $post->trans_insurance_id,
				'scheme_id'     => $post->trans_scheme_id
			)
		);

		// Substract returned items from original items

		$bill 					= new Bill($post->parent_id);
		$billed_items 			= $bill->items();
		$returned_items			= json_decode($items);
		$trans_items 			= array();

		$trans_cost  			= 0;
		$trans_subtotal 		= 0;
		$trans_payable 			= 0;

		foreach($returned_items as $returned_item) {
			$billed_item 		= $billed_items[$returned_item->item_id];
			$new_quantity		= $returned_item->quantity - $billed_item->quantity;
			if($new_quantity != 0) {
				$returned_item->quantity			= $new_quantity;
				$returned_item->requested_quantity	= $billed_item->quantity;
				$returned_item->qty					= $new_quantity * $billed_item->size;
				$returned_item->cost				= $new_quantity * $billed_item->unit_cost;
				$returned_item->price				= $new_quantity * $billed_item->unit_price;
				$trans_cost							+= $returned_item->cost;
				$trans_subtotal						+= $returned_item->price;
				$trans_payable						+= $returned_item->price;

				$trans_items[]						= $returned_item;
			}
		}

        $create_data            = array (
			'type'              => $this->type,
			'subtype'           => $post->trans_subtype,
			'store_id'          => $post->trans_store_id,
			'parent'          	=> $post->parent,
			'parent_id'         => $post->parent_id,
			'reference'         => $post->trans_reason,
			'reference_no'     	=> $trans_reason_category->name,

			'personnel'         => 'Customer',
			'personnel_id'      => $post->trans_customer_id,
			'code'              => $customer->code,
			'name'              => $post->trans_customer_name,
			'birthdate'         => $post->trans_customer_birthdate,
			'sex'               => $post->trans_customer_sex,
			'phone'             => $post->trans_customer_phone,
			'email'             => $customer->person()->email,

			'pricelist'         => $pricelist->code,
			'pricelist_id'      => $pricelist->id,
			'insurance'         => $insurance->code,
			'insurance_id'      => $insurance->id,
			'scheme'            => $scheme->code,
			'scheme_id'         => $scheme->id,

			'folio_id'          => $folio->id,
			'folio_no'          => $folio->folio_no,

			'visit_id'          => $visit->id,
			'visit_location_id' => $location->id,

			'requested_notes'   => $post->trans_requested_notes,
			'items'             => json_encode($trans_items),
			'cost'              => $trans_cost,
			'subtotal'          => $trans_subtotal,
			'billed_reference'  => $bill->subtotal,
			'due'           	=> $trans_payable + $bill->subtotal,
			'progeress'         => 'Requested',
			'progress_code'     => 1
		);

        $transaction_id         = $this->createWithMinimumData($create_data);

        if($transaction_id) {
            return $app->renderAlertSuccess($submodule->shortname.' created successfully', 'Success', 'modal_lg', true);
        }
    }

	function renderTableRow() {
		global $app;

		$link_options   = array('id'=>$this->id, 'name'=>$this->name, 'modal'=>'modal_lg');
		$scheme         = new Scheme($this->scheme_id);
		$html 			= '';

		$html .= '<tr>
		<td class="text-muted fw-semibold">'.$app->datetime($this->created_datetime).'</td>
		<td> <div class="fw-semibold text-auto-truncate">'.$this->id.' '.$this->subtype.'</div><div class="text-muted">'.$this->store()->location()->name.'</div></td>
		<td><div class="mb-1">'.$app->renderLinkOpen($link_options).'</div> <div>File '.$app->formatCode($this->code).' &rarr; '.$app->age($this->birthdate).' '.$this->sex.'<br /><small class="text-danger">'.$this->reference_no.'</small></div></td>
		<td>
		<div class="fw-semibold">'.$app->shortname($scheme->insurance()->code, 8).'</div>
		<div class="">'.$app->shortname($scheme->name, 9).'</div>
		</td>
		
		<td class="text-end">';

		$html .= '<s class="fw-bold text-muted">'.$app->formatMoney($this->billed_reference).'</s> <br /><span class="text-success fw-bold">'.$app->formatMoney($this->due).'</span>';

		//if(strlen($this->reference)) $html .= '<br /><small class="text-success">'.$this->reference.'</small>';
		
		$html .= '</td>
		<td class="text-end fw-bold text-muted">'.$app->formatMoney($this->payable).'</td>
		
		<td class="text-center">'.$this->renderButtonProgress().'</td>
		<td class="text-center text-nowrap">';

		if($this->progress == 'Requested') {
			$html .= $this->renderLinkAuthorize();
			$html .= $this->renderLinkDelete();
		} elseif($this->progress == 'Authorized') {
			$html .= $this->renderLinkApprove();
			$html .= $this->renderLinkDelete();
		} elseif($this->progress == 'Approved') {
			$html .= $this->renderLinkTransfer();
			$html .= $this->renderLinkDelete();
		}

		$html .= '</td>
		</tr>';

		return $html;
	}

	function renderTableRow2() {
		global $app;

		$scheme         = new Scheme($this->scheme_id);
		$html 			= '';

		$html .= '<tr>
		<td>'.$app->date($this->created_datetime).'</td>
		<td>
		<div class="list-text fw-bold">'.$this->name.'</div>
		<div class="text-muted">'.ucwords($app->age($this->birthdate).' '.$this->sex).'</div>
		</td>

		<td>
		<div class="list-text">'.$app->shortname($scheme->code, 8).'</div>
		<div class="text-muted d-none">'.$app->shortname($scheme->name, 8).'</div>
		</td>
		
		<td class="text-end fw-bold text-muted">';

		$html .= $app->formatMoney($this->billed_reference);

		if(strlen($this->reference)) $html .= '<br /><small class="text-success">'.$this->reference.'</small>';
		
		$html .= '</td>
		<td class="text-end fw-bold text-muted">'.$app->formatMoney($this->payable).'</td>
		<td class="text-end fw-bold text-muted">'.$app->formatMoney($this->due).'</td>
		
		<td class="text-center">'.$this->renderButtonProgress().'</td>
		<td class="text-center text-nowrap">';

		if($this->progress == 'Requested') {
			$html .= $this->renderLinkAuthorize();
			$html .= $this->renderLinkDelete();
		} elseif($this->progress == 'Authorized') {
			$html .= $this->renderLinkApprove();
			$html .= $this->renderLinkDelete();
		} elseif($this->progress == 'Approved') {
			$html .= $this->renderLinkTransfer();
			$html .= $this->renderLinkDelete();
		}

		$html .= '</td>
		</tr>';

		return $html;
	}

	function renderTransferConfirmation($submodule, $post) {
        global $app, $user;

        $user->checkSubmoduleAccess($submodule, 'edit');   

		$bill 							= new Bill($this->parent_id);
		$bill_items 					= $bill->items();
		$returned_items 				= $this->items();
		$trans_items 					= array();
		$trans_cost 					= 0;
		$trans_subtotal 				= 0;

		// If bill was dispensed return stock
		// $this->checkStockAvailability($returned_items, $bill->store()->id);
		
		foreach($returned_items as $returned_item) {
			if($bill_items[$returned_item->item_id]->dispensed_quantity >= ($returned_item->quantity * -1)) {
				$return_item				= new Item($returned_item->item_id);
				$returned_item->parent_obj 	= $this;
				$adjust_data 				= array (
					'item'					=> $returned_item,
					'quantity'				=> ($returned_item->quantity * -1),
					'adjust'				=> true
				);
				$return_item->adjustStock($adjust_data);
			}
		}

		$returned_items 				= $this->items();

		foreach($returned_items as $returned_item) {
			if(isset($bill_items[$returned_item->item_id])) {
				$temp_item				= $bill_items[$returned_item->item_id];
				$temp_item->quantity	= $temp_item->quantity + $returned_item->quantity;
				$temp_item->requested_quantity	= $temp_item->quantity;
				$temp_item->qty 		= $temp_item->quantity * $temp_item->size;
				$temp_item->cost 		= $temp_item->quantity * $temp_item->unit_cost;
				$temp_item->price 		= $temp_item->quantity * $temp_item->unit_price;
				$trans_cost 			+= $temp_item->cost;
				$trans_subtotal			+= $temp_item->price;
				$trans_items[]			= $temp_item;
			}
		}

		/*
		foreach($bill->invoices() as $invoice) {
			if(!$invoice->returned) {
				die($app->renderAlertEmpty('Invoice number '.$invoice->formatCode($invoice->id).' must be canceled first.', 'danger'));
			}
		}
		*/

        $update_data            = array (
            'returned_notes'    => $post->trans_transferred_notes,
            'items'             => json_encode($trans_items),
            'cost'              => $app->unformatMoney($trans_cost),
            'subtotal'          => $app->unformatMoney($trans_subtotal),
			'payable'			=> $app->unformatMoney($trans_subtotal)
        );

        $updated                = $bill->update($update_data);
        
		if($updated || 1) {
            $bill->resetItems($trans_items);
			// $bill->changeProgress('Requested');
			$bill->updateInlineItems();

			$this->changeProgress('Transferred');
			$this->updateInlineItems();

			// Topup wallet
			if($this->subtotal < 0) {

				$wallet			= $this->customer()->wallet();
				$update_data 	= array (
					'subtotal'	=> $wallet->subtotal + abs($this->subtotal)
				);
		
				$wallet->update($update_data);
				$wallet->read();

				// Create wallet transaction
				$wallet_transaction 	= new WalletTransaction();
				$items                  = json_encode($this->getItems());
				$create_data            = array (
					'type'              => $this->type,
					'type_id'           => $this->id,
					'subtype'           => $this->subtype,
					'subtype_id'        => $this->subtype_id,
					'store_id'          => $this->store_id,

					'parent'         	=> 'Wallet',
					'parent_id'      	=> $wallet->id,
					'personnel'         => $this->personnel,
					'personnel_id'      => $this->personnel_id,
					'code'              => $this->code,
					'name'              => $this->name,
					'birthdate'         => $this->birthdate,
					'sex'               => $this->sex,
					'phone'             => $this->phone,
					'email'             => $this->email,

					'pricelist'         => $this->pricelist,
					'pricelist_id'      => $this->pricelist_id,
					'insurance'         => $this->insurance,
					'insurance_id'      => $this->insurance_id,
					'scheme'            => $this->scheme,
					'scheme_id'         => $this->scheme_id,

					'folio_id'          => $this->folio_id,
					'folio_no'          => $this->folio_no,

					'visit_id'          => $this->visit_id,
					'visit_location_id' => $this->visit_location_id,

					'requested_notes'   => $this->requested_notes,
					'items'             => $items,
					'cost'              => $this->cost,
					'subtotal'          => abs($this->subtotal),
					'due'          		=> $wallet->subtotal,
					'progress'         	=> 'Requested',
					'progress_code'     => 1
				);

				$wallettransaction_id	= $wallet_transaction->createWithMinimumData($create_data);
				
			}

            return $app->renderAlertSuccess('Bill number '.$this->parent_id.' returned successfully', 'Success', 'modal_lg', 1);
        }
    }

	function renderTransactionItemsReadonly($title = 'Returned items') {
		global $app;

		$html = '<div class="trans-container my-3 mb-5"> <h5 class="trans-subtitle"> <span class="fa-regular fa-cart-circle-check fa-fw"></span> &nbsp;'.$title.':</h6>
		<div class="card p-3 my-3 bg-primary" style="--bs-bg-opacity: .05;">
		<div class="table-responsive">
		<table class="table table-hover align-middle">
		<thead class="h5">
		<tr>
		<th class="w-5"></th>
		<th class="">Item name</th>
		<th class="text-center w-12">Qty</th>
		<th class="w-20 text-end">Price</th>
		<th class="w-20 text-end">Amount</th>
		</tr>
		</thead>
		<tbody id="trans_items_container">';

		$bill		= new Bill($this->parent_id);
		$bill_items	= $bill->items();

		foreach($this->items() as $item) {
			$icon	= '<span class="fa-solid fa-plus"></span>';
			if($item->progress == 'Billed') $icon	= '<span class="fa-regular fa-money-check-dollar-pen"></span>';
            if($item->progress == 'Paid') $icon	= '<span class="fa-solid fa-check-double"></span>';
            if($item->progress == 'Dispensed') $icon	= '<span class="fa-solid fa-arrow-up-from-bracket"></span>';
            
			$details = strlen($item->details) ? '<div class="text-muted"><small>'.$item->details.'</small></div>' : '';
			
			$bill_item	= $bill_items[$item->item_id];

			$html   .= '<tr>
			<td class="text-success text-center">'.$icon.'</td>
			<td class="">'.$bill_item->name.' <b class="text-muted">('.$bill_item->pack.')</b> '.$details.'</td>
			<td class="text-center">'.$bill_item->requested_quantity.'</td>
			<td class="text-end">'.$app->formatMoney($bill_item->unit_price).'</td>
			<td class="text-end">'.$app->formatMoney($bill_item->price).'</td>
			</tr>';

			$html   .= '<tr class="text-danger">
			<td class="text-danger text-center"><span class="fa fa-minus"></span></td>
			<td class="text-danger">Returned</td>
			<td class="text-center text-danger">'.$item->quantity.'</td>
			<td class="text-end text-danger">'.$app->formatMoney($item->unit_price).'</td>
			<td class="text-end text-danger">'.$app->formatMoney($item->price).'</td>
			</tr>';

			$html   .= '<tr> <td colspan="5" class="border-0"></td> </tr>';
		}

		$html .= '</tbody>
		<tfoot>
		<tr>
		<th></th>
		<th class="fw-semibold">Total amount returned</th>
		<th></th>
		<th></th>
		<th class="text-end">'.$app->renderInputText('trans_subtotal', $app->formatMoney($this->subtotal), 'Subtotal', 'line-control-plaintext text-end fw-semibold', 'readonly').'</th>
        </tr>
		</tfoot>
		</table>
		</div>
		</div>
		</div>';
		return $html;
	}
}