<?php 

class Wallet extends Sale {
	function __construct($id=0) {	
		$this->table        = 'billing_wallets';
		$this->items_table	= 'billing_wallet_items';
		$this->primary_key  = 'id';
		parent::__construct($id);
		$this->type 		= 'Wallet';	
	}

	function renderCreateConfirmation($submodule, $post) {
        global $app, $user;

		// Get or create customer wallet
		$customer 				= new Customer($post->trans_customer_id);
		$person 				= $customer->person();
		$patient        		= $customer->patient();
		$wallet 				= $customer->wallet();

		// Create invoice
		$invoice 				= new Invoice();
        $items          		= $app->uncleanText($post->trans_items);
		$insurance      		= new Insurance($post->trans_insurance_id);
		$scheme         		= new Scheme($post->trans_scheme_id);
		$pricelist      		= new Pricelist($scheme->pricelist_id);
		$visit          		= $patient->visit();
		$location       		= $patient->location();
		$folio          		= $visit->folio();
		if(!json_decode($items)) die($app->renderAlertError('Please add at least one item'));

        $create_data            = array (
			'type'              => 'Invoice',
			'subtype'           => 'Wallet',
			'store_id'          => $post->trans_store_id,

			'parent'         	=> 'Wallet',
			'parent_id'      	=> $wallet->id,
			'personnel'         => 'Customer',
			'personnel_id'      => $customer->id,
			'code'              => $customer->code,
			'name'              => $person->name,
			'birthdate'         => $person->birthdate,
			'sex'               => $person->sex,
			'phone'             => $person->phone,
			'email'             => $person->email,

			'pricelist'         => $pricelist->code,
			'pricelist_id'      => $pricelist->id,
			'insurance'         => $insurance->code,
			'insurance_id'      => $insurance->id,
			'scheme'            => $scheme->code,
			'scheme_id'         => $scheme->id,

			'folio_id'          => $folio->id,
			'folio_no'          => $folio->folio_no,

			'visit_id'          => $visit->id,
			'visit_location_id' => $location->id,

			'requested_notes'   => $post->trans_requested_notes,
			'items'             => $items,
			'cost'              => $post->trans_cost,
			'subtotal'          => $post->trans_subtotal,
			'progeress'         => 'Requested',
			'progress_code'     => 1
		);

        $transaction_id         = $invoice->createWithMinimumData($create_data);
        if($transaction_id) {
            return $app->renderAlertSuccess('Wallet transaction requested successfully', 'Success', 'modal_lg', true);
        }
    }

	function topup($invoice_id) {
		global $app;

		$invoice 			= new Invoice($invoice_id);
		if($invoice->subtype == 'Wallet') {
			$amount 		= $invoice->subtotal;
		} else {
			$amount 		= $invoice->subtotal * -1;
		}

		$update_data 		= array (
			'subtotal'		=> $this->subtotal + $amount
		);

		$this->update($update_data);
		$this->read();

		if($amount < 0) {
			echo $app->renderAlertSuccess('TZS '.$app->formatMoney(abs($invoice->payable)).' withdrawn. New balance: TZS '.$app->formatMoney($this->subtotal), 'Wallet');
		} else {
			echo $app->renderAlertSuccess('TZS '.$app->formatMoney($invoice->payable).' deposited. New balance: TZS '.$app->formatMoney($this->subtotal), 'Wallet');
		}

		// Create wallet transaction
		$wallet_transaction 	= new WalletTransaction();
		$items                  = json_encode($invoice->getItems());
		$create_data            = array (
			'type'              => $invoice->type,
			'type_id'           => $invoice->id,
			'subtype'           => $invoice->subtype,
			'subtype_id'        => $invoice->subtype_id,
			'store_id'          => $invoice->store_id,

			'parent'         	=> 'Wallet',
			'parent_id'      	=> $this->id,
			'personnel'         => $invoice->personnel,
			'personnel_id'      => $invoice->personnel_id,
			'code'              => $invoice->code,
			'name'              => $invoice->name,
			'birthdate'         => $invoice->birthdate,
			'sex'               => $invoice->sex,
			'phone'             => $invoice->phone,
			'email'             => $invoice->email,

			'pricelist'         => $invoice->pricelist,
			'pricelist_id'      => $invoice->pricelist_id,
			'insurance'         => $invoice->insurance,
			'insurance_id'      => $invoice->insurance_id,
			'scheme'            => $invoice->scheme,
			'scheme_id'         => $invoice->scheme_id,

			'folio_id'          => $invoice->folio_id,
			'folio_no'          => $invoice->folio_no,

			'visit_id'          => $invoice->visit_id,
			'visit_location_id' => $invoice->visit_location_id,

			'requested_notes'   => $invoice->requested_notes,
			'items'             => $items,
			'cost'              => $invoice->cost,
			'subtotal'          => $amount,
			'due'          		=> $this->subtotal,
			'progress'         	=> 'Requested',
			'progress_code'     => 1
		);

		$wallettransaction_id	= $wallet_transaction->createWithMinimumData($create_data);
	}

	function renderPreviewCard() {
		global $app, $user, $root;

		$customer	= $this->customer();
		$name 		= $this->customer()->person()->name;
		if(!$name)	$name = '(NO NAME)';

		$html 		= '<div class="card shadow shadow-lg">
		<div class="card-body p-3 rounded border border-secondary" style="background-image: linear-gradient(to right, rgba(0,0,0, 0.02), rgba(0,0,0, 0.1))">
		<div class="row">
		<div class="col-3">
		<img src="'.$customer->picture().'" class="rounded" style="width:48px; " />
		</div>
		<div class="col-9 text-end">
		<h6 class="card-subtitle text-body-secondary mb-1">';

		if($user->tagged('accountant')) {
			$html 	.= '<a title="Adjust" class="remote-modal bs-tooltip btn btn-sm btn-outline-primary" data-bs-toggle="modal" data-bs-target="#modal_lg" data-bs-backdrop="static" data-bs-keyboard="false" tabindex="-1" href="adjust_form.php?action=edit&id='.$this->id.'"> <span class="fa-regular fa-pencil-alt"></span> ADJUST </a>';
		}
		
		$html 		.= ' WALLET BALANCE</h6>
		<h4 class="card-title my-1"><span class="fw-bold">TZS '.number_format($this->subtotal,2).'</span></h4>
		</div>
		</div>
		<a target="blank" href="'.$root.'pages/billing/reports/walletstatement?keyword='.$customer->code.'" class="d-block h4 my-4 mb-5 text-truncate text-decoration-none">'.$name.'</a>
		<div class="row">
		<div class="col-6 text-body-secondary">
		<h6 class="card-subtitle mb-1">CARD NUMBER</h6>
		<h4 class="card-title my-1">'.$app->formatCode($customer->code).'</h4>
		</div>
		<div class="col-6 text-end text-body-secondary">
		<h6 class="card-subtitle mb-1">DATE CREATED</h6>
		<h4 class="card-title my-1">'.date('Y/m', strtotime($this->created_datetime)).'</h4>
		</div>
		</div>
		</div>
		</div>';

		return $html;
	}
}

?>