<?php 

class Patient extends Crud {
	function __construct($id=0) {		
		$this->table        = 'clinical_patients';
		$this->primary_key  = 'id';
		parent::__construct($id);
	}

	function code() {
		return $this->id ? $this->code : $this->nextCode();
	}

	function formatCode() {
		$digits = 8;
		if(strlen($this->code) < 7) $digits = 6;
		return sprintf('%0'.$digits.'d', $this->code);
	}

	function person() {
		$person				= new Person($this->person_id);
		$person->personnel 	= 'Patient';
		return $person;
	}

	function customer() {
		return new Customer($this->customer_id);
	}

	public function picture() {
		global $root;
		$person		= $this->person();
		$age_string = $person->age();
		$sex		= strtolower($person->sex);
		$picture 	= 'unknown.png';
		$ages		= explode(' ', $age_string);
		
		if((count($ages) == 2) && ($sex == 'male' || $sex == 'female')) {
			$age_number		= $ages[0];
			$age_units		= $ages[1];
			$agegroup 	= 'baby';
			if($age_units 	== 'year' || $age_units 	== 'years') {
				if($age_number > 5) 	$agegroup = 'child';
				if($age_number > 18) 	$agegroup = 'adult';
				if($age_number > 60) 	$agegroup = 'old';
			}
			$picture	= $sex.'-'.$agegroup.'.png';
		}
		return $root.'assets/images/patients/'.$picture;
	}

	function signature() {
		global $app, $root;
		$attachment = $app->getAttachmentByCode('Customer', $this->id, 'CustomerSignature');
		if($attachment->exists()) return $attachment->renderOpenPath();
		return $root.'assets/images/default/signature.png';
	}

	function renderBio($name='') {

		$person 	= $this->person();
		$customer	= $this->customer();

		if($this->id) {
			$html	= '<img src="'.$this->picture().'" class="rounded-circle mx-auto d-block bg-light my-3" style="width:85%; max-width:185px; border:8px solid white" />
			<div class="px-3 me-2">
			<h4 class="text-white fw-bold">'.$customer->formatCode().'</h4>
			<h5 class="mb-1 text-truncate text-white fw-bold">'.$person->name.'</h5>
			<h5 class="text-white fw-semibold">'.ucwords($person->age()).' '.$person->sex.'</h5>
			<h6 class="fw-normal">'.$this->customer()->scheme()->name.'</h6>
			</div>';
		} else {
			$html	= '<img src="'.$this->picture().'" class="rounded-circle mx-auto d-block bg-light my-3" style="width:85%; max-width:185px; border:8px solid white" />
			<div class="px-3 me-2">
			<h4 class="text-white fw-bold">000000</h4>
			<h5 class="mb-1 text-truncate text-white fw-bold">Firstname Lastname</h5>
			<h5>Age - Sex</h5>
			</div>';
		}
		
		return $html; 
	}

	function checkinDot() {
		if($this->visit()->id) {
			$html = '<span class="text-success fa-solid fa-check-circle"></span>';
		} else {
			$html = '<span class="text-warning fa-solid fa-clock"></span>';
		}
		return $html;
	}

	function renderRegistrationCard() {
		global $app;

		$color 				= 'secondary';
		$customer 			= $this->customer();
		$person 			= $this->person();
		$visit_location 	= $this->lastVisitLocation();
		$button_options 	= array('id'=>$this->id, 'css'=>'btn btn-secondary btn-lg');
		$buttons			= $app->renderButtonEdit($button_options);

		$status_color 		= 'secondary';
		$status_icon 		= 'fa-solid fa-pencil';
		$back_details       = '<h5> Registered </h5>
		<div>'.$app->datetime($this->created_datetime).'</div>
		<div>'.$this->createdUser()->person()->name.'</div>
		<div>'.$app->age($this->created_datetime).' ago </div>';

		if($visit_location->admitted) {
			if($visit_location->ward_id) {
				$status_color 	= 'success';
				$status_icon 	= 'fa-solid fa-bed-pulse';
				$back_details   = '<h5> Admitted </h5>
				<h6 class="text-truncate">'.$visit_location->location()->location()->name.'</h6>
				<div class="text-truncate">'.$app->datetime($visit_location->created_datetime).'</div>
				<div>'.$visit_location->createdUser()->person()->name.'</div>
				<div>'.$app->age($visit_location->created_datetime).' ago </div>';
			} else {
				$doctor_user 	= new User($visit_location->doctor_id);
				$status_color 	= 'success';
				$status_icon 	= 'fa-solid fa-check';
				$back_details   = '<h5> Checked in </h5>
				<div class="text-truncate">'.$visit_location->location()->location()->name.'</div>
				<div>'.$app->datetime($visit_location->created_datetime).'</div>
				<div>'.$app->age($visit_location->created_datetime).' ago </div>';

				if($visit_location->doctor_id) {
					$back_details   .= '<div class="fw-bold"><span class="fa fa-regular fa-user-doctor fa-fw"></span> '.$doctor_user->person()->name.'</div>';
				}
			}
		} elseif($visit_location->id) {
			if($visit_location->ward_id) {
				$status_color 	= 'orange';
				$status_icon 	= 'fa-regular fa-clock';
				$back_details   = '<h5> Last admitted </h5>
				<h6 class="text-truncate">'.$visit_location->location()->location()->name.'</h6>
				<div class="text-truncate">'.$app->datetime($visit_location->created_datetime).'</div>
				<div>'.$app->age($visit_location->created_datetime).' ago </div>';
			} else {
				$doctor_user 	= new User($visit_location->doctor_id);
				$status_color 	= 'orange';
				$status_icon 	= 'fa-regular fa-clock';
				$back_details   = '<h5> Last checked in </h5>
				<div class="text-truncate">'.$visit_location->location()->location()->name.'</div>
				<div>'.$app->datetime($visit_location->created_datetime).'</div>
				<div>'.$app->age($visit_location->created_datetime).' ago </div>';

				if($visit_location->doctor_id) {
					$back_details   .= '<div class="fw-bold"><span class="fa fa-regular fa-user-doctor fa-fw"></span> '.$doctor_user->person()->name.'</div>';
				}
			}
		}
		
		$html 				= '<div id="card_patient_'.$this->id.'" class="auto-flip bs-tooltip" title="">
		<div class="shadow shadow-lg card border-'.$color.' text-center front " style="background-image: linear-gradient(to right, rgba(0,0,0, 0.01), rgba(0,0,0, 0.1))">
		<div class="card-img-top bg-'.$color.' text-white py-4" style="height:200px; min-height:200px">
		'.$buttons.'
		</div>
		<img src="'.$this->picture().'" class="rounded-circle mx-auto d-block bg-light" style="width:75%; max-width:190px; margin-top:-120px; border:8px solid white">
		<div style="display:inline-block; position:relative; top:-16px; font-size:.75em"> <span class="bg-'.$status_color.' text-white" style="padding:0.4em 0.45em; border:6px solid white; border-radius:50%;"> <span class="'.$status_icon.' fa-fw"></span> </div>
		<div class="card-body pt-0" style="min-height:180px;">
		
		<h4 class="fw-bold"> '.$this->formatCode().' </h4>
		<h5 class="fw-semibold" style="font-size:1.1em">'.$person->name.'</h5>
		
		<h6>'.ucwords($person->age().' '.$person->sex).'</h6>
		<h6 class="text-muted text-truncate my-3"><span class="fa-regular fa-tags fa-fw"></span> '.$app->shortname($customer->renderInsuranceScheme(), 30).' </h6>

		</div></div>
		
		<div class="shadow-sm card border-'.$color.' text-center back bg-'.$color.'" style="--bs-bg-opacity: .08">
		<div class="card-img-top bg-'.$color.' text-white py-4" style="height:200px; min-height:200px">
		<a class="btn btn-'.$color.' bs-tooltip" data-flip="card_patient_'.$this->id.'" title="Flip"> <span class="fa-solid fa-arrow-left"></span> </a>
		'.$back_details.'

		</div>
		<img src="'.$this->picture().'" class="rounded-circle mx-auto d-block bg-light" style="visibility:hidden; width:75%; max-width:190px; margin-top:-120px; border:7px solid white">
		<div class="card-body" style="min-height:180px;">
		<h5><span class="fa-light fa-phone fa-fw"></span></h5>
		<h4 class="fw-light">'.$person->phone.'</h4>
		<div class="fw-light"> '.$person->village()->name.' - '.$person->district()->name.' - '.$person->region()->name.'</div>
		</div></div></div>';
		
		return $html;
	}

	function renderAdmissionScheduleCard() {
		global $root, $app;

		$color 				= 'secondary';
		$customer 			= $this->customer();
		$person 			= $this->person();
		$visit_location 	= $this->visitLocation();
		$button_options 	= array('id'=>$this->id, 'css'=>'btn btn-secondary btn-lg', 'title'=>'Confirm admission');
		$buttons			= '<a class="remote-modal bs-tooltip btn btn-secondary btn-lg" title="Confirm admission" data-bs-toggle="modal" data-bs-target="#modal_lg" data-bs-backdrop="static" data-bs-keyboard="false" tabindex="-1" href="admission_form.php?action=edit&id='.$this->id.'"> <span class="fa-regular fa-pen-to-square"></span> </a>';
        
		$status_color 		= 'orange';
		$status_icon 		= 'fa-solid fa-pencil';
		$back_details       = '<div><h5> Registered </h5></div>
		<div>'.$app->datetime($this->created_datetime).'</div>
		<div>'.$this->createdUser()->person()->name.'</div>
		<div>'.$app->age($this->created_datetime).' ago </div>';

		$ward 			= new Ward($this->admission_location_id);
		$doctor 		= new User($this->admission_user_id);

		$status_color 	= 'success';
		$status_icon 	= 'fa-solid fa-user-check';
		$back_details   = '<div><h5> '.$ward->location()->name.' </h5></div>
		<div>'.$app->datetime($this->admission_datetime).'</div>
		<div>'.$doctor->person()->name.'</div>
		<div>'.$app->age($this->admission_datetime).' ago </div>';
		
		$html 				= '<div id="card_patient_'.$this->id.'" class="auto-flip bs-tooltip" title="">
		<div class="shadow shadow-lg card border-'.$color.' text-center front bg-'.$color.'" style="--bs-bg-opacity: .08">
		<div class="card-img-top bg-'.$color.' text-white py-4" style="height:200px; min-height:200px">
		'.$buttons.'
		</div>
		<img src="'.$this->picture().'" class="rounded-circle mx-auto d-block bg-light" style="width:75%; max-width:190px; margin-top:-120px; border:8px solid white">
		<div style="display:inline-block; position:relative; top:-16px; font-size:.75em"> <span class="bg-'.$status_color.' text-white" style="padding:0.4em 0.45em; border:6px solid white; border-radius:50%;"> <span class="'.$status_icon.' fa-fw"></span> </div>
		<div class="card-body pt-0" style="min-height:180px;">
		
		<h5 class="fw-bold">'.$person->name.'</h4>
		<h5 class="fw-semibold"> '.$customer->formatCode().' </h5>
		<h6>'.ucwords($person->age().' '.$person->sex).'</h6>
		<h6 class="text-muted text-truncate my-3"><span class="fa-regular fa-tags fa-fw"></span> '.$app->shortname($customer->renderInsuranceScheme(), 30).' </h6>

		</div></div>
		
		<div class="shadow-sm card border-'.$color.' text-center back bg-'.$color.'" style="--bs-bg-opacity: .08">
		<div class="card-img-top bg-'.$color.' text-white py-4" style="height:200px; min-height:200px">
		<a class="btn btn-'.$color.' bs-tooltip" data-flip="card_patient_'.$this->id.'" title="Flip"> <span class="fa-solid fa-arrow-left"></span> </a>
		'.$back_details.'

		</div>
		<img src="'.$this->picture().'" class="rounded-circle mx-auto d-block bg-light" style="visibility:hidden; width:75%; max-width:190px; margin-top:-120px; border:7px solid white">
		<div class="card-body" style="min-height:180px;">
		'.$this->admission_notes.'
		</div></div></div>';

		return $html;
	}



	function renderSearchCard() { 
		global $r, $app;

		$color 				= 'secondary';
		$patient 			= $this;
		$customer 			= $patient->customer();
		$person 			= $patient->person();
		$button_options 	= array('id'=>$patient->id, 'css'=>'btn btn-primary');
		$buttons			= $app->renderButtonEdit($button_options).''.$app->renderButtonCopy($button_options).''.$app->renderButtonDelete($button_options);
		$buttons 			= '<a class="remote-modal bs-tooltip btn btn-lg btn-'.$color.'" title="Patient file" data-bs-toggle="modal" data-bs-target="#modal_xl" href="form.php?action=open&id='.$patient->id.'&r='.$app->root($r).'"> <span class="fa-regular fa-folder-user"></span> </a>';

		$visit_location 	= $this->visitLocation();
		$status_color 		= 'orange';
		$status_icon 		= 'fa-solid fa-pencil';
		$back_details       = '<div><h5> Registered </h5></div>
		<div>'.$app->datetime($this->created_datetime).'</div>
		<div>'.$this->createdUser()->person()->name.'</div>
		<div>'.$app->age($this->created_datetime).' ago </div>';

		if($visit_location->id) {
			if($visit_location->care_type == 'Inpatient') {
				$status_color 	= 'success';
				$status_icon 	= 'fa-solid fa-bed-pulse';
				$back_details   = '<div><h5> Admitted </h5></div>
				<h6>'.$visit_location->location()->location()->name.'</h6>
				<div>'.$app->datetime($visit_location->created_datetime).'</div>
				<div>'.$visit_location->createdUser()->person()->name.'</div>
				<div>'.$app->age($visit_location->created_datetime).' ago </div>';
			} else {
				$status_color 	= 'success';
				$status_icon 	= 'fa-solid fa-user-doctor fa-beat';
				$back_details   = '<div><h5> Checked in </h5></div>
				<h6>'.$visit_location->location()->location()->name.'</h6>
				<div>'.$app->datetime($visit_location->created_datetime).'</div>
				<div>'.$app->age($visit_location->created_datetime).' ago </div>
				<div>'.$visit_location->createdUser()->person()->name.'</div>';
			}
		}

		$html 				= '<div id="card_patient_'.$patient->id.'" class="auto-flip bs-tooltip" title="">
		<div class="shadow shadow-lg card border-'.$color.' text-center front bg-'.$color.'" style="--bs-bg-opacity: .08">
		<div class="card-img-top bg-'.$color.' text-white py-4" style="height:200px; min-height:200px">
		'.$buttons.'
		</div>
		<img src="'.$patient->picture().'" class="rounded-circle mx-auto d-block bg-light" style="width:75%; max-width:190px; margin-top:-120px; border:8px solid white">
		<div style="display:inline-block; position:relative; top:-16px; font-size:.75em"> <span class="bg-'.$status_color.' text-white" style="padding:0.4em 0.45em; border:6px solid white; border-radius:50%;"> <span class="'.$status_icon.' fa-fw"></span> </div>
		<div class="card-body pt-0" style="min-height:180px;">
		
		<h5 class="fw-bold">'.$customer->formatCode().' </h5>
		<h5 class="fw-semibold">'.$person->name.'</h4>

		<h6>'.ucwords($person->age().' '.$person->sex).'</h6>
		<h6 class="text-muted text-truncate my-3"><span class="fa-regular fa-tags fa-fw"></span> '.$app->shortname($customer->renderInsuranceScheme(), 30).' </h6>

		</div></div>
		
		<div class="shadow-sm card border-'.$color.' text-center back bg-'.$color.'" style="--bs-bg-opacity: .08">
		<div class="card-img-top bg-'.$color.' text-white py-4" style="height:200px; min-height:200px">
		<a class="btn btn-'.$color.' bs-tooltip" data-flip="card_patient_'.$patient->id.'" title="Flip"> <span class="fa-solid fa-arrow-left"></span> </a>
		'.$back_details.'

		</div>
		<img src="'.$patient->picture().'" class="rounded-circle mx-auto d-block bg-light" style="visibility:hidden; width:75%; max-width:190px; margin-top:-120px; border:7px solid white">
		<div class="card-body" style="min-height:180px;">
		<h5><span class="fa-light fa-phone fa-fw"></span></h5>
		<h4 class="fw-light">'.$person->phone.'</h4>
		<div class="fw-light"> '.$person->village()->name.' - '.$person->district()->name.' - '.$person->region()->name.'</div>
		</div></div></div>';

		return $html;
	}

	function visit() {
		global $app;

		$sql = "SELECT id FROM clinical_visits WHERE patient_id = $this->id AND admitted = 1";
		foreach($app->read($sql) as $row) {
			return new Visit($row->id);
		}
		return new Visit();
	}

	function lastVisit() {
		global $app;

		$sql = "SELECT id FROM clinical_visits WHERE patient_id = $this->id ORDER BY id DESC";
		foreach($app->read($sql) as $row) {
			return new Visit($row->id);
		}
		return new Visit();
	}

	function visits() {
		global $app;

		$visits	= array();

		$sql 			= "SELECT id FROM clinical_visits WHERE patient_id = $this->id ORDER BY id";
		foreach($app->read($sql) as $row) {
			$visits[] = $row->id;
		}
		return $visits;
	}

	function listVisitOptions() {
		$options 	= array();
		$visits		= array_reverse($this->visits());
		foreach($visits as $visit_id) {
			$visit			= new Visit($visit_id);
			$visit_location	= $visit->lastVisitLocation();

			$options[]		= array (
				'name'		=> date('d M Y', strtotime($visit->created_datetime)).' &rarr; '.$visit_location->location()->location()->name, 
				'value'		=> $visit->id
			);
		}
		return $options;
	}

	function visitLocation() {
		global $app;

		$sql = "SELECT id FROM clinical_visit_locations WHERE patient_id = $this->id AND admitted = 1";
		foreach($app->read($sql) as $row) {
			return new VisitLocation($row->id);
		}
		return new VisitLocation();
	}

	function lastVisitLocation() {
		global $app;

		$sql = "SELECT id FROM clinical_visit_locations WHERE patient_id = $this->id ORDER BY id DESC";
		foreach($app->read($sql) as $row) {
			return new VisitLocation($row->id);
		}
		return new VisitLocation();
	}

	function location() {
		return $this->visitLocation();
	}

	function getCreateVisit($data = array()) {
		global $app;

		$sql 						= "SELECT id, checkin FROM clinical_visits WHERE patient_id = $this->id AND admitted = 1 ";
		foreach($app->read($sql) as $row) {
			if($row->checkin) return new Visit($row->id);
		}

		$customer 					= $this->customer();
		$visit 						= new Visit();
		$visit_data 				= array (
			'visit_type'     		=> isset($data['visit_type']) ? $data['visit_type'] : 'Outpatient',
			'care_type'     		=> isset($data['care_type']) ? $data['care_type'] : 'Outpatient',
			'referral_in'     		=> isset($data['referral_in']) ? $data['referral_in'] : 0,
			'referral_in_type'     	=> isset($data['referral_in_type']) ? $data['referral_in_type'] : 'Self',
			'referral_in_code'   	=> isset($data['referral_in_code']) ? $data['referral_in_code'] : '',
			'referral_in_id'    	=> isset($data['referral_in_id']) ? $data['referral_in_id'] : 0,
			'referral_in_location'  => isset($data['referral_in_location']) ? $data['referral_in_location'] : '',
			'referral_in_notes'  	=> isset($data['referral_in_notes']) ? $data['referral_in_notes'] : '',
			'patient_id'    		=> $this->id,
			'insurance_id'  		=> isset($data['insurance']) ? $data['insurance'] : $customer->insurance_id,
			'scheme_id'     		=> isset($data['scheme']) ? $data['scheme'] : $customer->scheme_id,
			'product_id'     		=> isset($data['product_id']) ? $data['product_id'] : $customer->product_id,
			'product_code'     		=> isset($data['product_code']) ? $data['product_code'] : $customer->product_code,
			'card_no'				=> isset($data['card_no']) ? $data['card_no'] : $customer->card_no,
			'authorization_no'		=> isset($data['authorization_no']) ? $data['authorization_no'] : $customer->authorization_no,
			'supplementary'			=> isset($data['supplementary']) ? $data['supplementary'] : $customer->supplementary
		);

		$visit_id 				= $visit->createWithMinimumData($visit_data);

		return new Visit($visit_id);
	}

	function renderVisitsNavigation($options = array()) {
		global $app, $root;

		$title 		= $options['title'];
		$code 		= $options['code'];
		$name 		= $options['name'];
		$icon 		= $options['icon'];
		$section 	= $options['section'];
		$form 		= $root.'pages/clinical/patient/'.$section.'/form.php?patient_id='.$this->id;
		$list 		= $root.'pages/clinical/patient/'.$section.'/list.php?patient_id='.$this->id;

		$html = '<div class="row gx-3 my-3">
		<div class="col-sm-3 my-3 py-1">
		<h5 class="fw-bold text-muted"> <span class="'.$icon.' fa-fw"></span> '.$title.' </h5>
		</div>
		<div class="col-sm-1 my-3"> <a class="btn btn-outline-secondary w-100" role="button"> <span class="fa-solid fa-arrow-left"></span> </a> </div>
		<div class="col-sm-4 my-3">
		'.$app->renderInputSelect('visit_id', '', 'Choose visit', 'select-search default-select', 'data-style="btn-outline-secondary"', 'settings_categories', 'code', 'name', "type='TribeType'", 'Religion').'
		</div>
		<div class="col-sm-1 my-3"> <a class="btn btn-outline-secondary w-100" role="button"> <span class="fa-solid fa-arrow-right"></span> </a> </div>
		<div class="col-sm-3 my-3">
		<a data-url="'.$form.'" data-container="'.$section.'_container" class="ajax-loader btn btn-outline-secondary w-100"> <span class="fa-solid fa-plus fa-fw"></span> Add '.$name.' </a>
		</div> 
		</div>';

		$html = '<div class="mb-2" id="'.$code.'_container"></div>';

		//$html .= '<script> app.ajaxLoad(\''.$section.'_container\', \''.$list.'\'); </script>';

		return $html;
	}

	function renderTriageCard($triage_code) {
		global $root, $app;

		$visit_id 			= $this->visit()->id;
		$triage_type 		= $app->getCategoryByCode('TriageType', $triage_code);
		$triage_code 		= strtolower($triage_type->code);
		$triage_value 		= '';
		$rows 				= $app->read("SELECT value FROM clinical_triage WHERE code = '$triage_code' AND visit_id = $visit_id ");
		foreach($rows as $row) {
			$triage_value 	= $row->value;
		}

		$form_options 		= array (
			'form_id'		=> 'ajax_form_'.$triage_code,
			'form_action'	=> '../patient/triage/process.php'
		);

		$html 				= '<div class="'.$triage_type->f1.'">
		<div class="PatientTriage-flip" id="flip_triage_'.$triage_code.'">
		<div class="card front">
		<div class="card-body bg-light rounded py-4 text-center" style="height:250px">
		<h2 class="my-2 text-muted"><span class="'.$triage_type->icon.'"></span></h2>
		<h6 class="text-secondary my-2">'.$triage_type->details.'</h6>
		<div id="'.$triage_code.'_value" class="'.$triage_type->f2.' my-4" style="min-height:50px">'.$triage_value.'</div>
		<a class="btn btn-outline-primary btn-sm rounded-pill px-3 " onclick="$(\'#flip_triage_'.$triage_code.'\').flip(true)"><span class="fa-regular fa-pencil fa-fw"></span> Update</a>
		</div></div>
		
		<div class="card back shadow">
		<div class="card-body rounded py-4 text-center" style="height:250px">
		<h2 class="my-2 text-muted"><span class="'.$triage_type->icon.'"></span></h2>
		<h6 class="text-secondary my-2">'.$triage_type->details.'</h6>
		<div class="my-4">';

		$html .= $app->renderFormOpen($form_options);
		$html .= $app->renderInputHidden('patient_id', $this->id);
		$html .= $app->renderInputHidden('code', $triage_type->code);

		if($triage_type->f5 == 'Select') {
			$html 	.= '<div class="px-3 dropup">'.$app->renderInputSelectNew('value', $triage_value, '', '', '', $triage_type->f6).'</div>';
		} elseif($triage_type->f5 == 'Textarea') {
			$html 	.= '<div class="px-4">'.$app->renderInputTextarea('value', $triage_value, '', ' line-control text-center fw-bold').'</div>';
		}else {
			$html 	.= '<div class="px-4">'.$app->renderInputText('value', $triage_value, '', ' line-control text-center fw-bold').'</div>';
		}

		$html .= $app->renderFormClose($form_options);
		
		$html .= '</div>
		<a class="btn btn-outline-primary btn-sm mx-1 rounded-pill px-3" onclick="$(\'#flip_triage_'.$triage_code.'\').flip(false)"><span class="fa-solid fa-arrow-left fa-fw"></span> Back </a>
		<a class="btn btn-primary btn-sm mx-1 rounded-pill px-3" onclick="$(\'#ajax_form_'.$triage_code.'\').submit()"><span class="fa-solid fa-check fa-fw"></span> Save </a>
		</div></div>
		
		</div></div>';

		return $html;
	}

	function renderVitalCard($vital_code) {
		global $app, $chart;

		$visit_id 			= $this->visit()->id;
		$vital_type 		= $app->getCategoryByCode('VitalSign', $vital_code);
		$vital_code 		= strtolower($vital_type->code);
		$vital_value 		= '';
		$vital_icon 		= '';
		$rows 				= $app->read("SELECT value, created_datetime FROM clinical_vitals WHERE code = '$vital_code' AND visit_id = $visit_id ");
		$status 			= (object) array ('color'=>'gray', 'value'=>0, 'flag'=>'No data');
		$previous_value 	= '--';
		$vital_string 		= 'Measurement not available';

		foreach($rows as $row) {
			$vital_value 	= (float) $row->value;
			$vital_string 	= ' &middot; '.$app->getAge($row->created_datetime, '', 'SHORT').' ago';
			if(is_numeric($previous_value)) {
				if(($vital_value - $previous_value) > 1) {
					$vital_string	= 'Raising from '.$previous_value.' '.$vital_type->f6.' '.$vital_string;
					$vital_icon		= 'fa-sharp fa-arrow-up text-success';
				} elseif($previous_value - $vital_value > 1) {
					$vital_string	= 'Dropping from '.$previous_value.' '.$vital_type->f6.' '.$vital_string;
					$vital_icon		= 'fa-sharp fa-arrow-up fa-flip-vertical text-danger';
				} else {
					$vital_string	= 'Updated '.$vital_string;
				}
			} else {
				$vital_string	= 'Updated '.$vital_string;
			}
			
			$previous_value	= $vital_value;
			$status 		= $this->getVitalStatus($vital_type, $vital_value);
		}

		$color  			= $status->color;
		
		$form_options 		= array (
			'form_id'		=> 'ajax_form_'.$vital_code,
			'form_action'	=> '../patient/vitals/process.php'
		);

		$html 				= '<div class="col-lg-6 col-xl-4 mt-2 mb-3">
		<div class="PatientVitals-flip" id="flip_vital_'.$vital_code.'">
		<div class="card front">
		<div class="card-body bg-light rounded py-4 text-center" style="height:300px">

		<canvas style="position:fixed; top:0; left:auto; height:150px;" id="'.$vital_type->code.'_chart"></canvas>

		<h2 class="mt-4 mb-2 text-muted"><span class="'.$vital_type->icon.'"></span></h2>
		<h6 class="text-secondary my-2">'.$vital_type->details.'</h6>
		<div class="h4 fw-bold mt-4 mb-2" style="min-height:50px">';

		$html .= $chart->render (
			array (
				'canvas'        => $vital_type->code.'_chart',
				'type'          => 'doughnut',
				'label'         => $vital_type->name,
				'border_width'  => 0,
				'border_radius' => 1,
				'cutout'        => '90%',
				'data'          => array (
					array ('value'=>$vital_value, 'color'=>'rgba('.$app->colors->$color->rgb.', 1)'),
					array ('value'=>$vital_type->f3, 'color'=>'rgba('.$app->colors->$color->rgb.', 0.1)')
				)
			)
		);

		$vital_value		= strlen($vital_value) ? $vital_value : '--';

		$html .= '<span id="'.$vital_code.'_value">
		<small style="font-size:0.6em; position:relative; top:-1px"> <span class="'.$vital_icon.' fa-fw"></span> </small>
		'.$vital_value.'
		<small class="text-secondary" style="font-size:0.5em">'.$vital_type->f6.'</small>
		<div class="text-secondary my-2" style="font-size:0.5em">~ '.$status->flag.' ~</div>
		</span>
		</div>
		<div class="mt-3 mb-2"> &nbsp; <div class="d-lg-none my-2"> &nbsp; </div> </div>
		<div class="text-secondary mt-4 mb-2" style="font-size:0.8em">'.$vital_string.'</div>
		<a class="btn btn-outline-primary btn-sm rounded-pill px-3 d-none" onclick="$(\'#flip_vital_'.$vital_code.'\').flip(true)"><span class="fa-regular fa-pencil fa-fw"></span> Update</a>
		</div></div>
		
		<div class="card back shadow">
		<div class="card-body rounded py-4 text-center" style="height:300px">
		<h2 class="my-2 text-muted"><span class="'.$vital_type->icon.'"></span></h2>
		<h6 class="text-secondary my-2">'.$vital_type->details.'</h6>
		<div class="mt-3 mb-2">';

		$html .= $app->renderFormOpen($form_options);
		$html .= $app->renderInputHidden('patient_id', $this->id);
		$html .= $app->renderInputHidden('code', $vital_type->code);

		$html 	.= '<div class="px-4">'.$app->renderInputNumber('value', $vital_value, '--', 'line-control text-center form-control-lg fw-bold h3').'</div>';

		$html .= $app->renderFormClose($form_options);
		
		$html .= '<small class="text-secondary">'.$vital_type->f6.'</small></div>
		<a class="btn btn-outline-primary btn-sm mx-1 rounded-pill px-3" onclick="$(\'#flip_vital_'.$vital_code.'\').flip(false)"><span class="fa-solid fa-arrow-left fa-fw"></span> Back </a>
		<a class="btn btn-primary btn-sm mx-1 rounded-pill px-3" onclick="$(\'#ajax_form_'.$vital_code.'\').submit()"><span class="fa-solid fa-check fa-fw"></span> Save </a>
		</div></div>
		
		</div></div>';

		return $html;
	}

	function renderLifestyleCard($lifestyle_code) {
		global $root, $app;

		$visit_id 			= $this->visit()->id;
		$lifestyle_type 		= $app->getCategoryByCode('PatientLifestyle', $lifestyle_code);
		$lifestyle_code 		= strtolower($lifestyle_type->code);
		$lifestyle_value 		= '';
		$lifestyle_notes 		= '';
		$rows 					= $app->read("SELECT value, notes FROM clinical_lifestyle WHERE code = '$lifestyle_code' AND visit_id = $visit_id ");
		foreach($rows as $row) {
			$lifestyle_value 	= $row->value;
			$lifestyle_notes 	= $row->notes;
		}

		$form_options 		= array (
			'form_id'		=> 'ajax_form_'.$lifestyle_code,
			'form_action'	=> '../patient/lifestyle/process.php'
		);

		$html 				= '<div class="'.$lifestyle_type->f1.'">
		<div class="PatientLifestyle-flip" id="flip_lifestyle_'.$lifestyle_code.'">
		<div class="card front">
		<div class="card-body bg-light rounded py-4 text-center" style="height:300px">
		<h2 class="my-2 text-muted"><span class="'.$lifestyle_type->icon.'"></span></h2>
		<h6 class="text-secondary my-2">'.$lifestyle_type->details.'</h6>
		<div id="'.$lifestyle_code.'_value" class="'.$lifestyle_type->f2.' my-4" style="min-height:80px">'.$lifestyle_value.' <div class="mt-3 text-muted fw-normal" style="font-size:0.8em">'.$lifestyle_notes.'</div></div>
		<a class="btn btn-outline-primary btn-sm rounded-pill px-3" onclick="$(\'#flip_lifestyle_'.$lifestyle_code.'\').flip(true)"><span class="fa-regular fa-pencil fa-fw"></span> Update</a>
		</div></div>
		
		<div class="card back shadow">
		<div class="card-body rounded py-4 text-center" style="height:300px">
		<h2 class="my-2 text-muted"><span class="'.$lifestyle_type->icon.'"></span></h2>
		<h6 class="text-secondary my-2">'.$lifestyle_type->details.'</h6>
		<div class="my-4">';

		$html .= $app->renderFormOpen($form_options);
		$html .= $app->renderInputHidden('patient_id', $this->id);
		$html .= $app->renderInputHidden('code', $lifestyle_type->code);

		$html 	.= $app->renderInputSelect('value', $lifestyle_value, 'Choose if applicable', 'line-control', '', array('Yes','No','Unknown'));
		$html 	.= '<div class="mb-1">&nbsp;</div>';
		$html 	.= $app->renderInputText('notes', $lifestyle_notes, 'Enter details', ' line-control');

		$html .= $app->renderFormClose($form_options);
		
		$html .= '</div>
		<a class="btn btn-outline-primary btn-sm mx-1 rounded-pill px-3" onclick="$(\'#flip_lifestyle_'.$lifestyle_code.'\').flip(false)"><span class="fa-solid fa-arrow-left fa-fw"></span> Back </a>
		<a class="btn btn-primary btn-sm mx-1 rounded-pill px-3" onclick="$(\'#ajax_form_'.$lifestyle_code.'\').submit()"><span class="fa-solid fa-check fa-fw"></span> Save </a>
		</div></div>
		
		</div></div>';

		return $html;
	}


	function renderEvaluationCard($evaluation_code) {
		global $root, $app;

		$visit_id 			= $this->visit()->id;
		$evaluation_type 		= $app->getCategoryByCode('PatientEvaluation', $evaluation_code);
		$evaluation_code 		= strtolower($evaluation_type->code);
		$evaluation_value 		= '';
		$evaluation_notes 		= '';
		$rows 					= $app->read("SELECT value, notes FROM clinical_evaluation WHERE code = '$evaluation_code' AND visit_id = $visit_id ");
		foreach($rows as $row) {
			$evaluation_value 	= $row->value;
			$evaluation_notes 	= $row->notes;
		}

		$form_options 		= array (
			'form_id'		=> 'ajax_form_'.$evaluation_code,
			'form_action'	=> '../patient/evaluation/process.php'
		);

		$html 				= '<div class="'.$evaluation_type->f1.'">
		<div class="PatientEvaluation-flip" id="flip_evaluation_'.$evaluation_code.'">
		<div class="card front">
		<div class="card-body bg-light rounded py-4 text-center" style="height:300px">
		<h2 class="my-2 text-muted"><span class="'.$evaluation_type->icon.'"></span></h2>
		<h6 class="text-secondary my-2">'.$evaluation_type->details.'</h6>
		<div id="'.$evaluation_code.'_value" class="'.$evaluation_type->f2.' my-4" style="min-height:80px">'.$evaluation_value.' <div class="mt-3 text-muted fw-normal" style="font-size:0.8em">'.$evaluation_notes.'</div></div>
		<a class="btn btn-outline-primary btn-sm rounded-pill px-3" onclick="$(\'#flip_evaluation_'.$evaluation_code.'\').flip(true)"><span class="fa-regular fa-pencil fa-fw"></span> Update</a>
		</div></div>
		
		<div class="card back shadow">
		<div class="card-body rounded py-4 text-center" style="height:300px">
		<h2 class="my-2 text-muted"><span class="'.$evaluation_type->icon.'"></span></h2>
		<h6 class="text-secondary my-2">'.$evaluation_type->details.'</h6>
		<div class="my-4">';

		$html .= $app->renderFormOpen($form_options);
		$html .= $app->renderInputHidden('patient_id', $this->id);
		$html .= $app->renderInputHidden('code', $evaluation_type->code);

		$html 	.= $app->renderInputSelect('value', $evaluation_value, 'Choose status', 'line-control', '', array('Normal','Abdormal','Not checked'));
		$html 	.= '<div class="mb-1">&nbsp;</div>';
		$html 	.= $app->renderInputText('notes', $evaluation_notes, 'Enter details', ' line-control');

		$html .= $app->renderFormClose($form_options);
		
		$html .= '</div>
		<a class="btn btn-outline-primary btn-sm mx-1 rounded-pill px-3" onclick="$(\'#flip_evaluation_'.$evaluation_code.'\').flip(false)"><span class="fa-solid fa-arrow-left fa-fw"></span> Back </a>
		<a class="btn btn-primary btn-sm mx-1 rounded-pill px-3" onclick="$(\'#ajax_form_'.$evaluation_code.'\').submit()"><span class="fa-solid fa-check fa-fw"></span> Save </a>
		</div></div>
		
		</div></div>';

		return $html;
	}


	function getVitalStatus($vital_type, $vital_value) {
		global $app;
		$status = array ('flag'=>'No data', 'color'=>'gray');
		if($vital_value > 0) {
			if($vital_value < $vital_type->f1) { 
				$status['flag'] = 'Very low'; $status['color'] = 'red'; 
			} elseif($vital_value < $vital_type->f2) { 
				$status['flag'] = 'Low'; $status['color'] = 'orange'; 
			} elseif($vital_value > $vital_type->f5) { 
				$status['flag'] = 'Very high'; $status['color'] = 'red'; 
			} elseif($vital_value > $vital_type->f4) { 
				$status['flag'] = 'High'; $status['color'] = 'orange'; 
			} else {
				$status['flag'] = 'Normal'; $status['color'] = 'success'; 
			}
		} else {
			$status['flag'] = 'No data'; $status['color'] = 'gray'; 
		}

		if($vital_type->name == 'Mood') {
			$category 			= $app->getCategoryByColumn('MoodType', 'priority', $vital_value);
			$status['message'] 	= $category->name;
			$status['icon']		= $category->icon;
			$status['val']		= $vital_value;
		} else {
			$status['val']		= $vital_value;
			$status['icon']		= $vital_type->icon;
		}
		return (object) $status;
	}

	function scheduleAdmission($data) {
		global $user, $now;

		$schedule_id = $this->update (
			array (
				'visit_id'				=> isset($data['visit_id']) ? $data['visit_id'] : $this->visit()->id,
				'visit_location_id'		=> isset($data['visit_location_id']) ? $data['visit_location_id'] : $this->visitLocation()->id,
				'admission_status'		=> 'Scheduled',
				'admission_reason'		=> isset($data['admission_reason']) ? $data['admission_reason'] : '',
				'admission_notes'		=> isset($data['admission_notes']) ? $data['admission_notes'] : '',
				'admission_location_id'	=> $data['admission_location_id'],
				'admission_user_id'		=> isset($data['admission_user_id']) ? $data['admission_user_id'] : $user->id,
				'admission_datetime'	=> isset($data['admission_datetime']) ? $data['admission_datetime'] : $now,
			)
		);

		return $schedule_id;
	}

	function cancelAdmissionSchedule() {
		global $user, $now;

		$schedule_id = $this->update (
			array (
				'admission_status'		=> 'Checkin',
				'admission_reason'		=> '',
				'admission_notes'		=> '',
				'admission_location_id'	=> 0,
				'admission_user_id'		=> 0,
				'admission_datetime'	=> 0,
			)
		);

		return $schedule_id;
	}
}

?>