<?php 

class Request extends Crud {

	function __construct($id=0) {	
		parent::__construct($id);
        if(!$id) $this->urgency = 'Routine';
	}

	function createWithMinimumData($data) {
		global $app, $user_id, $now;

        $items                          = isset($data['items']) ? htmlspecialchars($data['items'], ENT_QUOTES) : '[]';

		$create_data                    = $this->addCommonFields ( 
            array (
                'type'                  => $this->type,
                'type_id'               => 0,
                'subtype'               => $this->type,
                'subtype_id'            => 0,
                'patient_id'            => $data['patient_id'],
                'visit_id'              => $data['visit_id'],
                'visit_location_id'     => $data['visit_location_id'],
                'store_id'              => $data['store_id'],
                'insurance_id'          => $data['insurance_id'],
                'scheme_id'             => $data['scheme_id'],
                'bill_id'               => $data['bill_id'],
                'invoice_id'            => isset($data['invoice_id']) ? $data['invoice_id'] : 0,
                'section'               => isset($data['section']) ? $data['section'] : '',
                'section_id'            => isset($data['section_id']) ? $data['section_id'] : 0,
                'urgency'               => isset($data['urgency']) ? $data['urgency'] : 'Routine',
                'urgency_code'          => isset($data['urgency_code']) ? $data['urgency_code'] : 3,
                'progress'              => isset($data['progress']) ? (strlen($data['progress'] > 1) ? $data['progress'] : 'Requested') : 'Requested',
                'progress_code'         => isset($data['progress_code']) ? ($data['progress_code'] > 0 ? $data['progress_code'] : 1) : 1,
                'progress_notes'        => isset($data['progress_notes']) ? $data['progress_notes'] : '',
                'progress_user_id'      => isset($data['progress_user_id']) ? $data['progress_user_id'] : $user_id,
                'progress_datetime'     => isset($data['progress_datetime']) ? $data['progress_datetime'] : $now,
                'attachments'           => isset($data['attachments']) ? $data['attachments'] : '[]',
                'items'                 => $items,
                'code'                  => $data['code'],
                'name'                  => isset($data['name']) ? $app->formatName($data['name']) : '',
                'birthdate'             => isset($data['birthdate']) ? ($data['birthdate'] > '1900-01-01' ? $data['birthdate'] : date('Y-m-d')) : date('Y-m-d'),
                'sex'                   => isset($data['sex']) ? $data['sex'] : '',
                'phone'                 => isset($data['phone']) ? $data['phone'] : '',
                'email'                 => isset($data['email']) ? $data['email'] : '',
                'details'               => isset($data['details']) ? $data['details'] : '',
                'flags'                 => isset($data['flags']) ? $data['flags'] : '',
                'tags'                  => isset($data['tags']) ? $data['tags'] : '',
                'requested'             => 1,
                'requested_notes'       => isset($data['requested_notes']) ? $data['requested_notes'] : '',
            )
        );

        $request_id                     = $this->create($create_data);
      
        if(isset($data['items'])) {
            foreach(json_decode($data['items']) as $item) {
                $item                   = (array) $item;
                $request_item_class     = $this->type.'Item';
                $request_item           = new $request_item_class($this->type, $request_id);
                $request_item->createWithMinimumData($item);
            }
        }

        $this->id                       = $request_id;
        $this->updateInlineItems();

		return $request_id;
	}

    function renderCreateConfirmation($submodule, $post) {
        global $app, $user;
 
        $user->checkSubmoduleAccess($submodule, 'create');

		$bill_id        		= $this->createBill($submodule, $post);

        $customer       		= new Customer($post->trans_customer_id);
        $items          		= $app->uncleanText($post->trans_items);
        $person         		= $customer->person();
        $scheme         		= new Scheme($post->trans_scheme_id);
		$pricelist      		= new Pricelist($scheme->pricelist_id);
        $insurance      		= new Insurance($post->trans_insurance_id);
        $visit          		= new Visit($post->visit_id);
		$folio          		= $visit->folio();
        $subtype                = $this->type == 'Prescription' ? 'Pharmacy' : $this->type;  
		
        $create_data            = array (
			'subtype'           => $subtype,
			'parent'            => '',
			'store_id'          => $post->trans_store_id,
			
			'insurance_id'      => $insurance->id,
			'scheme_id'        	=> $scheme->id,
			'bill_id'           => $bill_id,

			'patient_id'        => $post->patient_id,
			'personnel'         => 'Patient',
			'personnel_id'      => $post->patient_id,
			'code'              => $customer->code,
			'name'              => $person->name,
			'birthdate'         => $person->birthdate,
			'sex'               => $person->sex,
			'phone'             => $person->phone,
			'email'             => $person->email,

			'pricelist'         => $pricelist->code,
			'pricelist_id'      => $pricelist->id,
			'insurance'         => $insurance->code,
			'insurance_id'      => $insurance->id,
			'scheme'            => $scheme->code,
			'scheme_id'         => $scheme->id,

			'folio_id'          => $folio->id,
			'folio_no'          => $folio->folio_no,

			'visit_id'          => $visit->id,
			'visit_location_id' => $post->visit_location_id,

			'requested_notes'   => $post->trans_requested_notes,
			'items'             => $items,
			'cost'              => $post->trans_cost,
			'subtotal'          => $post->trans_subtotal
		);

        $request_id         	= $this->createWithMinimumData($create_data);

        if($request_id) {
			$bill 				= new Bill($bill_id);
			$bill->set('subtype_id', $request_id);

            // Log request create event
            $user->timeline (
                array (
                    'action'			=> 'Create',
                    'subject'			=> $this->type,
                    'subject_id'		=> $request_id,
                    'subtype'			=> '',
                    'subtype_id'		=> 0,
                    'personnel'			=> 'Patient',
                    'personnel_id'		=> $post->patient_id,
                    'name'	            => $customer->person()->name,
                    'details'			=> 'Added '.strtolower($this->title),
                    'visit_id'          => $visit->id,
                    'visit_location_id' => $post->visit_location_id,
                )
            );

            return $app->renderAlertSuccess($this->type.' request created successfully');
        }
    }

    function customer() {
		return $this->patient()->customer();
	}

    function item($request_item_id) {
        $item_class = $this->type.'Item';
		return new $item_class($this->type, $this->id, $request_item_id);
	}

    function getItemByItemId($item_id) {
        global $app;
        $request_item_id    = $app->getColumnValue($this->items_table, 'id', 'item_id', $item_id, "parent_id = $this->id");
        return $this->item($request_item_id);
    }

    function visit() {
        return new Visit($this->visit_id);
    }

    function visitLocation() {
        return new VisitLocation($this->visit_location_id);
    }

    function bill() {
		return new Bill($this->bill_id);
	}

    function invoice() {
		return new Bill($this->bill_id);
	}

    function patient() {
		return new Patient($this->patient_id);
	}

	function store() {
		return new Store($this->store_id);
	}

    function renderLinkDelete() {
        return '<a title="Cancel request" class="btn bs-tooltip btn-outline-danger ajax-link rounded-pill me-2" data-target="Patient'.$this->type.'_container" data-link="../patient/'.strtolower($this->type).'/form.php?patient_id='.$this->patient_id.'&request_id='.$this->id.'&action=delete"><span class="fa-solid fa-trash fa-fw"></span><span class="d-none d-xl-inline"></span></a>';
    }

    function renderLinkBill() {
        global $root;
        return '<a title="Confirm bill" class="remote-modal bs-tooltip btn btn-outline-primary rounded-pill py-2 me-2" data-bs-toggle="modal" data-bs-target="#modal_lg" data-bs-backdrop="static" data-bs-keyboard="false" tabindex="-1" href="'.$root.'pages/billing/bills/form.php?action=bill&id='.$this->bill_id.'" aria-label="Bill"> <span class="fa-solid fa-memo-circle-check"></span> </a>';
    }

    function renderLinkReport($options=array()) {
        $modal = isset($options['modal']) ? $options['modal'] : 'modal_lg';
        return '<a title="Enter report" class="remote-modal bs-tooltip btn btn-outline-primary rounded-pill py-2 me-2" data-bs-toggle="modal" data-bs-target="#'.$modal.'" data-bs-backdrop="static" data-bs-keyboard="false" tabindex="-1" href="form.php?action=report&id='.$this->id.'" aria-label="Report"><span class="fa-solid fa-pen-to-square"></span></a>';
    }

    function renderLinkEdit($options=array()) {
        $modal = isset($options['modal']) ? $options['modal'] : 'modal_lg'; 
        return '<a title="Edit report" class="remote-modal bs-tooltip btn btn-outline-primary rounded-pill py-2 me-2" data-bs-toggle="modal" data-bs-target="#'.$modal.'" data-bs-backdrop="static" data-bs-keyboard="false" tabindex="-1" href="form.php?action=edit&id='.$this->id.'" aria-label="Edit"><span class="fa-solid fa-pen-to-square"></span></a>';
    }

    function renderLinkAuthorize($options=array()) {
        $modal = isset($options['modal']) ? $options['modal'] : 'modal_lg';
        $id    = isset($options['id']) ? $options['id'] : $this->id;

        return '<a title="Authorize" class="remote-modal bs-tooltip btn btn-outline-primary rounded-pill py-2 me-2" data-bs-toggle="modal" data-bs-target="#'.$modal.'" data-bs-backdrop="static" data-bs-keyboard="false" tabindex="-1" href="form.php?action=authorize&id='.$id.'" aria-label="Authorize"><span class="fa-solid fa-lock-open"></span></a>';
    }

    function renderButtonUnpaid($options=array()) {
        $modal = isset($options['modal']) ? $options['modal'] : 'modal_lg';
        return '<a title="Unpaid" class="remote-modal bs-tooltip btn btn-outline-danger rounded-pill px-3" data-bs-toggle="modal" data-bs-target="#'.$modal.'" data-bs-backdrop="static" data-bs-keyboard="false" tabindex="-1" href="form.php?action=open&id='.$this->id.'" aria-label="Unpaid"> <span class="fa-solid fa-lock"></span> Unpaid </a>';
    }

    function renderLinkPrint($options=array()) {
        $modal = isset($options['modal']) ? $options['modal'] : 'modal_lg';
        return '<a title="Print report" target="blank" class="bs-tooltip btn btn-outline-primary rounded-pill py-2 me-2" href="print.php?action=print&id='.$this->id.'" aria-label="Print"><span class="fa-solid fa-print"></span></a>';
    }


    function renderButtonUrgency() {
        $urgency    = $this->urgency();
        return '<a class="btn rounded-pill px-3 btn-outline-'.$urgency->color.' "> '.$this->urgency.'</a>';
    }

    function items() {
        global  $app;
        $sql    = "SELECT * FROM $this->items_table WHERE parent_id = $this->id ";
        $rows   = array();
        foreach($app->read($sql) as $row) {
            $rows[$row->item_id] = $row;
        }
        return $rows;
    }

    function requestItems() {
        global  $app;
        $sql            = "SELECT id, item_id FROM $this->items_table WHERE parent_id = $this->id ";
        $request_items  = array();
        $request_type   = $this->type.'Item';

        foreach($app->read($sql) as $row) {
            $request_items[$row->id] = new $request_type($this->type, $this->id, $row->id);
        }
        return $request_items;
    }

    function renderRequestHeader($customer) {

		$scheme_id 		= $customer->scheme_id;
		$scheme 		= new Scheme($scheme_id);
		$insurance_id 	= $scheme->insurance_id;

        if($this->type == 'Prescription' || $this->type == 'Consumable') {
            $store_type = 'Pharmacy';
        } if($this->type == 'Procedure') {
            $store_type = 'Location';
        } else {
            $store_type = $this->type;
        }

        if($this->type == 'Procedure') {
            $store_tag = 'Procedure';
        } else {
            $store_tag = $store_type;
        }
		
		$html = $this->id ? $this->renderTrackingButton() : '';
		$html .= '<div class="row">';
		$html .= '<div class="col-12 col-lg-6 mb-4">'.$this->renderInsuranceControl($insurance_id).'</div>';
		$html .= '<div class="col-12 col-lg-6 mb-4">'.$this->renderInsuranceSchemeControl($scheme_id).'</div>';
		$html .= '<div class="col-12 col-lg-6 mb-4">'.$this->renderStoreControl($store_type, 'create-sales', $store_tag).'</div>';
		$html .='</div>';
		return $html;
	}

    function renderProgressBlock($progress_code) {
        global $app;
        $code               = $progress_code == 'Paid' ? 'Payment' : $progress_code;
        $progress           = $app->getCategoryByCode('TransactionProgress', $progress_code);
        $notes              = strtolower($code.'_notes');
        
        if($progress_code == 'Created') $progress   = $app->getCategoryByCode('TransactionProgress', 'Requested');
        if($notes == 'created_notes') $notes   = 'requested_notes';
        $user               = strtolower($code.'_user_id');
        $datetime           = strtolower($code.'_datetime');
        $progress_user      = new User($this->$user);
        $progress_employee  = $progress_user->personnel();
        $progress_datetime  = $this->$datetime;

        $html               = '<img src="'.$progress_employee->picture().'" class="rounded-circle mx-auto d-block bg-light mt-2 mb-1" style="width:40%; max-width:100px; border:5px solid white;" />';
        $html               .= '<div style="display:inline-block; position:relative; top:-35px; right:-32px;"> <span class="bg-primary text-white" style="padding:3px 4px; border:4px solid white; border-radius:50%;"> <span class="'.$progress->icon.' fa-fw"></span> </span></div>';
        $html               .= '<h4 class="fw-bold" style="margin-top:-15px">'.$code.'</h4>';
        $html               .= '<div>'.$progress_user->person()->name.'</div>';
        $html               .= '<div>'.$app->datetime($progress_datetime).'</div>';

        if(strlen($this->$notes)) {
            $html           .= '<div class="text-muted my-3"><i>'.$this->$notes.'</i></div>';
        }
        
        return              $html;
    }

    function renderVerticalProgress() {
        global $app;
        $progress   = array();
        $glue       = '<div class="my-3"><span class="fa-solid fa-down-long"></span></div>';
        $sql        = "SELECT code FROM settings_categories WHERE type = 'TransactionProgress' ORDER BY priority";
        foreach($app->read($sql) as $row) {
            $code   = $row->code == 'Paid' ? 'Payment' : $row->code;

            $progress_user = strtolower($code).'_user_id';
            if($this->$progress_user) {
                $progress[] = $this->renderProgressBlock($row->code);
            }
        }
        return implode($glue, $progress);
    }

    function renderButtonProgress($options=array()) {
        global $app;

        $modal          = isset($options['modal']) ? $options['modal'] : 'modal_lg';
        $id             = isset($options['id']) ? $options['id'] : $this->id;

        $progress       = $this->progress;
        $progress_obj   = $this->getProgressObjFromProgressName($progress);
        $progress_code  = $progress_obj->code;
        $items          = $this->items();
        $count          = 0;
        $total_count    = is_array($items) ? count($items) : 0;

        if(is_array($items)) {
            foreach($items as $item) {
                if(!isset($item->progress)) $item->progress = 'Requested';
                $item_progress      = $this->getProgressObjFromProgressName($item->progress);
                if($item_progress->code > $progress_code) {
                    $count          = 1;
                    $progress       = $item->progress;
                    $temp_progress  = $this->getProgressObjFromProgressName($progress);
                    $progress_code  = $temp_progress->code;
                } elseif($item_progress->code == $progress_code) {
                    $count++;
                }
            }
        }

        $progress_obj   = $this->getProgressObjFromProgressName($progress);
        if($progress_obj->code) {
            $html       = '<a class="btn btn-outline-'.$progress_obj->color.' w-100 rounded-pill remote-modal text-decoration-none fw-normal" data-bs-toggle="modal" data-bs-target="#'.$modal.'" href="form.php?action=open&id='.$id.'">'.$progress.' '.$count.'/'.$total_count.'</a>';
        } else {
            $html       = '<a class="btn btn-outline-secondary w-100 rounded-pill remote-modal text-decoration-none fw-normal" data-bs-toggle="modal" data-bs-target="#'.$modal.'" href="form.php?action=open&id='.$id.'">'.$progress.' '.$count.'/'.$total_count.'</a>';
        }
        return          $html;
    }

    function getProgressObjFromProgressName($progress_name) {
        $progress_array = array ('code'=>1, 'color'=>'secondary');
        if($progress_name == 'Requested')   $progress_array = array('code'=>1, 'color'=>'secondary');
        if($progress_name == 'Billed')      $progress_array = array('code'=>2, 'color'=>'secondary');
        if($progress_name == 'Paid')        $progress_array = array('code'=>3, 'color'=>'primary');
        if($progress_name == 'Collected')   $progress_array = array('code'=>4, 'color'=>'primary');
        if($progress_name == 'Delivered')   $progress_array = array('code'=>5, 'color'=>'primary');
        if($progress_name == 'Received')    $progress_array = array('code'=>6, 'color'=>'primary');
        if($progress_name == 'Rejected')    $progress_array = array('code'=>7, 'color'=>'danger');
        if($progress_name == 'Processed')   $progress_array = array('code'=>8, 'color'=>'success');
        if($progress_name == 'Reported')    $progress_array = array('code'=>9, 'color'=>'success');
        if($progress_name == 'Dispensed')   $progress_array = array('code'=>10, 'color'=>'primary');
        if($progress_name == 'Edited')      $progress_array = array('code'=>10, 'color'=>'primary');
        if($progress_name == 'Authorized')  $progress_array = array('code'=>11, 'color'=>'success');
        if($progress_name == 'Approved')    $progress_array = array('code'=>12, 'color'=>'success');
        if($progress_name == 'Printed')     $progress_array = array('code'=>13, 'color'=>'success');
        return (object) $progress_array;
    }

    function hasItemProgress($progress) {
        $items              = json_decode($this->items);
        if(!$items) $items  = $this->items();

        foreach($items as $item) {
            if(!isset($item->progress)) $item->progress = 'Requested';
            if($item->progress == $progress) return true;
        }
        return false;
    }

    function hasPaidItem() {
        $items              = json_decode($this->items);
        if(!$items || 1) $items  = $this->items();
        foreach($items as $item) {
            if($item->payment && ($item->progress == 'Billed' || $item->progress == 'Paid')) return true;
        }
        return false;
    }

    function maxItemProgressCode() {
        $progress_code = 0;
        foreach(json_decode($this->items) as $item) {
            if($item->progress_code > $progress_code) $progress_code = $item->progress_code;
        }
        return $progress_code;
    }

    function getItems() {
        global  $app;
        $sql    = "SELECT * FROM $this->items_table WHERE parent_id = $this->id ";
        return $app->read($sql);
    }

    function mergeItems($first_items, $second_items) {
        global $app;

        $return_items   = array();

        foreach(array_keys($second_items) as $second_item_id) {
            if(isset($first_items[$second_item_id])) {
                // Check UOM compatibility
                if($first_items[$second_item_id]->pack == $second_items[$second_item_id]->pack) {
                    $first_items[$second_item_id]->id                   = $first_items[$second_item_id]->item_id;
                    $first_items[$second_item_id]->qty                  += $second_items[$second_item_id]->qty;
                    $first_items[$second_item_id]->quantity             += $second_items[$second_item_id]->quantity;
                    $first_items[$second_item_id]->requested_quantity   += $second_items[$second_item_id]->requested_quantity;
                    $first_items[$second_item_id]->cost                 = $first_items[$second_item_id]->unit_cost * $first_items[$second_item_id]->quantity;
                    $first_items[$second_item_id]->price                = $first_items[$second_item_id]->unit_price * $first_items[$second_item_id]->quantity;
                } else {
                    die($app->renderAlertError('Cannot add '.$second_items[$second_item_id]->quantity.' '.$second_items[$second_item_id]->pack.' to '.$first_items[$second_item_id]->quantity.' '.$first_items[$second_item_id]->pack, $second_items[$second_item_id]->name));
                }
            } else {
                $second_items[$second_item_id]->id              = $second_items[$second_item_id]->item_id;
                $second_items[$second_item_id]->parent     = $this->type;
                $second_items[$second_item_id]->parent_id  = $this->id;
                $first_items[$second_item_id]                   = $second_items[$second_item_id];
            }
        }
        foreach($first_items as $first_item) {
            $return_items[] = $first_item;
        }
        return $return_items;
    }

    function resetItems($items) {
        $this->clearItems();
        $transaction_item   = new TransactionItem($this->type, $this->id);
        foreach($items as $item) {  
            $transaction_item->createWithMinimumData((array) $item);
        }
        $this->set('items', json_encode($items));
    }

    function clearItems() {
        global $db;
        $sql = "DELETE FROM $this->items_table WHERE parent_id = $this->id ";
        return $db->update($sql);
    }

    function updateInlineItems() {
        $items = array();

        foreach($this->items() as $item) {  
            $items[] = $item;
        }
        $this->set('items', json_encode($items));
    }

    function urgency() {
        global $app;
        return $app->getCategoryByCode('UrgencyType', $this->urgency);
    }

    function progress() {
        global $app;
        return $app->getCategoryByCode('TransactionProgress', $this->progress);
    }

    function changeProgress($progress, $quantity_column='requested_quantity', $notes='') {
        global $app, $now, $user_id;

        $item_class             = $this->type.'Item';
        $column                 = strtolower($progress);
        $progress_obj           = $app->getCategoryByCode('TransactionProgress', $progress);
        
        if($column == 'paid')   {
            $column     = 'payment';
            $progress   = 'payment';
        }

        $request_data             = array (
            'progress'          => $progress_obj->code,
            'progress_code'     => $progress_obj->priority,
            'progress_notes'    => $notes,
            'progress_user_id'  => $user_id,
            'progress_datetime' => $now
        );

        $updated = $this->update($request_data);

        if($updated) {
            foreach($this->items() as $item) {
                $request_item     = new $item_class($this->type, $this->id, $item->id);       
                $request_item->update($request_data);
            }
        }

        $this->updateInlineItems();
        
        return $updated;
    }



    function user($action) {
        $user_type = $action.'_user_id';
        return new User($this->$user_type);
    }

    function renderPDF($title='Request') {

        global $root, $app, $client;

        $html               = '';
        $patient            = $this->patient();
        $person             = $patient->person();
        $heading            = ''.$patient->code.' - '.$person->name.' '.ucwords($person->age()).' '.$person->sex.'';
        $default_header     = '<table style="width:100%"><tr>
        <td style="width:8%"><img src="'.$root.CLIENT_PATH.'images/logo.png" /> </td>
        <td style="width:92%; text-align:left"><h3><span style="font-size:1.5em">'.$client->client_name.'</span> <br />'.$client->client_address.'</h3></td>
        </tr></table>';
        $document_header    = $app->getSystemSettings('client_document_header', $default_header);
        $document_header    = $app->uncleanText($document_header);
        $document_header    = str_replace('__ROOT__', $root, $document_header);
        $document_header    = str_replace('__CLIENT_PATH__', CLIENT_PATH, $document_header);
        
        $html               .= $document_header;
        $html               .= '<h2 style="text-align:center">'.ucwords($this->title).' Report</h2>';

        $html               .= '<table style="padding:5px 8px; background-color:rgb(200,200,200); border:1px solid rgb(90,90,90); margin:0">
        <tr><td style="width:50%"><b>PATIENT DETAILS</b></td> <td style="width:50%; "><b>'.strtoupper($this->type).' DETAILS</b></td> </tr>
        </table><table style="padding:3px 8px; border:1px solid rgb(90,90,90); margin:0">
		<tr><td width="15%">File no:</td><td width="35%"><b>'.$patient->code.'</b></td> <td width="16%">Request no:</td> <td width="34%"><b>'.$app->formatCode($this->id, 6).'</b></td></tr>
		<tr><td width="15%">Fullname:</td><td width="35%"><b>'.$person->name.'</b></td> <td width="16%">Urgency:</td> <td width="34%"><b>'.$this->urgency.'</b></td></tr>
		<tr><td width="15%">Date of birth:</td><td width="35%"><b>'.$person->birthdate.' ('.strtolower($person->age()).' old) </b></td> <td width="16%">Request from :</td><td width="34%"><b>'.$this->visitLocation()->location()->location()->name.'</b></td></tr>
		<tr><td width="15%">Sex:</td><td width="35%"><b><b>'.$person->sex.'</b></b></td> <td width="16%">Destination:</td><td width="34%"><b>'.$patient->visitLocation()->location()->location()->name.'</b></td></tr>
        <tr><td width="15%">Address:</td><td width="35%"><b>'.$person->address.'</b></td> <td width="16%">Requested by:</td><td width="34%"><b>'.$this->user('created')->person()->name.'</b></td></tr>
		<tr><td width="15%">Phone no:</td><td width="35%"><b>'.$person->phone.'</b></td> <td width="16%">Date and time</td><td width="34%"><b>'.$app->datetime($this->created_datetime).'</b></td></tr>
		</table><br /> <br />';

        $count = 0;
        foreach($this->requestItems() as $request_item) {
            // if(!$count++) continue;
            if($request_item->authorized) {
                $html           .= $request_item->renderReportPDF();
            }
        }

        $app->renderPDF($html, $this->type, $heading);
    }

    function renderTrackingButton() {
        global $app;
        $progress       = $app->getCategoryByCode('TransactionProgress', $this->progress);
        $code           = $this->progress == 'Paid' ? 'Payment' : $this->progress;
        $user           = strtolower($code.'_user_id');
        $user           = new User($this->$user);
        $datetime       = strtolower($code.'_datetime');
        $datetime       = $this->$datetime;
        $date           = date('Y-m-d', strtotime($datetime));
        if($date == date('Y-m-d')) $date = 'Today';
        if($date == date('Y-m-d', strtotime('Yesterday'))) $date = 'Yesterday';
        
        $title_label    = '<b>'.$this->type.' '.$this->progress.'</b>';
        $title_label    .= '<div class="px-4"><small class="text-muted">By '.$app->shortname($user->person()->name, 15).' '.$date.' at '.date('H:i', strtotime($datetime)).'</small></div>';

        $button_label   = '<span class="fa-solid fa-magnifying-glass fa-fw"></span> Track '.$this->type.' <span class="fa-solid fa-caret-down"></span>';
        $summary        = '<div class="row">
        <div class="col-lg-6 py-2"><span class="fa-solid fa-check-circle fa-fw"></span> '.$title_label.'</div>
        <div class="col-lg-6 py-3"><a onclick="transaction.loadTrackingDetails(\''.$this->type.'\', '.$this->id.')" class="btn btn-primary w-100 fw-semibold">'.$button_label.'</a></div>
        </div><div id="trans_tracking_container"></div>';

        $html = '<div class="alert alert-primary py-0 mb-5" style="margin-top:-15px">'.$summary.'</div>';
        return $html;
    }

    function renderInsuranceControl($insurance_id=0) {
        global $app;
        if(!$insurance_id) $insurance_id = $this->getInsuranceByCode('CASH')->id;

        $html   = '<div class="trans-container"> <h5 class="trans-subtitle"><span class="fa-solid fa-tags fa-fw"></span> Sponsor: </h5>';
		$html   .= $app->renderInputSelect('trans_insurance_id', $insurance_id, '', 'line-control select-search onchange-reload-children '.$this->readonly().'', 'data-order=priority', 'billing_insurances', 'id', 'name');
		$html   .= '</div>';
        return $html;
    }

    function renderInsuranceSchemeControl($scheme_id='') {
        global $app;
        if(!$scheme_id) $scheme_id = $this->getSchemeByCode('CASH')->id;

        $scheme         = new Scheme($scheme_id);
        $insurance_id   = $scheme->insurance_id; 
        $html           = '<div class="trans-container"> <h5 class="trans-subtitle"><span class="fa-solid fa-tags fa-fw"></span> Scheme: </h5>';
        $html           .= $app->renderInputSelect('trans_scheme_id', $scheme_id, '', 'line-control select-search '.$this->readonly().'', 'data-parent="trans_insurance_id" data-source="html/transactions/schemes_select.php"', 'billing_insurance_schemes', 'id', 'name', "insurance_id = $insurance_id OR 1");
		$html           .= '</div>';
        return $html;
    }

    function readonly() {
        return $this->id ? 'readonly' : '';
    }

    function getInsuranceByCode($code) {
        global $app;
        $insurance_id = $app->getColumnValue('billing_insurances', 'id', 'code', $code);
        return new Insurance($insurance_id);
    }

    function getSchemeByCode($code) {
        global $app;
        $scheme_id = $app->getColumnValue('billing_insurance_schemes', 'id', 'code', $code);
        return new Scheme($scheme_id);
    }

    function renderStoreControl($label='Store', $access='', $tags='') {
        global $app, $user;

        if($this->id) $access = '';
        $stores = $user->listStoresWithAccess($access, $tags);

        $html = '<div class="trans-container mb-4"> <h5 class="trans-subtitle"> <span class="fa-solid fa-store fa-fw"></span> '.$label.': </h5>';
		$html .= $app->renderInputSelect('trans_store_id', $this->store_id, '', 'line-control select-search '.$this->readonly().'', 'onchange="transaction.initializeTransaction()" data-parent="trans_subtype" data-source="html/transactions/stores_select.php"', $stores);
		$html .= '</div>';
        return $html;
    }

    function createBill($submodule, $post) {
        global $app, $user;

        $user->checkSubmoduleAccess($submodule, 'create');

        $bill           = new Bill();
        $customer       = new Customer($post->trans_customer_id);
        $items          = $app->uncleanText($post->trans_items);
        $person         = $customer->person();
        $scheme         = new Scheme($post->trans_scheme_id);
		$pricelist      = new Pricelist($scheme->pricelist_id);
        $insurance      = new Insurance($post->trans_insurance_id);
        $visit          = new Visit($post->visit_id);
		$folio          = $visit->folio();
        $subtype        = $this->type == 'Prescription' ? 'Pharmacy' : $this->type;              
		
		$items_obj 		= json_decode($items);
		if(!$items_obj) die($app->renderAlertError('Please add at least one item'));

		$bill->checkStockAvailability($post);

        $create_data            = array (
			'type'              => 'Bill',
			'subtype'           => $subtype,
			'store_id'          => $post->trans_store_id,

			'personnel'         => 'Customer',
			'personnel_id'      => $customer->id,
			'code'              => $customer->code,
			'name'              => $person->name,
			'birthdate'         => $person->birthdate,
			'sex'               => $person->sex,
			'phone'             => $person->phone,
			'email'             => $person->email,

			'pricelist'         => $pricelist->code,
			'pricelist_id'      => $pricelist->id,
			'insurance'         => $insurance->code,
			'insurance_id'      => $insurance->id,
			'scheme'            => $scheme->code,
			'scheme_id'         => $scheme->id,

			'folio_id'          => $folio->id,
			'folio_no'          => $folio->folio_no,

			'visit_id'          => $visit->id,
			'visit_location_id' => $post->visit_location_id,

			'requested_notes'   => $post->trans_requested_notes,
			'items'             => $items,
			'cost'              => $post->trans_cost,
			'subtotal'          => $post->trans_subtotal,
			'progeress'         => 'Requested',
			'progress_code'     => 1
		);

        $bill_id                = $bill->createWithMinimumData($create_data);
        if($bill_id) {
            $_SESSION['bill_id'] = $bill_id;
            echo $app->renderAlertSuccess($this->type.' bill created successfully', 'Success', 'modal_lg', 1);
            return $bill_id;
        }
    }

    function renderListHeader($options=array()) {
        global $app, $r;

        $patient_id = isset($options['patient_id']) ? $options['patient_id'] : 0;
        $action     = isset($options['action']) ? $options['action'] : 'readonly';

        $html       = '';

        $patient_id     = $_GET['patient_id'];
        $patient        = new Patient($patient_id);
        $visit          = $patient->visit();
        if($this->type == 'Prescription' || $this->type == 'Pharmacy') {
            $diagnoses      = $app->read("SELECT id FROM clinical_diagnoses WHERE visit_id = $visit->id AND patient_id = $patient->id AND certainty = 'Confirmed'");
        } elseif($this->type == 'Consumable') {
            $diagnoses      = array(1);
        } else {
            $diagnoses      = $app->read("SELECT id FROM clinical_diagnoses WHERE visit_id = $visit->id AND patient_id = $patient->id");
        }
        
        if($action == 'create') {
            if(count($diagnoses)) {
                $html = '<div class="row">
                <div class="col-sm-8 mb-4"><h4>'.$this->title.'</h4></div>
                <div class="col-sm-4 mb-4 text-end"> <a class="btn w-100 btn-primary fw-semibold ajax-link" data-target="Patient'.$this->type.'_container" data-link="../patient/'.strtolower($this->type).'/form.php?patient_id='.$patient_id.'" style="margin-top:-2px"> <span class="fa-solid fa-plus fa-fw"></span> Add '.$this->subtitle.'</a> </div>
                </div>';
            } elseif($this->type == 'Prescription' || $this->type == 'Pharmacy') {
                $html = $app->renderAlertError('Diagnosis is required before adding new medication', 'Diagnosis required');
            } else {
                $html = $app->renderAlertError('Diagnosis is required before adding new '.strtolower($this->type).' request', 'Diagnosis required');
            }
        }

        if($this->type == 'Radiology') {
            $html .= '<a class="mb-2 btn btn-outline-primary w-100 btn-lg remote-modal bs-tooltip" onclick="app.showModal(\'modal_fs\', \''.$r.'pages/clinical/radiology/dicom_form.php?action=read&patient_id='.$patient_id.'\')"> <span class="fa-regular fa-images-user fa-fw"></span> Open Dicom Image Viewer (New) </a>';
        }

        $html .= '<div class="request-container mt-3" id="Patient'.$this->type.'Container">';

        return $html;
    }

    function renderListBody() {
        global $app, $user, $root;

        $doctor         = new User($this->created_user_id);
        $person         = $doctor->person();
        $datetime       = $app->datetime($this->created_datetime);
        $visit_location = new VisitLocation($this->visit_location_id);
        $icon           = $this->icon;
        $color          = 'primary';
        $section        = $this->store()->location()->name;

        $html           = '<div class="card mb-4 bg-'.$color.'" style="--bs-bg-opacity: .05;">
        <div class="card-body p-4">
        <div class="row">
        <div class="col-sm-8">
        <h5 class="mb-4"><span class="'.$icon.' fa-fw"></span> <span class="fw-bold">'.$section.'</span></h5>
        </div><div class="col-sm-4 text-end">';

        if(strtotime($this->created_datetime) > strtotime(' - '.$app->getClinicalEditTime()) && $this->created_user_id == $user->id) {
            $html .= '<div class="row gx-3">
            <div class="col-6 mb-3"> </div>
            <div class="col-6 mb-3"><a class="btn btn-outline-danger w-100 ajax-link" data-target="Patient'.$this->type.'_container" data-link="../patient/'.strtolower($this->type).'/form.php?patient_id='.$this->patient_id.'&request_id='.$this->id.'&action=delete"> <span class="fa-solid fa-trash fa-fw"></span> <span class="d-none d-xl-inline">Delete</span> </a> </div>
            </div>';
        } else {
            $html .= '<h5>'.$app->getAge($this->created_datetime,'','SHORT').' ago</h5>';
        }

        $html .= '</div></div>';

        foreach($this->requestItems() as $request_item) {
            $html       .= $request_item->renderDataCard();
        }
        
        $html           .= '</div>
        <div class="px-4 pb-4">
        <div class="row text-secondary">
        <div class="col-xl-8 pt-2">
        <!-- <img src="" class="rounded-circle d-inline-block float-start mt-1 me-3" style="width:38px" /> -->
        <span class="fw-semibold">'.$person->name.'</span><br />'.$visit_location->location()->location()->name.'</div>
        <div class="col-xl-4 pt-2 text-end"><span class="fw-semibold">'.$datetime.'</span><br /> '.$app->getAge($this->created_datetime,'','SHORT').' ago </div>
        </div>
        </div>
        </div>';

        return $html;
    }

    function renderListFooter() {
        $html = '</div>';

        return $html;
    }

    function renderListData($options=array()) {
        global $app;
 
        $request_class  = $this->type;
        $patient_id     = isset($options['patient_id']) ? $options['patient_id'] : 0;
        $sql            = isset($options['sql']) ? $options['sql'] : "SELECT id FROM $this->table WHERE patient_id = $patient_id AND status = 1 ORDER BY id DESC ";
        $rows           = isset($options['rows']) ? $options['rows'] : $app->read($sql);

        $html           = $this->renderListHeader($options);

        foreach($rows as $row) {
            $obj        = new $request_class($row->id);
            $html       .=  $obj->renderListBody();
        }

        $html           .= $this->renderListFooter();

        return $html;
    }

    function renderRequestForm($options=array()) {
        global $app, $get;

        $patient            = new Patient($options['patient_id']);
        $visit_location     = $patient->visitLocation();
        $person             = $patient->person();
        $customer           = $patient->customer();
        $bill               = $this->bill();

        $form_options       = array (
            'form_id'       => 'ajax_form_'.strtolower($this->type),
            'form_action'   => '../patient/'.strtolower($this->type).'/process.php'
        );

        $html = $app->renderFormOpen($form_options);
        $html .= $app->renderInputHidden('patient_id', $patient->id);
        $html .= $app->renderInputHidden('visit_id', $visit_location->visit_id);
        $html .= $app->renderInputHidden('visit_location_id', $visit_location->id);
        $html .= '<div class="rounded p-4 pt-5 pb-4 mb-4 bg-primary" style="--bs-bg-opacity: .05;">';

        if($get->action == 'delete') {
            $html .= '<h3>Are you sure?</h3>';
            $html .= '<h5 class="my-4">'.$this->type.' request will be deleted</h5>';
            $html .= $app->renderInputHidden('action', 'delete');
            $html .= $app->renderInputHidden('request_id', $this->id);
        } else {
            $html .= $app->renderInputHidden('action', 'create');
            $html .= $app->renderInputHidden('trans_subtype', $this->type);
            $html .= $app->renderInputHidden('trans_customer_id', $customer->id);
            $html .= $app->renderInputHidden('trans_customer_name', $person->name);
            $html .= $app->renderInputHidden('customer_keyword', $person->name);
            $html .= $app->renderInputHidden('trans_customer_birthdate', $person->birthdate);
            $html .= $app->renderInputHidden('trans_customer_sex', $person->sex);
            $html .= $app->renderInputHidden('trans_customer_phone', $person->phone);
            $html .= $this->renderRequestHeader($customer);

            $html .= $bill->renderTransactionItemsEditable('Billing cart'); 

            $html .= $this->renderItemsControls();

            $html .= $bill->renderNotesControl('Requested');
        }

        $html .= '</div>';

        $html .= $app->renderFormClose($form_options);

        $html .= '<div class="row w-100 m-0 mt-3 px-0">
        <div class="col-12 col-lg-4"></div>
        <div class="col-12 col-lg-8 px-0">
        <div class="row pb-4 gx-3">
        <div class="col-6">
        <a class="btn btn-outline-primary w-100 ajax-link" data-target="Patient'.$this->type.'_container" data-link="../patient/'.strtolower($this->type).'/list.php?patient_id='.$patient->id.'&action=create"> <span class="fa fa-arrow-left fa-fw"></span> Cancel </a>
        </div><div class="col-6">
        <button id="ajax_form_'.strtolower($this->type).'_submit_button" type="button" class="btn btn-primary w-100"> <span class="fa fa-check fa-fw"></span> Submit </button>
        </div></div>
        </div></div><hr />';

        return $html;
    }

    function  renderInsuranceSchemesJson() {
        global $app;
        $sql			= "SELECT * FROM billing_insurance_schemes ";
        $schemes 		= array();
        foreach($app->read($sql) as $row) {
            $schemes[$row->id]	= $row;
        }
        $html           = '<script> transaction.insurance_schemes = '.$app->array2json($schemes).' </script>';
        return $html;
    }

    function renderItemsControls() {
		global $app;

        $html = $app->renderInputHidden('trans_items');

        $html .= $this->renderInsuranceSchemesJson();

		$html .= '<div class="row gx-3 mb-4">
        <div class="col-12 mt-3">
        <div class="">
        <h5 class="my-3"> <span class="fa-solid fa-cart-arrow-down fa-fw"></span>&nbsp; Add to cart: </h5>
        <div class="row gx-3">
        <div class="my-2 mb-3 col-12 col-lg-4">
        '.$app->renderInputSelect('type', $this->type, 'Filter categories', 'line-controlled select-search default-all', 'data-style="btn-outline-secondary"', array()).'
        </div><div id="trans_item_container" class="my-2 mb-3 col-12 col-lg-8">
        '.$app->renderInputSelect('trans_item_id', '', 'Choose item', 'line-controlled select-search default-none', 'data-style="btn-outline-secondary"', array()).'
        </div><div class="my-2 mb-3 col-12 col-lg-4">
        '.$app->renderInputNumber('trans_item_quantity', '', 'Quantity', 'line-controlled border border-1 border-secondary').'
        </div><div class="my-2 mb-3 col-12 col-lg-4" id="trans_item_uom_container">
        '.$app->renderInputSelect('trans_item_uom', '', 'Units', 'line-controlled select-search', 'data-style="btn-outline-secondary"', array()).'
        </div><div class="my-2 mb-3 col-12 col-lg-4">
        '.$app->renderInputMoney('trans_item_unit_price', '', 'Unit price', 'line-controlled text-end border border-1 border-secondary readonly').'
        </div><div class="my-2 mb-3 col-12 col-lg-8">
        '.$app->renderInputText('trans_item_details', '', 'Item specifications', 'line-controlled border border-1 border-secondary').'
        </div><div class="my-2 mb-3 col-12 col-lg-4">
        <a id="trans_item_add_button" class="btn btn-outline-primary w-100" onclick="transaction.addItemToCart()"> <span class="fa-solid fa-check fa-fw"></span> Add to cart </a>
        </div></div></div></div></div>';

		return $html;
	}
}

?>