<?php 

class Visit extends Crud {
	function __construct($id=0) {		
		$this->table        = 'clinical_visits';
		$this->primary_key  = 'id';
		parent::__construct($id);
	}

	function createWithMinimumData($data) {
		global $app, $user_id, $now;

		$patient				= new Patient($data['patient_id']);
		$customer				= $patient->customer();
		$person					= $customer->person();
		$scheme					= new Scheme($data['scheme_id']);
		$insurance				= new Insurance($data['insurance_id']);
		$pricelist				= $scheme->pricelist();

		$sql 					= "SELECT id FROM clinical_visits WHERE patient_id = $patient->id LIMIT 1";
		$checkin_type 			= 'New'; 
		foreach($app->read($sql) as $row) {
			$checkin_type 		= 'Revisit';
		}

		$create_data			= array (
			'type'					=> $checkin_type,
			'subtype'				=> '',
			'visit_type'			=> $data['visit_type'],
			'care_type'				=> $data['care_type'],
			'referral_in'			=> isset($data['referral_in']) ? $data['referral_in'] : 0,
			'referral_in_type'		=> isset($data['referral_in_type']) ? $data['referral_in_type'] : 'Self',
			'referral_in_id'		=> isset($data['referral_in_id']) ? $data['referral_in_id'] : 0,
			'referral_in_code'		=> isset($data['referral_in_code']) ? $data['referral_in_code'] : '',
			'referral_in_location'	=> isset($data['referral_in_location']) ? $data['referral_in_location'] : '',
			'referral_in_notes'		=> isset($data['referral_in_notes']) ? $data['referral_in_notes'] : '',
			'code'					=> $patient->code,
			'name'					=> $person->name,
			'birthdate'				=> $person->birthdate,
			'sex'					=> $person->sex,
			'phone'					=> $person->phone,
			'residence'				=> isset($data['residence']) ? $data['residence'] : '',
			'patient_id'			=> $data['patient_id'],
			'insurance_id'			=> $data['insurance_id'],
			'scheme_id'				=> $data['scheme_id'],
			'product_id'			=> isset($data['product_id']) ? $data['product_id'] : $customer->product_id,
			'product_code'			=> isset($data['product_code']) ? $data['product_code'] : $customer->product_code,
			'folio_id'				=> isset($data['folio_id']) ? $data['folio_id'] : 0,
			'folio_no'				=> isset($data['folio_no']) ? $data['folio_no'] : 0,
			'card_no'				=> isset($data['card_no']) ? $data['card_no'] : $customer->card_no,
			'supplementary'			=> isset($data['supplementary']) ? $data['supplementary'] : '',
			'authorization_no'		=> isset($data['authorization_no']) ? $data['authorization_no'] : '0',
			'checkin'				=> 1,
			'checkin_type'			=> $checkin_type,
			'checkin_notes'			=> '',
			'checkin_diagnoses'		=> isset($data['checkin_diagnoses']) ? $data['checkin_diagnoses'] : '[]',
			'checkin_user_id'		=> $user_id,
			'checkin_datetime'		=> $now,
			'checkout'				=> 0,
			'checkout_type'			=> '',
			'checkout_template'		=> '[]',
			'checkout_notes'		=> '',
			'checkout_diagnoses'	=> '[]',
			'checkout_user_id'		=> 0,
			'checkout_datetime'		=> 0,
			'referral_out'			=> 0,
			'referral_out_type'		=> '',
			'referral_out_id'		=> 0,
			'referral_out_code'		=> '',
			'referral_out_location'	=> '',
			'referral_out_notes'	=> '',
			'ticket_id'				=> 0,
			'ticket_no'				=> '',
			'bill_id'				=> isset($data['bill_id']) ? $data['bill_id'] : 0,
			'invoice_id'			=> isset($data['invoice_id']) ? $data['invoice_id'] : 0,
			'admitted'				=> 1
		);
		
		$visit_id 					= $this->create($this->addCommonFields($create_data));
		$this->id 					= $visit_id;
		$this->read();

		if($visit_id) {
			$folio 					= new Folio();
			$folio_no 				= '';

			$create_data            = array (
				'type'              => 'Folio',
				'subtype'           => 'Outpatient',
				'store_id'          => 0,
	
				'personnel'         => 'Customer',
				'personnel_id'      => $customer->id,
				'code'              => $customer->code,
				'name'              => $person->name,
				'birthdate'         => $person->birthdate,
				'sex'               => $person->sex,
				'phone'             => $person->phone,
				'email'             => $person->email,
	
				'pricelist'         => $pricelist->code,
				'pricelist_id'      => $pricelist->id,
				'insurance'         => $insurance->code,
				'insurance_id'      => $insurance->id,
				'scheme'            => $scheme->code,
				'scheme_id'         => $scheme->id,
	
				'folio_id'          => $visit_id,
				'folio_no'          => $folio_no,
	
				'visit_id'          => $visit_id,
				'visit_location_id' => 0,
	
				'requested_notes'   => '',
				'items'             => '[]',
				'cost'              => 0,
				'subtotal'          => 0,
				'progeress'         => 'Requested',
				'progress_code'     => 1
			);
	
			$folio_id         		= $folio->createWithMinimumData($create_data);

			$this->update ( 
				array (
					'folio_id'		=> $folio_id,
					'folio_no'		=> $folio_no
				)
			);
		}
		return $visit_id;
	}

	function patient() {
		return new Patient($this->patient_id);
	}

	function freeBeds() {
		global $db;

		// Free beds
		$bed_sql 	= "UPDATE clinical_ward_beds SET occupant = '', occupant_id = 0 WHERE occupant_id NOT IN(SELECT patient_id FROM clinical_visit_locations WHERE admitted = 1)";
		$bed_query	= $db->query($bed_sql);
	}

	function dischargePatient($options = array()) {
		global $user, $now;

		$last_visit_location 		= new VisitLocation($this->getLastVisitLocationId());
		$last_visit_location->dischargePatient($options);

		$checkout_type 				= isset($options['checkout_type']) ? $options['checkout_type'] : 'Auto';
		$checkout_template 			= isset($options['checkout_template']) ? $options['checkout_template'] : '[]';
		$checkout_notes 			= isset($options['checkout_notes']) ? $options['checkout_notes'] : '';
		$checkout_diagnoses 		= isset($options['checkout_diagnoses']) ? $options['checkout_diagnoses'] : array();
		$referral_out 				= isset($options['referral_out']) ? $options['referral_out'] : 0;
		$referral_out 				= isset($options['referral_out']) ? $options['referral_out'] : 0;
		$referral_out_type 			= isset($options['referral_out_type']) ? $options['referral_out_type'] : '';
		$referral_out_code 			= isset($options['referral_out_code']) ? $options['referral_out_code'] : '';
		$referral_out_location 		= isset($options['referral_out_location']) ? $options['referral_out_location'] : '';
		$referral_out_id 			= isset($options['referral_out_id']) ? $options['referral_out_id'] : 0;

		$discharged = $this->update (
			array (
				'checkout'				=> 1,
				'checkout_type'			=> $checkout_type,
				'checkout_template'		=> $checkout_template,
				'checkout_notes'		=> $checkout_notes,
				'checkout_diagnoses'	=> $checkout_diagnoses,
				'checkout_user_id'		=> $user->id,
				'checkout_datetime'		=> $now,
				'referral_out'			=> $referral_out,
				'referral_out_type'		=> $referral_out_type,
				'referral_out_id'		=> $referral_out_id,
				'referral_out_code'		=> $referral_out_code,
				'referral_out_location'	=> $referral_out_location,
				'admitted'				=> 0
			)
		);

		$this->freeBeds();

		return $discharged;
	}

	function createVisitLocation($data = array()) {

		global $app;

		// Clear all open visits
		$patient 				= new Patient($this->patient_id);
		$visit_location			= $patient->visitLocation();
		$visit_location->dischargePatient(array('checkout_notes'=>'Automatically discharged for readmission to another location'));
		$sql 					= "SELECT id FROM clinical_visit_locations WHERE patient_id = $this->patient_id AND admitted = 1";
		foreach($app->read($sql) as $row) {
			$visit_location		= new VisitLocation($row->id);
			$visit_location->dischargePatient(array('checkout_notes'=>'Automatically discharged for readmission to another location'));
		}

		$data['patient_id'] 	= $this->patient_id;
		$data['folio_id'] 		= $this->folio_id;
		$data['visit_id'] 		= $this->id;
		$data['care_type'] 		= $this->care_type;
		$data['checkin_type'] 	= $this->checkin_type;

		$visit_location_id 		= $visit_location->createWithMinimumData($data);

		if($data['care_type'] == 'Inpatient') {
			$this->set('care_type', 'Inpatient');
			$this->folio()->set('subtype', 'Inpatient');
		}

		return $visit_location_id;
	}

	function folio() {
		global $app;

		$sql = "SELECT id FROM billing_folios WHERE visit_id = $this->id ";
		foreach($app->read($sql) as $row) {
			return new Folio($row->id);
		}

		if($this->checkin) {
			$patient 				= $this->patient();
			$customer				= $patient->customer();
			$person					= $customer->person();
			$insurance				= $customer->insurance();
			$scheme					= $customer->scheme();
			$location 				= $patient->location();
			
			$folio_data             = array (
				'personnel'         => 'Customer',
				'personnel_id'      => $customer->id,
				'code'              => $customer->code,
				'name'              => $person->name,
				'birthdate'         => $person->birthdate,
				'sex'               => $person->sex,
				'phone'             => $person->phone,
				'email'             => $person->email,
				'insurance'         => $insurance->code,
				'insurance_id'      => $insurance->id,
				'scheme'            => $scheme->code,
				'scheme_id'         => $scheme->id,
				'card_no'         	=> $this->card_no,
				'authorization_no'  => $this->authorization_no,
				'folio_id'          => 0,
				'folio_no'          => 0,
				'visit_id'          => $this->id,
				'visit_location_id' => $location->id
			);

			$folio 					= new Folio();
			$folio_id 				= $folio->createWithMinimumData($folio_data);
			return new Folio($folio_id);
		} else {
			return new Folio();
		}
	}

	function scheme() {
		return new Scheme($this->scheme_id);
	}

	function insurance() {
		return new Insurance($this->insurance_id);
	}

	function getVisitLocationIds() {
		global $app;

		$sql = "SELECT id FROM clinical_visit_locations WHERE visit_id = $this->id ORDER BY id";
		return $app->read($sql);
		return array();
	}

	function getFirstVisitLocationId() {
		foreach($this->getVisitLocationIds() as $row) {
			return $row->id;
		}
		return 0;
	}

	function firstVisitLocation() {
		return new VisitLocation($this->getFirstVisitLocationId());
	}

	function getLastVisitLocationId() {
		$visit_location_id = 0;
		foreach($this->getVisitLocationIds() as $row) {
			$visit_location_id = $row->id;
		}
		return $visit_location_id;
	}

	function lastVisitLocation() {
		return new VisitLocation($this->getLastVisitLocationId());
	}

	public function getServices($service, $condition="1", $items_condition="1") {
		global $app;

		if($service == 'Procedure') {
			$table_name 	= 'clinical_procedures';
			$details_table	= 'clinical_procedure_items';
		} elseif($service == 'Prescription') {
			$table_name 	= 'clinical_prescriptions';
			$details_table	= 'clinical_prescription_items';
		} else {
			$table_name 	= 'clinical_'.strtolower($service);
			$details_table	= 'clinical_'.strtolower($service).'_items';
		}
		
		$services		= array();

		$request_str	= "SELECT $table_name.id, 
		personnel_persons.name create_staff_name,
		personnel_employees.designation create_staff_designation,
		$table_name.created_datetime,
		IF (clinical_visit_locations.clinic_id > 1, (SELECT name FROM location_locations JOIN clinical_clinics  WHERE clinic_id = clinical_visit_locations.clinic_id LIMIT 1), 
		(SELECT name FROM location_locations JOIN clinical_wards ON clinical_wards.location_id = location_locations.id WHERE ward_id = clinical_visit_locations.ward_id LIMIT 1)) AS location

		FROM $table_name 
		JOIN clinical_visit_locations ON clinical_visit_locations.id = $table_name.visit_location_id
		JOIN personnel_users ON personnel_users.id = $table_name.created_user_id
		JOIN personnel_persons ON personnel_persons.id = personnel_users.person_id
		JOIN personnel_employees ON personnel_employees.person_id = personnel_users.person_id
		
		WHERE $table_name.visit_id = $this->id AND $condition ";

		$requests		= $app->read($request_str);

		foreach($requests as $request) {
			$request_id	= $request->id;
			$details 	= array();

			if($service == 'Prescription' || $service == 'Pharmacy') {
				$details_str= "SELECT id, name, uom, dose, times, days
				FROM $details_table WHERE parent_id = $request_id AND $items_condition ";
			} else {
				$details_str= "SELECT id, name, report
				FROM $details_table WHERE parent_id = $request_id AND $items_condition ";
			}

			$details_datas = $app->read($details_str);

			foreach($details_datas as $details_data) {
				if($service == 'Prescription' || $service == 'Pharmacy') {
					$details[]	= (object) array(
						'id'			=> $details_data->id,
						'name'			=> $details_data->name,
						'report'		=> $details_data->dose.' '.$details_data->uom.', '.$details_data->times.' times a day for '.$details_data->days.' days'
					);
				} else {
					$details[]	= (object) array(
						'id'			=> $details_data->id,
						'name'			=> $details_data->name,
						'report'		=> $details_data->report
					);
				}
			}

			$services[] 	= (object) array (
				'id'						=> $request->id,
				'created_datetime'			=> $request->created_datetime,
				'create_staff_name'			=> $request->create_staff_name,
				'create_staff_designation'	=> $request->create_staff_designation,
				'details'					=> $details,
				'location'					=> $request->location
			);
		}

		return $services;
	}
	
	function getDiagnoses() {
		global $app;
		$diagnoses	= array();
		$sql		= "SELECT certainty, code, name FROM clinical_diagnoses WHERE visit_id = $this->id AND status = 1";
		foreach($app->read($sql) as $row) {
			$diagnoses[$row->code]['certainty'] = $row->certainty;
			$diagnoses[$row->code]['code'] 		= $row->code;
			$diagnoses[$row->code]['name'] 		= $row->name;
		}
		return $diagnoses;
	}

	function getProvisionalDiagnosesCodes() {
		global $app;
		$diagnoses	= array();
		$sql		= "SELECT code FROM clinical_diagnoses WHERE visit_id = $this->id AND certainty = 'Provisional' AND status = 1";
		foreach($app->read($sql) as $row) {
			$diagnoses[] = $row->code;
		}
		return $diagnoses;
	}

	function getConfirmedlDiagnosesCodes() {
		global $app;
		$diagnoses	= array();
		$sql		= "SELECT code FROM clinical_diagnoses WHERE visit_id = $this->id AND certainty = 'Confirmed' AND status = 1";
		foreach($app->read($sql) as $row) {
			$diagnoses[] = $row->code;
		}
		return $diagnoses;
	}

	function getClinicalNotes() {
		global $app;
		$sql		= "SELECT * FROM clinical_notes WHERE visit_id = $this->id ";
		return $app->read($sql);
	}

	function getProvisionalDiagnoses() {
		global $app;
		$sql		= "SELECT * FROM clinical_diagnoses WHERE visit_id = $this->id AND certainty = 'Provisional' AND status = 1";
		return $app->read($sql);
	}

	function getConfirmedDiagnoses() {
		global $app;
		$sql		= "SELECT * FROM clinical_diagnoses WHERE visit_id = $this->id AND certainty = 'Confirmed' AND status = 1";
		return $app->read($sql);
	}

	function signature() {
		global $app, $root;
		$attachment = $app->getAttachmentByCode('Visit', $this->id, 'PatientSignature');
		if($attachment->exists()) return $attachment->renderOpenPath();
		return $root.'assets/images/default/signature.png';
	}

	function getBills($options=array()) {
		global $app;
		$scheme 				= new Scheme($this->scheme_id); 

		$visit_filter 			= "";
		$join_filter			= "";
		if(isset($options['visit_type'])) {
			if($options['visit_type'] == 'Inpatient') {
				$visit_filter	= " AND clinical_visit_locations.ward_id > 0 ";
				$join_filter	= " JOIN clinical_visit_locations ON clinical_visit_locations.id = billing_invoices.visit_location_id ";
			} elseif($options['visit_type'] == 'Outpatient') {
				$visit_filter	= " AND clinical_visit_locations.clinic_id > 0 ";
				$join_filter	= " JOIN clinical_visit_locations ON clinical_visit_locations.id = billing_invoices.visit_location_id ";
			}
		}
		
		$bills_str      = "SELECT billing_invoices.id, 
		billing_invoices.subtype, 
		billing_invoice_items.item_id, 
		billing_invoice_items.name, 
		billing_invoice_items.quantity quantity, 
		billing_invoice_items.price billed,
		billing_insurances.paymode,
		(billing_invoice_items.paid + IFNULL(billing_promise_items.paid, 0)) paid, 
		billing_invoice_items.exemption exemption, 
		(IFNULL(billing_promise_items.promise, 0) - IFNULL(billing_promise_items.paid, 0)) promise,
		(billing_invoice_items.price - billing_invoice_items.paid - IFNULL(billing_promise_items.paid, 0) - billing_invoice_items.exemption) payable
		FROM billing_invoice_items 
		JOIN billing_invoices ON billing_invoices.id = billing_invoice_items.parent_id 
		$join_filter
		JOIN billing_insurances ON billing_insurances.id = billing_invoices.insurance_id
		LEFT JOIN billing_promise_items ON billing_promise_items.type_id = billing_invoices.type_id AND billing_promise_items.item_id = billing_invoice_items.item_id
		WHERE billing_invoices.visit_id = $this->id 
		AND billing_invoices.type = 'Bill'
		AND billing_invoices.payment = 1
		$visit_filter ";

		$bills      	= array();
		$bills_query	= $app->read($bills_str);
		$subtotal 		= 0;

		foreach($bills_query as $bill) {
			$subtype            = $bill->subtype;
			if($subtype == 'Prescription') $subtype = 'Pharmacy';
			if($subtype == 'Procedure') $subtype = 'Procedures';
			$subtotal	+= $bill->billed;

			if($bill->paymode != 'CASH') {
				$bill->paid 	= 0;
				$bill->payable 	= $bill->billed;
			}
			if(isset($bills[$subtype][$bill->item_id])) {
				$bills[$subtype][$bill->item_id]->quantity  += $bill->quantity;
				$bills[$subtype][$bill->item_id]->billed    += $bill->billed;
				$bills[$subtype][$bill->item_id]->paid      += $bill->paid;
				$bills[$subtype][$bill->item_id]->exemption += $bill->exemption;
				$bills[$subtype][$bill->item_id]->promise   += $bill->promise;
				$bills[$subtype][$bill->item_id]->payable   += $bill->payable;
			} else {
				$bills[$subtype][$bill->item_id]            = $bill;
			}
		}

		if($this->product_id > 0) {

			$item_id	= $this->product_id;
			$item 		= new Item($item_id);
			$pricelist	= $this->scheme()->pricelist();
			$amount 	= $item->getPrice($pricelist->id);

			foreach($bills_query as $bill) {
				$subtype            = $bill->subtype;
				if($subtype == 'Prescription') $subtype = 'Pharmacy';
				if($subtype == 'Procedure') $subtype = 'Procedures';
				if($bill->paymode != 'CASH') {
					$bill->paid 	= 0;
					$bill->payable 	= $bill->billed;
				}

				if($bill->item_id == $item_id) {
					$balance 		= $amount - $subtotal + $bill->billed;
					$bill->billed 	= $balance;
					$bill->paid 	= $balance;
					$bill->payable 	= 0;
				}
			}
		}

		return $bills;
	}
}

?>