<?php 

class VisitLocation extends Crud {
	function __construct($id=0) {		
		$this->table        = 'clinical_visit_locations';
		$this->primary_key  = 'id';
		parent::__construct($id);
	}

	function visit() {
		return new Visit($this->visit_id);
	}

	function dischargePatient($options = array()) {
		global $user, $now, $db;

		$checkout_type 				= isset($options['checkout_type']) ? $options['checkout_type'] : 'Auto';
		$checkout_template 			= isset($options['checkout_template']) ? $options['checkout_template'] : '[]';
		$checkout_notes 			= isset($options['checkout_notes']) ? $options['checkout_notes'] : '';
		$checkout_diagnoses 		= isset($options['checkout_diagnoses']) ? $options['checkout_diagnoses'] : json_encode(array());
		$referral_out 				= isset($options['referral_out']) ? $options['referral_out'] : 0;
		$referral_out_type 			= isset($options['referral_out_type']) ? $options['referral_out_type'] : '';
		$referral_out_code 			= isset($options['referral_out_code']) ? $options['referral_out_code'] : '';
		$referral_out_location 		= isset($options['referral_out_location']) ? $options['referral_out_location'] : '';
		$referral_out_id 			= isset($options['referral_out_id']) ? $options['referral_out_id'] : 0;

		$discharged = $this->update (
			array (
				'checkout'				=> 1,
				'checkout_type'			=> $checkout_type,
				'checkout_template'		=> $checkout_template,
				'checkout_notes'		=> $checkout_notes,
				'checkout_diagnoses'	=> $checkout_diagnoses,
				'checkout_user_id'		=> $user->id,
				'checkout_datetime'		=> $now,
				'referral_out'			=> $referral_out,
				'referral_out_type'		=> $referral_out_type,
				'referral_out_id'		=> $referral_out_id,
				'referral_out_code'		=> $referral_out_code,
				'referral_out_location'	=> $referral_out_location,
				'admitted'				=> 0
			)
		);

		$this->visit()->freeBeds();

		return $discharged;
	}

	function cancelVisit($options = array()) {
		global $app;

		$this->visit()->dischargePatient($options);
		$this->dischargePatient($options);

		return true;
	}

	function createWithMinimumData($data) {
		global $app, $user_id, $now;

		$referral_in_type 			= isset($data['referral_in_type']) ? $data['referral_in_type'] : 'Self';
		$referral_in				= $referral_in_type == 'Self' ? 0 : 1;

		$patient_id 				= $data['patient_id'];
		$care_type 					= $data['care_type'];
		$location_id 				= $data['location_id'];
		$sql 						= "SELECT id FROM clinical_visit_locations WHERE patient_id = $patient_id AND care_type = '$care_type' AND location_id = $location_id LIMIT 1";
		
		$checkin_type 				= 'New'; 
		foreach($app->read($sql) as $row) {
			$checkin_type 			= 'Revisit';
		}

		$create_data				= $this->addCommonFields (
			array (
				'type'					=> $checkin_type,
				'type_id'				=> 0,
				'subtype'				=> '',
				'subtype_id'			=> 0,
				'patient_id'	    	=> $data['patient_id'],
				'folio_id'				=> $data['folio_id'],
				'visit_id'				=> $data['visit_id'],
				'visit_type'			=> $data['visit_type'],
				'care_type'				=> $data['care_type'],
				'referral_in'			=> $referral_in,
				'referral_in_type'		=> $referral_in_type,
				'referral_in_id'		=> isset($data['referral_in_id']) ? $data['referral_in_id'] : 0,
				'referral_in_code'		=> isset($data['referral_in_code']) ? $data['referral_in_code'] : '',
				'referral_in_location'	=> isset($data['referral_in_location']) ? $data['referral_in_location'] : '',
				'referral_in_notes'		=> isset($data['referral_in_notes']) ? $data['referral_in_notes'] : '',
				'authorization_no'		=> isset($data['authorization_no']) ? $data['authorization_no'] : '0',
				'checkin'				=> 1,
				'checkin_type'			=> $checkin_type,
				'checkin_notes'			=> '',
				'checkin_diagnoses'		=> isset($data['checkin_diagnoses']) ? $data['checkin_diagnoses'] : '[]',
				'checkin_user_id'		=> $user_id,
				'checkin_datetime'		=> $now,
				'location'		    	=> $data['location'],
				'location_id'	    	=> $data['location_id'],
				'department_id'			=> $data['department_id'],
				'clinic_id'		    	=> isset($data['clinic_id']) ? $data['clinic_id'] : 0,
				'ward_id'		    	=> isset($data['ward_id']) ? $data['ward_id'] : 0,
				'room'		        	=> '',
				'room_id'	        	=> 0,
				'bed'		        	=> '',
				'bed_id'	        	=> 0,
				'doctor_id'	        	=> isset($data['doctor_id']) ? $data['doctor_id'] : 0,
				'queue_id'	        	=> isset($data['queue_id']) ? $data['queue_id'] : 0,
				'ticket_no'	        	=> isset($data['ticket_no']) ? $data['ticket_no'] : 0,
				'bill_id'	    		=> isset($data['bill_id']) ? $data['bill_id'] : 0,
				'invoice_id'	    	=> isset($data['invoice_id']) ? $data['invoice_id'] : 0,
				'checkout'				=> 0,
				'checkout_type'			=> '',
				'checkout_template'		=> '[]',
				'checkout_notes'		=> '',
				'checkout_diagnoses'	=> '[]',
				'checkout_user_id'		=> 0,
				'checkout_datetime'		=> 0,
				'referral_out'			=> 0,
				'referral_out_type'		=> '',
				'referral_out_id'		=> 0,
				'referral_out_code'		=> '',
				'referral_out_location'	=> '',
				'referral_out_notes'	=> '',
				'admitted'				=> 1
			)
		);

		$visit_location_id 				= $this->create($create_data);
		return $visit_location_id;
	}

	function createConsultationBill($consultation_item_id) {
		$store_id 				= $this->location()->store_id;
		$bill   				= new Bill();
		$visit 					= $this->visit();
		$patient 				= $this->patient();
		$person 				= $patient->person();
		$customer 				= $patient->customer();
		$insurance 				= $customer->insurance();
		$scheme 				= $customer->scheme();
		$pricelist				= $scheme->pricelist();
		$item 					= new Item($consultation_item_id);
		$price 					= $item->getPrice($pricelist->id);

		$transaction_item 		= array (
			'item_id'			=> $consultation_item_id,
			'name'				=> $item->name,
			'uom'				=> $item->uom,
			'pack'				=> $item->uom,
			'qty'				=> 1,
			'quantity'			=> 1,
			'unit_price'		=> $price,
			'price'				=> $price
		);
		
		$transaction_items[] 	= $transaction_item;

		$create_data        	= array (
            'type'          	=> 'Bill',
            'subtype'       	=> 'Consultation',
            'subtype_id'       	=> $this->id,
            'personnel'     	=> 'Customer',
            'personnel_id'  	=> $customer->id,
            'store_id'  		=> $store_id,
            'code'          	=> $customer->code,
            'name'          	=> $person->name,
            'birthdate'     	=> $person->birthdate,
            'sex'       		=> $person->sex,
            'phone'       		=> $person->phone,
            'email'       		=> $person->email,
            'pricelist'     	=> $pricelist->code,
            'pricelist_id'  	=> $pricelist->id,
            'insurance'     	=> $insurance->code,
            'insurance_id'  	=> $insurance->id,
            'scheme'     		=> $scheme->code,
            'scheme_id'  		=> $scheme->id,
			'product'     		=> $customer->product,
            'product_id'  		=> $customer->product_id,
            'card_no'       	=> $visit->card_no,
            'authorization_no' 	=> $visit->authorization_no,
            'supplementary' 	=> $visit->supplementary,
            'folio_id'      	=> $visit->folio_id,
            'folio_no'      	=> $visit->folio_no,
            'visit_id'      	=> $visit->id,
            'visit_location_id' => $this->id,
            
            'items'         	=> json_encode($transaction_items),
            'subtotal'      	=> $price,
            'progeress'     	=> 'Requested',
            'progress_code' 	=> 1
        );

        $bill_id     			= $bill->createWithMinimumData($create_data);
		$this->set('bill_id', $bill_id);
		$this->visit()->set('bill_id', $bill_id);
		return $bill_id;
	}

	function patient() {
		return new Patient($this->patient_id);
	}

	function location() {
		if($this->ward_id) return new Ward($this->ward_id);
		return new Clinic($this->clinic_id);
	}

	function getConsultationBill() {

	}

	function renderNurseStationCard() { 
		global $root, $app;

		$card_color 		= 'secondary';
		$patient 			= $this->patient();
		$customer 			= $patient->customer();
		$person 			= $patient->person();
		$border_color 		= 'rgb(255,255,255)';

		$back_details   	= '<h4> Checked in </h4>
		<div class="fw-semibold">'.$this->createdUser()->person()->name.'</div>
		<div>'.$app->datetime($this->created_datetime).'</div>
		<div>'.$app->age($this->created_datetime).' ago </div>';

		$seen 				= '<h4>Never attended</h4>';
		$notes_sql 			= "SELECT id, title, created_user_id, created_datetime FROM clinical_notes WHERE visit_id = $this->visit_id AND status = 1 ORDER BY id DESC LIMIT 1";
		foreach($app->read($notes_sql) as $row) {
			$doctor_user	= new User($row->created_user_id);
			$qualification 	= $doctor_user->qualification();
			$border_color 	= $qualification->color ? $qualification->color : 'white';
			$border_color	= 'rgb('.$app->colors->$border_color->rgb.')';
			$doctor			= $doctor_user->person()->name;
			$seen 			= '<h4>Seen by</h4><small>'.$qualification->name.'</small><div class="fw-bold">'.$doctor.'</div><div class="fw-semibold">'.$row->title.'</div>'.$app->getAge($row->created_datetime).' ago';
		}

		$icon 			= 'fa-solid fa-check';
		$icon_color 	= 'green';
		$button 		= '<a class="remote-modal bs-tooltip btn btn-lg btn-'.$card_color.'" title="Patient File" data-bs-toggle="modal" data-bs-target="#modal_xl" href="form.php?action=open&id='.$patient->id.'"> <span class="fa-regular fa-folder-user"></span> </a>';
					
		$html 				= '<div id="card_patient_'.$patient->id.'" class="auto-flip bs-tooltip" title="">
		<div class="shadow shadow-lg card border-'.$card_color.' text-center front bg-'.$card_color.'" style="--bs-bg-opacity: .1">
		<div class="card-img-top bg-'.$card_color.' text-white py-4" style="height:200px; min-height:200px">
		'.$button.'
		</div>
		<img src="'.$patient->picture().'" class="rounded-circle mx-auto d-block bg-light" style="width:75%; max-width:190px; margin-top:-120px; border:8px solid '.$border_color.'">
		<div style="display:inline-block; position:relative; top:-16px; font-size:.70em"> <span class="bg-'.$icon_color.' text-white" style="padding:0.4em 0.45em; border:6px solid white; border-radius:50%;"> <span class="'.$icon.' fa-fw"></span> </div>
		<div class="card-body pt-0" style="min-height:180px;">
		
		<h5 class="fw-bold"> '.$customer->formatCode().' </h5>
		<h6 class="fw-semibold text-truncate" style="font-size:1.1em">'.$person->name.'</h6>
		<h6 class="fw-normal my-0">'.ucwords($person->age().' '.$person->sex).'</h6>
		<div class="text-secondary text-truncate mt-1" style="font-size:0.9em">'.$customer->insurance()->code.' - '.$customer->scheme()->name.'</div>
		'.$this->renderServiceCard().'

		</div></div>
		
		<div class="shadow-sm card border-'.$card_color.' text-center back bg-'.$card_color.'" style="--bs-bg-opacity: .08">
		<div class="card-img-top bg-'.$card_color.' text-white py-4" style="height:200px; min-height:200px">
		<a class="btn btn-'.$card_color.' bs-tooltip" data-flip="card_patient_'.$patient->id.'" title="Flip"> <span class="fa-solid fa-arrow-left"></span> </a>
		'.$back_details.'
		</div>
		<img src="'.$patient->picture().'" class="rounded-circle mx-auto d-block bg-light" style="visibility:hidden; width:75%; max-width:190px; margin-top:-120px; border:7px solid white">
		<div class="card-body" style="min-height:180px;">
		'.$seen.'
		</div></div></div>';
		
		return $html;
	}

	function renderOutpatientCard() { 
		global $root, $app; 

		$card_color 		= 'secondary';
		$patient 			= $this->patient();
		$customer 			= $patient->customer();
		$person 			= $patient->person();
		$bill 				= new Bill($this->bill_id);
		$invoice 			= new Invoice($this->invoice_id);
		$insurance			= $customer->insurance();
		$border_color 		= 'rgb(255,255,255)';
		$form 				= 'form.php';

		$back_details   	= '<h4> Checked in </h4>
		<div class="fw-semibold">'.$this->createdUser()->person()->name.'</div>
		<div>'.$app->datetime($this->created_datetime).'</div>
		<div>'.$app->age($this->created_datetime).' ago </div>';

		$seen 				= '<h4>Never attended</h4>';
		$notes_sql 			= "SELECT id, title, created_user_id, created_datetime FROM clinical_notes WHERE visit_id = $this->visit_id AND status = 1 ORDER BY id DESC LIMIT 1";
		foreach($app->read($notes_sql) as $row) {
			$doctor_user	= new User($row->created_user_id);
			$qualification 	= $doctor_user->qualification();
			$border_color 	= $qualification->color ? $qualification->color : 'white';
			$border_color	= 'rgb('.$app->colors->$border_color->rgb.')';
			$doctor			= $doctor_user->person()->name;
			$seen 			= '<h4>Seen by</h4><small>'.$qualification->name.'</small><div class="fw-bold">'.$doctor.'</div><div class="fw-semibold">'.$row->title.'</div>'.$app->getAge($row->created_datetime).' ago';
		}

		if($this->id) {
			if($this->visit_type == 'Outpatient') {
				if($invoice->id) {
					if($invoice->payment || $insurance->code == 'NHIF') {
						$icon 			= 'fa-solid fa-check';
						$icon_color 	= 'green';
						$button 		= '<a class="remote-modal bs-tooltip btn btn-lg btn-'.$card_color.'" title="Patient File" data-bs-toggle="modal" data-bs-target="#modal_xl" href="form.php?action=open&id='.$patient->id.'"> <span class="fa-regular fa-folder-user"></span> </a>';
					} else {
						$icon 			= 'fa-solid fa-hand-holding-dollar';
						$icon_color 	= 'orange';
						$button 		= '<a class="remote-modal bs-tooltip btn btn-lg btn-'.$card_color.'" title="Consultation Not Paid" data-bs-toggle="modal" data-bs-target="#modal_lg" href="../info/unpaid_consultation_form.php?action=open&id='.$patient->id.'"> <span class="fa-regular fa-memo-circle-info"></span> </a>';
						$form 			= '../info/unpaid_consultation_form.php';
					}
				} elseif($bill->id) {
					if($insurance->code == 'NHIF') {
						$icon 			= 'fa-regular fa-cart-circle-plus';
						$icon_color 	= 'orange';
						$button 		= '<a class="remote-modal bs-tooltip btn btn-lg btn-'.$card_color.'" title="Add Consultation" data-bs-toggle="modal" data-bs-target="#modal_lg" href="../info/add_consultation_form.php?action=edit&id='.$patient->id.'"> <span class="fa-regular fa-cart-circle-plus"></span> </a>';
						$form 			= '../info/add_consultation_form.php';
					} else {
						$icon 			= 'fa-solid fa-xmark-to-slot';
						$icon_color 	= 'danger';
						$button 		= '<a class="remote-modal bs-tooltip btn btn-lg btn-'.$card_color.'" title="Consultation Not Billed" data-bs-toggle="modal" data-bs-target="#modal_lg" href="../info/unbilled_consultation_form.php?action=open&id='.$patient->id.'"> <span class="fa-regular fa-memo-circle-info"></span> </a>';
						$form 			= '../info/unbilled_consultation_form.php';
					}
				} else {

					// Bill not created
					if($insurance->code == 'NHIF') {
						$icon 			= 'fa-regular fa-cart-circle-plus';
						$icon_color 	= 'orange';
						$button 		= '<a class="remote-modal bs-tooltip btn btn-lg btn-'.$card_color.'" title="Add Consultation" data-bs-toggle="modal" data-bs-target="#modal_lg" href="../info/add_consultation_form.php?action=edit&id='.$patient->id.'"> <span class="fa-regular fa-cart-circle-plus"></span> </a>';
						$form 			= '../info/add_consultation_form.php';
					} else {
						$icon 			= 'fa-solid fa-hand-holding-dollar';
						$icon_color 	= 'danger';
						$button 		= '<a class="remote-modal bs-tooltip btn btn-lg btn-'.$card_color.'" title="No Consultation" data-bs-toggle="modal" data-bs-target="#modal_lg" href="../info/no_consultation_form.php?action=open&id='.$patient->id.'"> <span class="fa-regular fa-memo-circle-info"></span> </a>';
						$form 			= '../info/no_consultation_form.php';
					}
				}
			} else {
				$icon 			= 'fa-solid fa-check';
				$icon_color 	= 'success';
				$button 		= '<a class="remote-modal bs-tooltip btn btn-lg btn-'.$card_color.'" title="Patient File" data-bs-toggle="modal" data-bs-target="#modal_xl" href="form.php?action=open&id='.$patient->id.'"> <span class="fa-regular fa-folder-user"></span> </a>';
			}
		}

		$html 				= '<div id="card_patient_'.$patient->id.'" class="auto-flip bs-tooltip" title="">
		<div class="shadow shadow-lg card border-'.$card_color.' text-center front bg-'.$card_color.'" style="--bs-bg-opacity: .1">
		<div class="card-img-top bg-'.$card_color.' text-white py-4" style="height:200px; min-height:200px">
		'.$button.'
		</div>
		<img src="'.$patient->picture().'" class="rounded-circle mx-auto d-block bg-light" style="width:75%; max-width:190px; margin-top:-120px; border:8px solid '.$border_color.'">
		<div style="display:inline-block; position:relative; top:-16px; font-size:.70em"> <span class="bg-'.$icon_color.' text-white" style="padding:0.4em 0.45em; border:6px solid white; border-radius:50%;"> <span class="'.$icon.' fa-fw"></span> </div>
		<div class="card-body pt-0" style="min-height:180px;">
		
		<h5 class="fw-bold"> '.$customer->formatCode().' </h5>
		<h6 class="fw-semibold text-truncate" style="font-size:1.1em">'.$person->name.'</h6>
		<h6 class="fw-normal my-0">'.ucwords($person->age().' '.$person->sex).'</h6>
		<div class="text-secondary text-truncate mt-1" style="font-size:0.9em">'.$customer->insurance()->code.' - '.$customer->scheme()->name.'</div>
		'.$this->renderServiceCard($form).'

		</div></div>
		
		<div class="shadow-sm card border-'.$card_color.' text-center back bg-'.$card_color.'" style="--bs-bg-opacity: .08">
		<div class="card-img-top bg-'.$card_color.' text-white py-4" style="height:200px; min-height:200px">
		<a class="btn btn-'.$card_color.' bs-tooltip" data-flip="card_patient_'.$patient->id.'" title="Flip"> <span class="fa-solid fa-arrow-left"></span> </a>
		'.$back_details.'
		</div>
		<img src="'.$patient->picture().'" class="rounded-circle mx-auto d-block bg-light" style="visibility:hidden; width:75%; max-width:190px; margin-top:-120px; border:7px solid white">
		<div class="card-body" style="min-height:180px;">
		'.$seen.'
		</div></div></div>';
		
		return $html;
	}

	function serviceExists($table) {
		global $app;
		$sql	= "SELECT id FROM $table WHERE visit_id = $this->visit_id LIMIT 1";
		return $app->read($sql);
	}

	function renderServiceCard($form='form.php') {
		$vitals_css			= $this->serviceExists('clinical_vitals') ? 'btn-secondary' : 'btn-outline-secondary';
		$notes_css			= $this->serviceExists('clinical_notes') ? 'btn-secondary' : 'btn-outline-secondary';
		$diagnoses_css		= $this->serviceExists('clinical_diagnoses') ? 'btn-secondary' : 'btn-outline-secondary';
		$investigations_css	= $this->serviceExists('clinical_laboratory') ? 'btn-secondary' : 'btn-outline-secondary';
		$procedures_css		= $this->serviceExists('clinical_procedures') ? 'btn-secondary' : 'btn-outline-secondary';
		$prescriptions_css	= $this->serviceExists('clinical_prescriptions') ? 'btn-secondary' : 'btn-outline-secondary';

		$html = '<a title="Vitals" data-bs-toggle="modal" data-bs-target="#modal_xl" href="'.$form.'?action=open&id='.$this->patient_id.'&target=PatientVitals" class="remote-modal bs-tooltip btn btn-sm my-2 me-1 '.$vitals_css.'"> <span class="fa-regular fa-heart-pulse"></span> </a>
		<a title="Notes" data-bs-toggle="modal" data-bs-target="#modal_xl" href="'.$form.'?action=open&id='.$this->patient_id.'&target=PatientNotes" class="remote-modal bs-tooltip btn btn-sm my-2 me-1 '.$notes_css.'"> <span class="fa-regular fa-notes-medical"></span> </a>
		<a title="Diagnoses" data-bs-toggle="modal" data-bs-target="#modal_xl" href="'.$form.'?action=open&id='.$this->patient_id.'&target=PatientDiagnoses" class="remote-modal bs-tooltip btn btn-sm my-2 me-1 '.$diagnoses_css.'"> <span class="fa-regular fa-stethoscope"></span> </a>
		<a title="Investigations" data-bs-toggle="modal" data-bs-target="#modal_xl" href="'.$form.'?action=open&id='.$this->patient_id.'&target=PatientLaboratory" class="remote-modal bs-tooltip btn btn-sm my-2 me-1 '.$investigations_css.'"> <span class="fa-regular fa-microscope"></span> </a>
		<a title="Procedures" data-bs-toggle="modal" data-bs-target="#modal_xl" href="'.$form.'?action=open&id='.$this->patient_id.'&target=PatientProcedure" class="remote-modal bs-tooltip btn btn-sm my-2 me-1 '.$procedures_css.'"> <span class="fa-regular fa-scissors"></span> </a>
		<a title="Prescription" data-bs-toggle="modal" data-bs-target="#modal_xl" href="'.$form.'?action=open&id='.$this->patient_id.'&target=PatientPrescription" class="remote-modal bs-tooltip btn btn-sm my-2 '.$prescriptions_css.'"> <span class="fa-regular fa-pills"></span> </a>';

		return $html;
	}

	function renderInpatientCard() { 
		global $root, $app; 

		$card_color 		= 'secondary';
		$border_color 		= 'rgb(255,255,255)';
		$patient 			= $this->patient();
		$customer 			= $patient->customer();
		$person 			= $patient->person();
		
		$back_details   	= '<h4> Admitted by </h4>
		<div class="fw-semibold">'.$this->createdUser()->person()->name.'</div>
		<div>'.$app->datetime($this->created_datetime).'</div>
		<div>'.$app->age($this->created_datetime).' ago </div>';

		$seen 				= '<h4>Never attended</h4>';
		$notes_sql 			= "SELECT id, title, created_user_id, created_datetime FROM clinical_notes WHERE visit_id = $this->visit_id AND status = 1 ORDER BY id DESC LIMIT 1";
		foreach($app->read($notes_sql) as $row) {
			$doctor_user	= new User($row->created_user_id);
			$qualification 	= $doctor_user->qualification();
			$border_color 	= $qualification->color ? $qualification->color : 'white';
			$border_color	= 'rgb('.$app->colors->$border_color->rgb.')';
			$doctor			= $doctor_user->person()->name;
			$seen 			= '<h4>Seen by</h4><small>'.$qualification->name.'</small><div class="fw-bold">'.$doctor.'</div><div class="fw-semibold">'.$row->title.'</div>'.$app->getAge($row->created_datetime).' ago';
		}

		if($this->bed) {
			$icon 			= 'fa-solid fa-check';
			$icon_color 	= 'success';
			$button 		= '<a class="remote-modal bs-tooltip btn btn-lg btn-'.$card_color.'" title="Patient File" data-bs-toggle="modal" data-bs-target="#modal_xl" href="form.php?action=open&id='.$patient->id.'"> <span class="fa-regular fa-folder-user"></span> </a>';
		} else {
			$icon 			= 'fa-solid fa-bed';
			$icon_color 	= 'warning';
			$button 		= '<a class="remote-modal bs-tooltip btn btn-lg btn-'.$card_color.'" title="Assign bed" data-bs-toggle="modal" data-bs-target="#modal_lg" href="assign_bed_form.php?action=edit&visit_location_id='.$this->id.'&id='.$patient->id.'"> <span class="fa-regular fa-bed"></span> </a>';
		}	
		
		$html 				= '<div id="card_patient_'.$patient->id.'" class="auto-flip bs-tooltip" title="">
		<div class="shadow shadow-lg card border-'.$card_color.' text-center front bg-'.$card_color.'" style="--bs-bg-opacity: .1">
		<div class="card-img-top bg-'.$card_color.' text-white py-4" style="height:200px; min-height:200px">
		'.$button.'
		</div>
		<img src="'.$patient->picture().'" class="rounded-circle mx-auto d-block bg-light" style="width:75%; max-width:190px; margin-top:-120px; border:8px solid '.$border_color.'">
		<div style="display:inline-block; position:relative; top:-16px; font-size:.70em"> <span class="bg-'.$icon_color.' text-white" style="padding:0.4em 0.45em; border:6px solid white; border-radius:50%;"> <span class="'.$icon.' fa-fw"></span> </div>
		<div class="card-body pt-0" style="min-height:180px;">
		
		<h5 class="fw-bold"> '.$customer->formatCode().' </h5>
		<h6 class="fw-semibold text-truncate" style="font-size:1.1em">'.$person->name.'</h6>
		<h6 class="fw-normal my-0">'.ucwords($person->age().' '.$person->sex).'</h6>
		<div class="text-secondary text-truncate mt-1" style="font-size:0.9em">'.$customer->insurance()->code.' - '.$customer->scheme()->name.'</div>
		'.$this->renderServiceCard().'

		</div></div>
		
		<div class="shadow-sm card border-'.$card_color.' text-center back bg-'.$card_color.'" style="--bs-bg-opacity: .08">
		<div class="card-img-top bg-'.$card_color.' text-white py-4" style="height:200px; min-height:200px">
		<a class="btn btn-'.$card_color.' bs-tooltip" data-flip="card_patient_'.$patient->id.'" title="Flip"> <span class="fa-solid fa-arrow-left"></span> </a>
		'.$back_details.'
		<span class="fa-solid fa-bed fa-fw"></span><b> '.$this->bed.' </b>
		</div>
		<img src="'.$patient->picture().'" class="rounded-circle mx-auto d-block bg-light" style="visibility:hidden; width:75%; max-width:190px; margin-top:-120px; border:7px solid white">
		<div class="card-body" style="min-height:180px;">
		'.$seen.'
		</div></div></div>';

		return $html;
	}

	function room() {
		return new Room($this->room_id);
	}
}

?>