<?php 

$root           = '../../../../';
$submodule_code = 'PatientBills';
require($root.'config/config.php');

$patient        = new Patient($get->patient_id);
$person         = $patient->person();
$customer       = $patient->customer();
$visit          = $patient->lastVisit();
$visit_location = $patient->lastVisitLocation();
$wallet         = $customer->wallet();

$html           = $app->renderDocumentHeader();

if($visit->admitted) {
    $title      = 'Patient Draft Bill as of Today '.date('Y-m-d');
    $subtitle   = 'Print Draft Bill';
} else {
    $title      = 'Patient Final Bill as Closed on '.date('Y-m-d', strtotime($visit->checkout_datetime));
    $subtitle   = 'Print Final Bill';
}

$bills_str      = "SELECT billing_invoices.id, 
billing_invoices.subtype, 
billing_invoice_items.item_id, 
billing_invoice_items.name, 
billing_invoice_items.quantity quantity, 
billing_invoice_items.price billed,
(billing_invoice_items.paid + IFNULL(billing_promise_items.paid, 0)) paid, 
billing_invoice_items.exemption exemption, 
(IFNULL(billing_promise_items.promise, 0) - IFNULL(billing_promise_items.paid, 0)) promise,
(billing_invoice_items.price - billing_invoice_items.paid - IFNULL(billing_promise_items.paid, 0) - billing_invoice_items.exemption) payable
FROM billing_invoice_items 
JOIN billing_invoices ON billing_invoices.id = billing_invoice_items.parent_id 
LEFT JOIN billing_promise_items ON billing_promise_items.type_id = billing_invoices.type_id AND billing_promise_items.item_id = billing_invoice_items.item_id
WHERE billing_invoices.visit_id = $visit->id 
AND billing_invoices.type = 'Bill'
AND billing_invoices.payment = 1; ";

foreach($app->read($bills_str) as $bill) {
    $subtype            = $bill->subtype;
    if($subtype == 'Prescription') $subtype = 'Pharmacy';
    if($subtype == 'Procedure') $subtype = 'Procedures';
    if(isset($bills[$subtype][$bill->item_id])) {
        $bills[$subtype][$bill->item_id]->quantity  += $bill->quantity;
        $bills[$subtype][$bill->item_id]->billed    += $bill->billed;
        $bills[$subtype][$bill->item_id]->paid      += $bill->paid;
        $bills[$subtype][$bill->item_id]->exemption += $bill->exemption;
        $bills[$subtype][$bill->item_id]->promise   += $bill->promise;
        $bills[$subtype][$bill->item_id]->payable   += $bill->payable;
    } else {
        $bills[$subtype][$bill->item_id]            = $bill;
    }
}

$total_billed       = 0;
$total_paid         = 0;
$total_exemption    = 0;
$total_promise      = 0;
$total_payable      = 0;

$html               .= '<h2 class="text-center">'.$title.'</h2>';

$html .= '<table class="w-100 p-7">
<tr><td class="w-100 bg-light border"><h3>Patient details</h3></td></tr>
</table>';

$html .= '<table class="w-100 p-7 border bg-light">

<tr>
<td class="fw-bold" style="width:25%"> File no </td>
<td class="fw-bold text-muted" style="width:75%"> '.$patient->formatCode().' </td>
</tr>

<tr>
<td class="fw-bold" style="width:25%"> Patient name </td>
<td class="fw-bold text-muted" style="width:75%"> '.$person->name.' </td>
</tr>

<tr>
<td class="fw-bold" style="width:25%"> Age and sex </td>
<td class="fw-bold text-muted" style="width:75%"> '.$person->age().' '.$person->sex.'</td>
</tr>

<tr>
<td class="fw-bold" style="width:25%"> Sponsor </td>
<td class="fw-bold text-muted" style="width:75%"> '.$customer->insurance()->name.' - '.$customer->scheme()->name.'</td>
</tr>';

if($visit_location->ward_id) {
    $ward = new Ward($visit_location->ward_id);
    $end_date   = $visit->admitted ? 'Today' : $app->date($visit->checkout_datetime);
    $html .= '<tr>
    <td class="fw-bold" style="width:25%"> Admitted </td>
    <td class="fw-bold text-muted" style="width:75%"> '.$ward->location()->name.' (From '.$app->date($visit->checkin_datetime).' to '.$end_date.')</td>
    </tr>';
} else {
    $clinic = new Ward($visit_location->clinic_id);
    $html .= '<tr>
    <td class="fw-bold" style="width:25%"> Checked in </td>
    <td class="fw-bold text-muted" style="width:75%"> '.$clinic->location()->name.' </td>
    </tr>';
}

$html .= '</table>';

$html .= '<table class=""><tr><td></td></tr></table>';

foreach(array_keys($bills) as $bill_type) {

    $html .= '<table class="w-100 p-7">
    <tr><td class="w-100 bg-light border"><h3>'.$bill_type.' ('.count($bills[$bill_type]).')</h3></td></tr>
    </table>';

    //$html .= '<table class=""><tr><td></td></tr></table>';
        
    $html .= '<table class="w-100 p-7 border bg-light">
    <tr>
    <td class="fw-bold" style="width:25%"> Item name </td>
    <td class="fw-bold text-center" style="width:7.5%"> Qty </td>
    <td class="fw-bold text-end" style="width:13.5%"> Billed </td>
    <td class="fw-bold text-end" style="width:13.5%"> Paid </td>
    <td class="fw-bold text-end" style="width:13.5%"> Exempt. </td>
    <td class="fw-bold text-end" style="width:13.5%"> Promise </td>
    <td class="fw-bold text-end" style="width:13.5%"> Payable </td>
    </tr>
    </table>';

    $table_options      = array (
        'columns'       => array (
            array('name'=>'Service'),
            array('name'=>'Qty', 'css'=>'w-5 text-center'),
            array('name'=>'Billed', 'css'=>'w-12 text-end'),
            array('name'=>'Paid', 'css'=>'w-12 text-end'),
            array('name'=>'Exempt.', 'css'=>'w-12 text-end'),
            array('name'=>'Promise', 'css'=>'w-12 text-end'),
            array('name'=>'Payable', 'css'=>'w-12 text-end')
        ),
        'css'           => 'table-containerless'
    );
    
    $subtotal_billed       = 0;
    $subtotal_paid         = 0;
    $subtotal_exemption    = 0;
    $subtotal_promise      = 0;
    $subtotal_payable      = 0;

    $html .= '<table class="w-100 p-7 border">';
    
    foreach($bills[$bill_type] as $bill) {
        $html .= '<tr>
        <td class="border-top" style="width:25%">'.$bill->name.'</td>
        <td class="text-center border-top" style="width:7.5%">'.$bill->quantity.'</td>
        <td class="text-end border-top text-muted" style="width:13.5%">'.$app->formatMoney($bill->billed).'</td>
        <td class="text-end border-top text-muted" style="width:13.5%">'.$app->formatMoney($bill->paid).'</td>
        <td class="text-end border-top text-muted" style="width:13.5%">'.$app->formatMoney($bill->exemption).'</td>
        <td class="text-end border-top text-muted" style="width:13.5%">'.$app->formatMoney($bill->promise).'</td>
        <td class="text-end border-top text-muted" style="width:13.5%">'.$app->formatMoney($bill->payable).'</td>
        </tr>';

        $subtotal_billed        += $bill->billed;
        $subtotal_paid          += $bill->paid;
        $subtotal_exemption     += $bill->exemption;
        $subtotal_promise       += $bill->promise;
        $subtotal_payable       += $bill->payable;

        $total_billed           += $bill->billed;
        $total_paid             += $bill->paid;
        $total_exemption        += $bill->exemption;
        $total_promise          += $bill->promise;
        $total_payable          += $bill->payable;
    }

    $html .= '<tr>
    <td class="py-3 text-muted fw-bold border-top">'.$bill_type.' subtotal</td>
    <td class="text-center border-top"></td>
    <td class="text-end border-top text-muted fw-bold">'.$app->formatMoney($subtotal_billed).'</td>
    <td class="text-end border-top text-muted fw-bold">'.$app->formatMoney($subtotal_paid).'</td>
    <td class="text-end border-top text-muted fw-bold">'.$app->formatMoney($subtotal_exemption).'</td>
    <td class="text-end border-top text-muted fw-bold">'.$app->formatMoney($subtotal_promise).'</td>
    <td class="text-end border-top text-muted fw-bold">'.$app->formatMoney($subtotal_payable).'</td>
    </tr>';

    $html .= '</table>';

    $html .= '<table class=""><tr><td></td></tr></table>';
}

$html .= '<div style="page-break-inside:avoid;">';

$html .= '<table class="w-100 p-7">
<tr><td class="w-100 bg-light border"><h3>Billing summary</h3></td></tr>
</table>';

$html .= '<table class="w-100 p-7 border bg-light">
<tr>
<td class="fw-bold" style="width:25%"> Description </td>
<td class="fw-bold text-end" style="width:75%"> Amount (TZS) </td>
</tr>
</table>';

$html .= '</div>';

$html .= '<table class="w-100 p-7 border bg-light">';

$html .= '<tr><td class="py-3 border-top">Billed Amount</td> <td class="text-end border-top fw-bold">'.$app->formatMoney($total_billed).'</td></tr>
<tr><td class="py-3 border-top">Paid Amount</td> <td class="text-end border-top fw-bold">'.$app->formatMoney($total_paid).'</td></tr>
<tr><td class="py-3 border-top">Exempted Amount</td> <td class="text-end border-top fw-bold">'.$app->formatMoney($total_exemption).'</td></tr>
<tr><td class="py-3 border-top">Promise to Pay</td> <td class="text-end border-top fw-bold">'.$app->formatMoney($total_promise).'</td></tr>
<tr><td class="py-3 border-top">Payable Amount</td> <td class="text-end border-top fw-bold">'.$app->formatMoney($total_payable).'</td></tr>
<tr><td class="py-3 border-top">Wallet Balance</td> <td class="text-end border-top fw-bold">'.$app->formatMoney($wallet->subtotal).'</td></tr>';

$balance = $wallet->subtotal - $total_payable;

if($balance < 0) {
    $html .= '<tr><td class="border-top fw-bold">Wallet Topup Amount</td> <td class="text-end text-end border-top fw-bold">'.$app->formatMoney(abs($balance)).'</td></tr>';
} else {
    $html .= '<tr><td class="py-3 border-top text-success">Refund Amount </td> <td class="text-end py-3 text-success">'.$app->formatMoney($balance).'</td></tr>';
}

$html .= '</table>';

$html .= '<table class=""><tr><td></td></tr></table>';

$html .= '<table class="w-100 p-7 border" style="page-break-inside:avoid;">
<tr class="bg-light">
<td class="w-25 text-center border-left border-top border-bottom"><h3>Prepared by:</h3></td>
<td class="w-25 text-center border-top border-bottom"><h3>Authorized by:</h3></td>
<td class="w-25 text-center border-top border-bottom"><h3>Approved by:</h3></td>
<td class="w-25 text-center border-top border-bottom border-right"><h3>Printed by:</h3></td>
</tr><tr>
<td class="w-25 text-center"><br /> <br /> <br /> ..................................<br />'.$user->person()->name.'<br /><b>'.$app->date($now).'</b></td>
<td class="w-25 text-center"><br /> <br /> <br /> ..................................<br /><br /><b>'.$app->date($now).'</b></td>
<td class="w-25 text-center"><br /> <br /> <br /> ..................................<br /><br /><b>'.$app->date($now).'</b></td>
<td class="w-25 text-center"><br /> <br /> <br /> ..................................<br />'.$user->person()->name.'<br /><b>'.$app->date($now).'</b></td>
</tr>
</table>';

$app->renderPDF($html, ''.$title.'');

?>