<?php 

$root           = '../../../../';
$submodule_code = 'PatientBills';
require($root.'config/config.php');

$patient        = new Patient($get->patient_id);
$person         = $patient->person();
$customer       = $patient->customer();
$scheme         = $customer->scheme();
$visit          = new Visit($get->visit_id);
$visit_location = $patient->lastVisitLocation();
$wallet         = $customer->wallet();
$bills          = $visit->getBills(array('visit_type'=>$get->visit_type));
$invoice        = new Invoice();
$title          = 'Tax invoice';
$folio          = new Folio($visit->folio_id);
$practioner     = $folio->getClinician()->person();
$practioner_no  = $practioner->getIdentityByCode('MCT')->number;

$invoice_subtotal = 0;
foreach(array_keys($bills) as $bill_type) {
    foreach($bills[$bill_type] as $bill) {
        $invoice_subtotal   += $bill->payable;
    }
}

$invoice_data   = array (
    'type'          => 'Invoice',
    'store_id'      => 0,
    'name'          => $person->name,
    'cost'          => 0,
    'subtotal'      => $invoice_subtotal,
    'progeress'     => 'Requested',
    'progress_code' => 1
);

if($visit->invoice_id) {
    $invoice_id     = $visit->invoice_id;
} else {
    $invoice_id     = $invoice->createWithMinimumData($invoice_data);
    $invoice->delete();
    $visit->set('invoice_id', $invoice_id);
}

$html               = $app->renderDocumentHeader();

$total_billed       = 0;
$total_paid         = 0;
$total_exemption    = 0;
$total_promise      = 0;
$total_payable      = 0;

$html               .= '<h2 class="text-center">Tax Invoice '.$invoice_id.'</h2>';

$html .= '<table class="w-100 p-7 border bg-light">
<tr>
<td class="fw-bold" style="width:50%">Invoice Details </td>
<td class="fw-bold" style="width:50%">Customer Details </td>
</tr>
</table>';

$html .= '<table class="w-100 p-7 border">

<tr>
<td class="fw-bold" style="width:15%">Invoice no </td>
<td class="fw-bold text-muted" style="width:35%">'.$invoice_id.' </td>
<td class="fw-bold" style="width:15%">Company </td>
<td class="fw-bold text-muted" style="width:35%">'.$scheme->insurance()->name.' </td>
</tr>

<tr>
<td class="fw-bold" style="width:15%">Invoice date </td>
<td class="fw-bold text-muted" style="width:35%">'.date('d M Y', strtotime($visit->created_datetime)).'</td>
<td class="fw-bold" style="width:15%">Patient </td>
<td class="fw-bold text-muted" style="width:35%">'.$person->name.' </td>
</tr>

<tr>
<td class="fw-bold" style="width:15%">From </td>
<td class="fw-bold text-muted" style="width:35%">'.$client->client_name.' </td>
<td class="fw-bold" style="width:15%">Birth </td>
<td class="fw-bold text-muted" style="width:35%">'.$person->birthdate.' ('.$person->age().' '.$person->sex.')</td>
</tr>

<tr>
<td class="fw-bold" style="width:15%">Address </td>
<td class="fw-bold text-muted" style="width:35%">'.$client->client_address.'</td>
<td class="fw-bold" style="width:15%">File number </td>
<td class="fw-bold text-muted" style="width:35%">'.$customer->formatCode().'</td>
</tr>

<tr>
<td class="fw-bold" style="width:15%">Practioner </td>
<td class="fw-bold text-muted" style="width:35%">'.$practioner->name.'</td>
<td class="fw-bold" style="width:15%">Employer</td>
<td class="fw-bold text-muted" style="width:35%">'.$person->employer.'</td>
</tr>';

if($visit_location->ward_id) {
    $ward = new Ward($visit_location->ward_id);
    $end_date   = $visit->admitted ? 'Today' : $app->date($visit->checkout_datetime);
    $html .= '<tr>
    <td class="fw-bold" style="width:15%">Pract. no </td>
    <td class="fw-bold text-muted" style="width:35%">'.$practioner_no.'</td>
    <td class="fw-bold" style="width:15%">Employer no </td>
    <td class="fw-bold text-muted" style="width:35%">'.$person->employer_no.'</td>
    </tr>';
} else {
    $clinic = new Ward($visit_location->clinic_id);
    $html .= '<tr>
    <td class="fw-bold" style="width:15%">Practioner no </td>
    <td></td>
    <td class="fw-bold" style="width:15%">Employer no </td>
    <td class="fw-bold text-muted" style="width:35%">'.$person->employer_no.'</td>
    </tr>';
}

$html .= '</table>';

$html .= '<table class=""><tr><td></td></tr></table>';

foreach(array_keys($bills) as $bill_type) {

    $html .= '<table class="w-100 p-7">
    <tr><td class="w-100 bg-light border"><h3>'.$bill_type.' ('.count($bills[$bill_type]).')</h3></td></tr>
    </table>';

    //$html .= '<table class=""><tr><td></td></tr></table>';
        
    $html .= '<table class="w-100 p-7 border bg-light">
    <tr>
    <td class="fw-bold" style="width:50%"> Item name </td>
    <td class="fw-bold text-center" style="width:9.5%"> Qty </td>
    <td class="fw-bold text-end" style="width:13.5%"> Billed </td>
    <td class="fw-bold text-end" style="width:13.5%"> Paid </td>
    <td class="fw-bold text-end" style="width:13.5%"> Payable </td>
    </tr>
    </table>';

    $subtotal_billed       = 0;
    $subtotal_paid         = 0;
    $subtotal_exemption    = 0;
    $subtotal_promise      = 0;
    $subtotal_payable      = 0;

    $html .= '<table class="w-100 p-7 border">';
    
    foreach($bills[$bill_type] as $bill) {
        $html .= '<tr>
        <td class="border-top" style="width:50%">'.$bill->name.'</td>
        <td class="text-center border-top" style="width:9.5%">'.$bill->quantity.'</td>
        <td class="text-end border-top text-muted" style="width:13.5%">'.$app->formatMoney($bill->billed).'</td>
        <td class="text-end border-top text-muted" style="width:13.5%">'.$app->formatMoney($bill->paid).'</td>
        <td class="text-end border-top text-muted" style="width:13.5%">'.$app->formatMoney($bill->payable).'</td>
        </tr>';

        $subtotal_billed        += $bill->billed;
        $subtotal_paid          += $bill->paid;
        $subtotal_exemption     += $bill->exemption;
        $subtotal_promise       += $bill->promise;
        $subtotal_payable       += $bill->payable;

        $total_billed           += $bill->billed;
        $total_paid             += $bill->paid;
        $total_exemption        += $bill->exemption;
        $total_promise          += $bill->promise;
        $total_payable          += $bill->payable;
    }

    $html .= '<tr>
    <td class="py-3 text-muted fw-bold border-top">'.$bill_type.' subtotal</td>
    <td class="text-center border-top"></td>
    <td class="text-end border-top text-muted fw-bold">'.$app->formatMoney($subtotal_billed).'</td>
    <td class="text-end border-top text-muted fw-bold">'.$app->formatMoney($subtotal_paid).'</td>
    <td class="text-end border-top text-muted fw-bold">'.$app->formatMoney($subtotal_payable).'</td>
    </tr>';

    $html .= '</table>';

    $html .= '<table class=""><tr><td></td></tr></table>';
}

$html .= '<table class="w-100 p-7">
<tr><td class="w-100 bg-light border"><h3>Summary</h3></td></tr>
</table>';

$html .= '<table class="w-100 p-7 border ">';

$html .= '<tr><td class="py-3 border-top">Billed amount</td> <td class="text-end border-top fw-bold">'.$app->formatMoney($total_billed).'</td></tr>
<tr><td class="py-3 border-top">Paid amount</td> <td class="text-end border-top fw-bold">'.$app->formatMoney($total_paid).'</td></tr>
<tr><td class="py-3 border-top">Payable amount</td> <td class="text-end border-top fw-bold">'.$app->formatMoney($total_payable).'</td></tr>
</table>';

$html .= '<table class=""><tr><td></td></tr></table>';

$html .= '<table class="w-100 p-7 border" style="page-break-inside:avoid;">
<tr class="bg-light">
<td class="w-25 text-center border-left border-top border-bottom"><h3>Prepared by:</h3></td>
<td class="w-25 text-center border-top border-bottom"><h3>Authorized by:</h3></td>
<td class="w-25 text-center border-top border-bottom"><h3>Approved by:</h3></td>
<td class="w-25 text-center border-top border-bottom border-right"><h3>Printed by:</h3></td>
</tr><tr>
<td class="w-25 text-center"><br /> <br /> <br /> ..................................<br />'.$user->person()->name.'<br /><b>'.$app->date($now).'</b></td>
<td class="w-25 text-center"><br /> <br /> <br /> ..................................<br /><br /><b>'.$app->date($now).'</b></td>
<td class="w-25 text-center"><br /> <br /> <br /> ..................................<br /><br /><b>'.$app->date($now).'</b></td>
<td class="w-25 text-center"><br /> <br /> <br /> ..................................<br />'.$user->person()->name.'<br /><b>'.$app->date($now).'</b></td>
</tr>
</table>';

$app->renderPDF($html, ''.$title.'');

?>