<?php 

$root           = '../../../../';
$submodule_code = 'PatientBills';
require($root.'config/config.php');

$patient        = new Patient($get->patient_id);
$person         = $patient->person();
$customer       = $patient->customer();
$scheme         = $customer->scheme();
$visit          = new Visit($get->visit_id);
$visit_location = $patient->lastVisitLocation();
$wallet         = $customer->wallet();
$bills          = $visit->getBills(array('visit_type'=>$get->visit_type));
$invoice        = new Invoice();
$title          = 'Tax invoice';
$folio          = new Folio($visit->folio_id);
$practioner     = $folio->getClinician()->person();
$practioner_no  = $practioner->getIdentityByCode('MCT')->number;

$invoice_subtotal = 0;
foreach(array_keys($bills) as $bill_type) {
    foreach($bills[$bill_type] as $bill) {
        $invoice_subtotal   += $bill->payable;
    }
}

$invoice_data   = array (
    'type'          => 'Invoice',
    'store_id'      => 0,
    'name'          => $person->name,
    'cost'          => 0,
    'subtotal'      => $invoice_subtotal,
    'progeress'     => 'Requested',
    'progress_code' => 1
);
$invoice_id     = $invoice->createWithMinimumData($invoice_data);
$invoice->delete();

$html           = $app->renderDocumentHeader();

if($visit->admitted) {
    $title      = 'Patient Draft Bill as of Today '.date('Y-m-d');
    $subtitle   = 'Print Draft Bill';
} else {
    $title      = 'Patient Final Bill as Closed on '.date('Y-m-d', strtotime($visit->checkout_datetime));
    $subtitle   = 'Print Final Bill';
}

$total_billed       = 0;
$total_paid         = 0;
$total_exemption    = 0;
$total_promise      = 0;
$total_payable      = 0;

$html               .= '<h2 class="text-center">'.$title.'</h2>';

if(CLIENT_ID == 'bulyanhulu') {
    $html .= '<table class="w-100 p-7 border bg-light">
    <tr>
    <td class="fw-bold" style="width:50%">Invoice Details </td>
    <td class="fw-bold" style="width:50%">Customer Details </td>
    </tr>
    </table>';

    $html .= '<table class="w-100 p-7 border">

    <tr>
    <td class="fw-bold" style="width:15%">Invoice no </td>
    <td class="fw-bold text-muted" style="width:35%">'.$invoice_id.' </td>
    <td class="fw-bold" style="width:15%">Company </td>
    <td class="fw-bold text-muted" style="width:35%">'.$scheme->insurance()->name.' </td>
    </tr>

    <tr>
    <td class="fw-bold" style="width:15%">Invoice date </td>
    <td class="fw-bold text-muted" style="width:35%">'.date('d M Y').'</td>
    <td class="fw-bold" style="width:15%">Patient </td>
    <td class="fw-bold text-muted" style="width:35%">'.$person->name.' </td>
    </tr>

    <tr>
    <td class="fw-bold" style="width:15%">From </td>
    <td class="fw-bold text-muted" style="width:35%">'.$client->client_name.' </td>
    <td class="fw-bold" style="width:15%">Birth </td>
    <td class="fw-bold text-muted" style="width:35%">'.$person->birthdate.' ('.$person->age().' '.$person->sex.')</td>
    </tr>

    <tr>
    <td class="fw-bold" style="width:15%">Address </td>
    <td class="fw-bold text-muted" style="width:35%">'.$client->client_address.'</td>
    <td class="fw-bold" style="width:15%">File number </td>
    <td class="fw-bold text-muted" style="width:35%">'.$customer->formatCode().'</td>
    </tr>

    <tr>
    <td class="fw-bold" style="width:15%">Practioner </td>
    <td class="fw-bold text-muted" style="width:35%">'.$practioner->name.'</td>
    <td class="fw-bold" style="width:15%">Employer</td>
    <td class="fw-bold text-muted" style="width:35%">'.$person->employer.'</td>
    </tr>';

    if($visit_location->ward_id) {
        $ward = new Ward($visit_location->ward_id);
        $end_date   = $visit->admitted ? 'Today' : $app->date($visit->checkout_datetime);
        $html .= '<tr>
        <td class="fw-bold" style="width:15%">Pract. no </td>
        <td class="fw-bold text-muted" style="width:35%">'.$practioner_no.'</td>
        <td class="fw-bold" style="width:15%">Employer no </td>
        <td class="fw-bold text-muted" style="width:35%">'.$person->employer_no.'</td>
        </tr>';
    } else {
        $clinic = new Ward($visit_location->clinic_id);
        $html .= '<tr>
        <td class="fw-bold" style="width:15%">Practioner no </td>
        <td></td>
        <td class="fw-bold" style="width:15%">Employer no </td>
        <td class="fw-bold text-muted" style="width:35%">'.$person->employer_no.'</td>
        </tr>';
    }

    $html .= '</table>';
} else {

    $html .= '<table class="w-100 p-7 border bg-light">

    <tr>
    <td class="fw-bold" style="width:25%"> File no </td>
    <td class="fw-bold text-muted" style="width:75%"> '.$patient->formatCode().' </td>
    </tr>

    <tr>
    <td class="fw-bold" style="width:25%"> Patient name </td>
    <td class="fw-bold text-muted" style="width:75%"> '.$person->name.' </td>
    </tr>

    <tr>
    <td class="fw-bold" style="width:25%"> Age and sex </td>
    <td class="fw-bold text-muted" style="width:75%"> '.$person->age().' '.$person->sex.'</td>
    </tr>

    <tr>
    <td class="fw-bold" style="width:25%"> Sponsor </td>
    <td class="fw-bold text-muted" style="width:75%"> '.$customer->insurance()->name.' - '.$customer->scheme()->name.'</td>
    </tr>';

    if($visit_location->ward_id) {
        $ward = new Ward($visit_location->ward_id);
        $end_date   = $visit->admitted ? 'Today' : $app->date($visit->checkout_datetime);
        $html .= '<tr>
        <td class="fw-bold" style="width:25%"> Admitted </td>
        <td class="fw-bold text-muted" style="width:75%"> '.$ward->location()->name.' (From '.$app->date($visit->checkin_datetime).' to '.$end_date.')</td>
        </tr>';
    } else {
        $clinic = new Ward($visit_location->clinic_id);
        $html .= '<tr>
        <td class="fw-bold" style="width:25%"> Checked in </td>
        <td class="fw-bold text-muted" style="width:75%"> '.$clinic->location()->name.' </td>
        </tr>';
    }

    $html .= '</table>';
}

$html .= '<table class=""><tr><td></td></tr></table>';

//$html .= '<table class=""><tr><td></td></tr></table>';
    
$html .= '<table class="w-100 p-7 border bg-light">
<tr>
<td class="fw-bold" style="width:25%"> Item name </td>
<td class="fw-bold text-center" style="width:7.5%">  </td>
<td class="fw-bold text-end" style="width:13.5%"> Billed </td>
<td class="fw-bold text-end" style="width:13.5%"> Paid </td>
<td class="fw-bold text-end" style="width:13.5%"> Exempt. </td>
<td class="fw-bold text-end" style="width:13.5%"> Promise </td>
<td class="fw-bold text-end" style="width:13.5%"> Payable </td>
</tr>';

foreach(array_keys($bills) as $bill_type) {

    $table_options      = array (
        'columns'       => array (
            array('name'=>'Service'),
            array('name'=>'', 'css'=>'w-5 text-center'),
            array('name'=>'Billed', 'css'=>'w-12 text-end'),
            array('name'=>'Paid', 'css'=>'w-12 text-end'),
            array('name'=>'Exempt.', 'css'=>'w-12 text-end'),
            array('name'=>'Promise', 'css'=>'w-12 text-end'),
            array('name'=>'Payable', 'css'=>'w-12 text-end')
        ),
        'css'           => 'table-containerless'
    );
    
    $subtotal_billed       = 0;
    $subtotal_paid         = 0;
    $subtotal_exemption    = 0;
    $subtotal_promise      = 0;
    $subtotal_payable      = 0;

    foreach($bills[$bill_type] as $bill) {
        $subtotal_billed        += $bill->billed;
        $subtotal_paid          += $bill->paid;
        $subtotal_exemption     += $bill->exemption;
        $subtotal_promise       += $bill->promise;
        $subtotal_payable       += $bill->payable;

        $total_billed           += $bill->billed;
        $total_paid             += $bill->paid;
        $total_exemption        += $bill->exemption;
        $total_promise          += $bill->promise;
        $total_payable          += $bill->payable;
    }

    $html .= '<tr>
    <td class="py-3 text-muted fw-bold border-top">'.$bill_type.'</td>
    <td class="text-center border-top"></td>
    <td class="text-end border-top text-muted fw-bold">'.$app->formatMoney($subtotal_billed).'</td>
    <td class="text-end border-top text-muted fw-bold">'.$app->formatMoney($subtotal_paid).'</td>
    <td class="text-end border-top text-muted fw-bold">'.$app->formatMoney($subtotal_exemption).'</td>
    <td class="text-end border-top text-muted fw-bold">'.$app->formatMoney($subtotal_promise).'</td>
    <td class="text-end border-top text-muted fw-bold">'.$app->formatMoney($subtotal_payable).'</td>
    </tr>';
}

$html .= '</table>';

$html .= '<table class=""><tr><td></td></tr></table>';

$html .= '<table class="w-100 p-7 border bg-light">';

$html .= '<tr><td class="py-3 border-top">Billed Amount</td> <td class="text-end border-top fw-bold">'.$app->formatMoney($total_billed).'</td></tr>
<tr><td class="py-3 border-top">Paid Amount</td> <td class="text-end border-top fw-bold">'.$app->formatMoney($total_paid).'</td></tr>
<tr><td class="py-3 border-top">Exempted Amount</td> <td class="text-end border-top fw-bold">'.$app->formatMoney($total_exemption).'</td></tr>
<tr><td class="py-3 border-top">Promise to Pay</td> <td class="text-end border-top fw-bold">'.$app->formatMoney($total_promise).'</td></tr>
<tr><td class="py-3 border-top">Payable Amount</td> <td class="text-end border-top fw-bold">'.$app->formatMoney($total_payable).'</td></tr>
<tr><td class="py-3 border-top">Wallet Balance</td> <td class="text-end border-top fw-bold">'.$app->formatMoney($wallet->subtotal).'</td></tr>';

$balance = $wallet->subtotal - $total_payable;

if($balance < 0) {
    $html .= '<tr><td class="border-top fw-bold">Wallet Topup Amount</td> <td class="text-end text-end border-top fw-bold">'.$app->formatMoney(abs($balance)).'</td></tr>';
} else {
    $html .= '<tr><td class="py-3 border-top text-success">Refund Amount </td> <td class="text-end py-3 text-success">'.$app->formatMoney($balance).'</td></tr>';
}

$html .= '</table>';

$html .= '<table class=""><tr><td></td></tr></table>';

$html .= '<table class="w-100 p-7 border" style="page-break-inside:avoid;">
<tr class="bg-light">
<td class="w-25 text-center border-left border-top border-bottom"><h3>Prepared by:</h3></td>
<td class="w-25 text-center border-top border-bottom"><h3>Authorized by:</h3></td>
<td class="w-25 text-center border-top border-bottom"><h3>Approved by:</h3></td>
<td class="w-25 text-center border-top border-bottom border-right"><h3>Printed by:</h3></td>
</tr><tr>
<td class="w-25 text-center"><br /> <br /> <br /> ..................................<br />'.$user->person()->name.'<br /><b>'.$app->date($now).'</b></td>
<td class="w-25 text-center"><br /> <br /> <br /> ..................................<br /><br /><b>'.$app->date($now).'</b></td>
<td class="w-25 text-center"><br /> <br /> <br /> ..................................<br /><br /><b>'.$app->date($now).'</b></td>
<td class="w-25 text-center"><br /> <br /> <br /> ..................................<br />'.$user->person()->name.'<br /><b>'.$app->date($now).'</b></td>
</tr>
</table>';

$app->renderPDF($html, ''.$title.'');

?>