<?php 

$root           = '../../../../';
$submodule_code = 'PatientDiagnoses';
require($root.'config/config.php');

$obj            = new Diagnosis();
$patient        = new Patient($post->patient_id);

if(isset($post->visit_id)) {
    $visit          = new Visit($post->visit_id);
    $visit_location = new VisitLocation($visit->getLastVisitLocationId());
} else {
    $visit          = $patient->visit();
    $visit_location = $patient->visitLocation();
}

$diagnosis_id   = 0;

if($post->action == 'delete') {
    $obj                    = new Diagnosis($post->diagnosis_id);
    $deleted                = $obj->delete();

    if($deleted) {
        echo $app->renderAlertSuccess('Clinical diagnosis deleted successfully');

        // Log lifestyle create event
        $user->timeline (
            array (
                'action'			=> 'Delete',
                'subject'			=> 'Diagnosis',
                'subject_id'		=> $get->diagnosis_id,
                'subtype'			=> $obj->code,
                'subtype_id'		=> 0,
                'personnel'			=> 'Patient',
                'personnel_id'		=> $patient->id,
                'name'	            => $patient->person()->name,
                'details'			=> 'Deleted clinical diagnosis',
                'visit_id'          => $visit_location->visit_id,
                'visit_location_id' => $visit_location->visit()->id,
            )
        );
    }
} else {

    if(!isset($post->diagnoses)) {
        die($app->renderAlertError('Please add atleast one diagnoses'));
    }
    
    // Check diagnoses compatibility
    if($patient->customer()->insurance()->code == 'NHIF') {
        foreach($post->diagnoses as $row) {
            if(strlen($row['code']) > 5) die($app->renderAlertError('Diagnoses is not compatible with National Health Insurance Fund', $row['code']));
        }
    }
    
    $user->checkSubmoduleAccess($submodule, 'create');
    
    foreach($post->diagnoses as $row) {
        $create_data                = $obj->addCommonFields (
            array (
                'type'              => 'ICD10',
                'subtype'           => '2016',
                'patient_id'        => $post->patient_id,
                'visit_id'          => $visit->id,
                'visit_location_id' => $visit_location->id,
                'group'             => '',
                'group_id'          => 0,
                'certainty'         => $row['certainty'],
                'code'              => $row['code'],
                'name'              => $row['name'],
                'details'           => '',
                'notes'             => $row['notes'],
                'flag'              => '',
                'tags'              => '',
            )
        );
        $diagnosis_id               = $obj->create($create_data);
    }

    if($diagnosis_id) {
        echo $app->renderAlertSuccess('Diagnoses added successfully');

        // Log lifestyle create event
        $user->timeline (
            array (
                'action'			=> 'Create',
                'subject'			=> 'Diagnosis',
                'subject_id'		=> $diagnosis_id,
                'subtype'			=> $row['code'],
                'subtype_id'		=> 0,
                'personnel'			=> 'Patient',
                'personnel_id'		=> $patient->id,
                'name'	            => $patient->person()->name,
                'details'			=> 'Added '.strtolower($row['certainty']).' diagnosis',
                'visit_id'          => $visit_location->visit_id,
                'visit_location_id' => $visit_location->visit()->id,
            )
        );
    }
}

echo '<script>setTimeout(function() {$(\'#PatientDiagnoses_container\').html(loading_circle).load(r+\'pages/clinical/patient/diagnoses/list.php?action=create&patient_id='.$patient->id.'&visit_id='.$visit->id.'&claim=1&r='.$app->root($r).'\', function() {app.initializeControls()})}, 1000);</script>';
echo '<script>setTimeout(function() {$(\'#PatientLaboratory_container\').html(loading_circle).load(\'../patient/laboratory/list.php?patient_id='.$patient->id.'&action=create\', function() {app.initializeControls()})}, 1500);</script>';
echo '<script>setTimeout(function() {$(\'#PatientPrescription_container\').html(loading_circle).load(\'../patient/prescription/list.php?patient_id='.$post->patient_id.'&action=create\', function() {app.initializeControls()})}, 2000);</script>';

?>