<?php 

$root           = '../../../../';
$submodule_code = 'PatientEvaluation';
require($root.'config/config.php');

$obj            = new PatientEvaluation();
$patient        = new Patient($post->patient_id);
$visit          = $patient->visit();
$visit_location = $patient->visitLocation();
$user->checkSubmoduleAccess($submodule, 'create');
$evaluation_id  = 0;

if(isset($post->values)) {
    $evaluation_types                = array_keys($post->values);

    // Validate
    foreach($evaluation_types as $evaluation_type) {
        $evaluation_obj              = $app->getCategoryByCode('PatientEvaluation', $evaluation_type);
        $evaluation_value            = $post->values[$evaluation_type];
        $evaluation_notes            = $post->notes[$evaluation_type];
        if($evaluation_obj->tagged('required') && !$evaluation_value) {
            die($app->renderAlertError(ucfirst(strtolower($evaluation_obj->name)).' is required'));
        }
    }

    // Insert data
    foreach($evaluation_types as $evaluation_type) {
        $evaluation_value            = $post->values[$evaluation_type];
        $evaluation_notes            = $post->notes[$evaluation_type];

        $create_data                = $obj->addCommonFields (
            array (
                'type'              => 'Evaluation',
                'subtype'           => $evaluation_type,
                'patient_id'        => $post->patient_id,
                'visit_id'          => $visit->id,
                'visit_location_id' => $visit_location->id,
                'request'           => 'Multiple',
                'request_id'        => strtotime($now),
                'name'              => $evaluation_type,
                'code'              => $evaluation_type,
                'value'             => $evaluation_value,
                'min'               => '',
                'max'               => '',
                'offset'            => '',
                'flag'              => '',
                'notes'             => $evaluation_notes
            )
        );

        $evaluation_id               = $obj->create($create_data);
    }

    if($evaluation_id) {

        // Log evaluation create event
        $user->timeline (
            array (
                'action'			=> 'Create',
                'subject'			=> 'Evaluation',
                'subject_id'		=> $evaluation_id,
                'subtype'			=> 'Multiple',
                'subtype_id'		=> strtotime($now),
                'personnel'			=> 'Patient',
                'personnel_id'		=> $patient->id,
                'name'	            => $patient->person()->name,
                'details'			=> 'Updated ABCDE evaluation',
                'visit_id'          => $visit_location->visit_id,
                'visit_location_id' => $visit_location->visit()->id,
            )
        );

        echo $app->renderAlertSuccess('Evaluation information saved successfully');
        echo '<script>setTimeout(function() {$(\'#PatientEvaluation_container\').html(loading_circle).load(\'../patient/evaluation/list.php?patient_id='.$patient->id.'"\', function() {app.initializeControls()})}, 1000);</script>';
    }

} else {
    $evaluation_code                 = strtolower($post->code);
    $create_data                    = $obj->addCommonFields (
        array (
            'type'                  => 'Evaluation',
            'subtype'               => $post->code,
            'patient_id'            => $post->patient_id,
            'visit_id'              => $visit->id,
            'visit_location_id'     => $visit_location->id,
            'request'               => 'Single',
            'request_id'            => 0,
            'name'                  => $post->code,
            'code'                  => $post->code,
            'value'                 => $post->value,
            'min'                   => '',
            'max'                   => '',
            'offset'                => '',
            'flag'                  => '',
            'notes'                 => $post->notes,
        )
    );
    
    if($evaluation_id = $obj->create($create_data)) {

        // Log evaluation create event
        $user->timeline (
            array (
                'action'			=> 'Create',
                'subject'			=> 'Evaluation',
                'subject_id'		=> $evaluation_id,
                'subtype'			=> 'Single',
                'subtype_id'		=> 0,
                'personnel'			=> 'Patient',
                'personnel_id'		=> $patient->id,
                'name'	            => $patient->person()->name,
                'details'			=> 'Updated ABCDE evaluation',
                'visit_id'          => $visit_location->visit_id,
                'visit_location_id' => $visit_location->visit()->id,
            )
        );

        echo '<script>
        $(\'#'.$evaluation_code.'_value\').html(\''.$post->value.' <div class="mt-3 text-muted fw-normal" style="font-size:0.8em">'.$post->notes.'</div>\');
        $(\'#flip_evaluation_'.$evaluation_code.'\').flip(false);
        </script>';
    }
}

?>