<?php 

class Notes extends Crud {
	function __construct($id=0) {		
		$this->table        = 'clinical_notes';
		$this->primary_key  = 'id';
		parent::__construct($id);
	}

	function attachments() {
		$attachments 		= array();
		$attachments_array 	= json_decode($this->attachments);
		if(!$attachments_array) $attachments_array = array();

        foreach($attachments_array as $attachment_id) {
            $attachments[] 	= new Attachment($attachment_id);
        }
        return $attachments;
    }

	function createdUser() {
		return new User($this->created_user_id);
	}

	function visitLocation() {
		return new VisitLocation($this->visit_location_id);
	}

	function renderAttachmentsPreview($attachment_root = '') {
		$html = '';
        foreach($this->attachments() as $attachment) {
			if($attachment->exists()) {
            	$html .= '<div class="col-lg-4 my-3"> '.$attachment->renderCardOpen($attachment_root).' </div>';
			}
        }
        return $html;
    }

	function renderReadCard($options=array()) {
		global $app, $root, $r, $user_id;

		$attachments_root	= isset($options['attachments_root']) ? $options['attachments_root'] : $root;

		$doctor         = new User($this->created_user_id);
		$person         = $doctor->person();
		$datetime       = $app->datetime($this->created_datetime);
		$visit_location = new VisitLocation($this->visit_location_id);
		$icon           = 'fa-solid fa-notes-medical';
		$color          = 'primary';

		echo '<div class="card mb-4 bg-'.$color.'" style="--bs-bg-opacity: .05;">
		<div class="card-body p-4">
		<div class="row">
		<div class="col-sm-8">
		<h5 class="mb-4"><span class="'.$icon.' fa-fw"></span> <span class="fw-bold">'.$this->title.'</span></h5>
		</div><div class="col-sm-4 text-end">';

		echo '<span class="form-check form-switch h5 d-inline float-start">
		<input class="form-check-input" type="checkbox" role="switch" id="'.$this->id.'" name="print['.$this->id.']" value="'.$this->id.'" style="margin-top:3px">
		<label class="form-check-label fw-bold text-muted" for="'.$this->id.'" style="cursor:pointer">Print</label>
		</span>';

		if(strtotime($this->created_datetime) > strtotime(' - '.$app->getClinicalEditTime()) && $this->created_user_id == $user_id) {
			echo '<div class="row gx-3">
			<div class="col-6 mb-3"><a class="btn btn-outline-primary w-100 ajax-link" data-target="PatientNotes_container" data-link="'.$r.'pages/clinical/patient/notes/form.php?patient_id='.$this->patient_id.'&notes_id='.$this->id.'&action=edit&r='.$app->root($r).'"> <span class="fa-regular fa-pencil fa-fw"></span> <!--<span class="d-none d-xl-inline">Edit</span>--> </a> </div>
			<div class="col-6 mb-3"><a class="btn btn-outline-danger w-100 ajax-link" data-target="PatientNotes_container" data-link="'.$r.'pages/clinical/patient/notes/form.php?patient_id='.$this->patient_id.'&notes_id='.$this->id.'&action=delete&r='.$app->root($r).'"> <span class="fa-regular fa-trash fa-fw"></span> <!--<span class="d-none d-xl-inline">Delete</span>--> </a> </div>
			</div>';
		} else {
			echo '<h5 class="fw-bold">'.$app->getAge($this->created_datetime,'','SHORTEST').' ago</h5>';
		}

		echo '</div>
		</div>
		'.$app->readText($this->notes).'
		<div class="row mt-3">'.$this->renderAttachmentsPreview($attachments_root).'</div>
		</div>
		
		<div class="px-4 pb-4">
		<div class="row text-secondary">
		<div class="col-xl-8 pt-2">
        <!-- <img src="" class="rounded-circle d-inline-block float-start mt-1 me-3" style="width:38px" /> -->
        <span class="fw-semibold">'.$person->name.'</span><br />'.$visit_location->location()->location()->name.'</div>
        <div class="col-xl-4 pt-2 text-end"><span class="fw-semibold">'.$datetime.'</span><br /> '.$app->getAge($this->created_datetime,'','SHORT').' ago </div>
		</div>
		</div>
		</div>';
	}
}

?>