<?php 

$root           = '../../../../';
$submodule_code = 'PatientNotes';
require($root.'config/config.php');

$obj            = new Notes();
$patient        = new Patient($post->patient_id);
$visit          = $patient->visit();
$visit_location = $patient->visitLocation();

if(!$visit->id) {
    die($app->renderAlertError('Patient visit number was not found please check in or admit patient again'));
}

$user->checkSubmoduleAccess($submodule, 'create');

if($post->action != 'delete') {

    if(!isset($post->ip)) {
        die($app->renderAlertError('Notes could not be processed'));
    }

    $template       = new Template($post->template_id);
    $inputs 		= $post->ip['clinical_notes_container'];
    $data			= array();

    foreach($inputs as $ip) {
        $keys       = array_keys($ip);
        if(is_array($ip[$keys[0]])) {
            $ip     = $main->reArrangeDataArray($ip);
        } else {
            $params = json_decode($ip['parameters']);
            $params->value = $app->cleanText($app->cleanJson($ip['value']));
            $ip     = $params;
        }
        array_push($data, $ip);
    }

    $raw_notes 	    = json_encode($data);
    $notes 		    = $app->textFromParameters($inputs);

    $count		    = 0;
    foreach($inputs as $note) {
        if(isset($note['parameters'])) {
            $parameters 	= json_decode($note['parameters']);

            // Find mapped fields
            /*
            for($count		= 1; $count < 13; $count++) {
                $field 		= 'f'.$count;
                if($parameters->db == $field) $$field = $main->cleanText($note['value']);
            }
            */
            
            // Check required fields
            if(strpos($parameters->attributes, 'required') > -1 && strlen($note['value']) < 2) {
                die($app->renderAlertError('<b>'.ucfirst(str_replace('_', ' ', $parameters->name)).'</b> is required and it should be written under &quot;'.ucfirst(str_replace('_', ' ', $parameters->name)).'&quot; section.'));
            }
        }
    }
}

if($post->action == 'edit') {

    $obj                    = new Notes($post->notes_id);
    
    $update_data            = array (
        'template'          => $raw_notes,
        'notes'             => $notes,
        'modified_user_id'  => $user_id,
        'modified_datetime' => $now
    );

    $updated                   = $obj->update($update_data);

    if($updated) {
        echo $app->renderAlertSuccess('Clinical notes updated successfully');

        // Attachments
        $attachment                 = new Attachment(); 
        $attachments                = array();
        if(isset($_FILES['attachments'])) { 
            $files                  = $app->reArrangeFilesArray($_FILES['attachments']);
            foreach($files as $notes_file) {
                $attachment_data    = array (
                    'file'          => $notes_file,
                    'section'       => 'Notes',
                    'section_id'    => $post->notes_id
                );

                if($attachment_id   = $attachment->uploadFile($attachment_data)) {
                    $attachments[]  = $attachment_id;
                    echo $app->renderAlertSuccess(ucfirst(strtolower($notes_file['name'])).' uploaded successfully');
                }
            }

            $obj->set('attachments', json_encode($attachments));
        }

        $user->timeline (
            array (
                'action'			=> 'Update',
                'subject'			=> 'Notes',
                'subject_id'		=> $get->notes_id,
                'subtype'			=> $template->code,
                'subtype_id'		=> 0,
                'personnel'			=> 'Patient',
                'personnel_id'		=> $patient->id,
                'name'	            => $patient->person()->name,
                'details'			=> 'Modified clinical notes',
                'visit_id'          => $visit_location->visit_id,
                'visit_location_id' => $visit_location->visit()->id,
            )
        );
    }
} elseif($post->action == 'delete') {

    $obj                    = new Notes($post->notes_id);
    $deleted                = $obj->delete();

    if($deleted) {
        echo $app->renderAlertSuccess('Clinical notes deleted successfully');

        $user->timeline (
            array (
                'action'			=> 'Delete',
                'subject'			=> 'Notes',
                'subject_id'		=> $get->notes_id,
                'subtype'			=> $obj->subtype,
                'subtype_id'		=> 0,
                'personnel'			=> 'Patient',
                'personnel_id'		=> $patient->id,
                'name'	            => $patient->person()->name,
                'details'			=> 'Deleted clinical notes',
                'visit_id'          => $visit_location->visit_id,
                'visit_location_id' => $visit_location->visit()->id,
            )
        );
    }
} else {
    $create_data                = $obj->addCommonFields (
        array (
            'type'              => $template->code,
            'subtype'           => $template->code,
            'patient_id'        => $post->patient_id,
            'visit_id'          => $visit->id,
            'visit_location_id' => $visit_location->id,
            'title'             => $template->name,
            'template'          => $raw_notes,
            'template_id'       => $template->id,
            'notes'             => $notes,
            'complaint'         => '',
            'history'           => '',
            'systems'           => '',
            'examination'       => '',
            'diagnoses'         => '',
            'plan'              => '',
            'disposition'       => '',
            'attachments'       => '',
            'doctor_id'         => '',
            'doctors'           => '',
            'flag'              => '',
            'tags'              => '',
        )
    );

    $notes_id                   = $obj->create($create_data);

    if($notes_id) {

        $notes                      = new Notes($notes_id);

        // Attachments
        $attachment                 = new Attachment(); 
        $attachments                = array();
        if(isset($_FILES['attachments'])) { 
            $files                  = $app->reArrangeFilesArray($_FILES['attachments']);
            foreach($files as $notes_file) {
                $attachment_data    = array (
                    'file'          => $notes_file,
                    'section'       => 'Notes',
                    'section_id'    => $notes->id
                );

                if($attachment_id   = $attachment->uploadFile($attachment_data)) {
                    $attachments[]  = $attachment_id;
                    echo $app->renderAlertSuccess(ucfirst(strtolower($notes_file['name'])).' uploaded successfully');
                }
            }
        }

        $notes->set('attachments', json_encode($attachments));

        echo $app->renderAlertSuccess('Clinical notes added successfully');

        // Log lifestyle create event
        $user->timeline (
            array (
                'action'			=> 'Create',
                'subject'			=> 'Notes',
                'subject_id'		=> $notes_id,
                'subtype'			=> $template->code,
                'subtype_id'		=> 0,
                'personnel'			=> 'Patient',
                'personnel_id'		=> $patient->id,
                'name'	            => $patient->person()->name,
                'details'			=> 'Added clinical notes',
                'visit_id'          => $visit_location->visit_id,
                'visit_location_id' => $visit_location->visit()->id,
            )
        );
    }
}

echo '<script>setTimeout(function() {$(\'#PatientNotes_container\').html(loading_circle).load(r+\'pages/clinical/patient/notes/list.php?action=create&patient_id='.$patient->id.'&r='.$app->root($r).'\', function() {app.initializeControls()})}, 1000);</script>';
echo '<script>setTimeout(function() {$(\'#PatientDiagnoses_container\').html(loading_circle).load(r+\'pages/clinical/patient/diagnoses/list.php?action=create&patient_id='.$patient->id.'&visit_id='.$visit->id.'&claim=1&r='.$app->root($r).'\', function() {app.initializeControls()})}, 1500);</script>';
echo '<script>setTimeout(function() {$(\'#PatientLaboratory_container\').html(loading_circle).load(\'../patient/laboratory/list.php?patient_id='.$patient->id.'&action=create\', function() {app.initializeControls()})}, 2000);</script>';
echo '<script>setTimeout(function() {$(\'#PatientRadiology_container\').html(loading_circle).load(\'../patient/radiology/list.php?patient_id='.$patient->id.'&action=create\', function() {app.initializeControls()})}, 2500);</script>';
echo '<script>setTimeout(function() {$(\'#PatientProcedure_container\').html(loading_circle).load(\'../patient/procedure/list.php?patient_id='.$patient->id.'&action=create\', function() {app.initializeControls()})}, 3000);</script>';

?>