<?php 

class Procedure extends Request {
	function __construct($id=0, $patient_id=0) {		
		$this->table        = 'clinical_procedures';
		$this->primary_key  = 'id';
		$this->items_table  = 'clinical_procedure_items';
		$this->title 		= 'Operations and procedures';
		$this->subtitle     = 'Procedure';
		$this->icon 		= 'fa-solid fa-bed-pulse';
		parent::__construct($id);
		if(!$id) {
			$this->type 	= 'Procedure';
			$this->subtype	= 'Procedure';
		}
	}

	function renderLinkInlineBill() {
        return '<a title="Confirm bill" class="bs-tooltip btn btn-outline-primary rounded-pill me-2" onclick="confirmProcedureBill('.$this->bill_id.')"> <span class="fa-solid fa-memo-circle-check"></span> </a>';
    }

    function renderLinkInlineReport($options=array()) {
		return '<a title="Enter report" class="bs-tooltip btn btn-outline-primary rounded-pill me-2" onclick="enterProcedureReport('.$this->id.')"> <span class="fa-solid fa-pen-to-square"></span> </a>';
    }

	function renderLinkInlineEdit($options=array()) {
		return '<a title="Edit report" class="bs-tooltip btn btn-outline-primary rounded-pill me-2" onclick="editProcedureReport('.$this->id.')"> <span class="fa-solid fa-pen"></span> </a>';
    }

	function renderLinkInlineAuthorize($options=array()) {
		return '<a title="Authorize report" class="bs-tooltip btn btn-outline-primary rounded-pill me-2" onclick="authorizeProcedureReport('.$this->id.')"> <span class="fa-solid fa-lock-open"></span> </a>';
    }

	function renderLinkInlinePrint($options=array()) {
		global $r;
		return '<a target="blank" href="'.$r.'pages/clinical/procedures/print.php?action=print&id='.$this->id.' title="Print report" class="bs-tooltip btn btn-outline-primary rounded-pill me-2"> <span class="fa-solid fa-print"></span> </a>';
    }

    function renderLinkEdit($options=array()) {
        $modal = isset($options['modal']) ? $options['modal'] : 'modal_lg'; 
        return '<a title="Edit report" class="remote-modal bs-tooltip btn btn-outline-primary rounded-pill me-2" data-bs-toggle="modal" data-bs-target="#'.$modal.'" data-bs-backdrop="static" data-bs-keyboard="false" tabindex="-1" href="form.php?action=edit&id='.$this->id.'" aria-label="Edit"><span class="fa-solid fa-pen-to-square"></span></a>';
    }


	function renderListBody() {
        global $app, $user, $root;

        $doctor         = new User($this->created_user_id);
        $person         = $doctor->person();
        $datetime       = $app->datetime($this->created_datetime);
        $visit_location = new VisitLocation($this->visit_location_id);
        $icon           = $this->icon;
        $color          = 'primary';
        $section        = $this->store()->location()->name;
		$obj            = $this;
		$scheme         = new Scheme($obj->scheme_id);
		$link_options   = array('id'=>$obj->id, 'name'=>$obj->name, 'modal'=>'modal_lg');
	
        $html           = '<div class="card mb-4 bg-'.$color.'" style="--bs-bg-opacity: .05;">
        <div class="card-body p-4">
        <div class="row">
        <div class="col-sm-8">
        <h5 class="mb-4"><span class="'.$icon.' fa-fw"></span> <span class="fw-bold">'.$section.'</span></h5>
        </div><div class="col-sm-4 text-end">';

		if($this->hasItemProgress('Requested') && $scheme->insurance()->paymode == 'CREDIT') {
			$html .= $this->renderLinkInlineBill();
		}
		
		if($this->hasPaidItem('Paid')) $html .= $this->renderLinkInlineReport($link_options);
		if($this->hasItemProgress('Reported') || $this->hasItemProgress('Authorized')) $html .= $this->renderLinkInlineEdit($link_options);
		if($this->hasItemProgress('Reported')) $html .= $this->renderLinkInlineAuthorize($link_options);
		if($this->hasItemProgress('Authorized')) $html .= $this->renderLinkInlinePrint();

		if(strtotime($this->created_datetime) > strtotime(' - '.$app->getClinicalEditTime()) && $this->created_user_id == $user->id) {
            $html .= $this->renderLinkDelete();
        } 

        //$html .= '<h5>'.$app->getAge($this->created_datetime,'','SHORT').' ago</h5>';

        $html .= '</div></div>';

        foreach($this->requestItems() as $request_item) {
            $html       .= $request_item->renderDataCard();
        }
        
        $html           .= '</div>
        <div class="px-4 pb-4">
        <div class="row text-secondary">
        <div class="col-xl-8 pt-2">
        <!-- <img src="" class="rounded-circle d-inline-block float-start mt-1 me-3" style="width:38px" /> -->
        <span class="fw-semibold">'.$person->name.'</span><br />'.$visit_location->location()->location()->name.'</div>
        <div class="col-xl-4 pt-2 text-end"><span class="fw-semibold">'.$datetime.'</span><br /> '.$app->getAge($this->created_datetime,'','SHORT').' ago </div>
        </div>
        </div>
        </div>';

        return $html;
    }
}

?>