<?php 

class Treatment extends Request {
	function __construct($id=0, $patient_id=0) {		
		$this->table        = 'clinical_treatments';
		$this->primary_key  = 'id';
		$this->items_table  = 'clinical_treatment_items';
		$this->title 		= 'Inpatient treatments';
		$this->subtitle     = 'Treatment';
		$this->icon 		= 'fa-solid fa-pills';
		parent::__construct($id);
		if(!$id) {
			$this->type 	= 'Treatment';
			$this->subtype	= 'Treatment';
		}
	}

    function renderListHeader($options=array()) {
        global $app, $r;
        $html   = parent::renderListHeader($options);

        $table_options      = array (
            'columns'       => array (
                array('name'=>'Date', 'css'=>'w-10'),
                array('name'=>'Drug name', 'css'=>''),
                array('name'=>'<span class="fa-solid fa-moon"></span><br />Night', 'css'=>'w-10 text-center'),
                array('name'=>'<span class="fa-regular fa-star"></span><br />Morning', 'css'=>'w-10 text-center'),
                array('name'=>'<span class="fa-regular fa-sun"></span><br />Afternoon', 'css'=>'w-10 text-center'),
                array('name'=>'<span class="fa-solid fa-star"></span><br />Evening', 'css'=>'w-10 text-center'),
                array('name'=>'<br />Total', 'css'=>'w-10 text-center'),
                array('name'=>'', 'css'=>'w-5 text-center')
            )
        );

        $html   .= $app->renderTableOpen($table_options);

        return $html;
    }

    function renderListFooter($options=array()) {
        global $app, $r;
        $html   = $app->renderTableClose();
        $html   .= parent::renderListHeader($options);
        return $html;
    }

    function renderListData($options=array()) {
        global $app, $user;
 
        $request_class  = $this->type;
        $patient_id     = $options['patient_id'];
        $sql            = isset($options['sql']) ? $options['sql'] : "SELECT id FROM $this->table WHERE patient_id = $patient_id AND status = 1 ORDER BY id ";
        $rows           = isset($options['rows']) ? $options['rows'] : $app->read($sql);

        $html           = $this->renderListHeader($options);
        $totals         = array();

        foreach($rows as $row) {
            $obj        = new Treatment($row->id);

            foreach($obj->items() as $item) {
                $dose               = $item->dose.' '.$item->dose_units;
                if(isset($totals[$item->item_id])) {
                    $totals[$item->item_id] += $item->dose;
                } else {
                    $totals[$item->item_id] = $item->dose;
                }

                $html               .= '<tr>
                <td><small>'.date('M d', strtotime($item->created_datetime)).'</small><div class="text-muted fw-bold">'.date('H:i', strtotime($item->created_datetime)).'</div></td>
                <td>'.$item->name.'</td>';

                for($current_time=0; $current_time<24; ($current_time = $current_time + 6)) {
                    $create_time    = date('H', strtotime($obj->created_datetime));
                    $limit_time     = $current_time + 6;
                    $hour           = date('H:i', strtotime($obj->created_datetime));
                    
                    $html .= '<td class="text-center">';
                    if($create_time >= $current_time) {
                        if($create_time < $limit_time)  $html .= '<a class="btn btn-outline-success btn-sm fw-bold rounded-pill w-100 text-nowrap">'.$dose.'</a>';
                    }
                    $html .= '</td>';
                }

                $html   .= '<td><a class="btn btn-success btn-sm fw-bold rounded-pill w-100 text-nowrap">'.number_format($totals[$item->item_id]).'</a></td>';
                
                if(strtotime($obj->created_datetime) > strtotime(' - '.$app->getClinicalEditTime()) && $obj->created_user_id == $user->id) {
                    $html .= '<td><a class="btn btn-outline-danger rounded-pill w-100 ajax-link btn-sm" data-target="Patient'.$obj->type.'_container" data-link="../patient/'.strtolower($obj->type).'/form.php?patient_id='.$obj->patient_id.'&request_id='.$obj->id.'&request_item_id='.$item->id.'&action=delete"> <span class="fa-regular fa-trash-alt fa-fw"></span> </a> </td>';
                } else {
                    $html .= '<td></td>';
                }

                $html   .= '</tr>';
            }
        }

        $html           .= $this->renderListFooter();

        return $html;
    }

    function renderRequestForm($options=array()) {
        global $app, $get;

        $patient            = new Patient($options['patient_id']);
        $visit_location     = $patient->visitLocation();
        $person             = $patient->person();
        $customer           = $patient->customer();
        $bill               = $this->bill();

        $form_options       = array (
            'form_id'       => 'ajax_form_'.strtolower($this->type),
            'form_action'   => '../patient/'.strtolower($this->type).'/process.php'
        );

        $html = $app->renderFormOpen($form_options);
        $html .= $app->renderInputHidden('patient_id', $patient->id);
        $html .= $app->renderInputHidden('visit_id', $visit_location->visit_id);
        $html .= $app->renderInputHidden('visit_location_id', $visit_location->id);
        $html .= '<div class="rounded p-4 pt-5 pb-4 mb-4 bg-primary" style="--bs-bg-opacity: .05;">';

        if($get->action == 'delete') {
            $html .= '<h3>Are you sure?</h3>';
            $html .= '<h5 class="my-4">'.$this->type.' request will be deleted</h5>';
            $html .= $app->renderInputHidden('action', 'delete');
            $html .= $app->renderInputHidden('request_id', $get->request_id);
            $html .= $app->renderInputHidden('request_item_id', $get->request_item_id);
        } else {
            $html .= $app->renderInputHidden('action', 'create');
            $html .= $app->renderInputHidden('trans_subtype', $this->type);
            $html .= $app->renderInputHidden('trans_customer_id', $customer->id);
            $html .= $app->renderInputHidden('trans_customer_name', $person->name);
            $html .= $app->renderInputHidden('customer_keyword', $person->name);
            $html .= $app->renderInputHidden('trans_customer_birthdate', $person->birthdate);
            $html .= $app->renderInputHidden('trans_customer_sex', $person->sex);
            $html .= $app->renderInputHidden('trans_customer_phone', $person->phone);

            $html .= $this->renderTransactionItemsEditable('Selected medications'); 

            $html .= $this->renderItemsControls();

            $html .= $bill->renderNotesControl('Requested');
        }

        $html .= '</div>';

        $html .= $app->renderFormClose($form_options);

        $html .= '<div class="row w-100 m-0 mt-3 px-0">
        <div class="col-12 col-lg-4"></div>
        <div class="col-12 col-lg-8 px-0">
        <div class="row pb-4 gx-3">
        <div class="col-6">
        <a class="btn btn-outline-primary w-100 ajax-link" data-target="Patient'.$this->type.'_container" data-link="../patient/'.strtolower($this->type).'/list.php?patient_id='.$patient->id.'&action=create"> <span class="fa fa-arrow-left fa-fw"></span> Cancel </a>
        </div><div class="col-6">
        <button id="ajax_form_'.strtolower($this->type).'_submit_button" type="button" class="btn btn-primary w-100"> <span class="fa fa-check fa-fw"></span> Submit </button>
        </div></div>
        </div></div><hr />

        <script>transaction.trans_items = [];</script>';

        return $html;
    }

	function renderTransactionItemsEditable($title='Items in your cart') {
		global $app;
        
		$html = '<div class="trans-container">
		<h5 class="title my-3"><span class="fa-regular fa-cart-circle-check fa-fw"></span> &nbsp;'.$title.':</h5>
        <div class="card p-3 my-3 bg-primary" style="--bs-bg-opacity: .05;">
        <div class="table-responsive">
		<table class="table table-hover align-middle bg-transparent">
		<thead class="h5">
		<tr>
		<th class="ps-0">Medication</th>
		<th class="w-15">Dosage</th>
		<th class="w-15">Route</th>
		<th class="" style="width:10px"> <span class="fa-solid fa-ellipsis-h"></span></th>
		</tr>
		</thead>
		<tbody id="trans_items_container">

        </tbody>
		</table>
		</div>
		</div>
        </div>';

        $html .= $app->renderInputHidden('trans_cost', '');

		return $html;
	}


	function renderItemsControls() {
		global $app;

        $html = $app->renderInputHidden('trans_items');

        $html .= $this->renderInsuranceSchemesJson();

		$times 	= array();
		for($time = 1; $time < 24; $time++) $times[] = $time;
		
		$days 	= array();
		for($day = 1; $day < 91; $day++) $days[] = $day;

		$options 					= array();

		$sql						= "SELECT 
		id
		FROM inventory_items
		WHERE subtype 	= 'Pharmacy'
		AND status 		= 1
		ORDER BY name   ";

		foreach($app->read($sql) as $row) {
			$item					= new Item($row->id);
			$item->pricelists 		= $item->getPricelists();
			$item->packs 			= $item->getPacks();
			$items[$row->id]		= $item;
			$options[]				= array (
				'name'				=> $item->name,
				'value'				=> $item->id
			);
		}

		$html .= $app->renderInputHidden('trans_treatment', 'TreatmentChart');

		$html .= '<div class="row gx-3 mb-4">
        <div class="col-12 mt-3">
        <div class="">
        <h5 class="my-3"> <span class="fa-regular fa-cart-arrow-down fa-fw"></span>&nbsp; Choose medications: </h5>
        <div class="row gx-3">
        <div class="my-2 mb-3 col-12 col-lg-4">
        '.$app->renderInputSelect('type', $this->type, 'Filter categories', 'line-controlled select-search default-all', 'data-style="btn-outline-secondary"', array()).'
        </div><div class="my-2 mb-3 col-12 col-lg-8">
        '.$app->renderInputSelect('trans_item_id', '', 'Choose item', 'line-controlled select-search default-none', 'data-style="btn-outline-secondary"', $options).'
        </div>

		<div class="col-2 d-none" id="trans_item_uom_container"> <div class="mb-2">Units</div>
        '.$app->renderInputSelect('trans_item_uom', '', 'Units', 'select-search', 'data-style="btn-outline-secondary"', 'settings_categories', 'code', 'name', 'type="PrescriptionUnits"').'
        </div>

		<div id="trans_route_container" class="mb-3 col-4"> <div class="mb-2">Route</div>
        '.$app->renderInputSelect('trans_item_route', 'Oral', '', 'select-search', 'data-style="btn-outline-secondary"', 'settings_categories', 'code', 'name', 'type="PrescriptionRoute"').'
        </div><div id="trans_dose_container" class="mb-3 col-2"> <div class="mb-2">Dose</div>
        '.$app->renderInputNumber('trans_item_dose', '0', 'Dose', 'border border-secondary', 'onchange="transaction.calculatePrescription()"').'
        </div><div class="mb-3 col-2"> <div class="mb-2">Units</div>
        '.$app->renderInputSelect('trans_item_units', 'Each', '', 'select-search', 'data-style="btn-outline-secondary"', 'settings_categories', 'code', 'name', 'type="PrescriptionUnits"').'
        </div><div class="mb-3 col-2 d-none" id="trans_item_times_container"> <div class="mb-2">Times</div>
        '.$app->renderInputSelect('trans_item_times', '1', '', 'select-search', 'data-style="btn-outline-secondary" onchange="transaction.calculatePrescription()"', $times).'
        </div><div class="mb-3 col-2 d-none" id="trans_item_days_container"> <div class="mb-2">Days</div>
        '.$app->renderInputSelect('trans_item_days', '1', '', 'select-search', 'data-style="btn-outline-secondary" onchange="transaction.calculatePrescription()"', $days).'
        </div><div class="mb-3 col-2 d-none"> <div class="mb-2">Total quantity</div>
        '.$app->renderInputNumber('trans_item_quantity', '', 'Total', 'border border-1 border-secondary').'
        </div>
		
		<div class="my-2 mb-3 col-12 col-lg-4 d-none">
        '.$app->renderInputMoney('trans_item_unit_price', '', 'Unit price', 'line-controlled text-end border border-1 border-secondary readonly').'
        </div><div class="my-2 mb-3 col-12 col-lg-4 d-none">
        '.$app->renderInputText('trans_item_details', '', 'Instructions', 'line-controlled border border-1 border-secondary').'
        </div><div class="my-2 mb-3 col-12 col-lg-4"><div class=""><br /></div>
        <a id="trans_item_add_button" class="btn btn-outline-primary w-100" onclick="transaction.addItemToCart()"> <span class="fa-solid fa-check fa-fw"></span> Add to chart </a>
        </div></div></div></div></div>';

		$html .= '<script> transaction.store_items = '.$app->array2json($items).' </script>';

		return $html;
	}


    function renderCreateConfirmation($submodule, $post) {
        global $app, $user;
 
        $user->checkSubmoduleAccess($submodule, 'create');

		$bill_id        		= 0;

        $customer       		= new Customer($post->trans_customer_id);
        $items          		= $app->uncleanText($post->trans_items);
        $person         		= $customer->person();
        $scheme         		= $customer->scheme();
		$pricelist      		= $scheme->pricelist();
        $insurance      		= $scheme->insurance();
        $visit          		= new Visit($post->visit_id);
		$folio          		= $visit->folio();
        $subtype                = 'Treatment';  
		
        $create_data            = array (
			'subtype'           => $subtype,
			'parent'            => '',
			'store_id'          => 0,
			
			'insurance_id'      => $insurance->id,
			'scheme_id'        	=> $scheme->id,
			'bill_id'           => $bill_id,

			'patient_id'        => $post->patient_id,
			'personnel'         => 'Patient',
			'personnel_id'      => $post->patient_id,
			'code'              => $customer->code,
			'name'              => $person->name,
			'birthdate'         => $person->birthdate,
			'sex'               => $person->sex,
			'phone'             => $person->phone,
			'email'             => $person->email,

			'pricelist'         => $pricelist->code,
			'pricelist_id'      => $pricelist->id,
			'insurance'         => $insurance->code,
			'insurance_id'      => $insurance->id,
			'scheme'            => $scheme->code,
			'scheme_id'         => $scheme->id,

			'folio_id'          => $folio->id,
			'folio_no'          => $folio->folio_no,

			'visit_id'          => $visit->id,
			'visit_location_id' => $post->visit_location_id,

			'requested_notes'   => $post->trans_requested_notes,
			'items'             => $items,
			'cost'              => 0,
			'subtotal'          => 0
		);

        $request_id         	= $this->createWithMinimumData($create_data);

        if($request_id) {
            // Log request create event
            $user->timeline (
                array (
                    'action'			=> 'Create',
                    'subject'			=> $this->type,
                    'subject_id'		=> $request_id,
                    'subtype'			=> '',
                    'subtype_id'		=> 0,
                    'personnel'			=> 'Patient',
                    'personnel_id'		=> $post->patient_id,
                    'name'	            => $customer->person()->name,
                    'details'			=> 'Added '.strtolower($this->title),
                    'visit_id'          => $visit->id,
                    'visit_location_id' => $post->visit_location_id,
                )
            );

            return $app->renderAlertSuccess($this->type.' request created successfully');
        }
    }

}

?>