<?php 

$root               = '../../../';
$submodule_code     = 'Pharmacy';
require($root.'config/config.php');

$user_stores        = $user->getStoreIdsWithAccess('read-sales', 'pharmacy');
$user_stores_string = implode(',', $user_stores);
if(!$user_stores) {
    die($app->renderAlertEmpty('You do not have read access to any pharmacy', ''));
}

$keyword            = $get->keyword;

$start_date         = strlen($get->start_date) ? $app->morning($get->start_date) : $app->morning($app->monthStart());
$end_date           = strlen($get->end_date) ? $app->evening($get->end_date) : $app->evening($app->monthEnd());
$date_filter        = " AND billing_bills.created_datetime BETWEEN '$start_date' AND '$end_date' ";
$store_filter       = $get->store ? " AND billing_bills.store_id = $get->store " : " AND billing_bills.store_id IN ($user_stores_string) ";
$sponsor_filter     = $get->sponsor ? " AND billing_bills.insurance_id = $get->sponsor " : " ";
$type_filter        = " AND billing_bills.subtype IN ('Pharmacy', 'Consumable', 'Prescription') AND subtype_id > 0 ";
$join_filter        = " ";
if($get->progress) {
    $progress_filter    = "JOIN billing_bill_items ON billing_bill_items.parent_id = billing_bills.id AND billing_bill_items.progress = '$get->progress' ";
} else {
    $progress_filter    = " ";   
}

if(strlen($keyword)) {
    if(is_numeric($keyword)) {
        $sql                   = "SELECT 
        billing_bills.id, billing_bills.subtype, billing_bills.subtype_id
        FROM billing_bills
        $progress_filter
        WHERE billing_bills.status  = 1
        $type_filter
        $date_filter
        $store_filter
        $sponsor_filter
        AND (
            billing_bills.id        = $keyword
            OR billing_bills.code   = '$keyword'
            OR billing_bills.card_no = '$keyword'
        )
        ORDER BY id DESC ";
    } else {
        $sql                        = "SELECT 
        billing_bills.id, billing_bills.subtype, billing_bills.subtype_id
        FROM billing_bills
        $progress_filter
        WHERE billing_bills.status  = 1
        $type_filter
        $date_filter
        $store_filter
        $sponsor_filter
        AND billing_bills.name LIKE '%$keyword%'
        ORDER BY id DESC ";
    }
} else {
    $sql    = "SELECT 
    billing_bills.id, billing_bills.subtype, billing_bills.subtype_id
    FROM billing_bills
    $progress_filter
    WHERE billing_bills.status      = 1
    $type_filter
    $date_filter
    $store_filter
    $sponsor_filter
    ORDER BY id DESC ";
}

$data               = $app->pagenate($app->read($sql));

$options            = array (
    'columns'       => array (
        array('name'=>'Date', 'css'=>'w-10'),
        array('name'=>'Source', 'css'=>'w-15'),
        array('name'=>'Patient name', 'css'=>''),
        array('name'=>'Sponsor', 'css'=>'w-15'),
        array('name'=>'Urgency', 'css'=>'w-10'),
        array('name'=>'Status', 'css'=>'w-15 text-center'),
        array('name'=>'Options', 'css'=>'w-15 text-end')
    ),
    'css'           => 'bs-table'
);

echo $app->renderTableOpen($options);

foreach($data->rows as $row) {
    $subtype        = $row->subtype;
    $subtype_id     = $row->subtype_id;
    $obj            = new $subtype($subtype_id);
    $has_paid_item  = $obj->hasPaidItem('Paid');

    $scheme         = new Scheme($obj->scheme_id);
    $link_options   = array('id'=>$row->id, 'name'=>$obj->name, 'modal'=>'modal_lg');

    echo '<tr>
    <td class="fw-semibold text-muted">'.$app->datetime($obj->created_datetime).'</td>
    <td class="text-truncate"><div class="mb-1 fw-semibold">'.$obj->subtype.'</div><div class="text-muted">'.$obj->store()->location()->name.'</div></td>
    <td><div class="mb-1">'.$app->renderLinkOpen($link_options).'</div> <div>File '.$app->formatCode($obj->code).' &rarr; '.$app->age($obj->birthdate).' '.$obj->sex.'</div></td>
    <td><div class="mb-1 fw-semibold">'.$scheme->insurance()->code.'</div> <div class="auto-truncate">'.$app->shortname($scheme->name, 15).'</div></td>
    <td>'.$obj->renderButtonUrgency().'</td>
    <td class="text-nowrap">'.$obj->renderButtonProgress($link_options).'</td>
    <td class="text-nowrap text-center">';

    if($obj->hasItemProgress('Requested')) echo $obj->renderLinkAuthorize($link_options);
    if($obj->hasItemProgress('Authorized')) {
        if($scheme->insurance()->paymode == 'CREDIT') {
            echo $obj->renderLinkBill();
        } else {
            echo '<a class="btn btn-outline-danger rounded-pill px-3">Not billed</a>';
        }
    }

    if($obj->hasPaidItem('Paid')) {
        echo $obj->renderLinkDispense($link_options);
    } elseif($obj->hasItemProgress('Billed')) {
        echo '<a class="btn btn-outline-danger rounded-pill px-3">Not paid</a>';
    }

    echo '</td>
    </tr>';
}

echo $app->renderTableClose();

echo $data->pagination;

?>