<?php 

$root               = '../../../';
$submodule_code     = 'Radiology';
require($root.'config/config.php');

$user_stores        = $user->getStoreIdsWithAccess('read-sales', 'radiology');
$user_stores_string = implode(',', $user_stores);
if(!$user_stores) {
    die($app->renderAlertEmpty('You do not have read access to any radiology', ''));
}

$keyword            = $get->keyword;

$start_date         = strlen($get->start_date) ? $app->morning($get->start_date) : $app->morning($app->monthStart());
$end_date           = strlen($get->end_date) ? $app->evening($get->end_date) : $app->evening($app->monthEnd());
$date_filter        = " AND billing_bills.created_datetime BETWEEN '$start_date' AND '$end_date' ";
$store_filter       = $get->store ? " AND billing_bills.store_id = $get->store " : " AND billing_bills.store_id IN ($user_stores_string) ";
$sponsor_filter     = $get->sponsor ? " AND billing_bills.insurance_id = $get->sponsor " : " ";
$type_filter        = " AND billing_bills.subtype  = 'Radiology' ";
$join_filter        = " ";
if($get->progress) {
    $progress_filter    = "JOIN clinical_radiology_items ON clinical_radiology_items.parent_id = clinical_radiology.id AND clinical_radiology_items.progress = '$get->progress' ";
} else {
    $progress_filter    = " ";   
}

if(strlen($keyword)) {
    if(is_numeric($keyword)) {
        $sql                   = "SELECT 
        billing_bills.id, billing_bills.subtype, billing_bills.subtype_id
        FROM billing_bills
        JOIN clinical_radiology ON clinical_radiology.id = billing_bills.subtype_id
        $progress_filter
        WHERE billing_bills.status  = 1
        $type_filter
        $date_filter
        $store_filter
        $sponsor_filter
        AND (
            billing_bills.id        = $keyword
            OR billing_bills.code   = '$keyword'
            OR billing_bills.card_no = '$keyword'
        )
        ORDER BY id DESC ";
    } else {
        $sql                        = "SELECT 
        billing_bills.id, billing_bills.subtype, billing_bills.subtype_id
        FROM billing_bills
        JOIN clinical_radiology ON clinical_radiology.id = billing_bills.subtype_id
        $progress_filter
        WHERE billing_bills.status  = 1
        $type_filter
        $date_filter
        $store_filter
        $sponsor_filter
        AND billing_bills.name LIKE '%$keyword%'
        ORDER BY id DESC ";
    }
} else {
    $sql    = "SELECT 
    billing_bills.id, billing_bills.subtype, billing_bills.subtype_id
    FROM billing_bills
    JOIN clinical_radiology ON clinical_radiology.id = billing_bills.subtype_id
    $progress_filter
    WHERE billing_bills.status      = 1
    $type_filter
    $date_filter
    $store_filter
    $sponsor_filter
    ORDER BY id DESC ";
}

$data               = $app->pagenate($app->read($sql));

$options            = array (
    'columns'       => array (
        array('name'=>'Date', 'css'=>'w-10'),
        array('name'=>'Source', 'css'=>'w-15'),
        array('name'=>'Patient name', 'css'=>''),
        array('name'=>'Sponsor', 'css'=>'w-15'),
        array('name'=>'Urgency', 'css'=>'w-10'),
        array('name'=>'Status', 'css'=>'w-12 text-center'),
        array('name'=>'PACS', 'css'=>'w-5 text-center'),
        array('name'=>'Options', 'css'=>'w-15 text-end')
    ),
    'css'           => 'bs-table'
);

echo $app->renderTableOpen($options);

foreach($data->rows as $row) {
    $obj            = new Radiology($row->subtype_id);
    $scheme         = new Scheme($obj->scheme_id);
    $link_options   = array('id'=>$obj->id, 'name'=>$obj->name, 'modal'=>'modal_lg');

    echo '<tr>
    <td class="fw-semibold text-muted">'.$app->datetime($obj->created_datetime).'</td>
    <td class="text-truncate"><div class="mb-1 fw-semibold">'.$obj->subtype.'</div><div class="text-muted">'.$obj->store()->location()->name.'</div></td>
    <td><div class="mb-1">'.$app->renderLinkOpen($link_options).'</div> <div>File '.$app->formatCode($obj->code).' &rarr; '.$app->age($obj->birthdate).' '.$obj->sex.'</div></td>
    <td><div class="mb-1 fw-semibold">'.$scheme->insurance()->code.'</div> <div class="auto-truncate">'.$app->shortname($scheme->name, 15).'</div></td>
    <td>'.$obj->renderButtonUrgency().'</td>
    <td class="text-nowrap">'.$obj->renderButtonProgress($link_options).'</td>';

    echo '<td>
    <a title="Dicom" class="remote-modal bs-tooltip btn btn-outline-primary rounded-pill py-2 me-2" data-bs-toggle="modal" data-bs-target="#modal_fs" data-bs-backdrop="static" data-bs-keyboard="false" tabindex="-1" href="dicom_form.php?action=read&patient_id='.$obj->patient_id.'" aria-label="Report"><span class="fa-solid fa-images-user"></span></a>
    </td>';
    
    echo '<td class="text-nowrap text-center">';

    if($obj->hasItemProgress('Requested') && $scheme->insurance()->paymode == 'CREDIT') {
        echo $obj->renderLinkBill();
    }
    
    if($obj->hasPaidItem('Paid')) echo $obj->renderLinkreport($link_options);
    if($obj->hasItemProgress('Reported') || $obj->hasItemProgress('Authorized')) echo $obj->renderLinkEdit($link_options);
    if($obj->hasItemProgress('Reported')) echo $obj->renderLinkAuthorize($link_options);
    if($obj->hasItemProgress('Authorized')) echo $obj->renderLinkPrint();

    /*
    if($obj->invoice_id) {
        $invoice = new Invoice($obj->invoice_id);
        if($invoice->payment) {
            if($obj->hasPaidItem('Paid')) echo $obj->renderLinkreport($link_options);
            if($obj->hasItemProgress('Reported') || $obj->hasItemProgress('Authorized')) echo $obj->renderLinkEdit($link_options);
            if($obj->hasItemProgress('Reported')) echo $obj->renderLinkAuthorize($link_options);
            if($obj->hasItemProgress('Authorized')) echo $obj->renderLinkPrint();
        } else {
            echo '<a class="btn btn-outline-danger rounded-pill px-3">Not paid</a>';
        }  
    } else {
        if($scheme->insurance()->paymode == 'CREDIT') {
            echo $obj->renderLinkBill();
        } else {
            echo '<a class="btn btn-outline-danger rounded-pill px-3">Not billed</a>';
        }
    }
    */

    echo '</td>
    </tr>';
}

echo $app->renderTableClose();

echo $data->pagination;

?>