<?php 

$root               	= '../../../../';
$submodule_code     	= 'PatientsCensus';
require($root.'config/config.php');

function ageGroup($days) {
    if($days < 29) return '1-28 Days';
    if($days < 365) return '1-12 Months';
    if($days < 1825) return '1-5 Years';
    if($days < 21900) return '5-60 Years';
    return '60 Years+';
}

$start_date             = strlen($get->start_date) ? $app->morning($get->start_date) : $app->morning($app->monthStart());
$end_date               = strlen($get->end_date) ? $app->evening($get->end_date) : $app->evening($app->monthEnd());
$date_filter            = "AND clinical_visits.checkin_datetime > '$start_date' AND clinical_visits.checkout_datetime < '$end_date' ";

$gender_keys            = array ('M' => 0, 'F' => 0, 'T'=> 0);
$agegroup_keys          = array ('1-28 Days', '1-12 Months', '1-5 Years', '5-60 Years', '60 Years+', 'Total');
foreach($agegroup_keys as $agegroup_key) {
    $agegroups[$agegroup_key]     = $gender_keys;
}

$clinics                = array();
$clinic_names           = array();
$clinics_sql            = "SELECT clinical_clinics.id, location_locations.name FROM location_locations JOIN clinical_clinics ON clinical_clinics.location_id = location_locations.id ";
foreach($app->read($clinics_sql) as $row) {
    $clinics[$row->name]    = $agegroups;
    $clinic_names[$row->id] = $row->name;
}

$wards                  = array();
$ward_names             = array();
$wards_sql              = "SELECT clinical_wards.id, location_locations.name FROM location_locations JOIN clinical_wards ON clinical_wards.location_id = location_locations.id ";
foreach($app->read($wards_sql) as $row) {
    $wards[$row->name]      = $agegroups;
    $ward_names[$row->id]   = $row->name;
}

$visits                 = array (
    'OPD + IPD'         => $agegroups
);

$discharge_names        = array();
$discharges             = array(
    'Automatically discharged after 24 hours'   => $agegroups,
    'Transfered to another location'            => $agegroups
);

$discharge_names['Auto']        = 'Automatically discharged after 24 hours';
$discharge_names['Transfer']    = 'Transfered to another location';

$discharge_sql          = "SELECT code, name FROM settings_categories WHERE type = 'DispositionType' AND status = 1 ";
foreach($app->read($discharge_sql) as $row) {
    $discharges[$row->name]         = $agegroups;
    $discharge_names[$row->code]    = $row->name;
}

$referrals              = array (
    'Referrals in'      => $agegroups,
    'Referrals out'     => $agegroups
);

$tourism                = array(
    'Non Tanzanians'    => $agegroups
);

$census_data            = array(
    'Outpatient'        => $clinics,
    'Inpatient'         => $wards,
    'Total'             => $visits,
    'Discharges'        => $discharges,
    'Referrals'         => $referrals,
    'Medical tourism'   => $tourism
);

echo $app->renderReportNavigation (
	array (
		'period'		=> 'Month',
		'section_id'	=> 'summary',
		'section_title'	=> '',
		'current_date'	=> $start_date,
		'current_period'=> date('M Y', strtotime($start_date))
	)
);

echo '<div class="table-responsive">
<table class="table table-hover table-bordered table-striped bs-table">
<thead>
<tr>
<th class="h5 fw-bold text-center">Category</th>
<th class="h5 fw-bold text-center">Subcategory</th>';

foreach(array_keys($agegroups) as $age) {
    echo '<th class="h5 fw-bold text-center" colspan="3">'.$age.'</th>';
}

echo '</tr>
<tr>
<th></th>
<th class="h5 fw-bold" style="min-width:5%; width:5%"></th>';

foreach(array_keys($agegroups) as $age) {
    foreach(array_keys($gender_keys) as $gender) {
        echo '<th class="h5 fw-bold text-center" style="min-width:60px; width:60px">'.$gender.'</th>';
    }
}

echo '</tr>
</thead>
<tbody>';

// Attendance
$visits_sql         = "SELECT 
clinical_visit_locations.care_type,
clinical_visit_locations.clinic_id,
clinical_visit_locations.ward_id,
DATEDIFF(clinical_visits.checkin_datetime, personnel_persons.birthdate) `days`,
DATEDIFF(clinical_visits.checkin_datetime, clinical_patients.created_datetime) `revisit`,
personnel_persons.sex,
clinical_visits.checkout_type,
clinical_visits.referral_in,
clinical_visits.referral_out,
clinical_visit_locations.admitted,
personnel_persons.country
FROM clinical_visit_locations
JOIN clinical_visits ON clinical_visits.id = clinical_visit_locations.visit_id
JOIN clinical_patients ON clinical_patients.id = clinical_visits.patient_id
JOIN personnel_persons ON personnel_persons.id = clinical_patients.person_id
$date_filter ";

foreach($app->query($visits_sql) as $row) {
    $sex                = substr($row->sex, 0, 1);
    foreach($agegroup_keys as $agegroup_key) {
        $agegroup       = ageGroup($row->days);
        if($agegroup_key == $agegroup) {
            
            // OPD/IPD
            if($row->ward_id) {
                $ward_name  = $ward_names[$row->ward_id];

                $census_data['Inpatient'][$ward_name][$agegroup][$sex]++;
                $census_data['Inpatient'][$ward_name][$agegroup]['T']++;
                $census_data['Inpatient'][$ward_name]['Total'][$sex]++;
                $census_data['Inpatient'][$ward_name]['Total']['T']++;
            } elseif($row->clinic_id) {
                $clinic_name  = $clinic_names[$row->clinic_id];

                $census_data['Outpatient'][$clinic_name][$agegroup][$sex]++;
                $census_data['Outpatient'][$clinic_name][$agegroup]['T']++;
                $census_data['Outpatient'][$clinic_name]['Total'][$sex]++;
                $census_data['Outpatient'][$clinic_name]['Total']['T']++;
            }
            $census_data['Total']['OPD + IPD'][$agegroup][$sex]++;
            $census_data['Total']['OPD + IPD'][$agegroup]['T']++;
            $census_data['Total']['OPD + IPD']['Total'][$sex]++;
            $census_data['Total']['OPD + IPD']['Total']['T']++;

            // Discharges

            if(strlen($row->checkout_type) < 3) $row->checkout_type = 'Auto';
            if($row->checkout_type == 'Referral') $row->checkout_type = 'Domestic';
            $discharge_name             = $discharge_names[$row->checkout_type];
            if(!$row->admitted) {
                $census_data['Discharges'][$discharge_name][$agegroup][$sex]++;
                $census_data['Discharges'][$discharge_name][$agegroup]['T']++;
                $census_data['Discharges'][$discharge_name]['Total'][$sex]++;
                $census_data['Discharges'][$discharge_name]['Total']['T']++;
            }

            // Referrals
            if($row->referral_in) {
                $census_data['Referrals']['Referrals in'][$agegroup][$sex]++;
                $census_data['Referrals']['Referrals in'][$agegroup]['T']++;
                $census_data['Referrals']['Referrals in']['Total'][$sex]++;
                $census_data['Referrals']['Referrals in']['Total']['T']++;
            } elseif($row->referral_out) {
                $census_data['Referrals']['Referrals out'][$agegroup][$sex]++;
                $census_data['Referrals']['Referrals out'][$agegroup]['T']++;
                $census_data['Referrals']['Referrals out']['Total'][$sex]++;
                $census_data['Referrals']['Referrals out']['Total']['T']++;
            }

            // Medical tourism
            if($row->country != 'TZ') {
                $country    = new Country($row->country);
                if(isset($census_data['Medical tourism'][$country->name])) {
                    $census_data['Medical tourism'][$country->name][$agegroup][$sex]++;
                    $census_data['Medical tourism'][$country->name][$agegroup]['T']++;
                    $census_data['Medical tourism'][$country->name]['Total'][$sex]++;
                    $census_data['Medical tourism'][$country->name]['Total']['T']++;
                } else {
                    foreach(array_keys($agegroups) as $agegroup_key_inside) {
                        $census_data['Medical tourism'][$country->name] = $agegroups;
                    }
                    $census_data['Medical tourism'][$country->name][$agegroup][$sex]++;
                    $census_data['Medical tourism'][$country->name][$agegroup]['T']++;
                    $census_data['Medical tourism'][$country->name]['Total'][$sex]++;
                    $census_data['Medical tourism'][$country->name]['Total']['T']++;
                }
                $census_data['Medical tourism']['Non Tanzanians'][$agegroup][$sex]++;
                $census_data['Medical tourism']['Non Tanzanians'][$agegroup]['T']++;
                $census_data['Medical tourism']['Non Tanzanians']['Total'][$sex]++;
                $census_data['Medical tourism']['Non Tanzanians']['Total']['T']++;
            }
        }
    }
}

foreach(array_keys($census_data) as $category) {
    $count = 0;
    foreach(array_keys($census_data[$category]) as $subcategory) {
        echo '<tr><td class="fw-bold" style="width:200px; min-width:200px">';
        if(!$count++) echo $category;
        echo '</td>';
        echo '<td class="text-start" style="width:200px; min-width:200px">'.$subcategory.'</td>';
        foreach(array_keys($census_data[$category][$subcategory]) as $age) {
            foreach(array_keys($gender_keys) as $gender) {
                echo '<td class="text-center" style="width:60px; min-width:60px">'.number_format($census_data[$category][$subcategory][$age][$gender]).'</td>';
            }
        }
        echo '</tr>';
    }
}

echo '</tbody>
</table>
</div>';

?>