<?php 

$root               	= '../../../../';
$submodule_code     	= 'PatientVisits';
require($root.'config/config.php');

$start_date             = strlen($get->start_date) ? $app->morning($get->start_date) : $app->morning($app->monthStart());
$end_date               = strlen($get->end_date) ? $app->evening($get->end_date) : $app->evening($app->monthEnd());
$date_filter            = "AND clinical_visits.created_datetime BETWEEN '$start_date' AND '$end_date'";
$care_filter            = $get->care_type ? "AND clinical_visits.care_type = '$get->care_type'" : "";
$discharge_filter       = $get->discharge_type ? "AND clinical_visits.checkout_type = '$get->discharge_type'" : "";
$insurance_filter       = $get->insurance_id ? "AND clinical_visits.insurance_id = $get->insurance_id" : "";
$scheme_filter          = $get->scheme_id ? "AND clinical_visits.scheme_id = $get->scheme_id" : "";

$location_join          = "";
$location_filter        = "";
if($get->location_id) {
    if($get->care_type == 'Inpatient') {
        $location_join      = "JOIN clinical_visit_locations ON clinical_visit_locations.visit_id = clinical_visits.id";
        $location_filter    = "AND clinical_visit_locations.ward_id = $get->location_id";
    } else {
        $location_join      = "JOIN clinical_visit_locations ON clinical_visit_locations.visit_id = clinical_visits.id";
        $location_filter    = "AND clinical_visit_locations.clinic_id = $get->location_id";
    }
}

$diagnoses_join          = "";
$diagnoses_filter        = "";
if($get->diagnosis) {
    $diagnoses_join      = "JOIN clinical_diagnoses ON clinical_diagnoses.visit_id = clinical_visits.id AND clinical_diagnoses.certainty = 'Confirmed'";
    $diagnoses_filter    = "AND (clinical_diagnoses.code LIKE '%$get->diagnosis%' OR clinical_diagnoses.name LIKE '%$get->diagnosis%')";
}

$laboratory_join          = "";
$laboratory_filter        = "";
if($get->laboratory) {
    $laboratory_join      = "JOIN clinical_laboratory ON clinical_laboratory.visit_id = clinical_visits.id JOIN clinical_laboratory_items ON clinical_laboratory_items.parent_id = clinical_laboratory.id AND clinical_laboratory_items.payment=1";
    $laboratory_filter    = "AND clinical_laboratory_items.name LIKE '%$get->laboratory%'";
}

$radiology_join          = "";
$radiology_filter        = "";
if($get->radiology) {
    $radiology_join      = "JOIN clinical_radiology ON clinical_radiology.visit_id = clinical_visits.id JOIN clinical_radiology_items ON clinical_radiology_items.parent_id = clinical_radiology.id AND clinical_radiology_items.payment=1";
    $radiology_filter    = "AND clinical_radiology_items.name LIKE '%$get->radiology%'";
}

$procedure_join          = "";
$procedure_filter        = "";
if($get->procedure) {
    $procedure_join      = "JOIN clinical_procedures ON clinical_procedures.visit_id = clinical_visits.id JOIN clinical_procedure_items ON clinical_procedure_items.parent_id = clinical_procedures.id AND clinical_procedure_items.payment=1";
    $procedure_filter    = "AND clinical_procedure_items.name LIKE '%$get->procedure%'";
}

$prescription_join          = "";
$prescription_filter        = "";
if($get->prescription) {
    $prescription_join      = "JOIN clinical_prescriptions ON clinical_prescriptions.visit_id = clinical_visits.id JOIN clinical_prescription_items ON clinical_prescription_items.parent_id = clinical_prescriptions.id AND clinical_prescription_items.payment=1";
    $prescription_filter    = "AND clinical_prescription_items.name LIKE '%$get->prescription%'";
}

$sql                    = "SELECT clinical_visits.id
FROM clinical_visits 
$location_join
$diagnoses_join
$laboratory_join
$radiology_join
$procedure_join
$prescription_join
WHERE clinical_visits.status = 1
$date_filter
$care_filter
$location_filter
$discharge_filter
$diagnoses_filter
$laboratory_filter
$radiology_filter
$procedure_filter
$prescription_filter
ORDER BY id ";

$datas                  = $app->read($sql);
$data                   = $app->pagenate($datas);

$current_date			= $start_date;
$current_period			= date('d M Y', strtotime($start_date));

echo $app->renderReportNavigation (
	array (
		'period'		=> 'Day',
		'section_id'	=> 'summary',
		'section_title'	=> '',
		'current_date'	=> $current_date,
		'current_period'=> $current_period
	)
);

$table_options      = array (
    'columns'       => array (
        array('name'=>'No'),
        array('name'=>'Visit date'),
        array('name'=>'Time'),
        array('name'=>'Visit no'),
        array('name'=>'Patient name'),
        array('name'=>'Employer'),
        array('name'=>'Residence'),
        array('name'=>'Region'),
        array('name'=>'District'),
        array('name'=>'Ward'),
        array('name'=>'Phone'),
        array('name'=>'Birthdate'),
        array('name'=>'Age'),
        array('name'=>'Sex'),
        array('name'=>'Weight'),
        array('name'=>'Height'),
        array('name'=>'Insurance'),
        array('name'=>'Scheme'),
        array('name'=>'Caretype'),
        array('name'=>'Check in'),
        array('name'=>'Check out'),
        array('name'=>'Checkout date'),
        array('name'=>'Days'),
        array('name'=>'Diagnoses'),
        array('name'=>'Laboratory'),
        array('name'=>'Radiology'),
        array('name'=>'Procedures'),
        array('name'=>'Prescription'),
        array('name'=>'Outcome')
    ),
    'css'           => 'bs-table table-striped'
);

echo $app->renderTableOpen($table_options);

$count              = 1;
foreach($data->rows as $row) {
    $visit          = new Visit($row->id);
    $first_location = $visit->firstVisitLocation();
    $last_location  = $visit->lastVisitLocation();
    $patient        = $visit->patient();
    $person         = $patient->person();
    $insurance      = new Insurance($visit->insurance_id);
    $scheme         = new Scheme($visit->scheme_id);

    if($visit->admitted) {
        $days       = '';
    } else {
        $days       = floor((strtotime($visit->checkout_datetime) - strtotime($visit->checkin_datetime)) / (60 * 60 * 24));
    }

    $laboratory     = array();
    foreach($visit->getServices('Laboratory', '1', 'payment=1') as $laboratory_item) {
        foreach($laboratory_item->details as $details) {
            $laboratory[]   = '<a class="btn btn-link px-0" onclick="$(\'#laboratory_'.$details->id.'\').slideToggle()">'.$details->name.' <span class="fa-solid fa-caret-down"></span></a>
            <div id="laboratory_'.$details->id.'" class="text-muted" style="display:none">'.$details->report.'</div>';
        }
    }
    $laboratory     = implode('<br />', $laboratory);

    $radiology      = array();
    foreach($visit->getServices('Radiology', '1', 'payment=1') as $radiology_item) {
        foreach($radiology_item->details as $details) {
            $radiology[]   = '<a class="btn btn-link px-0" onclick="$(\'#laboratory_'.$details->id.'\').slideToggle()">'.$details->name.' <span class="fa-solid fa-caret-down"></span></a>
            <div id="laboratory_'.$details->id.'" class="text-muted" style="display:none">'.$details->report.'</div>';
        }
    }
    $radiology     = implode('<br />', $radiology);
     
    $procedures      = array();
    foreach($visit->getServices('Procedure', '1', 'payment=1') as $procedure_item) {
        foreach($procedure_item->details as $details) {
            $procedures[]   = '<a class="btn btn-link px-0" onclick="$(\'#laboratory_'.$details->id.'\').slideToggle()">'.$details->name.' <span class="fa-solid fa-caret-down"></span></a>
            <div id="laboratory_'.$details->id.'" class="text-muted" style="display:none">'.$details->report.'</div>';
        }
    }
    $procedures     = implode('<br />', $procedures);
    
    $prescription      = array();
    foreach($visit->getServices('Prescription', '1', 'dispensed=1') as $prescription_item) {
        foreach($prescription_item->details as $details) {
            $prescription[]   = '<a class="btn btn-link px-0" onclick="$(\'#laboratory_'.$details->id.'\').slideToggle()">'.$details->name.' <span class="fa-solid fa-caret-down"></span></a>
            <div id="laboratory_'.$details->id.'" class="text-muted" style="display:none">'.$details->report.'</div>';
        }
    }
    $prescription     = implode('<br />', $prescription);

    $diagnoses      = array();
    foreach($visit->getConfirmedDiagnoses() as $diagnosis) {
        $diagnoses[]    = $diagnosis->code.' '.$diagnosis->name;
    }
    $diagnoses      = implode('<br /><br />', $diagnoses);

    $residence      = strlen($person->house_no) > 10 ? $person->house_no : '';
    
    echo '<tr>';
    echo '<td class="text-center">'.$count++.'</td>';
    echo '<td style="min-width:100px">'.$app->date($visit->created_datetime).'</td>';
    echo '<td style="min-width:50px">'.$app->time($visit->created_datetime).'</td>';
    echo '<td>'.$row->id.'</td>';
    echo '<td style="min-width:200px">'.$person->name.'</td>';
    echo '<td style="min-width:100px">'.$person->employer.' '.$person->employer_no.'</td>';
    echo '<td style="min-width:100px">'.$residence.'</td>';
    echo '<td style="min-width:100px">'.$person->region()->name.'</td>';
    echo '<td style="min-width:100px">'.$person->district()->name.'</td>';
    echo '<td style="min-width:100px">'.$person->village()->name.'</td>';
    echo '<td style="min-width:100px">'.$person->phone.'</td>';
    echo '<td style="min-width:100px">'.$person->birthdate.'</td>';
    echo '<td style="min-width:100px">'.$app->getAge($person->birthdate, $app->date($visit->created_datetime), 'YEARS_MONTHS_DAYS').'</td>';
    echo '<td style="min-width:50px">'.$person->sex.'</td>';
    echo '<td style="min-width:100px"></td>';
    echo '<td style="min-width:100px"></td>';
    echo '<td style="min-width:100px">'.$insurance->code.'</td>';
    echo '<td style="min-width:100px">'.$scheme->code.'</td>';
    echo '<td style="min-width:100px">'.$visit->care_type.'</td>';
    echo '<td style="min-width:150px">'.$first_location->location()->location()->name.'</td>';
    echo '<td style="min-width:150px">'.$last_location->location()->location()->name.'</td>';
    echo '<td style="min-width:150px">'.$app->datetime($visit->checkout_datetime).'</td>';
    echo '<td class="text-center" style="min-width:50px">'.$days.'</td>';
    echo '<td style="min-width:300px">'.$diagnoses.'</td>';
    echo '<td style="min-width:300px">'.$laboratory.'</td>';
    echo '<td style="min-width:300px">'.$radiology.'</td>';
    echo '<td style="min-width:300px">'.$procedures.'</td>';
    echo '<td style="min-width:300px">'.$prescription.'</td>';
    echo '<td style="min-width:100px">'.$visit->checkout_type.'</td>';
    echo '</tr>';
}

echo $app->renderTableClose($table_options);

echo $data->pagination;

?>