<?php 

$root               	= '../../../../';
$submodule_code     	= 'TopDiagnoses';
require($root.'config/config.php');

function ageGroup($days) {
    if($days < 29) return '1-28 Days';
    if($days < 365) return '1-12 Months';
    if($days < 1825) return '1-5 Years';
    if($days < 21900) return '5-60 Years';
    return '60 Years+';
}

$start_date             = strlen($get->start_date) ? $app->morning($get->start_date) : $app->morning($app->monthStart());
$end_date               = strlen($get->end_date) ? $app->evening($get->end_date) : $app->evening($app->monthEnd());
$date_filter            = "AND clinical_diagnoses.created_datetime BETWEEN '$start_date' AND '$end_date' ";
$keyword_filter         = "AND clinical_diagnoses.name LIKE '%$get->keyword%' ";
$care_type_filter       = strlen($get->care_type) ? "AND clinical_visits.care_type = '$get->care_type' " : " ";
$location_filter        = $get->location_id ? "AND clinical_visits.care_type = '$get->care_type' AND clinical_visit_locations.location_id = $get->location_id " : " ";
$visit_type_filter      = strlen($get->visit_type) ? "AND clinical_visits.type = '$get->visit_type' " : " ";

$gender_keys            = array ('M' => 0, 'F' => 0, 'T'=> 0);
$agegroup_keys          = array ('1-28 Days', '1-12 Months', '1-5 Years', '5-60 Years', '60 Years+', 'Total');

echo $app->renderReportNavigation (
	array (
		'period'		=> 'Month',
		'section_id'	=> 'summary',
		'section_title'	=> '',
		'current_date'	=> $start_date,
		'current_period'=> date('M Y', strtotime($start_date))
	)
);

$diagnoses_sql          = "SELECT 
clinical_diagnoses.code, 
clinical_diagnoses.name, 
clinical_diagnoses.details,
personnel_persons.sex,
DATEDIFF(clinical_diagnoses.created_datetime, personnel_persons.birthdate) `days`
FROM clinical_diagnoses
JOIN clinical_visit_locations ON clinical_visit_locations.id = clinical_diagnoses.visit_location_id
JOIN clinical_visits ON clinical_visits.id = clinical_visit_locations.visit_id
JOIN clinical_patients ON clinical_patients.id = clinical_diagnoses.patient_id
JOIN personnel_persons ON personnel_persons.id = clinical_patients.person_id
WHERE clinical_diagnoses.certainty = 'Confirmed'
$date_filter
$care_type_filter
$location_filter
$visit_type_filter
$keyword_filter
";

$diagnoses              = array();
$datas                  = $app->read($diagnoses_sql);

foreach($datas as $row) {
    foreach($agegroup_keys as $agegroup_key) {
        $diagnoses[$row->name]['agegroups'][$agegroup_key]  = $gender_keys;
        $diagnoses[$row->name]['priority']                  = 0;
    }
}

foreach($datas as $row) {
    $sex                = substr($row->sex, 0, 1);
    foreach($agegroup_keys as $agegroup_key) {
        $agegroup       = ageGroup($row->days);
        if($agegroup_key == $agegroup) {
            $diagnoses[$row->name]['agegroups'][ageGroup($row->days)][$sex]++;
            $diagnoses[$row->name]['agegroups'][ageGroup($row->days)]['T']++;
            $diagnoses[$row->name]['agegroups']['Total'][$sex]++;
            $diagnoses[$row->name]['agegroups']['Total']['T']++;
            $diagnoses[$row->name]['priority']++;
        }
    }
}

uasort($diagnoses, function($a, $b) {
    if ($a['priority'] < $b['priority']) {
        return 1;
    } elseif ($a['priority'] > $b['priority']) {
        return -1;
    }
    return 0;
});

echo '<div class="my-3"><canvas id="top_diagnoses_bar" style="height:'.$report->getHorizontalChartHeight($diagnoses).'; width:100%"></canvas></div>';

echo '<div class="table-responsive">
<table class="table table-hover table-bordered table-striped bs-table">
<thead>
<tr>
<th class="h5 fw-bold text-center">No</th>
<th class="h5 fw-bold text-center">Details</th>';

foreach($agegroup_keys as $age) {
    echo '<th class="h5 fw-bold text-center" colspan="3">'.$age.'</th>';
}

echo '</tr>
<tr>
<th></th>
<th class="h5 fw-bold" style="min-width:5%; width:5%"></th>';

foreach($agegroup_keys as $age) {
    foreach(array_keys($gender_keys) as $gender) {
        echo '<th class="h5 fw-bold text-center" style="min-width:60px; width:60px">'.$gender.'</th>';
    }
}

echo '</tr>
</thead>
<tbody>';
$count      = 1;
$chart_data = array();
foreach(array_keys($diagnoses) as $diagnosis) {
    echo '<tr>
    <td class="text-center" style="width:50px; min-width:50px">'.$count++.'</td>
    <td class="text-start" style="width:250px; min-width:250px">';
    echo $diagnosis;
    echo '</td>';

    foreach($agegroup_keys as $age) {
        foreach(array_keys($gender_keys) as $gender) {
            echo '<td class="text-center" style="width:60px; min-width:60px">'.number_format($diagnoses[$diagnosis]['agegroups'][$age][$gender]).'</td>';
        }
    }

    echo '</tr>';

    if($count < 100) {
        $diagnosis_name      = $app->shortname($diagnosis, 50);
        $chart_data[$diagnosis_name]['Diagnoses']['value']   = $diagnoses[$diagnosis]['priority'];
	    $chart_data[$diagnosis_name]['Diagnoses']['color']   = $app->colors->primary->rgb;
    }
}

echo '</tbody>
</table>
</div>';

echo $report->renderHorizontalBarChart (
    array (
        'data'      => $chart_data,
        'canvas'    => 'top_diagnoses_bar',
        'options'   => array (
            'indexAxis' => 'y'
        )
    )
);

?>