<?php 

class Attachment extends Crud {
	function __construct($id=0) {	
		$this->table        = 'content_attachments';
		$this->primary_key  = 'id';
		parent::__construct($id);
	}

	function uploadImage($data) {
		global $root, $app, $user_id, $now;

		$source				= $data['file'];
		$destination 		= $data['section'];
		$width 				= isset($data['width']) ? $data['width'] : '';
		$height 			= isset($data['height']) ? $data['height'] : '';
		$quality 			= isset($data['quality']) ? $data['quality'] : 75;
		$extensions 		= isset($data['extensions']) ? $data['extensions'] : array();

		if(is_array($source)) {
			$name			= $source['name'];
			$extension		= explode('.', $name);
			$extension		= strtolower(end($extension));
			$source_path	= $source['tmp_name'];
		} else {
			$source_path	= $source;
			$temp_file  	= pathinfo($source_path);
			$name       	= $temp_file['basename'];
        	$extension  	= strtolower($temp_file['extension']);
		}
		$mimetype   		= mime_content_type($source_path);
		$filetype 			= explode('/', $mimetype);
		$filetype			= ucfirst($filetype[0]);
        $size       		= filesize($source_path);

		if(in_array($mimetype, array("image/png", "image/jpeg", "image/gif"))) {
			$destination	= $this->getSavingPath($destination).'.'.$extension;
			$url			= explode('/', $destination);
			$url			= end($url);
			$datetime 		= explode('.', $url);
			$datetime 		= $datetime[0];
			$hash 			= md5($url);
			
			if($app->resizeImage($source_path, $destination, $width, $height, $quality)) {
				
				$filename 		= isset($data['name']) ? $data['name'] : '';
				$filename       = strlen($filename) ? $filename : $name;
				$type			= isset($data['type']) ? $data['type'] : $filetype;
				$code			= isset($data['code']) ? $data['code'] : '';
				$details		= isset($data['details']) ? $data['details'] : '';
				$section		= isset($data['section']) ? $data['section'] : 'Drive';
				$section_id		= isset($data['section_id']) ? $data['section_id'] : 0;
				$subsection		= isset($data['subsection']) ? $data['subsection'] : $section;
				$subsection_id  = isset($data['subsection_id']) ? $data['subsection_id'] : 0;
				$owner			= isset($data['owner']) ? $data['owner'] : 'System';
				$owner_id		= isset($data['owner_id']) ? $data['owner_id'] : 0;
				$access			= isset($data['access']) ? $data['access'] : 'Protected';
				$action 		= $data['action'];
				$size       	= filesize($destination);

				if($action == 'create' || $action == 'copy' || !$this->id) {
					$create_data   	= array (
						'id'                => 0,
						'type'              => ucfirst($type),
						'code'              => $code,
						'name'              => $filename,
						'details'           => $details,
						'section'           => $section,
						'section_id'        => $section_id,
						'subsection'        => $subsection,
						'subsection_id'     => $subsection_id,
						'owner'             => $owner,
						'owner_id'          => $owner_id,
						'access'            => $access,
						'mimetype'          => $mimetype,
						'extension'         => $extension,
						'size'              => $size,
						'url'               => $url,
						'hash'              => $hash,
						'datetime'          => $datetime,
						'icon'              => 'fa-regular fa-paperclip',
						'color'             => 'primary',
						'tags'              => '',
						'f1'                => '',
						'f2'                => '',
						'f3'                => '',
						'f4'                => '',
						'f5'                => '',
						'f6'                => '',
						'f7'                => '',
						'f8'                => '',
						'f9'                => '',
						'created_user_id'   => $user_id,
						'created_datetime'  => $now,
						'modified_user_id'  => 0,
						'modified_datetime' => 0,
						'status'            => 1
					);
					return $this->create($create_data);

				} elseif($action == 'edit') {
					
					// Delete former attachment
					if(file_exists($this->path())) {
						unlink($this->path());
					}
					
					$update_data = array (
						'type'              => $type,
						'name'              => $filename,
						'code'              => $code,
						'details'           => $details,
						'section'           => $section,
						'section_id'        => $section_id,
						'subsection'        => $subsection,
						'subsection_id'     => $subsection_id,
						'owner'             => $owner,
						'owner_id'          => $owner_id,
						'access'            => $access,
						'mimetype'          => $mimetype,
						'extension'         => $extension,
						'size'              => $size,
						'url'               => $url,
						'hash'              => $hash,
						'datetime'          => $datetime
					);
					$this->update($update_data);

					return $this->id;
				}
			}
		} else {
			die($app->renderAlertError('The uploaded file is not a valid JPG, PNG or GIF image'));
		}
	}

	function uploadFile($data) {
		global $root, $app, $user_id, $now;

		$source				= $data['file'];
		$destination 		= $data['section'];
		$extensions 		= isset($data['extensions']) ? $data['extensions'] : array();
		
		if(is_array($source)) {
			$name			= $source['name'];
			$extension		= explode('.', $name);
			$extension		= strtolower(end($extension));
			$source_path	= $source['tmp_name']; 
		} else {
			$source_path	= $source;
			$temp_file  	= pathinfo($source_path);
			$name       	= $temp_file['basename'];
        	$extension  	= strtolower($temp_file['extension']);
		}

		$mimetype   		= mime_content_type($source_path);
		$filetype 			= explode('/', $mimetype);
		$filetype			= $filetype[0];
        $size       		= filesize($source_path);

		// Check extensions
 
		if(in_array($extension, $extensions) || !$extensions) {
			$destination	= $this->getSavingPath($destination).'.'.$extension;
			$url			= explode('/', $destination);
			$url			= end($url);
			$datetime 		= explode('.', $url);
			$datetime 		= $datetime[0];
			$hash 			= md5($url);
			
			if(move_uploaded_file($source_path, $destination)) {
				
				$filename 		= isset($data['name']) ? $data['name'] : '';
				$filename       = strlen($filename) ? $filename : $name;
				$type			= isset($data['type']) ? $data['type'] : $filetype;
				$code			= isset($data['code']) ? $data['code'] : '';
				$details		= isset($data['details']) ? $data['details'] : '';
				$section		= isset($data['section']) ? $data['section'] : 'Drive';
				$section_id		= isset($data['section_id']) ? $data['section_id'] : 0;
				$subsection		= isset($data['subsection']) ? $data['subsection'] : $section;
				$subsection_id  = isset($data['subsection_id']) ? $data['subsection_id'] : 0;
				$owner			= isset($data['owner']) ? $data['owner'] : 'System';
				$owner_id		= isset($data['owner_id']) ? $data['owner_id'] : 0;
				$access			= isset($data['access']) ? $data['access'] : 'Protected';
				$action 		=isset($data['action']) ? $data['action'] : 'create';

				if($action == 'create') {
					$create_data   	= array (
						'id'                => 0,
						'type'              => ucfirst($type),
						'code'              => $code,
						'name'              => $filename,
						'details'           => $details,
						'section'           => $section,
						'section_id'        => $section_id,
						'subsection'        => $subsection,
						'subsection_id'     => $subsection_id,
						'owner'             => $owner,
						'owner_id'          => $owner_id,
						'access'            => $access,
						'mimetype'          => $mimetype,
						'extension'         => $extension,
						'size'              => $size,
						'url'               => $url,
						'hash'              => $hash,
						'datetime'          => $datetime,
						'icon'              => 'fa-regular fa-paperclip',
						'color'             => 'primary',
						'tags'              => '',
						'f1'                => '',
						'f2'                => '',
						'f3'                => '',
						'f4'                => '',
						'f5'                => '',
						'f6'                => '',
						'f7'                => '',
						'f8'                => '',
						'f9'                => '',
						'created_user_id'   => $user_id,
						'created_datetime'  => $now,
						'modified_user_id'  => 0,
						'modified_datetime' => 0,
						'status'            => 1
					);
					return $this->create($create_data);

				} elseif($action == 'edit') {
					$update_data = array (
						'type'              => $type,
						'name'              => $filename,
						'code'              => $code,
						'details'           => $details,
						'section'           => $section,
						'section_id'        => $section_id,
						'subsection'        => $subsection,
						'subsection_id'     => $subsection_id,
						'owner'             => $owner,
						'owner_id'          => $owner_id,
						'access'            => $access,
						'mimetype'          => $mimetype,
						'extension'         => $extension,
						'size'              => $size,
						'url'               => $url,
						'hash'              => $hash,
						'datetime'          => $datetime
					);
					$this->update($update_data);
					return $this->id;
				}
			}
		} else {
			die($app->renderAlertError('The uploaded file is not a valid '.strtoupper($app->joinItems($extensions, 'or')).' file.'));
		}
	}


	function getSavingPath($section, $datetime=0) {
		global $root;
		if(!$datetime)	$datetime = date('Y-m-d');
		$section 		= strtolower($section);
		$year 			= date('Y', strtotime($datetime));
		$month			= date('m', strtotime($datetime));
		$date			= date('YmdHis', strtotime($datetime));
		$milliseconds 	= floor(microtime(true) * 1000);

		if(!file_exists($root.CLIENT_PATH.'attachments')) {
			mkdir($root.CLIENT_PATH.'attachments', 0777, true);
			file_put_contents($root.CLIENT_PATH.'attachments'.'/index.php', '');
		}

		if(!file_exists($root.CLIENT_PATH.'attachments/'.$section)) {
			mkdir($root.CLIENT_PATH.'attachments/'.$section, 0777, true);
			file_put_contents($root.CLIENT_PATH.'attachments/'.$section.'/index.php', '');
		}

		if(!file_exists($root.CLIENT_PATH.'attachments/'.$section.'/'.$year)) {
			mkdir($root.CLIENT_PATH.'attachments/'.$section.'/'.$year, 0777, true);
			file_put_contents($root.CLIENT_PATH.'attachments/'.$section.'/'.$year.'/index.php', '');
		}

		if(!file_exists($root.CLIENT_PATH.'attachments/'.$section.'/'.$year.'/'.$month)) {
			mkdir($root.CLIENT_PATH.'attachments/'.$section.'/'.$year.'/'.$month, 0777, true);
			file_put_contents($root.CLIENT_PATH.'attachments/'.$section.'/'.$year.'/'.$month.'/index.php', '');
		}

		$path 			= $root.CLIENT_PATH.'attachments/'.$section.'/'.$year.'/'.$month.'/'.$date.'.'.$milliseconds;
		return $path;
	}

	function renderLinkOpen($options=array()) {
		global $root;
		$hash	= isset($options['hash']) ? $options['hash'] : $this->hash;
		$html   = '<a target="blank" class="btn btn-default bs-tooltip" title="Open" href="'.$root.'pages/content/attachments/open?file='.$hash.'"><span class="fa-regular fa-arrow-up-right-from-square"></span></a>';
		return $html;
	}

	function renderLinkDownload($options=array()) {
		global $root;
		$hash	= isset($options['hash']) ? $options['hash'] : $this->hash;
		$html   = '<a target="blank" class="btn btn-default bs-tooltip" title="Download" href="'.$root.'pages/content/attachments/download?file='.$hash.'"><span class="fa-regular fa-download"></span></a>';
		return $html;
	}

	function renderOpenPath($attachment_root='') {
		global $root, $r;
		if(!$attachment_root) $attachment_root = $r;
		$hash	= isset($options['hash']) ? $options['hash'] : $this->hash;
		$html   = $attachment_root.'pages/content/attachments/open?file='.$hash;
		return $html;
	}

	function getAttachmentByHash($hash) {
		global $app;
		$id = $app->getColumnValue('content_attachments', 'id', 'hash', $hash);
		return new Attachment($id);
	}

	function path($attachment_root='') {
		global $root, $r;
		if(!$attachment_root) $attachment_root = $r;
		$section	= strtolower($this->section);
		$year		= date('Y', strtotime($this->datetime));
		$month		= date('m', strtotime($this->datetime));
		$url 		= $this->url;
		return $attachment_root.CLIENT_PATH.'attachments/'.$section.'/'.$year.'/'.$month.'/'.$url;
	}

	function exists() {
		global $root;
		$path		= $this->path($root);

		if(is_file($path)) {
			return true;
		} else {
			return false;
		}
	}

	function download() {
		ob_end_clean();
		header("Content-Description: File Transfer"); 
		header("Content-Type: application/octet-stream"); 
		header("Content-Disposition: attachment; filename=$this->name.$this->extension"); 
		readfile($this->path());
		exit();
	}

	function open() {
		ob_end_clean();
		header("Content-Description: File Transfer"); 
		header("Content-Type: $this->mimetype"); 
		header("Content-Disposition: inline; filename=$this->name.$this->extension"); 
		readfile($this->path());
		exit();
	}

	function renderPreviewEdit($options=array()) {
		global $app, $root;

		// if(!$this->exists()) return false;

		$id			= $options['id'];
		$code		= isset($options['code']) ? $options['code'] : '';
		$type		= isset($options['type']) ? $options['type'] : '';
		$name		= isset($options['name']) ? $options['name'] : $this->name;
		$details	= isset($options['details']) ? $options['details'] : $this->details;
		$subsection	= isset($options['subsection']) ? $options['subsection'] : '';
		$attr		= isset($options['attr']) ? $options['attr'] : '';
		$css		= isset($options['css']) ? $options['css'] : 'line-control';
		$accept 	= isset($options['accept']) ? $options['accept'] : '';

		if(strtolower($type) == 'image') {

			$accept 	= 'accept="image/jpeg, image/png, image/gif"';
			if($this->exists()) {
				$preview 	= '<img id="'.$id.'_preview" src="'.$this->renderOpenPath().'" alt="default" class="w-100 rounded mb-3" style="width:100%; height:90%; object-fit:cover" />';
			} else {
				$preview 	= '<img id="'.$id.'_preview" src="'.$root.'/assets/images/default.jpg" alt="default" class="w-100 rounded mb-3" style="width:100%; height:90%; object-fit:cover" />';
			}
			
			$preview 	.= '<div id="'.$id.'_preview_size" class="text-center text-white rounded-bottom" style="position:relative; bottom:46px; margin-bottom:-46px; height:30px; background-color:rgba(10,10,10,0.3)"> '.$this->size().' </div>';
	
			$html = '<div class="row">';
			$html .= '<div class="col-12 col-lg-4 pt-2">'.$preview.'</div>
			<div class="col-12 col-lg-8 mb-4">';
			$html .= $app->renderInputHidden('attachments['.$id.'][id]', $this->id);
			$html .= $app->renderInputHidden('attachments['.$id.'][code]', $code);
			$html .= $app->renderInputHidden('attachments['.$id.'][subsection]', $subsection);
			$html .= $app->renderInputText('attachments['.$id.'][name]', $name, 'Title', $css.' mb-2 fw-bold p-0 form-control-lg border-0', $attr);
			$html .= $app->renderInputHidden('attachments['.$id.'][details]', $details, '', $css.' mb-3', $attr);
			$html .= '<div class="row"><div class="col-6">';
			$html .= $app->renderInputFile(''.$id, '', '<span class="fa-regular fa-upload fa-fw"></span> Upload ', 'myb-3', 'data-target-preview='.$id.'_preview '.$accept);
			$html .= '</div><div class="col-6">';
			if($this->exists()) $html .= '<a href="'.$root.'pages/content/attachments/download?file='.$this->hash.'" class="btn btn-outline-primary w-100 mb-3"><span class="fa-regular fa-download fa-fw"></span> Downl. </a>';
			$html .= '</div></div>';
			$html .= '</div></div>';
		} else {

			$preview	= '<div id="'.$id.'_preview" class="bg-light rounded p-5 text-center border border-light" style="height:91%"> <span class="fa-light fa-file-text fa-3x"></span> </div>
			<div id="'.$id.'_preview_size" class="text-center text-white rounded-bottom" style="position:relative; bottom:30px; margin-bottom:-30px; height:30px; background-color:rgba(10,10,10,0.3)"> '.$this->size().' </div>';
	
			$html = '<div class="row">';
			$html .= '<div class="col-12 col-lg-4 mb-4 pt-2">'.$preview.'</div>
			<div class="col-12 col-lg-8 mb-4">';
			$html .= $app->renderInputHidden('attachments['.$id.'][id]', $this->id);
			$html .= $app->renderInputHidden('attachments['.$id.'][code]', $code);
			$html .= $app->renderInputHidden('attachments['.$id.'][subsection]', $subsection);
			$html .= $app->renderInputText('attachments['.$id.'][name]', $name, 'Title', $css.' mb-2 fw-bold p-0 form-control-lg border-0', $attr);
			$html .= $app->renderInputHidden('attachments['.$id.'][details]', $details, '', $css.' mb-3', $attr);
			$html .= '<div class="row"><div class="col-6">';
			$html .= $app->renderInputFile(''.$id, '', '<span class="fa-regular fa-upload fa-fw"></span> Upload ', 'myb-3', 'data-target-preview='.$id.'_preview '.$accept);
			$html .= '</div><div class="col-6">';
			if($this->exists()) $html .= '<a href="'.$root.'pages/content/attachments/download?file='.$this->hash.'" class="btn btn-outline-primary w-100 mb-3"><span class="fa-regular fa-download fa-fw"></span> Downl. </a>';
			$html .= '</div></div>';
			$html .= '</div></div>';
		}

		return $html;
	}

	function size() {
		global $app;
		return $app->filesize($this->size);
	}

	function uploadAttachments($options = array()) {
		global $app, $get, $post;

		$files					= $options['files'];
		$section 				= isset($options['section']) ? $options['section'] : 'Attachment';
		$section_id 			= isset($options['section_id']) ? $options['section_id'] : 0;
		$subsection 			= isset($options['subsection']) ? $options['subsection'] : 'Attachment';
		$subsection_id 			= isset($options['subsection_id']) ? $options['subsection_id'] : 0;
		$owner 					= isset($options['owner']) ? $options['owner'] : 'System';
		$owner_id 				= isset($options['owner_id']) ? $options['owner_id'] : 0;
		$access 				= isset($options['access']) ? $options['access'] : 'Protected';

		$code 					= isset($options['code']) ? $options['code'] : $section;
		$category 				= isset($options['category']) ? $options['category'] : $code.'Attachment';

        foreach(array_keys($files) as $attachment_name) {
            $attachment         = new Attachment($post->attachments[$attachment_name]['id']);
            $attach_category    = $app->getCategoryByCode($category, $post->attachments[$attachment_name]['code']);

            $attachment_data    = array (
                'id'            => $post->attachments[$attachment_name]['id'],
                'name'          => $post->attachments[$attachment_name]['name'],
                'code'          => $post->attachments[$attachment_name]['code'],
                'subsection'    => $post->attachments[$attachment_name]['subsection'],
                'file'          => $files[$attachment_name],
                'details'       => $post->attachments[$attachment_name]['details'],
                'section'       => $section,
                'section_id'    => $section_id,
                'subsection'    => $subsection,
                'subsection_id' => $subsection_id,
                'owner'         => $owner,
                'owner_id'      => $owner_id,
                'access'        => $access,
                'width'         => is_numeric($attach_category->f3) ? $attach_category->f3 : 800,
                'height'        => is_numeric($attach_category->f4) ? $attach_category->f4 : 600,
                'action'        => $post->attachments[$attachment_name]['id'] ? $get->action : 'create'
            );

            if(strtolower($attach_category->f1) == 'image') {
                if($attachment_id = $attachment->uploadImage($attachment_data)) {
                    echo $app->renderAlertSuccess(ucfirst(strtolower($attach_category->name)).' uploaded successfully');
                }
            } else {
                if($attachment_id = $attachment->uploadFile($attachment_data)) {
                    echo $app->renderAlertSuccess(ucfirst(strtolower($attach_category->name)).' uploaded successfully');
                }
            }
        }
    }

	function renderAttachmentsPreviewEdit($options = array()) {
		global $app;

		if(!$this->exists()) return false;

		$code					= $options['code'];
		$category				= isset($options['category']) ? $options['category'] : $code.'Attachment';
		$id						= isset($options['id']) ? $options['id'] : $this->id;
		$html 					= '';
		$attachments            = $app->getCategories($category);

		foreach($attachments as $attachment) {
			
			$attach_name        = $attachment->value;
			$attach             = $app->getAttachmentByCode($code, $id, $attach_name);

			$attach_category    = $app->getCategoryByCode($category, $attach_name);
			$html 				.= $attach->renderPreviewEdit (
				array (
					'id'        => $attach_name,
					'css'       => 'readonly',
					'type'      => $attach_category->f1,
					'name'      => $attach_category->name,
					'details'   => $attach_category->details,
					'code'      => $attach_category->code,
					'subsection'=> $attach_category->code
				)
			);
		}
		return $html;
	}

	function uploadSignature($signature_data) {
		return $this->uploadImage($signature_data);
	}

	function renderCardOpen($attachment_root='') {
		global $app;

		if(!$this->exists()) return false;
		
		$html 		= '';
		$pictures 	= array('jpg', 'jpeg', 'png', 'gif');
		$files 		= array(
			'doc'	=>'fa-regular fa-file-word text-blue',
			'docx'	=>'fa-regular fa-file-word text-blue',
			'ppt'	=>'fa-regular fa-file-powerpoint text-danger',
			'pptx'	=>'fa-regular fa-file-powerpoint text-danger',
			'pdf'	=>'fa-regular fa-file-pdf text-red',
			'xls'	=>'fa-regular fa-file-excel text-green',
			'xlsx'	=>'fa-regular fa-file-excel text-green',
			'php'	=>'fa-regular fa-file-code text-orange',
			'js'	=>'fa-regular fa-file-code text-orange',
			'html'	=>'fa-regular fa-file-code text-orange'
		);
		if(in_array($this->extension, $pictures)) {
			$thumbnail 	= $app->resizeImage($this->path(), '', 400, 400, 85, true);
			$html 		.= '<a href="'.$this->renderOpenPath($attachment_root).'" target="blank" class="text-decoration-none">
			<img class="img-fluid rounded" style="width:100%" src="data:image/jpeg;base64,'.$thumbnail.'" alt="'.$this->name.'" />
			</a>';
		} else {
			$icon 		= 'fa-regular fa-file-alt';
			if(array_key_exists($this->extension, $files)) {
				$icon 	= $files[$this->extension];
			}
			$html 		.= '<a href="'.$this->renderOpenPath($attachment_root).'" target="blank" class="text-decoration-none">
			<div class="card px-3 text-center bg-light align-middle" style="position: relative; width:100%; height: 0; padding-bottom: 100%;">
			<br /><h1><span class="'.$icon.'"></span></h1> <small>'.$app->shortname($this->name, 50).'</small> 
			</div></a>';
		}

		return $html;
	}

	function renderCardDelete() {
		global $r;
		$html = '<div id="attachment_container_'.$this->id.'">
		'.$this->renderCardOpen().'
		<div class="py-2 text-center"><a href="#" class="btn btn-outline-danger btn-sm w-100 my-2" onclick="if(confirm(\'Are you sure you want to remove this attachment?\')) $(\'#attachment_container_'.$this->id.'\').html(loading_circle).load(\''.$r.'pages/content/attachments/delete.php?id='.$this->id.'&attachment_container=attachment_container_'.$this->id.'\')"><span class="fa-solid fa-times fa-fw"></span> Remove attachment</a></div>
		</div>';

		return $html;
	}

	function delete(array $data=array()) {
		$path = $this->path();
		if(file_exists($path)) unlink($path);
		return parent::delete($data);
	}
}

?>