<?php 

class Adjustment extends Transaction {
	function __construct($id=0) {	
		$this->table        = 'inventory_adjustments';
		$this->items_table	= 'inventory_adjustment_items';
		$this->primary_key  = 'id';
		parent::__construct($id);
		$this->type 		= 'Adjustment';	
	}

	function renderTransactionHeader() {
		$html = $this->id ? $this->renderTrackingButton() : '';
		$html .= '<div class="row">';
		$html .= '<div class="col-12 col-lg-6 mb-3">'.$this->renderSubtypeControl('Adjustment reason').'</div>';
		$html .= '<div class="col-12 col-lg-6 mb-3">'.$this->renderStoreControl('Adjustment store').'</div>';
		$html .= '<div class="col-12 d-none">'.$this->renderPricelistControl($this->getCostPricelistId()).'</div>';
		$html .='</div>';
		return $html;
	}

	function renderModalForm($submodule, $obj, $get) {
        global $app;
		$modal_options  = array(
			'title'     => ucfirst($get->action).' stock adjustment',
			'left_html' => ($get->action == 'create' || $get->action == 'copy' || 1) ? $obj->renderVerticalCreator() : $obj->renderVerticalProgress()
		);

		$html = '';
		
		if($get->action == 'delete') {
			$html .= $submodule->renderModalDelete($get->id);
		} else {
			$html .= $submodule->renderModalFormOpen($modal_options);
			$html .= $app->renderInputHidden('action', $get->action);
			$html .= $app->renderInputHidden('id', $get->id);
		
			if($get->action == 'create') {
				$html .= $obj->renderTransactionHeader();
				$html .= $obj->renderTransactionItemsEditable('Adjustment items');
				// $html .= $obj->renderConsumptionControls();
				$html .= $obj->renderItemsControls();
				$html .= $obj->renderNotesControl('Requested');
				//$html .= $obj->initializeTransaction();
			} elseif($get->action == 'edit' || $get->action == 'copy') {
				$html .= $obj->renderTransactionHeader();
				$html .= $obj->renderTransactionItemsEditable();
				$html .= $obj->renderItemsControls();
				$html .= $obj->renderNotesControl('Requested');
			} elseif($get->action == 'authorize') {
				$html .= $obj->renderTransactionHeader();
				$html .= $obj->renderTransactionItemsReadonly();
				$html .= $obj->renderActionControl( array (
                    array('name'=>'Authorize adjustment', 'value'=>'Authorized', 'icon'=>'fa-regular fa-check-circle fa-fw'),
                    array('name'=>'Reject adjustment','value'=>'Rejected', 'icon'=>'fa-regular fa-circle-xmark fa-fw')
                ));
				$html .= $obj->renderNotesControl('Authorized');
			} elseif($get->action == 'approve') {
				$html .= $obj->renderTransactionHeader();
				$html .= $obj->renderTransactionItemsReadonly();
				$html .= $obj->renderActionControl( array (
                    array('name'=>'Approve adjustment', 'value'=>'Approved', 'icon'=>'fa-regular fa-check-circle fa-fw'),
                    array('name'=>'Reject adjustment','value'=>'Rejected', 'icon'=>'fa-regular fa-circle-xmark fa-fw')
                ));
                $html .= $obj->renderNotesControl('Approved');
			} elseif($get->action == 'transfer') {
				$html .= $obj->renderTransactionHeader();
				$html .= $obj->renderTransactionItemsWithBatchControl();
				$html .= $obj->renderActionControl( array (
                    array('name'=>'Adjust selected items', 'value'=>'Transferred', 'icon'=>'fa-regular fa-check-circle fa-fw'),
                    array('name'=>'Reject adjustment','value'=>'Rejected', 'icon'=>'fa-regular fa-circle-xmark fa-fw')
                ));
                $html .= $obj->renderNotesControl('Transferred');
			} elseif($get->action == 'open') {
				$html .= $obj->renderTransactionHeader();
				$html .= $obj->renderTransactionItemsReadonly('Adjusted items');
			}
			$html .= $submodule->renderModalFormClose();
		}
		return $html;
	}

	function renderSubtypeControl($title='Category') {
        global $app;
        $html   = '<div class="trans-container mb-3"> <h5 class="trans-subtitle"> <span class="fa-regular fa-square-check fa-fw"></span> '.$title.': </h5>';
		$html   .= $app->renderInputSelect('trans_subtype', $this->subtype, 'Choose adjustment category', 'line-control select-search', '', 'settings_categories', 'code', 'name', "type='AdjustmentType'");
		$html   .= '</div>';
        return $html; 
    }

	function subtype() {
		global $app;
		return $app->getCategoryByCode('AdjustmentType', $this->subtype);
	}

	function renderTableOpen($data) {
        global $app;

		$table_headers      = array (
			'columns'       => array (
				array('name'=>'Date', 'css'=>'w-10'),
				array('name'=>'Location'),
				array('name'=>'Requested', 'css'=>'w-18'),
				array('name'=>'Category', 'css'=>'w-15'),
				array('name'=>'Amount', 'css'=>'w-10 money text-end'),
				array('name'=>'Status', 'css'=>'w-15 text-center'),
				array('name'=>'Options', 'css'=>'w-10 text-end')
			),
			'css'           => 'bs-table',
			'rows'			=> $data->rows
		);

		return $app->renderTableOpen($table_headers);
    }

    function renderTableBody($data) {
        global $app;

        $html   = '';

        foreach($data->rows as $row) {
			$obj            = new Adjustment($row->id);
            $user           = new User($obj->created_user_id);

			$html .= '<tr>
			<td class="fw-semibold text-muted"> '.$app->datetime($obj->requested_datetime).'</td>
			
			<td>
			<div class="list-text fw-bold">'.$obj->store()->department()->location()->name.'</div>
			<div class="list-subtext">'.$obj->store()->location()->name.'</div>
			</td>
            <td> '.$user->person()->name.'</td>
			<td>'.$obj->subtype()->name.'</td>
			<td class="text-end">
			<div class="fw-bold text-muted">'.$app->formatMoney($obj->subtotal).'</div>
			</td>
			<td class="text-center">'.$obj->renderButtonProgress().'</td>
			<td class="text-center text-nowrap">';
		
			if($obj->progress == 'Requested') {
				$html .= $obj->renderLinkAuthorize();
				$html .= $obj->renderLinkCopy();
				$html .= $obj->renderLinkEdit();
				$html .= $obj->renderLinkDelete();
			} elseif($obj->progress == 'Authorized') {
				$html .= $obj->renderLinkApprove();
				$html .= $obj->renderLinkCopy();
				$html .= $obj->renderLinkDelete();
			} elseif($obj->progress == 'Approved') {
				$html .= $obj->renderLinkTransfer();
				$html .= $obj->renderLinkCopy();
				$html .= $obj->renderLinkDelete();
			} elseif($obj->progress == 'Transferred') {
				$html .= $obj->renderLinkCopy();
				$html .= $obj->renderLinkPrint();
			} elseif($obj->progress == 'Forwarded') {
				$html .= $obj->renderLinkCopy();
				$html .= $obj->renderLinkPrint();
			}
		
			$html .=' </td>
			</tr>';
		}
        return $html;
    }

    function renderTableClose() {
        global $app;
        return $app->renderTableClose();
    }

    function renderTableData($get) {
        global $app;

        $html       = '';
        $start_date = strlen($get->start_date) ? $app->morning($get->start_date) : $app->morning($app->monthStart());
        $end_date   = strlen($get->end_date) ? $app->evening($get->end_date) : $app->evening($app->monthEnd());

        $filters    = $get->department ? "AND location_stores.department_id = $get->department " : "";
		$filters    .= $get->store ? "AND location_stores.id = $get->store " : "";
		$filters    .= $get->category ? "AND inventory_adjustments.subtype = '$get->category'" : "";

		$sql    	= "SELECT inventory_adjustments.id 
        FROM inventory_adjustments 
		JOIN location_stores ON location_stores.id = inventory_adjustments.store_id
		JOIN location_departments ON location_departments.id = location_stores.department_id 
		WHERE inventory_adjustments.status = 1
        AND inventory_adjustments.created_datetime BETWEEN '$start_date' AND '$end_date'
        $filters ORDER BY id DESC ";

        $data       = $app->pagenate($app->read($sql));
        $html       .= $this->renderTableOpen($data);
        $html       .= $this->renderTableBody($data);
        $html       .= $this->renderTableClose();
        $html       .= $data->pagination;

        return $html;
    }

	function renderCreateConfirmation($submodule, $post) {
        global $app, $user;

        $user->checkSubmoduleAccess($submodule, 'create');

        $items                  = $app->uncleanText($post->trans_items);
        $store                  = new Store($post->trans_store_id);
		$create_data                = array (
            'type'                  => 'Adjustment',
            'subtype'               => $post->trans_subtype,
            'store_id'              => $post->trans_store_id,
            'name'                  => $store->location()->name,
            'requested_notes'       => $post->trans_requested_notes,
            'items'                 => $items,
            'cost'                  => $post->trans_subtotal,
            'subtotal'              => $post->trans_subtotal,
            'progeress'             => 'Requested',
            'progress_code'         => 1
        );

        $transaction_id         = $this->createWithMinimumData($create_data);
        if($transaction_id) {
            return $app->renderAlertSuccess($submodule->shortname.' created successfully', 'Success', 'modal_lg', true);
        }
    }

	function renderEditConfirmation($submodule, $post) {
        global $app, $user;

        $user->checkSubmoduleAccess($submodule, 'edit');   

        $items                  = $app->uncleanText($post->trans_items);
        $store                  = new Store($post->trans_store_id);

		$update_data            = array (
            'subtype'           => $post->trans_subtype,
            'store_id'          => $post->trans_store_id,
            'name'              => $store->location()->name,
            'requested_notes'   => $post->trans_requested_notes,
            'items'             => $items,
            'cost'              => $app->unformatMoney($post->trans_subtotal),
            'subtotal'          => $app->unformatMoney($post->trans_subtotal)
        );
        $updated             	= $this->update($update_data);
        if($updated) {
            $this->updateInlineItems();
            return $app->renderAlertSuccess($submodule->shortname.' updated successfully', 'Success', 'modal_lg', 1);
        }
    }

	function renderTransferConfirmation($submodule, $post) {
        global              $app, $user_id, $now;

        if($post->trans_action == 'Transferred') {

            $transferred_items     					= $post->trans_items;
            $transferred_item_ids					= array_keys($transferred_items);
            $transferring_items    	 				= array();

			foreach($transferred_item_ids as $transferred_item_id) {
                $transferred_item                   = $transferred_items[$transferred_item_id];
                $transferred_item['store_id']       = $this->store_id;
                $transferred_item['parent']    = $this->type; 
                $transferred_item['parent_id'] = $this->id; 
                $transferring_items[]               = $transferred_item;
            }

            // Check stock availability
            foreach($transferring_items as $adjust_options) {
                $item_obj                   		= new Item($adjust_options['item_id']);
                $transaction_item           		= new TransactionItem('Adjustment', $this->id, $adjust_options['transaction_item_id']);
                $transaction_item->batch           	= $adjust_options['batch'];
                $adjust_options['adjust']   		= false;
                $adjust_options['item']     		= $transaction_item; 
                $adjusted                   		= $item_obj->adjustStock($adjust_options);
            }

            // Make sure atleast one item submitted
            if(!$transferring_items) die($app->renderAlertError('Please submit at least one item to transfer'));

            // Adjust stock
            foreach($transferring_items as $adjust_options) {
				
                $item_obj                   		= new Item($adjust_options['item_id']);
                $transaction_item           		= new TransactionItem('Adjustment', $this->id, $adjust_options['transaction_item_id']);
                $transaction_item->batch           	= $adjust_options['batch'];
				$adjust_options['adjust']   		= true;
                $adjust_options['item']     		= $transaction_item;
                $adjusted                   		= $item_obj->adjustStock($adjust_options);

                $transaction_item->update (
                    array (
                        'progress'          		=> 'Transferred',
                        'transferred'          		=> 1,
                        'transferred_quantity' 		=> $transaction_item->requested_quantity,
                        'transferred_user_id'  		=> $user_id,
                        'transferred_datetime' 		=> $now
                    )
                );
            }

			$this->update (
				array (
					'progress'          		=> 'Transferred',
					'transferred'          		=> 1,
					'transferred_notes'  		=> $post->trans_transferred_notes,
					'transferred_user_id'  		=> $user_id,
					'transferred_datetime' 		=> $now
				)
			);

			$this->updateInlineItems();
			return $app->renderAlertSuccess('All items adjusted successfully', 'Success', 'modal_lg', 1);

        } else {
            die($app->renderAlertError('Please choose action'));
        }
    }


	function renderPDF($title='Transaction') {
        global $app;

		$title 					= 'Internal Stock Adjustment';
        $transaction_progress   = array ('created', 'authorized', 'approved', 'transferred');

        $html = $app->renderDocumentHeader();

        $html .= '<table class="w-100 py-5">
        <tr><td class="w-100 bg-light border"><h2 class="text-center">'.$title.'</h3></td></tr>
        </table>
        
        <table class=""><tr><td></td></tr></table>
        
        <table class="w-100 p-7 border bg-light">
        <tr>
        <td class="w-50 border-left border-top border-bottom"><h3>'.$this->type.' number</h3></td>
        <td class="w-50 border-right border-top border-bottom text-end"><h3>'.$this->formatCode($this->id).'</h3></td>
        </tr>
        </table>
        
        <table class="w-100 px-7 py-7 border">
        
        <tr class="">
        <td class="w-20 border-left border-top border-bottom">Adjustment store</td>
        <td class="w-30 border-top border-bottom text-muted">'.$this->store()->location()->name.'</td>
        <td class="w-20 border-top border-bottom">Adjustment reason:</td>
        <td class="w-30 border-top border-bottom border-right text-end text-muted">'.$app->getCategoryByCode('AdjustmentType', $this->subtype)->name.'</td>
        </tr>';

        foreach($transaction_progress as $progress) {
            $action     = ucfirst($progress).' by';
            $username   = $this->personnel($progress.'_user_id')->person()->name;
            $datetime   = $progress.'_datetime';
            $datetime   = $app->datetime($this->$datetime);

            $html .= '<tr>
            <td class="w-20 border-top">'.$action.'</td>
            <td class="w-30 border-top text-muted">'.$username.'</td>
            <td class="w-25 border-top">Datetime '.$progress.': </td>
            <td class="w-25 border-top text-end text-muted">'.$datetime.'</td>
            </tr>';
        }

        $html .= '</table>
        
        <br /><br /><table class="w-100 p-7">
        <tr>
        <td class="w-50 bg-light border-left border-top border-bottom"><h3>Adjusted Items </h3></td>
        <td class="w-50 bg-light border-right border-top border-bottom text-end"><h3> '.count($this->items()).'</h3></td>
        </tr>
        </table><br /><br />
        
        <table class="w-100 px-7 py-7 border">
        
        <tr class="bg-light">
        <td class="fw-bold text-center h4 border-top border-bottom border-left" style="width:6.5%">S/N</td>
        <td class="fw-bold h4 border-top border-bottom" style="width:25.5%">Item name</td>
        <td class="w-10 fw-bold h4 border-top border-bottom">Units</td>
        <td class="w-10 fw-bold h4 border-top border-bottom">Batch</td>
        <td class="w-12 fw-bold h4 border-top border-bottom">Expiry</td>
        <td class="w-8 fw-bold h4 border-top border-bottom text-end">Qty</td>
        <td class="w-12 fw-bold h4 border-top border-bottom text-end">Price</td>
        <td class="w-15 fw-bold h4 border-top border-bottom border-right text-end">Amount</td>
        </tr>';

        $count  		= 1;
		$total_price 	= 0;
        foreach($this->items() as $item) {
            $html .= '<tr>
            <td class="text-center border-top">'.$count++.'.</td>
            <td class="border-top">'.$item->name.'</td>
            <td class="border-top">'.$item->uom.'</td>
            <td class="border-top">'.$item->batch.'</td>
            <td class="border-top">'.$item->expiry.'</td>
            <td class="text-end border-top">'.$item->quantity.'</td>
            <td class="text-end border-top">'.number_format($item->unit_price).'</td>
            <td class="text-end border-top">'.number_format($item->price).'</td>
            </tr>';
			$total_price += $item->price;
        }

        $html .= '<tr class="fw-bold">
		<td class="border-top"></td>
		<td class="border-top">Total amount</td>
		<td class="border-top"></td>
		<td class="border-top"></td>
		<td class="border-top"></td>
		<td class="border-top"></td>
		<td class="border-top"></td>
		<td class="text-end border-top">'.number_format($total_price).'</td>
		</tr></table>
        
        <br /><br />
        
        <table class="w-100 p-7 border" style="page-break-inside:avoid;">
        <tr class="bg-light">
        <td class="w-25 text-center border-left border-top border-bottom"><h3>Requested by:</h3></td>
        <td class="w-25 text-center border-top border-bottom"><h3>Authorized by:</h3></td>
        <td class="w-25 text-center border-top border-bottom"><h3>Approved by:</h3></td>
        <td class="w-25 text-center border-top border-bottom border-right"><h3>Transferred by:</h3></td>
        </tr><tr>
		<td class="w-25 text-center"><br /> <br /> <br /> ..................................<br />'.$this->personnel('created_user_id')->person()->name.'<br /><b>'.$app->date($this->created_datetime).'</b></td>
        <td class="w-25 text-center"><br /> <br /> <br /> ..................................<br />'.$this->personnel('authorized_user_id')->person()->name.'<br /><b>'.$app->date($this->authorized_datetime).'</b></td>
        <td class="w-25 text-center"><br /> <br /> <br /> ..................................<br />'.$this->personnel('approved_user_id')->person()->name.'<br /><b>'.$app->date($this->approved_datetime).'</b></td>
        <td class="w-25 text-center"><br /> <br /> <br /> ..................................<br />'.$this->personnel('transferred_user_id')->person()->name.'<br /><b>'.$app->date($this->transferred_datetime).'</b></td>
        </tr>
        </table>';

        $app->renderPDF($html, ''.$title.'');
    }
}

?>