<?php 

$root               = '../../../../';
$submodule_code     = 'InventoryBulk';
require($root.'config/config.php');
$obj                = new Item();

if(!isset($_FILES['template_file'])) {
    die($app->renderAlertError('Template file is not uploaded'));
}

$template_file      = $_FILES['template_file']['tmp_name'];

$headings           = array (
    'ITEM_NAME'
);

// Validate item type
if(strlen($get->default_type) < 1) {
    $headings[]     = 'ITEM_TYPE';
}

// Validate prices
if(strlen($get->default_price) < 1) {
    $sql            = "SELECT code, name FROM billing_pricelists ORDER BY name ";
    foreach($app->read($sql) as $row) {
        $headings[] = $row->code;
    }
}

// Validate required columns
$csv->validateHeadings($template_file, $headings);
$data               = $csv->read($template_file);

// Valid item types
$sql                = "SELECT code, name FROM settings_categories WHERE type = 'ItemType'";
$valid_types        = array();
foreach($app->read($sql) as $row) {
    $valid_types[]  = $row->code;
}

// Validate required rows
$count              = 1;
$items              = array();

foreach($data as $row) {
    $count++;
    $item           = array();

    // Item type
    if(isset($row->ITEM_TYPE)) {
        if(in_array($row->ITEM_TYPE, $valid_types)) {
            $item['type'] = $row->ITEM_TYPE;
        } elseif(strlen($get->default_type)) {
            $item['type'] = $get->default_type;
        } else {
            die($app->renderAlertError('Invalid item type in row '.$count, 'ITEM_TYPE'));
        }
    } else {
        $item['type'] = $get->default_type;
    }

    // Item name
    if(strlen(trim($row->ITEM_NAME))) {
        $item['name'] = $app->cleanText($row->ITEM_NAME);
    } else {
        die($app->renderAlertError('Item name cannot be empty at row '.$count, 'ITEM_NAME'));
    }

    // Item units
    if(isset($row->ITEM_UNITS)) {
        if(strlen(trim($row->ITEM_UNITS))) {
            $item['units'] = $app->cleanText($row->ITEM_UNITS);
        } elseif(strlen($get->default_units)) {
            $item['units'] = $get->default_units;
        } else {
            die($app->renderAlertError('Item units cannot be empty at row '.$count, 'ITEM_UNITS'));
        }
    } elseif(strlen($get->default_units)) {
        $item['units'] = $get->default_units;
    } else {
        die($app->renderAlertError('Item units cannot be empty at row '.$count, 'ITEM_UNITS'));
    }

    // Item codes
    for($code = 1; $code < 10; $code++) {
        $item['code'.$code] = '';
        $item['prop'.$code] = '';
    }
    $sql                = "SELECT code, f1 FROM settings_categories WHERE type = 'ItemCode'";
    foreach($app->read($sql) as $code) {
        $code_column    = $code->code;
        if(isset($row->$code_column)) {
            $item[$code->f1] = $row->$code_column;
        }
    }
 
    // Item pricelists
    $pricelists     = array();
    $sql            = "SELECT id, code FROM billing_pricelists ";
    foreach($app->read($sql) as $pricelist) {
        $price_column   = $pricelist->code;
        $price          = isset($row->$price_column) ? (strlen($row->$price_column) ? $row->$price_column : (strlen($get->default_price) ? $get->default_price : 0)) : (strlen($get->default_price) ? $get->default_price : 0);
        $price          = $app->unformatMoney($price);
        $pricelists[$price_column] = array (
            'id'        => $pricelist->id,
            'code'      => $price_column,
            'account_id'=> 0,
            'pricemode' => $price < 0 ? 'Disabled' : 'Auto',
            'margin'    => 0,
            'price'     => $price,
            'restricted'=> '',
            'coverage'  => 100
        );
    }

    // Stockable
    if(in_array($item['type'], array('Admission', 'Consultation', 'Expense', 'Laboratory', 'Procedure', 'Radiology', 'Service'))) {
        $item['stock']          = 1;
        $item['purchase']       = 1;
        $item['sale']           = 1;
        $item['requisition']    = 1;
        $item['adjustment']     = 1;
        $item['production']     = 1;
        $item['prescription']   = 1;
    } else {
        $item['stock']          = 0;
        $item['purchase']       = 0;
        $item['sale']           = 0;
        $item['requisition']    = 0;
        $item['adjustment']     = 0;
        $item['production']     = 0;
        $item['prescription']   = 0;
    }

    $item['pricelists'] = $pricelists;
    $items[]            = $item;
}

// Import items
foreach($items as $item) {

    // Check duplicate
    $item_name              = $item['name'];
    $sql                    = "SELECT name FROM inventory_items WHERE name = '$item_name'";
    foreach($app->read($sql) as $found_item) {
        echo $app->renderAlertWarning('Item already exists', $item_name);
        continue 2;
    }

    if(in_array($item['type'], array('Asset', 'Product', 'Pharmacy', 'Reagent', 'Stationery'))) {
        $type = 'Product';
    } else {
        $type = 'Service';
    }

    $create_data            = array (
        'id'                => 0,
        'type'              => $type,
        'subtype'           => $item['type'],
        'category'          => '',
        'subcategory'       => '',
        'code'              => '',
        'name'              => $app->formatName($item['name']),
        'name_alt'          => '',
        'details'           => '',
        'uom'               => $item['units'],
        'icon'              => 'fa-regular fa-square-dashed',
        'color'             => 'primary',
        'tags'              => '',
        'cost'              => 0,
        'value'             => 0,
        'image_id'          => 0,
        'background_id'     => 0,
        'template_id'       => 0,
        'template'          => '[]',
        'extra'             => '',
        'min_stock'         => 0,
        'max_stock'         => 0,
        'expiry'            => '',
        'stock'             => $item['stock'],
        'purchase'          => $item['purchase'],
        'sale'              => $item['sale'],
        'requisition'       => $item['requisition'],
        'adjustment'        => $item['adjustment'],
        'production'        => $item['production'],
        'prescription'      => $item['prescription'],
        'qualification'     => '',
        'agegroup'          => '',
        'sex'               => '',
        'dosage'            => '',
        'strength'          => '',
        'strength_units'    => '',
        'volume'            => '',
        'volume_units'      => '',
        'container'         => '',
        'container_units'   => '',
        'code1'             => $item['code1'],
        'code2'             => $item['code2'],
        'code3'             => $item['code3'],
        'code4'             => $item['code4'],
        'code5'             => $item['code5'],
        'code6'             => $item['code6'],
        'code7'             => $item['code7'],
        'code8'             => $item['code8'],
        'code9'             => $item['code9'],
        'prop1'             => $item['prop1'],
        'prop2'             => $item['prop2'],
        'prop3'             => $item['prop3'],
        'prop4'             => $item['prop4'],
        'prop5'             => $item['prop5'],
        'prop6'             => $item['prop6'],
        'prop7'             => $item['prop7'],
        'prop8'             => $item['prop8'],
        'prop9'             => $item['prop9'],
        'f1'                => '',
        'f2'                => '',
        'f3'                => '',
        'f4'                => '',
        'f5'                => '',
        'f6'                => '',
        'f7'                => '',
        'f8'                => '',
        'f9'                => '',
        'created_user_id'   => $user_id,
        'created_datetime'  => $now,
        'modified_user_id'  => 0,
        'modified_datetime' => 0,
        'status'            => 1
    );

    $item_id                = $obj->create($create_data);
    $obj                    = new Item($item_id);

    if($item_id) {
        $obj                = new Item($item_id);
        $obj->createBatch(array('store_id'=>$post->store_id));
        $obj->resetPricelists($item['pricelists']);
        echo $app->renderAlertSuccess('Imported successfully', $item['name']);
    }
}

?>