<?php 

$root               = '../../../../';
$submodule_code     = 'InventoryStockMovement';
require($root.'config/config.php');

$item_filter        = $get->keyword ? " AND inventory_items.name LIKE '%$get->keyword%' " : " ";
$stock_date         = $get->date ? $get->date : date('Y-m-d');
$store_filter       = $get->store_id ? " AND inventory_item_stock.store_id = $get->store_id " : " ";
$subtype_filter     = $get->subtype ? " AND inventory_items.subtype = '$get->subtype' " : " ";
$pricelist_id       = $get->pricelist_id ? $get->pricelist_id : $app->getPricelistByCode('COST')->id;
$stock_store_id     = $get->store_id ? $get->store_id : 0;

$pricelist          = new Pricelist($pricelist_id);

$sql                = "SELECT DISTINCT
inventory_item_stock.item_id
FROM inventory_item_stock
JOIN inventory_items ON inventory_items.id = inventory_item_stock.item_id AND inventory_items.type = 'Product'
WHERE inventory_items.status = 1
$item_filter
$store_filter
$subtype_filter
ORDER BY inventory_items.name ";

$datas              = $app->read($sql);
$data               = $app->pagenate($datas);

$rows               = $data->rows;
$sum_stock          = 0;
$sum_cost           = 0;
$sum_price          = 0;
$sum_profit         = 0;

$options  = array (
    'columns'       => array (
        array('name'=>'Item'),
        array('name'=>'Units', 'css'=>'w-5'),
        array('name'=>'Location', 'css'=>'w-15'),
        array('name'=>'Batch', 'css'=>'w-10'),
        array('name'=>'Expiry', 'css'=>'w-10'),
        array('name'=>'Stock', 'css'=>'w-5 text-end'),
        array('name'=>'Unit cost', 'css'=>'w-8 text-end'),
        array('name'=>'Cost', 'css'=>'w-10 text-end'),
        array('name'=>'Unit price', 'css'=>'w-8 text-end'),
        array('name'=>'Price', 'css'=>'w-10 text-end')
    ),
    'css'           => 'bs-table'
);

$html = $app->renderTableOpen($options);

$count              = 1;
foreach($rows as $row) {
    $item           = new Item($row->item_id);
    $stores_stock   = $item->getClosingStockByStoreAndBatch($stock_store_id, $stock_date);
    $stores         = array_keys($stores_stock);
    $all_batches    = 0;

    $store_stock    = 0;
    $store_cost     = 0;
    $store_price    = 0;
    $store_profit   = 0;

    foreach($stores as $store_id) {
        $store          = new Store($store_id);

        $batch_stock    = 0;
        $batch_cost     = 0;
        $batch_price    = 0;
        $batch_profit   = 0;
        $batch_counter  = 0;
        $stores_counter = 0;

        foreach($stores_stock[$store_id] as $batch_obj) {
    
            if($all_batches++) {
                $item_name      = '';
                $item_uom       = '';
            } else {
                $item_name      = $item->name;
                $item_uom       = $item->uom;
            }
            
            $stock          = $batch_obj['stock'];
            $batch          = $batch_obj['batch'];
            $expiry         = $batch_obj['expiry'];
            $unit_cost      = $batch_obj['unit_cost'];
            $cost           = $stock * $unit_cost;
            $unit_price     = $item->getPrice($pricelist_id);
            $price          = $stock * $unit_price;
            $profit         = $price - $cost;

            $sum_stock      += $stock;
            $sum_cost       += $cost;
            $sum_price      += $price;
            $sum_profit     += $profit;

            $batch_stock    += $stock;
            $batch_cost     += $cost;
            $batch_price    += $price;
            $batch_profit   += $profit;

            $html .= '<tr>
            <td>'.$item_name.'</td>
            <td>'.$item_uom.'</td>
            <td>'.$store->location()->name.'</td>
            <td>'.$batch.'</td>
            <td>'.$app->date($expiry).'</td>
            <td class="text-end">'.number_format($stock).'</td>
            <td class="text-end">'.number_format($unit_cost, 2).'</td>
            <td class="text-end">'.number_format($cost, 2).'</td>
            <td class="text-end">'.number_format($unit_price, 2).'</td>
            <td class="text-end">'.number_format($price, 2).'</td>
            </tr>';
        }

        $store_stock    += $batch_stock;
        $store_cost     += $batch_cost;
        $store_price    += $batch_price;
        $store_profit   += $batch_profit;
    }
}

$html = '<div class="row">
<div class="col-xl-4 col-lg-6 mb-4">
<div class="p-4 rounded bg-secondary border bg-opacity-05">
<h4 class="text-normal text-muted"> <span class="fa-regular fa-tags fa-fw"></span> Cost</h4>
<h4 class="card-title fw-bold my-3">'.$client->currency_code.' '.number_format($sum_cost).'</h4>
</div>
</div>

<div class="col-xl-4 col-lg-6 mb-4">
<div class="p-4 rounded bg-secondary border bg-opacity-05">
<h4 class="text-normal text-muted"> <span class="fa-regular fa-tags fa-fw"></span> Price</h4>
<h4 class="card-title fw-bold my-3">'.$client->currency_code.' '.number_format($sum_price).'</h4>
</div>
</div>

<div class="col-xl-4 col-lg-6 mb-4">
<div class="p-4 rounded bg-secondary border bg-opacity-05">
<h4 class="text-normal text-muted"> <span class="fa-regular fa-tags fa-fw"></span> Gross</h4>
<h4 class="card-title fw-bold my-3">'.$client->currency_code.' '.number_format($sum_profit).'</h4>
</div>
</div>

</div>'.$html;

$html .= '<tr class="fw-semibold">
<td></td>
<td></td>
<td></td>
<td></td>
<td></td>
<td class="text-end">'.number_format($sum_stock).'</td>
<td class="text-end"></td>
<td class="text-end">'.number_format($sum_cost, 2).'</td>
<td class="text-end"></td>
<td class="text-end">'.number_format($sum_price, 2).'</td>
</tr>';

$html .= $app->renderTableClose();

$html .= $data->pagination;

echo $html;

?>