<?php 

class Requisition extends Transaction {
	function __construct($id=0) {	
		$this->table        = 'inventory_requisitions';
		$this->items_table	= 'inventory_requisition_items';
		$this->primary_key  = 'id';
		parent::__construct($id);
	}

	function renderTransactionHeader() {
		global $app, $user;
        
        $html = $this->id ? $this->renderTrackingButton() : '';
		$html .= '<div class="row">';
		$html .= '<div class="col-12 col-lg-6 mb-3">';
		$html .= '<div class="trans-container"> <h5 class="trans-subtitle"><span class="fa-regular fa-store fa-fw"></span>&nbsp; Source store: </h6>';
		$html .= $app->renderInputSelect('trans_store_id', $this->store_id, 'Choose store', 'line-control select-search', 'onchange="transaction.initializeTransaction()" data-parent="trans_subtype" data-source="html/transactions/stores_select.php"', $user->listStores(''));
		$html .= '</div></div>';
		$html .= '<div class="col-12 col-lg-6 mb-3">';
		$html .= '<div class="trans-container"> <h5 class="trans-subtitle"><span class="fa-regular fa-store fa-fw"></span>&nbsp; Destination store: </h6>';
		$html .= $app->renderInputSelect('trans_destination_store_id', $this->personnel_id, 'Choose store', 'line-control select-search', '', $user->listStores(''));
		$html .= '</div></div>';
		$html .= '<div class="col-12 d-none">'.$this->renderPricelistControl($this->getCostPricelistId()).'</div>';
		$html .='</div>';
		return $html;
	}

	function renderModalForm($submodule, $obj, $get) {
        global $app, $client;

        $modal_options  = array(
            'title'     => ucfirst($get->action).' '.$submodule->shortname,
            'left_html'	=> ($get->action == 'create' || $get->action == 'copy' || 1) ? $obj->renderVerticalCreator() : $obj->renderVerticalProgress()
        );
        
        $html           = '';
        if($get->action == 'delete') {
            $html .= $submodule->renderModalDelete($get->id);
        } else {
            $html .= $submodule->renderModalFormOpen($modal_options);
            $html .= $app->renderInputHidden('action', $get->action);
            $html .= $app->renderInputHidden('id', $get->id);
        
            if($get->action == 'create') {
                $html .= $obj->renderTransactionHeader();
                $html .= $obj->renderTransactionItemsEditable('Requested items');
                $html .= $obj->renderItemsControls();
                // $html .= $obj->renderConsumptionControls();
                $html .= $obj->renderNotesControl('Requested');
                $html .= $obj->initializeTransaction();
            } elseif($get->action == 'edit' || $get->action == 'copy') {
                $html .= $obj->renderTransactionHeader();
                $html .= $obj->renderTransactionItemsEditable();
                $html .= $obj->renderItemsControls();
                $html .= $obj->renderNotesControl('Requested');
            } elseif($get->action == 'authorize') {
                $html .= $obj->renderTransactionHeader();
                $html .= $obj->renderTransactionItemsReadonly('Requested items');
                $html .= $obj->renderActionControl( array (
                    array('name'=>'Authorize requisition', 'value'=>'Authorized', 'icon'=>'fa-regular fa-check-circle fa-fw'),
                    array('name'=>'Reject requisition','value'=>'Rejected', 'icon'=>'fa-regular fa-circle-xmark fa-fw')
                ));
                $html .= $obj->renderNotesControl('Authorized');
            } elseif($get->action == 'approve') {
                $html .= $obj->renderTransactionHeader();
                $html .= $obj->renderTransactionItemsReadonly('Authorized items');
                $html .= $obj->renderActionControl( array (
                    array('name'=>'Approve requisition', 'value'=>'Approved', 'icon'=>'fa-regular fa-check-circle fa-fw'),
                    array('name'=>'Reject requisition','value'=>'Rejected', 'icon'=>'fa-regular fa-circle-xmark fa-fw')
                ));
                $html .= $obj->renderNotesControl('Approved');
            } elseif($get->action == 'transfer') {
                $html .= $obj->renderTransactionHeader();
                $html .= $obj->renderTransactionItemsTransfer();
                $html .= $obj->renderActionControl( array (
                    array('name'=>'Transfer selected items', 'value'=>'Transferred', 'icon'=>'fa-regular fa-check-circle fa-fw'),
                    array('name'=>'Return request to first stage','value'=>'Rejected', 'icon'=>'fa-regular fa-circle-xmark fa-fw')
                ));
                $html .= $obj->renderNotesControl('Transferred');
            } elseif($get->action == 'receive') {
                $html .= $obj->renderTransactionHeader();
                $html .= $obj->renderTransactionItemsTransfer('Received items');
                $html .= $obj->renderActionControl( array (
                    array('name'=>'Receive selected items', 'value'=>'Received', 'icon'=>'fa-regular fa-check-circle fa-fw'),
                    array('name'=>'Return request to first stage','value'=>'Rejected', 'icon'=>'fa-regular fa-circle-xmark fa-fw')
                ));
                $html .= $obj->renderNotesControl('Transferred');
            } elseif($get->action == 'open') {
                $html .= $obj->renderTransactionHeader();
                $html .= $obj->renderTransactionItemsReadonly('Selected items');
				$html .= $obj->renderNotesControl('Requested');
            } 
            $html     .= $submodule->renderModalFormClose();
        }
        return $html;
    }

	function renderCreateConfirmation($submodule, $post) {
        global $app, $user;

        $user->checkSubmoduleAccess($submodule, 'create');

        $items                  = $app->uncleanText($post->trans_items);
        $store                  = new Store($post->trans_store_id);
        $create_data            = array (
            'type'              => 'Requisition',
            'store_id'          => $post->trans_store_id,
            'personnel'         => 'Store',
            'personnel_id'      => $post->trans_destination_store_id,
            'name'              => $store->location()->name,
            'requested_notes'   => $post->trans_requested_notes,
            'items'             => $items,
            'cost'              => $post->trans_cost,
            'subtotal'          => $post->trans_subtotal,
        );

        $transaction_id         = $this->createWithMinimumData($create_data);
        if($transaction_id) {
            return $app->renderAlertSuccess($submodule->shortname.' created successfully', 'Success', 'modal_lg', true);
        }
    }

	function renderTableOpen($data) {
        global $app;

        $table_options      = array (
            'columns'       => array (
                array('name'=>'Date', 'css'=>'w-10'),
                array('name'=>'Order No', 'css'=>'w-10'),
                array('name'=>'Source store', 'css'=>''),
                array('name'=>'Destination store', 'css'=>'w-20'),
                array('name'=>'Amount', 'css'=>'w-10 money text-end'),
                array('name'=>'Status', 'css'=>'w-15 text-center'),
                array('name'=>'Options', 'css'=>'w-15 text-end')
            ),
            'css'           => 'bs-table',
            'rows'          => $data->rows
        );

        return $app->renderTableOpen($table_options);
    }

    function renderTableBody($data) {
        global $app;

        $html   = '';

        foreach($data->rows as $row) {
            $transaction_type   = 'Requisition';
            $obj                = new $transaction_type($row->id);
			$source_store 		= $obj->store();
			$destination_store	= new Store($obj->personnel_id);

            $html .=  '<tr>
            <td class="fw-semibold text-muted"> '.$app->datetime($obj->requested_datetime).'</td>
            <td> '.$obj->formatCode($row->id).'</td>
            <td>
            <div class="list-text fw-semibold">'.$source_store->department()->location()->name.'</div>
            <div class="list-subtext">'.$source_store->location()->name.'<div>
            </td>
            <td>
            <div class="list-text fw-semibold">'.$destination_store->department()->location()->name.'</div>
            <div class="list-subtext">'.$destination_store->location()->name.'<div>
            </td>
            <td class="text-end">
            <div class="fw-bold text-muted">'.$app->formatMoney($obj->subtotal).'</div>
            </td>
            <td class="text-center">'.$obj->renderButtonProgress().'</td>
            <td class="text-center text-nowrap">';
        
            if($obj->progress == 'Requested') {
                $html .=  $obj->renderLinkAuthorize();
                $html .=  $obj->renderLinkCopy();
                $html .=  $obj->renderLinkEdit();
                $html .=  $obj->renderLinkDelete();
            } elseif($obj->progress == 'Authorized') {
                $html .=  $obj->renderLinkApprove();
                $html .=  $obj->renderLinkCopy();
                $html .=  $obj->renderLinkDelete();
            } elseif($obj->progress == 'Approved') {   
				$html .=  $obj->renderLinkTransfer();
                $html .=  $obj->renderLinkCopy();
                $html .=  $obj->renderLinkPrint();
            } elseif($obj->progress == 'Transferred') {   
				$html .=  $obj->renderLinkReceive();
                $html .=  $obj->renderLinkCopy();
                $html .=  $obj->renderLinkPrint();
            } elseif($obj->progress == 'Received') {
                $html .=  $obj->renderLinkCopy();
                $html .=  $obj->renderLinkPrint();
            }
        
            $html .= ' </td>
            </tr>';
        }
        return $html;
    }

    function renderTableClose() {
        global $app;
        return $app->renderTableClose();
    }

    function renderTableData($get) {
        global $app;

        $html       = '';
        $table_name = $this->table;

        $start_date = strlen($get->start_date) ? $app->morning($get->start_date) : $app->morning($app->monthStart());
        $end_date   = strlen($get->end_date) ? $app->evening($get->end_date) : $app->evening($app->monthEnd());

        $filters    = $get->source_store ? "AND location_stores.id = $get->source_store " : "";
        $filters    .= $get->destination_store ? "AND personnel_id = $get->destination_store " : "";
        $filters    .= $get->progress ? "AND $table_name.progress = '$get->progress'" : "";

        $sql         = "SELECT $table_name.id FROM $table_name 
        JOIN location_stores ON location_stores.id = $table_name.store_id
        JOIN location_departments ON location_departments.id = location_stores.department_id 
        WHERE $table_name.status = 1
        AND $table_name.created_datetime BETWEEN '$start_date' AND '$end_date'
        $filters 
        ORDER BY id DESC ";

        $data       = $app->pagenate($app->read($sql));
        $html       .= $this->renderTableOpen($data);
        $html       .= $this->renderTableBody($data);
        $html       .= $this->renderTableClose();
        $html       .= $data->pagination;

        return $html;
    }

	function renderEditConfirmation($submodule, $post) {
        global $app, $user;

        $user->checkSubmoduleAccess($submodule, 'edit');   

        $items                  = $app->uncleanText($post->trans_items);
        $store                  = new Store($post->trans_store_id);

        $update_data            = array (
            'store_id'          => $post->trans_store_id,
			'personnel'         => 'Store',
            'personnel_id'      => $post->trans_destination_store_id,
            'requested_notes'   => $post->trans_requested_notes,
            'items'             => $items,
            'cost'              => $app->unformatMoney($post->trans_subtotal),
            'subtotal'          => $app->unformatMoney($post->trans_subtotal)
        );
        $updated                = $this->update($update_data);
        if($updated) {
            $this->resetItems(json_decode($items));
            return $app->renderAlertSuccess($submodule->shortname.' updated successfully', 'Success', 'modal_lg', 1);
        }
    }

	function renderTransactionItemsReadonly($title = 'Items in your cart') {
		global $app;

		$html = '<div class="trans-container my-4"> <h5 class="trans-subtitle"><span class="fa-regular fa-cart-circle-check fa-fw"></span>&nbsp;'.$title.':</h5>
		<div class="card p-4 my-4 bg-primary" style="--bs-bg-opacity: .05;">
        <div class="table-responsive">
		<table class="table table-hover align-middle">
		<thead class="h5">
		<tr>
		<th class="ps-0">Item name</th>
		<th class="w-20 text-end">Price</th>
		<th class="w-20 text-end">Amount</th>
		</tr>
		</thead>
		<tbody id="trans_items_container">';

		foreach($this->items() as $item) {
			$item_obj		= new Item($item->item_id);
			$source_stock	= $item_obj->getStock($this->store_id);
            $dest_stock		= $item_obj->getStock($this->personnel_id);
			$pack_name 		= $item->size > 1 ? $item->pack.'/'.$item->size : $item->pack;

			$source_stock	= floor($source_stock / $item->size);
			$dest_stock		= floor($dest_stock / $item->size);
			
            $details 		= strlen($item->details) ? '<div class="text-muted"><small>'.$item->details.'</small></div>' : '';
			$html   		.= '<tr>
			<td class="ps-0">
			<b class="text-muted"> '.$item->name.' ('.$pack_name.')</b> '.$details.'
			<div class="row gx-0 my-3 text-muted">
			<div class="col-4 text-center">Source: <br /><b class="text-muted">'.$source_stock.'</b></div>
			<div class="col-4 text-center">Destination: <br /><b class="text-muted">'.$dest_stock.'</b></div>
			<div class="col-4 text-center">Requested: <br /><b class="text-muted">'.$item->requested_quantity.'</b></div>
			</div>
			</td>
			<td class="text-end align-bottom pb-4 text-muted">Cost:<br /><b>'.$app->formatMoney($item->unit_price).'</b></td>
			<td class="text-end align-bottom pb-4 text-muted">Amount:<br /><b>'.$app->formatMoney($item->price).'</b></td>
			</tr>';
		}

		$html .= '</tbody>
		<tfoot>
		<tr>
		<th class="ps-0 fw-semibold">Subtotal</th>
		<th></th>
		<th class="text-end fw-bold">'.$app->renderInputText('trans_subtotal', $app->formatMoney($this->subtotal), 'Subtotal', 'line-control-plaintext text-end fw-bold', 'readonly').'</th>
        </tr>
		</tfoot>
		</table>
		</div>
		</div>
        </div>';
		return $html;
	}

	function renderTransactionItemsTransfer($title = 'Transfer items') {
		global $app;

		$html = '<div class="trans-container my-4"> <h5 class="trans-subtitle"><span class="fa-regular fa-store fa-fw"></span>&nbsp;'.$title.':</h5>
		<div class="card p-4 my-4 bg-primary" style="--bs-bg-opacity: .05;">
        <div class="table-responsive">
		<table class="table table-hover align-middle">
		<thead>
		<tr>
		<th class="w-5 p-1">
		<div class="form-check form-switch">
		<input role="button" class="form-check-input toggle-child" data-child-class="trans-item" type="checkbox" id="trans_check_all" checked disabled onclick="return false" />
		</div>
		</th>
		<th class="h5 ps-0">Item name and units</th>
		<th class="h5 w-20">Batch</th>
		<th class="h5 w-20">Expiry</th>
		</tr>
		</thead>
		<tbody id="trans_items_container">';

		foreach($this->items() as $item) {
			$item_obj		= new Item($item->item_id);
			$source_stock	= $item_obj->getStock($this->store_id);
            $dest_stock		= $item_obj->getStock($this->personnel_id);
			$pack_name 		= $item->size > 1 ? $item->pack.'/'.$item->size : $item->pack;

			$source_stock	= floor($source_stock / $item->size);
			$dest_stock		= floor($dest_stock / $item->size);
			
            $details 		= strlen($item->details) ? '<div class="text-muted"><small>'.$item->details.'</small></div>' : '';
			
			$html   		.= '<tr>
			<td class="p-0 ps-1 pt-2 align-top">
			<div class="form-check form-switch">
			<input name="trans_itemss[checked][]" value="'.$item->item_id.'" role="button" class="form-check-input trans-item" type="checkbox" id="trans_check_'.$item->item_id.'" checked disabled onclick="return false" />
			</div>
			</td>
			<td class="ps-0">
			<b class="text-muted"> '.$item->name.' ('.$pack_name.')</b> '.$details.'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][transaction_item_id]', $item->id).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][item_id]', $item->item_id).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][name]', $item->name).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][uom]', $item->uom).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][pack]', $item->pack).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][size]', $item->size).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][unit_price]', $item->unit_price).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][quantity]', $item->quantity).'
			<div class="row gx-0 my-3 text-muted">
			<div class="col-4 text-center">Source: <br /><b class="text-muted">'.$source_stock.'</b></div>
			<div class="col-4 text-center">Destination: <br /><b class="text-muted">'.$dest_stock.'</b></div>
			<div class="col-4 text-center">Requested: <br /><b class="text-muted">'.$item->requested_quantity.'</b></div>
			</div>
			</td>
			<td class="align-bottom pb-3">'.$app->renderInputSelect('trans_items['.$item->item_id.'][batch]', '', '', 'data-list', '', $item_obj->listBatch($this->store_id)).'</td>
			<td class="align-bottom pb-3">'.$app->renderInputText('trans_items['.$item->item_id.'][expiry]', '', '', 'date-picker').'</td>
			</tr>';
		}

		$html .= '</tbody>
		</table>
		</div>
		</div>
        </div>';
		return $html;
	}

	function renderTransferConfirmation($submodule, $post) {
		global $app;

        if($post->trans_action == 'Transferred') {
            if($this->store()->location()->tagged('requisitions-auto-transfer')) {
                $this->transferTransactionItems ($submodule, $post);
            }
            
            $this->changeProgress('Transferred');
            echo $app->renderAlertSuccess('Items transferred successfully', 'Success', 'modal_lg', 1);
        } elseif($post->trans_action == 'Rejected') {
            if($this->changeProgress('Authorized', 'authorized_quantity', $post->trans_transferred_notes)) {
                return $app->renderAlertSuccess($submodule->shortname.' returned to approve stage', 'Success', 'modal_lg', 1);
            }
        } else {
            die($app->renderAlertError('Please choose action'));
        }     
    }

	function transferTransactionItems($submodule, $post) {
        global              $app, $user_id, $now;

        if($post->trans_action == 'Transferred' || $post->trans_action == 'Received') {

            $source_store_id    			    = $this->store_id;
            $dest_store_id    				    = $this->personnel_id;
            $transferred_items     			    = $post->trans_items;
            $transferred_item_ids			    = array_keys($transferred_items);
            $transferring_items    			    = array();

			foreach($transferred_item_ids as $transferred_item_id) {
                $transferred_item               = $transferred_items[$transferred_item_id];
                $transferred_item['store_id']   = $this->store_id;
                $transferred_item['parent']     = $this->type; 
                $transferred_item['parent_id']  = $this->id; 
                $transferring_items[]           = $transferred_item;
            }

            // Check stock availability in source store
            foreach($transferred_items as $adjust_options) {
                $item_obj                       = new Item($adjust_options['item_id']);
                $transaction_item               = new TransactionItem('Requisition', $this->id, $adjust_options['transaction_item_id']);
                $transaction_item->batch        = $adjust_options['batch'];
                $adjust_options['adjust']       = false;
                $adjust_options['item']         = $transaction_item;
                $adjust_options['quantity']     = $adjust_options['quantity'] * -1;
                $adjusted                       = $item_obj->adjustStock($adjust_options);
            }

            // Make sure atleast one item submitted
            if(!$transferring_items) die($app->renderAlertError('Please submit at least one item to transfer'));

            // Adjust stock from source store and destination store
            foreach($transferring_items as $adjust_options) {

                $item_obj                       = new Item($adjust_options['item_id']);
                $transaction_item               = new TransactionItem('Requisition', $this->id, $adjust_options['transaction_item_id']);
                $transaction_item->batch        = $adjust_options['batch'];
                $adjust_options['adjust']       = true;
                $adjust_options['item']         = $transaction_item;
				$quantity 					    = $adjust_options['quantity'];

				// Source store
                $adjust_options['store_id']     = $source_store_id;
                $adjust_options['quantity']     = $quantity * -1;
                $adjusted_batches               = $item_obj->adjustStock($adjust_options);

				// Destination store
                foreach($adjusted_batches as $adjusted_batch) {
                    $adjust_options['store_id'] = $dest_store_id;
                    $adjust_options['quantity'] = abs($adjusted_batch['stock']);
                    $adjust_options['requested_quantity'] = abs($adjusted_batch['stock']);
                    $adjust_options['cost']     = abs($adjusted_batch['stock']) * abs($adjust_options['quantity']);
                    $adjust_options['batch']    = $adjusted_batch['batch'];
                    $adjust_options['expiry']   = $adjusted_batch['expiry'];
                    $adjusted                   = $item_obj->adjustStock($adjust_options);
                }

                $transaction_item->update (
                    array (
                        'progress'          	=> 'Transferred',
                        'transferred'          	=> 1,
                        'transferred_quantity' 	=> $quantity,
                        'transferred_user_id'  	=> $user_id,
                        'transferred_datetime' 	=> $now
                    )
                );
            }
        } elseif($post->trans_action == 'Rejected') {
            if($this->changeProgress('Approved', 'approved_quantity', $post->trans_authorized_notes)) {
                return $app->renderAlertSuccess($submodule->shortname.' returned to approve stage', 'Success', 'modal_lg', 1);
            }
        } else {
			die($app->renderAlertError('Please choose action'));
		}
    }

	function renderReceiveConfirmation($submodule, $post) {
		global $app;

		if(!$this->store()->location()->tagged('requisitions-auto-transfer')) {
			$this->transferTransactionItems ($submodule, $post);
		}

		$this->changeProgress('Received');
		echo $app->renderAlertSuccess('Items received successfully', 'Success', 'modal_lg', 1);
    }

    function renderPDF($title='Transaction') {
        global $app;

        $transaction_progress   = array ('created', 'authorized', 'approved', 'transferred', 'received');

        $html = $app->renderDocumentHeader();

        $html .= '<table class="w-100 py-5">
        <tr><td class="w-100 bg-light border"><h2 class="text-center">Internal Stock Transfer</h3></td></tr>
        </table>
        
        <table class=""><tr><td></td></tr></table>
        
        <table class="w-100 p-7 border bg-light">
        <tr>
        <td class="w-50 border-left border-top border-bottom"><h3>Requisition number</h3></td>
        <td class="w-50 border-right border-top border-bottom text-end"><h3>'.$this->formatCode($this->id).'</h3></td>
        </tr>
        </table>
        
        <table class="w-100 p-7 border">
        
        <tr class="">
        <td class="w-20 border-left border-top border-bottom">Source store</td>
        <td class="w-30 border-top border-bottom text-muted">'.$this->store()->location()->name.'</td>
        <td class="w-20 border-top border-bottom">Destination store:</td>
        <td class="w-30 border-top border-bottom border-right text-end text-muted">'.$this->store('personnel_id')->location()->name.'</td>
        </tr>';

        foreach($transaction_progress as $progress) {
            $action     = ucfirst($progress).' by';
            $username   = $this->personnel($progress.'_user_id')->person()->name;
            $datetime   = $progress.'_datetime';
            $datetime   = $app->datetime($this->$datetime);

            $html .= '<tr>
            <td class="w-20 border-top">'.$action.'</td>
            <td class="w-30 border-top text-muted">'.$username.'</td>
            <td class="w-25 border-top">Datetime '.$progress.': </td>
            <td class="w-25 border-top text-end text-muted">'.$datetime.'</td>
            </tr>';
        }

        $html .= '</table>
        
        <br /><br /><table class="w-100 p-7">
        <tr>
        <td class="w-50 bg-light border-left border-top border-bottom"><h3>Requisition Items </h3></td>
        <td class="w-50 bg-light border-right border-top border-bottom text-end"><h3> '.count($this->items()).'</h3></td>
        </tr>
        </table><br /><br />
        
        <table class="w-100 p-7 border">
        
        <tr class="bg-light">
        <td class="fw-bold text-center h4 border-top border-bottom border-left" style="width:6.5%">S/N</td>
        <td class="fw-bold h4 border-top border-bottom" style="width:28%">Item name</td>
        <td class="w-10 fw-bold h4 border-top border-bottom">Units</td>
        <td class="w-10 fw-bold h4 border-top border-bottom">Batch</td>
        <td class="w-12 fw-bold h4 border-top border-bottom">Expiry</td>
        <td class="w-8 fw-bold h4 border-top border-bottom text-end">Qty</td>
        <td class="w-12 fw-bold h4 border-top border-bottom text-end">Price</td>
        <td class="w-12 fw-bold h4 border-top border-bottom border-right text-end">Amount</td>
        </tr>';

        $count  		= 1;
		$total_price 	= 0;
        foreach($this->items() as $item) {
            $html .= '<tr>
            <td class="text-center border-top">'.$count++.'.</td>
            <td class="border-top">'.$item->name.'</td>
            <td class="border-top">'.$item->uom.'</td>
            <td class="border-top">'.$item->batch.'</td>
            <td class="border-top">'.$item->expiry.'</td>
            <td class="text-end border-top">'.$item->quantity.'</td>
            <td class="text-end border-top">'.number_format($item->unit_price).'</td>
            <td class="text-end border-top">'.number_format($item->price).'</td>
            </tr>';
			$total_price += $item->price;
        }

        $html .= '<tr class="fw-bold">
		<td class="border-top"></td>
		<td class="border-top">Total amount</td>
		<td class="border-top"></td>
		<td class="border-top"></td>
		<td class="border-top"></td>
		<td class="border-top"></td>
		<td class="border-top"></td>
		<td class="text-end border-top">'.number_format($total_price).'</td>
		</tr></table>
        
        <br /><br />
        
        <table class="w-100 p-7 border" style="page-break-inside:avoid;">
        <tr class="bg-light">
        <td class="w-25 text-center border-top border-bottom"><h3>Requested by:</h3></td>
        <td class="w-25 text-center border-top border-bottom"><h3>Authorized by:</h3></td>
        <td class="w-25 text-center border-top border-bottom"><h3>Transferred by:</h3></td>
        <td class="w-25 text-center border-top border-bottom"><h3>Received by:</h3></td>
        </tr><tr>
        <td class="w-25 text-center"><br /> <br /> <br /> ..................................<br />'.$this->personnel('created_user_id')->person()->name.'<br /><b>'.$app->date($this->created_datetime).'</b></td>
        <td class="w-25 text-center"><br /> <br /> <br /> ..................................<br />'.$this->personnel('authorized_user_id')->person()->name.'<br /><b>'.$app->date($this->authorized_datetime).'</b></td>
        <td class="w-25 text-center"><br /> <br /> <br /> ..................................<br />'.$this->personnel('transferred_user_id')->person()->name.'<br /><b>'.$app->date($this->transferred_datetime).'</b></td>
        <td class="w-25 text-center"><br /> <br /> <br /> ..................................<br />'.$this->personnel('received_user_id')->person()->name.'<br /><b>'.$app->date($this->received_datetime).'</b></td>
        </tr>
        </table>';

        $app->renderPDF($html, 'Internal Stock Transfer');
    }
}

?>