<?php 

$root           = '../../../';
$submodule_code = 'items';
require($root.'config/config.php');

$subtype        = isset($get->subtype) ? $get->subtype : '';
$obj            = new Item($get->id, $subtype);
$store          = new Store($get->store);

$item_type      = $obj->subtype.'Type';
$category       = $obj->subtype.'Category';
$subtype_category = $app->getCategoryByCode('ItemType', $subtype);
$type           = $subtype_category->f1 ? 'Product' : 'Service';

if(!strlen($obj->uom)) $obj->uom = 'Each';

if($get->action == 'delete') {

    // Render delete modal
    echo $submodule->renderModalDelete($get->id);
} elseif($get->action == 'remove') {

    // Render remove modal

    $modal_options  = array (
        'title'         => 'Remove item',
        'action_label'  => '<span class="fa fa-check fa-fw"></span> Remove'
    );

    echo $submodule->renderModalFormOpen($modal_options);

    echo '<h5>Do you want to remove '.$obj->name.' from '.$store->location()->name.'? </h5>
    <h5 class="text-muted my-3">The item will still be available from other stores.</h5>';

    echo $app->renderInputHidden('action', $get->action);
    echo $app->renderInputHidden('id', $get->id);
    echo $app->renderInputHidden('store_id', $get->store);
    echo $app->renderInputHidden('type', $type);
    echo $app->renderInputHidden('subtype', $subtype);
    echo $submodule->renderModalFormClose($modal_options);

} elseif($get->action == 'copy2store') {

    // Render remove modal

    $modal_options  = array (
        'title'         => 'Copy item',
        'action_label'  => '<span class="fa fa-check fa-fw"></span> Copy'
    );

    echo $submodule->renderModalFormOpen($modal_options);

    echo '<h5 class="mb-4">Copy item '.$obj->name.' to the following store: </h5>';

    $stores             = $user->listStoresWithAccess('manage-items');
    echo $app->renderInputSelect('store_id', '', '', 'filter-control onchange-reload-children', 'data-style="btn-outline-secondary btn-lg"', $stores);
    echo '<br />';
    
    echo $app->renderInputHidden('action', $get->action);
    echo $app->renderInputHidden('id', $get->id);
    //echo $app->renderInputHidden('store_id', $get->store);
    echo $app->renderInputHidden('type', $type);
    echo $app->renderInputHidden('subtype', $subtype);
    echo $submodule->renderModalFormClose($modal_options);

} else {

    // Render create, read and update modal

    $tabs  = array (
        array ('id'=>'tab1', 'name'=>'Details', 'title'=>'Item details',  'icon'=>'fa-light fa-memo-circle-info', 'css'=>'active'),
    );

    if(in_array($subtype, array('Laboratory', 'Radiology', 'Procedure'))) {
        $tabs[]   = array ('id'=>'tab9', 'name'=>'Template', 'title'=>'Results template',  'icon'=>'fa-light fa-notes-medical'); 
    } elseif($subtype == 'Pharmacy') {
        $tabs[]   = array ('id'=>'tab9', 'name'=>'Prescription', 'title'=>'Prescription details',  'icon'=>'fa-light fa-prescription'); 
    }

    $tabs[] = array ('id'=>'tab3', 'name'=>'Restrictions', 'title'=>'Item restrictions',  'icon'=>'fa-light fa-file-lock');
    $tabs[] = array ('id'=>'tab4', 'name'=>'Attachments', 'title'=>'Add attachments',  'icon'=>'fa-light fa-images');
    $tabs[] = array ('id'=>'tab5', 'name'=>'Cost and price', 'title'=>'Item cost and prices',  'icon'=>'fa-light fa-tags');
    $tabs[] = array ('id'=>'tab6', 'name'=>'Item codes', 'title'=>'Integration codes',  'icon'=>'fa-light fa-qrcode');
    $tabs[] = array ('id'=>'tab7', 'name'=>'Item packs', 'title'=>'Item packs',  'icon'=>'fa-light fa-bag-shopping');
    $tabs[] = array ('id'=>'tab8', 'name'=>'Audit log', 'title'=>'Audit logs',  'icon'=>'fa-light fa-info-circle');

    echo $submodule->renderModalFormOpen(array('tabs'=>$tabs), 0);

    echo $app->renderTabsOpen();

    echo $app->renderInputHidden('action', $get->action);
    echo $app->renderInputHidden('id', $get->id);
    echo $app->renderInputHidden('store_id', $get->store);
    echo $app->renderInputHidden('type', $type);
    echo $app->renderInputHidden('subtype', $subtype);
    echo $app->renderInputHidden('code', $obj->code, '', 'line-control grid-control', 'required');

    echo $app->renderTabsNext('tab1', 'active');

    echo $app->renderInputText('name', $obj->name, '', 'line-control grid-control', 'required');
    echo $app->renderInputText('name_alt', $obj->name_alt, '', 'line-control grid-control', 'required', 'Alt name');
    echo $app->renderInputSelect('item_subtype', $obj->subtype, '', 'line-control grid-control select-search default-none', '', 'settings_categories', 'code', 'name', "type='ItemType'", 'Item type');
    echo $app->renderInputSelect('category', $obj->category, '', 'line-control grid-control select-search default-none', '', 'settings_categories', 'code', 'name', "type='$category'", 'Category');
    echo $app->renderInputTextarea('details', $obj->details, '', 'line-control grid-control', '', 'Details');
    echo $app->renderInputSelect('uom', $obj->uom, '', 'line-control grid-control select-search', '', 'settings_categories', 'code', 'name', "type='ItemUnits'", 'Units');
    echo $app->renderInputColor($obj->color, '', 'line-control grid-control select-search');
    echo $app->renderInputIcon($obj->icon, '', 'line-control grid-control select-search');
    echo $app->renderInputTags($obj->tags, '', 'line-control grid-control');

    echo $app->renderTabsNext('tab2');

    // Item properties

    $properties = array();
    foreach($app->getCategories($obj->subtype.'Property') as $property) {
        $properties[$property->value] = $property;
    }

    for($props = 1; $props < 10; $props++) {
        $prop   = 'prop'.$props;
        if(array_key_exists($prop, $properties)) {
            $input      = $app->getCategoryByCode($obj->subtype.'Property', $prop);
            if($input->f5 == 'Select') {
                $table = $app->stringToObject($input->f6);
                echo $app->renderInputSelect($prop, $obj->$prop, '', 'line-control grid-control select-search', '', $table->table, $table->id, $table->name, $table->condition, $input->name);
            } elseif($input->f5 == 'Number') {
                echo $app->renderInputNumber($prop, $obj->$prop, '', 'line-control grid-control', 'required', $input->name);
            } elseif($input->f5 == 'Textarea') {
                echo $app->renderInputTextarea($prop, $obj->$prop, '', 'line-control grid-control', 'required', $input->name);
            } else {
                echo $app->renderInputText($prop, $obj->$prop, '', 'line-control grid-control', 'required', $input->name);
            }
        } else {
            echo $app->renderInputHidden($prop, $obj->$prop);
        }
    }

    echo $app->renderTabsNext('tab3');

    echo $app->renderInputSelect('qualification', $obj->qualification, '', 'line-control grid-control select-search default-none', '', 'settings_categories', 'code', 'name', "type='QualificationType'", 'Qualification');
    echo $app->renderInputSelect('sex', $obj->sex, '', 'line-control grid-control select-search default-all', '', 'settings_categories', 'code', 'name', "type='SexType'", 'Applicable sex');
    echo $app->renderInputSelect('agegroup[]', $obj->agegroup, '', 'line-control grid-control select-search default-all', 'multiple', 'settings_categories', 'code', 'name', "type='AgegroupType'", 'Applicable age');
    echo $app->renderInputSelect('stock', $obj->stock, '', 'line-control grid-control select-search', '', 'settings_categories', 'code', 'name', "type='YesNo'", 'Can have stock');
    echo $app->renderInputMoney('min_stock', $obj->min_stock, '', 'line-control grid-control', 'data-dependable-inputs=stock;equal;1', 'Minimum_stock');
    echo $app->renderInputMoney('max_stock', $obj->max_stock, '', 'line-control grid-control', 'data-dependable-inputs=stock;equal;1', 'Maximum_stock');
    echo $app->renderInputSelect('purchase', $obj->purchase, '', 'line-control grid-control select-search', '', 'settings_categories', 'code', 'name', "type='YesNo'", 'Can be purchased');
    echo $app->renderInputSelect('production', $obj->production, '', 'line-control grid-control select-search', '', 'settings_categories', 'code', 'name', "type='YesNo'", 'Can be produced');
    echo $app->renderInputSelect('adjustment', $obj->adjustment, '', 'line-control grid-control select-search', '', 'settings_categories', 'code', 'name', "type='YesNo'", 'Can be adjusted');
    echo $app->renderInputSelect('requisition', $obj->requisition, '', 'line-control grid-control select-search', '', 'settings_categories', 'code', 'name', "type='YesNo'", 'Can be ordered');
    echo $app->renderInputSelect('sale', $obj->sale, '', 'line-control grid-control select-search', '', 'settings_categories', 'code', 'name', "type='YesNo'", 'Can be sold');
    echo $app->renderInputSelect('expiry', $obj->expiry, '', 'line-control grid-control select-search', '', 'settings_categories', 'code', 'name', "type='YesNo'", 'Can expire');
    
    echo $app->renderTabsNext('tab4');

    echo $app->renderAttachmentsPreviewEdit(array('code'=>'Item', 'id'=>$get->id));

    
    
    
    echo $app->renderTabsNext('tab5');


    $table_options  = array(
        'columns'   => array (
            array('name'=>'Pricelist', 'css'=>''),
            array('name'=>'Account', 'css'=>'w-25 text-end'),
            array('name'=>'Pricemode', 'css'=>'w-10 text-end'),
            array('name'=>'Margin', 'css'=>'w-10 text-end'),
            array('name'=>'Price', 'css'=>'w-15 text-end')
        ),
        'css'       => 'table-containerless'
    );

    echo '<div class="card p-4 bg-primary" style="--bs-bg-opacity: .05;">';

    echo $app->renderTableOpen($table_options);

    $pricelists     = $obj->getPricelists();
    $accounts       = $app->read("SELECT id value, name FROM accounting_accounts");

    foreach($pricelists as $pricelist) {
        echo $app->renderInputHidden('pricelists['.$pricelist->code.'][id]', $pricelist->id);
        echo $app->renderInputHidden('pricelists['.$pricelist->code.'][code]', $pricelist->code);
        echo '<tr>';
        echo '<td class="fw-bold text-muted">'.$pricelist->code.'</td>';
        echo '<td style="max-width:100px">'.$app->renderInputSelect('pricelists['.$pricelist->code.'][account_id]', $pricelist->account_id, '', 'select-search line-control ', '', $accounts).'</td>';
        echo '<td>'.$app->renderInputSelect('pricelists['.$pricelist->code.'][pricemode]', $pricelist->pricemode, '', 'select-search line-control ', '', 'settings_categories', 'code', 'name', "type='PriceMode'").'</td>';
        echo '<td>'.$app->renderInputMoney('pricelists['.$pricelist->code.'][margin]', $app->formatMoney($pricelist->margin), '', 'line-control text-end').'</td>';
        echo '<td>'.$app->renderInputMoney('pricelists['.$pricelist->code.'][price]', $app->formatMoney($pricelist->price), '', 'line-control text-end', '', $pricelist->name).'</td>';
        echo '</tr>';
    }

    echo $app->renderTableClose($table_options);

    echo '</div>';

    echo $app->renderVerticalPad('0px');


    
    
    echo $app->renderTabsNext('tab6');

    // Item codes
    
    $codes = array();
    foreach($app->getCategories('ItemCode', '') as $code) {
        $codes[$code->f1] = $code;
    }

    for($cods = 1; $cods < 10; $cods++) {
        $code   = 'code'.$cods;
        
        if(array_key_exists($code, $codes)) {
            $name       = $codes[$code]->name;
            $input      = $app->getCategoryByCode('ItemCode', $code);
            if($input->f5 == 'Select') {
                $table = $app->stringToObject($input->f6);
                echo $app->renderInputSelect($code, $obj->$code, '', 'line-control grid-control select-search', '', $table->table, $table->id, $table->name, $table->condition, $name);
            } elseif($input->f5 == 'Number') {
                echo $app->renderInputNumber($code, $obj->$code, '', 'line-control grid-control', 'required', $name);
            } elseif($input->f5 == 'Textarea') {
                echo $app->renderInputTextarea($code, $obj->$code, '', 'line-control grid-control', 'required', $name);
            } else {
                echo $app->renderInputText($code, $obj->$code, '', 'line-control grid-control', 'required', $name);
            }
        } else {
            echo $app->renderInputHidden($code, $obj->$code);
        }
    }

    echo $app->renderTabsNext('tab7');

    echo '<div class="bg-light rounded p-3 mb-3">
    <span class="fa-light fa-info-circle fa-fw"></span>
    Enter pack code, pack name and pack size as shown in example on the first line. 
    If you dont fill any of the field in the line, that line will be ignored.</div>';

    echo '<div class="card p-4 bg-primary" style="--bs-bg-opacity: .05;">';

    echo '<div class="row fw-bold text-muted">';
    echo '<div class="col-1 py-1">S/N</div>
    <div class="col-3 py-1">Pack code</div>
    <div class="col-5 py-1">Pack name</div>
    <div class="col-3 py-1">Pack size</div>';

    $counter    = 0;

    echo '<div class="col-1 pt-2">Eg.</div>
        <div class="col-3 py-2">'.$app->renderInputText('example_name', 'Dz', '', 'fw-bold line-control', 'disabled readonly').'</div>
        <div class="col-5 py-2">'.$app->renderInputText('example_code', 'Dozen', '', 'fw-bold line-control', 'disabled readonly').'</div>
        <div class="col-3 py-2">'.$app->renderInputMoney('example_size', 12, '', 'fw-bold line-control', 'disabled readonly').'</div>';
        $counter++;

    foreach($obj->getPacks() as $pack) {
        echo '<div class="col-1 pt-2">'.$counter.'.</div>
        <div class="col-3 py-2">'.$app->renderInputText('packs['.$counter.'][code]', $pack->code, '', 'line-control').'</div>
        <div class="col-5 py-2">'.$app->renderInputText('packs['.$counter.'][name]', $pack->name, '', 'line-control').'</div>
        <div class="col-3 py-2">'.$app->renderInputMoney('packs['.$counter.'][size]', $pack->size, '', 'line-control').'</div>';
        $counter++;
    }

    while($counter < 7) {
        echo '<div class="col-1 pt-2">'.$counter.'.</div>
        <div class="col-3 py-2">'.$app->renderInputText('packs['.$counter.'][code]', '', '', 'line-control').'</div>
        <div class="col-5 py-2">'.$app->renderInputText('packs['.$counter.'][name]', '', '', 'line-control').'</div>
        <div class="col-3 py-2">'.$app->renderInputMoney('packs['.$counter.'][size]', '', '', 'line-control').'</div>';
        $counter++;
    }

    echo '</div>';

    echo '</div>';




    //echo $app->renderInputFields($obj, 9, '', 'line-control grid-control');

    echo $app->renderTabsNext('tab8');

    echo $obj->renderAuditFields();

    echo $app->renderTabsNext('tab9');

    echo $app->renderInputSelect('prescription', $obj->prescription, '', 'line-control grid-control select-search', '', 'settings_categories', 'code', 'name', "type='YesNo'", 'Can be prescribed');
    echo $app->renderInputSelect('dosage', $obj->dosage, 'Format of the drug, the nature of the content', 'placeholder-underline line-control grid-control select-search', 'data-dependable-inputs=prescription;equal;1', 'settings_categories', 'code', 'name', "type='DosageForm'", 'Dosage form');
    echo $app->renderInputNumber('strength', $obj->strength, 'The quantity of active ingredients in 1 unit volume', 'placeholder-underline line-control grid-control', 'data-dependable-inputs=prescription;equal;1', 'Dosage strength');
    echo $app->renderInputSelect('strength_units', $obj->strength_units, 'Units of active ingredients ', 'placeholder-underline line-control grid-control select-search', 'data-dependable-inputs=prescription;equal;1', 'settings_categories', 'code', 'name', "type='DosageUnits'", 'Dosage units');
    echo $app->renderInputNumber('volume', $obj->volume, 'The volume of 1 unit of medication. (If not sure enter 1)', 'placeholder-underline line-control grid-control', 'data-dependable-inputs=prescription;equal;1', 'Unit volume');
    echo $app->renderInputSelect('volume_units', $obj->volume_units, 'Units of active ingredients', 'placeholder-underline line-control grid-control select-search', 'data-dependable-inputs=prescription;equal;1', 'settings_categories', 'code', 'name', "type='DosageUnits'", 'Volume units');
    echo $app->renderInputNumber('container', $obj->container, 'Stock container volume', 'placeholder-underline line-control grid-control', 'data-dependable-inputs=prescription;equal;1', 'Container volume');
    echo $app->renderInputText('container_units', $obj->container_units, 'Stock container unit, same as store units', 'placeholder-underline line-control grid-control readonly', 'data-dependable-inputs=prescription;equal;1 readonly', 'Container units');
    
    //echo $app->renderVerticalPad('10px');

    echo $app->renderTabsClose();
    echo $submodule->renderModalFormClose();

}

?>