<?php 

class Store extends Crud {
	function __construct($id=0) {		
		$this->table        = 'location_stores';
		$this->primary_key  = 'id';
		parent::__construct($id);
	}

	function location() {
		return new Location($this->location_id);
	}

	function department() {
		return new Department($this->department_id);
	}

	function listItemsStock($reset_transaction_items = true, $items_type=array(), $get=null) {

		global $app;
		$items						= array();
		$list 						= array();
		$types_condition 			= "";
		$scheme_id 					= isset($get->scheme_id) ? $get->scheme_id : 0;
		$scheme						= new Scheme($scheme_id);
		$pricelist 					= $scheme->pricelist();

		if($items_type) {
			$types					= array();
			foreach($items_type as $type) {
				$types[] 			= "'$type'";
			}
			$types 					= implode(',', $types);
			$types_condition		= " AND inventory_items.subtype IN($types) ";
		}

		$sql						= "SELECT inventory_item_stock.item_id, SUM(inventory_item_stock.stock) stock 
		FROM inventory_item_stock 
		JOIN inventory_items ON inventory_items.id = inventory_item_stock.item_id
		WHERE inventory_item_stock.store_id = $this->id 
		AND inventory_items.status = 1
		$types_condition
		GROUP BY inventory_item_stock.item_id 
		ORDER BY inventory_items.name ";

		foreach($app->read($sql) as $row) {
			$item					= new Item($row->item_id);
			$item->pricelists 		= $item->getPricelists();
			$item->packs 			= $item->getPacks();
			$items[$row->item_id]	= $item;
			$stock					= $item->stock ? ($row->stock ? ' &nbsp; (Stock: '.$row->stock.')' : ' &nbsp; (Out of stock)') : '';
			$pricelist_code 		= $pricelist->code;

			if(isset($item->pricelists->$pricelist_code)) {
				$pricelist			= $item->pricelists->$pricelist_code;
				if($pricelist->pricemode == 'Disabled') {
					$option			= array(
						'name'	=> $item->name.$stock, 
						'value'	=> $item->id,
						'attr'	=> 'disabled',
						'html'	=> $item->name.$stock.' (Disabled)'
					);
				} elseif($row->stock || $item->type != 'Product') {
					$option			= array (
						'name'	=> $item->name.$stock, 
						'value'	=> $item->id,
						'html'	=> $item->name.$stock
					);
				} elseif($pricelist_code == 'COST') {
					$option			= array (
						'name'	=> $item->name.$stock, 
						'value'	=> $item->id,
						'html'	=> $item->name.$stock
					);
				} else {
					$option			= array (
						'name'	=> $item->name.$stock, 
						'value'	=> $item->id,
						'attr'	=> 'disabled',
						'html'	=> $item->name.$stock
					);
				}
			} else {
				$option				= array(
					'name'	=> $item->name.$stock, 
					'value'	=> $item->id
				);
			}

			$list[]				= $option;
		}

		if($reset_transaction_items) echo '<script> if(transaction) transaction.store_items = '.$app->array2json($items).' </script>';

		return $list;
	}


	function getItemsWithStock() {
		global $app;

		$sql						= "SELECT 
		inventory_item_stock.item_id, 
		inventory_items.type, 
		inventory_items.name, 
		inventory_items.uom, 	
		SUM(inventory_item_stock.stock) stock 
		FROM inventory_item_stock 
		JOIN inventory_items ON inventory_items.id = inventory_item_stock.item_id
		WHERE inventory_item_stock.store_id = $this->id 
		GROUP BY inventory_item_stock.item_id 
		ORDER BY inventory_items.name ";

		return $app->read($sql);
	}

	function getAverageStockConsumption($period) {
		global $app;

		$items	= array();

		$sql 	= "SELECT item_id, SUM(qty) as qty 
		FROM inventory_item_movements 
		WHERE store_id = $this->id 
		GROUP BY item_id ";

		foreach($app->read($sql) as $row) {
			$item				= new Item($row->item_id);
			$pricelists 		= $item ->getPricelists();
			$cost 				= $pricelists->COST->price;

			$item->item_id		= $row->item_id;
			$item->pack			= $item->uom;
			$item->size			= 1;
			$item->qty			= $row->qty;
			$item->quantity		= $item->qty;
			$item->unit_price	= $cost;
			$item->price		= $item->unit_price * $item->qty;

			$items[]			= $item;
		}

		return $items;
	}
}

?>