<?php 

class Employee extends Crud {
	function __construct($id=0) {		
		$this->table        = 'personnel_employees';
		$this->primary_key  = 'id';
		parent::__construct($id);
	}

	function person($person_id='') {
		$person_id 	= strlen($person_id) ? $person_id : $this->person_id;
		$person		= new Person($person_id);
		$person->personnel = 'Employee';
		return $person;
	}

	function department() {
		return new Department($this->department_id);
	}

	function picture($attachment_root='') {
		global $app, $root, $r;
		if(!$attachment_root) $attachment_root = $r;
		
		$attachment = $app->getAttachmentByCode('Employee', $this->id, 'ProfilePicture');
		if($attachment->exists()) {
			return $attachment->path($attachment_root);
		} else {
			$person		= $this->person();
			$age_string = $person->age();
			$sex		= strtolower($person->sex);
			$picture 	= 'default.png';
			$ages		= explode(' ', $age_string);
			
			if((count($ages) == 2) && ($sex == 'male' || $sex == 'female')) {
				$age_number		= $ages[0];
				$age_units		= $ages[1];
				$agegroup 	= 'baby';
				if($age_units 	== 'year' || $age_units 	== 'years') {
					if($age_number > 5) 	$agegroup = 'child';
					if($age_number > 18) 	$agegroup = 'adult';
					if($age_number > 60) 	$agegroup = 'old';
				}
				$picture	= $sex.'-'.$agegroup.'.png';
			}
			return $r.'assets/images/employees/'.$picture;
		}
	}

	function signature() {
		global $app, $root;
		$attachment = $app->getAttachmentByCode('Employee', $this->id, 'EmployeeSignature');
		if($attachment->exists()) return $attachment->renderOpenPath();
		return $root.'assets/images/default/signature.png';
	}

	function list() {
		global $app;
		$sql	= "SELECT personnel_employees.id value,
		personnel_persons.name 
		FROM personnel_employees 
		JOIN personnel_persons ON personnel_persons.id = personnel_employees.person_id 
		WHERE personnel_employees.status = 1 ";
		return $app->query($sql);
	}

	function renderBio() {
		$person = $this->person();
		$user 	= $person->user();

		$html = '<img src="'.$this->picture().'" class="rounded-circle mx-auto d-block bg-light my-3" style="width:85%; max-width:185px; border:8px solid white" />
		<div class="px-3 me-2">
		<h4 class="mb-2 text-truncate text-white fw-bold">'.$person->name.'</h4>
		<h5 class="fw-semibold">'.$user->role.'</h5>
		<h6>'.ucwords($person->age()).' '.$person->sex.'</h6>
		</div>';

		return $html;
	}

	function renderProfileCard() {
		global $root, $app;
		$color 				= 'secondary';
		$person 			= $this->person();
		$button_options 	= array('id'=>$this->id, 'css'=>'btn btn-'.$color.' btn-lg');
		$buttons			= $app->renderButtonOpen($button_options).''.$app->renderButtonEdit($button_options).''.$app->renderButtonCopy($button_options).''.$app->renderButtonDelete($button_options);
		$person->sex 		= $person->sex ? $person->sex : 'ago';
		$status_category 	= $app->getCategoryByCode('EmployeeStatus', $this->employment_status);

		$html 				= '<div id="card_customer_'.$this->id.'" class="auto-flip bs-tooltip" title="Click card to flip">
		<div class="shadow shadow-lg card text-center front">
		<div class="card-img-top bg-'.$color.' text-white py-4" style="height:240px; min-height:240px; --bs-bg-opacity:1">
		'.$buttons.'
		</div>
		<img src="'.$this->picture().'" class="rounded-circle mx-auto d-block" style="width:85%; max-width:200px; margin-top:-150px; border:8px solid white">
		<div class="card-body" style="min-height:180px;">
		
		<h5 class="fw-semibold text-truncate">'.$person->name.'</h4>
		<h6 class="text-muted fw-semibold"> '.ucwords($person->age().' '.$person->sex).' </h6>
		<!-- <h6 class="text-muted "> '.$this->department()->location()->name.' </h6> -->
		<h6 class="text-muted fw-normal">'.$this->designation.'</h6>
		<a class="my-2 btn btn-sm btn-outline-'.$status_category->color.' rounded-pill px-3"> <span class="'.$status_category->icon.' fa-fw"></span> '.$status_category->name.' </a>
		
		</div></div>
		
		<div class="shadow card text-center back">
		
		<div class="card-img-top bg-'.$color.' text-white py-4" style="height:240px; min-height:240px">
		<a class="btn btn-'.$color.' bs-tooltip" data-flip="card_customer_'.$this->id.'" title="Flip"> <span class="fa-solid fa-arrow-left"></span> </a>
		<div><h5> Registered </h5></div>
		<div>'.$app->datetime($this->created_datetime).'</div>
		<div>'.$this->createdUser()->person()->name.'</div>
		<div>'.$app->age($this->created_datetime).' ago </div>

		</div>
		<img src="'.$this->picture().'" class="rounded-circle mx-auto d-block bg-light" style="visibility:hidden; width:75%; max-width:190px; margin-top:-120px; border:7px solid white">
		<div class="card-body" style="min-height:180px;">
		<h5><span class="fa-light fa-phone fa-fw"></span></h5>
		<h4 class="fw-light">'.$person->phone.'</h4>
		<div class="fw-light"> '.$person->village()->name.' - '.$person->district()->name.' - '.$person->region()->name.'</div>
		</div></div></div>';
		
		return $html;
	}
}

?>