<?php 

class Procurement extends Transaction {
	function __construct($id=0) {	
		parent::__construct($id);
        if(!$id) $this->urgency = 'Routine';
	}

    function createSupplierIfNotExists($data) {
        $supplier = new Supplier();
        return $supplier->createIfNotExists($data);
    }

    function renderTransactionHeader() {
		global $app, $user;
        
		$html = $this->id ? $this->renderTrackingButton() : '';
        $html .= '<div class="rounded p-4 bg-primary bg-opacity-10">';
		$html .= '<div class="row">';
		$html .= '<div class="col-12 col-lg-6">'.$this->renderUrgencyControl().'</div>';
		$html .= '<div class="col-12 col-lg-6">'.$this->renderStoreControl('Delivery store').'</div>';
		$html .= '<div class="col-12 d-none">'.$this->renderPricelistControl($this->getCostPricelistId()).'</div>';
		$html .='</div>';
		$html .='</div>';
		return $html;
	} 

    function renderModalForm($submodule, $obj, $get) {
        global $app, $client;

        if($this->type == 'PurchaseOrder' || $this->type == 'PurchaseGRN') {
            $modal_options  = array(
                'title'     => ucfirst($get->action).' '.$submodule->shortname,
                //'left_html'	=> ($get->action == 'create' || $get->action == 'copy' || 1) ? '<div id="supplier_bio_container">'.$obj->renderVerticalCreator().'</div>' : 'HERE'
                'left_html'	=> $this->supplier()->renderBio()
            );
        } else {
            $modal_options  = array(
                'title'     => ucfirst($get->action).' '.$submodule->shortname,
                'left_html'	=> ($get->action == 'create' || $get->action == 'copy' || 1) ? $obj->renderVerticalCreator() : $obj->renderVerticalProgress()
            );
        }
        
        $html           = '';
        if($get->action == 'delete') {
            $html .= $submodule->renderModalDelete($get->id);
        } else {
            $html .= $submodule->renderModalFormOpen($modal_options);
            $html .= $app->renderInputHidden('action', $get->action);
            $html .= $app->renderInputHidden('id', $get->id);
        
            if($get->action == 'create') {
                $html .= $obj->renderTransactionHeader();
                $html .= $obj->renderTransactionItemsEditable();
                $html .= $obj->renderItemsControls();
                //$html .= $obj->renderConsumptionControls();
                $html .= $obj->renderNotesControl('Requested');
                $html .= $obj->initializeTransaction();
            } elseif($get->action == 'edit' || $get->action == 'copy') {
                $html .= $obj->renderTransactionHeader();
                $html .= $obj->renderTransactionItemsEditable();
                $html .= $obj->renderItemsControls();
                $html .= $obj->renderNotesControl('Requested');
            } elseif($get->action == 'authorize') {
                $html .= $obj->renderTransactionHeader();
                $html .= $obj->renderTransactionItemsReadonly();
                $html .= $obj->renderActionControl( array (
                    array('name'=>'Authorize purchase request', 'value'=>'Authorized', 'icon'=>'fa-regular fa-check-circle fa-fw'),
                    array('name'=>'Reject purchase request','value'=>'Rejected', 'icon'=>'fa-regular fa-circle-xmark fa-fw')
                ));
                $html .= $obj->renderNotesControl('Authorized');
            } elseif($get->action == 'approve') {
                $html .= $obj->renderTransactionHeader();
                $html .= $obj->renderTransactionItemsReadonly();
                $html .= $obj->renderActionControl( array (
                    array('name'=>'Approve purchase request', 'value'=>'Approved', 'icon'=>'fa-regular fa-check-circle fa-fw'),
                    array('name'=>'Reject purchase request','value'=>'Rejected', 'icon'=>'fa-regular fa-circle-xmark fa-fw')
                ));
                $html .= $obj->renderNotesControl('Approved');
            } elseif($get->action == 'forward') {
                $html .= $obj->renderTransactionHeader();
                $html .= $obj->renderTransactionItemsForward();
                $html .= $obj->renderActionControl( array (
                    array('name'=>'Create LPO for selected items', 'value'=>'Forwarded', 'icon'=>'fa-regular fa-check-circle fa-fw'),
                    array('name'=>'Return request to first stage','value'=>'Rejected', 'icon'=>'fa-regular fa-circle-xmark fa-fw')
                ));

                $requests       = array (
                    array (
                        'name'  => 'Create a new LPO', 
                        'value' => 'New'
                    )
                );
                $requests_sql   = "SELECT id, name, subtotal, created_datetime FROM procurement_orders WHERE progress = 'Requested' ORDER BY id DESC ";
                foreach($app->read($requests_sql) as $row) {
                    $requests[] = array (
                        'name'  => 'Add to order no. '.$row->id.' '.$row->name.' '.$client->currency_code.' '.number_format($row->subtotal).' created '.$app->date($row->created_datetime),
                        'value' => $row->id
                    );
                }
                
                $html .= '<div class="trans-container mb-5"> <h5 class="trans-subtitle">
                <span class="fa-regular fa-bolt fa-fw"></span> Choose if you want to create new or update existing LPO: </h5>';
                $html .= $app->renderInputSelect('trans_order_type', '', 'Choose order type', 'line-control select-search', '', $requests);
                $html .= '</div>';

                $html .= $obj->renderNotesControl('Forwarded');
            } elseif($get->action == 'open') {
                $html .= $obj->renderTransactionHeader();
                $html .= $obj->renderTransactionItemsReadonly();
                $html .= $obj->renderNotesControl('Authorized');
            } 
            $html     .= $submodule->renderModalFormClose();
        }
        return $html;
    }

    function renderTableOpen($data) {
        global $app;

        $table_options      = array (
            'columns'       => array (
                array('name'=>'Date', 'css'=>'w-10'),
                array('name'=>'Request No', 'css'=>'w-10'),
                array('name'=>'Department', 'css'=>''),
                array('name'=>'User', 'css'=>'w-20'),
                array('name'=>'Amount', 'css'=>'w-10 money text-end'),
                array('name'=>'Progress', 'css'=>'w-15 text-center'),
                array('name'=>'Options', 'css'=>'w-15 text-end')
            ),
            'css'           => 'table-striped bs-table',
            'rows'          => $data->rows
        );

        return $app->renderTableOpen($table_options);
    }

    function renderTableBody($data) {
        global $app;

        $html   = '';

        foreach($data->rows as $row) {
            $transaction_type   = $this->type;
            $obj                = new $transaction_type($row->id);

            $html .=  '<tr>
            <td> '.$app->date($obj->requested_datetime).'</td>
            <td> '.$obj->formatCode($row->id).'</td>
            <td>
            <h6>'.$obj->store()->department()->location()->name.'</h6>
            <small class="text-muted">'.$obj->store()->location()->name.'</small>
            </td>
            <td>
            <h6>'.$obj->user('created')->person()->name.'</h6>
            <small class="text-muted">'.$obj->urgency.'</small>
            </td>
            <td class="text-end">
            <div class="fw-bold text-muted">'.$app->formatMoney($obj->subtotal).'</div>
            </td>
            <td class="text-center">'.$obj->renderButtonProgress().'</td>
            <td class="text-center text-nowrap">';
        
            if($obj->progress == 'Requested') {
                $html .=  $obj->renderLinkAuthorize();
                $html .=  $obj->renderLinkCopy();
                $html .=  $obj->renderLinkEdit();
                $html .=  $obj->renderLinkDelete();
            } elseif($obj->progress == 'Authorized') {
                $html .=  $obj->renderLinkApprove();
                $html .=  $obj->renderLinkCopy();
                $html .=  $obj->renderLinkDelete();
            } elseif($obj->progress == 'Approved') {
                if($this->type == 'PurchaseRequisition') $html .=  $obj->renderLinkForward('Request');
                if($this->type == 'PurchaseRequest') $html .=  $obj->renderLinkForward('LPO');
                if($this->type == 'PurchaseOrder') $html .=  $obj->renderLinkForward('GRN');
                if($this->type == 'PurchaseGRN') $html .=  $obj->renderLinkReceive();
                
                $html .=  $obj->renderLinkCopy();
                $html .=  $obj->renderLinkPrint();
            } elseif($obj->progress == 'Forwarded') {
                $html .=  $obj->renderLinkCopy();
                $html .=  $obj->renderLinkPrint();
            }
        
            $html .= ' </td>
            </tr>';
        }
        return $html;
    }

    function renderTableClose() {
        global $app;
        return $app->renderTableClose();
    }

    function renderTableData($get) {
        global $app; 

        $table_name = $this->table;
        $start_date = strlen($get->start_date) ? $app->morning($get->start_date) : $app->morning($app->monthStart());
        $end_date   = strlen($get->end_date) ? $app->evening($get->end_date) : $app->evening($app->monthEnd());

        $filters    = $get->department ? "AND location_stores.department_id = $get->department " : "";
        $filters    .= $get->store ? "AND location_stores.id = $get->store " : "";
        $filters    .= $get->progress ? "AND $table_name.progress = '$get->progress'" : "";
        $filters    .= " AND $table_name.created_datetime BETWEEN '$start_date' AND '$end_date' ";

        $sql         = "SELECT $table_name.id FROM $table_name 
        JOIN location_stores ON location_stores.id = $table_name.store_id
        JOIN location_departments ON location_departments.id = location_stores.department_id 
        WHERE $table_name.name LIKE '%$get->keyword%'
        AND $table_name.status = 1
        $filters 
        ORDER BY id DESC ";

        $data       = $app->pagenate($app->read($sql));
        $html       = '';
        $html       .= $this->renderTableOpen($data);
        $html       .= $this->renderTableBody($data);
        $html       .= $this->renderTableClose();
        $html       .= $data->pagination;

        return $html;
    }

    function renderCreateConfirmation($submodule, $post) {
        global $app, $user;

        $user->checkSubmoduleAccess($submodule, 'create');

        $items                  = $app->uncleanText($post->trans_items);
        $store                  = new Store($post->trans_store_id);
        $create_data            = array (
            'type'              => $this->type,
            'urgency'           => $post->trans_urgency,
            'store_id'          => $post->trans_store_id,
            'name'              => $store->location()->name,
            'requested_notes'   => $post->trans_requested_notes,
            'items'             => $items,
            'cost'              => $post->trans_cost,
            'subtotal'          => $post->trans_subtotal,
        );

        $transaction_id         = $this->createWithMinimumData($create_data);
        if($transaction_id) {
            return $app->renderAlertSuccess($submodule->shortname.' created successfully', 'Success', 'modal_lg', true);
        }
    }

    function renderEditConfirmation($submodule, $post) {
        global $app, $user;

        $user->checkSubmoduleAccess($submodule, 'edit');   

        $items                  = $app->uncleanText($post->trans_items);
        $store                  = new Store($post->trans_store_id);

        $update_data            = array (
            'store_id'          => $post->trans_store_id,
            'requested_notes'   => $post->trans_requested_notes,
            'items'             => $items,
            'cost'              => $app->unformatMoney($post->trans_subtotal),
            'subtotal'          => $app->unformatMoney($post->trans_subtotal)
        );
        $updated                = $this->update($update_data);
        if($updated) {
            $this->resetItems(json_decode($items));
            return $app->renderAlertSuccess($submodule->shortname.' updated successfully', 'Success', 'modal_lg', 1);
        }
    }

    function renderForwardConfirmation($submodule, $post) {
        global $app, $user;

        $forward_type = $post->forward_type;

        if(isset($post->trans_forwarded)) {
            if($post->trans_forward_option) {
                if($post->trans_forward_option == 'New') {
                    
                    // Create new purchase request
                    $user->checkSubmoduleAccess($forward_type, 'create');
    
                    $forward_obj           = new $forward_type();
                    $forward_obj_items     = $this->getItems();
                    $forward_obj_items     = json_encode($forward_obj_items);
                    $forward_obj_data      = array (
                        'type'             => $forward_type,
                        'urgency'          => $this->urgency,
                        'store_id'         => $this->store_id,
                        'name'             => $this->store()->location()->name,
                        'requested_notes'  => $post->trans_forwarded_notes,
                        'items'            => $forward_obj_items,
                        'cost'             => $this->cost,
                        'subtotal'         => $this->subtotal,
                        'progeress'        => 'Requested',
                        'progress_code'    => 1
                    );
    
                    $forward_obj_id        = $forward_obj->createWithMinimumData($forward_obj_data);
                    if($forward_obj_id) {
                        $this->changeProgress('Forwarded');
                        $this->update (
                            array (
                                'parent'   => $forward_type,
                                'parent_id'=> $forward_obj_id
                            )
                        );
                        echo $app->renderAlertSuccess($post->forward_title.' created successfully', 'Success', 'modal_lg', 1);
                    }
                } else {
                    $forward_obj_id        = $post->trans_forward_option;
                    $forward_obj           = new $forward_type($forward_obj_id);
                    $merged_items          = $forward_obj->mergeItems($forward_obj->items(), $this->items());
                    
                    $new_cost              = 0;
                    $new_subtotal          = 0;
                    foreach($merged_items as $item) {
                        $new_cost          += $item->cost;
                        $new_subtotal      += $item->price;
                    }
    
                    $merged = $forward_obj->update (
                        array (
                            'cost'          => ($new_cost),
                            'subtotal'      => ($new_subtotal)
                        )
                    );
                    $forward_obj->resetItems($merged_items);
                    if($merged) {
                        $this->changeProgress('Forwarded');
                        $this->update (
                            array (
                                'parent'    => $forward_type,
                                'parent_id' => $forward_obj_id
                            )
                        );
                        echo $app->renderAlertSuccess($post->forward_title.' updated successfully', 'Success', 'modal_lg', 1);
                    }
                }
            } else {
                die($app->renderAlertError('Please choose where to forward this request'));
            }
        } else {
            die($app->renderAlertError('Please confirm '.strtolower($submodule->shortname).' forwarding'));  
        }
    }

    function renderReceiveConfirmation($submodule, $post) {
        $received = $this->receiveTransactionItems ($submodule, $post);
        return $received;
    }

    function receiveTransactionItems($submodule, $post) {
        global              $app, $user_id, $now;

        if($post->trans_action == 'Accepted') {

            $store_id           = $this->store_id;
            $received_items     = $post->trans_items;
            $received_item_ids  = array_keys($received_items);
            $receiving_items    = array();

            // Validate received quantiy does not exceed requested quantity
            foreach($received_item_ids as $received_item_id) {
                $received_item                      = $received_items[$received_item_id];
                $received_item['store_id']          = $this->store_id;
                $received_item['parent']       = $this->type; 
                $received_item['parent_id']    = $this->id; 
                $receiving_items[]                  = $received_item;
            }

            // Make sure atleast one item submitted
            if(!$receiving_items) die($app->renderAlertError('Please submit at least one item to receive'));

            // Adjust and update received quantities
            foreach($receiving_items as $adjust_options) {
                $item_obj                   = new Item($adjust_options['item_id']);
                $transaction_item           = new TransactionItem($this->type, $this->id, $adjust_options['transaction_item_id']);
                $adjust_options['adjust']   = true;
                $adjust_options['item']     = $transaction_item;
                $adjusted                   = $item_obj->adjustStock($adjust_options);

                $transaction_item->update (
                    array (
                        'progress'          => 'Received',
                        'received'          => 1,
                        'received_quantity' => $adjust_options['quantity'] + $transaction_item->received_quantity,
                        'received_user_id'  => $user_id,
                        'received_datetime' => $now
                    )
                );
            }

            // If all items received change transaction progress
            $all_received = true;
            foreach($this->items() as $transaction_item) {
                if($transaction_item->received_quantity < $transaction_item->requested_quantity) {
                    $all_received = false;
                }
            }
            if($all_received) $this->changeProgress('Received');
            echo $app->renderAlertSuccess('Items received successfully', 'Success', 'modal_lg', 1);
        } else {
            die($app->renderAlertError('Please choose action'));
        }
    }

    function supplier() {
        return new Supplier($this->personnel_id);
    }

    function renderPDF($title='Transaction') {
        global $app;

        $transaction_progress   = array ('created', 'authorized', 'approved', 'transferred');

        $html = $app->renderDocumentHeader();

        $html .= '<table class="w-100 py-5">
        <tr><td class="w-100 bg-light border"><h2 class="text-center">'.$title.'</h3></td></tr>
        </table>
        
        <table class=""><tr><td></td></tr></table>
        
        <table class="w-100 p-7 border bg-light">
        <tr>
        <td class="w-50 border-left border-top border-bottom"><h3>'.$this->type.' number</h3></td>
        <td class="w-50 border-right border-top border-bottom text-end"><h3>'.$this->formatCode($this->id).'</h3></td>
        </tr>
        </table>
        
        <table class="w-100 px-7 py-7 border">
        
        <tr class="">
        <td class="w-20 border-left border-top border-bottom">Adjustment store</td>
        <td class="w-30 border-top border-bottom text-muted">'.$this->store()->location()->name.'</td>
        <td class="w-20 border-top border-bottom">Adjustment reason:</td>
        <td class="w-30 border-top border-bottom border-right text-end text-muted">'.$app->getCategoryByCode('AdjustmentType', $this->subtype)->name.'</td>
        </tr>';

        foreach($transaction_progress as $progress) {
            $action     = ucfirst($progress).' by';
            $username   = $this->personnel($progress.'_user_id')->person()->name;
            $datetime   = $progress.'_datetime';
            $datetime   = $app->datetime($this->$datetime);

            $html .= '<tr>
            <td class="w-20 border-top">'.$action.'</td>
            <td class="w-30 border-top text-muted">'.$username.'</td>
            <td class="w-25 border-top">Datetime '.$progress.': </td>
            <td class="w-25 border-top text-end text-muted">'.$datetime.'</td>
            </tr>';
        }

        $html .= '</table>
        
        <br /><br /><table class="w-100 p-7">
        <tr>
        <td class="w-50 bg-light border-left border-top border-bottom"><h3>Adjusted Items </h3></td>
        <td class="w-50 bg-light border-right border-top border-bottom text-end"><h3> '.count($this->items()).'</h3></td>
        </tr>
        </table><br /><br />
        
        <table class="w-100 px-7 py-7 border">
        
        <tr class="bg-light">
        <td class="fw-bold text-center h4 border-top border-bottom border-left" style="width:6.5%">S/N</td>
        <td class="fw-bold h4 border-top border-bottom" style="width:25.5%">Item name</td>
        <td class="w-10 fw-bold h4 border-top border-bottom">Units</td>
        <td class="w-10 fw-bold h4 border-top border-bottom">Batch</td>
        <td class="w-12 fw-bold h4 border-top border-bottom">Expiry</td>
        <td class="w-8 fw-bold h4 border-top border-bottom text-end">Qty</td>
        <td class="w-12 fw-bold h4 border-top border-bottom text-end">Price</td>
        <td class="w-15 fw-bold h4 border-top border-bottom border-right text-end">Amount</td>
        </tr>';

        $count  		= 1;
		$total_price 	= 0;
        foreach($this->items() as $item) {
            $html .= '<tr>
            <td class="text-center border-top">'.$count++.'.</td>
            <td class="border-top">'.$item->name.'</td>
            <td class="border-top">'.$item->uom.'</td>
            <td class="border-top">'.$item->batch.'</td>
            <td class="border-top">'.$item->expiry.'</td>
            <td class="text-end border-top">'.$item->quantity.'</td>
            <td class="text-end border-top">'.number_format($item->unit_price).'</td>
            <td class="text-end border-top">'.number_format($item->price).'</td>
            </tr>';
			$total_price += $item->price;
        }

        $html .= '<tr class="fw-bold">
		<td class="border-top"></td>
		<td class="border-top">Total amount</td>
		<td class="border-top"></td>
		<td class="border-top"></td>
		<td class="border-top"></td>
		<td class="border-top"></td>
		<td class="border-top"></td>
		<td class="text-end border-top">'.number_format($total_price).'</td>
		</tr></table>
        
        <br /><br />
        
        <table class="w-100 p-7 border" style="page-break-inside:avoid;">
        <tr class="bg-light">
        <td class="w-25 text-center border-left border-top border-bottom"><h3>Requested by:</h3></td>
        <td class="w-25 text-center border-top border-bottom"><h3>Authorized by:</h3></td>
        <td class="w-25 text-center border-top border-bottom"><h3>Approved by:</h3></td>
        <td class="w-25 text-center border-top border-bottom border-right"><h3>Transferred by:</h3></td>
        </tr><tr>
		<td class="w-25 text-center"><br /> <br /> <br /> ..................................<br />'.$this->personnel('created_user_id')->person()->name.'<br /><b>'.$app->date($this->created_datetime).'</b></td>
        <td class="w-25 text-center"><br /> <br /> <br /> ..................................<br />'.$this->personnel('authorized_user_id')->person()->name.'<br /><b>'.$app->date($this->authorized_datetime).'</b></td>
        <td class="w-25 text-center"><br /> <br /> <br /> ..................................<br />'.$this->personnel('approved_user_id')->person()->name.'<br /><b>'.$app->date($this->approved_datetime).'</b></td>
        <td class="w-25 text-center"><br /> <br /> <br /> ..................................<br />'.$this->personnel('transferred_user_id')->person()->name.'<br /><b>'.$app->date($this->transferred_datetime).'</b></td>
        </tr>
        </table>';

        $app->renderPDF($html, ''.$title.'');
    }
}

?>