<?php 

class PurchaseOrder extends Procurement {
	function __construct($id=0) {	
		$this->table        = 'procurement_orders';
		$this->items_table	= 'procurement_order_items';
		$this->primary_key  = 'id';
		parent::__construct($id);
		$this->type 		= 'PurchaseOrder';	
	}

    function renderTransactionHeader() {
		global $app, $user;
        
        $html = '';
        if($this->id) $html  = $this->renderTrackingButton();
		$html .= '<div class="row">';
		$html .= '<div class="col-12 col-lg-6">'.$this->renderDateControl('Delivery date').'</div>';
		$html .= '<div class="col-12 col-lg-6">'.$this->renderStoreControl('Delivery store').'</div>';
		$html .= '<div class="col-12 d-none">'.$this->renderPricelistControl($this->getCostPricelistId()).'</div>';
		$html .='</div>';
        
		return $html;
	}

	function renderModalForm($submodule, $obj, $get) {
        global $app;

		$modal_options  = array(
			'title'     => ucfirst($get->action).' '.$submodule->shortname,
            'left_html'	=> '<div id="supplier_bio_container">'.$this->supplier()->renderBio($this->name).'</div>'
		);
        
        $html           = '';
        if($get->action == 'delete') {
            $html .= $submodule->renderModalDelete($get->id);
        } else {
            $html .= $submodule->renderModalFormOpen($modal_options);
            $html .= $app->renderInputHidden('action', $get->action);
            $html .= $app->renderInputHidden('id', $get->id);
        
            if($get->action == 'create' || $get->action == 'copy') {
            	require('suppliers_form.php');
            } elseif($get->action == 'edit') {
                $html .= $obj->renderTransactionHeader();
                $html .= $obj->renderTransactionItemsEditable();
                $html .= $obj->renderItemsControls();
                $html .= $obj->renderNotesControl('Requested');
            } elseif($get->action == 'extend') {
                $html .= $obj->renderTransactionHeader();
                $html .= $obj->renderTransactionItemsReadonly('Requested items');
                $html .= $obj->renderNotesControl('Approved');
            } elseif($get->action == 'authorize') {
                $html .= $obj->renderTransactionHeader();
                $html .= $obj->renderTransactionItemsReadonly('Requested items');
                $html .= $obj->renderActionControl( array (
                    array('name'=>'Authorize purchase order', 'value'=>'Authorized', 'icon'=>'fa-regular fa-check-circle fa-fw'),
                    array('name'=>'Reject purchase order','value'=>'Rejected', 'icon'=>'fa-regular fa-circle-xmark fa-fw')
                ));
                $html .= $obj->renderNotesControl('Authorized');
            } elseif($get->action == 'approve') {
                $html .= $obj->renderTransactionHeader();
                $html .= $obj->renderTransactionItemsReadonly('Requested items');
                $html .= $obj->renderActionControl( array (
                    array('name'=>'Approve purchase order', 'value'=>'Approved', 'icon'=>'fa-regular fa-check-circle fa-fw'),
                    array('name'=>'Reject purchase order','value'=>'Rejected', 'icon'=>'fa-regular fa-circle-xmark fa-fw')
                ));
                $html .= $obj->renderNotesControl('Approved');
            } elseif($get->action == 'forward') { 
                $html .= '<div class="row">';
                $html .= '<div class="col-12 col-lg-6">'.$this->renderReferenceControl('Invoice number').'</div>';
                $html .= '<div class="col-12 col-lg-6">'.$this->renderStoreControl('Delivery store').'</div>';
                $html .= '<div class="col-12 d-none">'.$this->renderPricelistControl($this->getCostPricelistId()).'</div>';
                $html .='</div>';
                $html .= $obj->renderTransactionItemsForward();
                $html .= $obj->renderActionControl( array (
                    array('name'=>'Create GRN for selected items', 'value'=>'Forwarded', 'icon'=>'fa-regular fa-check-circle fa-fw'),
                    array('name'=>'Reject purchase order','value'=>'Rejected', 'icon'=>'fa-regular fa-circle-xmark fa-fw')
                ));
                $html .= $obj->renderNotesControl('Forwarded');
            } elseif($get->action == 'open') {
                $html .= $obj->renderTransactionHeader();
                $html .= $obj->renderTransactionItemsReadonly('Ordered items');
                $html .= $obj->renderNotesControl();
            }
            $html     .= $submodule->renderModalFormClose();
        }
        return $html;
    }

	function renderTableOpen($data) {
        global $app;

        $table_options      = array (
            'columns'       => array (
                array('name'=>'Date', 'css'=>'w-10'),
                array('name'=>'Order No', 'css'=>'w-10'),
                array('name'=>'Department', 'css'=>'w-20'),
                array('name'=>'Supplier', 'css'=>''),
                array('name'=>'Amount', 'css'=>'w-10 money text-end'),
                array('name'=>'Progress', 'css'=>'w-15 text-center'),
                array('name'=>'Options', 'css'=>'w-15 text-end')
            ),
            'css'           => 'table-stripped bs-table',
            'rows'          => $data->rows
        );

        return $app->renderTableOpen($table_options);
    }

	function renderTableBody($data) {
        global $app;

        $html   = '';

        foreach($data->rows as $row) {
            $transaction_type   = $this->type;
            $obj                = new $transaction_type($row->id);
            $color              = $obj->date > date('Y-m-d') ? 'success' : 'danger';

            $html .=  '<tr>
            <td> '.$app->date($obj->requested_datetime).'</td>
            <td> '.$obj->formatCode($row->id).'</td>
            <td>
            <div class="list-text">'.$obj->store()->department()->location()->name.'</div>
            <small class="text-muted">'.$obj->store()->location()->name.'</small>
            </td>
			<td>
            <div class="list-text fw-bold">'.$obj->name.'</div>
            <small class="text-'.$color.'">Delivery: '.$app->date($obj->date).' </small>
            </td>
            <td class="text-end">
            <div class="fw-bold text-muted">'.$app->formatMoney($obj->subtotal).'</div>
            </td>
            <td class="text-center">'.$obj->renderButtonProgress().'</td>
            <td class="text-center text-nowrap">';
        
            if($obj->progress == 'Requested') {
                $html .=  $obj->renderLinkAuthorize();
                $html .=  $obj->renderLinkCopy();
                $html .=  $obj->renderLinkEdit();
                $html .=  $obj->renderLinkDelete();
            } elseif($obj->progress == 'Authorized') {
                $html .=  $obj->renderLinkApprove();
                $html .=  $obj->renderLinkCopy();
                $html .=  $obj->renderLinkDelete();
            } elseif($obj->progress == 'Approved') {
              	$html .=  $obj->renderLinkForward('GRN');
                $html .=  $obj->renderLinkExtend();
                $html .=  $obj->renderLinkCopy();
                $html .=  $obj->renderLinkPrint();
            } elseif($obj->progress == 'Received') {
                $html .=  $obj->renderLinkCopy();
                $html .=  $obj->renderLinkPrint();
            } elseif($obj->progress == 'Forwarded') {
                $html .=  $obj->renderLinkCopy();
                $html .=  $obj->renderLinkPrint();
            }
        
            $html .= ' </td>
            </tr>';
        }
        return $html;
    }

    function renderCreateConfirmation($submodule, $post) {
        global $app, $user;

        $app->validate(array('name'=>'trans_date', 'message'=>'Delivery date is required', 'value'=>$post->trans_date, 'min'=>1));
        $app->validate(array('name'=>'trans_store_id', 'message'=>'Please choose store', 'value'=>$post->trans_store_id, 'min'=>1));
        $items      = $app->uncleanText($post->trans_items);
        if(!json_decode($items)) die($app->renderAlertError('Please select atleast one item.'));

        $user->checkSubmoduleAccess($submodule, 'create');

        $store                  = new Store($post->trans_store_id);
        $supplier               = $this->createSupplierIfNotExists(
            array (
                'id'            => $post->trans_supplier_id,
                'name'          => $post->trans_supplier_name,
                'phone'         => $post->trans_supplier_phone
            )
        );

        $create_data            = array (
            'type'              => $this->type,
            'personnel'         => 'Supplier',
            'personnel_id'      => $post->trans_supplier_id,
            'date'              => $post->trans_date,
            'store_id'          => $post->trans_store_id,
            'name'              => $post->trans_supplier_name,
            'requested_notes'   => $post->trans_requested_notes,
            'items'             => $items,
            'cost'              => $post->trans_cost,
            'subtotal'          => $post->trans_subtotal
        );

        $transaction_id         = $this->createWithMinimumData($create_data);
        if($transaction_id) {
            return $app->renderAlertSuccess('Purchase order created successfully', 'Success', 'modal_lg', true);
        }
    }

    function renderEditConfirmation($submodule, $post) {
        global $app, $user;

        $user->checkSubmoduleAccess($submodule, 'edit');   

        $items                  = $app->uncleanText($post->trans_items);
        $store                  = new Store($post->trans_store_id);

        $update_data            = array (
            'store_id'          => $post->trans_store_id,
            'date'              => $post->trans_date,
            'requested_notes'   => $post->trans_requested_notes,
            'items'             => $items,
            'cost'              => $app->unformatMoney($post->trans_subtotal),
            'subtotal'          => $app->unformatMoney($post->trans_subtotal)
        );
        $updated                = $this->update($update_data);
        if($updated) {
            $this->resetItems(json_decode($items));
            return $app->renderAlertSuccess($submodule->shortname.' updated successfully', 'Success', 'modal_lg', 1);
        }
    }

    function renderTransactionItemsForward() {
		global $app;

		$html = '<div class="trans-container mb-4"> <h5 class="trans-subtitle">
        <span class="fa-regular fa-cart-circle-check fa-fw"></span> &nbsp;Choose items to receive:</h5>
		<div class="card p-4 my-4 bg-primary" style="--bs-bg-opacity: .05;">
        <div class="table-responsive">
		<table class="table table-hover align-middle">
		<thead>
		<tr>
        <th class="w-5">
        <div class="form-check form-switch" style="margin-bottom:-1px">
		<input role="button" class="form-check-input toggle-child" data-child-class="trans-item" type="checkbox" id="trans_check_all" />
		</div>
        </th>
		<th class="h5 fw-bold ps-0">Item name</th>
		<th class="h5 fw-bold w-15 text-center">Ordered</th>
		<th class="h5 fw-bold w-15 text-end">Received</th>
		<th class="h5 fw-bold w-15 text-end">Receive</th>       
		</tr>
		</thead>
		<tbody id="trans_items_container">';
        
        foreach($this->items() as $item) {
            $item_obj   = new Item($item->item_id);
            $html       .= '<tr>
            <td class="align-top py-3">
			<div class="form-check form-switch">
			<input name="trans_items['.$item->item_id.'][checked]" value="'.$item->item_id.'" role="button" class="form-check-input trans-item" type="checkbox" id="trans_check_'.$item->item_id.'" />
			</div>
			</td>
            <td class="ps-0 py-3">
            <div class="pb-3">'.$item->name.' <b class="text-muted">('.$item->pack.')</b></div>
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][transaction_item_id]', $item->id).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][item_id]', $item->item_id).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][name]', $item->name).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][uom]', $item->uom).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][pack]', $item->pack).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][size]', $item->size).'
            '.$app->renderInputHidden('trans_items['.$item->item_id.'][unit_price]', $item->unit_price).'
            <div class="row gx-3">
            <div class="col-6">'.$app->renderInputSelect('trans_items['.$item->item_id.'][batch]', '', 'Batch', 'data-list', '', $item_obj->listBatch($this->store_id)).'</div>
			<div class="col-6">'.$app->renderInputText('trans_items['.$item->item_id.'][expiry]', '', 'Expiry', 'date-picker').'</div>
            </div>
            </td>
            <td class="py-3 align-bottom">'.$app->renderInputNumber('trans_items['.$item->item_id.'][requested_quantity]', $item->requested_quantity, '', 'readonly').'</td>
            <td class="py-3 align-bottom">'.$app->renderInputNumber('trans_items['.$item->item_id.'][forwarded_quantity]', $item->forwarded_quantity, '', 'readonly').'</td>
            <td class="py-3 align-bottom">'.$app->renderInputNumber('trans_items['.$item->item_id.'][forwarding_quantity]', ($item->requested_quantity - $item->forwarded_quantity), '', '').'</td>
            </tr>';
        }

        $html .='</tbody>
		</table>
		</div>
		</div>
        </div>';

		return $html;
	}

    function renderForwardConfirmation($submodule, $post) {
        global $app, $user_id, $now;

        if($post->trans_action == 'Forwarded') {
            $forwarded_items    = array();
            $grn_items          = array();
            $grn_subtotal       = 0;

            // Validate received quantity does not exceed requested quantity
            foreach(array_keys($post->trans_items) as $item_id) {
                if(isset($post->trans_items[$item_id]['checked'])) {
                    $pending_quantity = $post->trans_items[$item_id]['requested_quantity'] - $post->trans_items[$item_id]['forwarded_quantity'];
                    if($post->trans_items[$item_id]['forwarding_quantity'] < 0 ) {
                        die($app->renderAlertError('Please receive 1 or more units', $post->trans_items[$item_id]['name']));
                    } elseif($post->trans_items[$item_id]['forwarding_quantity'] > ($pending_quantity)) {
                        die($app->renderAlertError('You are trying to receive '.$post->trans_items[$item_id]['forwarding_quantity'].' out of '.$pending_quantity.' units', $post->trans_items[$item_id]['name']));
                    } else {
                        $forwarded_items[$item_id] = $post->trans_items[$item_id];
                        $forwarded_items[$item_id]['quantity']  = $post->trans_items[$item_id]['forwarding_quantity'];
                        $forwarded_items[$item_id]['price']     = $forwarded_items[$item_id]['quantity'] * $forwarded_items[$item_id]['unit_price'];
                        $grn_subtotal                           += $forwarded_items[$item_id]['price'];
                    }
                }
            }

            // Make sure atleast one item submitted
            if(!$forwarded_items) die($app->renderAlertError('Please submit at least one item to receive'));
            
            $app->validate(array('name'=>'trans_reference_no', 'message'=>'Invoice number is required', 'value'=>$post->trans_reference_no, 'min'=>1));
            
            // Update order with forwarded/received quantities
            foreach(array_keys($forwarded_items) as $forwarded_item_id) {
                $forwarded_item                 = $forwarded_items[$forwarded_item_id];
                $item_obj                       = new Item($forwarded_item['item_id']);
                $transaction_item               = new TransactionItem($this->type, $this->id, $forwarded_item['transaction_item_id']);
                $grn_items[]                    = $forwarded_item;

                $transaction_item->update (
                    array (
                        'progress'              => 'Received',
                        'forwarded'             => 1,
                        'forwarded_quantity'    => $forwarded_item['quantity'] + $transaction_item->forwarded_quantity,
                        'forwarded_user_id'     => $user_id,
                        'forwarded_datetime'    => $now,
                        'received'              => 1,
                        'received_quantity'     => $forwarded_item['quantity'] + $transaction_item->forwarded_quantity,
                        'received_user_id'      => $user_id,
                        'received_datetime'     => $now
                    )
                );
            }

            // Create GRN for forwarded items
            $grn                    = new PurchaseGRN();
            $create_data            = array (
                'type'              => 'PurchaseGRN',
                'parent'            => 'PurchaseOrder',
                'parent_id'         => $this->id,
                'personnel'         => $this->personnel,
                'personnel_id'      => $this->personnel_id,
                'reference'         => 'Invoice',
                'reference_no'      => $post->trans_reference_no,
                'store_id'          => $this->store_id,
                'name'              => $this->name,
                'requested_notes'   => $post->trans_forwarded_notes,
                'items'             => json_encode($grn_items),
                'cost'              => $grn_subtotal,
                'subtotal'          => $grn_subtotal
            );
    
            $grn_id                 = $grn->createWithMinimumData($create_data);
            if($grn_id) {
                echo $app->renderAlertSuccess('GRN created successfully', 'Success', 'modal_lg', true);
            }

            // If all items received change transaction progress
            $all_received = true;
            foreach($this->items() as $transaction_item) {
                if($transaction_item->received_quantity < $transaction_item->requested_quantity) {
                    $all_received = false;
                }
            }
           
            if($all_received) {
                $this->changeProgress('Received');
                echo $app->renderAlertSuccess('All items received successfully', 'Success', 'modal_lg', 1);
            }
        } else {
            die($app->renderAlertError('Please choose action'));
        }
    }

    function renderTransactionItemsReadonly($title='Requested items') {
		global $app;

		$html = '<div class="trans-container mt-3 mb-4"> <h5 class="trans-subtitle">
        <span class="fa-regular fa-cart-circle-check fa-fw"></span> &nbsp;'.$title.':</h5>
        <div class="card p-4 my-4 bg-primary" style="--bs-bg-opacity: .05;">
		<div class="table-responsive">
		<table class="table table-hover align-middle">
		<thead class="h5">
		<tr>
		<th class="ps-0">Item name</th>
		<th class="w-10 text-center">Ordered</th>
		<th class="w-10 text-center">Received</th>
		<th class="w-10 text-end">Price</th>
		<th class="w-12 text-end">Amount</th>
		</tr>
		</thead>
		<tbody id="trans_items_container">';

		foreach($this->items() as $item) {
            $details = strlen($item->details) ? '<div class="text-muted"><small>'.$item->details.'</small></div>' : '';
			$html   .= '<tr>
			<td class="ps-0 py-3">'.$item->name.' <b class="text-muted">('.$item->pack.')</b> '.$details.'</td>
			<td class="text-center">'.$item->requested_quantity.'</td>
			<td class="text-center">'.$item->received_quantity.'</td>
			<td class="text-end">'.$app->formatMoney($item->unit_price).'</td>
			<td class="text-end">'.$app->formatMoney($item->price).'</td>
			</tr>';
		}

		$html .= '</tbody>
		<tfoot>
		<tr>
		<th class="ps-0 fw-semibold">Subtotal</th>
		<th></th>
		<th></th>
		<th></th>
		<th class="text-end">'.$app->renderInputText('trans_subtotal', $app->formatMoney($this->subtotal), 'Subtotal', 'line-control-plaintext text-end fw-semibold', 'readonly').'</th>
        </tr>
		</tfoot>
		</table>
		</div>
		</div>
        </div>';
		return $html;
	}

    function renderExtendConfirmation($submodule, $post) {
        global $app, $user;

        $user->checkSubmoduleAccess($submodule, 'edit');   

        $update_data            = array (
            'date'              => $post->trans_date,
            'approved_notes'    => $post->trans_approved_notes
        );
        $updated                = $this->update($update_data);
        if($updated) {
            return $app->renderAlertSuccess($submodule->shortname.' delivery date changed successfully', 'Success', 'modal_lg', 1);
        }
    }

    function renderPDF($title='Transaction') {
        global $app;

        $transaction_progress   = array ('created', 'authorized', 'approved', 'received');

        $html = $app->renderDocumentHeader();

        $html .= '<table class="w-100 py-5">
        <tr><td class="w-100 bg-light border"><h2 class="text-center">'.$title.'</h3></td></tr>
        </table>
        
        <table class=""><tr><td></td></tr></table>
        
        <table class="w-100 p-7 border bg-light">
        <tr>
        <td class="w-50 border-left border-top border-bottom"><h3>LPO number</h3></td>
        <td class="w-50 border-right border-top border-bottom text-end"><h3>'.$this->formatCode($this->id).'</h3></td>
        </tr>
        </table>
        
        <table class="w-100 px-7 py-7 border">
        
        <tr class="">
        <td class="w-20 border-left border-top border-bottom">Delivery store</td>
        <td class="w-30 border-top border-bottom text-muted">'.$this->store()->location()->name.'</td>
        <td class="w-20 border-top border-bottom">Delivery date</td>
        <td class="w-30 border-top border-bottom border-right text-end text-muted">'.$this->date.'</td>
        </tr>';

        foreach($transaction_progress as $progress) {
            $action     = ucfirst($progress).' by';
            $username   = $this->personnel($progress.'_user_id')->person()->name;
            $datetime   = $progress.'_datetime';
            $datetime   = $app->datetime($this->$datetime);

            $html .= '<tr>
            <td class="w-20 border-top">'.$action.'</td>
            <td class="w-30 border-top text-muted">'.$username.'</td>
            <td class="w-25 border-top">Datetime '.$progress.': </td>
            <td class="w-25 border-top text-end text-muted">'.$datetime.'</td>
            </tr>';
        }

        $html .= '</table><br /><br />

        <table class="w-100 p-7 border bg-light">
        <tr>
        <td class="w-50 border-left border-top border-bottom"><h3>Supplier details</h3></td>
        <td class="w-50 border-right border-top border-bottom text-end"></td>
        </tr>
        </table>

        <table class="w-100 px-7 py-7 border">
        
        <tr class="">
        <td class="w-20 border-left border-top border-bottom">Supplier name</td>
        <td class="w-80 border-top border-bottom text-muted">'.$this->name.'</td>
        </tr><tr class="">
        <td class="w-20 border-left border-top border-bottom">Postal address</td>
        <td class="w-80 border-top border-bottom text-muted">'.$this->address.'</td>
        </tr><tr class="">
        <td class="w-20 border-left border-top border-bottom">Email address</td>
        <td class="w-80 border-top border-bottom text-muted">'.$this->email.'</td>
        </tr><tr class="">
        <td class="w-20 border-left border-top border-bottom">Phone number</td>
        <td class="w-80 border-top border-bottom text-muted">'.$this->phone.'</td>
        </tr></table><br /><br />
        
        <table class="w-100 py-7">
        <tr>
        <td class="w-50 bg-light border-left border-top border-bottom"><h3> Items to be delivered </h3></td>
        <td class="w-50 bg-light border-right border-top border-bottom text-end"><h3> '.count($this->items()).'</h3></td>
        </tr>
        </table><br /><br />
        
        <table class="w-100 px-7 py-7 border">
        
        <tr class="bg-light">
        <td class="fw-bold text-center h4 border-top border-bottom border-left" style="width:6.5%">S/N</td>
        <td class="fw-bold h4 border-top border-bottom" style="width:25.5%">Item name</td>
        <td class="w-10 fw-bold h4 border-top border-bottom">Units</td>
        <td class="w-10 fw-bold h4 border-top border-bottom">Batch</td>
        <td class="w-12 fw-bold h4 border-top border-bottom">Expiry</td>
        <td class="w-8 fw-bold h4 border-top border-bottom text-end">Qty</td>
        <td class="w-12 fw-bold h4 border-top border-bottom text-end">Price</td>
        <td class="w-15 fw-bold h4 border-top border-bottom border-right text-end">Amount</td>
        </tr>';

        $count  		= 1;
		$total_price 	= 0;
        foreach($this->items() as $item) {
            $html .= '<tr>
            <td class="text-center border-top">'.$count++.'.</td>
            <td class="border-top">'.$item->name.'</td>
            <td class="border-top">'.$item->uom.'</td>
            <td class="border-top">'.$item->batch.'</td>
            <td class="border-top">'.$item->expiry.'</td>
            <td class="text-end border-top">'.$item->quantity.'</td>
            <td class="text-end border-top">'.number_format($item->unit_price).'</td>
            <td class="text-end border-top">'.number_format($item->price).'</td>
            </tr>';
			$total_price += $item->price;
        }

        $html .= '<tr class="fw-bold">
		<td class="border-top"></td>
		<td class="border-top">Total amount</td>
		<td class="border-top"></td>
		<td class="border-top"></td>
		<td class="border-top"></td>
		<td class="border-top"></td>
		<td class="border-top"></td>
		<td class="text-end border-top">'.number_format($total_price).'</td>
		</tr></table>
        
        <br /><br />
        
        <table class="w-100 p-7 border" style="page-break-inside:avoid;">
        <tr class="bg-light">
        <td class="w-25 text-center border-left border-top border-bottom"><h3>Requested by:</h3></td>
        <td class="w-25 text-center border-top border-bottom"><h3>Authorized by:</h3></td>
        <td class="w-25 text-center border-top border-bottom"><h3>Approved by:</h3></td>
        <td class="w-25 text-center border-top border-bottom border-right"><h3>Received by:</h3></td>
        </tr><tr>
		<td class="w-25 text-center"><br /> <br /> <br /> ..................................<br />'.$this->personnel('created_user_id')->person()->name.'<br /><b>'.$app->date($this->created_datetime).'</b></td>
        <td class="w-25 text-center"><br /> <br /> <br /> ..................................<br />'.$this->personnel('authorized_user_id')->person()->name.'<br /><b>'.$app->date($this->authorized_datetime).'</b></td>
        <td class="w-25 text-center"><br /> <br /> <br /> ..................................<br />'.$this->personnel('approved_user_id')->person()->name.'<br /><b>'.$app->date($this->approved_datetime).'</b></td>
        <td class="w-25 text-center"><br /> <br /> <br /> ..................................<br />'.$this->personnel('received_user_id')->person()->name.'<br /><b>'.$app->date($this->received_datetime).'</b></td>
        </tr>
        </table>';

        $app->renderPDF($html, ''.$title.'');
    }
}

?>