<?php 

class Supplier extends Crud {
	function __construct($id=0) {		
		$this->table        = 'procurement_suppliers';
		$this->primary_key  = 'id';
		parent::__construct($id);
	}

	function person($person_id='') {
		$person_id 	= strlen($person_id) ? $person_id : $this->person_id;
		$person		= new Person($person_id);
		$person->personnel = 'Supplier';
		return $person;
	}

	function picture() {
		global $app, $root;
		$attachment = $app->getAttachmentByCode('Supplier', $this->id, 'ProfilePicture');
		if($attachment->exists()) {
			return $attachment->renderOpenPath();
		} elseif($this->type == 'Company') {
			return $root.'assets/images/suppliers/company.png';
		} else {
			$person		= $this->person();
			$age_string = $person->age();
			$sex		= strtolower($person->sex);
			$picture 	= 'individual.png';
			$ages		= explode(' ', $age_string);
			
			if((count($ages) == 2) && ($sex == 'male' || $sex == 'female')) {
				$age_number		= $ages[0];
				$age_units		= $ages[1];
				$agegroup 	= 'baby';
				if($age_units 	== 'year' || $age_units 	== 'years') {
					if($age_number > 5) 	$agegroup = 'child';
					if($age_number > 18) 	$agegroup = 'adult';
					if($age_number > 60) 	$agegroup = 'old';
				}
				$picture	= $sex.'-'.$agegroup.'.png';
			}
			return $root.'assets/images/suppliers/'.$picture;
		}
	}

	function signature() {
		global $app, $root;
		$attachment = $app->getAttachmentByCode('Supplier', $this->id, 'SupplierSignature');
		if($attachment->exists()) return $attachment->renderOpenPath();
		return $root.'assets/images/default/signature.png';
	}

	function createIfNotExists($data) {
        if($data['id']) {
            return new Supplier($data['id']);
        } else {
            $supplier       = new Supplier();
            $supplier_id    = $supplier->createWithMinimumData($data);
            return new Supplier($supplier_id);
        }
    }

	function createWithMinimumData(array $data) {
		
		// Create person
		$person 					= new Person();
        $person_id                  = $person->createWithMinimumData(
			array (
				'personnel'         => 'Supplier',
				'name'              => $data['name'],
				'phone'             => $data['phone']
			)
		);
        $person                     = new Person($person_id);
		$supplier					= new Supplier();

        // If person created create supplier
        if($person_id) {
            $supplier_data              = $this->addCommonFields (
               array (
                    'person_id'         => $person_id,
                    'branch_id'         => 1,
                    'personnel'         => 'Supplier',
                    'personnel_id'      => 0,
                    'code'              => $this->nextCode(),
                    'type'              => '',
                    'subtype'           => '',
                    'category'          => '',
                    'subcategory'       => '',
                    'registration_no'   => '',
                    'insurance'         => '',
                    'insurance_id'      => 0,
                    'card_no'           => '',
                    'card_expiry'       => 0,
                    'membership_no'     => '',
                    'authorization_no'  => '',
                    'supplementary'     => '',
                    'scheme'            => '',
                    'scheme_id'         => 0,
                    'scheme_code'       => '',
                    'product'           => '',
                    'product_id'        => 0,
                    'product_code'      => '',
                    'employer'          => '',
                    'employer_id'       => 0,
                    'employer_code'     => '',
                    'tin'               => '',
                    'vrn'               => ''
               )
            );
            $supplier_id    			= $this->create($supplier_data);
			$supplier					= new Supplier($supplier_id);
           
            if($supplier_id) {
                $person->set('personnel_id', $supplier_id);
                $supplier->set('personnel_id', $supplier_id);
            }
		}
		return $supplier_id;
	}

	function formatCode() {
		$digits = 8;
		if(strlen($this->code) < 7) $digits = 6;
		return sprintf('%0'.$digits.'d', $this->code);
	}

	function insurance() {
		return new Insurance($this->insurance_id);
	}

	function scheme() {
		return new Scheme($this->scheme_id);
	}

	function renderProfileCard() {
		global $root, $app;
		$color 				= 'secondary';
		$person 			= $this->person();
		$button_options 	= array('id'=>$this->id, 'css'=>'btn btn-'.$color.' btn-lg');
		$buttons			= $app->renderButtonOpen($button_options).''.$app->renderButtonEdit($button_options).''.$app->renderButtonCopy($button_options).''.$app->renderButtonDelete($button_options);
		$person->sex 		= $person->sex ? $person->sex : 'ago';

		$html 				= '<div id="card_customer_'.$this->id.'" class="auto-flip bs-tooltip" title="Click card to flip">
		<div class="shadow shadow-lg card text-center front">
		<div class="card-img-top bg-'.$color.' text-white py-4" style="height:240px; min-height:240px">
		'.$buttons.'
		</div>
		<img src="'.$this->picture().'" class="rounded-circle mx-auto d-block bg-light" style="width:80%; max-width:200px; margin-top:-140px; border:8px solid white">
		<div class="card-body" style="min-height:180px;">
		
		<h5 class="fw-bold text-truncate">'.$app->readText($person->name).'</h4>
		<h4 class="fw-bold"> '.$this->formatCode().' </h4>
		<h6>'.$this->type.'</h6>
		
		</div></div>
		
		<div class="shadow card text-center back">
		
		<div class="card-img-top bg-'.$color.' text-white py-4" style="height:240px; min-height:240px">
		<a class="btn btn-'.$color.' bs-tooltip" data-flip="card_customer_'.$this->id.'" title="Flip"> <span class="fa-solid fa-arrow-left"></span> </a>
		<div><h5> Registered </h5></div>
		<div>'.$app->datetime($this->created_datetime).'</div>
		<div>'.$this->createdUser()->person()->name.'</div>
		<div>'.$app->age($this->created_datetime).' ago </div>

		</div>
		<img src="'.$this->picture().'" class="rounded-circle mx-auto d-block bg-light" style="visibility:hidden; width:75%; max-width:190px; margin-top:-120px; border:7px solid white">
		<div class="card-body" style="min-height:180px;">
		<h5><span class="fa-light fa-phone fa-fw"></span></h5>
		<h4 class="fw-light">'.$person->phone.'</h4>
		<div class="fw-light"> '.$person->village()->name.' - '.$person->district()->name.' - '.$person->region()->name.'</div>
		</div></div></div>';
		
		return $html;
	}

	function renderBio($name='', $root='') {
		global $app;

		if($this->id) {
			$person = $this->person();
			$html	= '<img src="'.$this->picture().'" class="rounded-circle mx-auto d-block bg-light my-3" style="width:85%; max-width:185px; border:8px solid white" />
			<div class="px-3 me-2 text-white">
			<h5 class="fw-bold text-truncate">'.$app->readText($person->name).'</h5>
			<h4 class="fw-bold"> '.$this->formatCode().' </h4>
			<h5>'.$this->type.'</h5>
			</div>';
		} else {
			$html = '<img src="'.$root.$this->picture().'" class="rounded-circle mx-auto d-block bg-light my-3" style="width:6=75%; max-width:175px; border:8px solid white" />
			<div class="rounded p-3 bg-white mt-3 me-2">'.
			$app->renderInputText('temp_supplier_name', $name, 'Supplier name', 'line-control', 'onchange="setSupplierBio()"').
			'<div class="my-2"></div>'.
			$app->renderInputText('temp_supplier_phone', '', 'Phone number', 'line-control', 'onchange="setSupplierBio()"').
			'</div>';
		}
		
		return $html;
	}

	function renderInsuranceScheme() {
		$html = $this->insurance()->code.' - '.$this->scheme()->name;
		return $html;
	}
}

?>