<?php 

$root           = '../../../';
$submodule_code = 'SaccosImports';
require($root.'config/config.php');

if(!isset($_FILES['loans_file'])) die($app->renderAlertError('CSV file containing members loan has not been uploaded'));

$count              = 1;
$loan               = new Loan();
$guarantor          = new Table('saccos_loan_guarantors', 'id', 0);
$saccos_transaction = new SaccosTransaction();
$members_file       = $_FILES['loans_file']['tmp_name'];

$headings           = array (
    'AINA_MKOPO'      ,
    'NAMBA_MWANACHAMA',
    'JINA_MWANACHAMA' ,
    'MKOPO_MIEZI'     ,
    'MKOPO_KIASI'     ,
    'MKOPO_ULIOLIPWA' ,
    'MKOPO_ULIOSALIA' ,
    'RIBA_ASILIMIA'   ,
    'RIBA_KIASI'      ,
    'RIBA_ILIYOLIPWA' ,
    'RIBA_ILIYOSALIA' ,
    'TAREHE_KUANZA'   
);

$csv->validateHeadings($members_file, $headings);
$data                   = $csv->read($members_file);
foreach($data as $row) {
    $count++;
    $loan_type_code     = $row->AINA_MKOPO;
    $loan_type          = $app->getCategoryByCode('LoanType', $loan_type_code);
    if(!$loan_type->code) die($app->renderAlertError('Unknown loan category: '.$loan_type_code.' at row '.$count));
}

foreach($data as $row) {
    $member_id          = $app->getColumnValue('saccos_members', 'id', 'code', $row->NAMBA_MWANACHAMA);
    if(!$member_id) continue;

    $loan_type_code     = $row->AINA_MKOPO;
    $loan_duration      = $row->MKOPO_MIEZI;
    $loan_amount        = $app->unformatMoney($row->MKOPO_KIASI);
    $loan_paid          = $app->unformatMoney($row->MKOPO_ULIOLIPWA);
    $loan_due           = $app->unformatMoney($row->MKOPO_ULIOSALIA);
    $interest_percent   = $row->RIBA_ASILIMIA;
    $interest_amount    = $app->unformatMoney($row->RIBA_KIASI);
    $interest_paid      = $app->unformatMoney($row->RIBA_ILIYOLIPWA);
    $interest_due       = $app->unformatMoney($row->RIBA_ILIYOSALIA);
    $start_date         = $row->TAREHE_KUANZA;
    $end_date           = date('Y-m-d', strtotime($start_date.' + '.$loan_duration.' months'));
    $application_fee    = $loan_amount > 6000000 ? $loan_amount * 0.01 : 0;
    $insurance_fee      = $loan_amount > 10000000 ? $loan_amount * 0.018 : 0;

    $loan_type          = $app->getCategoryByCode('LoanType', $loan_type_code);
    $member             = new Member($member_id);
    $amount_limit       = $loan_type->f4;
    $duration_limit     = $loan_type->f3;
    $amount_monthly     = $loan_amount / $loan_duration;
    $interest_monthly   = $interest_amount / $loan_duration;
    $current_deposits   = $member->balance('Deposits');
    $current_savings    = $member->balance('Savings');
    $current_shares     = $member->balance('Shares');
    $current_loans      = $member->loanBalance('amount_due');
    $current_interests  = $member->loanBalance('interest_due');
    $current_penalties  = $member->loanBalance('penalty_due');
    $now                = $start_date;

    $loan_data                  = $loan->addCommonFields (
        array (
            'type'              => $loan_type->code,
            'subtype'           => '',
            'code'              => '',
            'name'              => $loan_type->name,
            'details'           => $loan_type->name,
            'member_id'         => $member_id,
            'start_date'        => $start_date,
            'end_date'          => $end_date,
            'application_fee'   => $application_fee,
            'insurance_fee'     => 0,
            'expenses_fee'      => $application_fee,
            'duration_limit'    => $duration_limit,
            'duration_months'   => $loan_duration,
            'currency_code'     => $client->currency_code,
            'currency_rate'     => $client->currency_rate,
            'amount_limit'      => $amount_limit,
            'amount'            => $loan_amount,
            'amount_monthly'    => $amount_monthly,
            'amount_paid'       => $loan_paid,
            'amount_due'        => $loan_due,
            'interest_code'     => 'SaccosInterest',
            'interest_percent'  => $interest_percent,
            'interest_amount'   => $interest_amount,
            'interest_monthly'  => $interest_monthly,
            'interest_paid'     => $interest_paid,
            'interest_due'      => $interest_due,
            'penalty_amount'    => 0,
            'penalty_paid'      => 0,
            'penalty_due'       => 0,
            'current_deposits'  => $current_deposits,
            'current_savings'   => $current_savings,
            'current_shares'    => $current_shares,
            'current_loans'     => $current_loans,
            'current_interests' => $current_interests,
            'current_penalties' => $current_penalties,
            'progress'          => 'Umeombwa',
            'progress_code'     => 0,
            'progress_datetime' => 0
        )
    );

    $loan_id = $loan->create($loan_data);

    if($loan_id) {
        echo $app->renderAlertSuccess($member->person()->name.' added '.$loan_type->name.' '.$client->currency_code.' '.number_format($loan_amount));
    }

    // Guarantors
    if($loan_id) {
        for($count = 1; $count < 4; $count++) {
            $guarantor_id                   = $member_id;
            $guarantor_member               = new Member($guarantor_id);
            $guarantor_datetime             = $start_date;
            $guarantor_data                 = $saccos_transaction->addCommonFields (
                array (
                    'loan_id'               => $loan_id,
                    'member_id'             => $member_id,
                    'guarantor_id'          => $guarantor_id,
                    'deposits'              => $guarantor_member->balance('Deposits'),
                    'savings'               => $guarantor_member->balance('Savings'),
                    'shares'                => $guarantor_member->balance('Shares'),
                    'loans'                 => 0,
                    'currency_code'         => $client->currency_code,
                    'currency_rate'         => $client->currency_rate,
                    'guarantee_status'      => 'Accepted',
                    'guarantee_notes'       => '',
                    'guarantee_datetime'    => 0
                )
            );
            $guarantor_id = $guarantor->create($guarantor_data);
        }

        $data                       = $saccos_transaction->addCommonFields (
            array (
                'member_id'         => $member_id,
                'date'              => $now,
                'transaction'       => '',
                'transaction_id'    => '',
                'parent'            => '',
                'parent_id'         => '',
                'paymethod'         => 'BankDeposit',
                'reference'         => '',
                'account_id'        => 1,
                'details'           => '',
                'currency'          => $client->currency_code,
                'rate'              => $client->currency_rate,
                'amount'            => 0,
                'withdrawn'         => 0,
                'deposited'         => 0,
                'balance'           => 0
            )
        ); 
    
        // Rejesha mkopo
        $trans_category         = $app->getCategoryByCode('SaccosTransaction', 'Repayment');
        $data['transaction']    = 'Repayment';
        $data['parent']         = 'Loan';
        $data['parent_id']      = $loan_id;
        $data['amount']         = $loan_paid;
        $data['deposited']      = $loan_paid;
        $data['balance']        = $loan_due;
        $saccos_transaction_id  = $saccos_transaction->create($data);
    
        // Rejesha riba
        $trans_category         = $app->getCategoryByCode('SaccosTransaction', 'Interest');
        $data['transaction']    = 'Interest';
        $data['parent']         = 'Loan';
        $data['parent_id']      = $loan_id;
        $data['amount']         = $interest_amount;
        $data['deposited']      = $interest_amount;
        $data['balance']        = $interest_due;
        $saccos_transaction_id  = $saccos_transaction->create($data);
    }
}

?>