<?php 

$root           = '../../../';
$submodule_code = 'SaccosImports';
require($root.'config/config.php');

if(!isset($_FILES['repayments_file'])) die($app->renderAlertError('CSV file containing members loan has not been uploaded'));

$count              = 1;
$saccos_transaction = new SaccosTransaction();
$members_file       = $_FILES['repayments_file']['tmp_name'];

$headings           = array (
    'NAMBA_MKOPO'      ,
    'AINA_MKOPO'       ,
    'NAMBA_MWANACHAMA' ,
    'JINA_MWANACHAMA'  ,
    'MKOPO_KIASI'      ,
    'MKOPO_ULIOSALIA'  ,
    'RIBA_KIASI'       ,
    'RIBA_ILIYOSALIA'  ,
    'TAREHE_KUANZA'    ,
    'MUDA_MIEZI'       ,
    'TAREHE_KUMALIZA'  ,
    'MKOPO_ALIOREJESHA',
    'RIBA_ALIYOREJESHA' 
);

$csv->validateHeadings($members_file, $headings);
$data                   = $csv->read($members_file);
foreach($data as $row) {

    $count++;
    $id                 = $row->NAMBA_MKOPO;
    $loan               = new Loan($row->NAMBA_MKOPO);

    $db_type            = $loan->type;
    $db_amount          = $loan->amount;
    $db_member          = $loan->member();
    $db_code            = $db_member->code;
    $db_member          = $db_member->person()->name;

    $import_type        = $row->AINA_MKOPO;
    $import_amount      = $app->unformatMoney($row->MKOPO_KIASI);
    $import_code        = $row->NAMBA_MWANACHAMA;
    $import_member      = $row->JINA_MWANACHAMA;
    $import_amount      = $row->MKOPO_KIASI;

    // Validation
    if($db_type != $import_type) {
        die($app->renderAlertError('Uploaded loan type:<b> '.$import_type.'</b> but system loan type: <b>'.$db_type.'</b>', 'Row: '.$count.' (Loan: '.$id.')'));
    }

    if(is_numeric($import_amount)) {
        if($db_amount != $import_amount) {
            die($app->renderAlertError('Uploaded loan amount:<b> TZS '.number_format($import_amount).'</b> but system loan amount: <b>TZS '.number_format($db_amount).'</b>', 'Row: '.$count.' (Loan: '.$id.')'));
        }
    }

    if($db_code != $import_code) {
        die($app->renderAlertError('Uploaded loan member number:<b> '.$import_code.'</b> but system loan member number: <b>'.$db_code.'</b>', 'Row: '.$count.' (Loan: '.$id.')'));
    }

    if($db_member != $import_member) {
        die($app->renderAlertError('Uploaded loan member name:<b> '.$import_member.'</b> but system loan member name: <b>'.$db_member.'</b>', 'Row: '.$count.' (Loan: '.$id.')'));
    }
}

foreach($data as $row) {
    $loan_id                    = $row->NAMBA_MKOPO;
    $loan                       = new Loan($row->NAMBA_MKOPO);
    $loan_paid                  = $app->unformatMoney($row->MKOPO_ALIOREJESHA);
    $interest_paid              = $app->unformatMoney($row->RIBA_ALIYOREJESHA);
    $member                     = $loan->member();
    $person                     = $member->person();

    $data                       = $saccos_transaction->addCommonFields (
        array (
            'member_id'         => $loan->member_id,
            'date'              => $now,
            'transaction'       => '',
            'transaction_id'    => '',
            'parent'            => '',
            'parent_id'         => '',
            'paymethod'         => 'BankDeposit',
            'reference'         => '',
            'account_id'        => 1,
            'details'           => '',
            'currency'          => $client->currency_code,
            'rate'              => $client->currency_rate,
            'amount'            => 0,
            'withdrawn'         => 0,
            'deposited'         => 0,
            'balance'           => 0
        )
    ); 

    if($loan_paid > 0) {
        // Rejesha mkopo
        $trans_category         = $app->getCategoryByCode('SaccosTransaction', 'Repayment');
        $data['transaction']    = 'Repayment';
        $data['parent']         = 'Loan';
        $data['parent_id']      = $loan_id;
        $data['amount']         = $loan_paid;
        $data['deposited']      = $loan_paid;
        $data['balance']        = $loan->amount_due - $loan_paid;
        $saccos_transaction_id  = $saccos_transaction->create($data);
        echo $app->renderAlertSuccess($person->name.' amerejesha mkopo TZS '.number_format($loan_paid));
    }

    if($interest_paid > 0) {
        // Rejesha riba
        $trans_category         = $app->getCategoryByCode('SaccosTransaction', 'Interest');
        $data['transaction']    = 'Interest';
        $data['parent']         = 'Loan';
        $data['parent_id']      = $loan_id;
        $data['amount']         = $interest_paid;
        $data['deposited']      = $interest_paid;
        $data['balance']        = $loan->interest_due - $interest_paid;
        $saccos_transaction_id  = $saccos_transaction->create($data);
        echo $app->renderAlertSuccess($person->name.' amerejesha riba TZS '.number_format($interest_paid));
    }
}

?>