<?php 

class Loan extends Crud {
	function __construct($id=0) {		
		$this->table        = 'saccos_loans';
		$this->primary_key  = 'id';
		parent::__construct($id);
	}

	function getMember() {
		return new Member($this->member_id);
	}

	function member() {
		return new Member($this->member_id);
	}

	function getPerson() {
		return new Person($this->getMember()->person_id);
	}

	function person() {
		return new Person($this->member()->person_id);
	}

	function category() {
		global $app;
		return $app->getCategoryByCode('LoanType', $this->type);
	}

	function repayment() {
		global $app;
		$repayment_code 	= $app->getColumnValue('saccos_loan_sources', 'code', 'loan_id', $this->id);
		return $app->getCategoryByCode('RepaymentSource', $repayment_code);
	}

	function guarantors() {
		global $app;
		$guarantors = array();
		$sql 		= "SELECT guarantor_id, datetime FROM saccos_loan_guarantors WHERE loan_id = $this->id ";
		foreach($app->read($sql) as $row) {
			$guarantors[] = (object) array (
				'guarantor'	=> new Member($row->guarantor_id),
				'datetime'	=> $app->datetime($row->datetime)
			);
		}
		return $guarantors;
	}

	function committees() {
		global $app;
		$committees = array();
		$sql 		= "SELECT authorizer_id, leader_title, datetime FROM saccos_loan_authorizers WHERE loan_id = $this->id AND leader_cadre = 'LoansCommitee' ";
		foreach($app->read($sql) as $row) {
			$committees[] = (object) array (
				'committee'	=> new Member($row->authorizer_id),
				'datetime'	=> $app->datetime($row->datetime),
				'category'	=> $app->getCategoryByCode('LeadershipTitle', $row->leader_title)
			);
		}
		return $committees;
	}

	function authorizers() {
		global $app;
		$authorizers = array();
		$sql 		= "SELECT authorizer_id, leader_title, datetime FROM saccos_loan_authorizers WHERE loan_id = $this->id AND leader_cadre = 'SaccosAdministration' ";
		foreach($app->read($sql) as $row) {
			$authorizers[] = (object) array (
				'authorizer'	=> new Member($row->authorizer_id),
				'datetime'		=> $app->datetime($row->datetime),
				'category'		=> $app->getCategoryByCode('LeadershipTitle', $row->leader_title)
			);
		}
		return $authorizers;
	}

	function disburse($start_date=0, $end_date=0) {
		global $app, $now, $user_id;

		$start_date				= $start_date ? $start_date : $now;
		$end_date				= $end_date ? $end_date : date('Y-m-d', strtotime(' + '.$this->duration_months.' months'));
		$journal				= new Journal();
		$member 				= $this->member();
		$debit					= 'Bank';
		$debit_id 				= 1;
		$credit					= 'Loan issued';
		$credit_id				= $this->id;
		
		$update_data 			= array (
			'start_date' 		=> $start_date,
			'end_date'			=> $end_date,
			'progress'			=> 'Disbursed',
			'progress_code'		=> 6,
			'progress_datetime'	=> $now
		);

		$journal_data      		= array (
            'id'                => 0,
            'date'              => $app->date($now),
            'name'              => $this->type.' loan to '.$member->code().' '.$member->person()->name,
            'details'           => $this->type.' loan to '.$member->code().' '.$member->person()->name,
            'transaction'       => 'LoanIssued',
            'transaction_id'    => $this->id,
            'personnel'         => 'Member',
            'personnel_id'      => $this->member_id,
            'debit'             => $debit,
            'debit_id'          => $debit_id,
            'credit'            => $credit,
            'credit_id'         => $credit_id,
            'amount'            => $this->amount,
            'currency'          => $this->currency_code,
            'rate'              => $this->currency_rate,
            'f1'                => '',
            'f2'                => '',
            'f3'                => '',
            'f4'                => '',
            'f5'                => '',
            'f6'                => '',
            'f7'                => '',
            'f8'                => '',
            'f9'                => '',
            'created_user_id'   => $user_id,
            'created_datetime'  => $now,
            'modified_user_id'  => 0,
            'modified_datetime' => 0,
            'status'            => 1
        );

		// Create loan journal entry
		if($this->update($update_data))
		$journal->create($journal_data);
	}

	function progress($progress_code='') {
		global $app;
		$progress_code	= strlen($progress_code) ? $progress_code : $this->progress;
		return $app->getCategoryByCode('LoanStatus', $progress_code);
	}

	function nextProgress() {
		global $app;
		$sql	= "SELECT id FROM settings_categories WHERE type = 'LoanStatus' AND priority > $this->progress_code ORDER BY priority ";
		foreach($app->read($sql) as $row) {
			return new Category($row->id);
		}
		return new Category(0);
	}

	function updateProgress($stage) {
		global $db, $now, $app, $client, $user_id, $notification;

		$stage		= $this->progress()->code;
		$progress	= $stage;

		// Wadhamini?

		if($stage == 'Guarantors') {

			$sql 						= "SELECT COUNT(guarantor_id) as guarantors FROM saccos_loan_guarantors WHERE loan_id = $this->id AND response = 'Pending' ";
			$rows 						= $db->read($sql);
			
			if($rows[0]->guarantors < 1) {



				$progress 				= 'LoansCommittee';
				$progress_code			= 3;

				$sql 					= "UPDATE saccos_loans SET progress = '$progress', progress_code = $progress_code, progress_datetime = '$now' WHERE id = $this->id ";
				$updated 				= $db->query($sql);

				// Kamati

				$loan               	= $this;
				$authorizer         	= new Table('saccos_loan_authorizers');

				$leaders_sql 			= "SELECT personnel_leaders.id, personnel_leaders.personnel_id, personnel_leaders.person_id
				FROM personnel_leaders 
				JOIN settings_categories ON settings_categories.code = personnel_leaders.title 
				WHERE settings_categories.type = 'LeadershipTitle'
				AND settings_categories.f5 = 3 ";

				foreach($db->read($leaders_sql) as $leader_row) {
					
					$leader         = new Leader($leader_row->id);
					$member         = $loan->member();
					$subtitle       = $member->person()->name.' ameomba mkopo wa '.$client->currency_code.' '.number_format($app->unformatMoney($loan->amount));
					$content        = $member->person()->title.' '.$member->person()->name.' ameomba '.$loan->category()->name.' wa '.$client->currency_code.' '.number_format($app->unformatMoney($loan->amount)).' atarejesha miezi '.$loan->duration_months.' kupitia '.$loan->repayment()->name.'. ';
					$alert_content  = $content.'Ukiwa '.$leader->title()->name.' tafadhali idhinisha.';
					$sms_content    = $content.'Idhinisha: '.$client->client_url.'/to?do=authorize';

					$authorizer_data            = array (
						'id'                    => 0,
						'loan_id'               => $loan->id,
						'member_id'             => $loan->member_id,
						'authorizer_id'         => $leader_row->personnel_id,
						'leader_id'         	=> $leader_row->id,
						'loan_stage'         	=> $progress,
						'stage_code'         	=> $progress_code,
						'leader_cadre'          => $leader->cadre()->code,
						'leader_title'          => $leader->title()->code,
						'reponse'               => 'Pending',
						'notes'                 => '',
						'datetime'              => 0,
						'f1'                    => '',
						'f2'                    => '',
						'f3'                    => '',
						'f4'                    => '',
						'f5'                    => '',
						'f6'                    => '',
						'f7'                    => '',
						'f8'                    => '',
						'f9'                    => '',
						'created_user_id'       => $user_id,
						'created_datetime'      => $now,
						'modified_user_id'      => 0,
						'modified_datetime'     => 0,
						'status'                => 1
					);

					$authorizer->create($authorizer_data);

					$notify_data            = array (
						'priority'          => 1,
						'section'           => 'Loan',
						'section_id'        => $loan->id,
						'subsection'        => 'Member',
						'subsection_id'     => $loan->member_id,
						'title'             => 'Loans',
						'subtitle'          => $subtitle,
						'action1'           => '<div class="col-12 col-lg-6 col-xl-4 col-xxl-3 py-2"> <a href="__ROOT__pages/saccos/loans/form.php?id='.$loan->id.'" class="remote-modal btn w-100 btn-outline-primary fw-normal" data-bs-toggle="modal" data-bs-target="#modal_lg"> <span class="fa-light fa-info-circle fa-fw"></span> Taarifa za mkopo </a> </div>',
                        'action2'           => '<div class="col-6 col-lg-3 col-xl-4 py-2 col-xxl-3 py-2"> <a href="__ROOT__pages/saccos/loans/action_form.php?id=__ID__&loan_id='.$loan->id.'&stage=LoansCommittee&action=edit&response=Rejected" class="remote-modal btn w-100 btn-outline-primary fw-normal" data-bs-toggle="modal" data-bs-target="#modal_lg"> <span class="fa-light fa-times fa-fw"></span> Sitisha mkopo </a> </div>',
                        'action3'           => '<div class="col-6 col-lg-3 col-xl-4 py-2 col-xxl-3 py-2"> <a href="__ROOT__pages/saccos/loans/action_form.php?id=__ID__&loan_id='.$loan->id.'&stage=LoansCommittee&action=edit&response=Accepted" class="remote-modal btn w-100 btn-primary fw-normal" data-bs-toggle="modal" data-bs-target="#modal_lg"> <span class="fa-light fa-check fa-fw"></span> Idhinisha mkopo </a> </div>',
						'sms_content'       => $sms_content,
						'alert_content'     => $alert_content,
						'personnel_id'      => $leader->person()->user()->id
					);

					$notification->push($notify_data);
				}
			}
		} elseif($stage == 'LoansCommittee') {
			$sql 						= "SELECT COUNT(authorizer_id) as authorizers FROM saccos_loan_authorizers WHERE loan_id = $this->id AND loan_stage = 'LoansCommittee' AND response = 'Pending' ";
			$rows 						= $db->read($sql);
			
			if($rows[0]->authorizers < 1) {

				$progress 				= 'SaccosLeaders';
				$progress_code			= 4;

				$sql 					= "UPDATE saccos_loans SET progress = '$progress', progress_code = $progress_code, progress_datetime = '$now' WHERE id = $this->id ";
				$updated 				= $db->query($sql);

				// Viongozi

				$loan               	= $this;
				$authorizer         	= new Table('saccos_loan_authorizers');

				$leaders_sql 			= "SELECT personnel_leaders.id, personnel_leaders.personnel_id, personnel_leaders.person_id
				FROM personnel_leaders 
				JOIN settings_categories ON settings_categories.code = personnel_leaders.title 
				WHERE settings_categories.type = 'LeadershipTitle'
				AND settings_categories.f5 = 4 ";

				foreach($db->read($leaders_sql) as $leader_row) {
					$leader         	= new Leader($leader_row->id);
					$member         	= $loan->member();
					$subtitle       	= $member->person()->name.' ameomba mkopo wa '.$client->currency_code.' '.number_format($app->unformatMoney($loan->amount));
					$content        	= $member->person()->title.' '.$member->person()->name.' ameomba '.$loan->category()->name.' wa '.$client->currency_code.' '.number_format($app->unformatMoney($loan->amount)).' atarejesha miezi '.$loan->duration_months.' kupitia '.$loan->repayment()->name.'. ';
					$alert_content  	= $content.'Ukiwa '.$leader->title()->name.' tafadhali idhinisha.';
					$sms_content    	= $content.'Idhinisha: '.$client->client_url.'/to?do=authorize';

					$authorizer_data            = array (
						'id'                    => 0,
						'loan_id'               => $loan->id,
						'member_id'             => $loan->member_id,
						'authorizer_id'         => $leader_row->personnel_id,
						'leader_id'         	=> $leader_row->id,
						'loan_stage'         	=> $progress,
						'stage_code'         	=> $progress_code,
						'leader_cadre'          => $leader->cadre()->code,
						'leader_title'          => $leader->title()->code,
						'reponse'               => 'Pending',
						'notes'                 => '',
						'datetime'              => 0,
						'f1'                    => '',
						'f2'                    => '',
						'f3'                    => '',
						'f4'                    => '',
						'f5'                    => '',
						'f6'                    => '',
						'f7'                    => '',
						'f8'                    => '',
						'f9'                    => '',
						'created_user_id'       => $user_id,
						'created_datetime'      => $now,
						'modified_user_id'      => 0,
						'modified_datetime'     => 0,
						'status'                => 1
					);

					$authorizer->create($authorizer_data);

					$notify_data            = array (
						'priority'          => 1,
						'section'           => 'Loan',
						'section_id'        => $loan->id,
						'subsection'        => 'Member',
						'subsection_id'     => $loan->member_id,
						'title'             => 'Loans',
						'subtitle'          => $subtitle,
						'action1'           => '<div class="col-12 col-lg-6 col-xl-4 col-xxl-3 py-2"> <a href="__ROOT__pages/saccos/loans/form.php?id='.$loan->id.'" class="remote-modal btn w-100 btn-outline-primary fw-normal" data-bs-toggle="modal" data-bs-target="#modal_lg"> <span class="fa-light fa-info-circle fa-fw"></span> Taarifa za mkopo </a> </div>',
                        'action2'           => '<div class="col-6 col-lg-3 col-xl-4 py-2 col-xxl-3 py-2"> <a href="__ROOT__pages/saccos/loans/action_form.php?id=__ID__&loan_id='.$loan->id.'&stage=SaccosLeaders&action=edit&response=Rejected" class="remote-modal btn w-100 btn-outline-primary fw-normal" data-bs-toggle="modal" data-bs-target="#modal_lg"> <span class="fa-light fa-times fa-fw"></span> Sitisha mkopo </a> </div>',
                        'action3'           => '<div class="col-6 col-lg-3 col-xl-4 py-2 col-xxl-3 py-2"> <a href="__ROOT__pages/saccos/loans/action_form.php?id=__ID__&loan_id='.$loan->id.'&stage=SaccosLeaders&action=edit&response=Accepted" class="remote-modal btn w-100 btn-primary fw-normal" data-bs-toggle="modal" data-bs-target="#modal_lg"> <span class="fa-light fa-check fa-fw"></span> Idhinisha mkopo </a> </div>',
						'sms_content'       => $sms_content,
						'alert_content'     => $alert_content,
						'personnel_id'      => $leader->person()->user()->id
					);

					$notification->push($notify_data);
				}
			}
		} elseif($stage == 'SaccosLeaders') {
			$sql 						= "SELECT COUNT(authorizer_id) as authorizers FROM saccos_loan_authorizers WHERE loan_id = $this->id AND loan_stage = 'SaccosLeaders' AND response = 'Pending' ";
			$rows 						= $db->read($sql);
			
			if($rows[0]->authorizers < 1) {

				$progress 				= 'Authorized';
				$progress_code			= 5;

				$sql 					= "UPDATE saccos_loans SET progress = '$progress', progress_code = $progress_code, progress_datetime = '$now' WHERE id = $this->id ";
				$updated 				= $db->query($sql);


				// Disburser

				$loan               	= $this;
				$authorizer         	= new Table('saccos_loan_authorizers');

				$leaders_sql 			= "SELECT personnel_leaders.id, personnel_leaders.personnel_id, personnel_leaders.person_id
				FROM personnel_leaders 
				JOIN settings_categories ON settings_categories.code = personnel_leaders.title 
				WHERE settings_categories.type = 'LeadershipTitle'
				AND personnel_leaders.tags LIKE '%disburser%' ";

				foreach($db->read($leaders_sql) as $leader_row) {
					$leader         	= new Leader($leader_row->id);
					$member         	= $loan->member();
					$subtitle       	= $member->person()->name.' ameomba mkopo wa '.$client->currency_code.' '.number_format($app->unformatMoney($loan->amount));
					$content        	= $member->person()->title.' '.$member->person()->name.' ameomba '.$loan->category()->name.' wa '.$client->currency_code.' '.number_format($app->unformatMoney($loan->amount)).' atarejesha miezi '.$loan->duration_months.' kupitia '.$loan->repayment()->name.'. ';
					$alert_content  	= $content.'Ukiwa '.$leader->title()->name.' tafdhali mpatie mkopo.';
					$sms_content    	= $content.'Toa mkopo: '.$client->client_url.'/to?do=disburse';

					$authorizer_data            = array (
						'id'                    => 0,
						'loan_id'               => $loan->id,
						'member_id'             => $loan->member_id,
						'authorizer_id'         => $leader_row->personnel_id,
						'leader_id'         	=> $leader_row->id,
						'loan_stage'         	=> $progress,
						'stage_code'         	=> $progress_code,
						'leader_cadre'          => $leader->cadre()->code,
						'leader_title'          => $leader->title()->code,
						'reponse'               => 'Pending',
						'notes'                 => '',
						'datetime'              => 0,
						'f1'                    => '',
						'f2'                    => '',
						'f3'                    => '',
						'f4'                    => '',
						'f5'                    => '',
						'f6'                    => '',
						'f7'                    => '',
						'f8'                    => '',
						'f9'                    => '',
						'created_user_id'       => $user_id,
						'created_datetime'      => $now,
						'modified_user_id'      => 0,
						'modified_datetime'     => 0,
						'status'                => 1
					);

					$authorizer->create($authorizer_data);

					$notify_data            = array (
						'priority'          => 1,
						'section'           => 'Loan',
						'section_id'        => $loan->id,
						'subsection'        => 'Member',
						'subsection_id'     => $loan->member_id,
						'title'             => 'Loans',
						'subtitle'          => $subtitle,
						'action1'           => '<div class="col-12 col-lg-6 col-xl-4 col-xxl-3 py-2"> <a href="__ROOT__pages/saccos/loans/form.php?id='.$loan->id.'" class="remote-modal btn w-100 btn-outline-primary fw-normal" data-bs-toggle="modal" data-bs-target="#modal_lg"> <span class="fa-light fa-info-circle fa-fw"></span> Taarifa za mkopo </a> </div>',
                        'action3'           => '<div class="col-6 col-lg-3 col-xl-4 py-2 col-xxl-3 py-2"> <a href="__ROOT__pages/saccos/loans/action_form.php?id=__ID__&loan_id='.$loan->id.'&stage=Authorized&action=edit&response=Accepted" class="remote-modal btn w-100 btn-primary fw-normal" data-bs-toggle="modal" data-bs-target="#modal_lg"> <span class="fa-light fa-check fa-fw"></span> Toa mkopo </a> </div>',
						'sms_content'       => $sms_content,
						'alert_content'     => $alert_content,
						'personnel_id'      => $leader->person()->user()->id
					);

					$notification->push($notify_data);
				}

			}
		} elseif($stage == 'Authorized') {
			$progress 				= 'Repayments';
			$progress_code			= 6;

			$sql 					= "UPDATE saccos_loans SET progress = '$progress', progress_code = $progress_code, progress_datetime = '$now' WHERE id = $this->id ";
			$updated 				= $db->query($sql);

			$this->disburse();
		}
		$this->progress				= $progress;
	}

	function setResponse($stage, $response, $personnel_id, $notification_id = 0) {

		global $db, $now, $app, $client, $user;
		$member			= new Member($personnel_id);
		$notification	= new Notification($notification_id);
		$sms_content	= '';
		
		// Guarantors response
		if($stage == 'Guarantors') {
			$sql 		= "UPDATE saccos_loan_guarantors SET response = '$response', datetime = '$now' WHERE loan_id = $this->id AND guarantor_id = $personnel_id ";
			$updated 	= $db->query($sql);
			
			$this->updateProgress($stage);

			$subtitle	= $member->person()->name.' amekudhamini '.$client->currency_code.' '.number_format($this->amount);
			$sms_content.= 'Mdhamini '.$member->person()->title.' '.$member->person()->name.' amekudhamini '.$this->category()->name.' wa '.$client->currency_code.' '.number_format($this->amount).'. ';
			$message 	= $response == 'Accepted' ? 'Ombi la udhamini limekubaliwa.' : 'Ombi la udhamini limekataliwa.';
		}

		// Committee response
		if($stage == 'LoansCommittee') {
			$leader_member_id = $user->personnel()->id;

			$leader_id		= $app->getColumnValue('saccos_loan_authorizers', 'leader_id', 'authorizer_id', $leader_member_id, "loan_id = $this->id AND loan_stage = '$stage'");
			$leader 		= new Leader($leader_id);

			$sql 			= "UPDATE saccos_loan_authorizers SET response = '$response', datetime = '$now' WHERE loan_id = $this->id AND authorizer_id = $personnel_id AND loan_stage = '$stage'";
			$updated 		= $db->query($sql);
			
			$this->updateProgress($stage);

			$subtitle		= $leader->title()->name.' ameidhinisha '.$client->currency_code.' '.number_format($this->amount).'';
			$sms_content	.= $leader->title()->name.' '.$leader->person()->title.' '.$leader->person()->name.' ameidhinisha '.$this->category()->name.' wa '.$client->currency_code.' '.number_format($this->amount).'. ';
			$message 		= $response == 'Accepted' ? 'Ombi la mkopo limeidhinishwa.' : 'Ombi la mkopo limekataliwa.';
		}

		// Leaders response
		if($stage == 'SaccosLeaders') {
			$leader_member_id = $user->personnel()->id;

			$leader_id		= $app->getColumnValue('saccos_loan_authorizers', 'leader_id', 'authorizer_id', $leader_member_id, "loan_id = $this->id AND loan_stage = '$stage'");
			$leader 		= new Leader($leader_id);

			$sql 			= "UPDATE saccos_loan_authorizers SET response = '$response', datetime = '$now' WHERE loan_id = $this->id AND authorizer_id = $personnel_id AND loan_stage = '$stage'";
			$updated 		= $db->query($sql);
			
			$this->updateProgress($stage);

			$subtitle		= $leader->title()->name.' ameidhinisha '.$client->currency_code.' '.number_format($this->amount).'';
			$sms_content	.= $leader->title()->name.' '.$leader->person()->title.' '.$leader->person()->name.' ameidhinisha '.$this->category()->name.' wa '.$client->currency_code.' '.number_format($this->amount).'. ';
			$message 		= $response == 'Accepted' ? 'Ombi la mkopo limeidhinishwa.' : 'Ombi la mkopo limekataliwa.';
		}

		// Disburser response
		if($stage == 'Authorized') {
			$leader_member_id = $user->personnel()->id;

			$leader_id		= $app->getColumnValue('saccos_loan_authorizers', 'leader_id', 'authorizer_id', $leader_member_id, "loan_id = $this->id AND loan_stage = '$stage'");
			$leader 		= new Leader($leader_id);

			$sql 			= "UPDATE saccos_loan_authorizers SET response = '$response', datetime = '$now' WHERE loan_id = $this->id AND authorizer_id = $personnel_id AND loan_stage = '$stage'";
			$updated 		= $db->query($sql);
			
			$this->updateProgress($stage);

			$subtitle		= $leader->title()->name.' ametoa mkopo wa '.$client->currency_code.' '.number_format($this->amount).'';
			$sms_content	.= $leader->title()->name.' '.$leader->person()->title.' '.$leader->person()->name.' ametoa '.$this->category()->name.' wa '.$client->currency_code.' '.number_format($this->amount).'. ';
			$message 		= $response == 'Accepted' ? 'Mkopo umetolewa.' : 'Ombi la mkopo limekataliwa.';
		}

		$sms_content 			.= 'Mkopo upo hatua ya '.$this->progress()->name.'.';

		$notify_data            = array (
			'priority'          => 1,
			'section'           => 'Loan',
			'section_id'        => $this->id,
			'subsection'        => 'Member',
			'subsection_id'     => $this->member_id,
			'title'             => 'Loans',
			'subtitle'          => $subtitle,
			'sms_content'       => $sms_content,
			'alert_content'     => $sms_content,
			'personnel'      	=> 'Member',
			'personnel_id'      => $this->member_id,
		);

		// Clear notification
		echo '<script> $(\'#notification_container_'.$notification_id.'\').slideUp(); </script>';

		$notification->destroy();
		$notification->push($notify_data);

		return $app->renderAlertSuccess($message, 'Success', 'modal_lg');
	}

	function renderBar() {
		
		$all_loans 	= $this->amount + $this->interest_amount;
		$due_loans	= $this->amount_due + $this->interest_due;
		
		if(!$all_loans) $all_loans		= 1;
		$percent	= ($all_loans - $due_loans) / $all_loans * 100;
		if($percent > 99) $percent	= 0;

		$color 						= 'danger';
		if($percent > 25) $color 	= 'warning';
		if($percent > 50) $color 	= 'success';
		
		$html = '<div class="fw-semibold pb-3 rounded text-start text-"'.$color.'">
		<div class="row g-0">
		<div class="col-6 fw-semibold"> '.str_replace('Mkopo wa ', '', $this->name).' ('.number_format($percent).'%)</div>
		<div class="col-6 text-end fw-semibold"> '.number_format($due_loans).' </diV>
		</div>
		<div class="row g-0">
		<div class="col-8 fw-normal"> Rejesho/Mwezi (X '.$this->duration_months.')</div>
		<div class="col-4 text-end fw-normal"> '.number_format($this->amount_monthly + $this->interest_monthly).' </diV>
		</div>
		<div class="progress mt-2" style="height:6px">
		<div class="progress-bar bg-'.$color.'" role="progressbar" aria-label="Success example" style="width: '.$percent.'%; " aria-valuenow="'.$percent.'" aria-valuemin="0" aria-valuemax="100"></div>
		</div></div>
		
		<div class="row g-0 text-muted mb-3" style="font-size: 0.8em; margin-top:-10px">
		<div class="col-6 text-start"> '.number_format($this->amount_due).' / <b>'.number_format($this->amount).'</b></div>
		<div class="col-6 text-end"> '.number_format($this->interest_due).' / <b>'.number_format($this->interest_amount).' </b></div>
		</div>';

		return $html;
	}
}

?>